@@ -20,22 +20,22 @@
 #ifndef __GENERICFILTER_H__
 #define __GENERICFILTER_H__
 
-#include <q3cstring.h>
-#include <QStringList>
+#include <qstring.h>
+#include <QVariantList>
 
 #include <KoFilter.h>
 #include <KoStore.h>
 
 class GenericFilter : public KoFilter
 {
-	Q_OBJECT
+    Q_OBJECT
 
 public:
-    GenericFilter(QObject *parent, const QStringList&);
+    GenericFilter(QObject *parent, const QVariantList&);
     virtual ~GenericFilter() {}
 
-    virtual KoFilter::ConversionStatus convert( const QByteArray& from,
-                                                const QByteArray& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from,
+            const QByteArray& to);
 private:
     KoFilter::ConversionStatus doImport();
     KoFilter::ConversionStatus doExport();
