@@ -26,19 +26,20 @@
 
 #include <KoFilter.h>
 #include <KoStore.h>
+#include <QVariantList>
 
 class AmiProImport : public KoFilter
 {
     Q_OBJECT
 
-  public:
+public:
 
-    AmiProImport(QObject* parent, const QStringList&);
+    AmiProImport(QObject* parent, const QVariantList&);
     AmiProImport();
 
     virtual ~AmiProImport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QByteArray& from, const QByteArray& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 
 #endif // __AMIPROIMPORT_H
