@@ -21,23 +21,25 @@
 #ifndef EXPORT_DIALOG_H
 #define EXPORT_DIALOG_H
 
-#include <QWidget>
+#include <QtGui/QWidget>
+#include <QtGui/QButtonGroup>
 
 #include <kdialog.h>
-
-class ExportDialogUI;
+#include <ui_ExportDialogUI.h>
 
 class AsciiExportDialog : public KDialog
 {
     Q_OBJECT
 public:
 
-    AsciiExportDialog(QWidget* parent=NULL);
+    AsciiExportDialog(QWidget* parent = NULL);
     ~AsciiExportDialog(void);
     QString getEndOfLine(void) const;
     QTextCodec* getCodec(void) const;
 private:
-    ExportDialogUI* m_dialog;
+    QWidget* const m_dialog;
+    Ui::ExportDialogUI m_ui;
+    QButtonGroup m_radioGroup;
 };
 
 #endif /* EXPORT_DIALOG_H */
