@@ -21,22 +21,24 @@
 #ifndef _IMPORT_DIALOG_H
 #define _IMPORT_DIALOG_H
 
-#include <QWidget>
+#include <QtGui/QWidget>
+#include <QtGui/QButtonGroup>
 
 #include <kdialog.h>
-
-class ImportDialogUI;
+#include <ui_ImportDialogUI.h>
 
 class AsciiImportDialog : public KDialog
 {
     Q_OBJECT
 public:
-    AsciiImportDialog(QWidget* parent=NULL);
-    ~AsciiImportDialog(void);
-    QTextCodec* getCodec(void) const;
-    int getParagraphStrategy(void) const;
+    AsciiImportDialog(QWidget *parent = 0);
+    ~AsciiImportDialog();
+    QTextCodec *getCodec() const;
+    int getParagraphStrategy() const;
+
 private:
-    ImportDialogUI* m_dialog;
+    Ui::ImportDialogUI m_ui;
+    QButtonGroup m_radioGroup;
 };
 
 #endif /* _IMPORT_DIALOG_H */
