@@ -24,19 +24,20 @@
 #include <KoStore.h>
 //Added by qt3to4:
 #include <QByteArray>
+#include <QVariantList>
 
 class WPExport : public KoFilter
 {
     Q_OBJECT
 
-  public:
+public:
 
-    WPExport(QObject* parent, const QStringList&);
+    WPExport(QObject* parent, const QVariantList&);
     WPExport();
 
     virtual ~WPExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QByteArray& from, const QByteArray& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 
 #endif // __WPEXPORT_H
