@@ -28,24 +28,25 @@
 
 #include <KoFilter.h>
 #include <KoStore.h>
+#include <QVariantList>
 
 class WPImport : public KoFilter
 {
     Q_OBJECT
 
 
-  protected:
+protected:
 
     QString m_output;
 
-  public:
+public:
 
-    WPImport(QObject* parent, const QStringList&);
+    WPImport(QObject* parent, const QVariantList&);
     WPImport();
 
     virtual ~WPImport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QByteArray& from, const QByteArray& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 
 };
 #endif // __WPIMPORT_H
