@@ -29,18 +29,19 @@
 
 #include <KoFilter.h>
 #include <KoStore.h>
+#include <QVariantList>
 
 class XSLTImport : public KoFilter
 {
 
-	Q_OBJECT
+    Q_OBJECT
 
-	public:
-    	XSLTImport(QObject* parent, const QStringList&);
-		virtual ~XSLTImport() {}
+public:
+    XSLTImport(QObject* parent, const QVariantList&);
+    virtual ~XSLTImport() {}
 
-		virtual KoFilter::ConversionStatus convert( const QByteArray& from,
-							const QByteArray& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from,
+            const QByteArray& to);
 };
 
 #endif /* __XSLTIMPORT_H__ */
