@@ -1,16 +1,16 @@
 /* This file is part of the KDE project
  * Copyright (c) 2009 Jan Hambrecht <jaham@gmx.net>
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
@@ -20,15 +20,15 @@
 #ifndef KARBONFILTEREFFECTSTOOLFACTORY_H
 #define KARBONFILTEREFFECTSTOOLFACTORY_H
 
-#include <KoToolFactory.h>
+#include <KoToolFactoryBase.h>
 
-class KarbonFilterEffectsToolFactory : public KoToolFactory
+class KarbonFilterEffectsToolFactory : public KoToolFactoryBase
 {
-Q_OBJECT
+    Q_OBJECT
 public:
     explicit KarbonFilterEffectsToolFactory(QObject *parent);
     virtual ~KarbonFilterEffectsToolFactory();
-    virtual KoTool * createTool(KoCanvasBase *canvas);
+    virtual KoToolBase * createTool(KoCanvasBase *canvas);
 };
 
 #endif // KARBONFILTEREFFECTSTOOLFACTORY_H
