@@ -77,6 +77,7 @@
 #include "prefs.h"
 
 #include "commands/ChartTypeCommand.h"
+#include <KoMainWindow.h>
 
 
 using namespace std;
@@ -115,7 +116,7 @@ KChartView::KChartView( KChartPart* part, QWidget* parent )
     m_canvas = new KChartCanvas( this, part );
 
     connect( m_canvas, SIGNAL( documentViewRectChanged( const QRectF& ) ),
-	     this,     SLOT( documentViewRectChanged( const QRectF& ) ) );
+             this,     SLOT( documentViewRectChanged( const QRectF& ) ) );
 
     m_canvasController = new KoCanvasController( this );
     m_canvasController->setCanvas( m_canvas );
@@ -151,25 +152,27 @@ KChartView::KChartView( KChartPart* part, QWidget* parent )
     m_zoomController->setDocumentSize( m_canvas->documentViewRect().size() );
 
     addStatusBarItem( zoomAction->createWidget( ( QWidget* )statusBar() ), 0 );
-    connect( m_zoomController, SIGNAL(zoomChanged(KoZoomMode::Mode, double)),
-             this, SLOT(zoomChanged(KoZoomMode::Mode, double)));
+    connect( m_zoomController, SIGNAL(zoomChanged(KoZoomMode::Mode, qreal)),
+             this, SLOT(zoomChanged(KoZoomMode::Mode, qreal)));
     m_zoomController->setZoomMode( KoZoomMode::ZOOM_PAGE );
 
     // initialize the configuration
     //    loadConfig();
 
     //KChartPart *part = (KChartPart*)koDocument();
-    
+
     KoToolManager::instance()->addController( m_canvasController );
     KoToolManager::instance()->registerTools( actionCollection(), m_canvasController );
 
-    KoDockerManager *dockerManager = new KoDockerManager(this);
-    connect( m_canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &) ),
-             dockerManager, SLOT( newOptionWidgets(const  QMap<QString, QWidget *> &) ) );
+    if (shell())
+    {
+        connect( m_canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &) ),
+             shell()->dockerManager(), SLOT( newOptionWidgets(const  QMap<QString, QWidget *> &) ) );
+
+        KoToolBoxFactory toolBoxFactory( m_canvasController, i18n("Tools") );
+        shell()->createDockWidget( &toolBoxFactory );
+    }
 
-    KoToolBoxFactory toolBoxFactory( m_canvasController, i18n("Tools") );
-    createDockWidget( &toolBoxFactory );
-	
     // Disable some things if we can't change the data, e.g. because
     // we are inside another application that provides the data for us.
     if ( !part->canChangeValue() ) {
@@ -177,9 +180,9 @@ KChartView::KChartView( KChartPart* part, QWidget* parent )
         m_importData->setEnabled( false );
     }
 
-    m_canvas->shapeManager()->add( part->chart() );
+    m_canvas->shapeManager()->addShape( part->chart() );
     connect( m_canvas->shapeManager()->selection(), SIGNAL( selectionChanged() ), this, SLOT( selectionChanged() ) );
-    
+
     m_canvasController->show();
 }
 
@@ -201,7 +204,7 @@ ViewAdaptor* KChartView::dbusObject()
 void KChartView::paintEvent( QPaintEvent* /*ev*/ )
 {
     //QPainter painter( this );
-    //QRect clipRect = 
+    //QRect clipRect =
 
     // ### TODO: Scaling
 
@@ -215,7 +218,7 @@ void KChartView::paintEvent( QPaintEvent* /*ev*/ )
 void KChartView::updateReadWrite( bool /*readwrite*/ )
 {
 #ifdef __GNUC__
-#warning TODO
+    #warning TODO: implement updateReadWrite
 #endif
 }
 
@@ -301,24 +304,24 @@ void KChartView::print(QPrinter &printer, QPrintDialog &printDialog)
     int  height;
     int  width;
     if ( !printer.previewOnly() ) {
-	int const scalex = printer.option("kde-kchart-printsizex").toInt();
-	int const scaley = printer.option("kde-kchart-printsizey").toInt();
+        int const scalex = printer.option("kde-kchart-printsizex").toInt();
+        int const scaley = printer.option("kde-kchart-printsizey").toInt();
 
-	width  = printer.width()  * scalex / 100;
-	height = printer.height() * scaley / 100;
+        width  = printer.width()  * scalex / 100;
+        height = printer.height() * scaley / 100;
     }
     else {
-	// Fill the whole page.
-	width  = printer.width();
-	height = printer.height();
+        // Fill the whole page.
+        width  = printer.width();
+        height = printer.height();
     }
 
     QRect  rect(0, 0, width, height);
     KDChart::print(&painter,
-		   ((KChartPart*)koDocument())->params(),
-		   ((KChartPart*)koDocument())->data(),
-		   0, 		// regions
-		   &rect);
+                   ((KChartPart*)koDocument())->params(),
+                   ((KChartPart*)koDocument())->data(),
+                   0, 		// regions
+                   &rect);
     painter.end();
 }
 #endif
@@ -331,9 +334,9 @@ void KChartView::importData()
 {
     // Get the name of the file to open.
     QString filename = KFileDialog::getOpenFileName(KUrl(QString()),// startDir
-						    QString(),// filter
-						    0,
-						    i18n("Import Data"));
+                                                    QString(),// filter
+                                                    0,
+                                                    i18n("Import Data"));
     kDebug(35001) <<"Filename = <" << filename <<">";
     if (filename.isEmpty())
       return;
@@ -341,10 +344,10 @@ void KChartView::importData()
     // Check to see if we can read the file.
     QFile  inFile(filename);
     if (!inFile.open(QIODevice::ReadOnly)) {
-	KMessageBox::sorry( 0, i18n("The file %1 could not be read.",
+        KMessageBox::sorry( 0, i18n("The file %1 could not be read.",
                                     filename) );
-	inFile.close();
-	return;
+        inFile.close();
+        return;
     }
 
     // Let the CSV dialog structure the data in the file.
@@ -354,8 +357,8 @@ void KChartView::importData()
     dialog->setData(inData);
 
     if ( !dialog->exec() ) {
-	// kDebug(35001) <<"Cancel was pressed";
-	return;
+        // kDebug(35001) <<"Cancel was pressed";
+        return;
     }
 
     //kDebug(35001) <<"OK was pressed";
@@ -373,35 +376,35 @@ void KChartView::importData()
     data.setUsedRows( rows );
     data.setUsedCols( cols );
     for (uint row = 0; row < rows; row++) {
-	for (uint col = 0; col < cols; col++) {
-	    bool     ok;
-	    QString  tmp;
-	    double   val;
+        for (uint col = 0; col < cols; col++) {
+            bool     ok;
+            QString  tmp;
+            qreal   val;
 
-	    // Get the text and convert to double unless in the headers.
-	    tmp = dialog->text( row, col );
-	    if ( ( row == 0 && hasRowHeaders )
-		 || ( col == 0 && hasColHeaders ) ) {
-		kDebug(35001) <<"Setting header (" << row <<"," << col
-			       << ") to value " << tmp << endl;
-		data.setCell( row, col, tmp );
-	    }
-	    else {
-		val = tmp.toDouble(&ok);
-		if (!ok)
-		    val = 0.0;
+            // Get the text and convert to qreal unless in the headers.
+            tmp = dialog->text( row, col );
+            if ( ( row == 0 && hasRowHeaders )
+                 || ( col == 0 && hasColHeaders ) ) {
+                kDebug(35001) <<"Setting header (" << row <<"," << col
+                               << ") to value " << tmp << endl;
+                data.setCell( row, col, tmp );
+            }
+            else {
+                val = tmp.toDouble(&ok);
+                if (!ok)
+                    val = 0.0;
 
-		kDebug(35001) <<"Setting (" << row <<"," << col
-			       << ") to value " << val << endl;
+                kDebug(35001) <<"Setting (" << row <<"," << col
+                               << ") to value " << val << endl;
 
-		// and do the actual setting.
-		data.setCell( row, col, val );
-	    }
-	}
+                // and do the actual setting.
+                data.setCell( row, col, val );
+            }
+        }
     }
 
     ((KChartPart*)koDocument())->doSetData( data,
-					    hasRowHeaders, hasColHeaders );
+                                            hasRowHeaders, hasColHeaders );
 #else
 //js 2008-03-19
 //?? warning C4930: 'QStandardItemModel data(void)': prototyped function not called (was a variable definition intended?)
@@ -431,6 +434,7 @@ void KChartView::extraCreateTemplate()
 void KChartView::selectionChanged()
 {
     KoSelection *selection = m_canvas->shapeManager()->selection();
+    Q_UNUSED(selection);
     emit selectionChange();
 }
 
@@ -440,11 +444,14 @@ void KChartView::documentViewRectChanged( const QRectF &viewRect )
     m_zoomController->setDocumentSize( size );
     m_zoomController->setPageSize( size );
     m_canvas->update();
-    m_canvasController->ensureVisible( m_canvas->shapeManager()->selection()->boundingRect() );
+    m_canvasController->ensureVisible(m_zoomHandler->documentToView(m_canvas->shapeManager()->selection()->boundingRect()));
 }
 
-void KChartView::zoomChanged( KoZoomMode::Mode mode, double zoom )
+void KChartView::zoomChanged( KoZoomMode::Mode mode, qreal zoom )
 {
+    Q_UNUSED(mode);
+    Q_UNUSED(zoom);
+
     QRectF documentViewRect = m_canvas->documentViewRect();
     m_zoomController->setDocumentSize( documentViewRect.size() );
     m_canvasController->setPreferredCenter( m_zoomHandler->documentToView( documentViewRect ).center().toPoint() );
