@@ -18,21 +18,22 @@
    Boston, MA 02110-1301, USA.
 */
 
-// Local
+// Own
 #include "TestProxyModel.h"
 
-// KChart
-#include "ChartProxyModel.h"
-#include "DataSet.h"
-#include "ChartTableModel.h"
-#include "CellRegion.h"
-
 // Qt
 #include <QRect>
 #include <QPoint>
 #include <QSize>
 #include <QDebug>
 
+// KChart
+#include "DataSetFactory.h"
+#include "DataSet.h"
+#include "ChartTableModel.h"
+#include "CellRegion.h"
+
+
 namespace QTest {
     template<>
     char *toString( const CellRegion &region )
@@ -59,6 +60,7 @@ namespace QTest {
             result += ")";
             return qstrdup( result.data() );
         }
+        return 0;
     }
 }
 
