@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  Copyright (C) 1998, 1999, 2000 Torben Weis <weis@kde.org>
- Copyright (C) 2004, 2005 Dag Andersen <danders@get2net.dk>
+ Copyright (C) 2004, 2010 Dag Andersen <danders@get2net.dk>
  Copyright (C) 2006 Raphael Langerhorst <raphael.langerhorst@kdemail.net>
  Copyright (C) 2007 Thorsten Zachmann <zachmann@kde.org>
 
@@ -41,6 +41,8 @@
 #include <KoStore.h>
 #include <KoStoreDevice.h>
 #include <KoOdfReadStore.h>
+#include <KoUpdater.h>
+#include <KoProgressUpdater.h>
 
 #include <QApplication>
 #include <qpainter.h>
@@ -76,9 +78,10 @@ Part::Part( QWidget *parentWidget, QObject *parent, bool singleViewMode )
         : KoDocument( parentWidget, parent, singleViewMode ),
         m_project( 0 ), m_parentWidget( parentWidget ),
         m_context( 0 ), m_xmlLoader(),
-        m_loadingTemplate( false )
+        m_loadingTemplate( false ),
+        m_viewlistModified( false )
 {
-    setComponentData( Factory::global() );
+    setComponentData( Factory::global(), false ); // Do not load plugins now (the view will load them)
     setTemplateType( "kplato_template" );
     m_config.setReadWrite( isReadWrite() || !isEmbedded() );
     // Add library translation files
@@ -96,17 +99,22 @@ Part::Part( QWidget *parentWidget, QObject *parent, bool singleViewMode )
 
     setProject( new Project( m_config ) ); // after config & plugins are loaded
     m_project->setId( m_project->uniqueNodeId() );
+    m_project->registerNodeId( m_project ); // register myself
 
-    qDebug()<<"setProject: initial start"<<isReadWrite()<<isEmbedded()<<singleViewMode;
     QTimer::singleShot ( 5000, this, SLOT( checkForWorkPackages() ) );
 
+    connect(this, SIGNAL(modified(bool)), SLOT(slotModified(bool)));
 }
 
 
 Part::~Part()
 {
-    delete m_project;
+    qDeleteAll( m_schedulerPlugins );
+    if ( m_project ) {
+        m_project->deref(); // deletes if last user
+    }
     qDeleteAll( m_mergedPackages );
+    delete m_context;
 }
 
 void Part::setReadWrite( bool rw )
@@ -155,12 +163,18 @@ void Part::setProject( Project *project )
 
 KoView *Part::createViewInstance( QWidget *parent )
 {
-    View *view = new View( this, parent );
+    // syncronize view selector
+    View *view = dynamic_cast<View*>( views().value( 0 ) );
+    if ( view && m_context ) {
+        QDomDocument doc = m_context->save( view );
+        m_context->setContent( doc.toString() );
+    }
+    view = new View( this, parent );
     connect( view, SIGNAL( destroyed() ), this, SLOT( slotViewDestroyed() ) );
     return view;
 }
 
-bool Part::loadOdf( KoOdfReadStore & odfStore )
+bool Part::loadOdf( KoOdfReadStore &odfStore )
 {
     kWarning()<< "OpenDocument not supported, let's try native xml format";
     return loadXML( odfStore.contentDoc(), 0 ); // We have only one format, so try to load that!
@@ -168,10 +182,11 @@ bool Part::loadOdf( KoOdfReadStore & odfStore )
 
 bool Part::loadXML( const KoXmlDocument &document, KoStore* )
 {
-    kDebug();
-    QTime dt;
-    dt.start();
-    emit sigProgress( 0 );
+    QPointer<KoUpdater> updater;
+    if (progressUpdater()) {
+        updater = progressUpdater()->startSubtask(1, "KPlato::Part::loadXML");
+        updater->setProgress(0);
+    }
 
     QString value;
     KoXmlElement plan = document.documentElement();
@@ -199,14 +214,15 @@ bool Part::loadXML( const KoXmlDocument &document, KoStore* )
             return false;
         }
     }
-    emit sigProgress( 5 );
-
+    if (updater) updater->setProgress(5);
+/*
 #ifdef KOXML_USE_QDOM
     int numNodes = plan.childNodes().count();
 #else
     int numNodes = plan.childNodesCount();
 #endif
-#if 0 
+*/
+#if 0
 This test does not work any longer. KoXml adds a couple of elements not present in the file!!
     if ( numNodes > 2 ) {
         //TODO: Make a proper bitching about this
@@ -223,11 +239,12 @@ This test does not work any longer. KoXml adds a couple of elements not present
         }
         KoXmlElement e = n.toElement();
         if ( e.tagName() == "project" ) {
-            Project * newProject = new Project( m_config );
+            Project *newProject = new Project( m_config );
             m_xmlLoader.setProject( newProject );
             if ( newProject->load( e, m_xmlLoader ) ) {
                 if ( newProject->id().isEmpty() ) {
                     newProject->setId( newProject->uniqueNodeId() );
+                    newProject->registerNodeId( newProject );
                 }
                 // The load went fine. Throw out the old project
                 setProject( newProject );
@@ -239,12 +256,8 @@ This test does not work any longer. KoXml adds a couple of elements not present
         }
     }
     m_xmlLoader.stopLoad();
-    emit sigProgress( 100 ); // the rest is only processing, not loading
 
-    kDebug() <<"Loading took" << ( float ) ( dt.elapsed() ) / 1000 <<" seconds";
-
-    // do some sanity checking on document.
-    emit sigProgress( -1 );
+    if (updater) updater->setProgress(100); // the rest is only processing, not loading
 
     setModified( false );
     emit changed();
@@ -268,7 +281,7 @@ QDomDocument Part::saveXML()
 
     // Save the project
     m_project->save( doc );
-    
+
     return document;
 }
 
@@ -299,11 +312,11 @@ QDomDocument Part::saveWorkPackageXML( const Node *node, long id, Resource *reso
 
     // Save the project
     m_project->saveWorkPackageXML( doc, node, id );
-    
+
     return document;
 }
 
-bool Part::saveWorkPackageToStream( QIODevice * dev, const Node *node, long id, Resource *resource )
+bool Part::saveWorkPackageToStream( QIODevice *dev, const Node *node, long id, Resource *resource )
 {
     QDomDocument doc = saveWorkPackageXML( node, id, resource );
     // Save to buffer
@@ -330,8 +343,8 @@ bool Part::saveWorkPackageFormat( const QString &file, const Node *node, long id
 
     QByteArray mimeType = "application/x-vnd.kde.kplato.work";
     kDebug() <<"MimeType=" << mimeType;
-    
-    KoStore* store = KoStore::createStore( file, KoStore::Write, mimeType, backend );
+
+    KoStore *store = KoStore::createStore( file, KoStore::Write, mimeType, backend );
 /*    if ( d->m_specialOutputFlag == SaveEncrypted && !d->m_password.isNull( ) ) {
         store->setPassword( d->m_password );
     }*/
@@ -355,7 +368,7 @@ bool Part::saveWorkPackageFormat( const QString &file, const Node *node, long id
         return false;
     }
     node->documents().saveToStore( store );
-    
+
     kDebug() <<"Saving done of url:" << file;
     if ( !store->finalize() ) {
         delete store;
@@ -367,7 +380,7 @@ bool Part::saveWorkPackageFormat( const QString &file, const Node *node, long id
     return true;
 }
 
-bool Part::saveWorkPackageUrl( const KUrl & _url, const Node *node, long id, Resource *resource )
+bool Part::saveWorkPackageUrl( const KUrl &_url, const Node *node, long id, Resource *resource )
 {
     //kDebug()<<_url;
     QApplication::setOverrideCursor( Qt::WaitCursor );
@@ -381,12 +394,11 @@ bool Part::saveWorkPackageUrl( const KUrl & _url, const Node *node, long id, Res
 
 bool Part::loadWorkPackage( Project &project, const KUrl &url )
 {
-    //qDebug()<<"loadWorkPackage:"<<url;
     if ( ! url.isLocalFile() ) {
         kDebug()<<"TODO: download if url not local";
         return false;
     }
-    KoStore * store = KoStore::createStore( url.path(), KoStore::Read, "", KoStore::Auto );
+    KoStore *store = KoStore::createStore( url.path(), KoStore::Read, "", KoStore::Auto );
     if ( store->bad() ) {
 //        d->lastErrorMessage = i18n( "Not a valid KOffice file: %1", file );
         kDebug()<<"bad store"<<url.prettyUrl();
@@ -433,11 +445,11 @@ Project *Part::loadWorkPackageXML( Project &project, QIODevice *, const KoXmlDoc
     // Check if this is the right app
     value = plan.attribute( "mime", QString() );
     if ( value.isEmpty() ) {
-        qDebug() << "No mime type specified!";
+        kDebug() << "No mime type specified!";
         setErrorMessage( i18n( "Invalid document. No mimetype specified." ) );
         return false;
     } else if ( value != "application/x-vnd.kde.kplato.work" ) {
-        qDebug() << "Unknown mime type " << value;
+        kDebug() << "Unknown mime type " << value;
         setErrorMessage( i18n( "Invalid document. Expected mimetype application/x-vnd.kde.kplato.work, got %1", value ) );
         return false;
     }
@@ -454,13 +466,13 @@ Project *Part::loadWorkPackageXML( Project &project, QIODevice *, const KoXmlDoc
         }
     }
     m_xmlLoader.setVersion( plan.attribute( "kplato-version", KPLATO_FILE_SYNTAX_VERSION ) );
-
+/*
 #ifdef KOXML_USE_QDOM
     int numNodes = plan.childNodes().count();
 #else
     int numNodes = plan.childNodesCount();
 #endif
-
+*/
     bool ok = true;
     QString timeTag;
     m_xmlLoader.startLoad();
@@ -507,16 +519,13 @@ Project *Part::loadWorkPackageXML( Project &project, QIODevice *, const KoXmlDoc
     if ( ok && proj->id() == project.id() && proj->childNode( 0 ) ) {
         ok = project.nodeDict().contains( proj->childNode( 0 )->id() );
         if ( ok && m_mergedPackages.contains( timeTag ) ) {
-            qDebug()<<"loadWorkPackageXML:"<<"Already merged:"<<timeTag<<proj->name()<<proj->childNode( 0 )->name();
             ok = false; // already merged
         }
         if ( ok && ! timeTag.isEmpty() && ! m_mergedPackages.contains( timeTag ) ) {
             m_mergedPackages[ timeTag ] = proj; // register this for next time
-            qDebug()<<"loadWorkPackageXML:"<<"New package:"<<timeTag<<proj->name()<<proj->childNode( 0 )->name();
         }
         if ( ok && timeTag.isEmpty() ) {
             kWarning()<<"Work package is not time tagged";
-            qDebug()<<"loadWorkPackageXML:"<<"Old package format:"<<timeTag<<proj->name()<<proj->childNode( 0 )->name();
             ok = false;
         }
     }
@@ -531,9 +540,7 @@ Project *Part::loadWorkPackageXML( Project &project, QIODevice *, const KoXmlDoc
 
 void Part::checkForWorkPackages()
 {
-    //qDebug()<<"checkForWorkPackages:";
     if ( ! isReadWrite() || ! m_config.checkForWorkPackages() || m_config.retrieveUrl().isEmpty() || m_project == 0 || m_project->numChildren() == 0 ) {
-        //qDebug()<<"checkForWorkPackages: idle:"<<isReadWrite()<<m_config.checkForWorkPackages()<<m_config.retrieveUrl()<<m_project <<(m_project?m_project->numChildren():0);
         QTimer::singleShot ( 10000, this, SLOT( checkForWorkPackages() ) );
         return;
     }
@@ -545,7 +552,6 @@ void Part::checkForWorkPackages()
 
 void Part::checkForWorkPackage()
 {
-    qDebug()<<"checkForWorkPackage: files ="<<m_infoList.count();
     if ( ! m_infoList.isEmpty() ) {
         loadWorkPackage( *m_project, KUrl( m_infoList.takeLast().absoluteFilePath() ) );
         if ( ! m_infoList.isEmpty() ) {
@@ -566,7 +572,7 @@ void Part::checkForWorkPackage()
         // Merge our workpackages
         if ( ! m_workpackages.isEmpty() ) {
             QStringList lst;
-            foreach ( Package *p, m_workpackages.keys() ) {
+            foreach ( const Package *p, m_workpackages.keys() ) {
                 lst << QString( "%1: %2" ).arg( static_cast<Task*>( p->project->childNode( 0 ) )->workPackage().ownerName() ).arg( p->project->childNode( 0 )->name() );
             }
             int r = KMessageBox::questionYesNoList( 0, "New work packages detected. Merge data with existing tasks?", lst );
@@ -577,7 +583,6 @@ void Part::checkForWorkPackage()
             m_workpackages.clear();
         }
     }
-    qDebug()<<"checkForWorkPackage: start again:";
     QTimer::singleShot ( 10000, this, SLOT( checkForWorkPackages() ) );
 }
 
@@ -605,22 +610,19 @@ void Part::mergeWorkPackage( const Package *package )
             return;
         }
         if ( KPlatoSettings::deleteFile() ) {
-            qDebug()<<"mergeWorkPackage: remove file"<<file.fileName();
             file.remove();
         } else if ( KPlatoSettings::saveFile() && ! KPlatoSettings::saveUrl(). isEmpty() ) {
             QDir dir( KPlatoSettings::saveUrl().path() );
             if ( ! dir.exists() ) {
                 if ( ! dir.mkpath( dir.path() ) ) {
                     //TODO message
-                    qDebug()<<"mergeWorkPackage: could not create directory:"<<dir.path();
+                    kDebug()<<"Could not create directory:"<<dir.path();
                     return;
                 }
             }
             QFileInfo from( file );
             QString name = KPlatoSettings::saveUrl().path() + '/' + from.fileName();
-            qDebug()<<"mergeWorkPackage: rename file"<<file.fileName();
             if ( file.rename( name ) ) {
-                qDebug()<<"mergeWorkPackage: Saved"<<file.fileName()<<"to"<<name;
                 return;
             }
             name = KPlatoSettings::saveUrl().path() + '/';
@@ -633,13 +635,10 @@ void Part::mergeWorkPackage( const Package *package )
             while ( ! ok && i < 1000 ) {
                 ++i;
                 ok = QFile::rename( file.fileName(), name.arg( i ) );
-                //qDebug()<<"mergeWorkPackage: tried to save"<<file.fileName()<<"to"<<name.arg(i);
             }
             if ( ! ok ) {
                 //TODO message
-                qDebug()<<"mergeWorkPackage: Failed to save"<<file.fileName();
-            } else {
-                qDebug()<<"mergeWorkPackage: Saved"<<file.fileName()<<"to"<<name.arg(i);
+                kDebug()<<"mergeWorkPackage: Failed to save"<<file.fileName();
             }
         }
     }
@@ -724,7 +723,6 @@ void Part::mergeWorkPackage( Task *to, const Task *from, const Package *package
     }
     wp->setTransmitionStatus( WorkPackage::TS_Receive );
     cmd->addCommand( new WorkPackageAddCmd( m_project, to, wp ) );
-    qDebug()<<"mergeWorkPackage:"<<from->name()<<to->name()<<cmd->isEmpty();
     if ( cmd->isEmpty() ) {
         KMessageBox::information( 0, i18n( "Nothing to save from this package" ) );
         delete cmd;
@@ -748,7 +746,7 @@ void Part::activate( QWidget *w )
         manager()->setActivePart( this, w );
 }
 
-void Part::openTemplate( const KUrl& url )
+void Part::openTemplate( const KUrl &url )
 {
     //kDebug()<<url;
     // HACK because we can't really reimplemt openTemplate() (private methods)
@@ -763,9 +761,15 @@ bool Part::completeLoading( KoStore *store )
     if ( m_loadingTemplate ) {
         //kDebug()<<"Loading template, generate unique ids";
         m_project->generateUniqueIds();
+        m_project->setConstraintStartTime( KDateTime( KDateTime::currentLocalDateTime().date(), QTime( 0, 0, 0 ) ) );
+        m_project->setConstraintEndTime( m_project->constraintStartTime().addYears( 2 ) );
     } else if ( isImporting() ) {
-        //kDebug()<<"Importing, generate unique node ids";
-        m_project->generateUniqueNodeIds();
+        // NOTE: I don't think this is a good idea.
+        // Let the filter generate ids for non-kplato files.
+        // If the user wants to create a new project from an old one,
+        // he should use Tools -> Insert Project File
+
+        //m_project->generateUniqueNodeIds();
     }
     if ( store == 0 ) {
         // can happen if loading a template
@@ -784,26 +788,26 @@ bool Part::completeLoading( KoStore *store )
 
 bool Part::completeSaving( KoStore *store )
 {
-    delete m_context;
-    m_context = 0;
     // Seems like a hack, but imo the best to do
     View *view = dynamic_cast<View*>( views().value( 0 ) );
     if ( view ) {
-        m_context = new Context();
-        m_context->save( view );
         if ( store->open( "context.xml" ) ) {
+            if ( m_context == 0 ) m_context = new Context();
             QDomDocument doc = m_context->save( view );
-            KoStoreDevice dev( store );
 
+            KoStoreDevice dev( store );
             QByteArray s = doc.toByteArray(); // this is already Utf8!
             (void)dev.write( s.data(), s.size() );
             (void)store->close();
+
+            m_viewlistModified = false;
+            emit viewlistModified( false );
         }
     }
     return true;
 }
 
-bool Part::loadAndParse(KoStore* store, const QString& filename, KoXmlDocument& doc)
+bool Part::loadAndParse(KoStore *store, const QString &filename, KoXmlDocument &doc)
 {
     //kDebug() << "oldLoadAndParse: Trying to open " << filename;
 
@@ -835,30 +839,51 @@ bool Part::loadAndParse(KoStore* store, const QString& filename, KoXmlDocument&
 
 void Part::insertFile( const QString &filename, Node *parent, Node *after )
 {
-    qDebug()<<"Part::insertFile:"<<filename<<parent->name()<<(after?after->name():"nil");
-    Part part;
-    if ( part.openUrl( KUrl( filename ) ) ) {
-        part.setReadWrite( false );
-        Project &p = part.getProject();
-        qDebug()<<"Part::insertFile:"<<p.name();
-        insertProject( p, parent, after );
+    Part *part = new Part();
+    part->disconnect(); // part shall not handle feedback from openUrl()
+    part->setAutoSave( 0 ); //disable
+    part->m_insertFileInfo.url = filename;
+    part->m_insertFileInfo.parent = parent;
+    part->m_insertFileInfo.after = after;
+    connect(part, SIGNAL(completed()), SLOT(insertFileCompleted()));
+    connect(part, SIGNAL(canceled(const QString&)), SLOT(insertFileCancelled(const QString&)));
+    connect(part, SIGNAL(started(KIO::Job*)), SLOT(slotStarted(KIO::Job*)));
+
+    part->openUrl( KUrl( filename ) );
+}
+
+void Part::insertFileCompleted()
+{
+    Part *part = qobject_cast<Part*>( sender() );
+    if ( part ) {
+        Project &p = part->getProject();
+        insertProject( p, part->m_insertFileInfo.parent, part->m_insertFileInfo.after );
+        part->deleteLater();
+    }
+}
+
+void Part::insertFileCancelled( const QString &error )
+{
+    if ( ! error.isEmpty() ) {
+        KMessageBox::error( 0, error );
+    }
+    Part *part = qobject_cast<Part*>( sender() );
+    if ( part ) {
+        part->deleteLater();
     }
 }
 
 bool Part::insertProject( Project &project, Node *parent, Node *after )
 {
-    qDebug()<<"Part::insertProject:";
     // make sure node ids in new project is unique also in old project
     QList<QString> existingIds = m_project->nodeDict().keys();
     foreach ( Node *n, project.allNodes() ) {
-        if ( ! n->id().isEmpty() && ! m_project->findNode( n->id() ) ) {
-            continue; // id is ok to use in m_project
-        }
-        bool res = n->setId( project.uniqueNodeId( existingIds ) );
-        Q_ASSERT( res );
+        QString oldid = n->id();
+        n->setId( project.uniqueNodeId( existingIds ) );
+        project.removeId( oldid ); // remove old id
+        project.registerNodeId( n ); // register new id
     }
-    qDebug()<<"Part::insertProject:"<<project.childNodeIterator();
-    MacroCommand *m = new InsertProjectCmd( project, parent==0?m_project:parent, after, i18n( "Insert project nodes" ) );
+    MacroCommand *m = new InsertProjectCmd( project, parent==0?m_project:parent, after, i18n( "Insert project" ) );
     if ( m->isEmpty() ) {
         delete m;
     } else {
@@ -867,6 +892,46 @@ bool Part::insertProject( Project &project, Node *parent, Node *after )
     return true;
 }
 
+void Part::insertViewListItem( View *view, const ViewListItem *item, const ViewListItem *parent, int index )
+{
+    foreach ( KoView *v, views() ) {
+        View *vv = dynamic_cast<View*>( v );
+        if ( vv == 0 || vv == view ) {
+            continue;
+        }
+        vv->addViewListItem( item, parent, index );
+    }
+    setModified( true );
+    m_viewlistModified = true;
+}
+
+void Part::removeViewListItem( View *view, const ViewListItem *item )
+{
+    foreach ( KoView *v, views() ) {
+        View *vv = dynamic_cast<View*>( v );
+        if ( vv == 0 || vv == view ) {
+            continue;
+        }
+        vv->removeViewListItem( item );
+    }
+    setModified( true );
+    m_viewlistModified = true;
+}
+
+void Part::slotModified( bool mod )
+{
+    if ( ! mod && m_viewlistModified ) {
+        setModified( true );
+    }
+}
+
+void Part::viewlistModified()
+{
+    if ( ! m_viewlistModified ) {
+        m_viewlistModified = true;
+        setModified( true );
+    }
+}
 
 }  //KPlato namespace
 
