@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
   Copyright (C) 1998, 1999, 2000 Torben Weis <weis@kde.org>
-  Copyright (C) 2002 - 2007 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2002 - 2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -30,10 +30,12 @@
 #include <QMenu>
 #include <QDockWidget>
 #include <QTimer>
+#include <QMap>
 
 class QProgressBar;
 class QStackedWidget;
 class QSplitter;
+class QUndoCommand;
 
 class KAction;
 class KToggleAction;
@@ -48,7 +50,7 @@ class View;
 class ViewBase;
 class ViewListItem;
 class ViewListWidget;
-class ViewInfo;
+struct ViewInfo;
 class AccountsView;
 class GanttView;
 class PertEditor;
@@ -58,7 +60,7 @@ class TaskEditor;
 class CalendarEditor;
 class ScheduleEditor;
 class ScheduleManager;
-
+class CalculateScheduleCmd;
 class ResourceAssignmentView;
 class TaskStatusView;
 class Calendar;
@@ -74,6 +76,9 @@ class Relation;
 class Context;
 class ViewAdaptor;
 class HtmlView;
+class ReportView;
+
+class ReportDesignDialog;
 
 //-------------
 class KPLATO_EXPORT View : public KoView
@@ -128,6 +133,7 @@ public:
     ViewBase *createAccountsView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
     ViewBase *createResourceAssignmentView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
     ViewBase *createChartView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createReportView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
 
     KoPrintJob * createPrintJob();
 
@@ -141,16 +147,17 @@ signals:
 public slots:
     void slotUpdate();
     void slotCreateTemplate();
-    void slotAddResource( ResourceGroup *group );
     void slotEditResource();
     void slotEditCut();
     void slotEditCopy();
     void slotEditPaste();
+    void slotRefreshView();
     void slotViewSelector( bool show );
 
     void slotAddTask();
     void slotAddSubTask();
     void slotAddMilestone();
+    void slotAddSubMilestone();
     void slotProjectEdit();
     void slotDefineWBS();
     void slotCurrencyConfig();
@@ -172,6 +179,9 @@ public slots:
     void slotPopupMenu( const QString& menuname, const QPoint &pos );
     void slotPopupMenu( const QString& menuname, const QPoint &pos, ViewListItem *item );
 
+    void addViewListItem( const ViewListItem *item, const ViewListItem *parent, int index );
+    void removeViewListItem( const ViewListItem *item );
+
 protected slots:
     void slotGuiActivated( ViewBase *view, bool );
     void slotViewActivated( ViewListItem*, ViewListItem* );
@@ -185,6 +195,8 @@ protected slots:
     void slotAddScheduleManager( Project *project );
     void slotDeleteScheduleManager( Project *project, ScheduleManager *sm );
     void slotCalculateSchedule( Project*, ScheduleManager* );
+    void slotCalculationStarted( Project *project, ScheduleManager *sm );
+    void slotCalculationFinished( Project *project, ScheduleManager *sm );
     void slotBaselineSchedule( Project *project, ScheduleManager *sm );
     void slotProgressChanged( int value );
 
@@ -224,6 +236,11 @@ protected slots:
 
     void slotUpdateViewInfo( ViewListItem *itm );
 
+    void slotEditReportDesign( ReportView *view );
+    void slotCreateReport();
+    void slotOpenReportFile();
+    void slotModifyReportDefinition( QUndoCommand *cmd );
+
 #ifndef NDEBUG
     void slotPrintDebug();
     void slotPrintSelectedDebug();
@@ -259,10 +276,41 @@ protected:
 private slots:
     void slotActionDestroyed( QObject *o );
     void slotViewListItemRemoved( ViewListItem *item );
-    void slotViewListItemInserted( ViewListItem *item );
+    void slotViewListItemInserted( ViewListItem *item, ViewListItem *parent, int index );
+
+    void slotCreateReportView( ReportDesignDialog *dlg );
+
+    void slotProjectEditFinished( int result );
+    void slotTaskEditFinished( int result );
+    void slotSummaryTaskEditFinished( int result );
+    void slotEditResourceFinished( int result );
+    void slotProjectWorktimeFinished( int result );
+    void slotDefineWBSFinished( int result );
+    void slotCurrencyConfigFinished( int result );
+    void slotInsertFileFinished( int result );
+    void slotAddSubTaskFinished( int result );
+    void slotAddTaskFinished( int result );
+    void slotAddSubMilestoneFinished( int result );
+    void slotAddMilestoneFinished( int result );
+    void slotTaskProgressFinished( int result );
+    void slotMilestoneProgressFinished( int result );
+    void slotTaskDescriptionFinished( int result );
+    void slotAddRelationFinished( int result );
+    void slotModifyRelationFinished( int result );
+    void slotReportDesignFinished( int result );
+    void slotOpenReportFileFinished( int result );
+    void slotCreateViewFinished( int result );
+
+    void slotRemoveCommands();
+
+    void slotMaxProgress( int p );
+    void slotSetProgress( int p );
 
 private:
     void createViews();
+    ViewBase *createView( ViewListItem *cat, const QString &type, const QString &tag, const QString &name, const QString &tip, int index = -1 );
+
+    QString standardTaskStatusReport() const;
 
 private:
     QSplitter *m_sp;
@@ -291,6 +339,9 @@ private:
     QActionGroup *m_scheduleActionGroup;
     QMap<QAction*, Schedule*> m_scheduleActions;
 
+    QMultiMap<ScheduleManager*, CalculateScheduleCmd*> m_calculationcommands;
+    QList<QUndoCommand*> m_undocommands;
+
     bool m_readWrite;
 
     // ------ File
@@ -314,6 +365,9 @@ private:
     KAction *actionInsertFile;
     KAction *actionCurrencyConfig;
 
+    KAction *actionCreateReport;
+    KAction *actionOpenReportFile;
+
     // ------ Settings
     KAction *actionConfigure;
 
@@ -334,11 +388,10 @@ private:
     KAction *actionEditRelation;
     KAction *actionDeleteRelation;
 
-    KAction *actionMailWorkpackage;
-
     //Test
     KAction *actNoInformation;
 
+    QMap<ViewListItem*, QAction*> m_reportActionMap;
 };
 
 } //Kplato namespace
