@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-  Copyright (C) 2007 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2007 -2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -27,16 +27,20 @@
 #include <QStyle>
 #include <QHeaderView>
 #include <QBrush>
+#include <QContextMenuEvent>
+#include <QTimer>
 
 #include <kmenu.h>
+#include <KMessageBox>
+#include <KComboBox>
 
 #include "KoDocument.h"
 
-#include "kptviewlistcommand.h"
 #include "kptviewbase.h"
 #include "kptpart.h"
 #include "kptviewlistdialog.h"
 #include "kptviewlistdocker.h"
+#include "kptschedulemodel.h"
 
 #include <assert.h>
 
@@ -52,12 +56,23 @@ class ViewCategoryDelegate : public QItemDelegate
         m_view( view )
         {}
 
+        QSize sizeHint ( const QStyleOptionViewItem & option, const QModelIndex & index ) const;
         virtual void paint( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const;
 
     private:
         QTreeView *m_view;
 };
 
+QSize ViewCategoryDelegate::sizeHint ( const QStyleOptionViewItem & option, const QModelIndex & index ) const
+{
+    const QAbstractItemModel * model = index.model();
+    Q_ASSERT( model );
+    if ( model->parent( index ).isValid() ) {
+        return QItemDelegate::sizeHint( option, index );
+    }
+    return QItemDelegate::sizeHint( option, index ).expandedTo( QSize( 0, 16 ) );
+}
+
 void ViewCategoryDelegate::paint( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
 {
     const QAbstractItemModel * model = index.model();
@@ -89,7 +104,7 @@ void ViewCategoryDelegate::paint( QPainter * painter, const QStyleOptionViewItem
         QRect textrect = QRect( r.left() + i * 2, r.top(), r.width() - ( ( 5 * i ) / 2 ), r.height() );
         QString text = elidedText( option.fontMetrics, textrect.width(), Qt::ElideMiddle,
                                 model->data( index, Qt::DisplayRole ).toString() );
-        m_view->style() ->drawItemText( painter, textrect, Qt::AlignLeft,
+        m_view->style() ->drawItemText( painter, textrect, Qt::AlignLeft|Qt::AlignVCenter,
         option.palette, m_view->isEnabled(), text );
 
     } else {
@@ -123,15 +138,15 @@ void ViewListItem::setReadWrite( bool rw )
     }
 }
 
-void ViewListItem::setView( KoView *view )
+void ViewListItem::setView( ViewBase *view )
 {
-    setData( 0, ViewListItem::DataRole_View,  qVariantFromValue(static_cast<QWidget*>( view ) ) );
+    setData( 0, ViewListItem::DataRole_View,  qVariantFromValue(static_cast<QObject*>( view ) ) );
 }
 
-KoView *ViewListItem::view() const
+ViewBase *ViewListItem::view() const
 {
     if ( data(0, ViewListItem::DataRole_View ).isValid() ) {
-        return static_cast<KoView*>( data(0, ViewListItem::DataRole_View ).value<QWidget*>() );
+        return static_cast<ViewBase*>( data(0, ViewListItem::DataRole_View ).value<QObject*>() );
     }
     return 0;
 }
@@ -149,38 +164,43 @@ KoDocument *ViewListItem::document() const
     return 0;
 }
 
-void ViewListItem::setDocumentChild( DocumentChild *child )
+QString ViewListItem::viewType() const
 {
-    setData( 0, ViewListItem::DataRole_ChildDocument,  qVariantFromValue(static_cast<QObject*>( child ) ) );
-}
-
-DocumentChild *ViewListItem::documentChild() const
-{
-    if ( data(0, ViewListItem::DataRole_ChildDocument ).isValid() ) {
-        return static_cast<DocumentChild*>( data(0, ViewListItem::DataRole_ChildDocument ).value<QObject*>() );
+    if ( type() != ItemType_SubView ) {
+        return QString();
     }
-    return 0;
+    QString name = view()->metaObject()->className();
+    if ( name.contains( ':' ) ) {
+        name = name.remove( 0, name.lastIndexOf( ':' ) + 1 );
+    }
+    return name;
 }
 
 void ViewListItem::save( QDomElement &element ) const
 {
     element.setAttribute( "itemtype", type() );
     element.setAttribute( "tag", tag() );
-    element.setAttribute( "name", text( 0 ) );
-    element.setAttribute( "tooltip", toolTip( 0 ) );
+    element.setAttribute( "name", m_viewinfo.name == text( 0 ) ? "" : text( 0 ) );
+    element.setAttribute( "tooltip", m_viewinfo.tip == toolTip( 0 ) ? TIP_USE_DEFAULT_TEXT : toolTip( 0 ) );
+
     if ( type() == ItemType_SubView ) {
-        QString name = view()->metaObject()->className();
-        if ( name.contains( ':' ) ) {
-            name = name.remove( 0, name.lastIndexOf( ':' ) + 1 );
-        }
-        kDebug()<<view()->metaObject()->className()<<" -> "<<name;
-        element.setAttribute( "viewtype", name );
+        element.setAttribute( "viewtype", viewType() );
     }
 }
 
 ViewListTreeWidget::ViewListTreeWidget( QWidget *parent )
     : QTreeWidget( parent )
 {
+    setWhatsThis( i18nc( "@info:whatsthis",
+        "<para>This is the list of available views and editors.</para>"
+        "<para>You can configure the list by using the context menu:"
+        "<list>"
+        "<item>Rename categories or views</item>"
+        "<item>Configure. Move, remove, rename or edit tool tip for categories or views</item>"
+        "<item>Insert categories and views</item>"
+        "</list>"
+    ) );
+
     header() ->hide();
     setRootIsDecorated( false );
     setItemDelegate( new ViewCategoryDelegate( this, this ) );
@@ -237,6 +257,7 @@ void ViewListTreeWidget::save( QDomElement &element ) const
         }
         QDomElement c = cs.ownerDocument().createElement( "category" );
         cs.appendChild( c );
+        emit const_cast<ViewListTreeWidget*>( this )->updateViewInfo( itm );
         itm->save( c );
         for ( int j = 0; j < itm->childCount(); ++j ) {
             ViewListItem *vi = static_cast<ViewListItem*>( itm->child( j ) );
@@ -245,6 +266,7 @@ void ViewListTreeWidget::save( QDomElement &element ) const
             }
             QDomElement el = c.ownerDocument().createElement( "view" );
             c.appendChild( el );
+            emit const_cast<ViewListTreeWidget*>( this )->updateViewInfo( vi );
             vi->save( el );
             QDomElement elm = el.ownerDocument().createElement( "settings" );
             el.appendChild( elm );
@@ -287,20 +309,39 @@ ViewListItem *ViewListTreeWidget::category( const KoView *view ) const
 ViewListWidget::ViewListWidget( Part *part, QWidget *parent )//QString name, KXmlGuiWindow *parent )
     : QWidget( parent ),
     m_part( part ),
-    m_prev( 0 )
+    m_prev( 0 ),
+    m_temp( 0 )
 {
     setObjectName("ViewListWidget");
     m_viewlist = new ViewListTreeWidget( this );
+    m_viewlist->setEditTriggers( QAbstractItemView::NoEditTriggers );
+
+    m_currentSchedule = new KComboBox( this );
+    m_model.setFlat( true );
+
+    m_sfModel.setFilterKeyColumn ( ScheduleModel::ScheduleScheduled );
+    m_sfModel.setFilterRole( Qt::EditRole );
+    m_sfModel.setFilterFixedString( "true" );
+    m_sfModel.setDynamicSortFilter ( true );
+    m_sfModel.setSourceModel( &m_model );
+    m_currentSchedule->setModel( &m_sfModel );
+
     QVBoxLayout *l = new QVBoxLayout( this );
     l->setMargin( 0 );
     l->addWidget( m_viewlist );
-    m_viewlist->setEditTriggers( QAbstractItemView::EditKeyPressed | QAbstractItemView::DoubleClicked );
+    l->addWidget( m_currentSchedule );
 
     connect( m_viewlist, SIGNAL( currentItemChanged( QTreeWidgetItem*, QTreeWidgetItem* ) ), SLOT( slotActivated( QTreeWidgetItem*, QTreeWidgetItem* ) ) );
 
     connect( m_viewlist, SIGNAL( itemChanged( QTreeWidgetItem*, int ) ), SLOT( slotItemChanged( QTreeWidgetItem*, int ) ) );
 
     setupContextMenus();
+
+    connect( m_currentSchedule, SIGNAL( activated( int ) ), SLOT( slotCurrentScheduleChanged( int ) ) );
+
+    connect( &m_model, SIGNAL( scheduleManagerAdded( ScheduleManager* ) ), SLOT( slotScheduleManagerAdded( ScheduleManager* ) ) );
+
+    connect( m_viewlist, SIGNAL( updateViewInfo( ViewListItem* ) ), SIGNAL( updateViewInfo( ViewListItem* ) ) );
 }
 
 ViewListWidget::~ViewListWidget()
@@ -316,32 +357,25 @@ void ViewListWidget::setReadWrite( bool rw )
     }
 }
 
-void ViewListWidget::slotItemChanged( QTreeWidgetItem *item, int col )
+void ViewListWidget::slotItemChanged( QTreeWidgetItem */*item*/, int /*col */)
 {
     //kDebug();
-    if ( item && item->type() == ViewListItem::ItemType_ChildDocument && col == 0 ) {
-        DocumentChild *ch = static_cast<ViewListItem*>(item)->documentChild();
-        if ( ch ) {
-            ch->setTitle( item->text( 0 ) );
-            //kDebug()<<ch->title();
-        }
-    }
 }
 
 void ViewListWidget::slotActivated( QTreeWidgetItem *item, QTreeWidgetItem *prev )
 {
-    //kDebug();
-    if ( item == 0 || item->type() == ViewListItem::ItemType_Category ) {
+    if ( m_prev ) {
+        m_prev->setData( 0, Qt::BackgroundRole, QVariant() );
+    }
+    if ( item && item->type() == ViewListItem::ItemType_Category ) {
         return ;
     }
-    QVariant v;
-    if ( m_prev ) {
-        m_prev->setData( 0, Qt::BackgroundRole, v );
-    }
     emit activated( static_cast<ViewListItem*>( item ), static_cast<ViewListItem*>( prev ) );
-    v = QBrush( QColor( Qt::yellow ) );
-    item->setData( 0, Qt::BackgroundRole, v );
-    m_prev = item;
+    if ( item ) {
+        QVariant v = QBrush( QColor( Qt::yellow ) );
+        item->setData( 0, Qt::BackgroundRole, v );
+        m_prev = static_cast<ViewListItem*>( item );
+    }
 }
 
 ViewListItem *ViewListWidget::addCategory( const QString &tag, const QString& name )
@@ -381,7 +415,6 @@ ViewListItem *ViewListWidget::category( const KoView *view ) const
 
 QString ViewListWidget::uniqueTag( const QString &seed ) const
 {
-    //kDebug() << endl;
     QString tag = seed;
     for ( int i = 1; findItem( tag ); ++i ) {
         tag = QString("%1-%2").arg( seed ).arg( i );
@@ -389,7 +422,7 @@ QString ViewListWidget::uniqueTag( const QString &seed ) const
     return tag;
 }
 
-ViewListItem *ViewListWidget::addView( QTreeWidgetItem *category, const QString &tag, const QString& name, KoView *view, KoDocument *doc, const QString& icon, int index )
+ViewListItem *ViewListWidget::addView( QTreeWidgetItem *category, const QString &tag, const QString& name, ViewBase *view, KoDocument *doc, const QString& icon, int index )
 {
     ViewListItem * item = new ViewListItem( uniqueTag( tag ), QStringList( name ), ViewListItem::ItemType_SubView );
     item->setView( view );
@@ -398,35 +431,32 @@ ViewListItem *ViewListWidget::addView( QTreeWidgetItem *category, const QString
         item->setData( 0, Qt::DecorationRole, KIcon( icon ) );
     }
     item->setFlags( item->flags() | Qt::ItemIsEditable );
-    category->insertChild( ( index == -1 ? category->childCount() : index ), item );
-    //kDebug() << "added: " << item->tag();
-    return item;
-}
+    insertViewListItem( item, category, index );
+
+    connect(view, SIGNAL(optionsModified()), SLOT(setModified()));
 
-ViewListItem *ViewListWidget::createChildDocumentView( const QString& tag, const QString& name, KoView *view, DocumentChild *ch, const QString& icon )
-{
-    ViewListItem * item = new ViewListItem( uniqueTag( tag ), QStringList( name ), ViewListItem::ItemType_ChildDocument );
-    item->setView( view );
-    item->setDocument( ch->document() );
-    item->setDocumentChild( ch );
-    if ( !icon.isEmpty() ) {
-        item->setData( 0, Qt::DecorationRole, KIcon( icon ) );
-    }
-    item->setFlags( item->flags() | Qt::ItemIsEditable );
-    //kDebug() <<"added:" << item;
     return item;
 }
 
 void ViewListWidget::setSelected( QTreeWidgetItem *item )
 {
     //kDebug()<<item<<","<<m_viewlist->currentItem();
-    if ( item == 0 ) {
-        return;
+    if ( item == 0 && m_viewlist->currentItem() ) {
+        m_viewlist->currentItem()->setSelected( false );
+        if ( m_prev ) {
+            m_prev->setData( 0, Qt::BackgroundRole, QVariant() );
+        }
     }
     m_viewlist->setCurrentItem( item );
     //kDebug()<<item<<","<<m_viewlist->currentItem();
 }
 
+void ViewListWidget::setCurrentItem( QTreeWidgetItem *item )
+{
+    m_viewlist->setCurrentItem( item );
+    //kDebug()<<item<<","<<m_viewlist->currentItem();
+}
+
 ViewListItem *ViewListWidget::currentItem() const
 {
     return static_cast<ViewListItem*>( m_viewlist->currentItem() );
@@ -487,7 +517,7 @@ ViewListItem *ViewListWidget::findItem( const QString &tag, QTreeWidgetItem *par
     return 0;
 }
 
-ViewListItem *ViewListWidget::findItem(  const QWidget *view, QTreeWidgetItem *parent ) const
+ViewListItem *ViewListWidget::findItem(  const ViewBase *view, QTreeWidgetItem *parent ) const
 {
     if ( parent == 0 ) {
         return findItem( view, m_viewlist->invisibleRootItem() );
@@ -506,14 +536,6 @@ ViewListItem *ViewListWidget::findItem(  const QWidget *view, QTreeWidgetItem *p
     return 0;
 }
 
-void ViewListWidget::slotCreatePart()
-{
-    const QString entryPath = sender()->objectName();
-    KService::Ptr serv = KService::serviceByDesktopPath( entryPath );
-    KoDocumentEntry entry = KoDocumentEntry( serv );
-    emit createKofficeDocument( entry );
-}
-
 void ViewListWidget::slotAddView()
 {
     emit createView();
@@ -521,19 +543,30 @@ void ViewListWidget::slotAddView()
 
 void ViewListWidget::slotRemoveCategory()
 {
-    if ( m_contextitem && m_contextitem->type() == ViewListItem::ItemType_Category ) {
-        kDebug()<<m_contextitem<<":"<<m_contextitem->type();
-        takeViewListItem( m_contextitem );
-        delete m_contextitem;
-        m_contextitem = 0;
+    if ( m_contextitem == 0 ) {
+        return;
     }
+    if ( m_contextitem->type() != ViewListItem::ItemType_Category ) {
+        return;
+    }
+    kDebug()<<m_contextitem<<":"<<m_contextitem->type();
+    if ( m_contextitem->childCount() > 0 ) {
+        if ( KMessageBox::warningContinueCancel( this, i18n( "Removing this category will also remove all its views." ) ) == KMessageBox::Cancel ) {
+            return;
+        }
+    }
+    takeViewListItem( m_contextitem );
+    delete m_contextitem;
+    emit modified();
+    m_contextitem = 0;
 }
 
 void ViewListWidget::slotRemoveView()
 {
     if ( m_contextitem ) {
-        kDebug()<<m_contextitem<<":"<<m_contextitem->type();
         takeViewListItem( m_contextitem );
+        delete m_contextitem;
+        emit modified();
     }
 }
 
@@ -542,7 +575,11 @@ void ViewListWidget::slotEditViewTitle()
     //QTreeWidgetItem *item = m_viewlist->currentItem();
     if ( m_contextitem ) {
         kDebug()<<m_contextitem<<":"<<m_contextitem->type();
+        QString title = m_contextitem->text( 0 );
         m_viewlist->editItem( m_contextitem );
+        if ( title != m_contextitem->text( 0 ) ) {
+            emit modified();
+        }
     }
 }
 
@@ -551,19 +588,28 @@ void ViewListWidget::slotConfigureItem()
     if ( m_contextitem == 0 ) {
         return;
     }
+    KDialog *dlg = 0;
     if ( m_contextitem->type() == ViewListItem::ItemType_Category ) {
         kDebug()<<m_contextitem<<":"<<m_contextitem->type();
-        ViewListEditCategoryDialog *dlg = new ViewListEditCategoryDialog( *this, m_contextitem, this );
-        dlg->exec();
-        delete dlg;
+        dlg = new ViewListEditCategoryDialog( *this, m_contextitem, this );
     } else if ( m_contextitem->type() == ViewListItem::ItemType_SubView ) {
-        ViewListEditViewDialog *dlg = new ViewListEditViewDialog( *this, m_contextitem, this );
-        dlg->exec();
-        delete dlg;
-    } else if ( m_contextitem->type() == ViewListItem::ItemType_ChildDocument ) {
-        ViewListEditViewDialog *dlg = new ViewListEditViewDialog( *this, m_contextitem, this );
-        dlg->exec();
-        delete dlg;
+        dlg = new ViewListEditViewDialog( *this, m_contextitem, this );
+    }
+    if ( dlg ) {
+        connect(dlg, SIGNAL(finished(int)), SLOT(slotDialogFinished(int)));
+        dlg->show();
+        dlg->raise();
+        dlg->activateWindow();
+    }
+}
+
+void ViewListWidget::slotDialogFinished( int result )
+{
+    if ( result == QDialog::Accepted ) {
+        emit modified();
+    }
+    if ( sender() ) {
+        sender()->deleteLater();
     }
 }
 
@@ -572,18 +618,11 @@ void ViewListWidget::slotEditDocumentTitle()
     //QTreeWidgetItem *item = m_viewlist->currentItem();
     if ( m_contextitem ) {
         kDebug()<<m_contextitem<<":"<<m_contextitem->type();
+        QString title = m_contextitem->text( 0 );
         m_viewlist->editItem( m_contextitem );
     }
 }
 
-void ViewListWidget::slotRemoveDocument()
-{
-    if ( m_contextitem ) {
-        kDebug()<<m_contextitem<<":"<<m_contextitem->type();
-        m_part->addCommand( new DeleteEmbeddedDocumentCmd( this, m_contextitem, i18n( "Remove Document" ) ) );
-    }
-}
-
 int ViewListWidget::removeViewListItem( ViewListItem *item )
 {
     QTreeWidgetItem *p = item->parent();
@@ -593,6 +632,7 @@ int ViewListWidget::removeViewListItem( ViewListItem *item )
     int i = p->indexOfChild( item );
     if ( i != -1 ) {
         p->takeChild( i );
+        emit modified();
     }
     return i;
 }
@@ -607,13 +647,23 @@ void ViewListWidget::addViewListItem( ViewListItem *item, QTreeWidgetItem *paren
         index = p->childCount();
     }
     p->insertChild( index, item );
+    emit modified();
 }
 
 int ViewListWidget::takeViewListItem( ViewListItem *item )
 {
+    while ( item->childCount() > 0 ) {
+        takeViewListItem( static_cast<ViewListItem*>( item->child( 0 ) ) );
+    }
     int pos = removeViewListItem( item );
     if ( pos != -1 ) {
         emit viewListItemRemoved( item );
+        if ( item == m_prev ) {
+            m_prev = 0;
+        }
+        if ( m_prev ) {
+            setCurrentItem( m_prev );
+        }
     }
     return pos;
 }
@@ -621,75 +671,57 @@ int ViewListWidget::takeViewListItem( ViewListItem *item )
 void ViewListWidget::insertViewListItem( ViewListItem *item, QTreeWidgetItem *parent, int index )
 {
     addViewListItem( item, parent, index );
-    emit viewListItemInserted( item );
+    emit viewListItemInserted( item, static_cast<ViewListItem*>( parent ), index );
 }
 
 void ViewListWidget::setupContextMenus()
 {
     // NOTE: can't use xml file as there may not be a factory()
     QAction *action;
-    // document insert actions
-    // Query for document types
-#if 0
-    m_lstEntries = KoDocumentEntry::query(KoDocumentEntry::OnlyEmbeddableDocuments);
-    QList<KoDocumentEntry>::const_iterator it = m_lstEntries.begin();
-    for( ; it != m_lstEntries.end(); ++it ) {
-        KService::Ptr serv = (*it).service();
-        // TODO: Make this test safer (possibly add desktopfile to define embeddable parts)
-        if ( serv->genericName().isEmpty() ||
-            serv->serviceTypes().contains( "application/vnd.oasis.opendocument.formula" ) ||
-            serv->serviceTypes().contains( "application/x-vnd.kde.kplato" ) ) {
-            continue;
-            }
-            action = new QAction( KIcon(serv->icon()), serv->genericName().replace('&',"&&"), this );
-            action->setObjectName( serv->entryPath() );
-            connect(action, SIGNAL( triggered( bool ) ), this, SLOT( slotCreatePart() ) );
-            m_adddocument.append( action );
-    }
-#endif
-    // no item actions
-    //action = new QAction( KIcon( "document-new" ), i18n( "New Category..." ), this );
-    //m_noitem.append( action );
-
-    // view insert actions
-    action = new QAction( KIcon( "list-add" ), i18nc( "@action Add View", "View..." ), this );
-    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotAddView() ) );
-    m_addview.append( action );
-
-    // Category edit actions
-    action = new QAction( KIcon( "edit-rename" ), i18n( "Rename" ), this );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( renameCategory() ) );
-    m_editcategory.append( action );
-    action = new QAction( KIcon( "list-remove" ), i18n( "Remove" ), this );
-    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotRemoveCategory() ) );
-    m_editcategory.append( action );
-    action = new QAction( KIcon( "configure" ), i18n( "Configure..." ), this );
-    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotConfigureItem() ) );
-    m_editcategory.append( action );
-
-    // view edit actions
-    action = new QAction( KIcon( "edit-rename" ), i18n( "Rename" ), this );
+    // view actions
+    action = new QAction( KIcon( "edit-rename" ), i18nc( "@action:inmenu rename view", "Rename" ), this );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotEditViewTitle() ) );
-    m_editview.append( action );
-    action = new QAction( KIcon( "list-remove" ), i18n( "Remove" ), this );
-    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotRemoveView() ) );
-    m_editview.append( action );
-    action = new QAction( KIcon( "configure" ), i18n( "Configure..." ), this );
-    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotConfigureItem() ) );
-    m_editview.append( action );
+    m_viewactions.append( action );
 
-    // document edit actions
-    action = new QAction( KIcon( "edit-rename" ), i18n( "Rename" ), this );
-    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotEditDocumentTitle() ) );
-    m_editdocument.append( action );
-    action = new QAction( KIcon( "edit-delete" ), i18n( "Remove" ), this );
-    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotRemoveDocument() ) );
-    m_editdocument.append( action );
+    action = new QAction( KIcon( "configure" ), i18nc( "@action:inmenu configure view", "Configure..." ), this );
+    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotConfigureItem() ) );
+    m_viewactions.append( action );
+
+    action = new QAction( KIcon( "list-remove" ), i18nc( "@action:inmenu remove view", "Remove" ), this );
+    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotRemoveView() ) );
+    m_viewactions.append( action );
+
+    action = new QAction( this );
+    action->setSeparator( true );
+    m_viewactions.append( action );
+
+    // Category actions
+    action = new QAction( KIcon( "edit-rename" ), i18nc( "@action:inmenu rename view category", "Rename" ), this );
+    connect( action, SIGNAL( triggered( bool ) ), SLOT( renameCategory() ) );
+    m_categoryactions.append( action );
+
+    action = new QAction( KIcon( "configure" ), i18nc( "@action:inmenu configure view category", "Configure..." ), this );
+    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotConfigureItem() ) );
+    m_categoryactions.append( action );
+
+    action = new QAction( KIcon( "list-remove" ), i18nc( "@action:inmenu Remove view category", "Remove" ), this );
+    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotRemoveCategory() ) );
+    m_categoryactions.append( action );
+
+    action = new QAction( this );
+    action->setSeparator( true );
+    m_categoryactions.append( action );
+
+    // list actions
+    action = new QAction( KIcon( "list-add" ), i18nc( "@action:inmenu Insert View", "Insert..." ), this );
+    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotAddView() ) );
+    m_listactions.append( action );
 }
 
 void ViewListWidget::renameCategory()
 {
     if ( m_contextitem ) {
+        QString title = m_contextitem->text( 0 );
         m_viewlist->editItem( m_contextitem, 0 );
     }
 }
@@ -699,37 +731,28 @@ void ViewListWidget::contextMenuEvent ( QContextMenuEvent *event )
     KMenu menu;
     QList<QAction*> lst;
     m_contextitem = static_cast<ViewListItem*>(m_viewlist->itemAt( event->pos() ) );
-    // first edit stuff
-    if ( m_contextitem != 0 ) {
+    if ( m_contextitem == 0 ) {
+        lst += m_listactions;
+    } else {
         if ( m_contextitem->type() == ViewListItem::ItemType_Category ) {
-            lst += m_editcategory;
+            lst += m_categoryactions;
         } else if ( m_contextitem->type() == ViewListItem::ItemType_SubView ) {
-            lst += m_editview;
+            lst += m_viewactions;
             ViewBase *v = dynamic_cast<ViewBase*>( m_contextitem->view() );
             if ( v ) {
                 lst += v->viewlistActionList();
             }
-        } else if ( m_contextitem->type() == ViewListItem::ItemType_ChildDocument ) {
-            lst += m_editdocument;
-        }
-        if ( ! lst.isEmpty() ) {
-            menu.addTitle( i18n( "Edit" ) );
-            foreach ( QAction *a, lst ) {
-                menu.addAction( a );
-            }
         }
+        lst += m_listactions;
     }
-    // then the "add" stuff
-    lst = m_addview;
-    lst += m_adddocument;
     if ( ! lst.isEmpty() ) {
-        menu.addTitle( i18n( "Insert" ) );
+        //menu.addTitle( i18n( "Edit" ) );
         foreach ( QAction *a, lst ) {
             menu.addAction( a );
         }
     }
     if ( ! menu.actions().isEmpty() ) {
-        menu.exec( event->globalPos());
+        menu.exec( event->globalPos() );
     }
 }
 
@@ -738,6 +761,51 @@ void ViewListWidget::save( QDomElement &element ) const
     m_viewlist->save( element );
 }
 
+void ViewListWidget::setProject( Project *project )
+{
+    kDebug()<<project;
+    m_model.setProject( project );
+}
+
+void ViewListWidget::slotCurrentScheduleChanged( int idx )
+{
+    kDebug()<<idx<<selectedSchedule();
+    emit selectionChanged( selectedSchedule() );
+}
+
+ScheduleManager *ViewListWidget::selectedSchedule() const
+{
+    QModelIndex idx = m_sfModel.index( m_currentSchedule->currentIndex(), m_currentSchedule->modelColumn() );
+    kDebug()<<idx;
+    return m_sfModel.manager( idx );
+}
+
+void ViewListWidget::setSelectedSchedule( ScheduleManager *sm )
+{
+    kDebug()<<sm<<m_model.index( sm );
+    QModelIndex idx = m_sfModel.mapFromSource( m_model.index( sm ) );
+    if ( sm && ! idx.isValid()) {
+        m_temp = sm;
+        return;
+    }
+    m_currentSchedule->setCurrentIndex( idx.row() );
+    kDebug()<<sm<<idx;
+    m_temp = 0;
+}
+
+void ViewListWidget::slotScheduleManagerAdded( ScheduleManager *sm )
+{
+    if ( m_temp && m_temp == sm ) {
+        setSelectedSchedule( sm );
+        m_temp = 0;
+    }
+}
+
+void ViewListWidget::setModified()
+{
+    emit modified();
+}
+
 }  //KPlato namespace
 
 #include "kptviewlist.moc"
