@@ -191,7 +191,43 @@ Account::CostPlace *Account::findCostPlace(const Node &node) const {
             return cp;
         }
     }
-    return 0;    
+    return 0;
+}
+
+Account::CostPlace *Account::findCostPlace(const Resource &resource) const {
+    foreach (Account::CostPlace *cp, m_costPlaces) {
+        if (&resource == cp->resource()) {
+            return cp;
+        }
+    }
+    return 0;
+}
+
+Account::CostPlace *Account::findRunning(const Resource &resource) const {
+    Account::CostPlace *cp = findCostPlace(resource);
+    return cp && cp->running() ? cp : 0;
+}
+
+void Account::removeRunning(const Resource &resource) {
+    Account::CostPlace *cp = findRunning(resource);
+    if (cp) {
+        cp->setRunning(false);
+        if (cp->isEmpty()) {
+            deleteCostPlace(cp);
+        }
+    }
+}
+
+void Account::addRunning(Resource &resource) {
+    Account::CostPlace *cp = findCostPlace(resource);
+    if (cp) {
+        cp->setRunning(true);
+        changed();
+        return;
+    }
+    cp = new CostPlace(this, &resource, true);
+    append(cp);
+    changed();
 }
 
 Account::CostPlace *Account::findRunning(const Node &node) const {
@@ -221,6 +257,7 @@ void Account::addRunning(Node &node) {
     changed();
 }
 
+
 Account::CostPlace *Account::findStartup(const Node &node) const {
     Account::CostPlace *cp = findCostPlace(node);
     return cp && cp->startup() ? cp : 0;
@@ -314,29 +351,9 @@ EffortCostMap Account::plannedCost(const QDate &start, const QDate &end, long id
         foreach ( Account *a, m_accountList ) {
             ec += a->plannedCost( start, end, id );
         }
-        return ec;
     }
     foreach (Account::CostPlace *cp, m_costPlaces) {
-        Node *n = cp->node();
-        if (n == 0) {
-            continue;
-        }
-        //kDebug()<<"n="<<n->name();
-        if (cp->running()) {
-            ec += n->plannedEffortCostPrDay(start, end, id);
-        }
-        if (cp->startup()) {
-            if ( ( ! start.isValid() || n->startTime( id ).date() >= start ) &&
-                 ( ! end.isValid() || n->startTime( id ).date() <= end ) ) {
-                ec.add(n->startTime( id ).date(), EffortCost(Duration::zeroDuration, n->startupCost()));
-            }
-        }
-        if (cp->shutdown()) {
-            if ( ( ! start.isValid() || n->endTime( id ).date() >= start ) &&
-                 ( ! end.isValid() || n->endTime( id ).date() <= end ) ) {
-                ec.add(n->endTime( id ).date(), EffortCost(Duration::zeroDuration, n->shutdownCost()));
-            }
-        }
+        ec += plannedCost( *cp, start, end, id );
     }
     if (isDefaultAccount()) {
         QList<Node*> list = m_list == 0 ? QList<Node*>() : m_list->allNodes();
@@ -364,34 +381,50 @@ EffortCostMap Account::plannedCost(const QDate &start, const QDate &end, long id
     return ec;
 }
 
-EffortCostMap Account::actualCost(long id)
+EffortCostMap Account::plannedCost( const Account::CostPlace &cp, const QDate &start, const QDate &end, long id) const
+{
+    EffortCostMap ec;
+    if ( cp.node() ) {
+        Node &node = *(cp.node());
+        //kDebug()<<"n="<<n->name();
+        if (cp.running()) {
+            ec += node.plannedEffortCostPrDay(start, end, id);
+        }
+        if (cp.startup()) {
+            if ( ( ! start.isValid() || node.startTime( id ).date() >= start ) &&
+                 ( ! end.isValid() || node.startTime( id ).date() <= end ) ) {
+                ec.add(node.startTime( id ).date(), EffortCost(Duration::zeroDuration, node.startupCost()));
+            }
+        }
+        if (cp.shutdown()) {
+            if ( ( ! start.isValid() || node.endTime( id ).date() >= start ) &&
+                 ( ! end.isValid() || node.endTime( id ).date() <= end ) ) {
+                ec.add(node.endTime( id ).date(), EffortCost(Duration::zeroDuration, node.shutdownCost()));
+            }
+        }
+    } else if ( cp.resource() ) {
+        if ( cp.running() ) {
+            ec += cp.resource()->plannedEffortCostPrDay(start, end, id);
+        }
+    }
+    return ec;
+}
+
+EffortCostMap Account::actualCost(long id) const
 {
     return actualCost( QDate(), QDate(), id );
 }
 
-EffortCostMap Account::actualCost(const QDate &start, const QDate &end, long id) {
+EffortCostMap Account::actualCost(const QDate &start, const QDate &end, long id) const
+{
     EffortCostMap ec;
+    if ( ! isElement() ) {
+        foreach ( Account *a, m_accountList ) {
+            ec += a->actualCost( start, end, id );
+        }
+    }
     foreach (Account::CostPlace *cp, costPlaces()) {
-        Node *n = cp->node();
-        if (n == 0) {
-            continue;
-        }
-        //kDebug()<<"n="<<n->name();
-        if (cp->running()) {
-            ec += n->actualEffortCostPrDay(start, end, id);
-        }
-        if (cp->startup()) {
-            if ( ( ! start.isValid() || n->startTime( id ).date() >= start ) &&
-                 ( ! end.isValid() || n->startTime( id ).date() <= end ) ) {
-                ec.add(n->startTime( id ).date(), EffortCost(Duration::zeroDuration, n->startupCost()));
-            }
-        }
-        if (cp->shutdown()) {
-            if ( ( ! start.isValid() || n->endTime( id ).date() >= start ) &&
-                 ( ! end.isValid() || n->endTime( id ).date() <= end ) ) {
-                ec.add(n->endTime( id ).date(), EffortCost(Duration::zeroDuration, n->shutdownCost()));
-            }
-        }
+        ec += actualCost( *cp, start, end, id );
     }
     if (isDefaultAccount()) {
         QList<Node*> list = m_list == 0 ? QList<Node*>() : m_list->allNodes();
@@ -400,23 +433,62 @@ EffortCostMap Account::actualCost(const QDate &start, const QDate &end, long id)
                 continue;
             }
             if (n->runningAccount() == 0) {
-                kDebug()<<"default, running:"<<n->name();
+                //kDebug()<<"default, running:"<<n->name();
                 ec += n->actualEffortCostPrDay(start, end, id);
             }
-            if (n->startupAccount() == 0) {
-                kDebug()<<"default, starup:"<<n->name();
-                if ( ( ! start.isValid() || n->startTime( id ).date() >= start ) &&
-                     ( ! end.isValid() || n->startTime( id ).date() <= end ) ) {
-                    ec.add(n->startTime( id ).date(), EffortCost(Duration::zeroDuration, n->startupCost()));
+            Task *t = dynamic_cast<Task*>( n ); // only tasks have completion
+            if ( t ) {
+                if (n->startupAccount() == 0 && t->completion().isStarted()) {
+                    const QDate startDate = t->completion().startTime().date();
+                    if ( ( ! start.isValid() || startDate >= start ) &&
+                        ( ! end.isValid() || startDate <= end ) ) {
+                        ec.add(startDate, EffortCost(Duration::zeroDuration, n->startupCost()));
+                    }
+                }
+                if (n->shutdownAccount() == 0 && t->completion().isFinished()) {
+                    //kDebug()<<"default, shutdown:"<<n->name();
+                    const QDate finishDate = t->completion().finishTime().date();
+                    if ( ( ! start.isValid() || finishDate >= start ) &&
+                        ( ! end.isValid() || finishDate <= end ) ) {
+                        ec.add(finishDate, EffortCost(Duration::zeroDuration, n->shutdownCost()));
+                    }
                 }
             }
-            if (n->shutdownAccount() == 0) {
-                kDebug()<<"default, shutdown:"<<n->name();
-                if ( ( ! start.isValid() || n->endTime( id ).date() >= start ) &&
-                     ( ! end.isValid() || n->endTime( id ).date() <= end ) ) {
-                    ec.add(n->endTime( id ).date(), EffortCost(Duration::zeroDuration, n->shutdownCost()));
+        }
+    }
+    return ec;
+}
+
+EffortCostMap Account::actualCost(const Account::CostPlace &cp, const QDate &start, const QDate &end, long id) const
+{
+    EffortCostMap ec;
+    if ( cp.node() ) {
+        Node &node = *(cp.node());
+        if (cp.running()) {
+            ec += node.actualEffortCostPrDay(start, end, id);
+        }
+        Task *t = dynamic_cast<Task*>( &node ); // only tasks have completion
+        if ( t ) {
+            if (cp.startup() && t->completion().isStarted()) {
+                const QDate startDate = t->completion().startTime().date();
+                if ( ( ! start.isValid() || startDate >= start ) &&
+                    ( ! end.isValid() || startDate <= end ) ) {
+                    ec.add(startDate, EffortCost(Duration::zeroDuration, node.startupCost()));
                 }
             }
+            if (cp.shutdown() && t->completion().isFinished()) {
+                const QDate finishDate = t->completion().finishTime().date();
+                if ( ( ! start.isValid() || finishDate >= start ) &&
+                    ( ! end.isValid() || finishDate <= end ) ) {
+                    ec.add(finishDate, EffortCost(Duration::zeroDuration, node.shutdownCost()));
+                }
+            }
+        }
+    } else if ( cp.resource() && m_list ) {
+        foreach ( Node *n, m_list->allNodes() ) {
+            if ( n->type() == Node::Type_Task ) {
+                ec += n->actualEffortCostPrDay( cp.resource(), start, end, id );
+            }
         }
     }
     return ec;
@@ -426,8 +498,9 @@ EffortCostMap Account::actualCost(const QDate &start, const QDate &end, long id)
 //------------------------------------
 Account::CostPlace::CostPlace(Account *acc, Node *node, bool running, bool strtup, bool shutdown)
     : m_account(acc), 
-    m_nodeId(node->id()),
+    m_objectId(node->id()),
     m_node(node),
+    m_resource(0),
     m_running( false ),
     m_startup( false ),
     m_shutdown( false )
@@ -439,6 +512,20 @@ Account::CostPlace::CostPlace(Account *acc, Node *node, bool running, bool strtu
     }
 }
 
+Account::CostPlace::CostPlace(Account *acc, Resource *resource, bool running)
+    : m_account(acc), 
+    m_objectId(resource->id()),
+    m_node(0),
+    m_resource(resource),
+    m_running( false ),
+    m_startup( false ),
+    m_shutdown( false )
+{
+    if (resource) {
+        if (running) setRunning(running);
+    }
+}
+
 Account::CostPlace::~CostPlace() {
     if (m_node) {
         if (m_running)
@@ -448,12 +535,19 @@ Account::CostPlace::~CostPlace() {
         if (m_shutdown)
             m_node->setShutdownAccount(0);
     }
+    if (m_resource) {
+        if (m_running)
+            m_resource->setAccount(0);
+    }
 }
 
 void Account::CostPlace::setRunning(bool on ) { 
     m_running = on;
-    if (m_node)
+    if (m_node) {
         m_node->setRunningAccount(on ? m_account : 0);
+    } else if ( m_resource ) {
+        m_resource->setAccount(on ? m_account : 0);
+    }
 }
 
 void Account::CostPlace::setStartup(bool on ) { 
@@ -468,21 +562,32 @@ void Account::CostPlace::setShutdown(bool on ) {
         m_node->setShutdownAccount(on ? m_account : 0);
 }
 
+//TODO
 bool Account::CostPlace::load(KoXmlElement &element, Project &project) {
     //kDebug();
-    m_nodeId = element.attribute("node-id");
-    if (m_nodeId.isEmpty()) {
-        kError()<<"No node id";
-        return false;
+    m_objectId = element.attribute("object-id");
+    if (m_objectId.isEmpty()) {
+        // check old format
+        m_objectId = element.attribute("node-id");
+        if (m_objectId.isEmpty()) {
+            kError()<<"No object id";
+            return false;
+        }
     }
-    m_node = project.findNode(m_nodeId);
+    m_node = project.findNode(m_objectId);
     if (m_node == 0) {
-        kError()<<"Cannot not find node with id: "<<m_nodeId;
-        return false;
+        m_resource = project.findResource(m_objectId);
+        if ( m_resource == 0 ) {
+            kError()<<"Cannot find object with id: "<<m_objectId;
+            return false;
+        }
     }
-    setRunning(element.attribute("running-cost").toInt());
-    setStartup(element.attribute("startup-cost").toInt());
-    setShutdown(element.attribute("shutdown-cost").toInt());
+    bool on = (bool)(element.attribute("running-cost").toInt());
+    if ( on ) setRunning( on );
+    on = (bool)(element.attribute("startup-cost").toInt());
+    if ( on ) setStartup( on );
+    on = (bool)(element.attribute("shutdown-cost").toInt());
+    if ( on ) setShutdown( on );
     return true;
 }
 
@@ -490,7 +595,7 @@ void Account::CostPlace::save(QDomElement &element) const {
     //kDebug();
     QDomElement me = element.ownerDocument().createElement("costplace");
     element.appendChild(me);
-    me.setAttribute("node-id", m_nodeId);
+    me.setAttribute("object-id", m_objectId);
     me.setAttribute("running-cost", m_running);
     me.setAttribute("startup-cost", m_startup);
     me.setAttribute("shutdown-cost", m_shutdown);
@@ -513,57 +618,24 @@ Accounts::~Accounts() {
     }
 }
 
-EffortCostMap Accounts::plannedCost(const Account &account, const QDate &start, const QDate &end, long id) {
+EffortCostMap Accounts::plannedCost(const Account &account, long id) const
+{
+    return account.plannedCost( id );
+}
+
+EffortCostMap Accounts::plannedCost(const Account &account, const QDate &start, const QDate &end, long id) const
+{
     return account.plannedCost( start, end, id );
 }
 
-EffortCostMap Accounts::actualCost(const Account &account, const QDate &start, const QDate &end, long id) {
-    EffortCostMap ec;
-    foreach (Account::CostPlace *cp, account.costPlaces()) {
-        Node *n = cp->node();
-        if (n == 0) {
-            continue;
-        }
-        //kDebug()<<"n="<<n->name();
-        if (cp->running()) {
-            ec += n->actualEffortCostPrDay(start, end, id);
-        }
-        if (cp->startup()) {
-            if (n->startTime( id ).date() >= start &&
-                n->startTime( id ).date() <= end)
-                ec.add(n->startTime( id ).date(), EffortCost(Duration::zeroDuration, n->startupCost()));
-        }
-        if (cp->shutdown()) {
-            if (n->endTime( id ).date() >= start &&
-                n->endTime( id ).date() <= end)
-                ec.add(n->endTime( id ).date(), EffortCost(Duration::zeroDuration, n->shutdownCost()));
-        }
-    }
-    if (&account == m_defaultAccount) {
-        QHash<QString, Node*> hash = m_project.nodeDict();
-        foreach (Node *n, hash) {
-            if ( n->numChildren() > 0 ) {
-                continue;
-            }
-            if (n->runningAccount() == 0) {
-                kDebug()<<"default, running:"<<n->name();
-                ec += n->actualEffortCostPrDay(start, end, id);
-            }
-            if (n->startupAccount() == 0) {
-                kDebug()<<"default, starup:"<<n->name();
-                if (n->startTime( id ).date() >= start &&
-                    n->startTime( id ).date() <= end)
-                    ec.add(n->startTime( id ).date(), EffortCost(Duration::zeroDuration, n->startupCost()));
-            }
-            if (n->shutdownAccount() == 0) {
-                kDebug()<<"default, shutdown:"<<n->name();
-                if (n->endTime( id ).date() >= start &&
-                    n->endTime( id ).date() <= end)
-                    ec.add(n->endTime( id ).date(), EffortCost(Duration::zeroDuration, n->shutdownCost()));
-            }
-        }
-    }
-    return ec;
+EffortCostMap Accounts::actualCost(const Account &account, long id) const
+{
+    return account.actualCost( id );
+}
+
+EffortCostMap Accounts::actualCost(const Account &account, const QDate &start, const QDate &end, long id) const
+{
+    return account.actualCost( start, end, id );
 }
 
 void Accounts::insert(Account *account, Account *parent, int index) {
@@ -657,7 +729,16 @@ QStringList Accounts::costElements() const {
 QStringList Accounts::nameList() const {
     return m_idDict.uniqueKeys();
 }
-    
+
+Account *Accounts::findRunningAccount(const Resource &resource) const {
+    foreach (Account *a, m_idDict) {
+        if (a->findRunning(resource)) {
+            return a;
+        }
+    }
+    return 0;
+}
+
 Account *Accounts::findRunningAccount(const Node &node) const {
     foreach (Account *a, m_idDict) {
         if (a->findRunning(node))
@@ -718,7 +799,6 @@ QString Accounts::uniqueId( const QString &seed ) const
     while (  findAccount( n ) ) {
         n = s.arg( ++i );
     }
-    kDebug()<<n;
     return n;
 }
 
