@@ -57,6 +57,11 @@ AppointmentInterval::~AppointmentInterval() {
     //kDebug()<<this;
 }
 
+Duration AppointmentInterval::effort() const
+{
+    return ( m_end - m_start ) * m_load / 100;
+}
+
 Duration AppointmentInterval::effort(const DateTime &start, const DateTime end) const {
     if (start >= m_end || end <= m_start) {
         return Duration::zeroDuration;
@@ -97,7 +102,7 @@ bool AppointmentInterval::loadXML(KoXmlElement &element, XMLLoaderObject &status
         m_end = DateTime::fromString(s, status.projectSpec());
     m_load = element.attribute("load", "100").toDouble(&ok);
     if (!ok) m_load = 100;
-    return m_start.isValid() && m_end.isValid();
+    return isValid();
 }
 
 void AppointmentInterval::saveXML(QDomElement &element) const {
@@ -110,7 +115,7 @@ void AppointmentInterval::saveXML(QDomElement &element) const {
 }
 
 bool AppointmentInterval::isValid() const {
-    return m_start.isValid() && m_end.isValid();
+    return m_start.isValid() && m_end.isValid() && m_start < m_end && m_load >= 0.0;
 }
 
 AppointmentInterval AppointmentInterval::firstInterval(const AppointmentInterval &interval, const DateTime &from) const {
@@ -162,12 +167,129 @@ AppointmentInterval AppointmentInterval::firstInterval(const AppointmentInterval
     return a;
 }
 
-bool AppointmentInterval::operator ==(const AppointmentInterval &interval ) const
+bool AppointmentInterval::operator==( const AppointmentInterval &interval ) const
 {
     return m_start == interval.m_start && m_end == interval.m_end && m_load == interval.m_load;
 }
 
+bool AppointmentInterval::operator<( const AppointmentInterval &other ) const
+{
+    if ( m_start < other.m_start ) {
+        return true;
+    } else if ( other.m_start < m_start ) {
+        return false;
+    }
+    // Start is assumed equal
+    return m_end < other.m_end;
+}
+
+QDebug &operator<<( QDebug &dbg, const KPlato::AppointmentInterval &i )
+{
+    dbg<<"AppointmentInterval["<<i.startTime().toString()<<i.endTime().toString()<<i.load()<<"%"<<']';
+    return dbg;
+}
+
 //-----------------------
+AppointmentIntervalList &AppointmentIntervalList::operator=( const AppointmentIntervalList &lst )
+{
+    clear();
+    if ( ! lst.isEmpty() ) {
+        foreach ( const AppointmentInterval &ai, lst ) {
+            inSort( ai );
+        }
+    }
+    return *this;
+}
+
+AppointmentIntervalList &AppointmentIntervalList::operator-=( const AppointmentIntervalList &lst )
+{
+    if ( lst.isEmpty() ) {
+        return *this;
+    }
+    foreach ( const AppointmentInterval &ai, lst ) {
+        subtract( ai.startTime(), ai.endTime(), ai.load() );
+    }
+    return *this;
+}
+
+void AppointmentIntervalList::subtract( const DateTime &st, const DateTime &et, double load )
+{
+    //kDebug()<<st<<et<<load;
+    Q_ASSERT( st < et );
+    DateTime s = st;
+    DateTime e = et;
+    if ( isEmpty() || et < values().first().startTime() || st > values().last().endTime() ) {
+        return;
+    } else {
+        // see if we have overlapping intervals
+        QMutableMapIterator<AppointmentInterval, AppointmentInterval> it( *this );
+        while ( it.hasNext() ) {
+            AppointmentInterval &ai = it.next().value();
+            DateTime ist = ai.startTime();
+            DateTime iet = ai.endTime();
+            //kDebug()<<"Check:"<<ai<<AppointmentInterval( st, et, load );
+            if ( e <= ist ) {
+                //kDebug()<<"Before first or in between";
+                break;
+            }
+            if ( s >= iet ) {
+                //kDebug()<<"After this interval, check next";
+                continue;
+            }
+            if ( s <= ist ) {
+                //kDebug()<<"Start <= start";
+                if ( e >= iet ) {
+                    //kDebug()<<"Total overlap, subtract load";
+                    ai.setLoad( ai.load() - load );
+                    if ( ai.load() <= 0 ) {
+                        //kDebug()<<"Total overlap, result is 0, remove";
+                        it.remove();
+                    }
+                } else if ( e < iet ) {
+                    //kDebug()<<"End in the midle, split this interval";
+                    double l = ai.load();
+                    it.remove();
+                    AppointmentInterval i( ist, e, l - load );
+                    if ( i.isValid() ) {
+                        //kDebug()<<"Subtract first part:"<<i;
+                        inSort( i );
+                    }
+                    i = AppointmentInterval( e, iet, l );
+                    //kDebug()<<"Keep last part:"<<i;
+                    inSort( i ); // keep the last part
+                }
+                return;
+            }
+            if ( s > ist ) {
+                //kDebug()<<"Start in the middle of an interval";
+                double l = ai.load();
+                it.remove();
+                if ( e < iet ) {
+                    //kDebug()<<"End in the midle, split this interval";
+                    AppointmentInterval i( ist, s, l );
+                    //kDebug()<<"Keep first part:"<<i;
+                    inSort( AppointmentInterval( ist, s, l ) ); // keep the first part
+                    if ( l > load ) {
+                        AppointmentInterval i( s, e, l - load );
+                        //kDebug()<<"Subtract:"<<i;
+                        inSort( i ); // subtract overlap
+                    }
+                    i = AppointmentInterval( ist, s, l );
+                    //kDebug()<<"Keep last part:"<<i;
+                    inSort( i ); // keep the last part
+                } else if ( e >= iet ) {
+                    //kDebug()<<"End >= end, split this interval";
+                    inSort( AppointmentInterval( ist, s, l ) ); // keep first part
+                    if ( l > load ) {
+                        inSort( AppointmentInterval( s, iet, l - load ) ); // subtract overlap
+                    }
+                }
+                return;
+            }
+        }
+    }
+}
+
 AppointmentIntervalList &AppointmentIntervalList::operator+=( const AppointmentIntervalList &lst )
 {
     if ( lst.isEmpty() ) {
@@ -179,6 +301,27 @@ AppointmentIntervalList &AppointmentIntervalList::operator+=( const AppointmentI
     return *this;
 }
 
+AppointmentIntervalList AppointmentIntervalList::extractIntervals( const DateTime &start, const DateTime &end ) const
+{
+    if ( isEmpty() ) {
+        return AppointmentIntervalList();
+    }
+    QList<AppointmentInterval> ilst = values();
+    if ( start <= ilst.first().startTime() && end >= ilst.last().endTime() ) {
+        return *this;
+    }
+    AppointmentIntervalList lst;
+    foreach ( const AppointmentInterval &i, ilst ) {
+        if ( end <= i.startTime() ) {
+            break;
+        }
+        if ( start < i.endTime() ) {
+            lst.inSort( AppointmentInterval( qMax( start, i.startTime() ), qMin( end, i.endTime() ), i.load() ) );
+        }
+    }
+    return lst;
+}
+
 void AppointmentIntervalList::add( const AppointmentInterval &ai )
 {
     add( ai.startTime(), ai.endTime(), ai.load() );
@@ -190,12 +333,12 @@ void AppointmentIntervalList::add( const DateTime &st, const DateTime &et, doubl
     Q_ASSERT( st < et );
     DateTime s = st;
     DateTime e = et;
-    if ( isEmpty() || et < values().first().startTime() || st > values().last().endTime() ) {
+    if ( isEmpty() || et <= values().first().startTime() || st >= values().last().endTime() ) {
         inSort( AppointmentInterval( s, e, load ) );
     } else {
         // see if we have overlapping intervals
         AppointmentIntervalList lst;
-        QMutableMapIterator<QString, AppointmentInterval> it( *this );
+        QMutableMapIterator<AppointmentInterval, AppointmentInterval> it( *this );
         while ( it.hasNext() ) {
             AppointmentInterval &ai = it.next().value();
             DateTime ist = ai.startTime();
@@ -285,7 +428,7 @@ void AppointmentIntervalList::add( const DateTime &st, const DateTime &et, doubl
 void AppointmentIntervalList::inSort(const AppointmentInterval &a)
 {
     Q_ASSERT( a.startTime() < a.endTime() );
-    insert( a.startTime().toString( KDateTime::ISODate ) + a.endTime().toString( KDateTime::ISODate ), a );
+    insert( a, a );
 }
 
 void AppointmentIntervalList::saveXML( QDomElement &element ) const
@@ -361,6 +504,31 @@ Appointment::~Appointment() {
     detach();
 }
 
+void Appointment::clear()
+{
+    m_intervals.clear();
+}
+
+// Returns the total effort
+Duration AppointmentIntervalList::effort() const
+{
+    Duration d;
+    foreach (const AppointmentInterval &i, *this ) {
+        d += i.effort();
+    }
+    return d;
+}
+
+// Returns the effort from start to end
+Duration AppointmentIntervalList::effort(const DateTime &start, const DateTime &end) const
+{
+    Duration d;
+    foreach (const AppointmentInterval &i, *this ) {
+        d += i.effort(start, end);
+    }
+    return d;
+}
+
 AppointmentIntervalList Appointment::intervals( const DateTime &start, const DateTime &end ) const
 {
     //kDebug()<<start<<end;
@@ -463,6 +631,7 @@ bool Appointment::loadXML(KoXmlElement &element, XMLLoaderObject &status, Schedu
     //kDebug()<<"res="<<m_resource<<" node="<<m_node;
     m_intervals.loadXML( element, status );
     if (isEmpty()) {
+        kError()<<"Appointment is empty (added anyway): "<<node->name()<<res->name();
         return false;
     }
     return true;
@@ -490,31 +659,37 @@ void Appointment::saveXML(QDomElement &element) const {
 }
 
 // Returns the total planned effort for this appointment
-Duration Appointment::plannedEffort() const {
+Duration Appointment::plannedEffort(EffortCostCalculationType type) const {
     Duration d;
-    foreach (const AppointmentInterval &i, m_intervals ) {
-        d += i.effort();
+    if ( type == ECCT_All || m_resource->resource()->type() == Resource::Type_Work ) {
+        foreach (const AppointmentInterval &i, m_intervals ) {
+            d += i.effort();
+        }
     }
     return d;
 }
 
 // Returns the planned effort on the date
-Duration Appointment::plannedEffort(const QDate &date) const {
+Duration Appointment::plannedEffort(const QDate &date, EffortCostCalculationType type) const {
     Duration d;
-    QDateTime s(date);
-    QDateTime e(date.addDays(1));
-    foreach (const AppointmentInterval &i, m_intervals ) {
-        d += i.effort(DateTime(s, i.startTime().timeSpec()), DateTime(e, i.startTime().timeSpec()));
+    if ( type == ECCT_All || m_resource->resource()->type() == Resource::Type_Work ) {
+        QDateTime s(date);
+        QDateTime e(date.addDays(1));
+        foreach (const AppointmentInterval &i, m_intervals ) {
+            d += i.effort(DateTime(s, i.startTime().timeSpec()), DateTime(e, i.startTime().timeSpec()));
+        }
     }
     return d;
 }
 
 // Returns the planned effort upto and including the date
-Duration Appointment::plannedEffortTo(const QDate& date) const {
+Duration Appointment::plannedEffortTo(const QDate& date, EffortCostCalculationType type) const {
     Duration d;
     QDate e(date.addDays(1));
-    foreach (const AppointmentInterval &i, m_intervals ) {
-        d += i.effort(e, true); // upto e, not including
+    if ( type == ECCT_All || m_resource->resource()->type() == Resource::Type_Work ) {
+        foreach (const AppointmentInterval &i, m_intervals ) {
+            d += i.effort(e, true); // upto e, not including
+        }
     }
     //kDebug()<<date<<d.toString();
     return d;
@@ -522,15 +697,16 @@ Duration Appointment::plannedEffortTo(const QDate& date) const {
 
 // Returns a list of efforts pr day for interval start, end inclusive
 // The list only includes days with any planned effort
-EffortCostMap Appointment::plannedPrDay(const QDate& pstart, const QDate& pend) const {
+EffortCostMap Appointment::plannedPrDay(const QDate& pstart, const QDate& pend, EffortCostCalculationType type) const {
     //kDebug()<<m_node->id()<<","<<m_resource->id();
     EffortCostMap ec;
-    Duration eff;
     QDate start = pstart.isValid() ? pstart : startTime().date();
     QDate end = pend.isValid() ? pend : endTime().date();
     QDate dt(start);
     QDate ndt(dt.addDays(1));
     double rate = m_resource ? m_resource->normalRatePrHour() : 0.0;
+    Resource::Type rt = m_resource->resource()->type();
+    Duration zero;
     //kDebug()<<rate<<m_intervals.count();
     foreach (const AppointmentInterval &i, m_intervals ) {
         DateTime st = i.startTime();
@@ -544,8 +720,23 @@ EffortCostMap Appointment::plannedPrDay(const QDate& pstart, const QDate& pend)
         ndt = dt.addDays(1);
         while (dt <= e.date() && dt <= end ) {
             //kDebug()<<start<<end<<dt;
-            eff = i.effort(DateTime(dt, QTime( 0, 0, 0 ), st.timeSpec()), DateTime(ndt, QTime( 0, 0, 0 ), st.timeSpec()));
-            ec.add(dt, eff, eff.toDouble(Duration::Unit_h) * rate);
+            Duration eff;
+            switch ( type ) {
+                case ECCT_All:
+                    eff = i.effort(DateTime(dt, QTime( 0, 0, 0 ), st.timeSpec()), DateTime(ndt, QTime( 0, 0, 0 ), st.timeSpec()));
+                    ec.add(dt, eff, eff.toDouble(Duration::Unit_h) * rate);
+                    break;
+                case ECCT_EffortWork:
+                    eff = i.effort(DateTime(dt, QTime( 0, 0, 0 ), st.timeSpec()), DateTime(ndt, QTime( 0, 0, 0 ), st.timeSpec()));
+                    ec.add(dt, (rt == Resource::Type_Work ? eff : zero), eff.toDouble(Duration::Unit_h) * rate);
+                    break;
+                case ECCT_Work:
+                    if ( rt == Resource::Type_Work ) {
+                        eff = i.effort(DateTime(dt, QTime( 0, 0, 0 ), st.timeSpec()), DateTime(ndt, QTime( 0, 0, 0 ), st.timeSpec()));
+                        ec.add(dt, eff, eff.toDouble(Duration::Unit_h) * rate);
+                    }
+                    break;
+            }
             if (dt < e.date() && dt < end) {
                 // loop trough the interval (it spans dates)
                 dt = ndt;
@@ -559,27 +750,52 @@ EffortCostMap Appointment::plannedPrDay(const QDate& pstart, const QDate& pend)
 }
 
 
-EffortCost Appointment::plannedCost() const {
+EffortCost Appointment::plannedCost(EffortCostCalculationType type) const {
     EffortCost ec;
-    ec.setEffort( plannedEffort() );
+    ec.setEffort( plannedEffort(type) );
     if (m_resource && m_resource->resource()) {
-        ec.setCost( ec.hours() * m_resource->resource()->normalRate() ); //FIXME overtime
+        switch ( type ) {
+            case ECCT_Work:
+                if ( m_resource->resource()->type() != Resource::Type_Work ) {
+                    break;
+                }
+                // fall through
+            default:
+                ec.setCost( ec.hours() * m_resource->resource()->normalRate() ); //FIXME overtime
+                break;
+        }
     }
     return ec;
 }
 
 //Calculates the planned cost on date
-double Appointment::plannedCost(const QDate &date) {
+double Appointment::plannedCost(const QDate &date, EffortCostCalculationType type) {
     if (m_resource && m_resource->resource()) {
-        return plannedEffort(date).toDouble(Duration::Unit_h) * m_resource->resource()->normalRate(); //FIXME overtime
+        switch ( type ) {
+            case ECCT_Work:
+                if ( m_resource->resource()->type() != Resource::Type_Work ) {
+                    break;
+                }
+                // fall through
+            default:
+                return plannedEffort(date).toDouble(Duration::Unit_h) * m_resource->resource()->normalRate(); //FIXME overtime
+        }
     }
     return 0.0;
 }
 
 //Calculates the planned cost upto and including date
-double Appointment::plannedCostTo(const QDate &date) {
+double Appointment::plannedCostTo(const QDate &date, EffortCostCalculationType type) {
     if (m_resource && m_resource->resource()) {
-        return plannedEffortTo(date).toDouble(Duration::Unit_h) * m_resource->resource()->normalRate(); //FIXME overtime
+        switch ( type ) {
+            case ECCT_Work:
+                if ( m_resource->resource()->type() != Resource::Type_Work ) {
+                    break;
+                }
+                // fall through
+            default:
+                return plannedEffortTo(date).toDouble(Duration::Unit_h) * m_resource->resource()->normalRate(); //FIXME overtime
+        }
     }
     return 0.0;
 }
@@ -607,26 +823,30 @@ void Appointment::detach() {
 }
 
 // Returns the effort from start to end
-Duration Appointment::effort(const DateTime &start, const DateTime &end) const {
-    Duration d;
-    foreach (const AppointmentInterval &i, m_intervals ) {
-        d += i.effort(start, end);
+Duration Appointment::effort(const DateTime &start, const DateTime &end, EffortCostCalculationType type) const {
+    Duration e;
+    if ( type == ECCT_All || m_resource->resource()->type() == Resource::Type_Work ) {
+        e = m_intervals.effort( start, end );
     }
-    return d;
+    return e;
 }
 // Returns the effort from start for the duration
-Duration Appointment::effort(const DateTime &start, const Duration &duration) const {
+Duration Appointment::effort(const DateTime &start, const Duration &duration, EffortCostCalculationType type) const {
     Duration d;
-    foreach (const AppointmentInterval &i, m_intervals ) {
-        d += i.effort(start, start+duration);
+    if ( type == ECCT_All || m_resource->resource()->type() == Resource::Type_Work ) {
+        foreach (const AppointmentInterval &i, m_intervals ) {
+            d += i.effort(start, start+duration);
+        }
     }
     return d;
 }
 // Returns the effort upto time / from time
-Duration Appointment::effortFrom(const QDate &time) const {
+Duration Appointment::effortFrom(const QDate &time, EffortCostCalculationType type) const {
     Duration d;
-    foreach (const AppointmentInterval &i, m_intervals ) {
-        d += i.effort(time, false);
+    if ( type == ECCT_All || m_resource->resource()->type() == Resource::Type_Work ) {
+        foreach (const AppointmentInterval &i, m_intervals ) {
+            d += i.effort(time, false);
+        }
     }
     return d;
 }
