@@ -23,6 +23,7 @@
 #include "kplatokernel_export.h"
 
 #include <QUndoCommand>
+#include <QPointer>
 
 #include "kptappointment.h"
 #include "kptnode.h"
@@ -104,7 +105,7 @@ protected:
 class KPLATOKERNEL_EXPORT CalendarAddCmd : public NamedCommand
 {
 public:
-    CalendarAddCmd( Project *project, Calendar *cal, Calendar *parent, const QString& name = QString() );
+    CalendarAddCmd( Project *project, Calendar *cal, int pos, Calendar *parent, const QString& name = QString() );
     ~CalendarAddCmd();
     void execute();
     void unexecute();
@@ -112,6 +113,7 @@ public:
 private:
     Project *m_project;
     Calendar *m_cal;
+    int m_pos;
     Calendar *m_parent;
     bool m_mine;
 };
@@ -128,10 +130,27 @@ private:
     Project *m_project;
     Calendar *m_parent;
     Calendar *m_cal;
+    int m_index;
     bool m_mine;
     MacroCommand *m_cmd;
 };
 
+class KPLATOKERNEL_EXPORT CalendarMoveCmd : public NamedCommand
+{
+public:
+    CalendarMoveCmd( Project *project, Calendar *cal, int position, Calendar *parent, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    Project *m_project;
+    Calendar *m_cal;
+    int m_newpos;
+    int m_oldpos;
+    Calendar *m_newparent;
+    Calendar *m_oldparent;
+};
+
 class KPLATOKERNEL_EXPORT CalendarModifyNameCmd : public NamedCommand
 {
 public:
@@ -159,6 +178,9 @@ private:
     Calendar *m_newvalue;
     Calendar *m_oldvalue;
     MacroCommand *m_cmd;
+
+    int m_oldindex;
+    int m_newindex;
 };
 
 class KPLATOKERNEL_EXPORT CalendarModifyTimeZoneCmd : public NamedCommand
@@ -543,7 +565,6 @@ class KPLATOKERNEL_EXPORT NodeMoveCmd : public NamedCommand
 {
 public:
     NodeMoveCmd( Project *project, Node *node, Node *newParent, int newPos, const QString& name = QString() );
-    ~NodeMoveCmd();
     void execute();
     void unexecute();
 
@@ -555,7 +576,7 @@ private:
     int m_newpos;
     int m_oldpos;
     bool m_moved;
-    MacroCommand *m_cmd;
+    MacroCommand m_cmd;
 };
 
 class KPLATOKERNEL_EXPORT AddRelationCmd : public NamedCommand
@@ -661,6 +682,19 @@ private:
 
 };
 
+class KPLATOKERNEL_EXPORT ModifyResourceRequestRequiredCmd : public NamedCommand
+{
+public:
+    ModifyResourceRequestRequiredCmd( ResourceRequest *request, const QList<Resource*> &value, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    ResourceRequest *m_request;
+    QList<Resource*> m_oldvalue, m_newvalue;
+
+};
+
 class KPLATOKERNEL_EXPORT ModifyResourceGroupRequestUnitsCmd : public NamedCommand
 {
 public:
@@ -821,6 +855,7 @@ public:
 private:
     QList<ResourceRequest*> m_requests;
     QList<Appointment*> m_appointments;
+    MacroCommand m_cmd;
 };
 
 class KPLATOKERNEL_EXPORT MoveResourceCmd : public NamedCommand
@@ -963,6 +998,52 @@ private:
     Calendar *m_newvalue;
     Calendar *m_oldvalue;
 };
+class KPLATOKERNEL_EXPORT ModifyRequiredResourcesCmd : public NamedCommand
+{
+public:
+    ModifyRequiredResourcesCmd( Resource *resource, const QList<Resource*> &value, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    Resource *m_resource;
+    QList<Resource*> m_newvalue;
+    QList<Resource*> m_oldvalue;
+};
+class KPLATOKERNEL_EXPORT ModifyResourceAccountCmd : public NamedCommand
+{
+public:
+    ModifyResourceAccountCmd( Resource *resource, Account *account, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    Resource *m_resource;
+    Account *m_newvalue;
+    Account *m_oldvalue;
+};
+class KPLATOKERNEL_EXPORT AddResourceTeamCmd : public NamedCommand
+{
+public:
+    AddResourceTeamCmd( Resource *team, Resource *member, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    Resource *m_team;
+    Resource *m_member;
+};
+class KPLATOKERNEL_EXPORT RemoveResourceTeamCmd : public NamedCommand
+{
+public:
+    RemoveResourceTeamCmd( Resource *team, Resource *member, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    Resource *m_team;
+    Resource *m_member;
+};
 
 class KPLATOKERNEL_EXPORT RemoveResourceGroupCmd : public NamedCommand
 {
@@ -1205,8 +1286,8 @@ private:
 class KPLATOKERNEL_EXPORT AddAccountCmd : public NamedCommand
 {
 public:
-    AddAccountCmd( Project &project, Account *account, Account *parent = 0, const QString& name = QString() );
-    AddAccountCmd( Project &project, Account *account, const QString& parent, const QString& name = QString() );
+    AddAccountCmd( Project &project, Account *account, Account *parent = 0, int index = -1, const QString& name = QString() );
+    AddAccountCmd( Project &project, Account *account, const QString& parent, int index = -1, const QString& name = QString() );
     ~AddAccountCmd();
     void execute();
     void unexecute();
@@ -1218,6 +1299,7 @@ private:
     Project &m_project;
     Account *m_account;
     Account *m_parent;
+    int m_index;
     QString m_parentName;
 };
 
@@ -1343,6 +1425,19 @@ private:
     Account *m_newvalue;
 };
 
+class KPLATOKERNEL_EXPORT ResourceModifyAccountCmd : public NamedCommand
+{
+public:
+    ResourceModifyAccountCmd( Resource &resource, Account *oldvalue, Account *newvalue, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    Resource &m_resource;
+    Account *m_oldvalue;
+    Account *m_newvalue;
+};
+
 class KPLATOKERNEL_EXPORT ProjectModifyConstraintCmd : public NamedCommand
 {
 public:
@@ -1389,8 +1484,8 @@ private:
 class KPLATOKERNEL_EXPORT AddScheduleManagerCmd : public NamedCommand
 {
 public:
-    AddScheduleManagerCmd( Project &project, ScheduleManager *sm, const QString& name = 0 );
-    AddScheduleManagerCmd( ScheduleManager *parent, ScheduleManager *sm, const QString& name = 0 );
+    AddScheduleManagerCmd( Project &project, ScheduleManager *sm, int index = -1, const QString& name = QString() );
+    AddScheduleManagerCmd( ScheduleManager *parent, ScheduleManager *sm, int index = -1, const QString& name = QString() );
     ~AddScheduleManagerCmd();
     void execute();
     void unexecute();
@@ -1407,7 +1502,7 @@ protected:
 class KPLATOKERNEL_EXPORT DeleteScheduleManagerCmd : public AddScheduleManagerCmd
 {
 public:
-    DeleteScheduleManagerCmd( Project &project, ScheduleManager *sm, const QString& name = 0 );
+    DeleteScheduleManagerCmd( Project &project, ScheduleManager *sm, const QString& name = QString() );
     void execute();
     void unexecute();
 
@@ -1418,7 +1513,7 @@ private:
 class KPLATOKERNEL_EXPORT ModifyScheduleManagerNameCmd : public NamedCommand
 {
 public:
-    ModifyScheduleManagerNameCmd( ScheduleManager &sm, const QString& value, const QString& name = 0 );
+    ModifyScheduleManagerNameCmd( ScheduleManager &sm, const QString& value, const QString& name = QString() );
     void execute();
     void unexecute();
 
@@ -1430,7 +1525,7 @@ private:
 class KPLATOKERNEL_EXPORT ModifyScheduleManagerAllowOverbookingCmd : public NamedCommand
 {
 public:
-    ModifyScheduleManagerAllowOverbookingCmd( ScheduleManager &sm, bool value, const QString& name = 0 );
+    ModifyScheduleManagerAllowOverbookingCmd( ScheduleManager &sm, bool value, const QString& name = QString() );
     void execute();
     void unexecute();
 
@@ -1442,7 +1537,7 @@ private:
 class KPLATOKERNEL_EXPORT ModifyScheduleManagerDistributionCmd : public NamedCommand
 {
 public:
-    ModifyScheduleManagerDistributionCmd( ScheduleManager &sm, bool value, const QString& name = 0 );
+    ModifyScheduleManagerDistributionCmd( ScheduleManager &sm, bool value, const QString& name = QString() );
     void execute();
     void unexecute();
 
@@ -1454,7 +1549,7 @@ private:
 class KPLATOKERNEL_EXPORT ModifyScheduleManagerCalculateAllCmd : public NamedCommand
 {
 public:
-    ModifyScheduleManagerCalculateAllCmd( ScheduleManager &sm, bool value, const QString& name = 0 );
+    ModifyScheduleManagerCalculateAllCmd( ScheduleManager &sm, bool value, const QString& name = QString() );
     void execute();
     void unexecute();
 
@@ -1466,13 +1561,13 @@ private:
 class KPLATOKERNEL_EXPORT CalculateScheduleCmd : public NamedCommand
 {
 public:
-    CalculateScheduleCmd( Project &project, ScheduleManager &sm, const QString& name = 0 );
+    CalculateScheduleCmd( Project &project, ScheduleManager *sm, const QString& name = QString() );
     void execute();
     void unexecute();
 
 private:
     Project &m_node;
-    ScheduleManager &m_sm;
+    QPointer<ScheduleManager> m_sm;
     bool m_first;
     MainSchedule *m_oldexpected;
     MainSchedule *m_oldoptimistic;
@@ -1485,7 +1580,7 @@ private:
 class KPLATOKERNEL_EXPORT BaselineScheduleCmd : public NamedCommand
 {
 public:
-    explicit BaselineScheduleCmd( ScheduleManager &sm, const QString& name = 0 );
+    explicit BaselineScheduleCmd( ScheduleManager &sm, const QString& name = QString() );
     void execute();
     void unexecute();
 
@@ -1496,7 +1591,7 @@ private:
 class KPLATOKERNEL_EXPORT ResetBaselineScheduleCmd : public NamedCommand
 {
 public:
-    explicit ResetBaselineScheduleCmd( ScheduleManager &sm, const QString& name = 0 );
+    explicit ResetBaselineScheduleCmd( ScheduleManager &sm, const QString& name = QString() );
     void execute();
     void unexecute();
 
@@ -1507,7 +1602,7 @@ private:
 class KPLATOKERNEL_EXPORT ModifyScheduleManagerSchedulingDirectionCmd : public NamedCommand
 {
 public:
-    ModifyScheduleManagerSchedulingDirectionCmd( ScheduleManager &sm, bool value, const QString& name = 0 );
+    ModifyScheduleManagerSchedulingDirectionCmd( ScheduleManager &sm, bool value, const QString& name = QString() );
     void execute();
     void unexecute();
 
@@ -1519,7 +1614,7 @@ private:
 class KPLATOKERNEL_EXPORT ModifyScheduleManagerSchedulerCmd : public NamedCommand
 {
 public:
-    ModifyScheduleManagerSchedulerCmd( ScheduleManager &sm, int value, const QString& name = 0 );
+    ModifyScheduleManagerSchedulerCmd( ScheduleManager &sm, int value, const QString& name = QString() );
     void execute();
     void unexecute();
 
