@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
   Copyright (C) 2001 Thomas Zander zander@kde.org
-  Copyright (C) 2004 - 2007 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2004 - 2010 Dag Andersen <danders@get2net.dk>
   Copyright (C) 2007 Florian Piquemal <flotueur@yahoo.fr>
   Copyright (C) 2007 Alexis Ménard <darktears31@gmail.com>
 
@@ -39,6 +39,7 @@
 #include <QMap>
 #include <QList>
 #include <QHash>
+#include <QPointer>
 
 #include <klocale.h>
 #include <ktimezone.h>
@@ -79,6 +80,11 @@ public:
     explicit Project( ConfigBase &config, Node *parent = 0 );
     ~Project();
 
+    /// Reference this project.
+    void ref() { ++m_refCount; }
+    /// De-reference this project. Deletes project of ref count <= 0
+    void deref();
+
     /// Returns the node type. Can be Type_Project or Type_Subproject.
     virtual int type() const;
 
@@ -196,38 +202,39 @@ public:
     /// Returns a list of all resources
     QList<Resource*> resourceList() const { return resourceIdDict.values(); }
 
-    virtual EffortCostMap plannedEffortCostPrDay( const QDate &start, const QDate &end, long id = -1 ) const;
+    virtual EffortCostMap plannedEffortCostPrDay( const QDate &start, const QDate &end, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
+    virtual EffortCostMap plannedEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
 
     /// Returns the total planned effort for this project (or subproject)
-    virtual Duration plannedEffort( long id = -1 ) const;
+    virtual Duration plannedEffort( long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     /// Returns the total planned effort for this project (or subproject) on date
-    virtual Duration plannedEffort( const QDate &date, long id = -1  ) const;
+    virtual Duration plannedEffort( const QDate &date, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All  ) const;
     /// Returns the planned effort up to and including date
-    virtual Duration plannedEffortTo( const QDate &date, long id = -1  ) const;
+    virtual Duration plannedEffortTo( const QDate &date, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All  ) const;
 
     /// Returns the actual effort
     virtual Duration actualEffort() const;
     /// Returns the actual effort on @p date
-    virtual Duration actualEffort( const QDate &date) const;
+    virtual Duration actualEffort( const QDate &date ) const;
     /// Returns the actual effort up to and including @p date
     virtual Duration actualEffortTo( const QDate &date ) const;
     /**
      * Returns the total planned cost for this project
      * @param id Identity of the schedule to be used
      */
-    virtual EffortCost plannedCost( long id = CURRENTSCHEDULE ) const;
+    virtual EffortCost plannedCost( long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     /**
      * Planned cost on date
      * @param date The cost is calulated for this date (only)
      * @param id Identity of the schedule to be used
      */
-    virtual double plannedCost( const QDate &date, long id = -1  ) const;
+    virtual double plannedCost( const QDate &date, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All  ) const;
     /**
      * Planned cost up to and including date
      * @param date The cost is calculated from the start of the project upto including date.
      * @param id Identity of the schedule to be used.
      */
-    virtual double plannedCostTo( const QDate &date, long id = -1  ) const;
+    virtual double plannedCostTo( const QDate &date, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All  ) const;
 
     /**
      * Returns the actually reported cost for this project
@@ -245,8 +252,10 @@ public:
      */
     virtual EffortCost actualCostTo( const QDate &date ) const;
     
-    virtual EffortCostMap actualEffortCostPrDay( const QDate &start, const QDate &end, long id = -1 ) const;
-    
+    virtual EffortCostMap actualEffortCostPrDay( const QDate &start, const QDate &end, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
+
+    virtual EffortCostMap actualEffortCostPrDay( const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
+
     double effortPerformanceIndex( const QDate &date, long id ) const;
     
     double schedulePerformanceIndex( const QDate &date, long id ) const;
@@ -266,7 +275,7 @@ public:
     Calendar *defaultCalendar() const { return m_defaultCalendar; }
     void setDefaultCalendar( Calendar *cal );
     const QList<Calendar*> &calendars() const;
-    void addCalendar( Calendar *calendar, Calendar *parent = 0 );
+    void addCalendar( Calendar *calendar, Calendar *parent = 0, int index = -1 );
     void takeCalendar( Calendar *calendar );
     int indexOf( const Calendar *calendar ) const;
     /// Returns the calendar with identity id.
@@ -299,32 +308,29 @@ public:
 
     virtual const QHash<QString, Node*> &nodeDict() { return nodeIdDict; }
     /// Return a list of all nodes in the project (exluding myself)
-    QList<Node*> allNodes();
+    QList<Node*> allNodes() const;
     /// Return the number of all nodes in the project (exluding myself)
     int nodeCount() const { return nodeIdDict.count() - 1; }
 
+    /// Return a list of all tasks and milestones int the wbs order
+    QList<Task*> allTasks( const Node *parent = 0 ) const;
+
     using Node::findNode;
     /// Find the node with identity id
-    virtual Node *findNode( const QString &id ) const
-    {
-        if ( m_parent == 0 ) {
-            if ( nodeIdDict.contains( id ) )
-                return nodeIdDict[ id ];
-            return 0;
-        }
-        return m_parent->findNode( id );
-    }
+    virtual Node *findNode( const QString &id ) const;
+
     using Node::removeId;
-    /// Remove the node with identity id from the register
+    /// Remove the node with identity id from the registers
     virtual bool removeId( const QString &id );
     
-    using Node::insertId;
-    /// Insert the node with identity id
-    virtual void insertId( const QString &id, Node *node );
-    /// Register node. The nodes id must be unique and non-empty.
+    /// Reserve @p id for the @p node
+    virtual void reserveId( const QString &id, Node *node );
+    /// Register @p node. The nodes id must be unique and non-empty.
     bool registerNodeId( Node *node );
     /// Create a unique id.
-    QString uniqueNodeId( int seed = 1 );
+    QString uniqueNodeId( int seed = 1 ) const;
+    /// Check if node @p id is used
+    bool nodeIdentExists( const QString &id ) const;
 
     /// Create a unique id.
     QString uniqueNodeId( const QList<QString> &existingIds, int seed = 1 );
@@ -393,7 +399,7 @@ public:
     const Accounts &accounts() const { return m_accounts; }
 
     /**
-     * Set current schedule to schedule with identity id, for me and my children
+     * Set current schedule to the schedule with identity @p id, for me and my children
      * Note that this is used (and may be changed) when calculating schedules
      */
     virtual void setCurrentSchedule( long id );
@@ -406,8 +412,14 @@ public:
     /// Set parent schedule for my children
     virtual void setParentSchedule( Schedule *sch );
     
+    /// Find the schedule manager that manages the Schedule with @p id
     ScheduleManager *scheduleManager( long id ) const;
+    /// Find the schedule manager with @p id
+    ScheduleManager *scheduleManager( const QString &id ) const;
+    /// Create a unique schedule name (This may later be changed by the user)
     QString uniqueScheduleName() const;
+    /// Create a unique schedule manager identity
+    QString uniqueScheduleManagerId() const;
     ScheduleManager *createScheduleManager();
     ScheduleManager *createScheduleManager( const QString name );
     /// Returns a list of all top level schedule managers
@@ -415,9 +427,9 @@ public:
     int numScheduleManagers() const { return m_managers.count(); }
     int indexOf( const ScheduleManager *sm ) const { return m_managers.indexOf( const_cast<ScheduleManager*>(sm) ); }
     bool isScheduleManager( void* ptr ) const;
-    void addScheduleManager( ScheduleManager *sm, ScheduleManager *parent = 0 );
+    void addScheduleManager( ScheduleManager *sm, ScheduleManager *parent = 0, int index = -1 );
     int takeScheduleManager( ScheduleManager *sm );
-    ScheduleManager *findScheduleManager( const QString &name ) const;
+    ScheduleManager *findScheduleManagerByName( const QString &name ) const;
     /// Returns a list of all schedule managers
     QList<ScheduleManager*> allScheduleManagers() const;
     /// Return true if schedule with identity @p id is baselined
@@ -479,27 +491,43 @@ public:
     void generateUniqueNodeIds();
     void generateUniqueIds();
     
-    const ConfigBase &config() const { return *m_config; }
+    const ConfigBase &config() const { return m_config ? *m_config : emptyConfig; }
     /// Set configuration data
     void setConfig( ConfigBase *config ) { m_config = config; }
 
-    const Task &taskDefaults() const { return m_config->taskDefaults(); }
+    const Task &taskDefaults() const { return config().taskDefaults(); }
 
     /// Return locale. (Used for currency, everything else is from KGlobal::locale)
-    KLocale *locale() { return m_config->locale(); }
+    KLocale *locale() { return const_cast<ConfigBase&>(config()).locale(); }
     /// Return locale. (Used for currency, everything else is from KGlobal::locale)
-    const KLocale *locale() const { return m_config->locale(); }
+    const KLocale *locale() const { return config().locale(); }
     /// Signal that locale data has changed
     void emitLocaleChanged();
     
-    void incProgress();
-
     void setSchedulerPlugins( const QMap<QString, SchedulerPlugin*> &plugins );
     const QMap<QString, SchedulerPlugin*> &schedulerPlugins() const { return m_schedulerPlugins; }
 
     void initiateCalculation( MainSchedule &sch );
     void initiateCalculationLists( MainSchedule &sch );
-    
+
+    void finishCalculation( ScheduleManager &sm );
+    void adjustSummarytask();
+
+    /// Increments progress and emits signal sigProgress()
+    void incProgress();
+    /// Emits signal maxProgress()
+    void emitMaxProgress( int value );
+
+    bool stopcalculation;
+
+public slots:
+    /// Sets m_progress to @p progress and emits signal sigProgress()
+    /// If @p sm is not 0, progress is also set for the schedule manager
+    void setProgress( int progress, ScheduleManager *sm = 0 );
+    /// Sets m_maxprogress to @p max and emits signal maxProgress()
+    /// If @p sm is not 0, max progress is also set for the schedule manager
+    void setMaxProgress( int max, ScheduleManager *sm = 0 );
+
 signals:
     /// Emitted when anything in the project is changed (use with care)
     void changed();
@@ -511,7 +539,12 @@ signals:
     void currentScheduleChanged();
     /// Use to show progress during calculation
     void sigProgress( int );
+    /// Use to set the maximum progress (minimum always 0)
     void maxProgress( int );
+    /// Emitted when calculation starts
+    void sigCalculationStarted( Project *project, ScheduleManager *sm );
+    /// Emitted when calculation is finished
+    void sigCalculationFinished( Project *project, ScheduleManager *sm );
     /// This signal is emitted when one of the nodes members is changed.
     void nodeChanged( Node* );
     /// This signal is emitted when the node is to be added to the project.
@@ -597,6 +630,11 @@ protected:
     /// Calculate critical path
     virtual bool calcCriticalPath( bool fromEnd );
 
+    /// Prepare task lists for scheduling
+    void tasksForward();
+    /// Prepare task lists for scheduling
+    void tasksBackward();
+
 protected:
     virtual void changed(Node *node);
     
@@ -612,7 +650,6 @@ protected:
     DateTime calculateBackward( int use );
     DateTime scheduleForward( const DateTime &earliest, int use );
     DateTime scheduleBackward( const DateTime &latest, int use );
-    void adjustSummarytask();
     DateTime checkStartConstraints( const DateTime &dt ) const;
     DateTime checkEndConstraints( const DateTime &dt ) const;
 
@@ -625,23 +662,28 @@ private:
     QHash<QString, ResourceGroup*> resourceGroupIdDict;
     QHash<QString, Resource*> resourceIdDict;
     QHash<QString, Node*> nodeIdDict;
+    QMap<QString, Node*> nodeIdReserved;
     QMap<QString, Calendar*> calendarIdDict;
+    QMap<QString, ScheduleManager*> m_managerIdMap;
 
     QList<ScheduleManager*> m_managers;
     KDateTime::Spec m_spec;
     
     WBSDefinition m_wbsDefinition;
     
-    //use in pert to store the project slack
-    int m_projectSlack;
-
-    ConfigBase *emptyConfig;
-    ConfigBase *m_config; // this one is not owned by me, don't delete
+    ConfigBase emptyConfig;
+    QPointer<ConfigBase> m_config; // this one is not owned by me, don't delete
 
     int m_progress;
 
     QMap<QString, SchedulerPlugin*> m_schedulerPlugins;
 
+    int m_refCount; // make it possible to use the project by different threads
+
+    QList<Task*> m_hardConstraints;
+    QList<Task*> m_softConstraints;
+    QList<Task*> m_terminalNodes;
+    
 #ifndef NDEBUG
 public:
     void printDebug( bool children, const QByteArray& indent );
