@@ -27,6 +27,7 @@
 #include "kptduration.h"
 #include "kptdatetime.h"
 #include "kptappointment.h"
+#include "kptcalendar.h"
 
 #include <qdom.h>
 #include <QHash>
@@ -44,6 +45,7 @@ class QTime;
 namespace KPlato
 {
 
+class Account;
 class Risk;
 class Effort;
 class Appointment;
@@ -53,13 +55,13 @@ class Project;
 class Resource;
 class ResourceRequest;
 class ResourceGroupRequest;
-class Calendar;
 class ResourceRequestCollection;
 class Schedule;
 class NodeSchedule;
 class ResourceSchedule;
 class Schedule;
 class XMLLoaderObject;
+class DateTimeInterval;
 
 /**
   * This class represents a group of similar resources to be assigned to a task
@@ -144,7 +146,7 @@ public:
     bool load( KoXmlElement &element, XMLLoaderObject &status );
     void save( QDomElement &element ) const;
     
-    /// Save workpackage document. Include only resources listed in @lst
+    /// Save workpackage document. Include only resources listed in @p lst
     void saveWorkPackageXML( QDomElement &element, const QList<Resource*> lst ) const;
 
     void initiateCalculation( Schedule &sch );
@@ -231,7 +233,7 @@ public:
     QString id() const { return m_id; }
     void setId( const QString& id );
 
-    enum Type { Type_Work, Type_Material };
+    enum Type { Type_Work, Type_Material, Type_Team };
     void setType( Type type );
     void setType( const QString &type );
     Type type() const { return m_type; }
@@ -253,13 +255,13 @@ public:
     ResourceGroup *parentGroup() const { return m_parent; }
     
     /// Set the time from when the resource is available to this project
-    void setAvailableFrom( const QDateTime &af ) { m_availableFrom.setDateTime( af ); changed();}
+    void setAvailableFrom( const QDateTime &af ) { m_availableFrom = KDateTime( af ); changed();}
     /// Set the time from when the resource is available to this project
     void setAvailableFrom( const DateTime &af ) { m_availableFrom = af; changed(); }
     /// Return the time when the resource is available to this project
     const DateTime &availableFrom() const { return m_availableFrom;}
     /// Set the time when the resource is no longer available to this project
-    void setAvailableUntil( const QDateTime &au ) { m_availableUntil.setDateTime( au ); changed(); }
+    void setAvailableUntil( const QDateTime &au ) { m_availableUntil = KDateTime( au ); changed(); }
     /// Set the time when the resource is no longer available to this project
     void setAvailableUntil( const DateTime &au ) { m_availableUntil = au; changed(); }
     /// Return the time when the resource is no longer available to this project.
@@ -281,17 +283,16 @@ public:
     Appointment *appointmentAt( int index, long id = -1 ) const { return appointments( id ).value( index ); }
     int indexOf( Appointment *a, long id = -1 ) const { return appointments( id ).indexOf( a ); }
     
-    Appointment *findAppointment( Node *node );
     /// Adds appointment to current schedule
     virtual bool addAppointment( Appointment *appointment );
     /// Adds appointment to schedule sch
     virtual bool addAppointment( Appointment *appointment, Schedule &main );
     /// Adds appointment to both this resource and node
-    virtual void addAppointment( Schedule *node, DateTime &start, DateTime &end, double load = 100 );
+    virtual void addAppointment( Schedule *node, const DateTime &start, const DateTime &end, double load = 100 );
 
     void initiateCalculation( Schedule &sch );
     bool isAvailable( Task *task );
-    void makeAppointment( Schedule *schedule, int load );
+    void makeAppointment( Schedule *schedule, int load, const QList<Resource*> &required = QList<Resource*>() );
 
     bool isOverbooked() const;
     /// check if overbooked on date.
@@ -311,19 +312,24 @@ public:
     /**
      * Set available units in percent
      */
-    void setUnits( int units ) { m_units = units; changed(); }
+    void setUnits( int units );
 
     Project *project() const { return m_project; }
     /// Return the resources timespec. Defaults to local.
     KDateTime::Spec timeSpec() const;
     
     /**
-     * Get the calendar for this resource. 
-     * If local=false, check if there is a default calendar.
+     * Get the calendar for this resource.
+     * Working resources may have a default calendar if the a calendar is marked as default,
+     * this is checked if local=false.
+     * If no calendar can be found for a working resource, the resource is not available.
+     *
+     * Material resources must have calendar explicitly set.
+     * If there is no calendar set for a material resource, the resource is always available.
      */
     Calendar *calendar( bool local = false ) const;
     //Calendar *calendar( const QString& id ) const;
-    void setCalendar( Calendar *calendar ) { m_calendar = calendar; changed(); }
+    void setCalendar( Calendar *calendar );
 
     /// Delete all requests for me
     void removeRequests();
@@ -341,17 +347,26 @@ public:
     const QList<ResourceRequest*> &requests() const
     { return m_requests; }
 
+    /// Returns a list of work intervals in the interval @p from, @p until.
+    /// Appointments are subtracted if @p schedule is not 0.
+    AppointmentIntervalList workIntervals( const DateTime &from, const DateTime &until, Schedule *schedule = 0 ) const;
+
+    /// Updates work interval cache a list of work intervals extracted from the resource calendar
+    /// with @p load in the interval @p from, @p until.
+    /// The load of the intervals is set to m_units
+    /// Note: The list may contain intervals outside @p from, @p until
+    void calendarIntervals( const DateTime &from, const DateTime &until ) const;
+
     /// Returns the effort that can be done starting at @p start within @p duration.
     /// The current schedule is used to check for appointments.
     /// If @p  backward is true, checks backward in time.
-    /// Status is returned in @p ok
-    Duration effort( const DateTime &start, const Duration &duration, bool backward = false, bool *ok = 0 ) const;
+    Duration effort( const DateTime &start, const Duration &duration, bool backward = false, const QList<Resource*> &required = QList<Resource*>() ) const;
 
     /// Returns the effort that can be done starting at @p start within @p duration.
     /// The schedule @p sch is used to check for appointments.
     /// If @p  backward is true, checks backward in time.
     /// Status is returned in @p ok
-    Duration effort( Schedule *sch, const DateTime &start, const Duration &duration, bool backward = false, bool *ok = 0 ) const;
+    Duration effort( Schedule *sch, const DateTime &start, const Duration &duration, bool backward = false, const QList<Resource*> &required = QList<Resource*>() ) const;
 
 
     /**
@@ -390,14 +405,16 @@ public:
 
     Appointment appointmentIntervals( long id ) const;
     Appointment appointmentIntervals() const;
-    Duration plannedEffort( const QDate &date ) const;
+    
+    EffortCostMap plannedEffortCostPrDay( const QDate &start, const QDate &end, long id, EffortCostCalculationType = ECCT_All );
+    Duration plannedEffort( const QDate &date, EffortCostCalculationType = ECCT_All ) const;
 
     void setCurrentSchedulePtr( Schedule *schedule ) { m_currentSchedule = schedule; }
     void setCurrentSchedule( long id ) { m_currentSchedule = findSchedule( id ); }
     Schedule *currentSchedule() const { return m_currentSchedule; }
 
     bool isScheduled() const;
-    QHash<long, Schedule*> &schedules() { return m_schedules; }
+    QHash<long, Schedule*> schedules() const { return m_schedules; }
     /**
      * Return schedule with @id
      * If @id == -1, return m_currentSchedule
@@ -437,6 +454,43 @@ public:
     DateTime startTime( long id ) const;
     DateTime endTime( long id ) const;
 
+    /// Returns the list requiered resources.
+    /// Note: This list is used as default for allocation dialog, not for scheduling.
+    QList<Resource*> requiredResources() const { return m_required; }
+    /// Set the list of required resources.
+    void setRequiredResources( const QList<Resource*> &lst ) { m_required = lst; }
+
+    /// Return the list of team members.
+    QList<Resource*> teamMembers() const { return m_teamMembers; }
+    /// Clear the list of team members.
+    void clearTeamMembers() { m_teamMembers.clear(); }
+    /// Add @p resource to the list of team members.
+    void addTeamMember( Resource *resource );
+    /// Remove @p resource to the list of team members.
+    void removeTeamMember( Resource *resource );
+
+    /// Used by Project::load() after all resources have been loaded
+    /// to translate resource ids to resources
+    void resolveRequiredResources( Project &project );
+
+    /// Return the account
+    Account *account() const { return cost.account; }
+    /// Set the @p account
+    void setAccount( Account *account );
+
+    class WorkInfoCache
+    {
+    public:
+        WorkInfoCache() { clear(); }
+        void clear() { start = end = DateTime(); effort = Duration::zeroDuration; intervals.clear(); version = -1; }
+        bool isValid() const { return start.isValid() && end.isValid(); }
+        DateTime start;
+        DateTime end;
+        Duration effort;
+        AppointmentIntervalList intervals;
+        int version;
+    };
+
 signals:
     void externalAppointmentToBeAdded( Resource *r, int row );
     void externalAppointmentAdded( Resource*, Appointment* );
@@ -445,7 +499,8 @@ signals:
     void externalAppointmentChanged( Resource *r, Appointment *a );
 
 protected:
-    void makeAppointment( Schedule *node, const DateTime &from, const DateTime &end, int load );
+    DateTimeInterval requiredAvailable(Schedule *node, const DateTime &start, const DateTime &end ) const;
+    void makeAppointment( Schedule *node, const DateTime &from, const DateTime &end, int load, const QList<Resource*> &required = QList<Resource*>() );
     virtual void changed();
 
 private:
@@ -469,15 +524,24 @@ private:
         double normalRate;
         double overtimeRate;
         double fixed ;
+        Account *account;
     }
     cost;
     
     Calendar *m_calendar;
-    QList<ResourceRequest*>
-    m_requests;
+    QList<ResourceRequest*> m_requests;
+    QList<Resource*> m_required;
+    QStringList m_requiredIds;
     
+    QList<Resource*> m_teamMembers;
+
     Schedule *m_currentSchedule;
 
+    mutable WorkInfoCache m_workinfocache;
+
+    // return this if resource has no calendar and is a material resource
+    Calendar m_materialCalendar;
+
 #ifndef NDEBUG
 public:
     void printDebug( const QString& ident );
@@ -518,6 +582,7 @@ class KPLATOKERNEL_EXPORT ResourceRequest
 {
 public:
     explicit ResourceRequest( Resource *resource = 0, int units = 1 );
+    explicit ResourceRequest( const ResourceRequest &r );
 
     ~ResourceRequest();
 
@@ -536,11 +601,6 @@ public:
     int units() const;
     void setUnits( int value );
 
-    /**
-    * Get amount of requested work units in percent
-    */
-    int workUnits() const;
-
     void registerRequest()
     {
         if ( m_resource )
@@ -552,15 +612,25 @@ public:
             m_resource->unregisterRequest( this );
     }
 
+    void makeAppointment( Schedule *schedule, int amount );
     void makeAppointment( Schedule *schedule );
     Task *task() const;
 
-    Schedule *resourceSchedule( Schedule *ns );
+    /// Return the datetime from when the resource is available.
+    /// If it is not valid, the project constraint start time is used.
+    /// For teams the earliest time for any team member is used.
+    DateTime availableFrom();
+    /// Return the datetime until when the resource is available.
+    /// If it is not valid, the project constraint end time is used.
+    /// For teams the latest time for any team member is used.
+    DateTime availableUntil();
+
+    Schedule *resourceSchedule( Schedule *ns, Resource *resource = 0 );
     DateTime availableAfter(const DateTime &time, Schedule *ns);
     DateTime availableBefore(const DateTime &time, Schedule *ns);
-    Duration effort( const DateTime &time, const Duration &duration, Schedule *ns, bool backward, bool *ok = 0 );
-    DateTime workTimeAfter(const DateTime &dt);
-    DateTime workTimeBefore(const DateTime &dt);
+    Duration effort( const DateTime &time, const Duration &duration, Schedule *ns, bool backward );
+    DateTime workTimeAfter(const DateTime &dt, Schedule *ns = 0);
+    DateTime workTimeBefore(const DateTime &dt, Schedule *ns = 0);
 
     /// Resource is allocated dynamically by the group request
     bool isDynamicallyAllocated() const { return m_dynamic; }
@@ -570,25 +640,41 @@ public:
     /// Return a measure of how suitable the resource is for allocation
     long allocationSuitability( const DateTime &time, const Duration &duration, Schedule *ns, bool backward );
 
+    /// Returns a list of all the required resources that will be used in scheduling.
+    /// Note: This list overrides the resources own list which is just used as default for allocation dialog.
+    QList<Resource*> requiredResources() const { return m_required; }
+    /// Set the list of required resources that will be used in scheduling.
+    void setRequiredResources( const QList<Resource*> &lst ) { m_required = lst; }
+
+    QList<ResourceRequest*> teamMembers() const;
+
 protected:
     void changed();
 
+    void setCurrentSchedulePtr( Schedule *ns );
+    void setCurrentSchedulePtr( Resource *resource, Schedule *ns );
+
 private:
     Resource *m_resource;
     int m_units;
     ResourceGroupRequest *m_parent;
     bool m_dynamic;
+    QList<Resource*> m_required;
+    mutable QList<ResourceRequest*> m_teamMembers;
 
 #ifndef NDEBUG
 public:
     void printDebug( const QString& ident );
 #endif
 };
+QDebug &operator<<( QDebug &dbg, const KPlato::ResourceRequest *r );
+QDebug &operator<<( QDebug &dbg, const KPlato::ResourceRequest &r );
 
 class KPLATOKERNEL_EXPORT ResourceGroupRequest
 {
 public:
     explicit ResourceGroupRequest( ResourceGroup *group = 0, int units = 0 );
+    explicit ResourceGroupRequest( const ResourceGroupRequest &group );
     ~ResourceGroupRequest();
 
     void setParent( ResourceRequestCollection *parent ) { m_parent = parent;}
@@ -597,7 +683,10 @@ public:
     ResourceGroup *group() const { return m_group; }
     void setGroup( ResourceGroup *group ) { m_group = group; }
     void unregister( const ResourceGroup *group ) { if ( group == m_group ) m_group = 0; }
-    QList<ResourceRequest*> &resourceRequests() { return m_resourceRequests; }
+    /// Return a list of resource requests.
+    /// If @p resolveTeam is true, include the team members,
+    /// if @p resolveTeam is false, include the team resource itself.
+    QList<ResourceRequest*> resourceRequests( bool resolveTeam=true ) const;
     void addResourceRequest( ResourceRequest *request );
     void deleteResourceRequest( ResourceRequest *request );
     int count() const { return m_resourceRequests.count(); }
@@ -608,7 +697,11 @@ public:
     ResourceRequest *resourceRequest( const QString &name );
     /// Return a list of allocated resources, allocation to group is not included by default.
     QStringList requestNameList( bool includeGroup = false ) const;
-    
+    /// Return a list of allocated resources.
+    /// Allocations to groups are not included.
+    /// Team resources are included but *not* the team members.
+    /// Any dynamically allocated resource is not included.
+    QList<Resource*> requestedResources() const;
     bool load( KoXmlElement &element, Project &project );
     void save( QDomElement &element ) const;
 
@@ -617,25 +710,14 @@ public:
     void setUnits( int value ) { m_units = value; changed(); }
 
     /**
-    * Get amount of allocated work units in percent
-    */
-    int workUnits() const;
-
-    /**
-    * Get the number of resources allocated
-    */
-    int workAllocation() const;
-
-    /**
-     * Returns the duration needed to do the effort  effort
-     * starting at start.
+     * Returns the duration needed to do the @p effort starting at @p start.
      */
     Duration duration( const DateTime &start, const Duration &effort, Schedule *ns, bool backward = false );
 
     DateTime availableAfter( const DateTime &time, Schedule *ns );
     DateTime availableBefore( const DateTime &time, Schedule *ns );
-    DateTime workTimeAfter(const DateTime &dt);
-    DateTime workTimeBefore(const DateTime &dt);
+    DateTime workTimeAfter(const DateTime &dt, Schedule *ns = 0);
+    DateTime workTimeBefore(const DateTime &dt, Schedule *ns = 0);
 
     /**
      * Makes appointments for task @param task to the 
@@ -680,7 +762,7 @@ public:
     explicit ResourceRequestCollection( Task *task = 0 );
     ~ResourceRequestCollection();
 
-    const QList<ResourceGroupRequest*> &requests() const { return m_requests; }
+    QList<ResourceGroupRequest*> requests() const { return m_requests; }
     void addRequest( ResourceGroupRequest *request );
     void deleteRequest( ResourceGroupRequest *request )
     {
@@ -713,38 +795,34 @@ public:
 
     bool contains( const QString &identity ) const;
     ResourceGroupRequest *findGroupRequestById( const QString &id ) const;
-    /// Return a list of allocated resources, allocations to groups are not included by default.
+    /// Return a list of names of allocated resources.
+    /// Allocations to groups are not included by default.
+    /// Team resources are included but *not* the team members.
+    /// Any dynamically allocated resource is not included.
     QStringList requestNameList( bool includeGroup = false ) const;
+    /// Return a list of allocated resources.
+    /// Allocations to groups are not included.
+    /// Team resources are included but *not* the team members.
+    /// Any dynamically allocated resource is not included.
     QList<Resource*> requestedResources() const;
+
+    /// Return a list of all resource requests.
+    /// If @p resolveTeam is true, include the team members,
+    /// if @p resolveTeam is false, include the team resource itself.
+    QList<ResourceRequest*> resourceRequests( bool resolveTeam=true ) const;
     
     //bool load(KoXmlElement &element, Project &project);
     void save( QDomElement &element ) const;
 
     /**
-    * Returns the total amount of resource units in percent
-    */
-//    int units() const;
-
-    /**
-    * Returns the amount of allocated work units in percent
-    */
-    int workUnits() const;
-
-    /**
-    * Returns the number of allocated working resources
-    */
-    int workAllocation() const;
-
-    /**
-    * Returns the duration needed to do the effort @param effort
-    * starting at @param time.
+    * Returns the duration needed to do the @p effort starting at @p time.
     */
     Duration duration( const DateTime &time, const Duration &effort, Schedule *sch, bool backward = false );
 
     DateTime availableAfter( const DateTime &time, Schedule *ns );
     DateTime availableBefore( const DateTime &time, Schedule *ns );
-    DateTime workTimeAfter(const DateTime &dt) const;
-    DateTime workTimeBefore(const DateTime &dt) const;
+    DateTime workTimeAfter(const DateTime &dt, Schedule *ns = 0) const;
+    DateTime workTimeBefore(const DateTime &dt, Schedule *ns = 0) const;
     DateTime workStartAfter(const DateTime &time, Schedule *ns);
     DateTime workFinishBefore(const DateTime &time, Schedule *ns);
 
@@ -763,19 +841,10 @@ public:
 
     void changed();
     
-    Duration effort(const QList<ResourceRequest*> &lst, const DateTime &time, const Duration &duration, Schedule *ns, bool backward, bool *ok) const;
+    Duration effort( const QList<ResourceRequest*> &lst, const DateTime &time, const Duration &duration, Schedule *ns, bool backward ) const;
     int numDays(const QList<ResourceRequest*> &lst, const DateTime &time, bool backward) const;
     Duration duration(const QList<ResourceRequest*> &lst, const DateTime &time, const Duration &_effort, Schedule *ns, bool backward);
 
-protected:
-    struct Interval
-    {
-        DateTime start;
-        DateTime end;
-        Duration effort;
-    };
-
-
 private:
     Task *m_task;
     QList<ResourceGroupRequest*> m_requests;
