@@ -26,6 +26,7 @@
 #include "kptproject.h"
 #include "kpttask.h"
 #include "kptxmlloaderobject.h"
+#include "kptschedulerplugin.h"
 
 #include <KoXmlReader.h>
 
@@ -47,6 +48,7 @@ Schedule::Schedule()
         m_id( 0 ),
         m_deleted( false ),
         m_parent( 0 ),
+        m_obstate( OBS_Parent ),
         m_calculationMode( Schedule::Scheduling )
 {}
 
@@ -55,6 +57,7 @@ Schedule::Schedule( Schedule *parent )
         m_id( 0 ),
         m_deleted( false ),
         m_parent( parent ),
+        m_obstate( OBS_Parent ),
         m_calculationMode( Schedule::Scheduling )
 {
 
@@ -72,6 +75,7 @@ Schedule::Schedule( const QString& name, Type type, long id )
         m_id( id ),
         m_deleted( false ),
         m_parent( 0 ),
+        m_obstate( OBS_Parent ),
         m_calculationMode( Schedule::Scheduling )
 {
 
@@ -167,12 +171,22 @@ bool Schedule::usePert() const
     return false;
 }
 
+void Schedule::setAllowOverbookingState( Schedule::OBState state )
+{
+    m_obstate = state;
+}
+
+Schedule::OBState Schedule::allowOverbookingState() const
+{
+    return m_obstate;
+}
+
 bool Schedule::allowOverbooking() const
 {
-    if ( m_parent ) {
+    if ( m_obstate == OBS_Parent && m_parent ) {
         return m_parent->allowOverbooking();
     }
-    return false;
+    return m_obstate == OBS_Allow;
 }
 
 bool Schedule::checkExternalAppointments() const
@@ -220,6 +234,26 @@ void Schedule::calcResourceOverbooked()
     }
 }
 
+DateTimeInterval Schedule::firstBookedInterval( const DateTimeInterval &interval, const Schedule *node  ) const
+{
+    QList<Appointment*> lst = m_appointments;
+    switch ( m_calculationMode ) {
+        case CalculateForward: lst = m_forward; break;
+        case CalculateBackward: lst = m_backward; break;
+        default: break;
+    }
+    foreach ( Appointment *a, lst ) {
+        if ( a->node() == node ) {
+            AppointmentIntervalList i = a->intervals( interval.first, interval.second );
+            if ( i.isEmpty() ) {
+                break;
+            }
+            return DateTimeInterval( i.values().first().startTime(), i.values().first().endTime() );
+        }
+    }
+    return DateTimeInterval();
+}
+
 QStringList Schedule::overbookedResources() const
 {
     QStringList rl;
@@ -337,6 +371,7 @@ bool Schedule::add( Appointment *appointment )
 
 void Schedule::takeAppointment( Appointment *appointment, int mode )
 {
+    Q_UNUSED(mode);
     //kDebug()<<"("<<this<<")"<<mode<<":"<<appointment<<","<<appointment->calculationMode();
     int i = m_forward.indexOf( appointment );
     if ( i != -1 ) {
@@ -384,7 +419,28 @@ Appointment *Schedule::findAppointment( Schedule *resource, Schedule *node, int
     return 0;
 }
 
-QList<Appointment*> &Schedule::appointments( int which)
+DateTime Schedule::appointmentStartTime() const
+{
+    DateTime dt;
+    foreach ( const Appointment *a, m_appointments ) {
+        if ( ! dt.isValid() || dt > a->startTime() ) {
+            dt = a->startTime();
+        }
+    }
+    return dt;
+}
+DateTime Schedule::appointmentEndTime() const
+{
+    DateTime dt;
+    foreach ( const Appointment *a, m_appointments ) {
+        if ( ! dt.isValid() || dt > a->endTime() ) {
+            dt = a->endTime();
+        }
+    }
+    return dt;
+}
+
+QList<Appointment*> Schedule::appointments( int which ) const
 {
     if ( which == CalculateForward ) {
         return m_forward;
@@ -418,90 +474,103 @@ Appointment Schedule::appointmentIntervals( int which ) const
 }
 
 
-EffortCostMap Schedule::bcwsPrDay() const
+EffortCostMap Schedule::bcwsPrDay( EffortCostCalculationType type ) const
 {
     //kDebug()<<m_name<<m_appointments;
     EffortCostMap ec;
     foreach ( Appointment *a, m_appointments ) {
-        ec += a->plannedPrDay( a->startTime().date(), a->endTime().date() );
+        ec += a->plannedPrDay( a->startTime().date(), a->endTime().date(), type );
     }
     return ec;
 }
 
-EffortCostMap Schedule::plannedEffortCostPrDay( const QDate &start, const QDate &end ) const
+EffortCostMap Schedule::plannedEffortCostPrDay( const QDate &start, const QDate &end, EffortCostCalculationType type ) const
 {
     //kDebug()<<m_name<<m_appointments;
     EffortCostMap ec;
     QListIterator<Appointment*> it( m_appointments );
     while ( it.hasNext() ) {
         //kDebug()<<m_name;
-        ec += it.next() ->plannedPrDay( start, end );
+        ec += it.next() ->plannedPrDay( start, end, type );
     }
     return ec;
 }
 
-Duration Schedule::plannedEffort() const
+EffortCostMap Schedule::plannedEffortCostPrDay( const Resource *resource, const QDate &start, const QDate &end, EffortCostCalculationType type ) const
+{
+    //kDebug()<<m_name<<m_appointments;
+    EffortCostMap ec;
+    foreach ( Appointment *a, m_appointments ) {
+        if ( a->resource() && a->resource()->resource() == resource ) {
+            ec += a->plannedPrDay( start, end, type );
+            break;
+        }
+    }
+    return ec;
+}
+
+Duration Schedule::plannedEffort( EffortCostCalculationType type ) const
 {
     //kDebug();
     Duration eff;
     QListIterator<Appointment*> it( m_appointments );
     while ( it.hasNext() ) {
-        eff += it.next() ->plannedEffort();
+        eff += it.next() ->plannedEffort( type );
     }
     return eff;
 }
 
-Duration Schedule::plannedEffort( const QDate &date ) const
+Duration Schedule::plannedEffort( const QDate &date, EffortCostCalculationType type ) const
 {
     //kDebug();
     Duration eff;
     QListIterator<Appointment*> it( m_appointments );
     while ( it.hasNext() ) {
-        eff += it.next() ->plannedEffort( date );
+        eff += it.next() ->plannedEffort( date, type );
     }
     return eff;
 }
 
-Duration Schedule::plannedEffortTo( const QDate &date ) const
+Duration Schedule::plannedEffortTo( const QDate &date, EffortCostCalculationType type ) const
 {
     //kDebug();
     Duration eff;
     QListIterator<Appointment*> it( m_appointments );
     while ( it.hasNext() ) {
-        eff += it.next() ->plannedEffortTo( date );
+        eff += it.next() ->plannedEffortTo( date, type );
     }
     return eff;
 }
 
-EffortCost Schedule::plannedCost() const
+EffortCost Schedule::plannedCost( EffortCostCalculationType type ) const
 {
     //kDebug();
     EffortCost c;
     QListIterator<Appointment*> it( m_appointments );
     while ( it.hasNext() ) {
-        c += it.next() ->plannedCost();
+        c += it.next() ->plannedCost( type );
     }
     return c;
 }
 
-double Schedule::plannedCost( const QDate &date ) const
+double Schedule::plannedCost( const QDate &date, EffortCostCalculationType type ) const
 {
     //kDebug();
     double c = 0;
     QListIterator<Appointment*> it( m_appointments );
     while ( it.hasNext() ) {
-        c += it.next() ->plannedCost( date );
+        c += it.next() ->plannedCost( date, type );
     }
     return c;
 }
 
-double Schedule::plannedCostTo( const QDate &date ) const
+double Schedule::plannedCostTo( const QDate &date, EffortCostCalculationType type ) const
 {
     //kDebug();
     double c = 0;
     QListIterator<Appointment*> it( m_appointments );
     while ( it.hasNext() ) {
-        c += it.next() ->plannedCostTo( date );
+        c += it.next() ->plannedCostTo( date, type );
     }
     return c;
 }
@@ -695,7 +764,7 @@ void NodeSchedule::saveXML( QDomElement &element ) const
     sch.setAttribute( "free-float", freeFloat.toString() );
 }
 
-void NodeSchedule::addAppointment( Schedule *resource, DateTime &start, DateTime &end, double load )
+void NodeSchedule::addAppointment( Schedule *resource, const DateTime &start, const DateTime &end, double load )
 {
     //kDebug();
     Appointment * a = findAppointment( resource, this, m_calculationMode );
@@ -705,8 +774,10 @@ void NodeSchedule::addAppointment( Schedule *resource, DateTime &start, DateTime
         return ;
     }
     a = new Appointment( resource, this, start, end, load );
-    Q_ASSERT ( add( a ) == true );
-    Q_ASSERT ( resource->add( a ) == true );
+    bool result = add( a );
+    Q_ASSERT ( result );
+    result = resource->add( a );
+    Q_ASSERT ( result );
     //kDebug()<<"Added interval to new"<<a;
 }
 
@@ -826,7 +897,7 @@ ResourceSchedule::~ResourceSchedule()
 }
 
 // called from the resource
-void ResourceSchedule::addAppointment( Schedule *node, DateTime &start, DateTime &end, double load )
+void ResourceSchedule::addAppointment( Schedule *node, const DateTime &start, const DateTime &end, double load )
 {
     Q_ASSERT( start < end );
     //kDebug()<<"("<<this<<")"<<node<<","<<m_calculationMode;
@@ -916,7 +987,7 @@ Duration ResourceSchedule::effort( const DateTimeInterval &interval ) const
 
 DateTimeInterval ResourceSchedule::available( const DateTimeInterval &interval ) const
 {
-    //kDebug()<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<interval.first<<","<<interval.second;
+    //const_cast<ResourceSchedule*>(this)->logDebug( QString( "Schedule available id=%1, Mode=%2: interval=%3 - %4" ).arg(m_id).arg(m_calculationMode).arg(interval.first.toString()).arg(interval.second.toString()) );
     if ( allowOverbooking() ) {
         return DateTimeInterval( interval.first, interval.second );
     }
@@ -929,26 +1000,41 @@ DateTimeInterval ResourceSchedule::available( const DateTimeInterval &interval )
         //kDebug()<<this<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<interval.first<<","<<interval.second<<" FREE";
         return DateTimeInterval( interval.first, interval.second );
     }
+    if ( interval.first >= a.endTime() || interval.second <= a.startTime() ) {
+        // free
+        return DateTimeInterval( interval.first, interval.second );
+    }
+    DateTimeInterval ci = interval;
     foreach ( const AppointmentInterval &i, a.intervals() ) {
-        if ( i.startTime() < interval.second && i.endTime() > interval.first ) {
+        //const_cast<ResourceSchedule*>(this)->logDebug( QString( "Schedule available check interval=%1 - %2" ).arg(i.startTime().toString()).arg(i.endTime().toString()) );
+        if ( i.startTime() < ci.second && i.endTime() > ci.first ) {
+            if ( ci.first >= i.startTime() && ci.second <= i.endTime() ) {
+                //busy
+                return DateTimeInterval( DateTime(), DateTime() );
+            }
             DateTime t = i.startTime();
-            if ( t > interval.first ) {
+            if ( t > ci.first ) {
                 //kDebug()<<this<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<interval.first<<","<<t<<" PART";
-                return DateTimeInterval( interval.first, t );
+                //const_cast<ResourceSchedule*>(this)->logDebug( QString( "Schedule available t>first: returns interval=%1 - %2" ).arg(ci.first.toString()).arg(t.toString()) );
+                return DateTimeInterval( ci.first, t );
             }
             t = i.endTime();
-            if ( t < interval.second ) {
-                //kDebug()<<this<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<t<<","<<interval.second<<" PART";
-                return DateTimeInterval( t, interval.second );
+            if ( t < ci.second ) {
+                ci.first = t;
+                continue;
             }
-            if ( i.startTime() <= interval.first && i.endTime() >= interval.second ) {
+            if ( i.startTime() <= ci.first && i.endTime() >= ci.second ) {
                 //kDebug()<<this<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<interval.first<<","<<interval.second<<" BUSY";
                 return DateTimeInterval( DateTime(), DateTime() );
             }
+            ci.first = qMax( ci.first, i.endTime() );
+            if ( ci.first >= ci.second ) {
+                return DateTimeInterval( DateTime(), DateTime() ); //busy
+            }
         }
     }
     //kDebug()<<this<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<interval.first<<","<<interval.second<<" FREE";
-    return DateTimeInterval( interval.first, interval.second );
+    return DateTimeInterval( ci.first, ci.second );
 }
 
 void ResourceSchedule::logError( const QString &msg, int phase )
@@ -1089,7 +1175,6 @@ bool MainSchedule::loadXML( const KoXmlElement &sch, XMLLoaderObject &status )
                     if ( e2.tagName() != "node" ) {
                         continue;
                     }
-                    kDebug()<<"node";
                     QString s = e2.attribute( "id" );
                     Node *node = status.project().findNode( s );
                     if ( node ) {
@@ -1227,12 +1312,7 @@ void MainSchedule::addCriticalPathNode( Node *node )
     m_currentCriticalPath->append( node );
 }
 
-QList<Schedule::Log> MainSchedule::logs()
-{
-    return m_log;
-}
-
-const QList<Schedule::Log> &MainSchedule::logs() const
+QList<Schedule::Log> MainSchedule::logs() const
 {
     return m_log;
 }
@@ -1243,6 +1323,9 @@ void MainSchedule::addLog( Schedule::Log &log )
         log.phase = m_log.last().phase;
     }
     m_log.append( log );
+    if ( m_manager ) {
+        m_manager->logAdded( log );
+    }
 }
 
 //static
@@ -1261,7 +1344,7 @@ QString MainSchedule::logSeverity( int severity )
 QStringList MainSchedule::logMessages() const
 {
     QStringList lst;
-    foreach ( Schedule::Log l, m_log ) {
+    foreach ( const Schedule::Log &l, m_log ) {
         lst << l.formatMsg();
     }
     return lst;
@@ -1279,6 +1362,9 @@ ScheduleManager::ScheduleManager( Project &project, const QString name )
     m_usePert( false ),
     m_recalculate( false ),
     m_schedulingDirection( false ),
+    m_scheduling( false ),
+    m_progress( 0 ),
+    m_maxprogress( 0 ),
     m_expected( 0 ),
     m_optimistic( 0 ),
     m_pessimistic( 0 )
@@ -1292,14 +1378,14 @@ ScheduleManager::~ScheduleManager()
     setParentManager( 0 );
 }
 
-void ScheduleManager::setParentManager( ScheduleManager *sm )
+void ScheduleManager::setParentManager( ScheduleManager *sm, int index )
 {
     if ( m_parent ) {
         m_parent->removeChild( this );
     }
     m_parent = sm;
     if ( sm ) {
-        sm->insertChild( this );
+        sm->insertChild( this, index );
     }
 }
 
@@ -1325,6 +1411,9 @@ void ScheduleManager::insertChild( ScheduleManager *sm, int index )
 void ScheduleManager::createSchedules()
 {
     setExpected( m_project.createSchedule( m_name, Schedule::Expected ) );
+    if ( m_recalculate ) {
+        m_calculateAll = false;
+    }
     if ( m_calculateAll ) {
         setOptimistic( m_project.createSchedule( m_name, Schedule::Optimistic ) );
         setPessimistic( m_project.createSchedule( m_name, Schedule::Pessimistic ) );
@@ -1357,8 +1446,8 @@ ScheduleManager *ScheduleManager::findManager( const QString& name ) const
 QList<ScheduleManager*> ScheduleManager::allChildren() const
 {
     QList<ScheduleManager*> lst;
-    lst << m_children;
     foreach ( ScheduleManager *sm, m_children ) {
+        lst << sm;
         lst << sm->allChildren();
     }
     return lst;
@@ -1380,6 +1469,9 @@ bool ScheduleManager::isParentOf( const ScheduleManager *sm ) const
 void ScheduleManager::setName( const QString& name )
 {
     m_name = name;
+#ifndef NDEBUG
+    setObjectName( name );
+#endif
     if ( m_expected ) {
         m_expected->setName( name );
         m_project.changed( m_expected );
@@ -1463,6 +1555,106 @@ void ScheduleManager::setSchedulingDirection( bool on )
     m_project.changed( this );
 }
 
+void ScheduleManager::setScheduling( bool on )
+{
+    m_scheduling = on;
+    if ( ! on ) {
+        m_project.setProgress( 0, this );
+    }
+    m_project.changed( this );
+}
+
+const QList<SchedulerPlugin*> ScheduleManager::schedulerPlugins() const
+{
+    return m_project.schedulerPlugins().values();
+}
+
+QString ScheduleManager::schedulerPluginId() const
+{
+    return m_schedulerPluginId;
+}
+
+void ScheduleManager::setSchedulerPluginId( const QString &id )
+{
+    m_schedulerPluginId = id;
+    m_project.changed( this );
+}
+
+SchedulerPlugin *ScheduleManager::schedulerPlugin() const
+{
+    if ( m_schedulerPluginId.isEmpty() ) {
+        // try to avoid crash
+        return m_project.schedulerPlugins().value( m_project.schedulerPlugins().keys().value( 0 ) );
+    }
+    return m_project.schedulerPlugins().value( m_schedulerPluginId );
+}
+
+QStringList ScheduleManager::schedulerPluginNames() const
+{
+    QStringList lst;
+    QMap<QString, SchedulerPlugin*>::const_iterator it = m_project.schedulerPlugins().constBegin();
+    QMap<QString, SchedulerPlugin*>::const_iterator end = m_project.schedulerPlugins().constEnd();
+    for ( ; it != end; ++it ) {
+        lst << it.value()->name();
+    }
+    return lst;
+}
+
+int ScheduleManager::schedulerPluginIndex() const
+{
+    if ( m_schedulerPluginId.isEmpty() ) {
+        return 0;
+    }
+    return m_project.schedulerPlugins().keys().indexOf( m_schedulerPluginId );
+}
+
+void ScheduleManager::setSchedulerPlugin( int index )
+{
+    if ( schedulerPlugin() ) {
+        schedulerPlugin()->stopCalculation( this ); // in case...
+    }
+
+    m_schedulerPluginId = m_project.schedulerPlugins().keys().value( index );
+    kDebug()<<index<<m_schedulerPluginId;
+    m_project.changed( this );
+}
+
+void ScheduleManager::calculateSchedule()
+{
+    m_calculationresult = CalculationRunning;
+    if ( schedulerPlugin() ) {
+        schedulerPlugin()->calculate( m_project, this );
+    }
+}
+
+void ScheduleManager::stopCalculation()
+{
+    if ( schedulerPlugin() ) {
+        schedulerPlugin()->stopCalculation( this );
+    }
+}
+
+void ScheduleManager::haltCalculation()
+{
+    if ( schedulerPlugin() ) {
+        schedulerPlugin()->haltCalculation( this );
+    }
+}
+
+void ScheduleManager::setMaxProgress( int value )
+{
+    m_maxprogress = value;
+    emit maxProgressChanged( value );
+    m_project.changed( this );
+}
+
+void ScheduleManager::setProgress( int value )
+{
+    m_progress = value;
+    emit progressChanged( value );
+    m_project.changed( this );
+}
+
 void ScheduleManager::setDeleted( bool on )
 {
     if ( m_expected ) {
@@ -1549,6 +1741,9 @@ QStringList ScheduleManager::state() const
     if ( isBaselined() ) {
         return lst << i18n( "Baselined" );
     }
+    if ( m_scheduling ) {
+        return lst << i18n( "Scheduling" );
+    }
     if ( m_expected == 0 && m_optimistic == 0 && m_pessimistic == 0 ) {
         return lst << i18n( "Not scheduled" );
     }
@@ -1591,6 +1786,30 @@ void ScheduleManager::incProgress()
     m_project.incProgress();
 }
 
+void ScheduleManager::logAdded( Schedule::Log &log )
+{
+    emit sigLogAdded( log );
+}
+
+void ScheduleManager::slotAddLog( KPlato::Schedule::Log log )
+{
+    if ( expected() ) {
+        //qDebug()<<"ScheduleManager::slotAddLog:"<<log;
+        expected()->addLog( log );
+        m_project.changed( this );
+    }
+}
+
+void ScheduleManager::slotAddLog( const QList<KPlato::Schedule::Log> &log )
+{
+    if ( expected() ) {
+        foreach ( const KPlato::Schedule::Log &l, log ) {
+            expected()->addLog( const_cast<KPlato::Schedule::Log&>( l ) );
+        }
+        m_project.changed( this );
+    }
+}
+
 bool ScheduleManager::loadXML( KoXmlElement &element, XMLLoaderObject &status )
 {
     MainSchedule *sch = 0;
@@ -1608,12 +1827,16 @@ bool ScheduleManager::loadXML( KoXmlElement &element, XMLLoaderObject &status )
         }
         return true;
     }
-    m_name = element.attribute( "name" );
-    m_usePert = (bool)(element.attribute( "distribution" ).toInt());
+    setName( element.attribute( "name" ) );
+    m_id = element.attribute( "id" );
+    m_usePert = (element.attribute( "distribution" ).toInt()) == 1;
     m_allowOverbooking = (bool)(element.attribute( "overbooking" ).toInt());
     m_checkExternalAppointments = (bool)(element.attribute( "check-external-appointments" ).toInt());
     m_schedulingDirection = (bool)(element.attribute( "scheduling-direction" ).toInt());
     m_baselined = (bool)(element.attribute( "baselined" ).toInt());
+    m_schedulerPluginId = element.attribute( "scheduler-plugin-id" );
+    m_recalculate = (bool)(element.attribute( "recalculate" ).toInt());
+    m_recalculateFrom = DateTime::fromString( element.attribute( "recalculate-from" ), status.projectSpec() );
     KoXmlNode n = element.firstChild();
     for ( ; ! n.isNull(); n = n.nextSibling() ) {
         if ( ! n.isElement() ) {
@@ -1634,7 +1857,7 @@ bool ScheduleManager::loadXML( KoXmlElement &element, XMLLoaderObject &status )
         } else if ( e.tagName() == "plan" ) {
             ScheduleManager *sm = new ScheduleManager( status.project() );
             if ( sm->loadXML( e, status ) ) {
-                sm->setParentManager( this );
+                m_project.addScheduleManager( sm, this );
             } else {
                 kError()<<"Failed to load schedule manager"<<endl;
                 delete sm;
@@ -1661,19 +1884,40 @@ MainSchedule *ScheduleManager::loadMainSchedule( KoXmlElement &element, XMLLoade
     return sch;
 }
 
+bool ScheduleManager::loadMainSchedule( MainSchedule *schedule, KoXmlElement &element, XMLLoaderObject &status ) {
+    long sid = schedule->id();
+    if ( schedule->loadXML( element, status ) ) {
+        if ( sid != schedule->id() && status.project().findSchedule( sid ) ) {
+            status.project().takeSchedule( schedule );
+        }
+        if ( ! status.project().findSchedule( schedule->id() ) ) {
+            status.project().addSchedule( schedule );
+        }
+        schedule->setNode( &(status.project()) );
+        status.project().setParentSchedule( schedule );
+        schedule->setScheduled( true );
+        return true;
+    }
+    return false;
+}
+
 void ScheduleManager::saveXML( QDomElement &element ) const
 {
     QDomElement el = element.ownerDocument().createElement( "plan" );
     element.appendChild( el );
     el.setAttribute( "name", m_name );
-    el.setAttribute( "distribution", m_usePert );
+    el.setAttribute( "id", m_id );
+    el.setAttribute( "distribution", m_usePert ? 1 : 0 );
     el.setAttribute( "overbooking", m_allowOverbooking );
     el.setAttribute( "check-external-appointments", m_checkExternalAppointments );
     el.setAttribute( "scheduling-direction", m_schedulingDirection );
     el.setAttribute( "baselined", m_baselined );
+    el.setAttribute( "scheduler-plugin-id", m_schedulerPluginId );
+    el.setAttribute( "recalculate", m_recalculate );
+    el.setAttribute( "recalculate-from", m_recalculateFrom.toString( KDateTime::ISODate ) );
     foreach ( MainSchedule *s, schedules() ) {
         //kDebug()<<m_name<<" id="<<s->id()<<(s->isDeleted()?"  Deleted":"");
-        if ( !s->isDeleted() && s->isScheduled() ) {
+        if ( !s->isDeleted() /*&& s->isScheduled()*/ ) {
             QDomElement schs = el.ownerDocument().createElement( "schedule" );
             el.appendChild( schs );
             s->saveXML( schs );
@@ -1691,7 +1935,8 @@ void ScheduleManager::saveWorkPackageXML( QDomElement &element, const Node &node
     QDomElement el = element.ownerDocument().createElement( "plan" );
     element.appendChild( el );
     el.setAttribute( "name", m_name );
-    el.setAttribute( "distribution", m_usePert );
+    el.setAttribute( "id", m_id );
+    el.setAttribute( "distribution", m_usePert ? 1 : 0 );
     el.setAttribute( "overbooking", m_allowOverbooking );
     el.setAttribute( "check-external-appointments", m_checkExternalAppointments );
     el.setAttribute( "scheduling-direction", m_schedulingDirection );
@@ -1833,3 +2078,10 @@ void MainSchedule::printDebug( const QString& _indent )
 
 } //namespace KPlato
 
+QDebug &operator<<( QDebug &dbg, const KPlato::Schedule::Log &log )
+{
+    dbg.nospace()<<"Schedule::Log:"<<log.formatMsg();
+    return dbg.space();
+}
+
+#include "kptschedule.moc"
