@@ -47,6 +47,7 @@ class Project;
 class Task;
 class ScheduleManager;
 class XMLLoaderObject;
+class SchedulerPlugin;
 
 /**
  * The Schedule class holds data calculated during project
@@ -96,6 +97,12 @@ public:
     
     virtual bool isBaselined() const;
     virtual bool usePert() const;
+    
+    enum OBState { OBS_Parent, OBS_Allow, OBS_Deny };
+    /// Sets wether overbooking resources is allowed locally for this schedule
+    /// If @p state is OBS_Parent, the parent is checked when allowOverbooking() is called
+    virtual void setAllowOverbookingState( OBState state );
+    OBState allowOverbookingState() const;
     virtual bool allowOverbooking() const;
     virtual bool checkExternalAppointments() const;
 
@@ -116,54 +123,60 @@ public:
     /// Return calculation mode
     int calculationMode() const { return m_calculationMode; }
     /// Return the list of appointments
-    QList<Appointment*> &appointments() { return m_appointments; }
+    QList<Appointment*> appointments() const { return m_appointments; }
     /// Return the list of appointments
     /// @param which specifies which list is returned
-    QList<Appointment*> &appointments( int which);
+    QList<Appointment*> appointments(int which) const;
     /// Adds appointment to this schedule only
     virtual bool add( Appointment *appointment );
     /// Adds appointment to both this resource schedule and node schedule
-    virtual void addAppointment( Schedule * /*other*/, DateTime & /*start*/, DateTime & /*end*/, double /*load*/ = 100 ) {}
+    virtual void addAppointment( Schedule * /*other*/, const DateTime & /*start*/, const DateTime & /*end*/, double /*load*/ = 100 ) {}
     /// Removes appointment without deleting it.
     virtual void takeAppointment( Appointment *appointment, int type = Scheduling );
     Appointment *findAppointment( Schedule *resource, Schedule *node, int type = Scheduling );
     /// Attach the appointment to appropriate list (appointment->calculationMode() specifies list)
     bool attatch( Appointment *appointment );
     
+    DateTime appointmentStartTime() const;
+    DateTime appointmentEndTime() const;
+
     virtual Appointment appointmentIntervals( int which = Scheduling ) const;
 
     virtual bool isOverbooked() const { return false; }
     virtual bool isOverbooked( const KDateTime & /*start*/, const KDateTime & /*end*/ ) const { return false; }
     virtual QStringList overbookedResources() const;
+    /// Returns the first booked interval to @p node that intersects @p interval (limited to @p interval)
+    virtual DateTimeInterval firstBookedInterval( const DateTimeInterval &interval, const Schedule *node ) const;
 
     /// Return the resources that has appointments to this schedule
     virtual QList<Resource*> resources() const;
     /// Return the resource names that has appointments to this schedule
     virtual QStringList resourceNameList() const;
     
-    virtual EffortCostMap bcwsPrDay() const;
-    virtual EffortCostMap plannedEffortCostPrDay( const QDate &start, const QDate &end ) const;
-
+    virtual EffortCostMap bcwsPrDay( EffortCostCalculationType type = ECCT_All ) const;
+    virtual EffortCostMap plannedEffortCostPrDay( const QDate &start, const QDate &end, EffortCostCalculationType type = ECCT_All ) const;
+    virtual EffortCostMap plannedEffortCostPrDay( const Resource *resource, const QDate &start, const QDate &end, EffortCostCalculationType type = ECCT_All ) const;
+    
     /// Returns the total planned effort for this schedule
-    virtual Duration plannedEffort() const;
+    virtual Duration plannedEffort( EffortCostCalculationType type = ECCT_All) const;
     /// Returns the total planned effort for this schedule on date
-    virtual Duration plannedEffort( const QDate &date ) const;
+    virtual Duration plannedEffort( const QDate &date, EffortCostCalculationType type = ECCT_All ) const;
     /// Returns the planned effort up to and including date
-    virtual Duration plannedEffortTo( const QDate &date ) const;
+    virtual Duration plannedEffortTo( const QDate &date, EffortCostCalculationType type = ECCT_All ) const;
 
     /**
      * Planned cost is the sum total of all resources and other costs
      * planned for this node.
      */
-    virtual EffortCost plannedCost() const;
+    virtual EffortCost plannedCost( EffortCostCalculationType type = ECCT_All ) const;
 
     /// Planned cost on date
-    virtual double plannedCost( const QDate &date ) const;
+    virtual double plannedCost( const QDate &date, EffortCostCalculationType type = ECCT_All ) const;
     /**
      * Planned cost from start of activity up to and including date
      * is the sum of all resource costs and other costs planned for this schedule.
      */
-    virtual double plannedCostTo( const QDate &date ) const;
+    virtual double plannedCostTo( const QDate &date, EffortCostCalculationType type = ECCT_All ) const;
     
     virtual double normalRatePrHour() const { return 0.0; }
 
@@ -229,6 +242,7 @@ protected:
     long m_id;
     bool m_deleted;
     Schedule *m_parent;
+    OBState m_obstate;
 
     int m_calculationMode;
     QList<Appointment*> m_appointments;
@@ -286,7 +300,7 @@ protected:
     bool resourceOverbooked;
     /// Set if the requested resource is not available
     bool resourceNotAvailable;
-    /// Set if the task cannot be scheduled to fullfill all the constraints
+    /// Set if the task cannot be scheduled to fullfil all the constraints
     bool schedulingError;
     /// Set if the node has not been scheduled
     bool notScheduled;
@@ -325,7 +339,7 @@ public:
     virtual void saveXML( QDomElement &element ) const;
 
     // tasks------------>
-    virtual void addAppointment( Schedule *resource, DateTime &start, DateTime &end, double load = 100 );
+    virtual void addAppointment( Schedule *resource, const DateTime &start, const DateTime &end, double load = 100 );
     virtual void takeAppointment( Appointment *appointment, int type = Schedule::Scheduling );
 
     virtual Node *node() const { return m_node; }
@@ -367,7 +381,7 @@ public:
 
     virtual bool isDeleted() const
     { return m_parent == 0 ? true : m_parent->isDeleted(); }
-    virtual void addAppointment( Schedule *node, DateTime &start, DateTime &end, double load = 100 );
+    virtual void addAppointment( Schedule *node, const DateTime &start, const DateTime &end, double load = 100 );
     virtual void takeAppointment( Appointment *appointment, int type = Scheduling );
 
     virtual bool isOverbooked() const;
@@ -442,7 +456,9 @@ public:
     QList<Node*> hardConstraints() const { return m_hardconstraints; }
     virtual void insertSoftConstraint( Node *node ) { m_softconstraints.append( node ); }
     QList<Node*> softConstraints() const { return m_softconstraints; }
+    QList<Node*> forwardNodes() const { return m_forwardnodes; }
     virtual void insertForwardNode( Node *node ) { m_forwardnodes.append( node ); }
+    QList<Node*> backwardNodes() const { return m_backwardnodes; }
     virtual void insertBackwardNode( Node *node ) { m_backwardnodes.append( node ); }
     virtual void insertStartNode( Node *node ) { m_startNodes.append( node ); }
     QList<Node*> startNodes() const { return m_startNodes; }
@@ -461,14 +477,16 @@ public:
     }
     void addCriticalPathNode( Node *node );
     
-    QList<Schedule::Log> logs();
-    const QList<Schedule::Log> &logs() const;
+    QList<Schedule::Log> logs() const;
+    void setLog( const QList<Schedule::Log> &log ) { m_log = log; }
     virtual void addLog( Schedule::Log &log );
     virtual void clearLogs() { m_log.clear(); m_logPhase.clear(); }
     
     void setPhaseName( int phase, const QString &name ) { m_logPhase[ phase ] = name; }
     QString logPhase( int phase ) const { return m_logPhase.value( phase ); }
     static QString logSeverity( int severity );
+    QMap<int, QString> phaseNames() const { return m_logPhase; }
+    void setPhaseNames( const QMap<int, QString> &pn ) { m_logPhase = pn; }
     
     virtual void incProgress();
 
@@ -509,25 +527,31 @@ public:
  * Expected-, Optimistic- and Pessimistic schedules.
  * A ScheduleManager can also have child manager(s).
  */
-class KPLATOKERNEL_EXPORT ScheduleManager
+class KPLATOKERNEL_EXPORT ScheduleManager : public QObject
 {
+    Q_OBJECT
 public:
+    enum CalculationResult { CalculationRunning = 0, CalculationDone, CalculationStopped, CalculationCanceled, CalculationError };
+
     explicit ScheduleManager( Project &project, const QString name = QString() );
     ~ScheduleManager();
     
     void setName( const QString& name );
     QString name() const { return m_name; }
 
+    void setManagerId( const QString &id ) { m_id = id; }
+    QString managerId() const { return m_id; }
+
     Project &project() const { return m_project; }
     
-    void setParentManager( ScheduleManager *sm );
+    void setParentManager( ScheduleManager *sm, int index = -1 );
     ScheduleManager *parentManager() const { return m_parent; }
     
-    long id() const { return m_expected == 0 ? NOTSCHEDULED : m_expected->id(); }
+    long scheduleId() const { return m_expected == 0 ? NOTSCHEDULED : m_expected->id(); }
     
     int removeChild( const ScheduleManager *sm );
     void insertChild( ScheduleManager *sm, int index = -1 );
-    const QList<ScheduleManager*> &children() const { return m_children; }
+    QList<ScheduleManager*> children() const { return m_children; }
     int childCount() const { return m_children.count(); }
     ScheduleManager *childAt( int index ) const { return m_children.value( index ); }
     /// Return list of all child managers (also childrens children)
@@ -536,11 +560,15 @@ public:
     bool isParentOf( const ScheduleManager *sm ) const;
     ScheduleManager *findManager( const QString &name ) const;
     
+    /// This sub-schedule will be re-calculated based on the parents completion data
     bool recalculate() const { return m_recalculate; }
+    /// Set re-calculate to @p on.
     void setRecalculate( bool on ) { m_recalculate = on; }
+    /// The datetime this schedule will be calculated from
     DateTime recalculateFrom() const { return m_recalculateFrom; }
+    /// Set the datetime this schedule will be calculated from to @p dt
     void setRecalculateFrom( const DateTime &dt ) { m_recalculateFrom = dt; }
-    long parentScheduleId() const { return m_parent == 0 ? NOTSCHEDULED : m_parent->id(); }
+    long parentScheduleId() const { return m_parent == 0 ? NOTSCHEDULED : m_parent->scheduleId(); }
     void createSchedules();
     
     void setDeleted( bool on );
@@ -576,6 +604,9 @@ public:
     void setSchedulingDirection( bool on );
     bool schedulingDirection() const { return m_schedulingDirection; }
 
+    void setScheduling( bool on );
+    bool scheduling() const { return m_scheduling; }
+
     QList<MainSchedule*> schedules() const;
     int numSchedules() const;
     int indexOf( const MainSchedule *sch ) const;
@@ -588,15 +619,64 @@ public:
             
     void scheduleChanged( MainSchedule *sch );
     
-    void incProgress();
+    const QList<SchedulerPlugin*> schedulerPlugins() const;
+    QString schedulerPluginId() const;
+    void setSchedulerPluginId( const QString &id );
+    SchedulerPlugin *schedulerPlugin() const;
+    QStringList schedulerPluginNames() const;
+    int schedulerPluginIndex() const;
+    void setSchedulerPlugin( int index );
 
-protected:
+    /// Stop calculation. Use result if possible.
+    void stopCalculation();
+    /// Terminate calculation. Forget any results.
+    void haltCalculation();
+    void calculateSchedule();
+    int calculationResult() const { return m_calculationresult; }
+    void setCalculationResult( int r ) { m_calculationresult = r; }
+
+    /// Increments progress in the project
+    void incProgress();
+    /// Returns current progress
+    int progress() const { return m_progress; }
+    /// Returns maximum progress value
+    int maxProgress() const { return m_maxprogress; }
+
+    /// Log added by MainSchedule
+    /// Emits sigLogAdded() to enable syncronization between schedules
+    void logAdded( Schedule::Log &log );
+
+    /// Create and load a MainSchedule
     MainSchedule *loadMainSchedule( KoXmlElement &element, XMLLoaderObject &status );
-    
+
+    /// Load an existing MainSchedule
+    bool loadMainSchedule( MainSchedule *schedule, KoXmlElement &element, XMLLoaderObject &status );
+
+public slots:
+    /// Set maximum progress. Emits signal maxProgressChanged
+    void setMaxProgress( int value );
+    /// Set progress. Emits signal progressChanged
+    void setProgress( int value );
+
+    /// Add @p log to expected()
+    /// Usually connected to sigLogAdded() (other manager) to syncronize this with the other manager
+    void slotAddLog( KPlato::Schedule::Log log );
+
+    void slotAddLog( const QList<KPlato::Schedule::Log> &log );
+
+signals:
+    void maxProgressChanged( int );
+    void progressChanged( int );
+
+    /// Emitted by logAdded() (but not by slotAddLog())
+    /// Usually connected to slotAddLog() (other manager) to syncronize the other manager with this manager
+    void sigLogAdded( Schedule::Log log );
+
 protected:
     Project &m_project;
     ScheduleManager *m_parent;
     QString m_name;
+    QString m_id;
     bool m_baselined;
     bool m_allowOverbooking;
     bool m_checkExternalAppointments;
@@ -605,14 +685,23 @@ protected:
     bool m_recalculate;
     DateTime m_recalculateFrom;
     bool m_schedulingDirection;
+    bool m_scheduling;
+    int m_progress;
+    int m_maxprogress;
     MainSchedule *m_expected;
     MainSchedule *m_optimistic;
     MainSchedule *m_pessimistic;
     QList<MainSchedule*> m_schedules;
     QList<ScheduleManager*> m_children;
+
+    QString m_schedulerPluginId;
+    
+    int m_calculationresult;
 };
 
 
 } //namespace KPlato
 
+QDebug &operator<<( QDebug &dbg, const KPlato::Schedule::Log &log );
+
 #endif
