@@ -32,6 +32,9 @@ private slots:
     void testSingleDay();
     void testWeekdays();
     void testCalendarWithParent();
+    void testTimezone();
+    void workIntervals();
+    void workIntervalsFullDays();
 };
 
 } //namespace KPlato
