@@ -20,6 +20,7 @@
 #include "kptcalendarmodel.h"
 
 #include "kptglobal.h"
+#include "kptcommonstrings.h"
 #include "kptcommand.h"
 #include "kptitemmodelbase.h"
 #include "kptcalendar.h"
@@ -129,20 +130,20 @@ void CalendarItemModel::setProject( Project *project )
 {
     if ( m_project ) {
         disconnect( m_project , SIGNAL( calendarChanged( Calendar* ) ), this, SLOT( slotCalendarChanged( Calendar* ) ) );
-        
+
         disconnect( m_project, SIGNAL( calendarAdded( const Calendar* ) ), this, SLOT( slotCalendarInserted( const Calendar* ) ) );
         disconnect( m_project, SIGNAL( calendarToBeAdded( const Calendar*, int ) ), this, SLOT( slotCalendarToBeInserted( const Calendar*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( calendarRemoved( const Calendar* ) ), this, SLOT( slotCalendarRemoved( const Calendar* ) ) );
         disconnect( m_project, SIGNAL( calendarToBeRemoved( const Calendar* ) ), this, SLOT( slotCalendarToBeRemoved( const Calendar* ) ) );
     }
     m_project = project;
     if ( project ) {
         connect( m_project, SIGNAL( calendarChanged( Calendar* ) ), this, SLOT( slotCalendarChanged( Calendar* ) ) );
-        
+
         connect( m_project, SIGNAL( calendarAdded( const Calendar* ) ), this, SLOT( slotCalendarInserted( const Calendar* ) ) );
         connect( m_project, SIGNAL( calendarToBeAdded( const Calendar*, int ) ), this, SLOT( slotCalendarToBeInserted( const Calendar*, int ) ) );
-        
+
         connect( m_project, SIGNAL( calendarRemoved( const Calendar* ) ), this, SLOT( slotCalendarRemoved( const Calendar* ) ) );
         connect( m_project, SIGNAL( calendarToBeRemoved( const Calendar* ) ), this, SLOT( slotCalendarToBeRemoved( const Calendar* ) ) );
     }
@@ -165,14 +166,14 @@ Qt::ItemFlags CalendarItemModel::flags( const QModelIndex &index ) const
             case 0:/*Name*/
                 flags |= ( Qt::ItemIsEditable | Qt::ItemIsUserCheckable );
                 break;
-            case 1: 
+            case 1:
                 if ( parent( index ).isValid() ) {
                     flags &= ~Qt::ItemIsEditable;
                 } else {
                     flags |= Qt::ItemIsEditable;
                 }
                 break;
-            default: 
+            default:
                 flags |= Qt::ItemIsEditable;
                 break;
         }
@@ -285,7 +286,7 @@ bool CalendarItemModel::setName( Calendar *a, const QVariant &value, int role )
     switch ( role ) {
         case Qt::EditRole:
             if ( value.toString() != a->name() ) {
-                emit executeCommand( new CalendarModifyNameCmd( a, value.toString(), i18n( "Modify Calendar Name" ) ) );
+                emit executeCommand( new CalendarModifyNameCmd( a, value.toString(), i18n( "Modify calendar name" ) ) );
                 return true;
             }
             break;
@@ -357,7 +358,7 @@ bool CalendarItemModel::setTimeZone( Calendar *a, const QVariant &value, int rol
             if ( !tz.isValid() ) {
                 return false;
             }
-            emit executeCommand( new CalendarModifyTimeZoneCmd( a, tz, i18n( "Modify Calendar Timezone" ) ) );
+            emit executeCommand( new CalendarModifyTimeZoneCmd( a, tz, i18n( "Modify calendar timezone" ) ) );
             return true;
         }
     }
@@ -375,7 +376,7 @@ QVariant CalendarItemModel::data( const QModelIndex &index, int role ) const
         case 0: result = name( a, role ); break;
         case 1: result = timeZone( a, role ); break;
         default:
-            kDebug()<<"data: invalid display value column"<<index.column();;
+            kDebug()<<"data: invalid display value column"<<index.column();
             return QVariant();
     }
     if ( result.isValid() ) {
@@ -482,9 +483,9 @@ QMimeData *CalendarItemModel::mimeData( const QModelIndexList & indexes ) const
     return m;
 }
 
-bool CalendarItemModel::dropMimeData( const QMimeData *data, Qt::DropAction action, int /*row*/, int /*column*/, const QModelIndex &parent )
+bool CalendarItemModel::dropMimeData( const QMimeData *data, Qt::DropAction action, int row, int /*column*/, const QModelIndex &parent )
 {
-    kDebug()<<action;
+    kDebug()<<action<<row;
     if (action == Qt::IgnoreAction) {
         return true;
     }
@@ -493,7 +494,7 @@ bool CalendarItemModel::dropMimeData( const QMimeData *data, Qt::DropAction acti
     }
     if ( action == Qt::MoveAction ) {
         kDebug()<<"MoveAction";
-        
+
         QByteArray encodedData = data->data( "application/x-vnd.kde.kplato.calendarid.internal" );
         QDataStream stream(&encodedData, QIODevice::ReadOnly);
         Calendar *par = 0;
@@ -504,8 +505,11 @@ bool CalendarItemModel::dropMimeData( const QMimeData *data, Qt::DropAction acti
         QList<Calendar*> lst = calendarList( stream );
         foreach ( Calendar *c, lst ) {
             if ( c->parentCal() != par ) {
-                if ( cmd == 0 ) cmd = new MacroCommand( i18n( "Re-parent Calendar" ) );
+                if ( cmd == 0 ) cmd = new MacroCommand( i18n( "Re-parent calendar" ) );
                 cmd->addCommand( new CalendarModifyParentCmd( m_project, c, par ) );
+            } else {
+                if ( cmd == 0 ) cmd = new MacroCommand( i18n( "Move calendar" ) );
+                cmd->addCommand( new CalendarMoveCmd( m_project, c, row, par ) );
             }
         }
         if ( cmd ) {
@@ -537,6 +541,9 @@ bool CalendarItemModel::dropAllowed( Calendar *on, const QMimeData *data )
     if ( !data->hasFormat("application/x-vnd.kde.kplato.calendarid.internal") ) {
         return false;
     }
+    if ( on == 0 && ! ( flags( QModelIndex() ) & (int)Qt::ItemIsDropEnabled ) ) {
+        return false;
+    }
     QByteArray encodedData = data->data( "application/x-vnd.kde.kplato.calendarid.internal" );
     QDataStream stream(&encodedData, QIODevice::ReadOnly);
     QList<Calendar*> lst = calendarList( stream );
@@ -544,7 +551,7 @@ bool CalendarItemModel::dropAllowed( Calendar *on, const QMimeData *data )
         if ( (flags( index( c ) ) & (int)Qt::ItemIsDropEnabled) == 0 ) {
             return false;
         }
-        if ( on == c->parentCal() ) {
+        if ( on != 0 && on == c->parentCal() ) {
             return false;
         }
         if ( on != 0 && ( on == c || on->isChildOf( c ) ) ) {
@@ -554,10 +561,10 @@ bool CalendarItemModel::dropAllowed( Calendar *on, const QMimeData *data )
     return true;
 }
 
-QModelIndex CalendarItemModel::insertCalendar ( Calendar *calendar, Calendar *parent )
+QModelIndex CalendarItemModel::insertCalendar ( Calendar *calendar, int pos, Calendar *parent )
 {
-    //kDebug();
-    emit executeCommand( new CalendarAddCmd( m_project, calendar, parent, i18n( "Add Calendar" ) ) );
+    //kDebug()<<calendar<<pos<<parent;
+    emit executeCommand( new CalendarAddCmd( m_project, calendar, pos, parent, i18n( "Add calendar" ) ) );
     int row = -1;
     if ( parent ) {
         row = parent->indexOf( calendar );
@@ -565,7 +572,7 @@ QModelIndex CalendarItemModel::insertCalendar ( Calendar *calendar, Calendar *pa
         row = m_project->indexOf( calendar );
     }
     if ( row != -1 ) {
-        //kDebug()<<"Inserted:"<<calendar->name();
+        //kDebug()<<"Inserted:"<<calendar->name()<<"row="<<row;
         return createIndex( row, 0, calendar );
     }
     return QModelIndex();
@@ -580,7 +587,7 @@ void CalendarItemModel::removeCalendar( Calendar *calendar )
     if ( calendar == 0 ) {
         return;
     }
-    emit executeCommand( new CalendarRemoveCmd( m_project, calendar, i18n( "Delete Calendar" ) ) );
+    emit executeCommand( new CalendarRemoveCmd( m_project, calendar, i18n( "Delete calendar" ) ) );
 }
 
 
@@ -596,6 +603,7 @@ CalendarDayItemModel::~CalendarDayItemModel()
 
 void CalendarDayItemModel::slotWorkIntervalAdded( CalendarDay *day, TimeInterval *ti )
 {
+    Q_UNUSED(ti);
     //kDebug()<<day<<","<<ti;
     int c = m_calendar->indexOfWeekday( day );
     if ( c == -1 ) {
@@ -606,6 +614,7 @@ void CalendarDayItemModel::slotWorkIntervalAdded( CalendarDay *day, TimeInterval
 
 void CalendarDayItemModel::slotWorkIntervalRemoved( CalendarDay *day, TimeInterval *ti )
 {
+    Q_UNUSED(ti);
     int c = m_calendar->indexOfWeekday( day );
     if ( c == -1 ) {
         return;
@@ -625,6 +634,7 @@ void CalendarDayItemModel::slotDayChanged( CalendarDay *day )
 
 void CalendarDayItemModel::slotTimeIntervalChanged( TimeInterval *ti )
 {
+    Q_UNUSED(ti);
 /*    CalendarDay *d = parentDay( ti );
     if ( d == 0 ) {
         return;
@@ -665,6 +675,7 @@ Qt::ItemFlags CalendarDayItemModel::flags( const QModelIndex &index ) const
 
 QModelIndex CalendarDayItemModel::parent( const QModelIndex &index ) const
 {
+    Q_UNUSED(index);
     return QModelIndex();
 }
 
@@ -775,7 +786,7 @@ bool CalendarDayItemModel::setDayState( CalendarDay *d, const QVariant &value, i
     switch ( role ) {
         case Qt::EditRole:
             int v = value.toInt();
-            emit executeCommand( new CalendarModifyStateCmd( m_calendar, d, static_cast<CalendarDay::State>( v ), i18n( "Modify Calendar State" ) ) );
+            emit executeCommand( new CalendarModifyStateCmd( m_calendar, d, static_cast<CalendarDay::State>( v ), i18n( "Modify calendar state" ) ) );
             return true;
     }
     return false;
@@ -795,7 +806,7 @@ QVariant CalendarDayItemModel::workDuration( const CalendarDay *day, int role )
             if ( day->state() == CalendarDay::Working ) {
                 KLocale *l = KGlobal::locale();
                 QStringList tip;
-                foreach ( TimeInterval *i, day->workingIntervals() ) {
+                foreach ( TimeInterval *i, day->timeIntervals() ) {
                     tip <<  i18nc( "1=time 2=The number of hours of work duration (non integer)", "%1, %2 hours", l->formatTime( i->startTime() ), l->formatNumber( i->hours() ) );
                 }
                 return tip.join( "\n" );
@@ -820,14 +831,50 @@ QVariant CalendarDayItemModel::data( const QModelIndex &index, int role ) const
         return result;
     }
     CalendarDay *d = day( index );
-    if ( d ) {
-        switch ( d->state() ) {
-            case CalendarDay::Working:
-                result = workDuration( d, role );
-                break;
-            default:
-                result = dayState( d, role );
-                break;
+    if ( d == 0 ) {
+        return QVariant();
+    }
+    switch ( role ) {
+        case Qt::DisplayRole: {
+            switch ( d->state() ) {
+                case CalendarDay::Working:
+                    result = workDuration( d, role );
+                    break;
+                case CalendarDay::NonWorking:
+                    result = dayState( d, role );
+                    break;
+                default: {
+                    // Return parent value (if any)
+                    for ( Calendar *c = m_calendar->parentCal(); c != 0; c = c->parentCal() ) {
+                        d = c->weekday( index.column() + 1 );
+                        Q_ASSERT( d );
+                        if ( d->state() == CalendarDay::Working ) {
+                            return workDuration( d, role );
+                        }
+                        if ( d->state() == CalendarDay::NonWorking ) {
+                           return  dayState( d, role );
+                        }
+                    }
+                    break;
+                }
+            }
+            break;
+        }
+        case Qt::FontRole: {
+            if ( d->state() != CalendarDay::Undefined ) {
+                return QVariant();
+            }
+            // If defined in parent, return italic
+            for ( Calendar *c = m_calendar->parentCal(); c != 0; c = c->parentCal() ) {
+                d = c->weekday( index.column() + 1 );
+                Q_ASSERT( d );
+                if ( d->state() != CalendarDay::Undefined ) {
+                    QFont f;
+                    f.setItalic( true );
+                    return f;
+                }
+            }
+            break;
         }
     }
     if ( result.isValid() ) {
@@ -882,6 +929,7 @@ CalendarDay *CalendarDayItemModel::day( const QModelIndex &index ) const
 
 QAbstractItemDelegate *CalendarDayItemModel::createDelegate( int column, QWidget *parent ) const
 {
+    Q_UNUSED(parent);
     switch ( column ) {
         default: return 0;
     }
@@ -911,6 +959,47 @@ void DateTableDataModel::setCalendar( Calendar *calendar )
     emit reset();
 }
 
+QVariant DateTableDataModel::data( const Calendar &cal, const QDate &date, int role ) const
+{
+    switch ( role ) {
+        case Qt::DisplayRole: {
+            CalendarDay *day = cal.findDay( date );
+            if ( day == 0 || day->state() == CalendarDay::Undefined ) {
+                if ( cal.parentCal() ) {
+                    return data( *( cal.parentCal() ), date, role );
+                }
+                return "";
+            }
+            if ( day->state() == CalendarDay::NonWorking ) {
+                return i18nc( "NonWorking", "NW" );
+            }
+            double v;
+            v = day->workDuration().toDouble( Duration::Unit_h );
+            return KGlobal::locale()->formatNumber( v, 1 );
+        }
+        case Qt::TextAlignmentRole:
+            return (uint)( Qt::AlignHCenter | Qt::AlignBottom );
+        case Qt::FontRole: {
+            CalendarDay *day = cal.findDay( date );
+            if ( day && day->state() != CalendarDay::Undefined ) {
+                if ( &cal != m_calendar ) {
+                    QFont f;
+                    f.setItalic( true );
+                    return f;
+                }
+                return QVariant();
+            }
+            if ( cal.parentCal() ) {
+                return data( *( cal.parentCal() ), date, role );
+            }
+            break;
+        }
+        default:
+            break;
+    }
+    return QVariant();
+}
+
 QVariant DateTableDataModel::data( const QDate &date, int role, int dataType ) const
 {
     //kDebug()<<date<<role<<dataType;
@@ -927,7 +1016,7 @@ QVariant DateTableDataModel::data( const QDate &date, int role, int dataType ) c
         }
         KLocale *l = KGlobal::locale();
         QStringList tip;
-        foreach ( TimeInterval *i, day->workingIntervals() ) {
+        foreach ( TimeInterval *i, day->timeIntervals() ) {
                     tip <<  i18nc( "1=time 2=The number of hours of work duration (non integer)", "%1, %2 hours", l->formatTime( i->startTime() ), l->formatNumber( i->hours() ) );
         }
         return tip.join( "\n" );
@@ -951,28 +1040,10 @@ QVariant DateTableDataModel::data( const QDate &date, int role, int dataType ) c
             break;
         }
         case 0: {
-            switch ( role ) {
-                case Qt::DisplayRole: {
-                    if ( m_calendar == 0 ) {
-                        return "";
-                    }
-                    CalendarDay *day = m_calendar->findDay( date );
-                    if ( day == 0 || day->state() == CalendarDay::Undefined ) {
-                        return "";
-                    }
-                    if ( day->state() == CalendarDay::NonWorking ) {
-                        return i18nc( "NonWorking", "NW" );
-                    }
-                    double v;
-                    v = day->workDuration().toDouble( Duration::Unit_h );
-                    return KGlobal::locale()->formatNumber( v, 1 );
-                }
-                case Qt::TextAlignmentRole:
-                    return (uint)( Qt::AlignHCenter | Qt::AlignBottom );
-                default:
-                    break;
+            if ( m_calendar == 0 ) {
+                return "";
             }
-            break;
+            return data( *m_calendar, date, role );
         }
         default:
             break;
@@ -982,11 +1053,15 @@ QVariant DateTableDataModel::data( const QDate &date, int role, int dataType ) c
 
 QVariant DateTableDataModel::weekDayData( int day, int role ) const
 {
+    Q_UNUSED(day);
+    Q_UNUSED(role);
     return QVariant();
 }
 
 QVariant DateTableDataModel::weekNumberData( int week, int role ) const
 {
+    Q_UNUSED(week);
+    Q_UNUSED(role);
     return QVariant();
 }
 
@@ -1008,7 +1083,7 @@ QRectF DateTableDateDelegate::paint( QPainter *painter, const StyleOptionViewIte
         return r;
     }
     painter->save();
-    const KCalendarSystem * calendar = KGlobal::locale()->calendar();
+    //const KCalendarSystem * calendar = KGlobal::locale()->calendar();
 
     painter->translate( r.width(), 0.0 );
     QRectF rect( 1, 1, option.rectF.right() - r.width(), option.rectF.bottom() );
@@ -1016,7 +1091,18 @@ QRectF DateTableDateDelegate::paint( QPainter *painter, const StyleOptionViewIte
 
     QString text = model->data( date, Qt::DisplayRole, 0 ).toString();
     int align = model->data( date, Qt::TextAlignmentRole, 0 ).toInt();
-    painter->setPen( option.palette.color( QPalette::Text ) );
+    QFont f = option.font;
+    QVariant v = model->data( date, Qt::FontRole, 0 );
+    if ( v.isValid() ) {
+        f = v.value<QFont>();
+    }
+    painter->setFont( f );
+
+    if ( option.state & QStyle::State_Selected ) {
+        painter->setPen( option.palette.highlightedText().color() );
+    } else {
+        painter->setPen( option.palette.color( QPalette::Text ) );
+    }
     painter->drawText(rect, align, text, &r);
 
     painter->restore();
