@@ -20,16 +20,23 @@
 #include "kptitemmodelbase.h"
 
 #include "kptproject.h"
+#include "kptschedule.h"
 #include "kptdurationspinbox.h"
+#include "kptresourcemodel.h"
+#include "kptresourceallocationmodel.h"
 
 #include <QApplication>
 #include <QComboBox>
 #include <QKeyEvent>
 #include <QModelIndex>
+#include <QItemSelection>
 #include <QStyleOptionViewItem>
 #include <QTimeEdit>
 #include <QPainter>
 #include <QToolTip>
+#include <QTreeView>
+#include <QHeaderView>
+#include <QStylePainter>
 
 #include <KComboBox>
 #include <klineedit.h>
@@ -107,26 +114,33 @@ void ProgressBarDelegate::paint( QPainter *painter, const QStyleOptionViewItem &
     style->drawPrimitive( QStyle::PE_PanelItemViewItem, &opt, painter );
 
     if ( !( opt.state & QStyle::State_Editing ) ) {
-        QStyleOptionProgressBar pbOption;
-        pbOption.QStyleOption::operator=( option );
-        initStyleOptionProgressBar( &pbOption, index );
+        bool ok = false;
+        (void) index.data().toInt(&ok);
+        if ( ok ) {
+            QStyleOptionProgressBar pbOption;
+            pbOption.QStyleOption::operator=( option );
+            initStyleOptionProgressBar( &pbOption, index );
 
-        style->drawControl( QStyle::CE_ProgressBar, &pbOption, painter );
-        // Draw focus, copied from qt
-        if (opt.state & QStyle::State_HasFocus) {
-            painter->save();
-            QStyleOptionFocusRect o;
-            o.QStyleOption::operator=( opt );
-            o.rect = style->subElementRect( QStyle::SE_ItemViewItemFocusRect, &opt, opt.widget );
-            o.state |= QStyle::State_KeyboardFocusChange;
-            o.state |= QStyle::State_Item;
-            QPalette::ColorGroup cg = ( opt.state & QStyle::State_Enabled )
-                            ? QPalette::Normal : QPalette::Disabled;
-            o.backgroundColor = opt.palette.color( cg, ( opt.state & QStyle::State_Selected )
-                                            ? QPalette::Highlight : QPalette::Window );
-            style->drawPrimitive( QStyle::PE_FrameFocusRect, &o, painter, opt.widget );
-            //kDebug()<<"Focus"<<o.rect<<opt.rect<<pbOption.rect;
-            painter->restore();
+            style->drawControl( QStyle::CE_ProgressBar, &pbOption, painter );
+            // Draw focus, copied from qt
+            if (opt.state & QStyle::State_HasFocus) {
+                painter->save();
+                QStyleOptionFocusRect o;
+                o.QStyleOption::operator=( opt );
+                o.rect = style->subElementRect( QStyle::SE_ItemViewItemFocusRect, &opt, opt.widget );
+                o.state |= QStyle::State_KeyboardFocusChange;
+                o.state |= QStyle::State_Item;
+                QPalette::ColorGroup cg = ( opt.state & QStyle::State_Enabled )
+                                ? QPalette::Normal : QPalette::Disabled;
+                o.backgroundColor = opt.palette.color( cg, ( opt.state & QStyle::State_Selected )
+                                                ? QPalette::Highlight : QPalette::Window );
+                style->drawPrimitive( QStyle::PE_FrameFocusRect, &o, painter, opt.widget );
+                //kDebug()<<"Focus"<<o.rect<<opt.rect<<pbOption.rect;
+                painter->restore();
+            }
+        } else {
+            EnumDelegate del;
+            del.paint( painter, option, index );
         }
     }
 }
@@ -148,10 +162,11 @@ QSize ProgressBarDelegate::sizeHint( const QStyleOptionViewItem &option, const Q
 void ProgressBarDelegate::initStyleOptionProgressBar( QStyleOptionProgressBar *option, const QModelIndex &index ) const
 {
     option->rect.adjust( 0, 1, 0, -1 );
-    option->maximum = 100;
     option->minimum = 0;
+    int max = index.data( Role::Maximum ).toInt();
+    option->maximum = max > option->minimum ? max : option->minimum + 100;
     option->progress = index.data().toInt();
-    option->text = index.data().toString() + QChar::fromAscii( '%' );
+    option->text = QString::number( ( option->progress * 100 ) / ( option->maximum - option->minimum ) ) + QChar::fromAscii( '%' );
     option->textAlignment = Qt::AlignCenter;
     option->textVisible = true;
 }
@@ -306,7 +321,69 @@ void EnumDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewI
     editor->setGeometry(r);
 }
 
-//----------------------
+//---------------------------
+RequieredResourceDelegate::RequieredResourceDelegate( QObject *parent )
+    : ItemDelegate( parent )
+{
+}
+
+QWidget *RequieredResourceDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &/* option */, const QModelIndex &index) const
+{
+    if ( index.data( Qt::CheckStateRole ).toInt() == Qt::Unchecked ) {
+        return 0;
+    }
+    TreeComboBox *editor = new TreeComboBox(parent);
+    editor->installEventFilter(const_cast<RequieredResourceDelegate*>(this));
+    ResourceItemSFModel *m = new ResourceItemSFModel( editor );
+    editor->setModel( m );
+    return editor;
+}
+
+void RequieredResourceDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
+{
+    TreeComboBox *box = static_cast<TreeComboBox*>(editor);
+    ResourceItemSFModel *pm = static_cast<ResourceItemSFModel*>( box->model() );
+    ResourceItemModel *rm = qobject_cast<ResourceItemModel*>( pm->sourceModel() );
+    Q_ASSERT( rm );
+    const ResourceAllocationItemModel *model = qobject_cast<const ResourceAllocationItemModel*>( index.model() );
+    Q_ASSERT( model );
+    rm->setProject( model->project() );
+    pm->addFilteredResource( model->resource( index ) );
+    QItemSelectionModel *sm = box->view()->selectionModel();
+    sm->clearSelection();
+    foreach ( const Resource *r, model->required( index ) ) {
+        QModelIndex i = pm->mapFromSource( rm->index( r ) );
+        sm->select( i, QItemSelectionModel::Select | QItemSelectionModel::Rows );
+    }
+    box->setCurrentIndexes( sm->selectedRows() );
+    box->view()->expandAll();
+}
+
+void RequieredResourceDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
+                                const QModelIndex &index) const
+{
+    TreeComboBox *box = static_cast<TreeComboBox*>(editor);
+
+    QAbstractProxyModel *pm = static_cast<QAbstractProxyModel*>( box->model() );
+    ResourceItemModel *rm = qobject_cast<ResourceItemModel*>( pm->sourceModel() );
+    QList<Resource*> lst;
+    foreach ( const QModelIndex &i, box->currentIndexes() ) {
+        lst << rm->resource( pm->mapToSource( i ) );
+    }
+    ResourceAllocationItemModel *mdl = qobject_cast<ResourceAllocationItemModel*>( model );
+    Q_ASSERT( mdl );
+    mdl->setRequired( index, lst );
+}
+
+void RequieredResourceDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &/* index */) const
+{
+    kDebug()<<editor<<":"<<option.rect<<","<<editor->sizeHint();
+    QRect r = option.rect;
+    r.setWidth( qMax( 100, r.width() ) );
+    editor->setGeometry(r);
+}
+
+//-------------------------------
 DurationSpinBoxDelegate::DurationSpinBoxDelegate( QObject *parent )
     : ItemDelegate( parent )
 {
@@ -497,6 +574,7 @@ void TimeDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewI
 ItemModelBase::ItemModelBase( QObject *parent )
     : QAbstractItemModel( parent ),
     m_project(0),
+    m_manager( 0 ),
     m_readWrite( false )//part->isReadWrite() )
 {
 }
@@ -510,6 +588,11 @@ void ItemModelBase::setProject( Project *project )
     m_project = project;
 }
 
+void ItemModelBase::setScheduleManager( ScheduleManager *sm )
+{
+    m_manager = sm;
+}
+
 void ItemModelBase::slotLayoutChanged()
 {
     kDebug();
@@ -534,8 +617,26 @@ bool ItemModelBase::dropAllowed( const QModelIndex &index, int, const QMimeData
     return false;
 }
 
+QVariant ItemModelBase::data( const QModelIndex &index, int role ) const
+{
+    if ( index.isValid() && role == Role::ColumnTag ) {
+        return columnMap().key( index.column() );
+    }
+    return QVariant();
+}
+
+QVariant ItemModelBase::headerData( int section, Qt::Orientation orientation, int role ) const
+{
+    Q_UNUSED(orientation);
+    if ( role == Role::ColumnTag ) {
+        return columnMap().key( section );
+    }
+    return QVariant();
+}
+
 bool ItemModelBase::setData( const QModelIndex &index, const QVariant &value, int role )
 {
+    Q_UNUSED(index);
     if ( role == Role::ReadWrite ) {
         setReadWrite( value.toBool() );
         return true;
