@@ -108,6 +108,7 @@ public:
         NodeStatusNote,
             
         // Scheduling errors
+        NodeSchedulingStatus,
         NodeNotScheduled,
         NodeAssignmentMissing,
         NodeResourceOverbooked,
@@ -138,7 +139,7 @@ public:
     void setManager( ScheduleManager *sm );
     Project *project() const { return m_project; }
     ScheduleManager *manager() const { return m_manager; }
-    long id() const { return m_manager == 0 ? -1 : m_manager->id(); }
+    long id() const { return m_manager == 0 ? -1 : m_manager->scheduleId(); }
     
     QVariant data( const Node *node, int property, int role = Qt::DisplayRole ) const; 
     static bool setData( Node *node, int property, const QVariant & value, int role = Qt::EditRole );
@@ -206,6 +207,8 @@ public:
     QVariant actualCostTo( const Node *node, int role ) const;
     QVariant note( const Node *node, int role ) const;
 
+    /// The nodes scheduling status
+    QVariant nodeSchedulingStatus( const Node *node, int role ) const;
     /// Set if the node has not been scheduled
     QVariant nodeIsNotScheduled( const Node *node, int role ) const;
     /// Set if EffortType == Effort, but no resource is requested
@@ -214,9 +217,9 @@ public:
     QVariant resourceIsOverbooked( const Node *node, int role ) const;
     /// Set if the requested resource is not available
     QVariant resourceIsNotAvailable( const Node *node, int role ) const;
-    /// Set if the task cannot be scheduled to fullfill all the constraints
+    /// Set if the task cannot be scheduled to fullfil all the constraints
     QVariant schedulingConstraintsError( const Node *node, int role ) const;
-    /// Resources could not fullfill estimate
+    /// Resources could not fullfil estimate
     QVariant effortNotMet( const Node *node, int role ) const;
 
     QVariant wbsCode( const Node *node, int role ) const;
@@ -252,7 +255,6 @@ public:
     virtual const QMetaEnum columnMap() const { return m_nodemodel.columnMap(); }
     
     virtual void setProject( Project *project );
-    void setManager( ScheduleManager *sm );
     ScheduleManager *manager() const { return m_nodemodel.manager(); }
     long id() const { return m_nodemodel.id(); }
 
@@ -288,9 +290,15 @@ public:
     
     virtual bool dropAllowed( const QModelIndex &index, int dropIndicatorPosition, const QMimeData *data );
     
+    bool projectShown() const { return m_projectshown; }
+
 signals:
     void nodeInserted( Node *node );
     
+public slots:
+    virtual void setScheduleManager( ScheduleManager *sm );
+    void setShowProject( bool on );
+
 protected slots:
     virtual void slotWbsDefinitionChanged();
     virtual void slotNodeChanged( Node* );
@@ -307,7 +315,7 @@ protected:
     QVariant allocation( const Node *node, int role ) const;
     bool setAllocation( Node *node, const QVariant &value, int role );
     bool setDescription( Node *node, const QVariant &value, int role );
-    bool setType( Node *node, const QVariant &value, int role );
+    virtual bool setType( Node *node, const QVariant &value, int role );
     bool setConstraint( Node *node, const QVariant &value, int role );
     bool setConstraintStartTime( Node *node, const QVariant &value, int role );
     bool setConstraintEndTime( Node *node, const QVariant &value, int role );
@@ -331,6 +339,7 @@ protected:
 protected:
     Node *m_node; // for sanety check
     NodeModel m_nodemodel;
+    bool m_projectshown;
 };
 
 //--------------------------------------
@@ -418,8 +427,10 @@ public:
     explicit MilestoneItemModel( QObject *parent = 0 );
     ~MilestoneItemModel();
 
+    /// Returns a column number/- name map for this model
+    virtual const QMetaEnum columnMap() const { return m_nodemodel.columnMap(); }
+
     virtual void setProject( Project *project );
-    void setManager( ScheduleManager *sm );
 
     virtual Qt::ItemFlags flags( const QModelIndex & index ) const;
 
@@ -455,6 +466,9 @@ public:
 
     QList<Node*> mileStones() const;
     
+public slots:
+    virtual void setScheduleManager( ScheduleManager *sm );
+
 protected slots:
     void slotNodeChanged( Node* );
     void slotNodeToBeInserted( Node *node, int row );
