@@ -20,6 +20,7 @@
 #include "kptpertcpmmodel.h"
 
 #include "kptglobal.h"
+#include "kptcommonstrings.h"
 #include "kptproject.h"
 #include "kpttask.h"
 #include "kptnode.h"
@@ -33,7 +34,7 @@
 
 namespace KPlato
 {
-  
+
 class Project;
 class Node;
 class Task;
@@ -51,7 +52,7 @@ CriticalPathItemModel::CriticalPathItemModel( QObject *parent )
 CriticalPathItemModel::~CriticalPathItemModel()
 {
 }
-    
+
 void CriticalPathItemModel::slotNodeToBeInserted( Node *, int )
 {
     //kDebug()<<node->name();
@@ -64,6 +65,7 @@ void CriticalPathItemModel::slotNodeInserted( Node * /*node*/ )
 
 void CriticalPathItemModel::slotNodeToBeRemoved( Node *node )
 {
+    Q_UNUSED(node);
     //kDebug()<<node->name();
 /*    if ( m_path.contains( node ) ) {
     }*/
@@ -71,6 +73,7 @@ void CriticalPathItemModel::slotNodeToBeRemoved( Node *node )
 
 void CriticalPathItemModel::slotNodeRemoved( Node *node )
 {
+    Q_UNUSED(node);
     //kDebug()<<node->name();
 }
 
@@ -81,7 +84,7 @@ void CriticalPathItemModel::setProject( Project *project )
         disconnect( m_project, SIGNAL( nodeToBeAdded( Node*, int ) ), this, SLOT( slotNodeToBeInserted(  Node*, int ) ) );
         disconnect( m_project, SIGNAL( nodeToBeRemoved( Node* ) ), this, SLOT( slotNodeToBeRemoved( Node* ) ) );
         disconnect( m_project, SIGNAL( nodeToBeMoved( Node* ) ), this, SLOT( slotLayoutToBeChanged() ) );
-    
+
         disconnect( m_project, SIGNAL( nodeAdded( Node* ) ), this, SLOT( slotNodeInserted( Node* ) ) );
         disconnect( m_project, SIGNAL( nodeRemoved( Node* ) ), this, SLOT( slotNodeRemoved( Node* ) ) );
         disconnect( m_project, SIGNAL( nodeMoved( Node* ) ), this, SLOT( slotLayoutChanged() ) );
@@ -93,7 +96,7 @@ void CriticalPathItemModel::setProject( Project *project )
         connect( m_project, SIGNAL( nodeToBeAdded( Node*, int ) ), this, SLOT( slotNodeToBeInserted(  Node*, int ) ) );
         connect( m_project, SIGNAL( nodeToBeRemoved( Node* ) ), this, SLOT( slotNodeToBeRemoved( Node* ) ) );
         connect( m_project, SIGNAL( nodeToBeMoved( Node* ) ), this, SLOT( slotLayoutToBeChanged() ) );
-    
+
         connect( m_project, SIGNAL( nodeAdded( Node* ) ), this, SLOT( slotNodeInserted( Node* ) ) );
         connect( m_project, SIGNAL( nodeRemoved( Node* ) ), this, SLOT( slotNodeRemoved( Node* ) ) );
         connect( m_project, SIGNAL( nodeMoved( Node* ) ), this, SLOT( slotLayoutChanged() ) );
@@ -109,7 +112,7 @@ void CriticalPathItemModel::setManager( ScheduleManager *sm )
     if ( m_project == 0 || m_manager == 0 ) {
         m_path.clear();
     } else {
-        m_path = m_project->criticalPath( m_manager->id(), 0 );
+        m_path = m_project->criticalPath( m_manager->scheduleId(), 0 );
     }
     kDebug()<<m_path;
     reset();
@@ -160,7 +163,7 @@ QVariant CriticalPathItemModel::duration( int role ) const
     switch ( role ) {
         case Qt::DisplayRole:
         case Qt::ToolTipRole: {
-            Duration v = m_project->duration( m_manager->id() );
+            Duration v = m_project->duration( m_manager->scheduleId() );
             return KGlobal::locale()->formatNumber( v.toDouble( presentationUnit( v ) ), 1 ) + Duration::unitToString( presentationUnit( v ) );
         }
         case Qt::StatusTipRole:
@@ -177,7 +180,7 @@ QVariant CriticalPathItemModel::variance( int role ) const
         case Qt::ToolTipRole: {
             double v = 0.0;
             foreach ( Node *n, m_path ) {
-                long id = m_manager->id();
+                long id = m_manager->scheduleId();
                 v += n->variance( id, presentationUnit( m_project->duration( id ) ) );
             }
             return KGlobal::locale()->formatNumber( v, 1 );
@@ -186,7 +189,7 @@ QVariant CriticalPathItemModel::variance( int role ) const
         case Qt::EditRole: {
             double v = 0.0;
             foreach ( Node *n, m_path ) {
-                v += n->variance( m_manager->id() );
+                v += n->variance( m_manager->scheduleId() );
             }
             return v;
         }
@@ -250,6 +253,8 @@ QVariant CriticalPathItemModel::headerData( int section, Qt::Orientation orienta
     }
     if ( role == Qt::ToolTipRole ) {
         return m_nodemodel.headerData( section, role );
+    } else if ( role == Qt::WhatsThisRole ) {
+        return m_nodemodel.headerData( section, role );
     }
     return ItemModelBase::headerData(section, orientation, role);
 }
@@ -310,7 +315,7 @@ PertResultItemModel::PertResultItemModel( QObject *parent )
 PertResultItemModel::~PertResultItemModel()
 {
 }
-    
+
 void PertResultItemModel::slotAboutToBeReset()
 {
     kDebug();
@@ -355,7 +360,7 @@ void PertResultItemModel::setProject( Project *project )
         disconnect( m_project, SIGNAL( nodeToBeAdded( Node*, int ) ), this, SLOT( slotNodeToBeInserted(  Node*, int ) ) );
         disconnect( m_project, SIGNAL( nodeToBeRemoved( Node* ) ), this, SLOT( slotNodeToBeRemoved( Node* ) ) );
         disconnect( m_project, SIGNAL( nodeToBeMoved( Node* ) ), this, SLOT( slotLayoutToBeChanged() ) );
-    
+
         disconnect( m_project, SIGNAL( nodeAdded( Node* ) ), this, SLOT( slotNodeInserted( Node* ) ) );
         disconnect( m_project, SIGNAL( nodeRemoved( Node* ) ), this, SLOT( slotNodeRemoved( Node* ) ) );
         disconnect( m_project, SIGNAL( nodeMoved( Node* ) ), this, SLOT( slotLayoutChanged() ) );
@@ -367,7 +372,7 @@ void PertResultItemModel::setProject( Project *project )
         connect( m_project, SIGNAL( nodeToBeAdded( Node*, int ) ), this, SLOT( slotNodeToBeInserted(  Node*, int ) ) );
         connect( m_project, SIGNAL( nodeToBeRemoved( Node* ) ), this, SLOT( slotNodeToBeRemoved( Node* ) ) );
         connect( m_project, SIGNAL( nodeToBeMoved( Node* ) ), this, SLOT( slotLayoutToBeChanged() ) );
-    
+
         connect( m_project, SIGNAL( nodeAdded( Node* ) ), this, SLOT( slotNodeInserted( Node* ) ) );
         connect( m_project, SIGNAL( nodeRemoved( Node* ) ), this, SLOT( slotNodeRemoved( Node* ) ) );
         connect( m_project, SIGNAL( nodeMoved( Node* ) ), this, SLOT( slotLayoutChanged() ) );
@@ -412,7 +417,7 @@ void PertResultItemModel::refresh()
     if ( m_project == 0 ) {
         return;
     }
-    long id = m_manager == 0 ? -1 : m_manager->id();
+    long id = m_manager == 0 ? -1 : m_manager->scheduleId();
     kDebug()<<id;
     if ( id == -1 ) {
         return;
@@ -470,7 +475,7 @@ Qt::ItemFlags PertResultItemModel::flags( const QModelIndex &index ) const
     return flags;
 }
 
-    
+
 QModelIndex PertResultItemModel::parent( const QModelIndex &index ) const
 {
     if ( !index.isValid() ) {
@@ -544,7 +549,7 @@ QModelIndex PertResultItemModel::index( const NodeList *lst ) const
 
 QVariant PertResultItemModel::name( int row, int role ) const
 {
-    
+
     switch ( role ) {
         case Qt::DisplayRole:
         case Qt::EditRole:
@@ -575,9 +580,9 @@ QVariant PertResultItemModel::earlyStart( const Task *node, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return node->earlyStart( m_manager->id() ).dateTime();
+            return node->earlyStart( m_manager->scheduleId() ).dateTime();
         case Qt::ToolTipRole:
-            return KGlobal::locale()->formatDate( node->earlyStart( m_manager->id() ).date() );
+            return KGlobal::locale()->formatDate( node->earlyStart( m_manager->scheduleId() ).date() );
         case Qt::EditRole:
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
@@ -590,9 +595,9 @@ QVariant PertResultItemModel::earlyFinish( const Task *node, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return node->earlyFinish( m_manager->id() ).dateTime();
+            return node->earlyFinish( m_manager->scheduleId() ).dateTime();
         case Qt::ToolTipRole:
-            return KGlobal::locale()->formatDate( node->earlyFinish( m_manager->id() ).date() );
+            return KGlobal::locale()->formatDate( node->earlyFinish( m_manager->scheduleId() ).date() );
         case Qt::EditRole:
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
@@ -605,9 +610,9 @@ QVariant PertResultItemModel::lateStart( const Task *node, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return node->lateStart( m_manager->id() ).dateTime();
+            return node->lateStart( m_manager->scheduleId() ).dateTime();
         case Qt::ToolTipRole:
-            return KGlobal::locale()->formatDate( node->lateStart( m_manager->id() ).date() );
+            return KGlobal::locale()->formatDate( node->lateStart( m_manager->scheduleId() ).date() );
         case Qt::EditRole:
             break;
         case Qt::StatusTipRole:
@@ -621,9 +626,9 @@ QVariant PertResultItemModel::lateFinish( const Task *node, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return node->lateFinish( m_manager->id() ).dateTime();
+            return node->lateFinish( m_manager->scheduleId() ).dateTime();
         case Qt::ToolTipRole:
-            return KGlobal::locale()->formatDate( node->lateFinish( m_manager->id() ).date() );
+            return KGlobal::locale()->formatDate( node->lateFinish( m_manager->scheduleId() ).date() );
         case Qt::EditRole:
             break;
         case Qt::StatusTipRole:
@@ -637,9 +642,9 @@ QVariant PertResultItemModel::positiveFloat( const Task *node, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return node->positiveFloat( m_manager->id() ).toString( Duration::Format_i18nHourFraction );
+            return node->positiveFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nHourFraction );
         case Qt::ToolTipRole:
-            return node->positiveFloat( m_manager->id() ).toString( Duration::Format_i18nDayTime );
+            return node->positiveFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nDayTime );
         case Qt::EditRole:
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
@@ -652,9 +657,9 @@ QVariant PertResultItemModel::freeFloat( const Task *node, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return node->freeFloat( m_manager->id() ).toString( Duration::Format_i18nHourFraction );
+            return node->freeFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nHourFraction );
         case Qt::ToolTipRole:
-            return node->freeFloat( m_manager->id() ).toString( Duration::Format_i18nDayTime );
+            return node->freeFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nDayTime );
         case Qt::EditRole:
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
@@ -667,9 +672,9 @@ QVariant PertResultItemModel::negativeFloat( const Task *node, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return node->negativeFloat( m_manager->id() ).toString( Duration::Format_i18nHourFraction );
+            return node->negativeFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nHourFraction );
         case Qt::ToolTipRole:
-            return node->negativeFloat( m_manager->id() ).toString( Duration::Format_i18nDayTime );
+            return node->negativeFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nDayTime );
         case Qt::EditRole:
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
@@ -682,9 +687,9 @@ QVariant PertResultItemModel::startFloat( const Task *node, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return node->startFloat( m_manager->id() ).toString( Duration::Format_i18nHourFraction );
+            return node->startFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nHourFraction );
         case Qt::ToolTipRole:
-            return node->startFloat( m_manager->id() ).toString( Duration::Format_i18nDayTime );
+            return node->startFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nDayTime );
         case Qt::EditRole:
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
@@ -697,9 +702,9 @@ QVariant PertResultItemModel::finishFloat( const Task *node, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return node->finishFloat( m_manager->id() ).toString( Duration::Format_i18nHourFraction );
+            return node->finishFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nHourFraction );
         case Qt::ToolTipRole:
-            return node->finishFloat( m_manager->id() ).toString( Duration::Format_i18nDayTime );
+            return node->finishFloat( m_manager->scheduleId() ).toString( Duration::Format_i18nDayTime );
         case Qt::EditRole:
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
@@ -729,7 +734,7 @@ QVariant PertResultItemModel::data( const QModelIndex &index, int role ) const
         result = m_nodemodel.data( n, index.column(), role );
     }
     if ( n->type() == Node::Type_Project ) {
-        Project *p = static_cast<Project*>( n );
+        //Project *p = static_cast<Project*>( n );
         switch ( index.column() ) {
             case NodeModel::NodeName: result = name( NodeModel::NodeName, role ); break;
             default:
@@ -758,6 +763,8 @@ QVariant PertResultItemModel::headerData( int section, Qt::Orientation orientati
     }
     if ( role == Qt::ToolTipRole ) {
         return m_nodemodel.headerData( section, role );
+    } else if ( role == Qt::WhatsThisRole ) {
+        return m_nodemodel.headerData( section, role );
     }
     return ItemModelBase::headerData(section, orientation, role);
 }
