@@ -19,6 +19,7 @@
 
 #include "kptresourceallocationmodel.h"
 
+#include "kptcommonstrings.h"
 #include "kptcommand.h"
 #include "kptitemmodelbase.h"
 #include "kptcalendar.h"
@@ -118,9 +119,9 @@ QVariant ResourceAllocationModel::type( const Resource *res, int role ) const
         case Qt::EditRole:
         case Qt::ToolTipRole:
             return res->typeToString( true );
-        case Role::EnumList: 
+        case Role::EnumList:
             return res->typeToStringList( true );
-        case Role::EnumListValue: 
+        case Role::EnumListValue:
             return (int)res->type();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
@@ -138,9 +139,9 @@ QVariant ResourceAllocationModel::type( const ResourceGroup *res, int role ) con
         case Qt::EditRole:
         case Qt::ToolTipRole:
             return res->typeToString( true );
-        case Role::EnumList: 
+        case Role::EnumList:
             return res->typeToStringList( true );
-        case Role::EnumListValue: 
+        case Role::EnumListValue:
             return (int)res->type();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
@@ -173,10 +174,10 @@ QVariant ResourceAllocationModel::allocation( const ResourceGroup *group, const
         case Qt::ToolTipRole: {
             int units = rr ? rr->units() : 0;
             if ( units == 0 ) {
-                return i18n( "Not allocated" );
+                return i18nc( "@info:tooltip", "Not allocated" );
             }
             // xgettext: no-c-format
-            return i18n( "Allocated units: %1%", units );
+            return i18nc( "@info:tooltip", "Allocated units: %1%", units );
         }
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
@@ -187,6 +188,9 @@ QVariant ResourceAllocationModel::allocation( const ResourceGroup *group, const
             return 0;
         case Role::Maximum:
             return 100;
+        case Qt::CheckStateRole:
+            return Qt::Unchecked;
+
     }
     return QVariant();
 }
@@ -196,7 +200,6 @@ QVariant ResourceAllocationModel::allocation( const ResourceGroup *res, int role
     if ( m_task == 0 ) {
         return QVariant();
     }
-    //qDebug()<<"ResourceAllocationModel::allocation:"<<res->name()<<role;
     const ResourceGroupRequest *req = m_task->requests().find( res );
     switch ( role ) {
         case Qt::DisplayRole:
@@ -237,6 +240,29 @@ QVariant ResourceAllocationModel::maximum( const Resource *res, int role ) const
     return QVariant();
 }
 
+QVariant ResourceAllocationModel::required( const Resource *res, int role ) const
+{
+    switch ( role ) {
+        case Qt::DisplayRole: {
+            QStringList lst;
+            foreach ( Resource *r, res->requiredResources() ) {
+                lst << r->name();
+            }
+            return lst.join( "," );
+        }
+        case Qt::EditRole:
+            return QVariant();//Not used
+        case Qt::ToolTipRole:
+            return QVariant();
+        case Qt::TextAlignmentRole:
+            return Qt::AlignCenter;
+        case Qt::StatusTipRole:
+        case Qt::WhatsThisRole:
+            return QVariant();
+    }
+    return QVariant();
+}
+
 QVariant ResourceAllocationModel::maximum( const ResourceGroup *res, int role ) const
 {
     switch ( role ) {
@@ -265,6 +291,7 @@ QVariant ResourceAllocationModel::data( const ResourceGroup *group, const Resour
         case RequestType: result = type( resource, role ); break;
         case RequestAllocation: result = allocation( group, resource, role ); break;
         case RequestMaximum: result = maximum( resource, role ); break;
+        case RequestRequired: result = required( resource, role ); break;
         default:
             kDebug()<<"data: invalid display value: property="<<property;
             break;
@@ -305,7 +332,8 @@ QVariant ResourceAllocationModel::headerData( int section, int role )
             case RequestName: return i18n( "Name" );
             case RequestType: return i18n( "Type" );
             case RequestAllocation: return i18n( "Allocation" );
-            case RequestMaximum: return i18n( "Maximum" );
+            case RequestMaximum: return i18nc( "@title:column", "Available" );
+            case RequestRequired: return i18nc( "@title:column", "Required Resources" );
             default: return QVariant();
         }
     } else if ( role == Qt::TextAlignmentRole ) {
@@ -318,7 +346,8 @@ QVariant ResourceAllocationModel::headerData( int section, int role )
             case RequestName: return ToolTip::resourceName();
             case RequestType: return ToolTip::resourceType();
             case RequestAllocation: return i18n( "Resource allocation" );
-            case RequestMaximum: return i18n( "Maximum amount of units or resources that can be allocated" );
+            case RequestMaximum: return i18nc( "@info:tootip", "Available resources or resource units" );
+            case RequestRequired: return i18nc( "@info:tootip", "Required Resources" );
             default: return QVariant();
         }
     }
@@ -342,7 +371,7 @@ void ResourceAllocationItemModel::slotResourceToBeInserted( const ResourceGroup
     beginInsertRows( index( group ), row, row );
 }
 
-void ResourceAllocationItemModel::slotResourceInserted( const Resource *resource )
+void ResourceAllocationItemModel::slotResourceInserted( const Resource */*resource */)
 {
     //kDebug()<<resource->name();
     endInsertRows();
@@ -356,19 +385,19 @@ void ResourceAllocationItemModel::slotResourceToBeRemoved( const Resource *resou
     beginRemoveRows( index( resource->parentGroup() ), row, row );
 }
 
-void ResourceAllocationItemModel::slotResourceRemoved( const Resource *resource )
+void ResourceAllocationItemModel::slotResourceRemoved( const Resource */*resource */)
 {
     //kDebug()<<resource->name();
     endRemoveRows();
 }
 
-void ResourceAllocationItemModel::slotResourceGroupToBeInserted( const ResourceGroup *group, int row )
+void ResourceAllocationItemModel::slotResourceGroupToBeInserted( const ResourceGroup */*group*/, int row )
 {
     //kDebug()<<group->name();
     beginInsertRows( QModelIndex(), row, row );
 }
 
-void ResourceAllocationItemModel::slotResourceGroupInserted( const ResourceGroup *group )
+void ResourceAllocationItemModel::slotResourceGroupInserted( const ResourceGroup */*group */)
 {
     //kDebug()<<group->name();
     endInsertRows();
@@ -381,7 +410,7 @@ void ResourceAllocationItemModel::slotResourceGroupToBeRemoved( const ResourceGr
     beginRemoveRows( QModelIndex(), row, row );
 }
 
-void ResourceAllocationItemModel::slotResourceGroupRemoved( const ResourceGroup *group )
+void ResourceAllocationItemModel::slotResourceGroupRemoved( const ResourceGroup */*group */)
 {
     //kDebug()<<group->name();
     endRemoveRows();
@@ -392,45 +421,45 @@ void ResourceAllocationItemModel::setProject( Project *project )
     if ( m_project ) {
         disconnect( m_project, SIGNAL( resourceChanged( Resource* ) ), this, SLOT( slotResourceChanged( Resource* ) ) );
         disconnect( m_project, SIGNAL( resourceGroupChanged( ResourceGroup* ) ), this, SLOT( slotResourceGroupChanged( ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceGroupToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupToBeRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupToBeRemoved( const ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceToBeRemoved( const Resource* ) ), this, SLOT( slotResourceToBeRemoved( const Resource* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupAdded( const ResourceGroup* ) ), this, SLOT( slotResourceGroupInserted( const ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupRemoved( const ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceAdded( const Resource* ) ), this, SLOT( slotResourceInserted( const Resource* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceRemoved( const Resource* ) ), this, SLOT( slotResourceRemoved( const Resource* ) ) );
-        
+
     }
     m_project = project;
     if ( m_project ) {
         connect( m_project, SIGNAL( resourceChanged( Resource* ) ), this, SLOT( slotResourceChanged( Resource* ) ) );
         connect( m_project, SIGNAL( resourceGroupChanged( ResourceGroup* ) ), this, SLOT( slotResourceGroupChanged( ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceGroupToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupToBeRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupToBeRemoved( const ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         connect( m_project, SIGNAL( resourceToBeRemoved( const Resource* ) ), this, SLOT( slotResourceToBeRemoved( const Resource* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupAdded( const ResourceGroup* ) ), this, SLOT( slotResourceGroupInserted( const ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupRemoved( const ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceAdded( const Resource* ) ), this, SLOT( slotResourceInserted( const Resource* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceRemoved( const Resource* ) ), this, SLOT( slotResourceRemoved( const Resource* ) ) );
-    
+
     }
     m_model.setProject( m_project );
 }
@@ -456,16 +485,26 @@ void ResourceAllocationItemModel::setTask( Task *task )
 
 void ResourceAllocationItemModel::filldata( Task *task )
 {
+    qDeleteAll( m_resourceCache );
     m_resourceCache.clear();
+    qDeleteAll( m_groupCache );
     m_groupCache.clear();
+    m_requiredChecked.clear();
     if ( task ) {
         foreach ( const ResourceGroup *g, m_project->resourceGroups() ) {
             const ResourceGroupRequest *gr = task->requests().find( g );
-            m_groupCache[ g->id() ] = gr ? gr->units() : 0;
+            if ( gr ) {
+                m_groupCache[ g ] = new ResourceGroupRequest( *gr );
+            }
         }
         foreach ( const Resource *r, m_project->resourceList() ) {
             const ResourceRequest *rr = task->requests().find( r );
-            m_resourceCache[ r->id() ] = rr ? rr->units() : 0;
+            if ( rr ) {
+                m_resourceCache[ r ] = new ResourceRequest( *rr );
+                if ( ! m_resourceCache[ r ]->requiredResources().isEmpty() ) {
+                    m_requiredChecked[ r ] = Qt::Checked;
+                }
+            }
         }
     }
 }
@@ -485,6 +524,16 @@ Qt::ItemFlags ResourceAllocationItemModel::flags( const QModelIndex &index ) con
         case ResourceAllocationModel::RequestAllocation:
             flags |= ( Qt::ItemIsEditable | Qt::ItemIsUserCheckable );
             break;
+        case ResourceAllocationModel::RequestRequired: {
+            Resource *r = resource( index );
+            if ( m_resourceCache.contains( r ) && m_resourceCache[ r ]->units() > 0 ) {
+                flags |= ( Qt::ItemIsEditable | Qt::ItemIsUserCheckable );
+            }
+            if ( r && r->type() != Resource::Type_Work ) {
+                flags &= ~( Qt::ItemIsEditable | Qt::ItemIsUserCheckable );
+            }
+            break;
+        }
         default:
             flags &= ~Qt::ItemIsEditable;
             break;
@@ -506,7 +555,7 @@ QModelIndex ResourceAllocationItemModel::parent( const QModelIndex &index ) cons
         int row = m_project->indexOf(  r->parentGroup() );
         return createIndex( row, 0, r->parentGroup() );
     }
-    
+
     return QModelIndex();
 }
 
@@ -584,21 +633,32 @@ QVariant ResourceAllocationItemModel::allocation( const ResourceGroup *group, co
     if ( m_model.task() == 0 ) {
         return QVariant();
     }
-    if ( ! m_resourceCache.contains( res->id() ) ) {
+    if ( ! m_resourceCache.contains( res ) ) {
+        if ( role == Qt::EditRole ) {
+            ResourceRequest *req = m_model.task()->requests().find( res );
+            if ( req == 0 ) {
+                req = new ResourceRequest( const_cast<Resource*>( res ), 0 );
+            }
+            const_cast<ResourceAllocationItemModel*>( this )->m_resourceCache.insert( res, req );
+            return req->units();
+        }
         return m_model.allocation( group, res, role );
     }
     switch ( role ) {
         case Qt::DisplayRole: {
             // xgettext: no-c-format
-            return i18nc( "<value>%", "%1%", m_resourceCache[ res->id() ] );
+            return i18nc( "<value>%", "%1%", m_resourceCache[ res ]->units() );
         }
         case Qt::EditRole:
-            return m_resourceCache[ res->id() ];
+            return m_resourceCache[ res ]->units();
         case Qt::ToolTipRole: {
-            return i18np( "Not allocated", "Allocated units: %2", m_resourceCache[ res->id() ], allocation( group, res, Qt::DisplayRole ).toString() );
+            if ( res->units() == 0 ) {
+                return i18nc( "@info:tooltip", "Not allocated" );
+            }
+            return i18nc( "@info:tooltip", "%1 allocated out of %2 available", allocation( group, res, Qt::DisplayRole ).toString(), m_model.maximum( res, Qt::DisplayRole ).toString() );
         }
         case Qt::CheckStateRole:
-            return m_resourceCache[ res->id() ] == 0 ? Qt::Unchecked : Qt::Checked;
+            return m_resourceCache[ res ]->units() == 0 ? Qt::Unchecked : Qt::Checked;
         default:
             return m_model.allocation( group, res, role );
     }
@@ -610,13 +670,13 @@ QVariant ResourceAllocationItemModel::allocation( const ResourceGroup *res, int
     if ( m_model.task() == 0 ) {
         return QVariant();
     }
-    if ( ! m_groupCache.contains( res->id() ) ) {
+    if ( ! m_groupCache.contains( res ) ) {
         return m_model.allocation( res, role );
     }
     switch ( role ) {
         case Qt::DisplayRole:
         case Qt::EditRole:
-            return qMax( m_groupCache[ res->id() ], allocation( res, Role::Minimum ).toInt() );
+            return qMax( m_groupCache[ res ]->units(), allocation( res, Role::Minimum ).toInt() );
         case Qt::ToolTipRole:
             return i18np( "%1 resource allocated", "%1 resources allocated", allocation( res, Qt::DisplayRole ).toInt() );
         case Role::Minimum: {
@@ -628,20 +688,21 @@ QVariant ResourceAllocationItemModel::allocation( const ResourceGroup *res, int
     return QVariant();
 }
 
-
 bool ResourceAllocationItemModel::setAllocation( Resource *res, const QVariant &value, int role )
 {
-    //qDebug()<<"ResourceAllocationItemModel::setAllocation:"<<res->name()<<value<<role;
     switch ( role ) {
         case Qt::EditRole: {
-            m_resourceCache[ res->id() ] = value.toInt();
+            m_resourceCache[ res ]->setUnits( value.toInt() );
             return true;
         }
         case Qt::CheckStateRole:
-            if ( m_resourceCache[ res->id() ] == 0 ) {
-                m_resourceCache[ res->id() ] = res->units();
+            if ( ! m_resourceCache.contains( res ) ) {
+                m_resourceCache[ res ] = new ResourceRequest( res, 0 );
+            }
+            if ( m_resourceCache[ res ]->units() == 0 ) {
+                m_resourceCache[ res ]->setUnits( 100 );
             } else {
-                m_resourceCache[ res->id() ] = 0;
+                m_resourceCache[ res ]->setUnits( 0 );
             }
             return true;
     }
@@ -652,12 +713,83 @@ bool ResourceAllocationItemModel::setAllocation( ResourceGroup *res, const QVari
 {
     switch ( role ) {
         case Qt::EditRole:
-            m_groupCache[ res->id() ] = value.toInt();
+            if ( ! m_groupCache.contains( res ) ) {
+                m_groupCache[ res ] = new ResourceGroupRequest( res, 0 );
+            }
+            m_groupCache[ res ]->setUnits( value.toInt() );
             return true;
     }
     return false;
 }
 
+QVariant ResourceAllocationItemModel::required( const QModelIndex &idx, int role ) const
+{
+    if ( m_model.task() == 0 ) {
+        return QVariant();
+    }
+    Resource *res = resource( idx );
+    if ( res == 0 ) {
+        return QVariant();
+    }
+    switch ( role ) {
+        case Qt::DisplayRole: {
+            if ( res->type() == Resource::Type_Work ) {
+                QStringList lst;
+                if ( m_requiredChecked[ res ] ) {
+                    foreach ( const Resource *r, required( idx ) ) {
+                        lst << r->name();
+                    }
+                }
+                return lst.isEmpty() ? i18n( "None" ) : lst.join( "," );
+            }
+            break;
+        }
+        case Qt::EditRole: break;
+        case Qt::ToolTipRole: 
+            switch ( res->type() ) {
+                case Resource::Type_Work: {
+                    QStringList lst;
+                    if ( m_requiredChecked[ res ] ) {
+                        foreach ( const Resource *r, required( idx ) ) {
+                            lst << r->name();
+                        }
+                    }
+                    return lst.isEmpty() ? i18nc( "@info:tooltip", "No required resources" ) : lst.join( "\n" );
+                }
+                case Resource::Type_Material:
+                    return i18nc( "@info:tooltip", "Material resources cannot have required resources" );
+                case Resource::Type_Team:
+                    return i18nc( "@info:tooltip", "Team resources cannot have required resources" );
+            }
+            break;
+        case Qt::CheckStateRole:
+            if ( res->type() == Resource::Type_Work ) {
+                return m_requiredChecked[ res ];
+            }
+            break;
+        default:
+            break;
+    }
+    return QVariant();
+}
+
+bool ResourceAllocationItemModel::setRequired( const QModelIndex &idx, const QVariant &value, int role )
+{
+    Resource *res = resource( idx );
+    if ( res == 0 ) {
+        return false;
+    }
+    switch ( role ) {
+        case Qt::CheckStateRole:
+            m_requiredChecked[ res ] = value.toInt();
+            if ( value.toInt() == Qt::Unchecked ) {
+                m_resourceCache[ res ]->setRequiredResources( QList<Resource*>() );
+            }
+            emit dataChanged( idx, idx );
+            return true;
+    }
+    return false;
+}
 
 QVariant ResourceAllocationItemModel::notUsed( const ResourceGroup *, int role ) const
 {
@@ -691,6 +823,9 @@ QVariant ResourceAllocationItemModel::data( const QModelIndex &index, int role )
         if ( index.column() == ResourceAllocationModel::RequestAllocation ) {
             return allocation( r->parentGroup(), r, role );
         }
+        if ( index.column() == ResourceAllocationModel::RequestRequired ) {
+            return required( index, role );
+        }
         result = m_model.data( r->parentGroup(), r, index.column(), role );
     } else {
         ResourceGroup *g = qobject_cast<ResourceGroup*>( obj );
@@ -719,7 +854,7 @@ bool ResourceAllocationItemModel::setData( const QModelIndex &index, const QVari
     Resource *r = qobject_cast<Resource*>( obj );
     if ( r ) {
         switch (index.column()) {
-            case ResourceAllocationModel::RequestAllocation: 
+            case ResourceAllocationModel::RequestAllocation:
                 if ( setAllocation( r, value, role ) ) {
                     emit dataChanged( index, index );
                     QModelIndex idx = this->index( index.row(), ResourceAllocationModel::RequestAllocation, parent( parent( index ) ) );
@@ -727,6 +862,8 @@ bool ResourceAllocationItemModel::setData( const QModelIndex &index, const QVari
                     return true;
                 }
                 return false;
+            case ResourceAllocationModel::RequestRequired:
+                return setRequired( index, value, role );
             default:
                 //qWarning("data: invalid display value column %d", index.column());
                 return false;
@@ -773,6 +910,7 @@ QAbstractItemDelegate *ResourceAllocationItemModel::createDelegate( int col, QWi
 {
     switch ( col ) {
         case ResourceAllocationModel::RequestAllocation: return new SpinBoxDelegate( parent );
+        case ResourceAllocationModel::RequestRequired: return new RequieredResourceDelegate( parent );
         default: break;
     }
     return 0;
@@ -807,6 +945,32 @@ void ResourceAllocationItemModel::slotResourceGroupChanged( ResourceGroup *res )
     }
 }
 
+Resource *ResourceAllocationItemModel::resource( const QModelIndex &idx ) const
+{
+    return qobject_cast<Resource*>( object( idx ) );
+}
+
+void ResourceAllocationItemModel::setRequired( const QModelIndex &idx, const QList<Resource*> &lst )
+{
+    Resource *r = resource( idx );
+    Q_ASSERT( r );
+    if ( m_resourceCache.contains( r ) ) {
+        m_resourceCache[ r ]->setRequiredResources( lst );
+        emit dataChanged( idx, idx );
+    }
+}
+
+QList<Resource*> ResourceAllocationItemModel::required( const QModelIndex &idx ) const
+{
+    Resource *r = resource( idx );
+    Q_ASSERT( r );
+    if ( m_resourceCache.contains( r ) ) {
+        ResourceRequest* request = m_resourceCache[ r ];
+        return request->requiredResources();
+    }
+    return r->requiredResources();
+}
+
 } // namespace KPlato
 
 #include "kptresourceallocationmodel.moc"
