@@ -19,6 +19,7 @@
 
 #include "kptresourcemodel.h"
 
+#include "kptcommonstrings.h"
 #include "kptcommand.h"
 #include "kptitemmodelbase.h"
 #include "kptcalendar.h"
@@ -112,9 +113,9 @@ QVariant ResourceModel::type( const Resource *res, int role ) const
         case Qt::EditRole:
         case Qt::ToolTipRole:
             return res->typeToString( true );
-        case Role::EnumList: 
+        case Role::EnumList:
             return res->typeToStringList( true );
-        case Role::EnumListValue: 
+        case Role::EnumListValue:
             return (int)res->type();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
@@ -132,9 +133,9 @@ QVariant ResourceModel::type( const ResourceGroup *res, int role ) const
         case Qt::EditRole:
         case Qt::ToolTipRole:
             return res->typeToString( true );
-        case Role::EnumList: 
+        case Role::EnumList:
             return res->typeToStringList( true );
-        case Role::EnumListValue: 
+        case Role::EnumListValue:
             return (int)res->type();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
@@ -181,25 +182,44 @@ QVariant ResourceModel::email( const Resource *res, int role ) const
 QVariant ResourceModel::calendar( const Resource *res, int role ) const
 {
     switch ( role ) {
-        case Qt::DisplayRole:
-        case Qt::ToolTipRole: {
+        case Qt::DisplayRole: {
+            if ( res->type() == Resource::Type_Team ) {
+                return " ";
+            }
             QString s = i18n( "None" );
             Calendar *cal = res->calendar( true ); // don't check for default calendar
-            if ( cal == 0 ) {
+            if ( cal ) {
+                s = cal->name();
+            } else if ( res->type() == Resource::Type_Work ) {
                 // Do we get a default calendar
                 cal = res->calendar();
                 if ( cal ) {
                     s = i18nc( "Default (calendar name)", "Default (%1)", cal->name() );
                 }
-            } else {
+            }
+            return s;
+        }
+        case Qt::ToolTipRole: {
+            if ( res->type() == Resource::Type_Team ) {
+                return i18nc( "@info:tooltip", "A team resource does not have a calendar" );
+            }
+            QString s = i18nc( "@info:tooltip", "No calendar" );
+            Calendar *cal = res->calendar( true ); // don't check for default calendar
+            if ( cal ) {
                 s = cal->name();
+            } else if ( res->type() == Resource::Type_Work ) {
+                // Do we get a default calendar
+                cal = res->calendar();
+                if ( cal ) {
+                    s = i18nc( "@info:tooltip 1=calendar name", "Using default calendar: %1", cal->name() );
+                }
             }
             return s;
         }
         case Role::EnumList: {
             Calendar *cal = m_project->defaultCalendar();
             QString s = i18n( "None" );
-            if ( cal ) {
+            if ( cal &&  res->type() == Resource::Type_Work ) {
                 s = i18nc( "Default (calendar name)", "Default (%1)", cal->name() );
             }
             return QStringList() << s << m_project->calendarNames();
@@ -243,8 +263,12 @@ QVariant ResourceModel::availableFrom( const Resource *res, int role ) const
             return res->availableFrom().dateTime();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
-        case Qt::ToolTipRole:
-            return i18n( "Available from: %1", KGlobal::locale()->formatDateTime( res->availableFrom(), KLocale::LongDate, KLocale::TimeZone ) );
+        case Qt::ToolTipRole: {
+            if ( res->availableFrom().isValid() ) {
+                return i18nc( "infor:tooltip", "Available from: %1", KGlobal::locale()->formatDateTime( res->availableFrom(), KLocale::LongDate, KLocale::TimeZone ) );
+            }
+            return i18nc( "infor:tooltip", "Available from project target start time: %1", KGlobal::locale()->formatDateTime( m_project->constraintStartTime(), KLocale::LongDate, KLocale::TimeZone ) );
+        }
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
@@ -261,8 +285,12 @@ QVariant ResourceModel::availableUntil( const Resource *res, int role ) const
             return res->availableUntil().dateTime();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
-        case Qt::ToolTipRole:
-            return i18n( "Available until: %1", KGlobal::locale()->formatDateTime( res->availableUntil(), KLocale::LongDate, KLocale::TimeZone ) );
+        case Qt::ToolTipRole: {
+            if ( res->availableFrom().isValid() ) {
+                return i18nc( "infor:tooltip", "Available until: %1", KGlobal::locale()->formatDateTime( res->availableUntil(), KLocale::LongDate, KLocale::TimeZone ) );
+            }
+            return i18nc( "infor:tooltip", "Available from project target finish time: %1", KGlobal::locale()->formatDateTime( m_project->constraintEndTime(), KLocale::LongDate, KLocale::TimeZone ) );
+        }
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
@@ -274,13 +302,13 @@ QVariant ResourceModel::normalRate( const Resource *res, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return KGlobal::locale()->formatMoney( res->normalRate() );
+            return m_project->locale()->formatMoney( res->normalRate() );
         case Qt::EditRole:
             return res->normalRate();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
         case Qt::ToolTipRole:
-            return i18n( "Cost per hour, normal time: %1", KGlobal::locale()->formatMoney( res->normalRate() ) );
+            return i18n( "Cost per hour, normal time: %1", m_project->locale()->formatMoney( res->normalRate() ) );
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
@@ -292,13 +320,42 @@ QVariant ResourceModel::overtimeRate( const Resource *res, int role ) const
 {
     switch ( role ) {
         case Qt::DisplayRole:
-            return KGlobal::locale()->formatMoney( res->overtimeRate() );
+            return m_project->locale()->formatMoney( res->overtimeRate() );
         case Qt::EditRole:
             return res->overtimeRate();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
         case Qt::ToolTipRole:
-            return i18n( "Cost per hour, overtime: %1", KGlobal::locale()->formatMoney( res->overtimeRate() ) );
+            return i18n( "Cost per hour, overtime: %1", m_project->locale()->formatMoney( res->overtimeRate() ) );
+        case Qt::StatusTipRole:
+        case Qt::WhatsThisRole:
+            return QVariant();
+    }
+    return QVariant();
+}
+
+QVariant ResourceModel::account( const Resource *resource, int role ) const
+{
+    switch ( role ) {
+        case Qt::DisplayRole: {
+            Account *a = resource->account();
+            return a == 0 ? i18n( "None" ) : a->name();
+        }
+        case Qt::ToolTipRole: {
+            Account *a = resource->account();
+            return i18n( "Account: %1", (a == 0 ? i18n( "None" ) : a->name() ) );
+        }
+        case Role::EnumListValue:
+        case Qt::EditRole: {
+            Account *a = resource->account();
+            return a == 0 ? 0 : ( m_project->accounts().costElements().indexOf( a->name() ) + 1 );
+        }
+        case Role::EnumList: {
+            QStringList lst;
+            lst << i18n("None");
+            lst += m_project->accounts().costElements();
+            return lst;
+        }
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
@@ -308,6 +365,9 @@ QVariant ResourceModel::overtimeRate( const Resource *res, int role ) const
 
 QVariant ResourceModel::data( const Resource *resource, int property, int role ) const
 {
+    if ( role == Role::ObjectType ) {
+        return OT_Resource;
+    }
     QVariant result;
     if ( resource == 0 ) {
         return result;
@@ -323,6 +383,7 @@ QVariant ResourceModel::data( const Resource *resource, int property, int role )
         case ResourceAvailableUntil: result = availableUntil( resource, role ); break;
         case ResourceNormalRate: result = normalRate( resource, role ); break;
         case ResourceOvertimeRate: result = overtimeRate( resource, role ); break;
+        case ResourceAccount: result = account( resource, role ); break;
         default:
             kDebug()<<"data: invalid display value: property="<<property;
             break;
@@ -332,6 +393,9 @@ QVariant ResourceModel::data( const Resource *resource, int property, int role )
 
 QVariant ResourceModel::data( const ResourceGroup *group, int property, int role ) const
 {
+    if ( role == Role::ObjectType ) {
+        return OT_ResourceGroup;
+    }
     QVariant result;
     if ( group == 0 ) {
         return result;
@@ -368,6 +432,7 @@ QVariant ResourceModel::headerData( int section, int role )
             case ResourceAvailableUntil: return i18n( "Available Until" );
             case ResourceNormalRate: return i18n( "Normal Rate" );
             case ResourceOvertimeRate: return i18n( "Overtime Rate" );
+            case ResourceAccount: return i18n( "Account" );
             default: return QVariant();
         }
     } else if ( role == Qt::TextAlignmentRole ) {
@@ -387,6 +452,7 @@ QVariant ResourceModel::headerData( int section, int role )
             case ResourceAvailableUntil: return ToolTip::resourceAvailableUntil();
             case ResourceNormalRate: return ToolTip::resourceNormalRate();
             case ResourceOvertimeRate: return ToolTip::resourceOvertimeRate();
+            case ResourceAccount: return ToolTip::resourceAccount();
             default: return QVariant();
         }
     }
@@ -473,54 +539,63 @@ void ResourceItemModel::slotResourceGroupRemoved( const ResourceGroup *group )
     m_group = 0;
 }
 
+void ResourceItemModel::slotLayoutChanged()
+{
+    emit layoutAboutToBeChanged();
+    emit layoutChanged();
+}
+
 void ResourceItemModel::setProject( Project *project )
 {
     if ( m_project ) {
+        disconnect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotLayoutChanged() ) );
         disconnect( m_project, SIGNAL( resourceChanged( Resource* ) ), this, SLOT( slotResourceChanged( Resource* ) ) );
         disconnect( m_project, SIGNAL( resourceGroupChanged( ResourceGroup* ) ), this, SLOT( slotResourceGroupChanged( ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceGroupToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupToBeRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupToBeRemoved( const ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceToBeRemoved( const Resource* ) ), this, SLOT( slotResourceToBeRemoved( const Resource* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupAdded( const ResourceGroup* ) ), this, SLOT( slotResourceGroupInserted( const ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupRemoved( const ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceAdded( const Resource* ) ), this, SLOT( slotResourceInserted( const Resource* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceRemoved( const Resource* ) ), this, SLOT( slotResourceRemoved( const Resource* ) ) );
-        
+
         disconnect( m_project, SIGNAL( defaultCalendarChanged( Calendar* ) ), this, SLOT( slotCalendarChanged( Calendar* ) ) );
     }
     m_project = project;
     if ( m_project ) {
+        connect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotLayoutChanged() ) );
         connect( m_project, SIGNAL( resourceChanged( Resource* ) ), this, SLOT( slotResourceChanged( Resource* ) ) );
         connect( m_project, SIGNAL( resourceGroupChanged( ResourceGroup* ) ), this, SLOT( slotResourceGroupChanged( ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceGroupToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupToBeRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupToBeRemoved( const ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         connect( m_project, SIGNAL( resourceToBeRemoved( const Resource* ) ), this, SLOT( slotResourceToBeRemoved( const Resource* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupAdded( const ResourceGroup* ) ), this, SLOT( slotResourceGroupInserted( const ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupRemoved( const ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceAdded( const Resource* ) ), this, SLOT( slotResourceInserted( const Resource* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceRemoved( const Resource* ) ), this, SLOT( slotResourceRemoved( const Resource* ) ) );
-    
+
         connect( m_project, SIGNAL( defaultCalendarChanged( Calendar* ) ), this, SLOT( slotCalendarChanged( Calendar* ) ) );
     }
     m_model.setProject( m_project );
+    reset();
 }
 
 Qt::ItemFlags ResourceItemModel::flags( const QModelIndex &index ) const
@@ -565,7 +640,7 @@ QModelIndex ResourceItemModel::parent( const QModelIndex &index ) const
         int row = m_project->indexOf(  r->parentGroup() );
         return createIndex( row, 0, r->parentGroup() );
     }
-    
+
     return QModelIndex();
 }
 
@@ -687,9 +762,9 @@ QVariant ResourceItemModel::type( const ResourceGroup *res, int role ) const
         case Qt::EditRole:
         case Qt::ToolTipRole:
             return res->typeToString( true );
-        case Role::EnumList: 
+        case Role::EnumList:
             return res->typeToStringList( true );
-        case Role::EnumListValue: 
+        case Role::EnumListValue:
             return (int)res->type();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
@@ -802,7 +877,7 @@ bool ResourceItemModel::setAvailableFrom( Resource *res, const QVariant &value,
     }
     return false;
 }
-    
+
 bool ResourceItemModel::setAvailableUntil( Resource *res, const QVariant &value, int role )
 {
     switch ( role ) {
@@ -842,6 +917,23 @@ bool ResourceItemModel::setOvertimeRate( Resource *res, const QVariant &value, i
     return false;
 }
 
+bool ResourceItemModel::setAccount( Resource *res, const QVariant &value, int role )
+{
+    switch ( role ) {
+        case Qt::EditRole:
+            QStringList lst = m_model.account( res, Role::EnumList ).toStringList();
+            if ( value.toInt() >= lst.count() ) {
+                return false;
+            }
+            Account *a = m_project->accounts().findAccount( lst.at( value.toInt() ) );
+            Account *old = res->account();
+            if ( old != a ) {
+                emit executeCommand( new ResourceModifyAccountCmd( *res, old, a, i18n( "Modify resource account" ) ) );
+                return true;
+            }
+    }
+    return false;
+}
 
 QVariant ResourceItemModel::notUsed( const ResourceGroup *, int role ) const
 {
@@ -907,11 +999,12 @@ bool ResourceItemModel::setData( const QModelIndex &index, const QVariant &value
             case ResourceModel::ResourceAvailableUntil: return setAvailableUntil( r, value, role );
             case ResourceModel::ResourceNormalRate: return setNormalRate( r, value, role );
             case ResourceModel::ResourceOvertimeRate: return setOvertimeRate( r, value, role );
+            case ResourceModel::ResourceAccount: return setAccount( r, value, role );
             default:
                 qWarning("data: invalid display value column %d", index.column());
                 return false;
         }
-    } else { 
+    } else {
         ResourceGroup *g = qobject_cast<ResourceGroup*>( obj );
         if ( g ) {
             switch (index.column()) {
@@ -946,11 +1039,12 @@ QVariant ResourceItemModel::headerData( int section, Qt::Orientation orientation
     return ItemModelBase::headerData(section, orientation, role);
 }
 
-QItemDelegate *ResourceItemModel::createDelegate( int col, QWidget *parent ) const
+QAbstractItemDelegate *ResourceItemModel::createDelegate( int col, QWidget *parent ) const
 {
     switch ( col ) {
         case ResourceModel::ResourceType: return new EnumDelegate( parent );
         case ResourceModel::ResourceCalendar: return new EnumDelegate( parent );
+        case ResourceModel::ResourceAccount: return new EnumDelegate( parent );
         default: break;
     }
     return 0;
@@ -966,6 +1060,11 @@ QObject *ResourceItemModel::object( const QModelIndex &index ) const
     return o;
 }
 
+Resource *ResourceItemModel::resource( const QModelIndex &index ) const
+{
+    return qobject_cast<Resource*>( object( index ) );
+}
+
 void ResourceItemModel::slotCalendarChanged( Calendar* )
 {
     foreach ( Resource *r, m_project->resourceList() ) {
@@ -1001,9 +1100,9 @@ Qt::DropActions ResourceItemModel::supportedDropActions() const
 
 bool ResourceItemModel::dropAllowed( const QModelIndex &index, int dropIndicatorPosition, const QMimeData *data )
 {
+    Q_UNUSED(data);
     //kDebug()<<index;
     // TODO: if internal, don't allow dropping on my own parent
-    // TODO: if internal, only MoveAction is allowed, but there is no indication...
     switch ( dropIndicatorPosition ) {
         case ItemModelBase::OnItem:
             return qobject_cast<ResourceGroup*>( object( index ) ); // Allow only on group
@@ -1015,7 +1114,7 @@ bool ResourceItemModel::dropAllowed( const QModelIndex &index, int dropIndicator
 
 QStringList ResourceItemModel::mimeTypes() const
 {
-    return QStringList() 
+    return QStringList()
             << "text/x-vcard"
             << "text/directory"
             << "application/x-vnd.kde.kplato.resourceitemmodel.internal";
@@ -1043,26 +1142,45 @@ bool ResourceItemModel::dropMimeData( const QMimeData *data, Qt::DropAction acti
     kDebug()<<data->formats()<<g->name();
     if ( data->hasFormat( "application/x-vnd.kde.kplato.resourceitemmodel.internal" ) ) {
         kDebug()<<action<<Qt::MoveAction;
-        if ( action != Qt::MoveAction ) {
-            return false;
-        }
-        MacroCommand *m = 0;
-        QByteArray encodedData = data->data( "application/x-vnd.kde.kplato.resourceitemmodel.internal" );
-        QDataStream stream(&encodedData, QIODevice::ReadOnly);
-        int i = 0;
-        foreach ( Resource *r, resourceList( stream ) ) {
-            if ( r->parentGroup() == g ) {
-                continue;
+        if ( action == Qt::MoveAction ) {
+            MacroCommand *m = 0;
+            QByteArray encodedData = data->data( "application/x-vnd.kde.kplato.resourceitemmodel.internal" );
+            QDataStream stream(&encodedData, QIODevice::ReadOnly);
+            int i = 0;
+            foreach ( Resource *r, resourceList( stream ) ) {
+                if ( r->parentGroup() == g ) {
+                    continue;
+                }
+                if ( m == 0 ) m = new MacroCommand( "" );
+                m->addCommand( new MoveResourceCmd( g, r ) );
+                ++i;
             }
-            if ( m == 0 ) m = new MacroCommand( "" );
-            m->addCommand( new MoveResourceCmd( g, r ) );
-            ++i;
+            if ( m ) {
+                QString msg = i18np( "Move resource", "Move %1 resources", i );
+                MacroCommand *c = new MacroCommand( msg );
+                c->addCommand( m );
+                emit executeCommand( c );
+            }
+            return true;
         }
-        if ( m ) {
-            QString msg = i18np( "Move resource", "Move %1 resources", i );
-            MacroCommand *c = new MacroCommand( msg );
-            c->addCommand( m );
-            emit executeCommand( c );
+        if ( action == Qt::CopyAction ) {
+            MacroCommand *m = 0;
+            QByteArray encodedData = data->data( "application/x-vnd.kde.kplato.resourceitemmodel.internal" );
+            QDataStream stream(&encodedData, QIODevice::ReadOnly);
+            int i = 0;
+            foreach ( Resource *r, resourceList( stream ) ) {
+                Resource *nr = new Resource( r );
+                if ( m == 0 ) m = new MacroCommand( "" );
+                m->addCommand( new AddResourceCmd( g, nr ) );
+                ++i;
+            }
+            if ( m ) {
+                QString msg = i18np( "Copy resource", "Copy %1 resources", i );
+                MacroCommand *c = new MacroCommand( msg );
+                c->addCommand( m );
+                emit executeCommand( c );
+            }
+            return true;
         }
         return true;
     }
@@ -1075,8 +1193,8 @@ bool ResourceItemModel::dropMimeData( const QMimeData *data, Qt::DropAction acti
         QByteArray vcard = data->data( f );
         KABC::VCardConverter vc;
         KABC::Addressee::List lst = vc.parseVCards( vcard );
-        foreach( KABC::Addressee a, lst ) {
-            if ( m == 0 ) m = new MacroCommand( i18np( "Add resource from addressbook", "Add %1 resources from addressbook", lst.count() ) );
+        foreach( const KABC::Addressee &a, lst ) {
+            if ( m == 0 ) m = new MacroCommand( i18np( "Add resource from address book", "Add %1 resources from address book", lst.count() ) );
             Resource *r = new Resource();
             QString uid = a.uid();
             if ( ! m_project->findResource( uid ) ) {
@@ -1115,7 +1233,7 @@ QMimeData *ResourceItemModel::mimeData( const QModelIndexList & indexes ) const
     QByteArray encodedData;
     QDataStream stream(&encodedData, QIODevice::WriteOnly);
     QList<int> rows;
-    foreach (QModelIndex index, indexes) {
+    foreach (const QModelIndex &index, indexes) {
         if ( index.isValid() && !rows.contains( index.row() ) ) {
             //kDebug()<<index.row();
             Resource *r = ::qobject_cast<Resource*>( object( index ) );
@@ -1158,6 +1276,56 @@ QModelIndex ResourceItemModel::insertResource( ResourceGroup *g, Resource *r, Re
     return QModelIndex();
 }
 
+//-------------------
+ResourceItemSFModel::ResourceItemSFModel( QObject *parent )
+    : QSortFilterProxyModel( parent )
+{
+    setDynamicSortFilter( true );
+    setSourceModel( new ResourceItemModel( this ) );
+}
+
+void ResourceItemSFModel::setProject( Project *project )
+{
+    static_cast<ResourceItemModel*>( sourceModel() )->setProject( project );
+}
+
+Resource *ResourceItemSFModel::resource( const QModelIndex &idx ) const
+{
+    return static_cast<ResourceItemModel*>( sourceModel() )->resource( mapToSource( idx ) );
+}
+
+QModelIndex ResourceItemSFModel::index( Resource *r ) const
+{
+    return mapFromSource( static_cast<ResourceItemModel*>( sourceModel() )->index( r ) );
+}
+
+Qt::ItemFlags ResourceItemSFModel::flags( const QModelIndex & index ) const
+{
+    Qt::ItemFlags f = QSortFilterProxyModel::flags( index );
+    if ( index.isValid() && ! parent( index ).isValid() ) {
+        // group, not selectable
+        f &= ~Qt::ItemIsSelectable;
+    }
+    return f;
+}
+
+void ResourceItemSFModel::addFilteredResource( const Resource *r )
+{
+    if ( ! m_filteredResources.contains( r ) ) {
+        m_filteredResources << r;
+    }
+}
+
+bool ResourceItemSFModel::filterAcceptsRow( int source_row, const QModelIndex & source_parent ) const
+{
+    //TODO make this general filter
+    ResourceItemModel *m = static_cast<ResourceItemModel*>( sourceModel() );
+    if ( m->index( source_row, ResourceModel::ResourceType, source_parent ).data( Role::EnumListValue ).toInt() == ResourceGroup::Type_Work ) {
+        return false;
+    }
+    QModelIndex idx = m->index( source_row, 0, source_parent );
+    return ! m_filteredResources.contains( m->resource( idx ) );
+}
 
 } // namespace KPlato
 
