@@ -20,6 +20,7 @@
 #include "kptschedulemodel.h"
 
 #include "kptglobal.h"
+#include "kptcommonstrings.h"
 #include "kptcommand.h"
 #include "kptitemmodelbase.h"
 #include "kptduration.h"
@@ -153,45 +154,45 @@ void ScheduleItemModel::setProject( Project *project )
 {
     if ( m_project ) {
         disconnect( m_project, SIGNAL( scheduleManagerChanged( ScheduleManager* ) ), this, SLOT( slotManagerChanged( ScheduleManager* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleManagerToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleManagerToBeInserted( const ScheduleManager*, int) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleManagerToBeRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerToBeRemoved( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleManagerAdded( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerInserted( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleManagerRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerRemoved( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleChanged( MainSchedule* ) ), this, SLOT( slotScheduleChanged( MainSchedule* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleToBeInserted( const ScheduleManager*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleToBeRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleToBeRemoved( const MainSchedule* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleAdded( const MainSchedule* ) ), this, SLOT( slotScheduleInserted( const MainSchedule* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleRemoved( const MainSchedule* ) ) );
     }
     m_project = project;
     if ( m_project ) {
         connect( m_project, SIGNAL( scheduleManagerChanged( ScheduleManager* ) ), this, SLOT( slotManagerChanged( ScheduleManager* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleManagerToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleManagerToBeInserted( const ScheduleManager*, int) ) );
-        
+
         connect( m_project, SIGNAL( scheduleManagerToBeRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerToBeRemoved( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleManagerAdded( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerInserted( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleManagerRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerRemoved( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleChanged( MainSchedule* ) ), this, SLOT( slotScheduleChanged( MainSchedule* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleToBeInserted( const ScheduleManager*, int ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleToBeRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleToBeRemoved( const MainSchedule* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleAdded( const MainSchedule* ) ), this, SLOT( slotScheduleInserted( const MainSchedule* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleRemoved( const MainSchedule* ) ) );
     }
     setFlat( m_flat ); // update m_managerlist
@@ -347,7 +348,7 @@ bool ScheduleItemModel::setName( const QModelIndex &index, const QVariant &value
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerNameCmd( *sm, value.toString(), "Modify Schedule Name" ) );
+            emit executeCommand( new ModifyScheduleManagerNameCmd( *sm, value.toString(), i18n( "Modify schedule name" ) ) );
             return true;
     }
     return false;
@@ -361,7 +362,17 @@ QVariant ScheduleItemModel::state( const QModelIndex &index, int role ) const
     }
     switch ( role ) {
         case Qt::DisplayRole:
-        case Qt::EditRole: 
+        {
+            if ( sm->progress() > 0 ) {
+                return sm->progress();
+            }
+            QStringList l = sm->state();
+            if ( l.isEmpty() ) {
+                return "";
+            }
+            return l.first();
+        }
+        case Qt::EditRole:
         {
             QStringList l = sm->state();
             if ( l.isEmpty() ) {
@@ -376,6 +387,10 @@ QVariant ScheduleItemModel::state( const QModelIndex &index, int role ) const
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
+        case Role::Maximum:
+            return sm->maxProgress();
+        case Role::Minimum:
+            return 0;
     }
     return QVariant();
 }
@@ -396,7 +411,7 @@ QVariant ScheduleItemModel::allowOverbooking( const QModelIndex &index, int role
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->allowOverbooking();
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
@@ -422,7 +437,7 @@ bool ScheduleItemModel::setAllowOverbooking( const QModelIndex &index, const QVa
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerAllowOverbookingCmd( *sm, value.toBool(), "Modify Schedule Allow Overbooking" ) );
+            emit executeCommand( new ModifyScheduleManagerAllowOverbookingCmd( *sm, value.toBool(), i18n( "Modify allow overbooking" ) ) );
             return true;
     }
     return false;
@@ -436,7 +451,7 @@ QVariant ScheduleItemModel::usePert( const QModelIndex &index, int role ) const
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->usePert();
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
@@ -462,7 +477,7 @@ bool ScheduleItemModel::setUsePert( const QModelIndex &index, const QVariant &va
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerDistributionCmd( *sm, value.toBool(), "Modify Schedule Distribution" ) );
+            emit executeCommand( new ModifyScheduleManagerDistributionCmd( *sm, value.toBool(), i18n( "Modify scheduling distribution" ) ) );
             emit slotManagerChanged( static_cast<ScheduleManager*>( sm ) );
             return true;
     }
@@ -476,7 +491,7 @@ QVariant ScheduleItemModel::calculateAll( const QModelIndex &index, int role ) c
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->calculateAll();
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
@@ -502,7 +517,8 @@ bool ScheduleItemModel::setCalculateAll( const QModelIndex &index, const QVarian
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerCalculateAllCmd( *sm, value.toBool(), "Modify Schedule Calculate" ) );
+            //FIXME remove?
+            emit executeCommand( new ModifyScheduleManagerCalculateAllCmd( *sm, value.toBool(), "Modify schedule calculate" ) );
             return true;
     }
     return false;
@@ -515,7 +531,7 @@ QVariant ScheduleItemModel::projectStart( const QModelIndex &index, int role ) c
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
             if ( sm->expected() ) {
@@ -538,7 +554,7 @@ QVariant ScheduleItemModel::projectEnd( const QModelIndex &index, int role ) con
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
             if ( sm->expected() ) {
@@ -561,7 +577,7 @@ QVariant ScheduleItemModel::schedulingDirection( const QModelIndex &index, int r
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->schedulingDirection();
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
@@ -587,7 +603,7 @@ bool ScheduleItemModel::setSchedulingDirection( const QModelIndex &index, const
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerSchedulingDirectionCmd( *sm, value.toBool(), "Modify Scheduling Direction" ) );
+            emit executeCommand(new ModifyScheduleManagerSchedulingDirectionCmd( *sm, value.toBool(), i18n( "Modify scheduling direction" ) ) );
             emit slotManagerChanged( static_cast<ScheduleManager*>( sm ) );
             return true;
     }
@@ -600,13 +616,14 @@ QVariant ScheduleItemModel::scheduler( const QModelIndex &index, int role ) cons
     if ( sm == 0 ) {
         return QVariant();
     }
+    SchedulerPlugin *pl = sm->schedulerPlugin();
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->schedulerPluginId();
         case Qt::DisplayRole:
-            return sm->schedulerPlugin()->name();
+            return pl ? pl->name() : i18n( "Unknown" );
         case Qt::ToolTipRole:
-            return sm->schedulerPlugin()->comment();
+            return pl ? pl->comment() : QString();
         case Role::EnumList:
             return sm->schedulerPluginNames();
         case Role::EnumListValue:
@@ -626,7 +643,7 @@ bool ScheduleItemModel::setScheduler( const QModelIndex &index, const QVariant &
     if ( sm != 0 ) {
         switch ( role ) {
             case Qt::EditRole: {
-                emit executeCommand(new ModifyScheduleManagerSchedulerCmd( *sm, value.toInt(), "Modify Scheduling Direction" ) );
+                emit executeCommand( new ModifyScheduleManagerSchedulerCmd( *sm, value.toInt(), i18n( "Modify scheduler" ) ) );
                 return true;
             }
         }
@@ -641,7 +658,7 @@ QVariant ScheduleItemModel::isScheduled( const QModelIndex &index, int role ) co
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->isScheduled();
         case Qt::DisplayRole:
             return sm->isScheduled() ? i18n( "Scheduled" ) : i18n( "Not scheduled" );
@@ -666,13 +683,13 @@ QVariant ScheduleItemModel::data( const QModelIndex &index, int role ) const
         case ScheduleModel::ScheduleDirection: result = schedulingDirection( index, role ); break;
         case ScheduleModel::ScheduleOverbooking: result = allowOverbooking( index, role ); break;
         case ScheduleModel::ScheduleDistribution: result = usePert( index, role ); break;
-        case ScheduleModel::ScheduleCalculate: result = calculateAll( index, role ); break;
+//        case ScheduleModel::ScheduleCalculate: result = calculateAll( index, role ); break;
         case ScheduleModel::SchedulePlannedStart: result = projectStart(  index, role ); break;
         case ScheduleModel::SchedulePlannedFinish: result = projectEnd( index, role ); break;
         case ScheduleModel::ScheduleScheduler: result = scheduler( index, role ); break;
         case ScheduleModel::ScheduleScheduled: result = isScheduled( index, role ); break;
         default:
-            kDebug()<<"data: invalid display value column"<<index.column();;
+            kDebug()<<"data: invalid display value column"<<index.column();
             return QVariant();
     }
     if ( result.isValid() ) {
@@ -699,7 +716,7 @@ bool ScheduleItemModel::setData( const QModelIndex &index, const QVariant &value
         case ScheduleModel::ScheduleDirection: return setSchedulingDirection( index, value, role );
         case ScheduleModel::ScheduleOverbooking: return setAllowOverbooking( index, value, role );
         case ScheduleModel::ScheduleDistribution: return setUsePert( index, value, role );
-        case ScheduleModel::ScheduleCalculate: return setCalculateAll( index, value, role );
+//        case ScheduleModel::ScheduleCalculate: return setCalculateAll( index, value, role );
         case ScheduleModel::SchedulePlannedStart: return false;
         case ScheduleModel::SchedulePlannedFinish: return false;
         case ScheduleModel::ScheduleScheduler: return setScheduler( index, value, role ); break;
@@ -721,7 +738,7 @@ QVariant ScheduleItemModel::headerData( int section, Qt::Orientation orientation
                 case ScheduleModel::ScheduleDirection: return i18n( "Direction" );
                 case ScheduleModel::ScheduleOverbooking: return i18n( "Overbooking" );
                 case ScheduleModel::ScheduleDistribution: return i18n( "Distribution" );
-                case ScheduleModel::ScheduleCalculate: return i18n( "Calculate" );
+//                case ScheduleModel::ScheduleCalculate: return i18n( "Calculate" );
                 case ScheduleModel::SchedulePlannedStart: return i18n( "Planned Start" );
                 case ScheduleModel::SchedulePlannedFinish: return i18n( "Planned Finish" );
                 case ScheduleModel::ScheduleScheduler: return i18n( "Scheduler" );
@@ -741,24 +758,34 @@ QVariant ScheduleItemModel::headerData( int section, Qt::Orientation orientation
             case ScheduleModel::ScheduleDirection: return ToolTip::schedulingDirection();
             case ScheduleModel::ScheduleOverbooking: return ToolTip::scheduleOverbooking();
             case ScheduleModel::ScheduleDistribution: return ToolTip::scheduleDistribution();
-            case ScheduleModel::ScheduleCalculate: return ToolTip::scheduleCalculate();
+//            case ScheduleModel::ScheduleCalculate: return ToolTip::scheduleCalculate();
             case ScheduleModel::SchedulePlannedStart: return ToolTip::scheduleStart();
             case ScheduleModel::SchedulePlannedFinish: return ToolTip::scheduleFinish();
             case ScheduleModel::ScheduleScheduler: return ToolTip::scheduleScheduler();
             case ScheduleModel::ScheduleScheduled: return QVariant();
             default: return QVariant();
         }
+    } else if ( role == Qt::WhatsThisRole ) {
+        switch ( section ) {
+            case ScheduleModel::ScheduleDirection: return WhatsThis::schedulingDirection();
+            case ScheduleModel::ScheduleOverbooking: return WhatsThis::scheduleOverbooking();
+            case ScheduleModel::ScheduleDistribution: return WhatsThis::scheduleDistribution();
+            case ScheduleModel::ScheduleScheduler: return WhatsThis::scheduleScheduler();
+            default: return QVariant();
+        }
     }
+
     return ItemModelBase::headerData(section, orientation, role);
 }
 
 QAbstractItemDelegate *ScheduleItemModel::createDelegate( int column, QWidget *parent ) const
 {
     switch ( column ) {
+        case ScheduleModel::ScheduleState: return new ProgressBarDelegate( parent );
         case ScheduleModel::ScheduleDirection: return new EnumDelegate( parent );
         case ScheduleModel::ScheduleOverbooking: return new EnumDelegate( parent );
         case ScheduleModel::ScheduleDistribution: return new EnumDelegate( parent );
-        case ScheduleModel::ScheduleCalculate: return new EnumDelegate( parent );
+//        case ScheduleModel::ScheduleCalculate: return new EnumDelegate( parent );
         case ScheduleModel::ScheduleScheduler: return new EnumDelegate( parent );
     }
     return 0;
@@ -766,6 +793,8 @@ QAbstractItemDelegate *ScheduleItemModel::createDelegate( int column, QWidget *p
 
 void ScheduleItemModel::sort( int column, Qt::SortOrder order )
 {
+    Q_UNUSED(column);
+    Q_UNUSED(order);
 }
 
 QMimeData * ScheduleItemModel::mimeData( const QModelIndexList & ) const
@@ -842,6 +871,8 @@ void ScheduleLogItemModel::slotScheduleManagerRemoved( const ScheduleManager *ma
 
 void ScheduleLogItemModel::slotScheduleToBeInserted( const ScheduleManager *manager, int row )
 {
+    Q_UNUSED(manager);
+    Q_UNUSED(row);
     if ( m_manager && m_manager->expected() /*== ??*/ ) {
         //TODO
     }
@@ -875,37 +906,37 @@ void ScheduleLogItemModel::setProject( Project *project )
     kDebug()<<m_project<<"->"<<project;
     if ( m_project ) {
         disconnect( m_project, SIGNAL( scheduleManagerChanged( ScheduleManager* ) ), this, SLOT( slotManagerChanged( ScheduleManager* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleManagerToBeRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerToBeRemoved( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleManagerRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerRemoved( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleChanged( MainSchedule* ) ), this, SLOT( slotScheduleChanged( MainSchedule* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleToBeInserted( const ScheduleManager*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleToBeRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleToBeRemoved( const MainSchedule* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleAdded( const MainSchedule* ) ), this, SLOT( slotScheduleInserted( const MainSchedule* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleRemoved( const MainSchedule* ) ) );
     }
     m_project = project;
     if ( m_project ) {
         connect( m_project, SIGNAL( scheduleManagerChanged( ScheduleManager* ) ), this, SLOT( slotManagerChanged( ScheduleManager* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleManagerToBeRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerToBeRemoved( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleManagerRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerRemoved( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleChanged( MainSchedule* ) ), this, SLOT( slotScheduleChanged( MainSchedule* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleToBeInserted( const ScheduleManager*, int ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleToBeRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleToBeRemoved( const MainSchedule* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleAdded( const MainSchedule* ) ), this, SLOT( slotScheduleInserted( const MainSchedule* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleRemoved( const MainSchedule* ) ) );
     }
 }
@@ -930,26 +961,29 @@ void ScheduleLogItemModel::refresh()
     QStringList lst;
     lst << i18n( "Name" ) << i18n( "Phase" ) << i18n( "Severity" ) << i18n( "Message" );
     setHorizontalHeaderLabels( lst );
-    
+
     if ( m_schedule == 0 ) {
         kDebug()<<"No main schedule";
         return;
     }
     kDebug()<<m_schedule<<m_schedule->logs().count();
     QStandardItem *parentItem = invisibleRootItem();
-    QList<Schedule::Log>::ConstIterator it;
-    for ( it = m_schedule->logs().constBegin(); it != m_schedule->logs().constEnd(); ++it ) {
+    QListIterator<Schedule::Log> it( m_schedule->logs() );
+    while ( it.hasNext() ) {
+        Schedule::Log log = it.next();
         QList<QStandardItem*> lst;
-        if ( (*it).resource ) {
-            lst.append( new QStandardItem( (*it).resource->name() ) );
+        if ( log.resource ) {
+            lst.append( new QStandardItem( log.resource->name() ) );
+        } else if ( log.node ) {
+            lst.append( new QStandardItem( log.node->name() ) );
         } else {
-            lst.append( new QStandardItem( (*it).node->name() ) );
+            lst.append(  new QStandardItem( "" ) );
         }
-        lst.append( new QStandardItem( m_schedule->logPhase( (*it).phase ) ) );
-        QStandardItem *item = new QStandardItem( m_schedule->logSeverity( (*it).severity ) );
-        item->setData( (*it).severity );
+        lst.append( new QStandardItem( m_schedule->logPhase( log.phase ) ) );
+        QStandardItem *item = new QStandardItem( m_schedule->logSeverity( log.severity ) );
+        item->setData( log.severity );
         lst.append( item );
-        lst.append( new QStandardItem( (*it).message ) );
+        lst.append( new QStandardItem( log.message ) );
         parentItem->appendRow( lst );
         //kDebug()<<rowCount()<<columnCount()<<parentItem<<lst;
     }
@@ -976,6 +1010,7 @@ void ScheduleLogItemModel::slotScheduleChanged( MainSchedule *sch )
 
 Qt::ItemFlags ScheduleLogItemModel::flags( const QModelIndex &index ) const
 {
+    Q_UNUSED(index);
     return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
 }
 
