@@ -66,13 +66,13 @@ CalendarTreeView::CalendarTreeView( QWidget *parent )
 {
     header()->setContextMenuPolicy( Qt::CustomContextMenu );
     setModel( new CalendarItemModel() );
-    
+
     setSelectionBehavior( QAbstractItemView::SelectRows );
-    setSelectionMode( QAbstractItemView::ExtendedSelection );
+    setSelectionMode( QAbstractItemView::SingleSelection );
     setSelectionModel( new QItemSelectionModel( model() ) );
 
     setItemDelegateForColumn( 1, new EnumDelegate( this ) ); // timezone
-    
+
     connect( header(), SIGNAL( customContextMenuRequested ( const QPoint& ) ), this, SLOT( headerContextMenuRequested( const QPoint& ) ) );
 }
 
@@ -113,6 +113,7 @@ void CalendarTreeView::currentChanged( const QModelIndex & current, const QModel
 {
     //kDebug();
     TreeViewBase::currentChanged( current, previous );
+    // possible bug in qt: in QAbstractItemView::SingleSelection you can select multiple items/rows
     selectionModel()->select( current, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows );
     emit currentChanged( current );
 }
@@ -156,14 +157,14 @@ void CalendarTreeView::dragMoveEvent(QDragMoveEvent *event)
     event->ignore();
     QModelIndex index = indexAt( event->pos() );
     if ( ! index.isValid() ) {
-            if ( model()->dropAllowed( 0, event->mimeData() ) ) {
-                event->accept();
-            }
+        if ( model()->dropAllowed( 0, event->mimeData() ) ) {
+            event->accept();
+        }
         return;
     }
     Calendar *c = model()->calendar( index );
     if ( c == 0 ) {
-        kError()<<"no calendar to drop on!"<<endl;
+        kError()<<"no calendar to drop on!";
         return; // hmmm
     }
     switch ( dropIndicatorPosition() ) {
@@ -219,6 +220,10 @@ void CalendarDayView::slotSetWork()
     if ( receivers( SIGNAL( executeCommand( QUndoCommand* ) ) ) == 0 ) {
         return;
     }
+    Calendar *cal = model()->calendar();
+    if ( cal == 0 ) {
+        return;
+    }
     QModelIndexList lst = selectionModel()->selectedIndexes();
     if ( lst.isEmpty() ) {
         lst << currentIndex();
@@ -226,28 +231,34 @@ void CalendarDayView::slotSetWork()
     if ( lst.isEmpty() ) {
         return;
     }
-    IntervalEditDialog *dlg = new IntervalEditDialog( model()->day( lst.first() ), this );
-    if ( dlg->exec() == QDialog::Accepted ) {
-        bool mod = false;
-        MacroCommand *m = new MacroCommand( i18n( "Modify Weekday State" ) );
-        foreach ( const QModelIndex &i, lst ) {
-            CalendarDay *day = model()->day( i );
-            if ( day == 0 ) {
-                continue;
-            }
-            MacroCommand *cmd = dlg->buildCommand( model()->calendar(), day );
-            if ( cmd ) {
-                mod = true;
-                m->addCommand( cmd );
-            }
+    QList<CalendarDay*> days;
+    foreach ( const QModelIndex &i, lst ) {
+        CalendarDay *day = model()->day( i );
+        if ( day == 0 ) {
+            continue;
         }
-        if ( mod ) {
-            emit executeCommand( m );
-        } else {
-            delete m;
+        days << day;
+    }
+    IntervalEditDialog *dlg = new IntervalEditDialog( cal, days, this );
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotIntervalEditDialogFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
+}
+
+void CalendarDayView::slotIntervalEditDialogFinished( int result )
+{
+    IntervalEditDialog *dlg = qobject_cast<IntervalEditDialog*>( sender() );
+    if ( dlg == 0 ) {
+        return;
+    }
+    if ( result == QDialog::Accepted ) {
+        MacroCommand *cmd = dlg->buildCommand();
+        if ( cmd ) {
+            emit executeCommand( cmd );
         }
     }
-    delete dlg;
+    dlg->deleteLater();
 }
 
 void CalendarDayView::slotSetVacation()
@@ -315,7 +326,7 @@ void CalendarDayView::setCurrentCalendar( Calendar *calendar )
     model()->setCalendar( calendar );
 }
 
-void CalendarDayView::headerContextMenuRequested( const QPoint &pos )
+void CalendarDayView::headerContextMenuRequested( const QPoint &/*pos*/ )
 {
 //    kDebug()<<header()->logicalIndexAt(pos)<<" at"<<pos;
 }
@@ -330,9 +341,9 @@ void CalendarDayView::contextMenuEvent ( QContextMenuEvent *event )
     menu.addAction( actionSetWork );
     menu.addAction( actionSetVacation );
     menu.addAction( actionSetUndefined );
-    
+
     menu.exec( event->globalPos(), actionSetWork );
-    
+
     //emit contextMenuRequested( indexAt(event->pos()), event->globalPos() );
 }
 
@@ -380,22 +391,38 @@ CalendarEditor::CalendarEditor( KoDocument *part, QWidget *parent )
     : ViewBase( part, parent ),
     m_model( new DateTableDataModel( this ) )
 {
+    setWhatsThis( i18nc( "@info:whatsthis",
+        "<title>Work & Vacation Editor</title>"
+        "<para>"
+        "A calendar defines availability for resources or tasks of type <emphasis>Duration</emphasis>. "
+        "A calendar can be specific to a resource or task, or shared by multiple resources or tasks. "
+        "A day can be of type <emphasis>Undefined</emphasis>, <emphasis>Non-working day</emphasis> or <emphasis>Working day</emphasis>. "
+        "A working day has one or more work intervals defined. "
+        "</para><para>"
+        "A calendar can have sub calendars. If a day is undefined in a calendar, the parent calendar is checked. "
+        "An <emphasis>Undefined</emphasis> day defaults to <emphasis>Non-working</emphasis> if used by a resource, or <emphasis>available all day</emphasis> if used by a task."
+        "</para><para>"
+        "A calendar can be defined as the <emphasis>Default calendar</emphasis>. "
+        "The default calendar is used by a working resource, when the resources calendar is not explicitly set."
+        "</para>"
+        ) );
+
     setupGui();
-    
+
     QVBoxLayout *l = new QVBoxLayout( this );
     l->setMargin( 0 );
     QSplitter *sp = new QSplitter( this );
     l->addWidget( sp );
 
     m_calendarview = new CalendarTreeView( sp );
-    
+
     QFrame *f = new QFrame( sp );
     l = new QVBoxLayout( f );
     l->setMargin( 0 );
 
     m_dayview = new CalendarDayView( f );
     l->addWidget( m_dayview );
-    
+
     sp = new QSplitter( f );
     l->addWidget( sp );
     m_datePicker = new KDatePicker( sp );
@@ -421,17 +448,17 @@ CalendarEditor::CalendarEditor( KoDocument *part, QWidget *parent )
     KDateTable::BackgroundMode bgMode = KDateTable::CircleMode;
     const QColor bgColor( Qt::lightGray);
     m_datePicker->dateTable()->setCustomDatePainting( date, fgColor, bgMode, bgColor );*/
-    
-    
+
+
     m_calendarview->setEditTriggers( m_calendarview->editTriggers() | QAbstractItemView::EditKeyPressed );
-    
+
     m_dayview->setEditTriggers( m_dayview->editTriggers() | QAbstractItemView::EditKeyPressed );
 
     m_calendarview->setDragDropMode( QAbstractItemView::InternalMove );
     m_calendarview->setDropIndicatorShown ( true );
     m_calendarview->setDragEnabled ( true );
     m_calendarview->setAcceptDrops( true );
-    
+
     connect( m_calendarview->model(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
     connect( m_dayview->model(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
     connect( m_dayview, SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
@@ -439,16 +466,16 @@ CalendarEditor::CalendarEditor( KoDocument *part, QWidget *parent )
     connect( m_calendarview, SIGNAL( currentChanged( QModelIndex ) ), this, SLOT( slotCurrentCalendarChanged( QModelIndex ) ) );
     connect( m_calendarview, SIGNAL( selectionChanged( const QModelIndexList ) ), this, SLOT( slotCalendarSelectionChanged( const QModelIndexList ) ) );
     connect( m_calendarview, SIGNAL( contextMenuRequested( QModelIndex, const QPoint& ) ), this, SLOT( slotContextMenuCalendar( QModelIndex, const QPoint& ) ) );
-    
+
     connect( m_dayview, SIGNAL( currentChanged( QModelIndex ) ), this, SLOT( slotCurrentDayChanged( QModelIndex ) ) );
     connect( m_dayview, SIGNAL( selectionChanged( const QModelIndexList ) ), this, SLOT( slotDaySelectionChanged( const QModelIndexList ) ) );
     connect( m_dayview, SIGNAL( contextMenuRequested( QModelIndex, const QPoint& ) ), this, SLOT( slotContextMenuDay( QModelIndex, const QPoint& ) ) );
 
     connect( m_dayview->model(), SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ), this, SLOT( slotEnableActions() ) );
-    
+
     connect( m_calendarview, SIGNAL( focusChanged() ), this, SLOT( slotEnableActions() ) );
     connect( m_dayview, SIGNAL( focusChanged() ), this, SLOT( slotEnableActions() ) );
-    
+
 }
 
 void CalendarEditor::draw( Project &project )
@@ -476,7 +503,6 @@ void CalendarEditor::setGuiActive( bool activate )
 
 void CalendarEditor::slotContextMenuDate( KMenu *menu, const QList<QDate> &dates )
 {
-    kDebug()<<menu<<dates;
     if ( ! isReadWrite() ) {
         return;
     }
@@ -502,7 +528,7 @@ void CalendarEditor::slotContextMenuDate( KMenu *menu, const QDate &date )
     menu->addAction( actionSetUndefined );
 }
 
-void CalendarEditor::slotContextMenuCalendar( QModelIndex index, const QPoint& pos )
+void CalendarEditor::slotContextMenuCalendar( QModelIndex /*index*/, const QPoint& pos )
 {
     if ( ! isReadWrite() ) {
         return;
@@ -555,7 +581,7 @@ void CalendarEditor::slotCurrentCalendarChanged(  const QModelIndex & )
     }
 }
 
-void CalendarEditor::slotCalendarSelectionChanged( const QModelIndexList list )
+void CalendarEditor::slotCalendarSelectionChanged( const QModelIndexList /*list */)
 {
     //kDebug()<<list.count();
     updateActionsEnabled( true );
@@ -591,26 +617,26 @@ void CalendarEditor::setupGui()
 {
     KActionCollection *coll = actionCollection();
     QString name = "calendareditor_calendar_list";
-    
+
     actionAddCalendar   = new KAction(KIcon( "resource-calendar-insert" ), i18n("Add Calendar"), this);
     coll->addAction("add_calendar", actionAddCalendar  );
     actionAddCalendar ->setShortcut( KShortcut( Qt::CTRL + Qt::Key_I ) );
     connect( actionAddCalendar , SIGNAL( triggered( bool ) ), SLOT( slotAddCalendar () ) );
-    
+
     actionAddSubCalendar   = new KAction(KIcon( "resource-calendar-child-insert" ), i18n("Add Subcalendar"), this);
     coll->addAction("add_subcalendar", actionAddSubCalendar  );
     actionAddSubCalendar ->setShortcut( KShortcut( Qt::SHIFT + Qt::CTRL + Qt::Key_I ) );
     connect( actionAddSubCalendar , SIGNAL( triggered( bool ) ), SLOT( slotAddSubCalendar () ) );
-    
-    actionDeleteSelection  = new KAction(KIcon( "edit-delete" ), i18n("Delete Selected Calendar"), this);
+
+    actionDeleteSelection  = new KAction(KIcon( "edit-delete" ), i18nc( "@action", "Delete" ), this );
     coll->addAction("delete_calendar_selection", actionDeleteSelection );
     actionDeleteSelection->setShortcut( KShortcut( Qt::Key_Delete ) );
     connect( actionDeleteSelection, SIGNAL( triggered( bool ) ), SLOT( slotDeleteCalendar() ) );
-    
+
     addAction( name, actionAddCalendar  );
     addAction( name, actionAddSubCalendar  );
     addAction( name, actionDeleteSelection );
-    
+
     actionSetWork = new KAction( i18n( "Work..." ), this );
     connect( actionSetWork, SIGNAL( triggered( bool ) ), SLOT( slotSetWork() ) );
     actionSetVacation = new KAction( i18n( "Non-working" ), this );
@@ -631,11 +657,13 @@ void CalendarEditor::slotAddCalendar ()
 {
     //kDebug();
     // get parent through sibling
-    Calendar *parent = m_calendarview->selectedCalendar ();
-    if ( parent ) {
-        parent = parent->parentCal();
+    Calendar *cal = m_calendarview->selectedCalendar();
+    Calendar *parent = cal ? cal->parentCal() : 0;
+    int pos = parent ? parent->indexOf( cal ) : project()->indexOf( cal );
+    if ( pos >= 0 ) {
+        ++pos; // after selected calendar
     }
-    insertCalendar ( new Calendar (), parent );
+    insertCalendar ( new Calendar(), parent, pos );
 }
 
 void CalendarEditor::slotAddSubCalendar ()
@@ -644,9 +672,9 @@ void CalendarEditor::slotAddSubCalendar ()
     insertCalendar ( new Calendar (), m_calendarview->selectedCalendar () );
 }
 
-void CalendarEditor::insertCalendar ( Calendar *calendar, Calendar *parent )
+void CalendarEditor::insertCalendar ( Calendar *calendar, Calendar *parent, int pos )
 {
-    QModelIndex i = m_calendarview->model()->insertCalendar ( calendar, parent );
+    QModelIndex i = m_calendarview->model()->insertCalendar ( calendar, pos, parent );
     if ( i.isValid() ) {
         QModelIndex p = m_calendarview->model()->parent( i );
         //if (parent) kDebug()<<" parent="<<parent->name()<<":"<<p.row()<<","<<p.column();
@@ -726,40 +754,27 @@ void CalendarEditor::slotSetWork()
     if ( currentCalendar() == 0 || m_currentMenuDateList.isEmpty() ) {
         return;
     }
-    IntervalEditDialog *dlg = new IntervalEditDialog( currentCalendar()->findDay( m_currentMenuDateList.first() ), this );
-    if ( dlg->exec() == QDialog::Accepted ) {
-        bool mod = false;
-        MacroCommand *m = new MacroCommand( i18n ( "Modify Calendar" ) );
-        foreach ( const QDate &date, m_currentMenuDateList ) {
-            CalendarDay *day = currentCalendar()->findDay( date );
-            if ( day == 0 ) {
-                mod = true;
-                day = new CalendarDay( date, CalendarDay::Working );
-                day->setIntervals( dlg->intervals() );
-                m->addCommand( new CalendarAddDayCmd( currentCalendar(), day ) );
-                if ( m_currentMenuDateList.count() == 1 ) {
-                    m->setText( i18n( "%1: Set to Working", date.toString() ) );
-                }
-            } else {
-                MacroCommand *cmd = dlg->buildCommand( currentCalendar(), day );
-                if ( cmd ) {
-                    mod = true;
-                    kDebug()<<date<<"Modify day"<<day;
-                    m->addCommand( cmd );
-                    if ( m_currentMenuDateList.count() == 1 ) {
-                        m->setText( i18n( "%1: Modify Work Interval", date.toString() ) );
-                    }
-                }
-            }
-        }
-        if ( mod ) {
-            part()->addCommand( m );
-        } else {
-            delete m;
+    IntervalEditDialog *dlg = new IntervalEditDialog( currentCalendar(), m_currentMenuDateList, this );
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotIntervalEditDialogFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
+    m_currentMenuDateList.clear();
+}
+
+void CalendarEditor::slotIntervalEditDialogFinished( int result )
+{
+    IntervalEditDialog *dlg = qobject_cast<IntervalEditDialog*>( sender() );
+    if ( dlg == 0 ) {
+        return;
+    }
+    if ( result == QDialog::Accepted ) {
+        MacroCommand *cmd = dlg->buildCommand();
+        if ( cmd ) {
+            part()->addCommand( cmd );
         }
     }
-    m_currentMenuDateList.clear();
-    delete dlg;
+    dlg->deleteLater();
 }
 
 void CalendarEditor::slotSetVacation()
