@@ -33,6 +33,7 @@
 
 #include <QGraphicsView>
 #include <QGraphicsItem>
+#include <QTimer>
 
 #include <klocale.h>
 
@@ -64,6 +65,8 @@ public:
     int documentLastPage() const;
     void printPage( int page, QPainter &painter );
     
+    QList<QWidget*> createOptionWidgets() const;
+
 private:
     DependencyView *m_depview;
 };
@@ -84,7 +87,11 @@ public:
     virtual QPointF startPoint() const { return QPointF(); }
     virtual QPointF endPoint() const { return QPointF(); }
 
+    bool isEditable() const { return m_editable; }
+    void setEditable( bool on ) { m_editable = on; }
+
 public:
+    bool m_editable;
     DependencyNodeItem *predItem;
     DependencyNodeItem *succItem;
     Relation *relation;
@@ -144,9 +151,13 @@ public:
     void setPredConnector( DependencyConnectorItem *item );
     void setSuccConnector( DependencyConnectorItem *item );
 
+    bool isEditable() const { return m_editable; }
+    void setEditable( bool on ) { m_editable = on; }
+
 public:
     DependencyConnectorItem *predConnector;
     DependencyConnectorItem *succConnector;
+    bool m_editable;
 };
 
 //-----------------------
@@ -207,6 +218,9 @@ public:
     QList<DependencyLinkItem*> predecessorItems( ConnectorType ctype ) const;
     QList<DependencyLinkItem*> successorItems( ConnectorType ctype ) const;
 
+    bool isEditable() const { return m_editable; }
+    void setEditable( bool on ) { m_editable = on; }
+
 protected:
     void moveToY( qreal y );
     void moveToX( qreal x );
@@ -229,6 +243,8 @@ private:
     
     QList<DependencyLinkItem*> m_parentrelations;
     QList<DependencyLinkItem*> m_childrelations;
+
+    bool m_editable;
 };
 
 //-----------------------
@@ -236,14 +252,18 @@ class KPLATOUI_EXPORT DependencyNodeSymbolItem : public QGraphicsPathItem
 {
 public:
     explicit DependencyNodeSymbolItem( DependencyNodeItem *parent = 0 )
-        : QGraphicsPathItem( parent )
+        : QGraphicsPathItem( parent ),
+        m_editable( false )
     {}
     enum  { Type = QGraphicsItem::UserType + 3 };
     int type() const { return Type; }
     
     void setSymbol( int type, const QRectF &rect );
+    bool isEditable() const { return m_editable; }
+    void setEditable( bool on ) { m_editable = on; }
 private:
     GanttItemDelegate m_delegate;
+    bool m_editable;
 };
 
 //-----------------------
@@ -265,16 +285,24 @@ public:
     QList<DependencyLinkItem*> predecessorItems() const;
     QList<DependencyLinkItem*> successorItems() const;
 
+    bool isEditable() const { return m_editable; }
+    void setEditable( bool on ) { m_editable = on; }
+
 protected:
     void hoverEnterEvent ( QGraphicsSceneHoverEvent *event );
     void hoverLeaveEvent ( QGraphicsSceneHoverEvent *event );
     void mousePressEvent(QGraphicsSceneMouseEvent *event);
     void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
+    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
 
     void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
     
 private:
     DependencyNodeItem::ConnectorType m_ctype;
+
+    QPointF m_mousePressPos;
+
+    bool m_editable;
 };
 
 //-----------------------
@@ -288,7 +316,7 @@ public:
     void setProject( Project *p ) { m_project = p; }
     
     QList<QGraphicsItem*> itemList( int type ) const;
-    void clear();
+    void clearScene();
     
     qreal horizontalGap() const { return 40.0; }
     qreal verticalGap() const { return 6.0; }
@@ -335,7 +363,7 @@ public:
     void multiConnectorClicked( DependencyConnectorItem *item );
     bool connectionIsValid( DependencyConnectorItem *pred, DependencyConnectorItem *succ );
     void clearConnection();
-    
+
     /// Used when a node has been moved
     void moveItem( DependencyNodeItem *item, const QList<Node*> &lst );
     QList<DependencyNodeItem*> removeChildItems( DependencyNodeItem *item );
@@ -343,6 +371,8 @@ public:
     DependencyNodeItem *nodeItem( int row ) const;
     const QList<DependencyNodeItem*> &nodeItems() const { return m_allItems; }
 
+    void setReadWrite( bool on );
+
 signals:
     void connectorClicked( DependencyConnectorItem *item );
     void connectItems( DependencyConnectorItem *pred, DependencyConnectorItem *succ );
@@ -363,6 +393,7 @@ protected:
 private:
     Project *m_project;
     NodeItemModel *m_model;
+    bool m_readwrite;
     QList<DependencyNodeItem*> m_allItems;
     QMap<int, DependencyNodeItem*> m_visibleItems;
     QMap<int, DependencyNodeItem*> m_hiddenItems;
@@ -409,6 +440,7 @@ public slots:
     
 protected:
     void keyPressEvent(QKeyEvent *event);    
+    void mouseMoveEvent( QMouseEvent *mouseEvent );
 
 protected slots:
     void slotSelectionChanged();
@@ -423,8 +455,14 @@ protected slots:
     void slotDependencyContextMenuRequested( DependencyLinkItem *item, DependencyConnectorItem *connector );
     void slotFocusItemChanged( QGraphicsItem* );
 
+private slots:
+    void slotAutoScroll();
+
 private:
     Project *m_project;
+
+    QPoint m_cursorPos;
+    QTimer m_autoScrollTimer;
 };
 
 //------------------------------
@@ -445,7 +483,7 @@ public:
     
     virtual Relation *currentRelation() const;
 
-    virtual void updateReadWrite( bool /*readwrite*/ ) {}
+    virtual void updateReadWrite( bool readwrite );
 
     virtual KoPrintJob *createPrintJob();
     
@@ -458,6 +496,7 @@ signals:
     void addTask();
     void addMilestone();
     void addSubtask();
+    void addSubMilestone();
     void deleteTaskList( QList<Node*> );
 
     void addRelation( Node*, Node*, int );
@@ -485,6 +524,7 @@ private slots:
     void slotAddTask();
     void slotAddSubtask();
     void slotAddMilestone();
+    void slotAddSubMilestone();
     void slotDeleteTask();
 
 private:
@@ -496,9 +536,12 @@ private:
     Relation *m_currentrelation;
     ScheduleManager *m_manager;
 
+    KActionMenu *menuAddTask;
+    KActionMenu *menuAddSubTask;
     KAction *actionAddTask;
     KAction *actionAddMilestone;
     KAction *actionAddSubtask;
+    KAction *actionAddSubMilestone;
     KAction *actionDeleteTask;
 };
 
