@@ -51,12 +51,18 @@ GanttItemDelegate::GanttItemDelegate( QObject *parent )
     showCriticalPath( false ),
     showCriticalTasks( false ),
     showAppointments( false ),
-    showTimeConstraint( false ) // NOTE: atm for test, activate when the ui can be changed
+    showTimeConstraint( false ), // NOTE: atm for test, activate when the ui can be changed
+    showSchedulingError( false )
 {
     QLinearGradient b( 0., 0., 0., QApplication::fontMetrics().height() );
     b.setColorAt( 0., Qt::red );
     b.setColorAt( 1., Qt::darkRed );
     m_criticalBrush = QBrush( b );
+
+    QLinearGradient sb( 0., 0., 0., QApplication::fontMetrics().height() );
+    sb.setColorAt( 0., Qt::yellow );
+    sb.setColorAt( 1., Qt::darkYellow );
+    m_schedulingErrorBrush = QBrush( sb );
 }
 
 QVariant GanttItemDelegate::data( const QModelIndex& idx, int column, int role ) const
@@ -215,7 +221,7 @@ KDGantt::Span GanttItemDelegate::itemBoundingSpan( const KDGantt::StyleOptionGan
     if ( !idx.isValid() ) return KDGantt::Span();
 
     QRectF itemRect = opt.itemRect;
-    
+
     int typ = idx.model()->data( idx, KDGantt::ItemTypeRole ).toInt();
     QString txt = itemText( idx, typ );
     int tw = 0;
@@ -289,19 +295,19 @@ KDGantt::Span GanttItemDelegate::itemBoundingSpan( const KDGantt::StyleOptionGan
 void GanttItemDelegate::paintGanttItem( QPainter* painter, const KDGantt::StyleOptionGanttItem& opt, const QModelIndex& idx )
 {
     if ( !idx.isValid() ) return;
-    
+
     int specialtype = idx.model()->data( idx, GanttItemModel::SpecialItemTypeRole ).toInt();
     if ( specialtype > 0 ) {
         return paintSpecialItem( painter, opt, idx, specialtype );
     }
     const KDGantt::ItemType typ = static_cast<KDGantt::ItemType>( idx.model()->data( idx, KDGantt::ItemTypeRole ).toInt() );
-    
+
     QString txt = itemText( idx, typ );
     QRectF itemRect = opt.itemRect;
     QRectF boundingRect = opt.boundingRect;
     boundingRect.setY( itemRect.y() );
     boundingRect.setHeight( itemRect.height() );
-    
+
     QRectF textRect = itemRect;
     if ( ! txt.isEmpty() ) {
         int tw = opt.fontMetrics.width( txt ) + static_cast<int>( itemRect.height()/1.5 );
@@ -399,8 +405,25 @@ void GanttItemDelegate::paintGanttItem( QPainter* painter, const KDGantt::StyleO
             if ( critical ) {
                 painter->setBrush( m_criticalBrush );
             }
+            if ( showSchedulingError ) {
+                QList<int> lst;
+                lst << NodeModel::NodeAssignmentMissing
+                    << NodeModel::NodeResourceOverbooked
+                    << NodeModel::NodeResourceUnavailable
+                    << NodeModel::NodeConstraintsError
+                    << NodeModel::NodeEffortNotMet;
+                foreach ( int i, lst ) {
+                    QVariant v = data( idx, i, Qt::EditRole );
+                    //kDebug()<<idx.data(NodeModel::NodeName).toString()<<": showSchedulingError"<<i<<v;
+                    if (  v.toBool() ) {
+                        painter->setBrush( m_schedulingErrorBrush );
+                        break;
+                    }
+                }
+
+            }
             painter->drawRect( r );
-            
+
             if ( showProgress ) {
                 bool ok;
                 qreal completion = idx.model()->data( idx, KDGantt::TaskCompletionRole ).toDouble( &ok );
@@ -480,7 +503,7 @@ void GanttItemDelegate::paintGanttItem( QPainter* painter, const KDGantt::StyleO
             path.lineTo( delta, 2.*delta );
             path.lineTo( 0., delta );
             path.closeSubpath();
-            
+
             painter->save();
             painter->translate( r.topLeft() );
             painter->translate( 0.5, 0.5 );
@@ -503,13 +526,13 @@ void GanttItemDelegate::paintGanttItem( QPainter* painter, const KDGantt::StyleO
 
 /*! Paints the gantt item \a idx using \a painter and \a opt
  */
-void GanttItemDelegate::paintSpecialItem( QPainter* painter, const KDGantt::StyleOptionGanttItem& opt, const QModelIndex& idx, int typ )
+void GanttItemDelegate::paintSpecialItem( QPainter* painter, const KDGantt::StyleOptionGanttItem& opt, const QModelIndex& /*idx*/, int typ )
 {
     QRectF itemRect = opt.itemRect;
     QRectF boundingRect = opt.boundingRect;
     boundingRect.setY( itemRect.y() );
     boundingRect.setHeight( itemRect.height() );
-    
+
     painter->save();
 
     QPen pen = defaultPen( KDGantt::TypeEvent );
@@ -642,7 +665,6 @@ void ResourceGanttItemDelegate::paintGanttItem( QPainter* painter, const KDGantt
     QRectF boundingRect = opt.boundingRect;
     boundingRect.setY( itemRect.y() );
     boundingRect.setHeight( itemRect.height() );
-    //qDebug() << "itemRect="<<itemRect<<", boundingRect="<<boundingRect;
 
     painter->save();
 
@@ -668,7 +690,6 @@ void ResourceGanttItemDelegate::paintGanttItem( QPainter* painter, const KDGantt
             }
             painter->translate( 0.5, 0.5 );
             painter->drawRect( r );
-            bool ok;
             QModelIndex ix = idx.model()->index( idx.row(), ResourceAppointmentsRowModel::Load, idx.parent() );
             painter->restore();
             Qt::Alignment ta;
@@ -713,7 +734,6 @@ void ResourceGanttItemDelegate::paintGanttItem( QPainter* painter, const KDGantt
         }
         break;
     case KDGantt::TypeEvent: /* TODO */
-        //qDebug() << opt.boundingRect << opt.itemRect;
         if ( opt.boundingRect.isValid() ) {
             const qreal pw = painter->pen().width() / 2. - 1;
             const QRectF r = QRectF( opt.rect ).adjusted( -pw, -pw, pw, pw );
@@ -769,7 +789,7 @@ void ResourceGanttItemDelegate::paintResourceItem( QPainter* painter, const KDGa
     // TODO check load vs units properly, it's not as simple as below!
     foreach ( const AppointmentInterval &i, tot.intervals() ) {
         int il = i.load();
-        QString txt = KGlobal::locale()->formatNumber( (double)il / (double)rl, 1 ); 
+        QString txt = KGlobal::locale()->formatNumber( (double)il / (double)rl, 1 );
         QPen pen = painter->pen();
         if ( il > rl ) {
             painter->setBrush( m_overloadBrush );
