@@ -92,7 +92,7 @@ PertResult::PertResult( KoDocument *part, QWidget *parent )
 {
     kDebug() << " ---------------- KPlato: Creating PertResult ----------------";
     widget.setupUi(this);
-    PertResultItemModel *m = new PertResultItemModel();
+    PertResultItemModel *m = new PertResultItemModel( widget.treeWidgetTaskResult );
     widget.treeWidgetTaskResult->setModel( m );
     widget.treeWidgetTaskResult->setStretchLastSection( false );
     widget.treeWidgetTaskResult->setSelectionMode( QAbstractItemView::ExtendedSelection );
@@ -138,7 +138,7 @@ void PertResult::draw()
     widget.scheduleName->setText( i18n( "None" ) );
     widget.totalFloat->clear();
     if ( m_project && model()->manager() && model()->manager()->isScheduled() ) {
-        long id = model()->manager()->id();
+        long id = model()->manager()->scheduleId();
         if ( id == -1 ) {
             return;
         }
@@ -167,6 +167,7 @@ void PertResult::slotSplitView()
 {
     kDebug();
     widget.treeWidgetTaskResult->setViewSplitMode( ! widget.treeWidgetTaskResult->isViewSplit() );
+    emit optionsModified();
 }
 
 Node *PertResult::currentNode() const
@@ -221,8 +222,10 @@ void PertResult::slotOptions()
 {
     kDebug();
     SplitItemViewSettupDialog *dlg = new SplitItemViewSettupDialog( widget.treeWidgetTaskResult, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 void PertResult::slotUpdate(){
@@ -308,7 +311,7 @@ PertCpmView::PertCpmView( KoDocument *part, QWidget *parent )
     widget.probabilityFrame->setVisible( false );
 
     widget.cpmTable->setStretchLastSection ( false );
-    CriticalPathItemModel *m = new CriticalPathItemModel();
+    CriticalPathItemModel *m = new CriticalPathItemModel( widget.cpmTable );
     widget.cpmTable->setModel( m );
     
     setupGui();
@@ -362,6 +365,7 @@ void PertCpmView::slotSplitView()
 {
     kDebug();
     widget.cpmTable->setViewSplitMode( ! widget.cpmTable->isViewSplit() );
+    emit optionsModified();
 }
 
 Node *PertCpmView::currentNode() const
@@ -416,8 +420,10 @@ void PertCpmView::slotOptions()
 {
     kDebug();
     SplitItemViewSettupDialog *dlg = new SplitItemViewSettupDialog( widget.cpmTable, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 void PertCpmView::slotScheduleSelectionChanged( ScheduleManager *sm )
@@ -484,7 +490,7 @@ void PertCpmView::draw()
     bool enbl = m_project && current_schedule && current_schedule->isScheduled() && current_schedule->usePert();
     widget.probabilityFrame->setVisible( enbl );
     if ( m_project && current_schedule && current_schedule->isScheduled() ) {
-        long id = current_schedule->id();
+        long id = current_schedule->scheduleId();
         if ( id == -1 ) {
             return;
         }
@@ -508,7 +514,7 @@ void PertCpmView::slotFinishTimeChanged( const QDateTime &dt )
     block = true;
     double var = model()->variance( Qt::EditRole ).toDouble();
     double dev = sqrt( var );
-    DateTime et = m_project->endTime( current_schedule->id() );
+    DateTime et = m_project->endTime( current_schedule->scheduleId() );
     DateTime t = DateTime( KDateTime( dt ) );
     double d = ( et - t ).toDouble();
     d = t < et ? -d : d;
@@ -528,7 +534,7 @@ void PertCpmView::slotProbabilityChanged( int value )
     block = true;
     double var = model()->variance( Qt::EditRole ).toDouble();
     double dev = sqrt( var );
-    DateTime et = m_project->endTime( current_schedule->id() );
+    DateTime et = m_project->endTime( current_schedule->scheduleId() );
     double p = valueZ( value );
     DateTime t = et + Duration( qint64( p * dev ) );
     widget.finishTime->setDateTime( t.dateTime() );
