@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-  Copyright (C) 2006 - 2007 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2006 - 2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -20,12 +20,14 @@
 #include "kpttaskeditor.h"
 
 #include "kptglobal.h"
+#include "kptcommonstrings.h"
 #include "kptnodeitemmodel.h"
 #include "kptcommand.h"
 #include "kptproject.h"
 #include "kptitemviewsettup.h"
 #include "kptworkpackagesenddialog.h"
 #include "kptworkpackagesendpanel.h"
+#include "kptdatetime.h"
 
 #include <KoDocument.h>
 
@@ -38,6 +40,7 @@
 
 #include <kicon.h>
 #include <kaction.h>
+#include <kactionmenu.h>
 #include <kglobal.h>
 #include <klocale.h>
 #include <ktoggleaction.h>
@@ -50,6 +53,180 @@
 namespace KPlato
 {
 
+//--------------------
+TaskEditorItemModel::TaskEditorItemModel( QObject *parent )
+: NodeItemModel( parent )
+{
+}
+
+Qt::ItemFlags TaskEditorItemModel::flags( const QModelIndex &index ) const
+{
+    if ( index.column() == NodeModel::NodeType ) {
+        if ( ! m_readWrite || isColumnReadOnly( index.column() ) ) {
+            return QAbstractItemModel::flags( index );
+        }
+        Node *n = node( index );
+        if ( n && ( n->type() == Node::Type_Task || n->type() == Node::Type_Milestone ) ) {
+            return QAbstractItemModel::flags( index ) | Qt::ItemIsEditable | Qt::ItemIsDropEnabled;
+        }
+        return QAbstractItemModel::flags( index ) | Qt::ItemIsDropEnabled;
+    }
+    return NodeItemModel::flags( index );
+}
+
+QVariant TaskEditorItemModel::headerData( int section, Qt::Orientation orientation, int role ) const
+{
+    if ( orientation == Qt::Horizontal && section == NodeModel::NodeType ) {
+        if ( role == Qt::ToolTipRole ) {
+            return i18nc( "@info:tooltip", "The type of task or the estimate type of the task" );
+        } else if ( role == Qt::WhatsThisRole ) {
+            return i18nc( "@info:whatsthis",
+                          "<p>Indicates the type of task or the estimate type of the task.</p>"
+                          "The type can be set to <emphasis>Milestone</emphasis>, <emphasis>Effort</emphasis> or <emphasis>Duration</emphasis>.<nl/>"
+                          "<note>If the type is <emphasis>Summary</emphasis> or <emphasis>Project</emphasis> the type is not editable.</note>");
+        }
+    }
+    return NodeItemModel::headerData(section, orientation, role);
+}
+
+QVariant TaskEditorItemModel::data( const QModelIndex &index, int role ) const
+{
+    Node *n = node( index );
+    if ( n != 0 && index.column() == NodeModel::NodeType ) {
+        return type( n, role );
+    }
+    return NodeItemModel::data( index, role );
+}
+
+bool TaskEditorItemModel::setData( const QModelIndex &index, const QVariant &value, int role )
+{
+    Node *n = node( index );
+    if ( n != 0 && role == Qt::EditRole && index.column() == NodeModel::NodeType ) {
+        return setType( n, value, role );
+    }
+    return NodeItemModel::setData( index, value, role );
+}
+
+QVariant TaskEditorItemModel::type( const Node *node, int role ) const
+{
+    switch ( role ) {
+        case Qt::DisplayRole: {
+            if ( node->type() == Node::Type_Task ) {
+                return node->estimate()->typeToString( true );
+            }
+            return node->typeToString( true );
+        }
+        case Qt::EditRole:
+            return node->type();
+        case Qt::TextAlignmentRole:
+            return Qt::AlignCenter;
+        case Qt::ToolTipRole: {
+            if ( node->type() == Node::Type_Task ) {
+                return i18nc( "@info:tooltip", "Task with estimate type: %1", node->estimate()->typeToString( true ) );
+            }
+            return i18nc( "@info:tooltip", "Task type: %1", node->typeToString( true ) );
+        }
+        case Qt::StatusTipRole:
+        case Qt::WhatsThisRole:
+            return QVariant();
+        case Role::EnumListValue: {
+            if ( node->type() == Node::Type_Milestone ) {
+                return 0;
+            }
+            if ( node->type() == Node::Type_Task ) {
+                return node->estimate()->type() + 1;
+            }
+            return -1;
+        }
+        case Role::EnumList: {
+            QStringList lst;
+            lst << Node::typeToString( Node::Type_Milestone, true );
+            lst += Estimate::typeToStringList( true );
+            return lst;
+        }
+    }
+    return QVariant();
+}
+
+bool TaskEditorItemModel::setType( Node *node, const QVariant &value, int role )
+{
+    switch ( role ) {
+        case Qt::EditRole: {
+            if ( node->type() == Node::Type_Summarytask ) {
+                return false;
+            }
+            int v = value.toInt();
+            switch ( v ) {
+                case 0: { // Milestone
+                    NamedCommand *cmd = 0;
+                    if ( node->constraint() == Node::FixedInterval ) {
+                        cmd = new NodeModifyConstraintEndTimeCmd( *node, node->constraintStartTime().dateTime(), i18n( "Set type to Milestone" ) );
+                    } else {
+                        cmd =  new ModifyEstimateCmd( *node, node->estimate()->expectedEstimate(), 0.0, i18n( "Set type to Milestone" ) );
+                    }
+                    emit executeCommand( cmd );
+                    return true;
+                }
+                default: { // Estimate
+                    --v;
+                    MacroCommand *m = new MacroCommand( i18n( "Set type to %1", Estimate::typeToString( (Estimate::Type)v, true ) ) );
+                    m->addCommand( new ModifyEstimateTypeCmd( *node, node->estimate()->type(), v ) );
+                    if ( node->type() == Node::Type_Milestone ) {
+                        if ( node->constraint() == Node::FixedInterval ) {
+                            m->addCommand( new NodeModifyConstraintEndTimeCmd( *node, node->constraintStartTime().addDays( 1 ).dateTime() ) );
+                        } else {
+                            m->addCommand( new ModifyEstimateUnitCmd( *node, node->estimate()->unit(), Duration::Unit_d ) );
+                            m->addCommand( new ModifyEstimateCmd( *node, node->estimate()->expectedEstimate(), 1.0 ) );
+                        }
+                    }
+                    emit executeCommand( m );
+                    return true;
+                }
+            }
+            break;
+        }
+        default: break;
+    }
+    return false;
+}
+
+//--------------------
+TaskEditorTreeView::TaskEditorTreeView( QWidget *parent )
+    : DoubleTreeViewBase( parent )
+{
+    TaskEditorItemModel *m = new TaskEditorItemModel( this );
+    setModel( m );
+    //setSelectionBehavior( QAbstractItemView::SelectItems );
+    setSelectionMode( QAbstractItemView::ExtendedSelection );
+    setSelectionBehavior( QAbstractItemView::SelectRows );
+
+    createItemDelegates( m );
+    setItemDelegateForColumn( NodeModel::NodeType, new EnumDelegate( this ) );
+
+    connect( this, SIGNAL( dropAllowed( const QModelIndex&, int, QDragMoveEvent* ) ), SLOT(slotDropAllowed( const QModelIndex&, int, QDragMoveEvent* ) ) );
+}
+
+NodeItemModel *TaskEditorTreeView::baseModel() const
+{
+    NodeSortFilterProxyModel *pr = proxyModel();
+    if ( pr ) {
+        return static_cast<NodeItemModel*>( pr->sourceModel() );
+    }
+    return static_cast<NodeItemModel*>( model() );
+}
+
+void TaskEditorTreeView::slotDropAllowed( const QModelIndex &index, int dropIndicatorPosition, QDragMoveEvent *event )
+{
+    QModelIndex idx = index;
+    NodeSortFilterProxyModel *pr = proxyModel();
+    if ( pr ) {
+        idx = pr->mapToSource( index );
+    }
+    if ( baseModel()->dropAllowed( idx, dropIndicatorPosition, event->mimeData() ) ) {
+        event->accept();
+    }
+}
+
 //--------------------
 NodeTreeView::NodeTreeView( QWidget *parent )
     : DoubleTreeViewBase( parent )
@@ -59,7 +236,7 @@ NodeTreeView::NodeTreeView( QWidget *parent )
     //setSelectionBehavior( QAbstractItemView::SelectItems );
     setSelectionMode( QAbstractItemView::ExtendedSelection );
     setSelectionBehavior( QAbstractItemView::SelectRows );
-    
+
     createItemDelegates( m );
 
     connect( this, SIGNAL( dropAllowed( const QModelIndex&, int, QDragMoveEvent* ) ), SLOT(slotDropAllowed( const QModelIndex&, int, QDragMoveEvent* ) ) );
@@ -73,7 +250,7 @@ NodeItemModel *NodeTreeView::baseModel() const
     }
     return static_cast<NodeItemModel*>( model() );
 }
-    
+
 void NodeTreeView::slotDropAllowed( const QModelIndex &index, int dropIndicatorPosition, QDragMoveEvent *event )
 {
     QModelIndex idx = index;
@@ -86,6 +263,7 @@ void NodeTreeView::slotDropAllowed( const QModelIndex &index, int dropIndicatorP
     }
 }
 
+
 //-----------------------------------
 TaskEditor::TaskEditor( KoDocument *part, QWidget *parent )
     : ViewBase( part, parent )
@@ -93,7 +271,7 @@ TaskEditor::TaskEditor( KoDocument *part, QWidget *parent )
     kDebug()<<"----------------- Create TaskEditor ----------------------";
     QVBoxLayout * l = new QVBoxLayout( this );
     l->setMargin( 0 );
-    m_view = new NodeTreeView( this );
+    m_view = new TaskEditorTreeView( this );
     l->addWidget( m_view );
     kDebug()<<m_view->actionSplitView();
     setupGui();
@@ -105,48 +283,54 @@ TaskEditor::TaskEditor( KoDocument *part, QWidget *parent )
     m_view->setDragEnabled ( true );
     m_view->setAcceptDrops( true );
     m_view->setAcceptDropsOnView( true );
-    
+
     QList<int> lst1; lst1 << 1 << -1; // only display column 0 (NodeName) in left view
     QList<int> show;
-    show << NodeModel::NodeType
-            << NodeModel::NodeResponsible
+    show << NodeModel::NodeResponsible
             << NodeModel::NodeAllocation
-            << NodeModel::NodeEstimateType
+            << NodeModel::NodeType
             << NodeModel::NodeEstimateCalendar
             << NodeModel::NodeEstimate
             << NodeModel::NodeOptimisticRatio
             << NodeModel::NodePessimisticRatio
             << NodeModel::NodeRisk
-            << NodeModel::NodeConstraint 
-            << NodeModel::NodeConstraintStart 
-            << NodeModel::NodeConstraintEnd 
-            << NodeModel::NodeRunningAccount 
-            << NodeModel::NodeStartupAccount 
-            << NodeModel::NodeStartupCost 
-            << NodeModel::NodeShutdownAccount 
-            << NodeModel::NodeShutdownCost 
+            << NodeModel::NodeConstraint
+            << NodeModel::NodeConstraintStart
+            << NodeModel::NodeConstraintEnd
+            << NodeModel::NodeRunningAccount
+            << NodeModel::NodeStartupAccount
+            << NodeModel::NodeStartupCost
+            << NodeModel::NodeShutdownAccount
+            << NodeModel::NodeShutdownCost
             << NodeModel::NodeDescription;
 
-    QList<int> lst2; 
+    QList<int> lst2;
     for ( int i = 0; i < model()->columnCount(); ++i ) {
         if ( ! show.contains( i ) ) {
             lst2 << i;
         }
     }
+    for ( int i = 0; i < show.count(); ++i ) {
+        int sec = m_view->slaveView()->header()->visualIndex( show[ i ] );
+        //kDebug()<<"move section:"<<i<<show[i]<<sec;
+        if ( i != sec ) {
+            m_view->slaveView()->header()->moveSection( sec, i );
+        }
+    }
     m_view->hideColumns( lst1, lst2 );
-    m_view->masterView()->setDefaultColumns( QList<int>() << 0 );
+    m_view->masterView()->setDefaultColumns( QList<int>() << NodeModel::NodeName );
     m_view->slaveView()->setDefaultColumns( show );
-    
+
     connect( model(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
-    
+
     connect( m_view, SIGNAL( currentChanged( const QModelIndex &, const QModelIndex & ) ), this, SLOT ( slotCurrentChanged( const QModelIndex &, const QModelIndex & ) ) );
 
     connect( m_view, SIGNAL( selectionChanged( const QModelIndexList ) ), this, SLOT ( slotSelectionChanged( const QModelIndexList ) ) );
-    
+
     connect( m_view, SIGNAL( contextMenuRequested( const QModelIndex&, const QPoint& ) ), SLOT( slotContextMenuRequested( const QModelIndex&, const QPoint& ) ) );
 
     connect( m_view, SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
-    
+
 }
 
 void TaskEditor::updateReadWrite( bool rw )
@@ -185,19 +369,30 @@ void TaskEditor::slotSelectionChanged( const QModelIndexList list)
     slotEnableActions();
 }
 
-int TaskEditor::selectedNodeCount() const
+QModelIndexList TaskEditor::selectedRows() const
 {
-    QItemSelectionModel* sm = m_view->selectionModel();
-    return sm->selectedRows().count();
+#if 0
+// Qt bug?
+    return m_view->selectionModel()->selectedRows();
+#else
+    QModelIndexList lst;
+    foreach ( QModelIndex i, m_view->selectionModel()->selectedIndexes() ) {
+        if ( i.column() == 0 ) {
+            lst << i;
+        }
+    }
+    return lst;
+#endif
+}
+
+int TaskEditor::selectedRowCount() const
+{
+    return selectedRows().count();
 }
 
 QList<Node*> TaskEditor::selectedNodes() const {
     QList<Node*> lst;
-    QItemSelectionModel* sm = m_view->selectionModel();
-    if ( sm == 0 ) {
-        return lst;
-    }
-    foreach ( const QModelIndex &i, sm->selectedRows() ) {
+    foreach ( const QModelIndex &i, selectedRows() ) {
         Node * n = m_view->baseModel()->node( i );
         if ( n != 0 && n->type() != Node::Type_Project ) {
             lst.append( n );
@@ -232,6 +427,9 @@ void TaskEditor::slotContextMenuRequested( const QModelIndex& index, const QPoin
     kDebug()<<node->name()<<" :"<<pos;
     QString name;
     switch ( node->type() ) {
+        case Node::Type_Project:
+            name = "task_edit_popup";
+            break;
         case Node::Type_Task:
             name = node->isScheduled( baseModel()->id() ) ? "task_popup" : "task_edit_popup";
             break;
@@ -241,8 +439,6 @@ void TaskEditor::slotContextMenuRequested( const QModelIndex& index, const QPoin
         case Node::Type_Summarytask:
             name = "summarytask_popup";
             break;
-        case Node::Type_Project:
-            break;
         default:
             name = "node_popup";
             break;
@@ -258,7 +454,7 @@ void TaskEditor::slotContextMenuRequested( const QModelIndex& index, const QPoin
 void TaskEditor::setScheduleManager( ScheduleManager *sm )
 {
     //kDebug()<<endl;
-    m_view->baseModel()->setManager( sm );
+    m_view->baseModel()->setScheduleManager( sm );
 }
 
 void TaskEditor::slotEnableActions()
@@ -269,17 +465,21 @@ void TaskEditor::slotEnableActions()
 void TaskEditor::updateActionsEnabled( bool on )
 {
     Project *p = m_view->project();
-    
-    bool o = ( on && p && selectedNodeCount() <= 1 );
-    actionAddTask->setEnabled( o );
-    actionAddMilestone->setEnabled( o );
-    
-    actionDeleteTask->setEnabled( on && p && selectedNodeCount() > 0 );
-    
-    o = ( on && p && selectedNodeCount() == 1 );
-    Node *n = selectedNode();
-    
+    int selCount = selectedRowCount();
+    Node *n = selectedNode(); // 0 if not task or milestone
+    bool o = ( on && p && selCount <= 1 );
+    menuAddTask->setEnabled( o && n != p );
+    actionAddTask->setEnabled( o && n != p );
+    actionAddMilestone->setEnabled( o && n != p );
+
+    int projSelected = selCount == 1 && n == 0;
+    actionDeleteTask->setEnabled( on && p && ! projSelected && selCount > 0 );
+
+    o = ( on && p && selCount == 1 );
+
+    menuAddSubTask->setEnabled( o );
     actionAddSubtask->setEnabled( o );
+    actionAddSubMilestone->setEnabled( o );
     actionMoveTaskUp->setEnabled( o && p->canMoveTaskUp( n ) );
     actionMoveTaskDown->setEnabled( o && p->canMoveTaskDown( n ) );
     actionIndentTask->setEnabled( o && p->canIndentTask( n ) );
@@ -289,53 +489,71 @@ void TaskEditor::updateActionsEnabled( bool on )
 void TaskEditor::setupGui()
 {
     QString name = "taskeditor_add_list";
-    actionAddTask  = new KAction(KIcon( "view-task-add" ), i18n("Add Task..."), this);
+
+    menuAddTask = new KActionMenu( KIcon( "view-task-add" ), i18n( "Add Task" ), this );
+    actionCollection()->addAction("add_task", menuAddTask );
+    connect( menuAddTask, SIGNAL( triggered( bool ) ), SLOT( slotAddTask() ) );
+    addAction( name, menuAddTask );
+
+    actionAddTask  = new KAction( i18n( "Add Task" ), this);
     actionAddTask->setShortcut( KShortcut( Qt::CTRL + Qt::Key_I ) );
-    actionCollection()->addAction("add_task", actionAddTask );
     connect( actionAddTask, SIGNAL( triggered( bool ) ), SLOT( slotAddTask() ) );
-    addAction( name, actionAddTask );
-    
-    actionAddSubtask  = new KAction(KIcon( "view-task-child-add" ), i18n("Add Sub-Task..."), this);
-    actionAddSubtask->setShortcut( KShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_I ) );
-    actionCollection()->addAction("add_sub_task", actionAddSubtask );
-    connect( actionAddSubtask, SIGNAL( triggered( bool ) ), SLOT( slotAddSubtask() ) );
-    addAction( name, actionAddSubtask );
-    
-    actionAddMilestone  = new KAction(KIcon( "view-milestone-add" ), i18n("Add Milestone..."), this);
+    menuAddTask->addAction( actionAddTask );
+
+    actionAddMilestone  = new KAction( i18n( "Add Milestone" ), this );
     actionAddMilestone->setShortcut( KShortcut( Qt::CTRL + Qt::ALT + Qt::Key_I ) );
-    actionCollection()->addAction("add_milestone", actionAddMilestone );
     connect( actionAddMilestone, SIGNAL( triggered( bool ) ), SLOT( slotAddMilestone() ) );
-    addAction( name, actionAddMilestone );
-    
-    actionDeleteTask  = new KAction(KIcon( "edit-delete" ), i18n("Delete Task"), this);
+    menuAddTask->addAction( actionAddMilestone );
+
+
+    menuAddSubTask = new KActionMenu( KIcon( "view-task-child-add" ), i18n( "Add Sub-Task" ), this );
+    actionCollection()->addAction("add_subtask", menuAddTask );
+    connect( menuAddSubTask, SIGNAL( triggered( bool ) ), SLOT( slotAddSubtask() ) );
+    addAction( name, menuAddSubTask );
+
+    actionAddSubtask  = new KAction( i18n( "Add Sub-Task" ), this );
+    actionAddSubtask->setShortcut( KShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_I ) );
+    connect( actionAddSubtask, SIGNAL( triggered( bool ) ), SLOT( slotAddSubtask() ) );
+    menuAddSubTask->addAction( actionAddSubtask );
+
+    actionAddSubMilestone = new KAction( i18n( "Add Sub-Milestone" ), this );
+    actionAddSubMilestone->setShortcut( KShortcut( Qt::CTRL + Qt::SHIFT + Qt::ALT + Qt::Key_I ) );
+    connect( actionAddSubMilestone, SIGNAL( triggered( bool ) ), SLOT( slotAddSubMilestone() ) );
+    menuAddSubTask->addAction( actionAddSubMilestone );
+
+    actionDeleteTask  = new KAction(KIcon( "edit-delete" ), i18nc( "@action", "Delete" ), this);
     actionDeleteTask->setShortcut( KShortcut( Qt::Key_Delete ) );
     actionCollection()->addAction("delete_task", actionDeleteTask );
     connect( actionDeleteTask, SIGNAL( triggered( bool ) ), SLOT( slotDeleteTask() ) );
     addAction( name, actionDeleteTask );
 
-    
+
     name = "taskeditor_move_list";
     actionIndentTask  = new KAction(KIcon("format-indent-more"), i18n("Indent Task"), this);
     actionCollection()->addAction("indent_task", actionIndentTask );
     connect(actionIndentTask, SIGNAL(triggered(bool) ), SLOT(slotIndentTask()));
     addAction( name, actionIndentTask );
-    
+
     actionUnindentTask  = new KAction(KIcon("format-indent-less"), i18n("Unindent Task"), this);
     actionCollection()->addAction("unindent_task", actionUnindentTask );
     connect(actionUnindentTask, SIGNAL(triggered(bool) ), SLOT(slotUnindentTask()));
     addAction( name, actionUnindentTask );
-    
+
     actionMoveTaskUp  = new KAction(KIcon("arrow-up"), i18n("Move Up"), this);
     actionCollection()->addAction("move_task_up", actionMoveTaskUp );
     connect(actionMoveTaskUp, SIGNAL(triggered(bool) ), SLOT(slotMoveTaskUp()));
     addAction( name, actionMoveTaskUp );
-    
+
     actionMoveTaskDown  = new KAction(KIcon("arrow-down"), i18n("Move Down"), this);
     actionCollection()->addAction("move_task_down", actionMoveTaskDown );
     connect(actionMoveTaskDown, SIGNAL(triggered(bool) ), SLOT(slotMoveTaskDown()));
     addAction( name, actionMoveTaskDown );
 
     // Add the context menu actions for the view options
+    actionShowProject = new KToggleAction( i18n( "Show Project" ), this );
+    connect(actionShowProject, SIGNAL(triggered(bool) ), baseModel(), SLOT(setShowProject(bool)));
+    addContextAction( actionShowProject );
+
     connect(m_view->actionSplitView(), SIGNAL(triggered(bool) ), SLOT(slotSplitView()));
     addContextAction( m_view->actionSplitView() );
 
@@ -346,6 +564,7 @@ void TaskEditor::slotSplitView()
 {
     kDebug();
     m_view->setViewSplitMode( ! m_view->isViewSplit() );
+    emit optionsModified();
 }
 
 
@@ -353,18 +572,20 @@ void TaskEditor::slotOptions()
 {
     kDebug();
     SplitItemViewSettupDialog *dlg = new SplitItemViewSettupDialog( m_view, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 void TaskEditor::slotAddTask()
 {
     kDebug();
-    if ( selectedNodeCount() == 0 ) {
-        // insert under main project
+    if ( selectedRowCount() == 0 || ( selectedRowCount() == 1 && selectedNode() == 0 ) ) {
         Task *t = m_view->project()->createTask( m_view->project()->taskDefaults(),  m_view->project() );
         QModelIndex idx = m_view->baseModel()->insertSubtask( t, t->parentNode() );
         Q_ASSERT( idx.isValid() );
+        m_view->setParentsExpanded( idx, true ); // rightview is not automatically expanded
         edit( idx );
         return;
     }
@@ -381,12 +602,13 @@ void TaskEditor::slotAddTask()
 void TaskEditor::slotAddMilestone()
 {
     kDebug();
-    if ( selectedNodeCount() == 0 ) {
-        // insert under main project
+    if ( selectedRowCount() == 0  || ( selectedRowCount() == 1 && selectedNode() == 0 ) ) {
+        // None selected or only project selected: insert under main project
         Task *t = m_view->project()->createTask( m_view->project() );
         t->estimate()->clear();
         QModelIndex idx = m_view->baseModel()->insertSubtask( t, t->parentNode() );
         Q_ASSERT( idx.isValid() );
+        m_view->setParentsExpanded( idx, true ); // rightview is not automatically expanded
         edit( idx );
         return;
     }
@@ -398,6 +620,26 @@ void TaskEditor::slotAddMilestone()
     t->estimate()->clear();
     QModelIndex idx = m_view->baseModel()->insertTask( t, sib );
     Q_ASSERT( idx.isValid() );
+    m_view->setParentsExpanded( idx, true ); // rightview is not automatically expanded
+    edit( idx );
+}
+
+void TaskEditor::slotAddSubMilestone()
+{
+    kDebug();
+    Node *parent = selectedNode();
+    if ( parent == 0 && selectedRowCount() == 1 ) {
+        // project selected
+        parent = m_view->project();
+    }
+    if ( parent == 0 ) {
+        return;
+    }
+    Task *t = m_view->project()->createTask( m_view->project()->taskDefaults(), parent );
+    t->estimate()->clear();
+    QModelIndex idx = m_view->baseModel()->insertSubtask( t, parent );
+    Q_ASSERT( idx.isValid() );
+    m_view->setParentsExpanded( idx, true ); // rightview is not automatically expanded
     edit( idx );
 }
 
@@ -405,6 +647,10 @@ void TaskEditor::slotAddSubtask()
 {
     kDebug();
     Node *parent = selectedNode();
+    if ( parent == 0 && selectedRowCount() == 1 ) {
+        // project selected
+        parent = m_view->project();
+    }
     if ( parent == 0 ) {
         return;
     }
@@ -418,6 +664,10 @@ void TaskEditor::slotAddSubtask()
 void TaskEditor::edit( QModelIndex i )
 {
     if ( i.isValid() ) {
+        if ( m_view->slaveView()->hasFocus() ) {
+            m_view->masterView()->setFocus();
+        }
+
         m_view->selectionModel()->select( i, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect );
         QModelIndex p = m_view->model()->parent( i );
         m_view->selectionModel()->setCurrentIndex( i, QItemSelectionModel::NoUpdate );
@@ -509,11 +759,15 @@ void TaskEditor::slotMoveTaskDown()
 bool TaskEditor::loadContext( const KoXmlElement &context )
 {
     kDebug();
+    bool show = (bool)(context.attribute( "show-project", "0" ).toInt() );
+    actionShowProject->setChecked( show );
+    baseModel()->setShowProject( show ); // why is this not called by the action?
     return m_view->loadContext( baseModel()->columnMap(), context );
 }
 
 void TaskEditor::saveContext( QDomElement &context ) const
 {
+    context.setAttribute( "show-project", baseModel()->projectShown() );
     m_view->saveContext( baseModel()->columnMap(), context );
 }
 
@@ -542,7 +796,7 @@ TaskView::TaskView( KoDocument *part, QWidget *parent )
     m_view->setAcceptDrops( false );
     m_view->setAcceptDropsOnView( false );
 
-    QList<int> readonly; 
+    QList<int> readonly;
     readonly << NodeModel::NodeName
             << NodeModel::NodeResponsible
             << NodeModel::NodeAllocation
@@ -552,21 +806,21 @@ TaskView::TaskView( KoDocument *part, QWidget *parent )
             << NodeModel::NodeOptimisticRatio
             << NodeModel::NodePessimisticRatio
             << NodeModel::NodeRisk
-            << NodeModel::NodeConstraint 
-            << NodeModel::NodeConstraintStart 
-            << NodeModel::NodeConstraintEnd 
-            << NodeModel::NodeRunningAccount 
-            << NodeModel::NodeStartupAccount 
-            << NodeModel::NodeStartupCost 
-            << NodeModel::NodeShutdownAccount 
-            << NodeModel::NodeShutdownCost 
+            << NodeModel::NodeConstraint
+            << NodeModel::NodeConstraintStart
+            << NodeModel::NodeConstraintEnd
+            << NodeModel::NodeRunningAccount
+            << NodeModel::NodeStartupAccount
+            << NodeModel::NodeStartupCost
+            << NodeModel::NodeShutdownAccount
+            << NodeModel::NodeShutdownCost
             << NodeModel::NodeDescription;
     foreach ( int c, readonly ) {
         m_view->baseModel()->setReadOnly( c, true );
     }
 
     QList<int> lst1; lst1 << 1 << -1;
-    QList<int> show; 
+    QList<int> show;
     show << NodeModel::NodeStatus
             << NodeModel::NodeCompleted
             << NodeModel::NodeResponsible
@@ -576,11 +830,11 @@ TaskView::TaskView( KoDocument *part, QWidget *parent )
             << NodeModel::NodeBCWP
             << NodeModel::NodeACWP
             << NodeModel::NodeDescription;
-    
+
     for ( int s = 0; s < show.count(); ++s ) {
         m_view->slaveView()->mapToSection( show[s], s );
     }
-    QList<int> lst2; 
+    QList<int> lst2;
     for ( int i = 0; i < m_view->model()->columnCount(); ++i ) {
         if ( ! show.contains( i ) ) {
             lst2 << i;
@@ -589,13 +843,13 @@ TaskView::TaskView( KoDocument *part, QWidget *parent )
     m_view->hideColumns( lst1, lst2 );
     m_view->masterView()->setDefaultColumns( QList<int>() << 0 );
     m_view->slaveView()->setDefaultColumns( show );
-    
+
     connect( m_view->baseModel(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
 
     connect( m_view, SIGNAL( currentChanged( const QModelIndex &, const QModelIndex & ) ), this, SLOT ( slotCurrentChanged( const QModelIndex &, const QModelIndex & ) ) );
 
     connect( m_view, SIGNAL( selectionChanged( const QModelIndexList ) ), this, SLOT ( slotSelectionChanged( const QModelIndexList ) ) );
-    
+
     connect( m_view, SIGNAL( contextMenuRequested( const QModelIndex&, const QPoint& ) ), SLOT( slotContextMenuRequested( const QModelIndex&, const QPoint& ) ) );
 
     connect( m_view, SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
@@ -705,7 +959,7 @@ void TaskView::slotContextMenuRequested( const QModelIndex& index, const QPoint&
 void TaskView::setScheduleManager( ScheduleManager *sm )
 {
     //kDebug()<<endl;
-    m_view->baseModel()->setManager( sm );
+    m_view->baseModel()->setScheduleManager( sm );
 }
 
 void TaskView::slotEnableActions()
@@ -713,19 +967,23 @@ void TaskView::slotEnableActions()
     updateActionsEnabled( true );
 }
 
-void TaskView::updateActionsEnabled( bool on )
+void TaskView::updateActionsEnabled( bool /*on*/ )
 {
-    
+
 }
 
 void TaskView::setupGui()
 {
-    KActionCollection *coll = actionCollection();
-    
+//    KActionCollection *coll = actionCollection();
+
     // Add the context menu actions for the view options
+    actionShowProject = new KToggleAction( i18n( "Show Project" ), this );
+    connect(actionShowProject, SIGNAL(triggered(bool) ), baseModel(), SLOT(setShowProject(bool)));
+    addContextAction( actionShowProject );
+
     connect(m_view->actionSplitView(), SIGNAL(triggered(bool) ), SLOT(slotSplitView()));
     addContextAction( m_view->actionSplitView() );
-    
+
     createOptionAction();
 }
 
@@ -733,24 +991,30 @@ void TaskView::slotSplitView()
 {
     kDebug();
     m_view->setViewSplitMode( ! m_view->isViewSplit() );
+    emit optionsModified();
 }
 
 void TaskView::slotOptions()
 {
     kDebug();
     SplitItemViewSettupDialog *dlg = new SplitItemViewSettupDialog( m_view, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 bool TaskView::loadContext( const KoXmlElement &context )
 {
-    kDebug();
+    bool show = (bool)(context.attribute( "show-project", "0" ).toInt() );
+    actionShowProject->setChecked( show );
+    baseModel()->setShowProject( show ); // why is this not called by the action?
     return m_view->loadContext( m_view->baseModel()->columnMap(), context );
 }
 
 void TaskView::saveContext( QDomElement &context ) const
 {
+    context.setAttribute( "show-project", baseModel()->projectShown() );
     m_view->saveContext( m_view->baseModel()->columnMap(), context );
 }
 
@@ -768,7 +1032,7 @@ GeneralNodeTreeView::GeneralNodeTreeView( QWidget *parent )
     //setSelectionBehavior( QAbstractItemView::SelectItems );
     setSelectionMode( QAbstractItemView::ExtendedSelection );
     setSelectionBehavior( QAbstractItemView::SelectRows );
-    
+
     createItemDelegates( m );
 
     connect( this, SIGNAL( dropAllowed( const QModelIndex&, int, QDragMoveEvent* ) ), SLOT(slotDropAllowed( const QModelIndex&, int, QDragMoveEvent* ) ) );
@@ -787,7 +1051,7 @@ GeneralNodeItemModel *GeneralNodeTreeView::baseModel() const
     }
     return static_cast<GeneralNodeItemModel*>( model() );
 }
-    
+
 void GeneralNodeTreeView::slotDropAllowed( const QModelIndex &index, int dropIndicatorPosition, QDragMoveEvent *event )
 {
     QModelIndex idx = index;
@@ -811,7 +1075,7 @@ TaskWorkPackageView::TaskWorkPackageView( KoDocument *part, QWidget *parent )
     m_view->setModus( GeneralNodeItemModel::Flat | GeneralNodeItemModel::WorkPackage );
     NodeSortFilterProxyModel *p = new NodeSortFilterProxyModel( m_view->baseModel(), m_view, false );
     m_view->setModel( p );
-    
+
     m_view->setSortingEnabled( true );
     m_view->sortByColumn( NodeModel::NodeWBSCode, Qt::AscendingOrder );
 
@@ -830,7 +1094,7 @@ TaskWorkPackageView::TaskWorkPackageView( KoDocument *part, QWidget *parent )
     m_view->setAcceptDrops( false );
     m_view->setAcceptDropsOnView( false );
 
-    QList<int> readonly; 
+    QList<int> readonly;
     readonly << NodeModel::NodeName
             << NodeModel::NodeResponsible
             << NodeModel::NodeAllocation
@@ -840,31 +1104,31 @@ TaskWorkPackageView::TaskWorkPackageView( KoDocument *part, QWidget *parent )
             << NodeModel::NodeOptimisticRatio
             << NodeModel::NodePessimisticRatio
             << NodeModel::NodeRisk
-            << NodeModel::NodeConstraint 
-            << NodeModel::NodeConstraintStart 
-            << NodeModel::NodeConstraintEnd 
-            << NodeModel::NodeRunningAccount 
-            << NodeModel::NodeStartupAccount 
-            << NodeModel::NodeStartupCost 
-            << NodeModel::NodeShutdownAccount 
-            << NodeModel::NodeShutdownCost 
+            << NodeModel::NodeConstraint
+            << NodeModel::NodeConstraintStart
+            << NodeModel::NodeConstraintEnd
+            << NodeModel::NodeRunningAccount
+            << NodeModel::NodeStartupAccount
+            << NodeModel::NodeStartupCost
+            << NodeModel::NodeShutdownAccount
+            << NodeModel::NodeShutdownCost
             << NodeModel::NodeDescription;
     foreach ( int c, readonly ) {
         m_view->baseModel()->setReadOnly( c, true );
     }
 
     QList<int> lst1; lst1 << 1 << -1;
-    QList<int> show; 
+    QList<int> show;
     show << NodeModel::NodeStatus
             << NodeModel::NodeCompleted
             << NodeModel::NodeResponsible
             << NodeModel::NodeAssignments
             << NodeModel::NodeDescription;
-    
+
     for ( int s = 0; s < show.count(); ++s ) {
         m_view->slaveView()->mapToSection( show[s], s );
     }
-    QList<int> lst2; 
+    QList<int> lst2;
     for ( int i = 0; i < m_view->model()->columnCount(); ++i ) {
         if ( ! show.contains( i ) ) {
             lst2 << i;
@@ -873,13 +1137,13 @@ TaskWorkPackageView::TaskWorkPackageView( KoDocument *part, QWidget *parent )
     m_view->hideColumns( lst1, lst2 );
     m_view->masterView()->setDefaultColumns( QList<int>() << 0 );
     m_view->slaveView()->setDefaultColumns( show );
-    
+
     connect( m_view->baseModel(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
 
     connect( m_view, SIGNAL( currentChanged( const QModelIndex &, const QModelIndex & ) ), this, SLOT ( slotCurrentChanged( const QModelIndex &, const QModelIndex & ) ) );
 
     connect( m_view, SIGNAL( selectionChanged( const QModelIndexList ) ), this, SLOT ( slotSelectionChanged( const QModelIndexList ) ) );
-    
+
     connect( m_view, SIGNAL( contextMenuRequested( const QModelIndex&, const QPoint& ) ), SLOT( slotContextMenuRequested( const QModelIndex&, const QPoint& ) ) );
 
     connect( m_view, SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
@@ -939,7 +1203,7 @@ QList<Node*> TaskWorkPackageView::selectedNodes() const {
     if ( sm == 0 ) {
         return lst;
     }
-    foreach ( QModelIndex i, sm->selectedRows() ) {
+    foreach ( const QModelIndex &i, sm->selectedRows() ) {
         Node * n = m_view->baseModel()->node( proxyModel()->mapToSource( i ) );
         if ( n != 0 && n->type() != Node::Type_Project ) {
             lst.append( n );
@@ -995,7 +1259,7 @@ void TaskWorkPackageView::slotContextMenuRequested( const QModelIndex& index, co
 void TaskWorkPackageView::setScheduleManager( ScheduleManager *sm )
 {
     //kDebug()<<endl;
-    m_view->baseModel()->setManager( sm );
+    m_view->baseModel()->setScheduleManager( sm );
 }
 
 void TaskWorkPackageView::slotEnableActions()
@@ -1011,8 +1275,8 @@ void TaskWorkPackageView::updateActionsEnabled( bool on )
 
 void TaskWorkPackageView::setupGui()
 {
-    KActionCollection *coll = actionCollection();
-    
+//    KActionCollection *coll = actionCollection();
+
     QString name = "workpackage_list";
     actionMailWorkpackage  = new KAction(KIcon( "mail-send" ), i18n("Send..."), this);
     actionMailWorkpackage->setShortcut( KShortcut( Qt::CTRL + Qt::Key_M ) );
@@ -1023,7 +1287,7 @@ void TaskWorkPackageView::setupGui()
     // Add the context menu actions for the view options
     connect(m_view->actionSplitView(), SIGNAL(triggered(bool) ), SLOT(slotSplitView()));
     addContextAction( m_view->actionSplitView() );
-    
+
     createOptionAction();
 }
 
@@ -1035,7 +1299,7 @@ void TaskWorkPackageView::slotMailWorkpackage()
         m_cmd = new MacroCommand( "Log Send Workpackages" );
         WorkPackageSendDialog *dlg = new WorkPackageSendDialog( lst, scheduleManager(), this );
         connect ( dlg->panel(), SIGNAL( sendWorkpackages( QList<Node*>&, Resource* ) ), this, SIGNAL( mailWorkpackages( QList<Node*>&, Resource* ) ) );
-        
+
         connect ( dlg->panel(), SIGNAL( sendWorkpackages( QList<Node*>&, Resource* ) ), this, SLOT( slotWorkPackageSent( QList<Node*>&, Resource* ) ) );
         dlg->exec();
         delete dlg;
@@ -1050,7 +1314,6 @@ void TaskWorkPackageView::slotMailWorkpackage()
 
 void TaskWorkPackageView::slotWorkPackageSent( QList<Node*> &nodes, Resource *resource )
 {
-    qDebug()<<"slotWorkPackageSent:"<<nodes<<resource->name();
     foreach ( Node *n, nodes ) {
         WorkPackage *wp = new WorkPackage( static_cast<Task*>( n )->workPackage() );
         wp->setOwnerName( resource->name() );
@@ -1065,14 +1328,17 @@ void TaskWorkPackageView::slotSplitView()
 {
     kDebug();
     m_view->setViewSplitMode( ! m_view->isViewSplit() );
+    emit optionsModified();
 }
 
 void TaskWorkPackageView::slotOptions()
 {
     kDebug();
     SplitItemViewSettupDialog *dlg = new SplitItemViewSettupDialog( m_view, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 bool TaskWorkPackageView::loadContext( const KoXmlElement &context )
