@@ -108,10 +108,6 @@ TaskProgressPanel::TaskProgressPanel( Task &task, ScheduleManager *sm, StandardW
     connect(finishTime, SIGNAL(dateTimeChanged(const QDateTime &)), SLOT(slotFinishTimeChanged( const QDateTime& )));
 }
 
-bool TaskProgressPanel::ok() {
-    return true;
-}
-
 MacroCommand *TaskProgressPanel::buildCommand()
 {
     Project *project = dynamic_cast<Project*>( m_task.projectNode() );
@@ -281,7 +277,9 @@ void TaskProgressPanelImpl::slotEditmodeChanged( int idx )
 void TaskProgressPanelImpl::slotStartedChanged(bool state) {
     m_completion.setStarted( state );
     if (state) {
-        m_completion.setStartTime( KDateTime::currentLocalDateTime() );
+        QTime t = QTime::currentTime();
+        t.setHMS( t.hour(), t.minute(), 0 );
+        m_completion.setStartTime( KDateTime( QDateTime( QDate::currentDate(), t ) ) );
         startTime->setDateTime( m_completion.startTime().dateTime() );
         slotCalculateEffort();
     }
@@ -289,7 +287,9 @@ void TaskProgressPanelImpl::slotStartedChanged(bool state) {
 }
 
 void TaskProgressPanelImpl::setFinished() {
-    finishTime->setDateTime( QDateTime::currentDateTime() );
+    QTime t = QTime::currentTime();
+    t.setHMS( t.hour(), t.minute(), 0 );
+    finishTime->setDateTime( QDateTime( QDate::currentDate(), t ) );
     slotFinishTimeChanged( finishTime->dateTime() );
 }
 
@@ -311,9 +311,6 @@ void TaskProgressPanelImpl::slotFinishTimeChanged( const QDateTime &dt )
         return;
     }
     m_completion.setFinishTime( KDateTime( dt, KDateTime::Spec(KDateTime::LocalZone) ) );
-    if ( m_completion.percentFinished() == 100 ) {
-        m_completion.takeEntry( m_completion.entryDate() );
-    }
     if ( m_completion.percentFinished() < 100 ) {
         m_completion.setPercentFinished( dt.date(), 100 );
     }
