@@ -51,7 +51,7 @@ UsedEffortItemModel::UsedEffortItemModel ( QWidget *parent )
 
 Qt::ItemFlags UsedEffortItemModel::flags ( const QModelIndex &index ) const
 {
-    
+
     Qt::ItemFlags flags = QAbstractItemModel::flags( index );
     if ( ! index.isValid() || index.column() == 8 ) {
         return flags;
@@ -159,12 +159,12 @@ bool UsedEffortItemModel::setData ( const QModelIndex &idx, const QVariant &valu
             if ( idx.column() == 0 ) {
                 const Resource *er = resource( idx );
                 Q_ASSERT( er != 0 );
-                
+
                 Q_ASSERT ( m_editlist.count() > value.toInt() );
 
                 const Resource *v = m_editlist.values().value( value.toInt() );
                 Q_ASSERT( v != 0 );
-                
+
                 int x = m_resourcelist.indexOf( er );
                 Q_ASSERT( x != -1 );
                 m_resourcelist.replace( x, v );
@@ -339,9 +339,9 @@ UsedEffortEditor::UsedEffortEditor( QWidget *parent )
 {
     UsedEffortItemModel *m = new UsedEffortItemModel(this );
     setModel( m );
-    
+
     setItemDelegateForColumn ( 0, new EnumDelegate( this ) );
-    
+
     setItemDelegateForColumn ( 1, new DoubleSpinBoxDelegate( this ) );
     setItemDelegateForColumn ( 2, new DoubleSpinBoxDelegate( this ) );
     setItemDelegateForColumn ( 3, new DoubleSpinBoxDelegate( this ) );
@@ -349,9 +349,9 @@ UsedEffortEditor::UsedEffortEditor( QWidget *parent )
     setItemDelegateForColumn ( 5, new DoubleSpinBoxDelegate( this ) );
     setItemDelegateForColumn ( 6, new DoubleSpinBoxDelegate( this ) );
     setItemDelegateForColumn ( 7, new DoubleSpinBoxDelegate( this ) );
-    
+
     connect ( model(), SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ), SIGNAL( changed() ) );
-    
+
     connect ( m, SIGNAL( rowInserted( const QModelIndex& ) ), SIGNAL( resourceAdded() ) );
 }
 
@@ -389,6 +389,7 @@ void UsedEffortEditor::addResource()
 CompletionEntryItemModel::CompletionEntryItemModel ( QWidget *parent )
     : QAbstractItemModel( parent ),
     m_node( 0 ),
+    m_project( 0 ),
     m_manager( 0 ),
     m_completion( 0 )
 {
@@ -404,6 +405,15 @@ CompletionEntryItemModel::CompletionEntryItemModel ( QWidget *parent )
 
 }
 
+void CompletionEntryItemModel::setTask( Task *t )
+{
+    m_node = t;
+    m_project = 0;
+    if ( m_node && m_node->projectNode() ) {
+        m_project = static_cast<Project*>( m_node->projectNode() );
+    }
+}
+
 void CompletionEntryItemModel::slotDataChanged()
 {
     refresh();
@@ -426,8 +436,8 @@ Qt::ItemFlags CompletionEntryItemModel::flags ( const QModelIndex &index ) const
 QVariant CompletionEntryItemModel::date ( int row, int role ) const
 {
     switch ( role ) {
-        case Qt::DisplayRole: 
-        case Qt::EditRole: 
+        case Qt::DisplayRole:
+        case Qt::EditRole:
             return m_datelist.value( row );
         default: break;
     }
@@ -441,8 +451,8 @@ QVariant CompletionEntryItemModel::percentFinished ( int row, int role ) const
         return QVariant();
     }
     switch ( role ) {
-        case Qt::DisplayRole: 
-        case Qt::EditRole: 
+        case Qt::DisplayRole:
+        case Qt::EditRole:
             return e->percentFinished;
         default: break;
     }
@@ -464,10 +474,9 @@ QVariant CompletionEntryItemModel::remainingEffort ( int row, int role ) const
             return e->remainingEffort.toDouble( Duration::Unit_h );
         case Role::DurationScales: {
             QVariantList lst; // TODO: week
-            if ( m_node && m_node->projectNode() ) {
-                Project *p = static_cast<Project*>( m_node->projectNode() );
+            if ( m_node && m_project ) {
                 if ( m_node->estimate()->type() == Estimate::Type_Effort ) {
-                    lst.append( p->standardWorktime()->day() );
+                    lst.append( m_project->standardWorktime()->day() );
                 }
             }
             if ( lst.isEmpty() ) {
@@ -478,6 +487,10 @@ QVariant CompletionEntryItemModel::remainingEffort ( int row, int role ) const
         }
         case Role::DurationUnit:
             return static_cast<int>( Duration::Unit_h );
+        case Role::Minimum:
+            return m_project->config().minimumDurationUnit();
+        case Role::Maximum:
+            return m_project->config().maximumDurationUnit();
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
@@ -507,20 +520,23 @@ QVariant CompletionEntryItemModel::actualEffort ( int row, int role ) const
             return e->totalPerformed.toDouble( Duration::Unit_h );
         case Role::DurationScales: {
             QVariantList lst; // TODO: week
-            if ( m_node && m_node->projectNode() ) {
-                Project *p = static_cast<Project*>( m_node->projectNode() );
+            if ( m_node && m_project ) {
                 if ( m_node->estimate()->type() == Estimate::Type_Effort ) {
-                    lst.append( p->standardWorktime()->day() );
+                    lst.append( m_project->standardWorktime()->day() );
                 }
             }
             if ( lst.isEmpty() ) {
-                lst.append( 24.0 );
+                lst.append( 24 );
             }
-            lst << 60.0 << 60.0 << 1000.0;
+            lst << 60 << 60 << 1000;
             return lst;
         }
         case Role::DurationUnit:
             return static_cast<int>( Duration::Unit_h );
+        case Role::Minimum:
+            return m_project->config().minimumDurationUnit();
+        case Role::Maximum:
+            return m_project->config().maximumDurationUnit();
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
@@ -528,7 +544,7 @@ QVariant CompletionEntryItemModel::actualEffort ( int row, int role ) const
     return QVariant();
 }
 
-QVariant CompletionEntryItemModel::plannedEffort ( int row, int role ) const
+QVariant CompletionEntryItemModel::plannedEffort ( int /*row*/, int role ) const
 {
     if ( m_node == 0 || m_manager == 0 ) {
         return QVariant();
@@ -536,7 +552,7 @@ QVariant CompletionEntryItemModel::plannedEffort ( int row, int role ) const
     switch ( role ) {
         case Qt::DisplayRole:
         case Qt::ToolTipRole: {
-            Duration v = m_node->plannedEffort( m_manager->id() );
+            Duration v = m_node->plannedEffort( m_manager->scheduleId(), ECCT_EffortWork );
             //kDebug()<<m_node->name()<<": "<<v<<" "<<unit;
             return v.format();
         }
@@ -544,10 +560,9 @@ QVariant CompletionEntryItemModel::plannedEffort ( int row, int role ) const
             return QVariant();
         case Role::DurationScales: {
             QVariantList lst; // TODO: week
-            if ( m_node && m_node->projectNode() ) {
-                Project *p = static_cast<Project*>( m_node->projectNode() );
+            if ( m_node && m_project ) {
                 if ( m_node->estimate()->type() == Estimate::Type_Effort ) {
-                    lst.append( p->standardWorktime()->day() );
+                    lst.append( m_project->standardWorktime()->day() );
                 }
             }
             if ( lst.isEmpty() ) {
@@ -558,6 +573,10 @@ QVariant CompletionEntryItemModel::plannedEffort ( int row, int role ) const
         }
         case Role::DurationUnit:
             return static_cast<int>( Duration::Unit_h );
+        case Role::Minimum:
+            return m_project->config().minimumDurationUnit();
+        case Role::Maximum:
+            return m_project->config().maximumDurationUnit();
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
@@ -581,19 +600,17 @@ QVariant CompletionEntryItemModel::data ( const QModelIndex &index, int role ) c
     return QVariant();
 }
 
-QList<double> CompletionEntryItemModel::scales() const
+QList<qint64> CompletionEntryItemModel::scales() const
 {
-    QList<double> lst;
-    if ( m_node && m_node->projectNode() ) {
-        Project *p = static_cast<Project*>( m_node->projectNode() );
+    QList<qint64> lst;
+    if ( m_node && m_project ) {
         if ( m_node->estimate()->type() == Estimate::Type_Effort ) {
-            lst.append( p->standardWorktime()->day() );
+            lst = m_project->standardWorktime()->scales();
         }
     }
     if ( lst.isEmpty() ) {
-        lst.append( 24.0 );
+        lst = Estimate::defaultScales();
     }
-    lst << 60.0 << 60.0 << 1000.0;
     //kDebug()<<lst;
     return lst;
 
@@ -619,7 +636,7 @@ bool CompletionEntryItemModel::setData ( const QModelIndex &idx, const QVariant
                 e->percentFinished = value.toInt();
                 if ( m_completion->entrymode() == Completion::EnterCompleted && m_node ) {
                     // calculate used/remaining
-                    Duration est = m_node->plannedEffort( id() );
+                    Duration est = m_node->plannedEffort( id(), ECCT_EffortWork );
                     e->totalPerformed = est * e->percentFinished / 100;
                     e->remainingEffort = est - e->totalPerformed;
                 }
@@ -690,7 +707,7 @@ QVariant CompletionEntryItemModel::headerData ( int section, Qt::Orientation ori
     return QVariant();
 }
 
-int CompletionEntryItemModel::columnCount(const QModelIndex & parent ) const
+int CompletionEntryItemModel::columnCount(const QModelIndex & /*parent */) const
 {
     return 5;
 }
@@ -773,7 +790,7 @@ void CompletionEntryItemModel::addEntry( const QDate date )
     Completion::Entry *e = new Completion::Entry();
     if ( m_completion->entries().isEmpty() ) {
         if ( m_node ) {
-            e->remainingEffort = m_node->plannedEffort( id() );
+            e->remainingEffort = m_node->plannedEffort( id(), ECCT_EffortWork );
         }
     } else {
         e->percentFinished = m_completion->percentFinished();
@@ -796,11 +813,11 @@ CompletionEntryEditor::CompletionEntryEditor( QWidget *parent )
 {
     CompletionEntryItemModel *m = new CompletionEntryItemModel(this );
     setModel( m );
-    
+
     setItemDelegateForColumn ( 1, new ProgressBarDelegate( this ) );
     setItemDelegateForColumn ( 2, new DurationSpinBoxDelegate( this ) );
     setItemDelegateForColumn ( 3, new DurationSpinBoxDelegate( this ) );
-    
+
     connect ( m, SIGNAL( rowInserted( const QDate ) ), SIGNAL( rowInserted( const QDate ) ) );
     connect ( m, SIGNAL( rowRemoved( const QDate ) ), SIGNAL( rowRemoved( const QDate ) ) );
     connect ( model(), SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ), SIGNAL( changed() ) );
