@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-  Copyright (C) 2006 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2006 -2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -28,6 +28,7 @@
 
 #include <KoView.h>
 #include <KoPrintingDialog.h>
+#include <KoPageLayout.h>
 
 #include <QMap>
 #include <QTreeView>
@@ -43,6 +44,7 @@ class QAbstractItemModel;
 
 class KoDocument;
 class KoPrintJob;
+class KoPageLayoutWidget;
 
 /// The main namespace
 namespace KPlato
@@ -124,6 +126,9 @@ public:
     PrintingOptions printingOptions() const;
     void setPrintingOptions( const PrintingOptions &opt);
     
+    QWidget *createPageLayoutWidget() const;
+    QAbstractPrintDialog::PrintDialogOptions printDialogOptions() const;
+
 signals:
     void changed ( const PrintingOptions &opt );
     
@@ -135,9 +140,11 @@ protected:
     int headerFooterHeight( const PrintingOptions::Data &options ) const;
     void drawBottomRect( QPainter &p, const QRect &r );
 
+    void setPrinterPageLayout();
+
 protected:
     ViewBase *m_view;
-    QPointer<PrintingHeaderFooter> m_widget;
+    PrintingHeaderFooter *m_widget;
 };
 
 class KPLATOUI_EXPORT ViewActionLists
@@ -195,9 +202,9 @@ public:
     bool isActive() const;
     
     /// Set the project this view shall handle.
-    virtual void setProject( Project * /*project*/ ) {}
+    virtual void setProject( Project *project ) { m_proj = project; }
     /// Return the project
-    virtual Project *project() const { return 0; }
+    virtual Project *project() const { return m_proj; }
     /// Draw data from current part / project
     virtual void draw() {}
     /// Draw data from project.
@@ -225,8 +232,6 @@ public:
     virtual bool loadContext( const KoXmlElement &/*context*/ ) { return false; }
     /// Save context info from this view. Reimplement.
     virtual void saveContext( QDomElement &/*context*/ ) const {}
-    
-    virtual ViewBase *hitView( const QPoint &pos );
 
     virtual KoPrintJob *createPrintJob();
     PrintingOptions printingOptions() const { return m_printingOptions; }
@@ -234,16 +239,21 @@ public:
     
     void addAction( const QString list, QAction *action ) { ViewActionLists::addAction( list, action );  }
 
+    KoPageLayout pageLayout() const { return m_pagelayout; }
+
 public slots:
     /// Activate/deactivate the gui
     virtual void setGuiActive( bool activate );
-
+    virtual void setScheduleManager( ScheduleManager *sm ) { m_schedulemanager = sm; }
     void slotUpdateReadWrite( bool );
     virtual void slotHeaderContextMenuRequested( const QPoint &pos );
 
     virtual void slotEditCopy() {}
     virtual void slotEditCut() {}
     virtual void slotEditPaste() {}
+    virtual void slotRefreshView() {}
+    
+    void setPageLayout( const KoPageLayout &layout ) { m_pagelayout = layout; }
 
 signals:
     /// Emitted when the gui has been activated or deactivated
@@ -251,14 +261,23 @@ signals:
     /// Request for a context menu popup
     void requestPopupMenu( const QString&, const QPoint & );
     
+    /// Emitted when options are modified
+    void optionsModified();
+
 protected slots:
     virtual void slotOptions() {}
+    virtual void slotOptionsFinished( int result );
 
 protected:
     void createOptionAction();
     
     bool m_readWrite;
     PrintingOptions m_printingOptions;
+    
+    Project *m_proj;
+    ScheduleManager *m_schedulemanager;
+    
+    KoPageLayout m_pagelayout;
 };
 
 //------------------
@@ -272,6 +291,8 @@ public:
     virtual int documentFirstPage() const { return 1; }
     virtual int documentLastPage() const;
 
+    QList<QWidget*> createOptionWidgets() const;
+
 protected:
     virtual void printPage( int pageNumber, QPainter &painter );
 
@@ -416,6 +437,8 @@ public:
     virtual int documentFirstPage() const { return 1; }
     virtual int documentLastPage() const;
 
+    QList<QWidget*> createOptionWidgets() const;
+
 protected:
     virtual void printPage( int pageNumber, QPainter &painter );
     
@@ -529,7 +552,8 @@ signals:
 
 public slots:
     void edit( const QModelIndex &index );
-
+    void expandAll();
+    
 protected slots:
     void slotSelectionChanged( const QItemSelection &sel, const QItemSelection & );
     void slotToRightView( const QModelIndex &index );
