@@ -95,9 +95,9 @@ int KPlatoWork_Application::newInstance()
         return status;
     }
     QList<KMainWindow*> lst = KMainWindow::memberList();
-    qDebug()<<"newInstance() windows"<<lst.count();
+    kDebug()<<"windows"<<lst.count();
     if ( lst.count() > 1 ) {
-        qDebug()<<"newInstance() windows"<<lst.count();
+        kDebug()<<"windows"<<lst.count();
         return 1; // should never happen
     }
     if ( lst.isEmpty() ) {
@@ -111,7 +111,7 @@ int KPlatoWork_Application::newInstance()
     KCmdLineArgs *args= KCmdLineArgs::parsedArgs();
     int argsCount = args->count();
     if ( argsCount > 0 ) {
-        short int n=0; // number of documents open
+        //short int n=0; // number of documents open
         for(int i=0; i < argsCount; i++ ) {
             // For now create an empty document
             if ( ! m_mainwindow->openDocument( args->url(i) ) ) {
