@@ -95,7 +95,7 @@ void TaskWorkPackageModel::slotNodeToBeInserted( Node *parent, int row )
     beginInsertRows( indexForNode( parent ), row, row );
 }
 
-void TaskWorkPackageModel::slotNodeInserted( Node *node )
+void TaskWorkPackageModel::slotNodeInserted( Node */*node*/ )
 {
     //kDebug()<<node->parentNode()->name()<<"-->"<<node->name();
     endInsertRows();
@@ -108,7 +108,7 @@ void TaskWorkPackageModel::slotNodeToBeRemoved( Node *node )
     beginRemoveRows( indexForNode( node->parentNode() ), row, row );
 }
 
-void TaskWorkPackageModel::slotNodeRemoved( Node *node )
+void TaskWorkPackageModel::slotNodeRemoved( Node */*node*/ )
 {
     //kDebug()<<node->name();
     endRemoveRows();
@@ -219,15 +219,15 @@ QVariant TaskWorkPackageModel::projectManager( const Node *node, int role ) cons
 int TaskWorkPackageModel::rowCount( const QModelIndex &parent ) const
 {
     if ( ! parent.isValid() ) {
-        //qDebug()<<"TaskWorkPackageModel::rowCount:"<<parent<<"nodes:"<<m_part->workPackageCount();
+        //kDebug()<<parent<<"nodes:"<<m_part->workPackageCount();
         return m_part->workPackageCount(); // == no of nodes (1 node pr wp)
     }
     Node *n = nodeForIndex( parent );
     if ( n ) {
-        //qDebug()<<"TaskWorkPackageModel::rowCount:"<<parent<<"docs:"<<n->documents().count();
+        //kDebug()<<parent<<"docs:"<<n->documents().count();
         return n->documents().count();
     }
-    //qDebug()<<"TaskWorkPackageModel::rowCount:"<<parent<<"rows:"<<0;
+    //kDebug()<<parent<<"rows:"<<0;
     return 0; // documents have no children
 }
 
@@ -288,7 +288,7 @@ QVariant TaskWorkPackageModel::nodeData( Node *n, int column, int role ) const
 
 QVariant TaskWorkPackageModel::documentData( Document *doc, int column, int role ) const
 {
-    //qDebug()<<"TaskWorkPackageModel::documentData:"<<doc->url().fileName()<<column<<role;
+    //kDebug()<<doc->url().fileName()<<column<<role;
     if ( role != Qt::DisplayRole ) {
         return QVariant();
     }
@@ -310,7 +310,7 @@ bool TaskWorkPackageModel::setCompletion( Node *node, const QVariant &value, int
         QDateTime dt = QDateTime::currentDateTime();
         QDate date = dt.date();
         // xgettext: no-c-format
-        MacroCommand *m = new MacroCommand( i18n( "Modify % Completed" ) );
+        MacroCommand *m = new MacroCommand( i18n( "Modify completion" ) );
         if ( ! c.isStarted() ) {
             m->addCommand( new ModifyCompletionStartedCmd( c, true ) );
             m->addCommand( new ModifyCompletionStartTimeCmd( c, dt ) );
@@ -360,7 +360,7 @@ bool TaskWorkPackageModel::setRemainingEffort( Node *node, const QVariant &value
         double d( value.toList()[0].toDouble() );
         Duration::Unit unit = static_cast<Duration::Unit>( value.toList()[1].toInt() );
         Duration dur( d, unit );
-        emit executeCommand( new ModifyCompletionRemainingEffortCmd( t->completion(), QDate::currentDate(), dur, i18n( "Modify Remaining Effort" ) ) );
+        emit executeCommand( new ModifyCompletionRemainingEffortCmd( t->completion(), QDate::currentDate(), dur, i18n( "Modify remaining effort" ) ) );
         return true;
     }
     return false;
@@ -373,7 +373,7 @@ bool TaskWorkPackageModel::setActualEffort( Node *node, const QVariant &value, i
         double d( value.toList()[0].toDouble() );
         Duration::Unit unit = static_cast<Duration::Unit>( value.toList()[1].toInt() );
         Duration dur( d, unit );
-        emit executeCommand( new ModifyCompletionActualEffortCmd( t->completion(), QDate::currentDate(), dur, i18n( "Modify Actual Effort" ) ) );
+        emit executeCommand( new ModifyCompletionActualEffortCmd( t->completion(), QDate::currentDate(), dur, i18n( "Modify actual effort" ) ) );
         return true;
     }
     return false;
@@ -527,7 +527,7 @@ Node *TaskWorkPackageModel::nodeForIndex( const QModelIndex &index ) const
 {
     WorkPackage *wp = ptrToWorkPackage( index );
     if ( wp ) {
-        //qDebug()<<"TaskWorkPackageModel::nodeForIndex:"<<index<<parent->node()->name();
+        //kDebug()<<index<<parent->node()->name();
         return wp->node();
     }
     return 0;
@@ -538,7 +538,7 @@ Document *TaskWorkPackageModel::documentForIndex( const QModelIndex &index ) con
     if ( index.isValid() ) {
         Node *parent = ptrToNode( index );
         if ( parent && index.row() < parent->documents().count() ) {
-            //qDebug()<<"TaskWorkPackageModel::documentForIndex:"<<index<<parent->name();
+            //kDebug()<<index<<parent->name();
             return parent->documents().value( index.row() );
         }
     }
