@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
-   Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2006-2010 Thorsten Zachmann <zachmann@kde.org>
+  Copyright (C) 2010 Benjamin Port <port.benjamin@gmail.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -21,6 +22,7 @@
 
 
 #include "KPrView.h"
+#include "KPresenter.h"
 #include "KPrPage.h"
 #include "KPrMasterPage.h"
 #include "KPrShapeApplicationData.h"
@@ -28,12 +30,17 @@
 #include "KPrViewModeNotes.h"
 #include "KPrPlaceholderShapeFactory.h"
 #include "KPrSoundCollection.h"
+#include "KPrDeclarations.h"
+#include "KPrShapeManagerDisplayMasterStrategy.h"
+#include "KPrPageSelectStrategyActive.h"
 #include "pagelayout/KPrPageLayouts.h"
 #include "tools/KPrPlaceholderToolFactory.h"
 #include "commands/KPrSetCustomSlideShowsCommand.h"
 #include <KoPACanvas.h>
+#include <KoPACanvasItem.h>
 #include <KoPAViewModeNormal.h>
 #include <KoPASavingContext.h>
+#include <KoPALoadingContext.h>
 #include <KoShapeManager.h>
 #include <KoShapeLoadingContext.h>
 #include <KoShapeRegistry.h>
@@ -45,6 +52,10 @@
 #include <KConfigGroup>
 #include <KGlobal>
 #include <KStandardDirs>
+#include <KMessageBox>
+
+#include <QTimer>
+#include <QCoreApplication>
 
 class InitOnce
 {
@@ -63,6 +74,7 @@ KPrDocument::KPrDocument( QWidget* parentWidget, QObject* parent, bool singleVie
 , m_customSlideShows(new KPrCustomSlideShows())
 , m_presentationMonitor( 0 )
 , m_presenterViewEnabled( false )
+, m_declarations( new KPrDeclarations() )
 {
     K_GLOBAL_STATIC( InitOnce, s_initOnce );
     InitOnce * initOnce = s_initOnce;
@@ -81,10 +93,12 @@ KPrDocument::KPrDocument( QWidget* parentWidget, QObject* parent, bool singleVie
                                                        KoXmlNS::presentation, "class",
                                                        "presentation:class" ) );
 
-    KPrSoundCollection *soundCol = new KPrSoundCollection();
-    insertIntoDataCenterMap("SoundCollection", soundCol);
+    QVariant variant;
+    variant.setValue(new KPrSoundCollection(this));
+    resourceManager()->setResource(KPresenter::SoundCollection, variant);
 
-    insertIntoDataCenterMap( PageLayouts, new KPrPageLayouts() );
+    variant.setValue(new KPrPageLayouts(this));
+    resourceManager()->setResource(KPresenter::PageLayouts, variant);
 
     loadKPrConfig();
 }
@@ -100,11 +114,25 @@ KoView * KPrDocument::createViewInstance( QWidget *parent )
     return new KPrView( this, parent );
 }
 
+QGraphicsItem *KPrDocument::createCanvasItem()
+{
+    KoPACanvasItem *canvasItem = new KoPACanvasItem(this);
+    canvasItem->masterShapeManager()->setPaintingStrategy(new KPrShapeManagerDisplayMasterStrategy(canvasItem->masterShapeManager(),
+						  new KPrPageSelectStrategyActive(canvasItem)));
+    return canvasItem;
+}
+
 const char * KPrDocument::odfTagName( bool withNamespace )
 {
     return withNamespace ? "office:presentation": "presentation";
 }
 
+bool KPrDocument::saveOdfProlog( KoPASavingContext & context )
+{
+    m_declarations->saveOdf( context );
+    return true;
+}
+
 bool KPrDocument::saveOdfEpilogue( KoPASavingContext & context )
 {
     context.xmlWriter().startElement( "presentation:settings" );
@@ -119,7 +147,8 @@ bool KPrDocument::saveOdfEpilogue( KoPASavingContext & context )
 void KPrDocument::saveOdfDocumentStyles( KoPASavingContext & context )
 {
     KoPADocument::saveOdfDocumentStyles( context );
-    KPrPageLayouts * layouts = dynamic_cast<KPrPageLayouts *>( dataCenterMap().value( PageLayouts ) );
+    KPrPageLayouts *layouts = resourceManager()->resource(KPresenter::PageLayouts).value<KPrPageLayouts*>();
+
     Q_ASSERT( layouts );
     if ( layouts ) {
         layouts->saveOdf( context );
@@ -146,8 +175,7 @@ bool KPrDocument::loadOdfEpilogue( const KoXmlElement & body, KoPALoadingContext
 
 bool KPrDocument::loadOdfDocumentStyles( KoPALoadingContext & context )
 {
-    KPrPageLayouts * layouts = dynamic_cast<KPrPageLayouts *>( dataCenterMap().value( PageLayouts ) );
-    Q_ASSERT( layouts );
+    KPrPageLayouts *layouts = resourceManager()->resource(KPresenter::PageLayouts).value<KPrPageLayouts*>();
     if ( layouts ) {
         layouts->loadOdf( context );
     }
@@ -341,5 +369,48 @@ void KPrDocument::setActiveCustomSlideShow( const QString &customSlideShow )
     }
 }
 
+bool KPrDocument::loadOdfProlog( const KoXmlElement & body, KoPALoadingContext & context )
+{
+    return m_declarations->loadOdf( body, context );
+}
+
+KPrDeclarations * KPrDocument::declarations() const
+{
+    return m_declarations;
+}
+
+void KPrDocument::showStartUpWidget( KoMainWindow * parent, bool alwaysShow )
+{
+    // Go through all (optional) plugins we require and quit if necessary
+    bool error = false;
+    KoShapeFactoryBase * factory;
+
+    // TODO: Uncomment i18n calls after release of 2.3
+    factory = KoShapeRegistry::instance()->value( "TextShapeID" );
+    if ( !factory ) {
+        m_errorMessage = /*i18n(*/ "Can not find needed text component, KPresenter will quit now." /*)*/;
+        error = true;
+    }
+    factory = KoShapeRegistry::instance()->value( "PictureShape" );
+    if ( !factory ) {
+        m_errorMessage = /*i18n(*/ "Can not find needed picture component, KPresenter will quit now." /*)*/;
+        error = true;
+    }
+
+    if ( error ) {
+        QTimer::singleShot( 0, this, SLOT( showErrorAndDie() ) );
+    } else {
+        KoDocument::showStartUpWidget( parent, alwaysShow );
+    }
+}
+
+void KPrDocument::showErrorAndDie()
+{
+    KMessageBox::error( widget(), m_errorMessage, i18n( "Installation Error" ) );
+    // This means "the environment is incorrect" on Windows
+    // FIXME: Is this uniform on all platforms?
+    QCoreApplication::exit( 10 );
+}
+
 #include "KPrDocument.moc"
 
