@@ -25,6 +25,7 @@
 #include <KoPADocument.h>
 #include "KPrCustomSlideShows.h"
 
+class KPrDeclarations;
 class KPrShapeAnimation;
 class KPrShapeAnimations;
 
@@ -54,7 +55,7 @@ public:
      * @brief Remove animation from shape
      *
      * @param animation animation to remove from shape
-     * @param removeFromApplicationData if true the animation will also be removed from the 
+     * @param removeFromApplicationData if true the animation will also be removed from the
      *        application data
      */
     void removeAnimation( KPrShapeAnimation * animation, bool removeFromApplicationData = true );
@@ -123,6 +124,9 @@ public:
     /// reimplemented
     virtual bool loadOdfDocumentStyles( KoPALoadingContext & context );
 
+    /// reimplemented
+    virtual bool loadOdfProlog( const KoXmlElement & body, KoPALoadingContext & context );
+
     /**
      * Get the page type used in the document
      *
@@ -130,6 +134,19 @@ public:
      */
     virtual KoPageApp::PageType pageType() const;
 
+    /**
+     * Get the KPrDeclarations pointer
+     */
+    KPrDeclarations * declarations() const;
+
+    /**
+     * Creates and shows the start up widget. Reimplemented from KoDocument.
+     *
+     * @param parent the KoMainWindow used as parent for the widget.
+     * @param alwaysShow always show the widget even if the user has configured it to not show.
+     */
+    void showStartUpWidget( KoMainWindow * parent, bool alwaysShow );
+
 public slots:
     virtual void initEmpty();
 
@@ -152,11 +169,16 @@ protected:
     /// reimplemented
     virtual KoView * createViewInstance( QWidget *parent );
     /// reimplemented
+    virtual QGraphicsItem *createCanvasItem();
+    /// reimplemented
     virtual const char *odfTagName( bool withNamespace );
 
     /// reimplemented
     virtual bool loadOdfEpilogue( const KoXmlElement & body, KoPALoadingContext & context );
 
+    /// reimplemented
+    virtual bool saveOdfProlog( KoPASavingContext & paContext );
+
     /// reimplemented
     virtual bool saveOdfEpilogue( KoPASavingContext & context );
 
@@ -178,13 +200,20 @@ protected:
      * @brief get the animations of the page
      */
     KPrShapeAnimations & animationsByPage( KoPAPageBase * page );
-    
+
     KPrCustomSlideShows *m_customSlideShows;
 
+protected slots:
+    /// Quits KPresenter with error message from m_errorMessage.
+    void showErrorAndDie();
+
 private:
     int m_presentationMonitor;
     bool m_presenterViewEnabled;
     QString m_activeCustomSlideShow;
+    /// Message shown before KPresenter quits with an error if something is wrong
+    QString m_errorMessage;
+    KPrDeclarations *m_declarations;
 };
 
 #endif /* KPRDOCUMENT_H */
