@@ -20,20 +20,18 @@
 #ifndef KPRPLACEHOLDERSHAPEFACTORY_H
 #define KPRPLACEHOLDERSHAPEFACTORY_H
 
-#include "KoShapeFactory.h"
+#include "KoShapeFactoryBase.h"
 
 class KoShape;
 
-class KPrPlaceholderShapeFactory : public KoShapeFactory
+class KPrPlaceholderShapeFactory : public KoShapeFactoryBase
 {
 public:
     explicit KPrPlaceholderShapeFactory( QObject *parent );
     virtual ~KPrPlaceholderShapeFactory();
 
-    KoShape * createDefaultShape() const;
-    KoShape * createShape( const KoProperties * params ) const;
-    virtual bool supports( const KoXmlElement & e ) const;
-    virtual bool hidden() const;
+    virtual KoShape *createDefaultShape(KoResourceManager *documentResources = 0) const;
+    virtual bool supports( const KoXmlElement & e, KoShapeLoadingContext &context ) const;
 };
 
 #endif /* KPRPLACEHOLDERSHAPEFACTORY_H */
