@@ -20,7 +20,7 @@
 
 #include <KPrShapeApplicationData.h>
 
-#include "shapeanimations/KPrShapeAnimation.h"
+#include "animations/KPrShapeAnimation.h"
 
 KPrShapeApplicationData::KPrShapeApplicationData()
 {
