@@ -26,6 +26,7 @@
 #include <QtGui/QDesktopWidget>
 
 #include <kdebug.h>
+#include <kcursor.h>
 
 #include <KoPointerEvent.h>
 #include <KoCanvasController.h>
@@ -34,13 +35,13 @@
 #include <KoPADocument.h>
 #include <KoPAView.h>
 #include <KoZoomHandler.h>
+#include <KPrView.h>
 
 #include "KPrDocument.h"
-#include "KPrPresentationTool.h"
 #include "KPrPresenterViewWidget.h"
 #include "KPrEndOfSlideShowPage.h"
 
-KPrViewModePresentation::KPrViewModePresentation( KoPAView * view, KoPACanvas * canvas )
+KPrViewModePresentation::KPrViewModePresentation( KoPAViewBase * view, KoPACanvasBase * canvas )
 : KoPAViewMode( view, canvas )
 , m_savedParent( 0 )
 , m_tool( new KPrPresentationTool( *this ) )
@@ -49,7 +50,10 @@ KPrViewModePresentation::KPrViewModePresentation( KoPAView * view, KoPACanvas *
 , m_presenterViewCanvas( 0 )
 , m_presenterViewWidget( 0 )
 , m_endOfSlideShowPage( 0 )
+, m_view( static_cast<KPrView *>(view) )
 {
+    // TODO: make this viewmode work with non-QWidget-based canvases as well
+    m_baseCanvas = dynamic_cast<KoPACanvas*>(canvas);
 }
 
 KPrViewModePresentation::~KPrViewModePresentation()
@@ -58,9 +62,9 @@ KPrViewModePresentation::~KPrViewModePresentation()
     delete m_tool;
 }
 
-KoViewConverter * KPrViewModePresentation::viewConverter( KoPACanvas * canvas )
+KoViewConverter * KPrViewModePresentation::viewConverter( KoPACanvasBase * canvas )
 {
-    if ( m_animationDirector && m_canvas == canvas ) {
+    if ( m_baseCanvas && m_animationDirector && m_baseCanvas == canvas ) {
         return m_animationDirector->viewConverter();
     }
     else if ( m_pvAnimationDirector && m_presenterViewCanvas == canvas ) {
@@ -71,18 +75,19 @@ KoViewConverter * KPrViewModePresentation::viewConverter( KoPACanvas * canvas )
     }
 }
 
-void KPrViewModePresentation::paintEvent( KoPACanvas * canvas,  QPaintEvent* event )
+void KPrViewModePresentation::paint(KoPACanvasBase* canvas, QPainter& painter, const QRectF &paintRect)
 {
-    if ( m_canvas == canvas && m_animationDirector ) {
-        m_animationDirector->paintEvent( event );
+    if ( m_baseCanvas && m_baseCanvas == canvas && m_animationDirector ) {
+        m_animationDirector->paint( painter, paintRect);
     } else if ( m_presenterViewCanvas == canvas && m_pvAnimationDirector ) {
-        m_pvAnimationDirector->paintEvent( event );
+        m_pvAnimationDirector->paint( painter, paintRect );
     }
 }
 
 void KPrViewModePresentation::tabletEvent( QTabletEvent *event, const QPointF &point )
 {
-
+    Q_UNUSED(event);
+    Q_UNUSED(point);
 }
 
 void KPrViewModePresentation::mousePressEvent( QMouseEvent *event, const QPointF &point )
@@ -138,9 +143,11 @@ void KPrViewModePresentation::closeEvent( QCloseEvent * event )
 
 void KPrViewModePresentation::activate( KoPAViewMode * previousViewMode )
 {
+    if (!m_baseCanvas) return;
+
     m_savedViewMode = previousViewMode;               // store the previous view mode
-    m_savedParent = m_canvas->parentWidget();
-    m_canvas->setParent( ( QWidget* )0, Qt::Window ); // set parent to 0 and
+    m_savedParent = m_baseCanvas->parentWidget();
+    m_baseCanvas->setParent( ( QWidget* )0, Qt::Window ); // set parent to 0 and
 
     QDesktopWidget desktop;
 
@@ -155,30 +162,36 @@ void KPrViewModePresentation::activate( KoPAViewMode * previousViewMode )
 
     QRect presentationRect = desktop.screenGeometry( presentationscreen );
 
-    m_canvas->setWindowState( m_canvas->windowState() | Qt::WindowFullScreen ); // detach widget to make
-    m_canvas->show();
-    m_canvas->setFocus();                             // it shown full screen
+    m_baseCanvas->setParent(desktop.screen(presentationscreen), Qt::Window); // detach widget to the presentation screen
+    m_baseCanvas->setWindowState( m_baseCanvas->windowState() | Qt::WindowFullScreen ); // make it show full screen
 
     // the main animation director needs to be created first since it will set the active page
     // of the presentation
-    // the animation director needs to be set before m_canvas->move is called as this might try to call
+    // the animation director needs to be set before m_baseCanvas->move is called as this might try to call
     // viewConverter.
-    m_animationDirector = new KPrAnimationDirector( m_view, m_canvas, pages, m_view->activePage() );
+    m_animationDirector = new KPrAnimationDirector( m_view, m_baseCanvas, pages, m_view->activePage() );
     // move and resize now as otherwise it is not set when we call activate on the tool.
-    m_canvas->move( presentationRect.topLeft() );
-    m_canvas->resize( presentationRect.size() );
+    m_baseCanvas->move( presentationRect.topLeft() );
+    m_baseCanvas->resize( presentationRect.size() );
+    // show and setFocus needs to be done after move and resize as otherwise the move and resize have no effect
+    m_baseCanvas->show();
+    m_baseCanvas->setFocus();
+
+    KCursor::setAutoHideCursor( m_baseCanvas, true );
 
     if ( presenterViewEnabled ) {
+
         if ( desktop.numScreens() > 1 ) {
             int newscreen = desktop.numScreens() - presentationscreen - 1; // What if we have > 2 screens?
             QRect pvRect = desktop.screenGeometry( newscreen );
 
             m_presenterViewCanvas = new KoPACanvas( m_view, document );
             m_presenterViewWidget = new KPrPresenterViewWidget( this, pages, m_presenterViewCanvas );
-            m_presenterViewWidget->setParent( ( QWidget* )0, Qt::Window );
+            m_presenterViewWidget->setParent( desktop.screen(newscreen), Qt::Window );
             m_presenterViewWidget->setWindowState(
                     m_presenterViewWidget->windowState() | Qt::WindowFullScreen );
             m_presenterViewWidget->move( pvRect.topLeft() );
+            m_presenterViewWidget->resize( pvRect.size() );
             m_presenterViewWidget->updateWidget( pvRect.size(), presentationRect.size() );
             m_presenterViewWidget->show();
             m_presenterViewWidget->setFocus();                             // it shown full screen
@@ -192,7 +205,7 @@ void KPrViewModePresentation::activate( KoPAViewMode * previousViewMode )
         }
     }
 
-    m_tool->activate( false );
+    m_tool->activate(KoToolBase::DefaultActivation, QSet<KoShape*>());
 
     emit activated();
     emit pageChanged( m_animationDirector->currentPage(), m_animationDirector->numStepsInPage() );
@@ -203,6 +216,7 @@ void KPrViewModePresentation::deactivate()
 {
     emit deactivated();
 
+    m_animationDirector->deactivate();
     KoPAPageBase * page = m_view->activePage();
     if ( m_endOfSlideShowPage ) {
         if ( page == m_endOfSlideShowPage ) {
@@ -211,10 +225,14 @@ void KPrViewModePresentation::deactivate()
     }
     m_tool->deactivate();
 
-    m_canvas->setParent( m_savedParent, Qt::Widget );
-    m_canvas->setFocus();
-    m_canvas->setWindowState( m_canvas->windowState() & ~Qt::WindowFullScreen ); // reset
-    m_canvas->show();
+    if (!m_baseCanvas) return;
+
+    m_baseCanvas->setParent( m_savedParent, Qt::Widget );
+    m_baseCanvas->setFocus();
+    m_baseCanvas->setWindowState( m_baseCanvas->windowState() & ~Qt::WindowFullScreen ); // reset
+    m_baseCanvas->show();
+    KCursor::setAutoHideCursor( m_baseCanvas, false );
+    m_baseCanvas->setMouseTracking( true );
     m_view->setActivePage( page );
 
     // only delete after the new page has been set
@@ -283,6 +301,12 @@ int KPrViewModePresentation::currentStep() const
     return m_animationDirector ? m_animationDirector->currentStep() : -1;
 }
 
+KPrPresentationTool * KPrViewModePresentation::presentationTool() const
+{
+    Q_ASSERT( 0 != m_animationDirector );
+    return m_tool;
+}
+
 void KPrViewModePresentation::navigate( KPrAnimationDirector::Navigation navigation )
 {
     Q_ASSERT( 0 != m_animationDirector );
@@ -320,3 +344,8 @@ void KPrViewModePresentation::navigateToPage( int index )
     emit pageChanged( m_animationDirector->currentPage(), m_animationDirector->numStepsInPage() );
     emit stepChanged( m_animationDirector->currentStep() );
 }
+
+bool KPrViewModePresentation::isActivated()
+{
+    return m_view->isPresentationRunning();
+}
