@@ -115,7 +115,7 @@ void KPrPlaceholders::setLayout( KPrPageLayout * layout, KoPADocument * document
         QList<QRectF>::const_iterator listIt( list.begin() );
         for ( ; listIt != list.end(); ++listIt ) {
              KPrPlaceholderShape * shape = new KPrPlaceholderShape( itPlaceholder.key() );
-             shape->initStrategy( document->dataCenterMap() );
+             shape->initStrategy(document->resourceManager());
              shape->setAdditionalAttribute( "presentation:placeholder", "true" );
              shape->setAdditionalAttribute( "presentation:class", itPlaceholder.key() );
              shape->setSize( ( * listIt ).size() );
@@ -187,7 +187,7 @@ void KPrPlaceholders::add( const QList<KoShape *> & shapes )
         }
         KoShapeContainer* container = dynamic_cast<KoShapeContainer*>( shape );
         if ( container ) {
-            add( container->childShapes() );
+            add( container->shapes() );
         }
     }
 }
@@ -236,7 +236,6 @@ void KPrPlaceholders::applyStyle( KPrPlaceholderShape * shape, const QString & p
         QTextCursor newCursor( newData->document() );
         newCursor.select( QTextCursor::Document );
         KoTextDocument textDocument( newData->document() );
-        KoStyleManager * styleManager = textDocument.styleManager();
 
         QTextBlockFormat blockFormat( cursor.blockFormat() );
         // TODO check if needed
