@@ -34,7 +34,7 @@ class KRITAIMAGE_EXPORT KisPaintOpSettingsWidget : public KisConfigWidget
 public:
 
     KisPaintOpSettingsWidget(QWidget * parent = 0, Qt::WFlags f = 0)
-            : KisConfigWidget(parent, f) {
+            : KisConfigWidget(parent, f, 10) {
     }
 
     virtual ~KisPaintOpSettingsWidget() {
@@ -49,6 +49,30 @@ public:
     virtual void setImage(KisImageWSP image) {
         m_image = image;
     }
+    
+    /**
+     * XXX: Document!
+     */
+    virtual void changePaintOpSize(qreal x, qreal y) {
+        Q_UNUSED(x);
+        Q_UNUSED(y);
+    }
+
+    /**
+     * This is true for all of the paintop widget except for the Custom brush tab in the Brush tip dialog
+     */
+    virtual bool presetIsValid(){
+        return true;
+    }
+    
+    /**
+     * Some paintops are more complicated and require full canvas with layers, projections and KisImage etc.
+     * Example is duplicate paintop. In this case simple canvas like scratchbox does not work.
+     * Every paintop supports the scratchbox by default, override and return false if paintop does not.
+     */
+    virtual bool supportScratchBox(){
+        return true;
+    }
 
 protected:
 
