@@ -21,12 +21,11 @@
 
 #include "kis_image_change_layers_command.h"
 #include "kis_image_command.h"
-#include "kis_image_convert_type_command.h"
+#include "kis_image_set_projection_color_space_command.h"
 #include "kis_image_layer_add_command.h"
 #include "kis_image_layer_move_command.h"
 #include "kis_image_layer_remove_command.h"
 #include "kis_image_lock_command.h"
-#include "kis_image_props_command.h"
 #include "kis_image_resize_command.h"
 
 #endif
