@@ -18,21 +18,10 @@
  */
 
 #include "kis_image_commands.h"
-#include <QString>
-#include <QBitArray>
+#include "kis_image.h"
 
 #include <klocale.h>
 
-#include "KoColorSpaceRegistry.h"
-#include "KoColor.h"
-#include "KoColorProfile.h"
-#include "KoColorSpace.h"
-
-
-#include "kis_image.h"
-#include "kis_layer.h"
-#include "kis_group_layer.h"
-#include "kis_undo_adapter.h"
 
 KisImageLockCommand::KisImageLockCommand(KisImageWSP image, bool lockImage)
         : KisImageCommand("lock image", image)  // Not for translation, this is only ever used inside a macro command.
@@ -42,28 +31,21 @@ KisImageLockCommand::KisImageLockCommand(KisImageWSP image, bool lockImage)
 
 void KisImageLockCommand::redo()
 {
-    setUndo(false);
     if (m_lockImage) {
         m_image->lock();
     } else {
         m_image->unlock();
     }
-    m_image->rootLayer()->setDirty();
     m_image->refreshGraph();
-
-    setUndo(true);
 }
 
 void KisImageLockCommand::undo()
 {
-    setUndo(false);
     if (m_lockImage) {
         m_image->unlock();
     } else {
         m_image->lock();
     }
-    m_image->rootLayer()->setDirty();
     m_image->refreshGraph();
-    setUndo(true);
 }
 
