@@ -19,13 +19,11 @@
 
 #include "kis_image.h"
 
-#include <config-endian.h> // WORDS_BIGENDIAN
+#include <KoConfig.h> // WORDS_BIGENDIAN
 
 #include <stdlib.h>
 #include <math.h>
 
-#include <lcms.h>
-
 #include <QImage>
 #include <QPainter>
 #include <QSize>
@@ -39,12 +37,11 @@
 #include "KoColorSpaceRegistry.h"
 #include "KoColor.h"
 #include "KoColorConversionTransformation.h"
-#include "colorprofiles/KoIccColorProfile.h"
+#include "KoColorProfile.h"
 
 #include "recorder/kis_action_recorder.h"
 #include "kis_adjustment_layer.h"
 #include "kis_annotation.h"
-#include "kis_projection.h"
 #include "kis_background.h"
 #include "kis_change_profile_visitor.h"
 #include "kis_colorspace_convert_visitor.h"
@@ -57,7 +54,6 @@
 #include "kis_meta_data_merge_strategy_registry.h"
 #include "kis_name_server.h"
 #include "kis_paint_device.h"
-#include "kis_paint_device_action.h"
 #include "kis_paint_layer.h"
 #include "kis_painter.h"
 #include "kis_perspective_grid.h"
@@ -69,7 +65,25 @@
 #include "kis_crop_visitor.h"
 #include "kis_meta_data_merge_strategy.h"
 
-#include "kis_refresh_visitor.h"
+#include "kis_image_config.h"
+#include "kis_projection.h"
+#include "kis_update_scheduler.h"
+
+
+KisAbstractUpdateScheduler* createUpdateScheduler(KisImageWSP image)
+{
+    KisImageConfig config;
+
+    if(config.useUpdateScheduler()) {
+        dbgImage<<"Creating KisUpdateScheduler";
+        return new KisUpdateScheduler(image);
+    }
+    else {
+        dbgImage<<"Created KisProjection";
+        return new KisProjection(image);
+    }
+}
+
 
 class KisImage::KisImagePrivate
 {
@@ -101,11 +115,11 @@ public:
     KisSelectionSP globalSelection;
     KisSelectionSP deselectedGlobalSelection;
 
-    KisProjection* projection;
+    KisAbstractUpdateScheduler* projection;
 
     bool startProjection;
-
 };
+
 KisImage::KisImage(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name, bool startProjection)
         : QObject(0)
         , KisShared()
@@ -121,7 +135,7 @@ KisImage::KisImage(const KisImage& rhs)
         : QObject()
         , KisNodeFacade(rhs)
         , KisNodeGraphListener(rhs)
-        , KisShared(rhs)
+        , KisShared()
         , m_d(new KisImagePrivate())
 {
 
@@ -151,8 +165,7 @@ KisImage::KisImage(const KisImage& rhs)
 
         m_d->projection = 0;
         if (m_d->startProjection) {
-            m_d->projection = new KisProjection(this);
-            m_d->projection->start();
+            m_d->projection = createUpdateScheduler(this);
         }
     }
 }
@@ -160,16 +173,15 @@ KisImage::KisImage(const KisImage& rhs)
 KisImage::~KisImage()
 {
     dbgImage << "deleting kisimage" << objectName();
+
     if (m_d->projection) {
-        m_d->projection->stop();
-        m_d->projection->deleteLater();
+        delete m_d->projection;
     }
     delete m_d->perspectiveGrid;
     delete m_d->nserver;
     delete m_d;
 }
 
-
 void KisImage::aboutToAddANode(KisNode *parent, int index)
 {
     emit sigAboutToAddANode(parent, index);
@@ -177,16 +189,6 @@ void KisImage::aboutToAddANode(KisNode *parent, int index)
 
 void KisImage::nodeHasBeenAdded(KisNode *parent, int index)
 {
-    KisLayer * layer = dynamic_cast<KisLayer*>(parent->at(index).data());
-    if (layer) {
-        // The addition of temporary layers is not interesting
-        KoProperties props;
-        props.setProperty("temporary", false);
-
-        if (!layer->check(props)) {
-            emit sigLayerMoved(layer);
-        }
-    }
     emit sigNodeHasBeenAdded(parent, index);
 }
 
@@ -199,45 +201,22 @@ void KisImage::nodeHasBeenRemoved(KisNode *parent, int index)
 {
     // XXX: Temporarily for compatibility
     emit sigNodeHasBeenRemoved(parent, index);
-
-    KisLayer * l = dynamic_cast<KisLayer*>(parent->at(index).data());
-    if (l) {
-        // The removal of temporary layers is not interesting
-        KoProperties props;
-        props.setProperty("temporary", false);
-
-        if (!l->check(props)) {
-            emit sigLayerRemoved(l);
-        }
-    }
 }
 
 void KisImage::aboutToMoveNode(KisNode *parent, int oldIndex, int newIndex)
 {
-    Q_UNUSED(parent);
-    Q_UNUSED(oldIndex);
-    Q_UNUSED(newIndex);
+    emit sigAboutToMoveNode(parent, oldIndex, newIndex);
 }
 
 void KisImage::nodeHasBeenMoved(KisNode *parent, int oldIndex, int newIndex)
 {
-    Q_UNUSED(oldIndex);
-
-    KisLayer * l = dynamic_cast<KisLayer*>(parent->at(newIndex).data());
-    if (l) {
-
-        // The moving of temporary layers is not interesting
-
-        KoProperties props;
-        props.setProperty("temporary", false);
-
-        if (!l->check(props)) {
-
-            emit sigLayerMoved(l);
-        }
-    }
+    emit sigNodeHasBeenMoved(parent, oldIndex, newIndex);
 }
 
+void KisImage::nodeChanged(KisNode* node)
+{
+    emit sigNodeChanged(node);
+}
 
 KisSelectionSP KisImage::globalSelection() const
 {
@@ -247,7 +226,7 @@ KisSelectionSP KisImage::globalSelection() const
 void KisImage::setGlobalSelection(KisSelectionSP globalSelection)
 {
     if (globalSelection == 0)
-        m_d->globalSelection = new KisSelection(m_d->rootLayer->projection());
+        m_d->globalSelection = new KisSelection(m_d->rootLayer->projection(), new KisDefaultBounds(this));
     else
         m_d->globalSelection = globalSelection;
 }
@@ -257,7 +236,7 @@ void KisImage::removeGlobalSelection()
     m_d->globalSelection = 0;
 }
 
-KisSelectionSP KisImage::deleselectedGlobalSelection()
+KisSelectionSP KisImage::deselectedGlobalSelection()
 {
     return m_d->deselectedGlobalSelection;
 }
@@ -311,7 +290,7 @@ void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const
 
     m_d->colorSpace = colorSpace;
 
-    setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE));
+    setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE_U8));
 
     m_d->xres = 1.0;
     m_d->yres = 1.0;
@@ -319,12 +298,11 @@ void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const
     m_d->width = width;
     m_d->height = height;
 
-    m_d->recorder = new KisActionRecorder();
+    m_d->recorder = new KisActionRecorder(this);
 
     m_d->projection = 0;
     if (m_d->startProjection) {
-        m_d->projection = new KisProjection(this);
-        m_d->projection->start();
+        m_d->projection = createUpdateScheduler(this);
     }
 }
 
@@ -335,6 +313,7 @@ bool KisImage::locked() const
 
 void KisImage::lock()
 {
+//  blockSignals(true);
     if (!locked()) {
         if (m_d->projection) {
             m_d->projection->lock();
@@ -360,6 +339,7 @@ void KisImage::unlock()
                 m_d->projection->unlock();
             }
         }
+//      blockSignals(false);
     }
 }
 
@@ -375,71 +355,42 @@ void KisImage::notifyLayerUpdated(KisLayerSP layer)
     }
 }
 
-
-void KisImage::resize(const QRect& rc, bool cropLayers)
+void KisImage::setSize(const QSize& size)
 {
-    resize(rc.width(), rc.height(), rc.x(), rc.y(), cropLayers);
+    m_d->width = size.width();
+    m_d->height = size.height();
+    emitSizeChanged();
+}
+
+void KisImage::resize(const QRect& newRect, bool cropLayers)
+{
+    if(newRect == bounds())
+        return;
+
+    QString macroName = cropLayers ? i18n("Crop Image") : i18n("Resize Image");
+    m_d->adapter->beginMacro(macroName);
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+    m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), newRect.size()));
+
+    if(cropLayers) {
+        KisCropVisitor visitor(newRect, m_d->adapter);
+        m_d->rootLayer->accept(visitor);
+    }
+
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
+    m_d->adapter->endMacro();
+
+    setModified();
 }
 
 void KisImage::resize(qint32 w, qint32 h, qint32 x, qint32 y, bool cropLayers)
 {
-    if (w != width() || h != height()) {
-
-        lock();
-
-        if (undo()) {
-            if (cropLayers)
-                m_d->adapter->beginMacro(i18n("Crop Image"));
-            else
-                m_d->adapter->beginMacro(i18n("Resize Image"));
-
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
-            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
-        }
-
-        m_d->width = w;
-        m_d->height = h;
-
-        if (cropLayers) {
-            KisCropVisitor v(QRect(x, y, w, h), m_d->adapter);
-            m_d->rootLayer->accept(v);
-        }
-
-        emitSizeChanged();
-
-        unlock();
-
-        if (undo()) {
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
-            m_d->adapter->endMacro();
-        }
-    }
+    resize(QRect(x, y, w, h), cropLayers);
 }
 
 void KisImage::resizeWithOffset(qint32 w, qint32 h, qint32 xOffset, qint32 yOffset)
 {
-    if (w == width() && h == height() && xOffset == 0 && yOffset == 0)
-        return;
-
-    lock();
-    if (undo()) {
-        m_d->adapter->beginMacro(i18n("Size Canvas"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
-        m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
-    }
-
-    KisCropVisitor v(QRect(-xOffset, -yOffset, w, h), m_d->adapter);
-    m_d->rootLayer->accept(v);
-
-    emitSizeChanged();
-
-    unlock();
-
-    if (undo()) {
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
-        m_d->adapter->endMacro();
-    }
-
+    resize(QRect(-xOffset, -yOffset, w, h), true);
 }
 
 void KisImage::emitSizeChanged()
@@ -452,56 +403,33 @@ void KisImage::emitSizeChanged()
 }
 
 
-void KisImage::scale(double sx, double sy, KoUpdater *progress, KisFilterStrategy *filterStrategy)
+void KisImage::scale(double sx, double sy, KoUpdater *progress, KisFilterStrategy *filterStrategy, bool scaleOnlyShapes)
 {
-    QStringList list;
-    list << "KisLayer";
-
-    KisCountVisitor visitor(list, KoProperties());
-    m_d->rootLayer->accept(visitor);
-
-    if (visitor.count() == 0) return; // Nothing to scale
-
     // New image size. XXX: Pass along to discourage rounding errors?
     qint32 w, h;
     w = (qint32)((width() * sx) + 0.5);
     h = (qint32)((height() * sy) + 0.5);
 
-    if (w != width() || h != height()) {
+    QSize newSize(w, h);
+    if(newSize == size()) return;
 
-        lock();
-
-        if (undo()) {
-            m_d->adapter->beginMacro(i18n("Scale Image"));
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
-        }
-
-        {
-            KisTransformVisitor visitor(KisImageWSP(this), sx, sy, 0.0, 0.0, 0.0, 0, 0, progress, filterStrategy);
-            m_d->rootLayer->accept(visitor);
-        }
-
-        if (undo()) {
-            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
-        }
-
-        m_d->width = w;
-        m_d->height = h;
-
-        emitSizeChanged();
-
-        unlock();
-
-        if (undo()) {
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
-            m_d->adapter->endMacro();
-        }
+    m_d->adapter->beginMacro(i18n("Scale Image"));
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+    
+    if(!scaleOnlyShapes) {
+        m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), newSize));
     }
+    
+    KisTransformVisitor visitor(KisImageWSP(this), sx, sy, 0.0, 0.0, 0.0, 0, 0, progress, filterStrategy, scaleOnlyShapes);
+    m_d->rootLayer->accept(visitor);
+
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
+    m_d->adapter->endMacro();
+
+    setModified();
 }
 void KisImage::rotate(double radians, KoUpdater *progress)
 {
-    lock();
-
     qint32 w = width();
     qint32 h = height();
     qint32 tx = qint32((w * cos(radians) - h * sin(radians) - w) / 2 + 0.5);
@@ -512,28 +440,19 @@ void KisImage::rotate(double radians, KoUpdater *progress)
     tx -= (w - width()) / 2;
     ty -= (h - height()) / 2;
 
-    if (undo()) {
-        m_d->adapter->beginMacro(i18n("Rotate Image"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
-    }
+    m_d->adapter->beginMacro(i18n("Rotate Image"));
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+    m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), QSize(w,h)));
 
     KisFilterStrategy *filter = KisFilterStrategyRegistry::instance()->value("Triangle");
+
     KisTransformVisitor visitor(KisImageWSP(this), 1.0, 1.0, 0, 0, radians, -tx, -ty, progress, filter);
     m_d->rootLayer->accept(visitor);
 
-    if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
+    m_d->adapter->endMacro();
 
-    m_d->width = w;
-    m_d->height = h;
-
-    emitSizeChanged();
-
-    unlock();
-
-    if (undo()) {
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
-        m_d->adapter->endMacro();
-    }
+    setModified();
 }
 
 void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
@@ -560,64 +479,84 @@ void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
             h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)));
     }
 
-    if (w != width() || h != height()) {
+    QSize newSize(w, h);
+    if(newSize == size()) return;
 
-        lock();
+    m_d->adapter->beginMacro(i18n("Shear Image"));
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+    m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), newSize));
 
-        if (undo()) {
-            m_d->adapter->beginMacro(i18n("Shear Image"));
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
-        }
+    KisShearVisitor visitor(angleX, angleY, progress);
+    visitor.setUndoAdapter(m_d->adapter);
+    rootLayer()->accept(visitor);
 
-        KisShearVisitor v(angleX, angleY, progress);
-        v.setUndoAdapter(m_d->adapter);
-        rootLayer()->accept(v);
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
+    m_d->adapter->endMacro();
 
-        if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
-
-        m_d->width = w;
-        m_d->height = h;
-
-        emitSizeChanged();
-
-        unlock();
-
-        if (undo()) {
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
-            m_d->adapter->endMacro();
-        }
-    }
+    setModified();
 }
 
-void KisImage::convertTo(const KoColorSpace *dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
+void KisImage::convertImageColorSpace(const KoColorSpace *dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
 {
-    if (*m_d->colorSpace == *dstColorSpace) {
-        return;
-    }
+    if (*m_d->colorSpace == *dstColorSpace) return;
 
-    lock();
-
-    const KoColorSpace * oldCs = m_d->colorSpace;
-
-    if (undo()) {
-        m_d->adapter->beginMacro(i18n("Convert Image Type"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
-    }
+    m_d->adapter->beginMacro(i18n("Convert Image Color Space"));
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+    m_d->adapter->addCommand(new KisImageSetProjectionColorSpaceCommand(KisImageWSP(this), dstColorSpace));
 
     KisColorSpaceConvertVisitor visitor(this, dstColorSpace, renderingIntent);
     m_d->rootLayer->accept(visitor);
 
-    if (undo()) {
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
+    m_d->adapter->endMacro();
 
-        m_d->adapter->addCommand(new KisImageConvertTypeCommand(KisImageWSP(this), oldCs, dstColorSpace));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
-        m_d->adapter->endMacro();
-    } else {
-        setColorSpace(dstColorSpace);
-    }
+    setModified();
+}
 
-    unlock();
-    refreshGraph();
+void KisImage::assignImageProfile(const KoColorProfile *profile)
+{
+    if(!profile) return;
+
+    m_d->adapter->beginMacro(i18n("Assign Profile"));
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+
+    const KoColorSpace *dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->colorModelId().id(), colorSpace()->colorDepthId().id(), profile);
+    const KoColorSpace *srcCs = colorSpace();
+
+    KisChangeProfileVisitor visitor(srcCs, dstCs);
+    m_d->rootLayer->accept(visitor);
+
+    m_d->adapter->addCommand(new KisImageSetProjectionColorSpaceCommand(KisImageWSP(this), dstCs));
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
+    m_d->adapter->endMacro();
+
+    setModified();
+    emit sigProfileChanged(profile);
+}
+
+void KisImage::convertProjectionColorSpace(const KoColorSpace *dstColorSpace)
+{
+    if (*m_d->colorSpace == *dstColorSpace) return;
+
+    m_d->adapter->beginMacro(i18n("Convert Projection Color Space"));
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+    m_d->adapter->addCommand(new KisImageSetProjectionColorSpaceCommand(KisImageWSP(this), dstColorSpace));
+    m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
+    m_d->adapter->endMacro();
+
+    setModified();
+}
+
+void KisImage::setProjectionColorSpace(const KoColorSpace * colorSpace)
+{
+    m_d->colorSpace = colorSpace;
+    m_d->rootLayer->resetCache();
+    emit sigColorSpaceChanged(colorSpace);
+}
+
+const KoColorSpace * KisImage::colorSpace() const
+{
+    return m_d->colorSpace;
 }
 
 const KoColorProfile * KisImage::profile() const
@@ -625,26 +564,6 @@ const KoColorProfile * KisImage::profile() const
     return colorSpace()->profile();
 }
 
-void KisImage::setProfile(const KoColorProfile *profile)
-{
-    if (profile == 0) return;
-
-    const KoColorSpace *dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->id(), profile);
-    if (dstCs) {
-
-        lock();
-
-        const KoColorSpace *oldCs = colorSpace();
-        setColorSpace(dstCs);
-        emit(sigProfileChanged(const_cast<KoColorProfile*>(profile)));
-
-        KisChangeProfileVisitor visitor(oldCs, dstCs);
-        m_d->rootLayer->accept(visitor);
-
-        unlock();
-    }
-}
-
 double KisImage::xRes() const
 {
     return m_d->xres;
@@ -744,109 +663,139 @@ qint32 KisImage::nHiddenLayers() const
     return visitor.count();
 }
 
-void KisImage::flatten()
+QRect KisImage::realNodeExtent(KisNodeSP rootNode, QRect currentRect)
 {
-    KisGroupLayerSP oldRootLayer = m_d->rootLayer;
+    KisNodeSP node = rootNode->firstChild();
 
-    KisPaintLayer *dst = new KisPaintLayer(this, nextLayerName(), OPACITY_OPAQUE, colorSpace());
-    Q_CHECK_PTR(dst);
-
-    QRect rc = mergedImage()->extent();
-
-    KisPainter gc(dst->paintDevice());
-    gc.setCompositeOp(COMPOSITE_OVER);
-    gc.bitBlt(rc.x(), rc.y(), mergedImage(), rc.left(), rc.top(), rc.width(), rc.height());
-
-    setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE));
-
-    if (undo()) {
-        m_d->adapter->beginMacro(i18n("Flatten Image"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
-        m_d->adapter->addCommand(new KisImageChangeLayersCommand(KisImageWSP(this), oldRootLayer, m_d->rootLayer, ""));
+    while(node) {
+        currentRect |= realNodeExtent(node, currentRect);
+        node = node->nextSibling();
     }
 
-    lock();
+    // TODO: it would be better to count up changeRect inside
+    // node's extent() method
+    currentRect |= rootNode->changeRect(rootNode->extent());
 
-    addNode(dst, m_d->rootLayer.data(), 0);
+    return currentRect;
+}
 
-    unlock();
+void KisImage::refreshHiddenArea(KisNodeSP rootNode, const QRect &preparedArea)
+{
+    QRect realNodeRect = realNodeExtent(rootNode);
+    if(!preparedArea.contains(realNodeRect)) {
 
-    notifyLayersChanged();
+        QRegion dirtyRegion = realNodeRect;
+        dirtyRegion -= preparedArea;
 
-    if (undo()) {
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
-        m_d->adapter->endMacro();
+        foreach(const QRect &rc, dirtyRegion.rects()) {
+            refreshGraph(rootNode, rc, realNodeRect);
+        }
     }
 }
 
+void KisImage::flatten()
+{
+    KisGroupLayerSP oldRootLayer = m_d->rootLayer;
+    KisGroupLayerSP newRootLayer =
+        new KisGroupLayer(this, "root", OPACITY_OPAQUE_U8);
 
+    refreshHiddenArea(oldRootLayer, bounds());
+
+    lock();
+    KisPaintDeviceSP projectionCopy =
+        new KisPaintDevice(*oldRootLayer->projection());
+    unlock();
+
+    KisPaintLayerSP flattenLayer =
+        new KisPaintLayer(this, nextLayerName(), OPACITY_OPAQUE_U8, projectionCopy);
+    Q_CHECK_PTR(flattenLayer);
+
+    addNode(flattenLayer, newRootLayer, 0);
+
+    m_d->adapter->beginMacro(i18n("Flatten Image"));
+    // NOTE: KisImageChangeLayersCommand performs all the locking for us
+    m_d->adapter->addCommand(new KisImageChangeLayersCommand(KisImageWSP(this), oldRootLayer, newRootLayer, ""));
+    m_d->adapter->endMacro();
+
+    setModified();
+}
+
+// FIXME: Rename to Merge Down?
 KisLayerSP KisImage::mergeLayer(KisLayerSP layer, const KisMetaData::MergeStrategy* strategy)
 {
-    if (!layer->prevSibling()) return 0;
+    if(!layer->prevSibling()) return 0;
+
     // XXX: this breaks if we allow free mixing of masks and layers
-    KisLayerSP layer2 = dynamic_cast<KisLayer*>(layer->prevSibling().data());
-    dbgImage << "Merge " << layer << " with " << layer2;
-    if (!layer2) return 0;
+    KisLayerSP prevLayer = dynamic_cast<KisLayer*>(layer->prevSibling().data());
+    if (!prevLayer) return 0;
 
-    KisPaintLayerSP newLayer = new KisPaintLayer(this, layer->name(), OPACITY_OPAQUE, colorSpace());
-    Q_CHECK_PTR(newLayer);
 
-    QRect layerExtent = layer->extent();
-    QRect layerPrevSiblingExtent = layer->prevSibling()->extent();
+    refreshHiddenArea(layer, bounds());
+    refreshHiddenArea(prevLayer, bounds());
 
-    QRect rc = layerExtent | layerPrevSiblingExtent;
+    QRect layerProjectionExtent = layer->projection()->extent();
+    QRect prevLayerProjectionExtent = prevLayer->projection()->extent();
 
-    undoAdapter()->beginMacro(i18n("Merge with Layer Below"));
+    lock();
+    KisPaintDeviceSP mergedDevice = new KisPaintDevice(*prevLayer->projection());
+    unlock();
 
-    KisPainter gc(newLayer->paintDevice());
-    gc.setCompositeOp(newLayer->colorSpace()->compositeOp(COMPOSITE_COPY));
-    gc.bitBlt(rc.topLeft(), layer2->projection(), rc);
-
-    gc.setCompositeOp(layer->compositeOp());
+    KisPainter gc(mergedDevice);
+    gc.setChannelFlags(layer->channelFlags());
+    gc.setCompositeOp(mergedDevice->colorSpace()->compositeOp(layer->compositeOpId()));
     gc.setOpacity(layer->opacity());
-    gc.bitBlt(rc.topLeft(), layer->projection(), rc);
+    gc.bitBlt(layerProjectionExtent.topLeft(), layer->projection(), layerProjectionExtent);
+
+    KisPaintLayerSP mergedLayer = new KisPaintLayer(this, prevLayer->name(), OPACITY_OPAQUE_U8, mergedDevice);
+    Q_CHECK_PTR(mergedLayer);
+
 
     // Merge meta data
     QList<const KisMetaData::Store*> srcs;
-    srcs.append(static_cast<KisLayer*>(layer->prevSibling().data())->metaData());
+    srcs.append(prevLayer->metaData());
     srcs.append(layer->metaData());
     QList<double> scores;
-    int layerPrevSiblingArea = layerPrevSiblingExtent.width() * layerPrevSiblingExtent.height();
-    int layerArea = layerExtent.width() * layerExtent.height();
-    double norm = qMax(layerPrevSiblingArea, layerArea);
-    scores.append(layerPrevSiblingArea / norm);
+    int prevLayerArea = prevLayerProjectionExtent.width() * prevLayerProjectionExtent.height();
+    int layerArea = layerProjectionExtent.width() * layerProjectionExtent.height();
+    double norm = qMax(prevLayerArea, layerArea);
+    scores.append(prevLayerArea / norm);
     scores.append(layerArea / norm);
-    strategy->merge(newLayer->metaData(), srcs, scores);
+    strategy->merge(mergedLayer->metaData(), srcs, scores);
 
     KisNodeSP parent = layer->parent(); // parent is set to null when the layer is removed from the node
     dbgImage << ppVar(parent);
 
     // XXX: merge the masks!
+    // AAA: do you really think you need it? ;)
 
-    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, newLayer, parent, layer));
-    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer->prevSibling()));
+    // FIXME: "Merge Down"?
+    undoAdapter()->beginMacro(i18n("Merge with Layer Below"));
+
+    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, mergedLayer, parent, layer));
+    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, prevLayer));
     undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer));
 
     undoAdapter()->endMacro();
 
-    return newLayer;
+    return mergedLayer;
 }
 
 KisLayerSP KisImage::flattenLayer(KisLayerSP layer)
 {
     if (!layer->firstChild()) return layer;
 
-    undoAdapter()->beginMacro(i18n("Flatten Layer"));
-    KisPaintLayerSP newLayer = new KisPaintLayer(this, layer->name(), layer->opacity(), colorSpace());
-    newLayer->setCompositeOp(layer->compositeOp()->id());
-    newLayer->metaData();
-    QRect rc = layer->extent();
+    refreshHiddenArea(layer, bounds());
 
-    KisPainter gc(newLayer->paintDevice());
-    gc.setCompositeOp(newLayer->colorSpace()->compositeOp(COMPOSITE_COPY));
-    gc.bitBlt(rc.topLeft(), layer->projection(), rc);
-    gc.end();
-    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, newLayer, layer->parent(), layer->nextSibling()));
+    lock();
+    KisPaintDeviceSP mergedDevice = new KisPaintDevice(*layer->projection());
+    unlock();
+
+    KisPaintLayerSP newLayer = new KisPaintLayer(this, layer->name(), layer->opacity(), mergedDevice);
+    newLayer->setCompositeOp(layer->compositeOp()->id());
+
+
+    undoAdapter()->beginMacro(i18n("Flatten Layer"));
+    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, newLayer, layer->parent(), layer));
 
     KisNodeSP node = layer->firstChild();
     while (node) {
@@ -884,8 +833,19 @@ void KisImage::renderToPainter(qint32 srcX,
                                QPainter &painter,
                                const KoColorProfile *  monitorProfile)
 {
-    QImage img = convertToQImage(srcX, srcY, width, height, monitorProfile);
-    painter.drawImage(dstX, dstY, img, 0, 0, width, height);
+    QImage image = convertToQImage(srcX, srcY, width, height, monitorProfile);
+    painter.drawImage(dstX, dstY, image, 0, 0, width, height);
+}
+
+QImage KisImage::convertToQImage(QRect imageRect,
+                                 const KoColorProfile * profile)
+{
+    qint32 x;
+    qint32 y;
+    qint32 w;
+    qint32 h;
+    imageRect.getRect(&x, &y, &w, &h);
+    return convertToQImage(x, y, w, h, profile);
 }
 
 QImage KisImage::convertToQImage(qint32 x,
@@ -896,14 +856,14 @@ QImage KisImage::convertToQImage(qint32 x,
 {
     KisPaintDeviceSP dev = m_d->rootLayer->projection();
     if (!dev) return QImage();
-    QImage img = dev->convertToQImage(const_cast<KoColorProfile*>(profile), x, y, w, h);
+    QImage image = dev->convertToQImage(const_cast<KoColorProfile*>(profile), x, y, w, h);
 
     if (m_d->backgroundPattern) {
-        m_d->backgroundPattern->paintBackground(img, QRect(x, y, w, h));
+        m_d->backgroundPattern->paintBackground(image, QRect(x, y, w, h));
     }
-    if (!img.isNull()) {
+    if (!image.isNull()) {
 #ifdef WORDS_BIGENDIAN
-        uchar * data = img.bits();
+        uchar * data = image.bits();
         for (int i = 0; i < w * h; ++i) {
             uchar r, g, b, a;
             a = data[0];
@@ -918,7 +878,7 @@ QImage KisImage::convertToQImage(qint32 x,
         }
 #endif
 
-        return img;
+        return image;
     }
 
     return QImage();
@@ -926,10 +886,10 @@ QImage KisImage::convertToQImage(qint32 x,
 
 
 
-QImage KisImage::convertToQImage(const QRect& r, const QSize& scaledImageSize, const KoColorProfile *profile)
+QImage KisImage::convertToQImage(const QRect& scaledRect, const QSize& scaledImageSize, const KoColorProfile *profile)
 {
 
-    if (r.isEmpty() || scaledImageSize.isEmpty()) {
+    if (scaledRect.isEmpty() || scaledImageSize.isEmpty()) {
         return QImage();
     }
 
@@ -942,30 +902,27 @@ QImage KisImage::convertToQImage(const QRect& r, const QSize& scaledImageSize, c
 
     QRect srcRect;
 
-    srcRect.setLeft(static_cast<int>(r.left() * xScale));
-    srcRect.setRight(static_cast<int>(ceil((r.right() + 1) * xScale)) - 1);
-    srcRect.setTop(static_cast<int>(r.top() * yScale));
-    srcRect.setBottom(static_cast<int>(ceil((r.bottom() + 1) * yScale)) - 1);
+    srcRect.setLeft(static_cast<int>(scaledRect.left() * xScale));
+    srcRect.setRight(static_cast<int>(ceil((scaledRect.right() + 1) * xScale)) - 1);
+    srcRect.setTop(static_cast<int>(scaledRect.top() * yScale));
+    srcRect.setBottom(static_cast<int>(ceil((scaledRect.bottom() + 1) * yScale)) - 1);
 
     KisPaintDeviceSP mergedImage = m_d->rootLayer->projection();
-    QTime t;
-    t.start();
-
-    quint8 *scaledImageData = new quint8[r.width() * r.height() * pixelSize];
+    quint8 *scaledImageData = new quint8[scaledRect.width() * scaledRect.height() * pixelSize];
 
     quint8 *imageRow = new quint8[srcRect.width() * pixelSize];
     const qint32 imageRowX = srcRect.x();
 
-    for (qint32 y = 0; y < r.height(); ++y) {
+    for (qint32 y = 0; y < scaledRect.height(); ++y) {
 
-        qint32 dstY = r.y() + y;
-        qint32 dstX = r.x();
+        qint32 dstY = scaledRect.y() + y;
+        qint32 dstX = scaledRect.x();
         qint32 srcY = (dstY * imageHeight) / scaledImageSize.height();
 
         mergedImage->readBytes(imageRow, imageRowX, srcY, srcRect.width(), 1);
 
-        quint8 *dstPixel = scaledImageData + (y * r.width() * pixelSize);
-        quint32 columnsRemaining = r.width();
+        quint8 *dstPixel = scaledImageData + (y * scaledRect.width() * pixelSize);
+        quint32 columnsRemaining = scaledRect.width();
 
         while (columnsRemaining > 0) {
 
@@ -978,14 +935,12 @@ QImage KisImage::convertToQImage(const QRect& r, const QSize& scaledImageSize, c
             --columnsRemaining;
         }
     }
-    dbgRender << "Time elapsed scaling image: " << t.elapsed() << endl;
-
     delete [] imageRow;
 
-    QImage image = colorSpace()->convertToQImage(scaledImageData, r.width(), r.height(), const_cast<KoColorProfile*>(profile), KoColorConversionTransformation::IntentPerceptual);
+    QImage image = colorSpace()->convertToQImage(scaledImageData, scaledRect.width(), scaledRect.height(), const_cast<KoColorProfile*>(profile), KoColorConversionTransformation::IntentPerceptual);
 
     if (m_d->backgroundPattern) {
-        m_d->backgroundPattern->paintBackground(image, r, scaledImageSize, QSize(imageWidth, imageHeight));
+        m_d->backgroundPattern->paintBackground(image, scaledRect, scaledImageSize, QSize(imageWidth, imageHeight));
     }
 
     delete [] scaledImageData;
@@ -1049,25 +1004,11 @@ KisActionRecorder* KisImage::actionRecorder() const
     return m_d->recorder;
 }
 
-bool KisImage::undo() const
-{
-    return (m_d->adapter && m_d->adapter->undo());
-}
-
-const KoColorSpace * KisImage::colorSpace() const
-{
-    return m_d->colorSpace;
-}
-
-void KisImage::setColorSpace(const KoColorSpace * colorSpace)
-{
-    m_d->colorSpace = colorSpace;
-    m_d->rootLayer->resetCache();
-    emit sigColorSpaceChanged(colorSpace);
-}
-
 void KisImage::setRootLayer(KisGroupLayerSP rootLayer)
 {
+    if(m_d->rootLayer)
+        m_d->rootLayer->disconnect();
+
     m_d->rootLayer = rootLayer;
     m_d->rootLayer->disconnect();
     m_d->rootLayer->setGraphListener(this);
@@ -1118,14 +1059,17 @@ vKisAnnotationSP_it KisImage::beginAnnotations()
     KisAnnotationSP annotation;
 
     if (profile) {
-        // XXX we hardcode icc, this is correct for lcms?
+#ifdef __GNUC__
+#warning "KisImage::beginAnnotations: make it possible to save any profile, not just icc profiles."
+#endif
+#if 0
+        // XXX we hardcode icc, this is correct for icc?
         // XXX productName(), or just "ICC Profile"?
-        if (profile->valid()) {
-            const KoIccColorProfile* iccprofile = dynamic_cast<const KoIccColorProfile*>(profile);
-            if (iccprofile && !iccprofile->rawData().isEmpty()) {
-                annotation = new  KisAnnotation("icc", iccprofile->name(), iccprofile->rawData());
+        if (profile->valid() && profile->type() == "icc" && !profile->rawData().isEmpty()) {
+                annotation = new  KisAnnotation("icc", profile->name(), profile->rawData());
             }
         }
+#endif        
     }
 
     if (annotation)
@@ -1148,10 +1092,18 @@ KisPerspectiveGrid* KisImage::perspectiveGrid()
     return m_d->perspectiveGrid;
 }
 
-void KisImage::refreshGraph()
+void KisImage::refreshGraph(KisNodeSP root)
 {
-    KisRefreshVisitor refresher(this);
-    rootLayer()->accept(refresher);
+    refreshGraph(root, bounds(), bounds());
+}
+
+void KisImage::refreshGraph(KisNodeSP root, const QRect &rc, const QRect &cropRect)
+{
+    if (!root) root = m_d->rootLayer;
+
+    if (!locked() && m_d->projection) {
+        m_d->projection->fullRefresh(root, rc, cropRect);
+    }
 }
 
 void KisImage::slotProjectionUpdated(const QRect & rc)
@@ -1162,8 +1114,8 @@ void KisImage::slotProjectionUpdated(const QRect & rc)
 void KisImage::updateProjection(KisNodeSP node, const QRect& rc)
 {
     if (!locked() && m_d->projection) {
-        dbgImage << "Updating node " << node << "rect" << rc;
-        m_d->projection->updateProjection(node, rc);
+        dbgImage << "KisImage: requested and update for" << node->name() << rc;
+        m_d->projection->updateProjection(node, rc, bounds());
     }
 }
 
