@@ -20,7 +20,7 @@
 #include "kis_paint_device.h"
 
 #include <QRect>
-#include <QMatrix>
+#include <QTransform>
 #include <QImage>
 #include <QList>
 #include <QHash>
@@ -50,23 +50,136 @@
 #include "kis_selection_component.h"
 #include "kis_pixel_selection.h"
 #include "kis_repeat_iterators_pixel.h"
+#include <KoColorModelStandardIds.h>
+
+#include "tiles3/kis_hline_iterator.h"
+#include "tiles3/kis_vline_iterator.h"
+#include "tiles3/kis_rect_iterator.h"
+#include "tiles3/kis_random_accessor.h"
+
+
+class CacheData : public KisDataManager::AbstractCache
+{
+public:
+    bool m_thumbnailsValid;
+    QMap<int, QMap<int, QImage> > m_thumbnails;
+
+    bool m_exactBoundsValid;
+    QRect m_exactBounds;
+
+    bool m_regionValid;
+    QRegion m_region;
+};
+
+class PaintDeviceCache
+{
+public:
+    PaintDeviceCache(KisPaintDevice *paintDevice) {
+        m_paintDevice = paintDevice;
+    }
+
+
+    void setupCache() {
+        CacheData *data = static_cast<CacheData *>(m_paintDevice->m_datamanager->cache());
+
+        if(!data) {
+            data = new CacheData();
+            m_paintDevice->m_datamanager->setCache(data);
+        }
+
+        m_data = data;
+        invalidate();
+    }
+
+    void invalidate() {
+        m_data->m_thumbnailsValid = false;
+        m_data->m_exactBoundsValid = false;
+        m_data->m_regionValid = false;
+    }
+
+    QRect exactBounds() {
+        if (m_data->m_exactBoundsValid) {
+            return m_data->m_exactBounds;
+        }
+        m_data->m_exactBounds = m_paintDevice->calculateExactBounds();
+        m_data->m_exactBoundsValid = true;
+        return m_data->m_exactBounds;
+    }
+
+    QRegion region() {
+        if (m_data->m_regionValid) {
+            return m_data->m_region;
+        }
+        m_data->m_region = m_paintDevice->dataManager()->region();
+        m_data->m_regionValid = true;
+        return m_data->m_region;
+
+    }
+
+    QImage createThumbnail(qint32 w, qint32 h) {
+        QImage thumbnail;
+
+        if(m_data->m_thumbnailsValid) {
+            thumbnail = findThumbnail(w, h);
+        }
+        else {
+            m_data->m_thumbnails.clear();
+            m_data->m_thumbnailsValid = true;
+        }
+
+        if(thumbnail.isNull()) {
+            thumbnail = m_paintDevice->createThumbnail(w, h, 0, QRect());
+            cacheThumbnail(w, h, thumbnail);
+        }
+
+        Q_ASSERT(!thumbnail.isNull());
+        return thumbnail;
+    }
+
+private:
+    inline QImage findThumbnail(qint32 w, qint32 h) {
+        QImage resultImage;
+        if (m_data->m_thumbnails.contains(w) && m_data->m_thumbnails[w].contains(h)) {
+            resultImage = m_data->m_thumbnails[w][h];
+        }
+        return resultImage;
+    }
+
+    inline void cacheThumbnail(qint32 w, qint32 h, QImage image) {
+        m_data->m_thumbnails[w][h] = image;
+    }
+
+private:
+    KisPaintDevice *m_paintDevice;
+    CacheData *m_data;
+};
+
 
 class KisPaintDevice::Private
 {
 
 public:
 
+    Private(KisPaintDevice *paintDevice)
+        : cache(paintDevice)
+    {
+    }
+
     KisNodeWSP parent;
+    KisDefaultBoundsSP defaultBounds;
     qint32 x;
     qint32 y;
     KoColorSpace* colorSpace;
     qint32 pixelSize;
     qint32 nChannels;
+
+    PaintDeviceCache cache;
 };
 
+
 KisPaintDevice::KisPaintDevice(const KoColorSpace * colorSpace, const QString& name)
-        : QObject(0)
-        , m_d(new Private())
+    : QObject(0)
+    , m_d(new Private(this))
 {
     setObjectName(name);
 
@@ -80,20 +193,21 @@ KisPaintDevice::KisPaintDevice(const KoColorSpace * colorSpace, const QString& n
 
     quint8 *defaultPixel = new quint8 [ m_d->pixelSize ];
     memset(defaultPixel, 0, m_d->pixelSize);
+
     m_datamanager = new KisDataManager(m_d->pixelSize, defaultPixel);
     delete [] defaultPixel;
-
     Q_CHECK_PTR(m_datamanager);
+    m_d->cache.setupCache();
 
     m_d->parent = 0;
 
-    m_d->colorSpace = colorSpace->clone();
-
+    m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(colorSpace);
+    setDefaultBounds(new KisDefaultBounds());
 }
 
-KisPaintDevice::KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpace, const QString& name)
-        : QObject(0)
-        , m_d(new Private())
+KisPaintDevice::KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpace, KisDefaultBoundsSP defaultBounds, const QString& name)
+    : QObject(0)
+    , m_d(new Private(this))
 {
     setObjectName(name);
     Q_ASSERT(colorSpace);
@@ -101,9 +215,9 @@ KisPaintDevice::KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpac
     m_d->x = 0;
     m_d->y = 0;
 
-    m_d->parent = parent;
+    setParentNode(parent);
 
-    m_d->colorSpace = colorSpace->clone();
+    m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(colorSpace);
 
     Q_ASSERT(m_d->colorSpace);
 
@@ -112,59 +226,104 @@ KisPaintDevice::KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpac
 
     quint8 *defaultPixel = new quint8[ m_d->pixelSize ];
     m_d->colorSpace->fromQColor(Qt::black, defaultPixel);
-    m_d->colorSpace->setAlpha(defaultPixel, OPACITY_TRANSPARENT, 1);
+    m_d->colorSpace->setOpacity(defaultPixel, OPACITY_TRANSPARENT_U8, 1);
 
     m_datamanager = new KisDataManager(m_d->pixelSize, defaultPixel);
     delete [] defaultPixel;
     Q_CHECK_PTR(m_datamanager);
+    m_d->cache.setupCache();
 
+    setDefaultBounds(defaultBounds);
 }
 
 
 KisPaintDevice::KisPaintDevice(const KisPaintDevice& rhs)
-        : QObject()
-        , KisShared(rhs)
-        , m_d(new Private())
+    : QObject()
+    , KisShared()
+    , m_d(new Private(this))
 {
     if (this != &rhs) {
         m_d->parent = 0;
         if (rhs.m_datamanager) {
             m_datamanager = new KisDataManager(*rhs.m_datamanager);
             Q_CHECK_PTR(m_datamanager);
+            m_d->cache.setupCache();
         } else {
             warnKrita << "rhs " << rhs.objectName() << " has no datamanager\n";
         }
         m_d->x = rhs.m_d->x;
         m_d->y = rhs.m_d->y;
-        m_d->colorSpace = rhs.m_d->colorSpace->clone();
 
+        m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(rhs.m_d->colorSpace);
         m_d->pixelSize = rhs.m_d->pixelSize;
-
         m_d->nChannels = rhs.m_d->nChannels;
+
+        setDefaultBounds(rhs.defaultBounds());
     }
 }
 
 KisPaintDevice::~KisPaintDevice()
 {
-    delete m_d->colorSpace;
+    KoColorSpaceRegistry::instance()->releaseColorSpace(m_d->colorSpace);
     delete m_d;
 }
 
+void KisPaintDevice::prepareClone(KisPaintDeviceSP src)
+{
+    clear();
+    m_d->parent = 0;
+    m_d->x = src->x();
+    m_d->y = src->y();
+
+    if(!(*colorSpace() == *src->colorSpace())) {
+        KoColorSpaceRegistry::instance()->releaseColorSpace(m_d->colorSpace);
+        m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(src->colorSpace());
+        m_d->nChannels = m_d->colorSpace->channelCount();
+
+        if ((quint32)m_d->pixelSize != m_d->colorSpace->pixelSize()) {
+            m_datamanager = 0;
+            m_datamanager = new KisDataManager(src->pixelSize(), src->defaultPixel());
+            m_d->pixelSize = src->pixelSize();
+            m_d->cache.setupCache();
+        }
+        else {
+            setDefaultPixel(src->defaultPixel());
+        }
+    }
+    setDefaultBounds(src->defaultBounds());
+
+    Q_ASSERT(fastBitBltPossible(src));
+}
+
+void KisPaintDevice::makeCloneFrom(KisPaintDeviceSP src, const QRect &rect)
+{
+    prepareClone(src);
+    fastBitBlt(src, rect);
+}
+
+void KisPaintDevice::makeCloneFromRough(KisPaintDeviceSP src, const QRect &minimalRect)
+{
+    prepareClone(src);
+    fastBitBltRough(src, minimalRect);
+}
+
 void KisPaintDevice::setDirty(const QRect & rc)
 {
-
+    m_d->cache.invalidate();
     if (m_d->parent.isValid())
         m_d->parent->setDirty(rc);
 }
 
 void KisPaintDevice::setDirty(const QRegion & region)
 {
+    m_d->cache.invalidate();
     if (m_d->parent.isValid())
         m_d->parent->setDirty(region);
 }
 
 void KisPaintDevice::setDirty()
 {
+    m_d->cache.invalidate();
     if (m_d->parent.isValid())
         m_d->parent->setDirty();
 }
@@ -174,6 +333,17 @@ void KisPaintDevice::setParentNode(KisNodeWSP parent)
     m_d->parent = parent;
 }
 
+void KisPaintDevice::setDefaultBounds(KisDefaultBoundsSP defaultBounds)
+{
+    m_d->defaultBounds = defaultBounds;
+    m_d->cache.invalidate();
+}
+
+KisDefaultBoundsSP KisPaintDevice::defaultBounds() const
+{
+    return m_d->defaultBounds;
+}
+
 void KisPaintDevice::move(qint32 x, qint32 y)
 {
     QRect dirtyRegion = extent();
@@ -203,12 +373,24 @@ void KisPaintDevice::extent(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const
 
 QRect KisPaintDevice::extent() const
 {
+    QRect extent;
+
     qint32 x, y, w, h;
     m_datamanager->extent(x, y, w, h);
     x += m_d->x;
     y += m_d->y;
+    extent = QRect(x, y, w, h);
 
-    return QRect(x, y, w, h);
+    quint8 defaultOpacity = colorSpace()->opacityU8(defaultPixel());
+    if (defaultOpacity != OPACITY_TRANSPARENT_U8)
+        extent |= m_d->defaultBounds->bounds();
+
+    return extent;
+}
+
+QRegion KisPaintDevice::region() const
+{
+    return m_d->cache.region();
 }
 
 void KisPaintDevice::exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const
@@ -220,9 +402,24 @@ void KisPaintDevice::exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) con
     h = rc.height();
 }
 
-
 QRect KisPaintDevice::exactBounds() const
 {
+    return m_d->cache.exactBounds();
+}
+
+QRect KisPaintDevice::calculateExactBounds() const
+{
+    quint8 defaultOpacity = colorSpace()->opacityU8(defaultPixel());
+    if(defaultOpacity != OPACITY_TRANSPARENT_U8) {
+        /**
+         * We will not calculate exact bounds for the device,
+         * that is knows to be at least not smaller than image.
+         * It isn't worth it.
+         */
+
+        return extent();
+    }
+
     // Solution n°2
     qint32  x, y, w, h, boundX2, boundY2, boundW2, boundH2;
     QRect rc = extent();
@@ -232,53 +429,53 @@ QRect KisPaintDevice::exactBounds() const
     w = boundW2 = rc.width();
     h = boundH2 = rc.height();
 
+    // XXX: a small optimization is possible by using H/V line iterators in the first
+    //      and third cases, at the cost of making the code a bit more complex
+
     const quint8* defaultPixel = m_datamanager->defaultPixel();
+    KisRandomConstAccessorSP accessor = createRandomConstAccessorNG(x, y);
+
     bool found = false;
     {
-        KisHLineConstIterator it = const_cast<KisPaintDevice *>(this)->createHLineConstIterator(x, y, w);
         for (qint32 y2 = y; y2 < y + h ; ++y2) {
-            while (!it.isDone() && found == false) {
-                if (memcmp(it.rawData(), defaultPixel, m_d->pixelSize) != 0) {
+            for (qint32 x2 = x; x2 < x + w || found; ++ x2) {
+                accessor->moveTo(x2, y2);
+                if (memcmp(accessor->oldRawData(), defaultPixel, m_d->pixelSize) != 0) {
                     boundY2 = y2;
                     found = true;
                     break;
                 }
-                ++it;
             }
             if (found) break;
-            it.nextRow();
         }
     }
 
     found = false;
 
     for (qint32 y2 = y + h - 1; y2 >= y ; --y2) {
-        KisHLineConstIterator it = const_cast<KisPaintDevice *>(this)->createHLineConstIterator(x, y2, w);
-        while (!it.isDone() && found == false) {
-            if (memcmp(it.rawData(), defaultPixel, m_d->pixelSize) != 0) {
+        for (qint32 x2 = x + w - 1; x2 >= x || found; --x2) {
+            accessor->moveTo(x2, y2);
+            if (memcmp(accessor->oldRawData(), defaultPixel, m_d->pixelSize) != 0) {
                 boundH2 = y2 - boundY2 + 1;
                 found = true;
                 break;
             }
-            ++it;
         }
         if (found) break;
     }
     found = false;
 
     {
-        KisVLineConstIterator it = const_cast<KisPaintDevice *>(this)->createVLineConstIterator(x, boundY2, boundH2);
         for (qint32 x2 = x; x2 < x + w ; ++x2) {
-            while (!it.isDone() && found == false) {
-                if (memcmp(it.rawData(), defaultPixel, m_d->pixelSize) != 0) {
+            for (qint32 y2 = y; y2 < y + h || found; ++y2) {
+                accessor->moveTo(x2, y2);
+                if (memcmp(accessor->oldRawData(), defaultPixel, m_d->pixelSize) != 0) {
                     boundX2 = x2;
                     found = true;
                     break;
                 }
-                ++it;
             }
             if (found) break;
-            it.nextCol();
         }
     }
 
@@ -288,41 +485,38 @@ QRect KisPaintDevice::exactBounds() const
     {
 
         for (qint32 x2 = x + w - 1; x2 >= x; --x2) {
-
-            KisVLineConstIterator it = const_cast<KisPaintDevice *>(this)->createVLineConstIterator(x2, boundY2, boundH2);
-            while (!it.isDone() && found == false) {
-
-                if (memcmp(it.rawData(), defaultPixel, m_d->pixelSize) != 0) {
-
+            for (qint32 y2 = y + h -1; y2 >= y || found; --y2) {
+                accessor->moveTo(x2, y2);
+                if (memcmp(accessor->oldRawData(), defaultPixel, m_d->pixelSize) != 0) {
                     boundW2 = x2 - boundX2 + 1;
                     found = true;
                     break;
                 }
-                ++it;
             }
             if (found) break;
         }
     }
+
     return QRect(boundX2, boundY2, boundW2, boundH2);
 }
 
 
 void KisPaintDevice::crop(qint32 x, qint32 y, qint32 w, qint32 h)
 {
-    m_datamanager->setExtent(x - m_d->x, y - m_d->y, w, h);
+    crop(QRect(x, y, w, h));
 }
 
 
-void KisPaintDevice::crop(const QRect & r)
+void KisPaintDevice::crop(const QRect &rect)
 {
-    QRect rc(r);
-    rc.translate(-m_d->x, -m_d->y);
-    m_datamanager->setExtent(rc);
+    m_datamanager->setExtent(rect.translated(-m_d->x, -m_d->y));
+    m_d->cache.invalidate();
 }
 
 void KisPaintDevice::setDefaultPixel(const quint8 *defPixel)
 {
     m_datamanager->setDefaultPixel(defPixel);
+    m_d->cache.invalidate();
 }
 
 const quint8 *KisPaintDevice::defaultPixel() const
@@ -333,17 +527,28 @@ const quint8 *KisPaintDevice::defaultPixel() const
 void KisPaintDevice::clear()
 {
     m_datamanager->clear();
+    m_d->cache.invalidate();
+}
+
+void KisPaintDevice::clear(const QRect & rc)
+{
+    m_datamanager->clear(rc.x() - m_d->x, rc.y() - m_d->y,
+                         rc.width(), rc.height(),
+                         m_datamanager->defaultPixel());
+    m_d->cache.invalidate();
+}
+
+void KisPaintDevice::fill(const QRect & rc, const KoColor &color)
+{
+    m_datamanager->clear(rc.x() - m_d->x, rc.y() - m_d->y,
+                         rc.width(), rc.height(), color.data());
+    m_d->cache.invalidate();
 }
 
 void KisPaintDevice::fill(qint32 x, qint32 y, qint32 w, qint32 h, const quint8 *fillPixel)
 {
-    m_datamanager->clear(x, y, w, h, fillPixel);
-}
-
-
-void KisPaintDevice::clear(const QRect & rc)
-{
-    m_datamanager->clear(rc.x(), rc.y(), rc.width(), rc.height(), m_datamanager->defaultPixel());
+    m_datamanager->clear(x - m_d->x, y - m_d->y, w, h, fillPixel);
+    m_d->cache.invalidate();
 }
 
 bool KisPaintDevice::write(KoStore *store)
@@ -357,13 +562,15 @@ bool KisPaintDevice::write(KoStore *store)
 bool KisPaintDevice::read(KoStore *store)
 {
     bool retval = m_datamanager->read(store);
-    emit ioProgress(100);
+    m_d->cache.invalidate();
 
+    emit ioProgress(100);
     return retval;
 }
 
 QUndoCommand* KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
 {
+    m_d->cache.invalidate();
     dbgImage << this << colorSpace()->id() << dstColorSpace->id() << renderingIntent;
     if (*colorSpace() == *dstColorSpace) {
         return 0;
@@ -408,62 +615,67 @@ QUndoCommand* KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoCo
     }
 
     KisDataManagerSP oldData = m_datamanager;
-    const KoColorSpace *oldColorSpace = m_d->colorSpace;
+    KoColorSpace *oldColorSpace = m_d->colorSpace;
 
     KisPaintDeviceConvertTypeCommand* cmd = new KisPaintDeviceConvertTypeCommand(this,
-            oldData,
-            oldColorSpace,
-            dst.m_datamanager,
-            dstColorSpace);
+                                                                                 oldData,
+                                                                                 oldColorSpace,
+                                                                                 dst.m_datamanager,
+                                                                                 dstColorSpace);
 
     setDataManager(dst.m_datamanager, dstColorSpace);
 
-    emit colorSpaceChanged(dstColorSpace);
-    delete oldColorSpace;
-
     return cmd;
 }
 
 void KisPaintDevice::setProfile(const KoColorProfile * profile)
 {
     if (profile == 0) return;
-
+    m_d->cache.invalidate();
     const KoColorSpace * dstSpace =
-        KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->id(), profile);
-    if (dstSpace)
-        m_d->colorSpace = dstSpace->clone();
+            KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->colorModelId().id(), colorSpace()->colorDepthId().id(), profile);
+    if (dstSpace) {
+        KoColorSpaceRegistry::instance()->releaseColorSpace(m_d->colorSpace);
+        m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(dstSpace);
+    }
     emit profileChanged(profile);
 }
 
 void KisPaintDevice::setDataManager(KisDataManagerSP data, const KoColorSpace * colorSpace)
 {
     m_datamanager = data;
-    //delete m_d->colorSpace;
-    m_d->colorSpace = colorSpace->clone();
-    m_d->pixelSize = m_d->colorSpace->pixelSize();
-    m_d->nChannels = m_d->colorSpace->channelCount();
+    m_d->cache.setupCache();
+
+    if(colorSpace) {
+        KoColorSpaceRegistry::instance()->releaseColorSpace(m_d->colorSpace);
+        m_d->colorSpace = KoColorSpaceRegistry::instance()->grabColorSpace(colorSpace);
+        m_d->pixelSize = m_d->colorSpace->pixelSize();
+        m_d->nChannels = m_d->colorSpace->channelCount();
+        emit colorSpaceChanged(colorSpace);
+    }
 }
 
-void KisPaintDevice::convertFromQImage(const QImage& image, const QString &srcProfileName,
+void KisPaintDevice::convertFromQImage(const QImage& _image, const QString &srcProfileName,
                                        qint32 offsetX, qint32 offsetY)
 {
-    QImage img = image;
+    QImage image = _image;
 
-    if (img.format() != QImage::Format_ARGB32) {
-        img = img.convertToFormat(QImage::Format_ARGB32);
+    if (image.format() != QImage::Format_ARGB32) {
+        image = image.convertToFormat(QImage::Format_ARGB32);
     }
     // Don't convert if not no profile is given and both paint dev and qimage are rgba.
     if (srcProfileName.isEmpty() && colorSpace()->id() == "RGBA") {
-        writeBytes(img.bits(), offsetX, offsetY, img.width(), img.height());
+        writeBytes(image.bits(), offsetX, offsetY, image.width(), image.height());
     } else {
-        quint8 * dstData = new quint8[img.width() * img.height() * pixelSize()];
+        quint8 * dstData = new quint8[image.width() * image.height() * pixelSize()];
         KoColorSpaceRegistry::instance()
-        ->colorSpace("RGBA", srcProfileName)
-        ->convertPixelsTo(img.bits(), dstData, colorSpace(), img.width() * img.height());
+                ->colorSpace(RGBAColorModelID.id(), Integer8BitsColorDepthID.id(), srcProfileName)
+                ->convertPixelsTo(image.bits(), dstData, colorSpace(), image.width() * image.height());
 
-        writeBytes(dstData, offsetX, offsetY, img.width(), img.height());
+        writeBytes(dstData, offsetX, offsetY, image.width(), image.height());
         delete[] dstData;
     }
+    m_d->cache.invalidate();
 }
 
 QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile) const
@@ -508,20 +720,23 @@ QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint3
     //      This copies potentially a lot of data.
     readBytes(data, x1, y1, w, h);
     QImage image = colorSpace()->convertToQImage(data, w, h, dstProfile,
-                   KoColorConversionTransformation::IntentPerceptual);
+                                                 KoColorConversionTransformation::IntentPerceptual);
     delete[] data;
 
     return image;
 }
 
-KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h, const KisSelection * selection) const
+KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h, const KisSelection * selection, QRect rect) const
 {
     KisPaintDeviceSP thumbnail = new KisPaintDevice(colorSpace());
-    thumbnail->clear();
 
     int srcWidth, srcHeight;
     int srcX0, srcY0;
-    const QRect e = extent();
+    QRect e;
+    if (!rect.isValid())
+        e = extent();
+    else
+        e = rect.translated(-m_d->x, -m_d->y);
     e.getRect(&srcX0, &srcY0, &srcWidth, &srcHeight);
 
 
@@ -556,14 +771,22 @@ KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h, const
 
 }
 
-QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h, const KisSelection *selection)
+QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h, const KisSelection *selection, QRect rect)
 {
-    KisPaintDeviceSP dev = createThumbnailDevice(w, h, selection);
-    return dev->convertToQImage(KoColorSpaceRegistry::instance()->rgb8()->profile());
+    KisPaintDeviceSP dev = createThumbnailDevice(w, h, selection, rect);
+    QImage thumbnail = dev->convertToQImage(KoColorSpaceRegistry::instance()->rgb8()->profile());
+    return thumbnail;
+}
+
+QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h)
+{
+    return m_d->cache.createThumbnail(w, h);
 }
 
 KisRectIteratorPixel KisPaintDevice::createRectIterator(qint32 left, qint32 top, qint32 w, qint32 h, const KisSelection * selection)
 {
+    m_d->cache.invalidate();
+
     KisDataManager* selectionDm = 0;
 
     if (selection)
@@ -585,6 +808,7 @@ KisRectConstIteratorPixel KisPaintDevice::createRectConstIterator(qint32 left, q
 
 KisHLineIteratorPixel  KisPaintDevice::createHLineIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection)
 {
+    m_d->cache.invalidate();
     KisDataManager* selectionDm = 0;
 
     if (selection)
@@ -593,6 +817,43 @@ KisHLineIteratorPixel  KisPaintDevice::createHLineIterator(qint32 x, qint32 y, q
     return KisHLineIteratorPixel(m_datamanager.data(), selectionDm, x, y, w, m_d->x, m_d->y);
 }
 
+
+KisHLineIteratorSP KisPaintDevice::createHLineIteratorNG(qint32 x, qint32 y, qint32 w)
+{
+    m_d->cache.invalidate();
+    return new KisHLineIterator2(m_datamanager.data(), x, y, w, m_d->x, m_d->y, true);
+}
+
+KisHLineConstIteratorSP KisPaintDevice::createHLineConstIteratorNG(qint32 x, qint32 y, qint32 w) const
+{
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
+    return new KisHLineIterator2(dm, x, y, w, m_d->x, m_d->y, false);
+}
+
+KisVLineIteratorSP KisPaintDevice::createVLineIteratorNG(qint32 x, qint32 y, qint32 w)
+{
+    m_d->cache.invalidate();
+    return new KisVLineIterator2(m_datamanager.data(), x, y, w, m_d->x, m_d->y, true);
+}
+
+KisVLineConstIteratorSP KisPaintDevice::createVLineConstIteratorNG(qint32 x, qint32 y, qint32 w) const
+{
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
+    return new KisVLineIterator2(dm, x, y, w, m_d->x, m_d->y, false);
+}
+
+KisRectIteratorSP KisPaintDevice::createRectIteratorNG(qint32 x, qint32 y, qint32 w, qint32 h)
+{
+    m_d->cache.invalidate();
+    return new KisRectIterator2(m_datamanager.data(), x, y, w, h, m_d->x, m_d->y, true);
+}
+
+KisRectConstIteratorSP KisPaintDevice::createRectConstIteratorNG(qint32 x, qint32 y, qint32 w, qint32 h) const
+{
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
+    return new KisRectIterator2(dm, x, y, w, h, m_d->x, m_d->y, false);
+}
+
 KisHLineConstIteratorPixel  KisPaintDevice::createHLineConstIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection) const
 {
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
@@ -628,6 +889,7 @@ KisRepeatVLineConstIteratorPixel KisPaintDevice::createRepeatVLineConstIterator(
 
 KisVLineIteratorPixel  KisPaintDevice::createVLineIterator(qint32 x, qint32 y, qint32 h, const KisSelection * selection)
 {
+    m_d->cache.invalidate();
     KisDataManager* selectionDm = 0;
 
     if (selection)
@@ -649,6 +911,7 @@ KisVLineConstIteratorPixel  KisPaintDevice::createVLineConstIterator(qint32 x, q
 
 KisRandomAccessorPixel KisPaintDevice::createRandomAccessor(qint32 x, qint32 y, const KisSelection * selection)
 {
+    m_d->cache.invalidate();
     KisDataManager* selectionDm = 0;
 
     if (selection)
@@ -668,8 +931,22 @@ KisRandomConstAccessorPixel KisPaintDevice::createRandomConstAccessor(qint32 x,
     return KisRandomConstAccessorPixel(dm, selectionDm, x, y, m_d->x, m_d->y);
 }
 
+KisRandomAccessorSP KisPaintDevice::createRandomAccessorNG(qint32 x, qint32 y)
+{
+    m_d->cache.invalidate();
+    return new KisRandomAccessor2(m_datamanager.data(), x, y, m_d->x, m_d->y, true);
+}
+
+KisRandomConstAccessorSP KisPaintDevice::createRandomConstAccessorNG(qint32 x, qint32 y) const
+{
+    KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
+
+    return new KisRandomAccessor2(dm, x, y, m_d->x, m_d->y, false);
+}
+
 KisRandomSubAccessorPixel KisPaintDevice::createRandomSubAccessor(const KisSelection * selection) const
 {
+    m_d->cache.invalidate();
     // XXX: shouldn't we use the selection here?
     Q_UNUSED(selection);
     KisPaintDevice* pd = const_cast<KisPaintDevice*>(this);
@@ -685,20 +962,24 @@ void KisPaintDevice::clearSelection(KisSelectionSP selection)
         KisHLineIterator devIt = createHLineIterator(r.x(), r.y(), r.width());
         KisHLineConstIterator selectionIt = selection->createHLineIterator(r.x(), r.y(), r.width());
 
+        const quint8* defaultPixel_ = defaultPixel();
+        bool transparentDefault = (m_d->colorSpace->opacityU8(defaultPixel_) == OPACITY_TRANSPARENT_U8);
         for (qint32 y = 0; y < r.height(); y++) {
 
             while (!devIt.isDone()) {
                 // XXX: Optimize by using stretches
 
                 m_d->colorSpace->applyInverseAlphaU8Mask(devIt.rawData(), selectionIt.rawData(), 1);
-
+                if (transparentDefault && m_d->colorSpace->opacityU8(devIt.rawData()) == OPACITY_TRANSPARENT_U8) {
+                    memcpy(devIt.rawData(), defaultPixel_, m_d->colorSpace->pixelSize());
+                }
                 ++devIt;
                 ++selectionIt;
             }
             devIt.nextRow();
             selectionIt.nextRow();
         }
-
+        m_datamanager->purge(r.translated(-m_d->x, -m_d->y));
         setDirty(r);
     }
 }
@@ -726,9 +1007,9 @@ void KisPaintDevice::applySelectionMask(KisSelectionSP mask)
     }
 }
 
-bool KisPaintDevice::pixel(qint32 x, qint32 y, QColor *c)
+bool KisPaintDevice::pixel(qint32 x, qint32 y, QColor *c) const
 {
-    KisHLineConstIteratorPixel iter = createHLineIterator(x, y, 1);
+    KisHLineConstIteratorPixel iter = createHLineConstIterator(x, y, 1);
 
     const quint8 *pix = iter.rawData();
 
@@ -740,11 +1021,11 @@ bool KisPaintDevice::pixel(qint32 x, qint32 y, QColor *c)
 }
 
 
-bool KisPaintDevice::pixel(qint32 x, qint32 y, KoColor * kc)
+bool KisPaintDevice::pixel(qint32 x, qint32 y, KoColor * kc) const
 {
-    KisHLineIteratorPixel iter = createHLineIterator(x, y, 1);
+    KisHLineConstIteratorPixel iter = createHLineConstIterator(x, y, 1);
 
-    quint8 *pix = iter.rawData();
+    const quint8 *pix = iter.rawData();
 
     if (!pix) return false;
 
@@ -758,7 +1039,7 @@ bool KisPaintDevice::setPixel(qint32 x, qint32 y, const QColor& c)
     KisHLineIteratorPixel iter = createHLineIterator(x, y, 1);
 
     colorSpace()->fromQColor(c, iter.rawData());
-
+    m_d->cache.invalidate();
     return true;
 }
 
@@ -774,7 +1055,7 @@ bool KisPaintDevice::setPixel(qint32 x, qint32 y, const KoColor& kc)
         pix = kc.data();
         memcpy(iter.rawData(), pix, m_d->colorSpace->pixelSize());
     }
-
+    m_d->cache.invalidate();
     return true;
 }
 
@@ -797,13 +1078,36 @@ qint32 KisPaintDevice::rowStride(qint32 x, qint32 y) const
 void KisPaintDevice::setX(qint32 x)
 {
     m_d->x = x;
+    m_d->cache.invalidate();
 }
 
 void KisPaintDevice::setY(qint32 y)
 {
     m_d->y = y;
+    m_d->cache.invalidate();
 }
 
+bool KisPaintDevice::fastBitBltPossible(KisPaintDeviceSP src)
+{
+    return m_d->x == src->x() && m_d->y == src->y() &&
+            *colorSpace() == *src->colorSpace();
+}
+
+void KisPaintDevice::fastBitBlt(KisPaintDeviceSP src, const QRect &rect)
+{
+    Q_ASSERT(fastBitBltPossible(src));
+
+    m_datamanager->bitBlt(src->dataManager(), rect.translated(-m_d->x, -m_d->y));
+    m_d->cache.invalidate();
+}
+
+void KisPaintDevice::fastBitBltRough(KisPaintDeviceSP src, const QRect &rect)
+{
+    Q_ASSERT(fastBitBltPossible(src));
+
+    m_datamanager->bitBltRough(src->dataManager(), rect.translated(-m_d->x, -m_d->y));
+    m_d->cache.invalidate();
+}
 
 void KisPaintDevice::readBytes(quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h) const
 {
@@ -818,6 +1122,7 @@ void KisPaintDevice::readBytes(quint8 * data, const QRect &rect)
 void KisPaintDevice::writeBytes(const quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h)
 {
     m_datamanager->writeBytes(data, x - m_d->x, y - m_d->y, w, h);
+    m_d->cache.invalidate();
 }
 
 void KisPaintDevice::writeBytes(const quint8 * data, const QRect &rect)
@@ -833,6 +1138,7 @@ QVector<quint8*> KisPaintDevice::readPlanarBytes(qint32 x, qint32 y, qint32 w, q
 void KisPaintDevice::writePlanarBytes(QVector<quint8*> planes, qint32 x, qint32 y, qint32 w, qint32 h)
 {
     m_datamanager->writePlanarBytes(planes, channelSizes(), x, y, w, h);
+    m_d->cache.invalidate();
 }
 
 
