@@ -20,6 +20,7 @@
 #define KIS_PAINT_DEVICE_IMPL_H_
 
 #include <QObject>
+#include <QRect>
 
 #include "kis_debug.h"
 
@@ -29,9 +30,11 @@
 #include "kis_global.h"
 #include "kis_shared.h"
 #include "kis_iterators_pixel.h"
+#include "kis_default_bounds.h"
 
 #include <krita_export.h>
 
+class QUndoCommand;
 class QRect;
 class QImage;
 class QPoint;
@@ -43,10 +46,11 @@ class KoColor;
 class KoColorSpace;
 class KoColorProfile;
 
+class KisHLineIteratorNG;
 class KisRandomSubAccessorPixel;
 class KisDataManager;
 class KisSelectionComponent;
-class KisPainterlyOverlay;
+
 
 typedef KisSharedPtr<KisDataManager> KisDataManagerSP;
 
@@ -58,8 +62,8 @@ typedef KisSharedPtr<KisDataManager> KisDataManagerSP;
  * when pixels are accessed by an iterator.
  */
 class KRITAIMAGE_EXPORT KisPaintDevice
-            : public QObject
-            , public KisShared
+        : public QObject
+        , public KisShared
 {
 
     Q_OBJECT
@@ -82,7 +86,7 @@ public:
      * @param colorSpace the colorspace of this paint device
      * @param name for debugging purposes
      */
-    KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpace, const QString& name = QString());
+    KisPaintDevice(KisNodeWSP parent, const KoColorSpace * colorSpace, KisDefaultBoundsSP defaultBounds = new KisDefaultBounds(), const QString& name = QString());
 
     KisPaintDevice(const KisPaintDevice& rhs);
     virtual ~KisPaintDevice();
@@ -101,6 +105,22 @@ public:
 
 public:
 
+    /**
+     * set the parent node of the paint device
+     */
+    void setParentNode(KisNodeWSP parent);
+
+    /**
+     * set the default bounds for the paint device when
+     * the default pixel in not completely transarent
+     */
+    virtual void setDefaultBounds(KisDefaultBoundsSP bounds);
+
+     /**
+     * the default bounds rect of the paint device
+     */
+    KisDefaultBoundsSP defaultBounds() const;
+
     /**
      * Moves the device to these new coordinates (so no incremental move or so)
      */
@@ -124,12 +144,12 @@ public:
     /**
      * set the X offset of the paint device
      */
-    void setX(qint32 x);
+    virtual void setX(qint32 x);
 
     /**
      * set the Y offset of the paint device
      */
-    void setY(qint32 y);
+    virtual void setY(qint32 y);
 
     /**
      * Retrieve the bounds of the paint device. The size is not exact,
@@ -137,20 +157,41 @@ public:
      * For instance, the tiled datamanager keeps the extent to the nearest
      * multiple of 64.
      */
-    void extent(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const;
-
-    /// Convience method for the above
     virtual QRect extent() const;
 
-    /**
-     * Get the exact bounds of this paint device. This may be very slow,
-     * especially on larger paint devices because it does a linear scanline search.
-     */
-    void exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const;
-        
     /// Convience method for the above
+    void extent(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const;
+
+    /**
+     * Get the exact bounds of this paint device. The real solution is
+     * very slow because it does a linear scanline search, but it
+     * uses caching, so calling to this function without changing
+     * the device is quite cheap.
+     *
+     * Exactbounds follows these rules:
+     *
+     * <ul>
+     * <li>if there is actual pixel data in the paint device, it is the exact bounds of this data,
+     *     no matter the value of the default pixel.
+     * <li>if there no pixel data in the paint device, and the default pixel is transparent,
+     *     exactBounds is empty
+     * <li>If there is no pixel data in the paint device, and the default pixel is not fully transparent,
+     *     exactBounds is the size of the image.
+     * </ul>
+     * \see calculateExactBounds()
+     */
     virtual QRect exactBounds() const;
 
+    /// Convience method for the above
+    KDE_DEPRECATED void exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const;
+
+    /**
+     * Returns a rough approximation of region covered by device.
+     * For tiled data manager, it region will consist of a number
+     * of rects each corresponding to a tile.
+     */
+    virtual QRegion region() const;
+
     /**
      * Cut the paint device down to the specified rect. If the crop
      * area is bigger than the paint device, nothing will happen.
@@ -178,17 +219,92 @@ public:
     const quint8 *defaultPixel() const;
 
     /**
-     * Clear the given rectangle to transparent black. This does not take the selection
-     * into account.
+     * Clear the given rectangle to transparent black. The paint device will expand to
+     * contain the given rect.
      */
     void clear(const QRect & rc);
 
     /**
-     * Fill the given rectangle with the given pixel. This does not take the
-     * selection into account.
+     * Fill the given rectangle with the given pixel. The paint device will expand to
+     * contain the given rect.
+     */
+    void fill(const QRect & rc, const KoColor &color);
+
+    /**
+     * Overloaded function. For legacy purposes only.
+     * Please use fill(const QRect & rc, const KoColor &color) instead
      */
     void fill(qint32 x, qint32 y, qint32 w, qint32 h, const quint8 *fillPixel);
 
+public:
+
+    /**
+     * Prepares the device for fastBitBlt opreration. It clears
+     * the device, switches x,y shifts and colorspace if needed.
+     * After this call fastBitBlt will return true.
+     * May be used for initialization of temporary devices.
+     */
+    void prepareClone(KisPaintDeviceSP src);
+
+    /**
+     * Make this device to become a clone of \a src. It will have the same
+     * x,y shifts, colorspace and will share pixels inside \a rect.
+     * After calling this function:
+     * (this->extent() >= this->exactBounds() == rect).
+     */
+    void makeCloneFrom(KisPaintDeviceSP src, const QRect &rect);
+
+    /**
+     * Make this device to become a clone of \a src. It will have the same
+     * x,y shifts, colorspace and will share pixels inside \a rect.
+     * Be careful, this function will copy *at least* \a rect
+     * of pixels. Actual copy area will be a bigger - it will
+     * be aligned by tiles borders. So after calling this function:
+     * (this->extent() == this->exactBounds() >= rect).
+     */
+    void makeCloneFromRough(KisPaintDeviceSP src, const QRect &minimalRect);
+
+
+protected:
+    friend class KisPaintDeviceTest;
+
+    /**
+     * Checks whether a src paint device can be used as source
+     * of fast bitBlt operation. The result of the check may
+     * depend on whether color spaces coinside, whether there is
+     * any shift of tiles between the devices and etc.
+     *
+     * WARNING: This check must be done <i>before</i> performing any
+     * fast bitBlt operation!
+     *
+     * \see fastBitBlt
+     * \see fastBitBltRough
+     */
+    bool fastBitBltPossible(KisPaintDeviceSP src);
+
+    /**
+     * Clones rect from another paint device. The cloned area will be
+     * shared between both paint devices as much as possible using
+     * copy-on-write. Parts of the rect that cannot be shared
+     * (cross tiles) are deep-copied,
+     *
+     * \see fastBitBltPossible
+     * \see fastBitBltRough
+     */
+    void fastBitBlt(KisPaintDeviceSP src, const QRect &rect);
+
+    /**
+     * Clones rect from another paint device in a rough and fast way.
+     * All the tiles touched by rect will be shared, between both
+     * devices, that means it will copy a bigger area than was
+     * requested. This method is supposed to be used for bitBlt'ing
+     * into temporary paint devices.
+     *
+     * \see fastBitBltPossible
+     * \see fastBitBlt
+     */
+    void fastBitBltRough(KisPaintDeviceSP src, const QRect &rect);
+
 public:
     /**
      * Read the bytes representing the rectangle described by x, y, w, h into
@@ -200,7 +316,7 @@ public:
      * Reading from areas not previously initialized will read the default
      * pixel value into data but not initialize that region.
      */
-    void readBytes(quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h);
+    void readBytes(quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h) const;
 
     /**
      * Read the bytes representing the rectangle rect into
@@ -262,9 +378,11 @@ public:
     void writePlanarBytes(QVector<quint8*> planes, qint32 x, qint32 y, qint32 w, qint32 h);
 
     /**
-     *   Converts the paint device to a different colorspace
+     * Converts the paint device to a different colorspace
+     *
+     * @return a command that can be used to undo the conversion.
      */
-    void convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent = KoColorConversionTransformation::IntentPerceptual);
+    QUndoCommand* convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent = KoColorConversionTransformation::IntentPerceptual);
 
     /**
      * Changes the profile of the colorspace of this paint device to the given
@@ -273,10 +391,10 @@ public:
     void setProfile(const KoColorProfile * profile);
 
     /**
-     * Fill this paint device with the data from img; starting at (offsetX, offsetY)
-     * @param srcProfileName name of the RGB profile to interpret the img as. "" is interpreted as sRGB
+     * Fill this paint device with the data from image; starting at (offsetX, offsetY)
+     * @param srcProfileName name of the RGB profile to interpret the image as. "" is interpreted as sRGB
      */
-    virtual void convertFromQImage(const QImage& img, const QString &srcProfileName, qint32 offsetX = 0, qint32 offsetY = 0);
+    virtual void convertFromQImage(const QImage& image, const QString &srcProfileName, qint32 offsetX = 0, qint32 offsetY = 0);
 
     /**
      * Create an RGBA QImage from a rectangle in the paint device.
@@ -289,7 +407,7 @@ public:
      * case it's up to the color strategy to choose a profile (most
      * like sRGB).
      */
-    virtual QImage convertToQImage(const KoColorProfile *  dstProfile, qint32 x, qint32 y, qint32 w, qint32 h);
+    virtual QImage convertToQImage(const KoColorProfile *  dstProfile, qint32 x, qint32 y, qint32 w, qint32 h) const;
 
     /**
      * Create an RGBA QImage from a rectangle in the paint device. The
@@ -299,7 +417,7 @@ public:
      * case it's up to the color strategy to choose a profile (most
      * like sRGB).
      */
-    virtual QImage convertToQImage(const KoColorProfile *  dstProfile);
+    virtual QImage convertToQImage(const KoColorProfile *  dstProfile) const;
 
     /**
      * Creates a paint device thumbnail of the paint device, retaining
@@ -307,13 +425,18 @@ public:
      * won't exceed \p maxw and \p maxw, but they may be smaller.
      */
 
-    KisPaintDeviceSP createThumbnailDevice(qint32 w, qint32 h) const;
+    KisPaintDeviceSP createThumbnailDevice(qint32 w, qint32 h, const KisSelection *selection = 0, QRect rect = QRect()) const;
 
     /**
      * Creates a thumbnail of the paint device, retaining the aspect ratio.
      * The width and height of the returned QImage won't exceed \p maxw and \p maxw, but they may be smaller.
      * The colors are not corrected for display!
      */
+    virtual QImage createThumbnail(qint32 maxw, qint32 maxh, const KisSelection *selection, QRect rect = QRect());
+
+    /**
+     * Cached version of createThumbnail(qint32 maxw, qint32 maxh, const KisSelection *selection, QRect rect)
+     */
     virtual QImage createThumbnail(qint32 maxw, qint32 maxh);
 
     /**
@@ -321,11 +444,10 @@ public:
      *
      * The color values will be transformed from the profile of
      * this paint device to the display profile.
-     
-     * @deprecated use iterators instead
+     *
      * @return true if the operation was successful.
      */
-    bool KDE_DEPRECATED pixel(qint32 x, qint32 y, QColor *c);
+    bool pixel(qint32 x, qint32 y, QColor *c) const;
 
     /**
      * Fill kc with the values found at x and y. This method differs
@@ -334,10 +456,9 @@ public:
      * The color values will be transformed from the profile of
      * this paint device to the display profile.
      *
-     * @deprecated use iterators instead
      * @return true if the operation was successful.
      */
-    bool KDE_DEPRECATED pixel(qint32 x, qint32 y, KoColor * kc);
+    bool pixel(qint32 x, qint32 y, KoColor * kc) const;
 
     /**
      * Set the specified pixel to the specified color. Note that this
@@ -351,14 +472,12 @@ public:
      * Note that this will use 8-bit values and may cause a significant
      * degradation when used on 16-bit or hdr quality images.
      *
-     * @deprecated use iterators instead
      * @return true if the operation was successful
-     *
      */
-    bool KDE_DEPRECATED setPixel(qint32 x, qint32 y, const QColor& c);
+    bool setPixel(qint32 x, qint32 y, const QColor& c);
 
     /// Convience method for the above
-    bool KDE_DEPRECATED setPixel(qint32 x, qint32 y, const KoColor& kc);
+    bool setPixel(qint32 x, qint32 y, const KoColor& kc);
 
     /**
      * @return the colorspace of the pixels in this paint device
@@ -378,7 +497,7 @@ public:
     /**
      * Replace the pixel data, color strategy, and profile.
      */
-    void setDataManager(KisDataManagerSP data, const KoColorSpace * colorSpace);
+    void setDataManager(KisDataManagerSP data, const KoColorSpace * colorSpace = 0);
 
     /**
      * Return the number of bytes a pixel takes.
@@ -390,50 +509,26 @@ public:
      */
     virtual quint32 channelCount() const;
 
-    /**
-     * Return the painterly overlay -- a special paint device that
-     * uses the KisPainterlyOverlayColorSpace that defines things
-     * like wetness and canvas height. You need to explicitly create
-     * the painterly overlay before accessing it. Running a physics
-     * filter may also create the painterly overlay.
-     *
-     * Note: this may be 0.
-     */
-    KisPainterlyOverlaySP painterlyOverlay();
-
-    /**
-     * Create a painterly overlay on this paint layer
-     */
-    void createPainterlyOverlay();
-
-    /**
-     * Sets the painterly overlay associated with this paint layer to 0.
-     */
-    void removePainterlyOverlay();
-
 public:
-    
+
     /**
      * Add the specified rect to the parent layer's set of dirty rects
      * (if there is a parent layer)
-     * @deprecated: call setDirty on the node
      */
-    virtual void KDE_DEPRECATED setDirty(const QRect & rc);
+    virtual void setDirty(const QRect & rc);
 
     /**
      *  Add the specified region to the parent layer's dirty region
      *  (if there is a parent layer)
-     * @deprecated: call setDirty on the node
      */
-    virtual void KDE_DEPRECATED setDirty(const QRegion & region);
+    virtual void setDirty(const QRegion & region);
 
     /**
      *  Set the parent layer completely dirty, if this paint device has
      *  as parent layer.
-     * @deprecated: call setDirty on the node
      */
-    virtual void KDE_DEPRECATED setDirty();
-    
+    virtual void setDirty();
+
 public:
 
     /**
@@ -445,7 +540,7 @@ public:
      * @param selection an up-to-date selection that has the same origin as the paint device
      * @return an iterator which points to the first pixel of an rectangle
      */
-    KisRectIteratorPixel createRectIterator(qint32 left, qint32 top, qint32 w, qint32 h, const KisSelection * selection = 0);
+    KDE_DEPRECATED KisRectIteratorPixel createRectIterator(qint32 left, qint32 top, qint32 w, qint32 h, const KisSelection * selection = 0);
 
     /**
      * Create an iterator over a rectangle section of a paint device, the path followed by
@@ -456,47 +551,73 @@ public:
      * @param selection an up-to-date selection that has the same origin as the paint device* @return an iterator which points to the first pixel of an rectangle, this iterator
      * does not allow to change the pixel values
      */
-    KisRectConstIteratorPixel createRectConstIterator(qint32 left, qint32 top, qint32 w, qint32 h, const KisSelection * selection = 0) const;
+    KDE_DEPRECATED  KisRectConstIteratorPixel createRectConstIterator(qint32 left, qint32 top, qint32 w, qint32 h, const KisSelection * selection = 0) const;
 
     /**
      * @param selection an up-to-date selection that has the same origin as the paint device
      * @return an iterator which points to the first pixel of a horizontal line, this iterator
      * does not allow to change the pixel values
      */
-    KisHLineConstIteratorPixel createHLineConstIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection = 0) const;
+    KDE_DEPRECATED KisHLineConstIteratorPixel createHLineConstIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection = 0) const;
+
+    KisHLineIteratorSP createHLineIteratorNG(qint32 x, qint32 y, qint32 w);
+
+    KisHLineConstIteratorSP createHLineConstIteratorNG(qint32 x, qint32 y, qint32 w) const;
+
+    KisVLineIteratorSP createVLineIteratorNG(qint32 x, qint32 y, qint32 h);
+
+    KisVLineConstIteratorSP createVLineConstIteratorNG(qint32 x, qint32 y, qint32 h) const;
+
+    KisRectIteratorSP createRectIteratorNG(qint32 x, qint32 y, qint32 w, qint32 h);
+
+    KisRectConstIteratorSP createRectConstIteratorNG(qint32 x, qint32 y, qint32 w, qint32 h) const;
+
+    KisRandomAccessorSP createRandomAccessorNG(qint32 x, qint32 y);
+
+    KisRandomConstAccessorSP createRandomConstAccessorNG(qint32 x, qint32 y) const;
 
     /**
      * Create an iterator that will "artificially" extend the paint device with the
      * value of the border when trying to access values outside the range of data.
-     * 
+     *
      * @param rc indicates the rectangle that truly contains data
      */
     KisRepeatHLineConstIteratorPixel createRepeatHLineConstIterator(qint32 x, qint32 y, qint32 w, const QRect& _dataWidth, const KisSelection * selection = 0) const;
     /**
      * Create an iterator that will "artificially" extend the paint device with the
      * value of the border when trying to access values outside the range of data.
-     * 
+     *
      * @param rc indicates the rectangle that trully contains data
      */
     KisRepeatVLineConstIteratorPixel createRepeatVLineConstIterator(qint32 x, qint32 y, qint32 h, const QRect& _dataWidth, const KisSelection * selection = 0) const;
-    
+
     /**
     * @param selection an up-to-date selection that has the same origin as the paint device
      * @return an iterator which points to the first pixel of a horizontal line
      */
-    KisHLineIteratorPixel createHLineIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection = 0);
+    KDE_DEPRECATED KisHLineIteratorPixel createHLineIterator(qint32 x, qint32 y, qint32 w, const KisSelection * selection = 0);
 
     /**
      * @param selection an up-to-date selection that has the same origin as the paint device
      * This function return an iterator which points to the first pixel of a vertical line
      */
-    KisVLineIteratorPixel createVLineIterator(qint32 x, qint32 y, qint32 h, const KisSelection * selection = 0);
+    KDE_DEPRECATED KisVLineIteratorPixel createVLineIterator(qint32 x, qint32 y, qint32 h, const KisSelection * selection = 0);
 
     /**
      * @param selection an up-to-date selection that has the same origin as the paint device
      * This function return an iterator which points to the first pixel of a vertical line
      */
-    KisVLineConstIteratorPixel createVLineConstIterator(qint32 x, qint32 y, qint32 h, const KisSelection * selection = 0) const;
+    KDE_DEPRECATED KisVLineConstIteratorPixel createVLineConstIterator(qint32 x, qint32 y, qint32 h, const KisSelection * selection = 0) const;
+
+    /**
+     * This function creates a random accessor which allow to randomly access any pixels on
+     * the paint device.
+     * <b>Note:</b> random access is way slower than iterators, always use iterators whenever
+     * you can.
+     * @param x, y starting point of the accessor
+     * @param selection an up-to-date selection that has the same origin as the paint device
+     */
+    KDE_DEPRECATED KisRandomAccessorPixel createRandomAccessor(qint32 x, qint32 y, const KisSelection * selection = 0);
 
     /**
      * This function creates a random accessor which allow to randomly access any pixels on
@@ -505,16 +626,7 @@ public:
      * you can.
      * @param selection an up-to-date selection that has the same origin as the paint device
      */
-    KisRandomAccessorPixel createRandomAccessor(qint32 x, qint32 y, const KisSelection * selection = 0);
-
-    /**
-     * This function creates a random accessor which allow to randomly access any pixels on
-     * the paint device.
-     * <b>Note:</b> random access is way slower than iterators, always use iterators whenever
-     * you can.
-     * @param selection an up-to-date selection that has the same origin as the paint device
-     */
-    KisRandomConstAccessorPixel createRandomConstAccessor(qint32 x, qint32 y, const KisSelection * selection = 0) const;
+    KDE_DEPRECATED KisRandomConstAccessorPixel createRandomConstAccessor(qint32 x, qint32 y, const KisSelection * selection = 0) const;
 
     /**
      * This function create a random accessor which can easily access to sub pixel values.
@@ -538,13 +650,19 @@ signals:
     void profileChanged(const KoColorProfile *  profile);
     void colorSpaceChanged(const KoColorSpace *colorspace);
 
+private:
+    friend class PaintDeviceCache;
+
     /**
-     * Emitted when the default empty painterly overlay for this paint
-     * device is replaced with a real painterly overlay. Used to
-     * manage the undo system.
+     * Caclculates exact bounds of the device. Used internally
+     * by a transparent caching system. The solution is very slow
+     * because it does a linear scanline search. So the complexity
+     * is n*n at worst.
+     *
+     * \see exactBounds()
      */
-    void painterlyOverlayCreated();
-    
+    QRect calculateExactBounds() const;
+
 private:
 
     KisPaintDevice& operator=(const KisPaintDevice&);
@@ -576,11 +694,8 @@ private:
      */
     QVector<qint32> channelSizes();
 
-protected:
-
-    KisDataManagerSP m_datamanager;
-
 private:
+    KisDataManagerSP m_datamanager;
 
     class Private;
     Private * const m_d;
