@@ -21,6 +21,7 @@
 
 #include "kis_types.h"
 #include "kis_layer.h"
+#include "kis_indirect_painting_support.h"
 
 class KoColorSpace;
 
@@ -39,9 +40,34 @@ class KRITAIMAGE_EXPORT KisPaintLayer : public KisLayer, public KisIndirectPaint
     Q_OBJECT
 
 public:
-    KisPaintLayer(KisImageWSP img, const QString& name, quint8 opacity, KisPaintDeviceSP dev);
-    KisPaintLayer(KisImageWSP img, const QString& name, quint8 opacity);
-    KisPaintLayer(KisImageWSP img, const QString& name, quint8 opacity, const KoColorSpace * colorSpace);
+    /**
+     * Construct a paint layer with the given parameters. The default bounds of the paintdevice are overwriten.
+     * @param image this layer belongs to, or null, if it shouldn't belong to any image
+     * @param name of the layer
+     * @param opacity in the range between OPACITY_TRANSPARENT_U8 and OPACITY_OPAQUE_U8
+     * @param dev is the paint device, that should be used
+     */
+    KisPaintLayer(KisImageWSP image, const QString& name, quint8 opacity, KisPaintDeviceSP dev);
+
+    /**
+     * Construct a paint layer with the given parameters
+     * @param image this layer belongs to. it must not be null and it must have a valid color space.
+     * @param name of the layer
+     * @param opacity in the range between OPACITY_TRANSPARENT_U8 and OPACITY_OPAQUE_U8
+     */
+    KisPaintLayer(KisImageWSP image, const QString& name, quint8 opacity);
+
+    /**
+     * Construct a paint layer with the given parameters
+     * @param image this layer belongs to, or null, if it shouldn't belong to any image. image must not be null, if colorSpace is null
+     * @param name of the layer
+     * @param opacity in the range between OPACITY_TRANSPARENT_U8 and OPACITY_OPAQUE_U8
+     * @param colorSpace is the color space, that should be used to construct the paint device. it can be null, if the image is valid.
+     */
+    KisPaintLayer(KisImageWSP image, const QString& name, quint8 opacity, const KoColorSpace * colorSpace);
+    /**
+     * Copy Constructor
+     */
     KisPaintLayer(const KisPaintLayer& rhs);
     virtual ~KisPaintLayer();
 
@@ -53,9 +79,6 @@ public:
 
     const KoColorSpace * colorSpace() const;
 
-    QRect repaintOriginal(KisPaintDeviceSP original,
-                          const QRect& rect);
-
     bool needProjection() const;
 
     void copyOriginalToProjection(const KisPaintDeviceSP original,
@@ -63,9 +86,13 @@ public:
                                   const QRect& rect) const;
 
     QIcon icon() const;
+    void setImage(KisImageWSP image);
+
     KoDocumentSectionModel::PropertyList sectionModelProperties() const;
+    void setSectionModelProperties(const KoDocumentSectionModel::PropertyList &properties);
 
 public:
+
     QRect extent() const;
     QRect exactBounds() const;
 
@@ -83,17 +110,14 @@ public:
     KisPaintDeviceSP original() const;
 
     /**
-     * Return the dried pixels -- i.e., pixels that have been fixed
-     * through the physics process. This may be 0. The physics process
-     * will automatically set this paint device when it becomes needed.
+     * @returns true when painting should not affect the alpha channel
      */
-    //KisPaintDeviceSP driedPaintDevice();
+    bool alphaLocked() const;
 
     /**
-     * Sets the dried paint device associated with this paint layer to
-     * 0.
+     * @param l if true, the alpha channel will be protected from modification
      */
-    //void removeDriedPaintDevice();
+    void setAlphaLocked(bool l);
 
 public slots:
 
