@@ -75,13 +75,13 @@ public:
      * @param dev the parent paint device. The selection will never be
      * bigger than the parent paint device.
      */
-    KisSelection(KisPaintDeviceSP dev);
+    KisSelection(KisPaintDeviceSP dev, KisDefaultBoundsSP defaultBounds = new KisDefaultBounds());
 
     /**
      * Create a new KisSelection from the given mask. The selection
      * will share its pixel data with the mask
      */
-    KisSelection(KisPaintDeviceSP parent, KisMaskSP mask);
+    KisSelection(KisPaintDeviceSP parent, KisMaskSP mask, KisDefaultBoundsSP defaultBounds = new KisDefaultBounds());
 
     /**
      * Create a new KisSelection. This selection will not have a
@@ -194,6 +194,18 @@ public:
 
     void setDeselected(bool deselected);
     bool isDeselected();
+    void setVisible(bool visible);
+    bool isVisible();
+    
+    ///Reimplemented
+    void setDefaultBounds(KisDefaultBoundsSP bounds);
+
+    ///Reimplemented
+    virtual void setX(qint32 x);
+
+    ///Reimplemented
+    virtual void setY(qint32 y);
+
 
 private:
 
