@@ -1,7 +1,8 @@
 /*
  *  Copyright (c) 2004 Michael Thaler <michael.thaler@physik.tu-muenchen.de> filters
  *  Copyright (c) 2005-2007 Casper Boemann <cbr@boemann.dk>
- *  Copyright (c) 2005 Boudewijn Rempt <boud@valdyas.org> right angle rotators
+ *  Copyright (c) 2005, 2010 Boudewijn Rempt <boud@valdyas.org>
+ *  Copyright (c) 2010 Marc Pegon <pe.marc@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +24,8 @@
 
 #include <KoProgressUpdater.h>
 #include <KoColorSpace.h>
+#include <KoCompositeOp.h>
+#include <KoColor.h>
 
 #include "kis_paint_device.h"
 #include "kis_debug.h"
@@ -37,6 +40,7 @@
 KisTransformWorker::KisTransformWorker(KisPaintDeviceSP dev,
                                        double xscale, double yscale,
                                        double xshear, double yshear,
+                                       double xshearOrigin, double yshearOrigin,
                                        double rotation,
                                        qint32 xtranslate, qint32 ytranslate,
                                        KoUpdaterPtr progress,
@@ -48,8 +52,10 @@ KisTransformWorker::KisTransformWorker(KisPaintDeviceSP dev,
     m_yscale = yscale;
     m_xshear = xshear;
     m_yshear = yshear;
+    m_xshearOrigin = xshearOrigin;
+    m_yshearOrigin = yshearOrigin;
     m_rotation = rotation,
-                 m_xtranslate = xtranslate;
+    m_xtranslate = xtranslate;
     m_ytranslate = ytranslate;
     m_progressUpdater = progress;
     m_filter = filter;
@@ -63,12 +69,10 @@ KisTransformWorker::~KisTransformWorker()
 void KisTransformWorker::rotateNone(KisPaintDeviceSP src, KisPaintDeviceSP dst)
 {
     qint32 pixelSize = src->pixelSize();
-    QRect r;
+    QRect r(m_boundRect);
     KoColorSpace *cs = src->colorSpace();
     Q_UNUSED(cs);
 
-    r = src->exactBounds();
-
     KisHLineIteratorPixel hit = src->createHLineIterator(r.x(), r.top(), r.width());
     KisHLineIterator vit = dst->createHLineIterator(r.x(), r.top(), r.width());
     for (qint32 i = 0; i < r.height(); ++i) {
@@ -96,10 +100,9 @@ void KisTransformWorker::rotateNone(KisPaintDeviceSP src, KisPaintDeviceSP dst)
 void KisTransformWorker::rotateRight90(KisPaintDeviceSP src, KisPaintDeviceSP dst)
 {
     qint32 pixelSize = src->pixelSize();
-    QRect r;
+    QRect r(m_boundRect);
     KoColorSpace *cs = src->colorSpace();
     Q_UNUSED(cs);
-    r = src->exactBounds();
 
     for (qint32 y = r.bottom(); y >= r.top(); --y) {
         KisHLineIteratorPixel hit = src->createHLineIterator(r.x(), y, r.width());
@@ -122,15 +125,16 @@ void KisTransformWorker::rotateRight90(KisPaintDeviceSP src, KisPaintDeviceSP ds
             break;
         }
     }
+
+    m_boundRect = QRect(- r.bottom(), r.x(), r.height(), r.width());
 }
 
 void KisTransformWorker::rotateLeft90(KisPaintDeviceSP src, KisPaintDeviceSP dst)
 {
     qint32 pixelSize = src->pixelSize();
-    QRect r;
+    QRect r(m_boundRect);
     KoColorSpace *cs = src->colorSpace();
     Q_UNUSED(cs);
-    r = src->exactBounds();
 
     KisHLineIteratorPixel hit = src->createHLineIterator(r.x(), r.top(), r.width());
 
@@ -147,7 +151,7 @@ void KisTransformWorker::rotateLeft90(KisPaintDeviceSP src, KisPaintDeviceSP dst
         }
         hit.nextRow();
 
-        //progress info
+        // Progress info
         m_progressStep += r.width();
         if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
@@ -157,15 +161,16 @@ void KisTransformWorker::rotateLeft90(KisPaintDeviceSP src, KisPaintDeviceSP dst
             break;
         }
     }
+
+    m_boundRect = QRect(r.top(), - r.x() - r.width(), r.height(), r.width());
 }
 
 void KisTransformWorker::rotate180(KisPaintDeviceSP src, KisPaintDeviceSP dst)
 {
     qint32 pixelSize = src->pixelSize();
-    QRect r;
+    QRect r(m_boundRect);
     KoColorSpace *cs = src->colorSpace();
     Q_UNUSED(cs);
-    r = src->exactBounds();
 
     KisHLineIteratorPixel srcIt = src->createHLineIterator(r.x(), r.top(), r.width());
 
@@ -190,6 +195,8 @@ void KisTransformWorker::rotate180(KisPaintDeviceSP src, KisPaintDeviceSP dst)
             break;
         }
     }
+
+    m_boundRect = QRect(- r.x() - r.width(), - r.bottom(), r.width(), r.height());
 }
 
 template <class iter> iter createIterator(KisPaintDevice *dev, qint32 start, qint32 lineNum, qint32 len);
@@ -206,12 +213,11 @@ template <> KisVLineIteratorPixel createIterator <KisVLineIteratorPixel>
     return dev->createVLineIterator(lineNum, start, len);
 }
 
-template <class iter> void calcDimensions(KisPaintDevice *dev, qint32 &srcStart, qint32 &srcLen, qint32 &firstLine, qint32 &numLines);
+template <class iter> void calcDimensions(QRect rc, qint32 &srcStart, qint32 &srcLen, qint32 &firstLine, qint32 &numLines);
 
 template <> void calcDimensions <KisHLineIteratorPixel>
-(KisPaintDevice *dev, qint32 &srcStart, qint32 &srcLen, qint32 &firstLine, qint32 &numLines)
+(QRect rc, qint32 &srcStart, qint32 &srcLen, qint32 &firstLine, qint32 &numLines)
 {
-    QRect rc = dev->exactBounds();
     srcStart = rc.x();
     firstLine = rc.y();
     srcLen = rc.width();
@@ -219,15 +225,40 @@ template <> void calcDimensions <KisHLineIteratorPixel>
 }
 
 template <> void calcDimensions <KisVLineIteratorPixel>
-(KisPaintDevice *dev, qint32 &srcStart, qint32 &srcLen, qint32 &firstLine, qint32 &numLines)
+(QRect rc, qint32 &srcStart, qint32 &srcLen, qint32 &firstLine, qint32 &numLines)
 {
-    QRect rc = dev->exactBounds();
     firstLine = rc.x();
     srcStart = rc.y();
     numLines = rc.width();
     srcLen = rc.height();
 }
 
+template <class iter> void updateBounds(QRect &boundRect, double floatscale, double shear, qint32 dx);
+
+template <> void updateBounds <KisHLineIteratorPixel>
+(QRect &boundRect, double floatscale, double shear, qint32 dx)
+{
+    QPoint topLeft(qFloor(boundRect.left() * floatscale + boundRect.top() * shear + dx), boundRect.top());
+    QPoint topRight(qCeil(boundRect.right() * floatscale + boundRect.top() * shear + dx), boundRect.top());
+    QPoint bottomRight(qCeil(boundRect.right() * floatscale + boundRect.bottom() * shear + dx), boundRect.bottom());
+    QPoint bottomLeft(qFloor(boundRect.left() * floatscale + boundRect.bottom() * shear + dx), boundRect.bottom());
+    QPolygon p;
+    p << topLeft << topRight << bottomRight << bottomLeft;
+    boundRect = p.boundingRect();
+}
+
+template <> void updateBounds <KisVLineIteratorPixel>
+(QRect &boundRect, double floatscale, double shear, qint32 dx)
+{
+    QPoint topLeft(boundRect.left(), qFloor(boundRect.top() * floatscale + boundRect.left() * shear + dx));
+    QPoint topRight(boundRect.right(), qFloor(boundRect.top() * floatscale + boundRect.right() * shear + dx));
+    QPoint bottomRight(boundRect.right(), qCeil(boundRect.bottom() * floatscale + boundRect.right() * shear + dx));
+    QPoint bottomLeft(boundRect.left(), qCeil(boundRect.bottom() * floatscale + boundRect.left() * shear + dx));
+    QPolygon p;
+    p << topLeft << topRight << bottomRight << bottomLeft;
+    boundRect = p.boundingRect();
+}
+
 struct FilterValues {
     quint8 numWeights;
     qint16 *weight;
@@ -241,9 +272,10 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
                                        double floatscale, double shear, qint32 dx,
                                        KisFilterStrategy *filterStrategy, bool fixBorderAlpha)
 {
+    m_fixBorderAlpha = fixBorderAlpha;
+
     qint32 lineNum, srcStart, firstLine, srcLen, numLines;
     qint32 center, begin, end;    /* filter calculation variables */
-    quint8 *data;
     quint8 pixelSize = src->pixelSize();
     KoColorSpace * cs = src->colorSpace();
     KoMixColorsOp * mixOp = cs->mixColorsOp();
@@ -252,13 +284,20 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
     qint32 shearFracOffset;
 
 
-    calcDimensions <T>(src, srcStart, srcLen, firstLine, numLines);
+    calcDimensions <T>(m_boundRect, srcStart, srcLen, firstLine, numLines);
 
     scale = int(floatscale * srcLen + 0.5);
+    if (scale == 0) {
+        dst->clear();
+        return;
+    }
+
     scaleDenom = srcLen;
 
-    if (scaleDenom == 0)
+    if (scaleDenom == 0) {
+        updateBounds <T>(m_boundRect, floatscale, shear, dx);
         return;
+    }
 
     qint32 support = filterStrategy->intSupport();
     qint32 dstLen, dstStart;
@@ -358,7 +397,7 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
         while (i < srcLen + extraLen) {
             data = srcIt.rawData();
             memcpy(&tmpLine[i*pixelSize], data, pixelSize);
-            cs->setAlpha(data, 0, 1);
+            memcpy(data, src->defaultPixel(), pixelSize);
             if (i < srcLen + extraLen - 1) // duplicate pixels along edge
                 ++srcIt;
             i++;
@@ -424,15 +463,34 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
     delete [] colors;
     delete [] tmpLine;
     delete [] filterWeights;
+
+    updateBounds <T>(m_boundRect, floatscale, shear, dx);
 }
 
 bool KisTransformWorker::run()
 {
-//return false;
-    //progress info
+    /* Check for nonsense and let the user know, this helps debugging.
+    Otherwise the program will crash at a later point, in a very obscure way, probably by division by zero */
+    Q_ASSERT_X(m_xscale != 0, "KisTransformer::run() validation step", "xscale == 0");
+    Q_ASSERT_X(m_yscale != 0, "KisTransformer::run() validation step", "yscale == 0");
+    // Fallback safety line in case Krita is compiled without ASSERTS
+    if (m_xscale == 0 || m_yscale == 0) return false;
+    
+    // Progress info
     m_progressTotalSteps = 0;
     m_progressStep = 0;
-    QRect r = m_dev->exactBounds();
+
+    KoColor defaultPixel(m_dev->defaultPixel(), m_dev->colorSpace());
+    if (defaultPixel.opacityU8() != OPACITY_TRANSPARENT_U8)
+        m_boundRect = m_dev->dataManager()->extent();
+    else
+        m_boundRect = m_dev->exactBounds();
+
+    if (m_boundRect.isNull()) {
+        if (!m_progressUpdater.isNull())
+            m_progressUpdater->setProgress(100);
+        return true;
+    }
 
     KisPaintDeviceSP tmpdev1 = KisPaintDeviceSP(new KisPaintDevice(m_dev->colorSpace()));
     KisPaintDeviceSP srcdev = m_dev;
@@ -445,6 +503,21 @@ bool KisTransformWorker::run()
     qint32 xtranslate = m_xtranslate;
     qint32 ytranslate = m_ytranslate;
 
+    m_progressTotalSteps = 0;
+
+    // Apply shear X and Y
+    if (xshear != 0 || yshear != 0) {
+        m_progressTotalSteps += (yscale * m_boundRect.height() * (m_boundRect.width() + m_xshear * m_boundRect.height()));
+        m_progressTotalSteps += (xscale * m_boundRect.width() * (m_boundRect.height() + m_yshear * m_boundRect.width()));
+
+        int dx = - qRound(m_yshearOrigin * yscale * m_xshear);
+        int dy = - qRound(m_xshearOrigin * xscale * m_yshear);
+        transformPass <KisHLineIteratorPixel>(srcdev.data(), srcdev.data(), xscale, yscale *  m_xshear, dx, m_filter, m_fixBorderAlpha);
+        transformPass <KisVLineIteratorPixel>(srcdev.data(), srcdev.data(), yscale, m_yshear, dy, m_filter, m_fixBorderAlpha);
+        yscale = 1.;
+        xscale = 1.;
+    }
+
     if (rotation < 0.0)
         rotation = -fmod(-rotation, 2 * M_PI) + 2 * M_PI;
     else
@@ -456,38 +529,36 @@ bool KisTransformWorker::run()
     switch (rotQuadrant) {
     default: // just to shut up the compiler
     case 0:
-        m_progressTotalSteps = 0;
+        m_progressTotalSteps += 0;
         break;
     case 1:
         rotation -= M_PI / 2;
         tmp = xscale;
         xscale = yscale;
         yscale = tmp;
-        m_progressTotalSteps = r.width() * r.height();
+        m_progressTotalSteps += m_boundRect.width() * m_boundRect.height();
         break;
     case 2:
         rotation -= M_PI;
-        m_progressTotalSteps = r.width() * r.height();
+        m_progressTotalSteps += m_boundRect.width() * m_boundRect.height();
         break;
     case 3:
         rotation -= -M_PI / 2 + 2 * M_PI;
         tmp = xscale;
         xscale = yscale;
         yscale = tmp;
-        m_progressTotalSteps = r.width() * r.height();
+        m_progressTotalSteps += m_boundRect.width() * m_boundRect.height();
         break;
     }
 
-    // Calculate some auxillary values
+    //// Calculate some auxillary values
     yshear = sin(rotation);
     xshear = -tan(rotation / 2);
     xtranslate -= int(xshear * ytranslate);
 
     // Calculate progress steps
-    m_progressTotalSteps += int(yscale * r.width() * r.height());
-    m_progressTotalSteps += int(xscale * r.width() * (r.height() * yscale + r.width() * yshear));
-
-    m_lastProgressReport = 0;
+    m_progressTotalSteps += int(yscale * m_boundRect.width() * m_boundRect.height());
+    m_progressTotalSteps += int(xscale * m_boundRect.width() * (m_boundRect.height() * yscale + m_boundRect.width() * yshear));
 
     // Now that we have everything in place it's time to do the actual right angle rotations
     switch (rotQuadrant) {
@@ -511,8 +582,9 @@ bool KisTransformWorker::run()
         break;
     }
 
-    // Handle simple move case possibly with rotation of 90,180,270
+    //// Handle simple move case possibly with rotation of 90,180,270
     if (rotation == 0.0 && xscale == 1.0 && yscale == 1.0) {
+        m_boundRect.translate(xtranslate, ytranslate);
         if (rotQuadrant == 0) {
             // When we didn't move the m_dev to a temp device we can simply just move its coords
             srcdev->move(srcdev->x() + xtranslate, srcdev->y() + ytranslate);
@@ -522,8 +594,7 @@ bool KisTransformWorker::run()
         }
         //progress info
         if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(100);
-        return false;
-
+        return true;
     }
 
     if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
@@ -531,6 +602,7 @@ bool KisTransformWorker::run()
         return false;
     }
 
+    // First pass
     transformPass <KisHLineIteratorPixel>(srcdev.data(), srcdev.data(), xscale, yscale*xshear, 0, m_filter, m_fixBorderAlpha);
 
     if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
@@ -546,9 +618,9 @@ bool KisTransformWorker::run()
         return false;
     }
 
-    if (xshear != 0.0)
+    if (xshear != 0.0) {
         transformPass <KisHLineIteratorPixel>(srcdev.data(), m_dev.data(), 1.0, xshear, xtranslate, m_filter, m_fixBorderAlpha);
-    else {
+    } else {
         // No need to filter again when we are only scaling
         srcdev->move(srcdev->x() + xtranslate, srcdev->y());
         if (rotQuadrant != 0)  // no need to copy back if we have not copied the device in the first place
@@ -557,28 +629,27 @@ bool KisTransformWorker::run()
 
     //CBRm_dev->setDirty();
 
-    //progress info
+    // Progress info
     if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(100);
-
     if (!m_progressUpdater.isNull()) return m_progressUpdater->interrupted();
+    
     return true;
 }
 
 QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* selection)
 {
-    KisPaintDeviceSP dst = dev;
-    bool hasCurrentMemento = dev->dataManager()->hasCurrentMemento();
     int pixelSize = dev->pixelSize();
-
-    if (!hasCurrentMemento) {
-        dst = new KisPaintDevice(dev->colorSpace());
-    }
+    KisPaintDeviceSP dst = new KisPaintDevice(dev->colorSpace());
 
     QRect r;
     if (selection) {
         r = selection->selectedExactRect();
     } else {
-        r = dev->exactBounds();
+        KoColor defaultPixel(dev->defaultPixel(), dev->colorSpace());
+        if (defaultPixel.opacityU8() != OPACITY_TRANSPARENT_U8)
+            r = dev->dataManager()->extent();
+        else
+            r = dev->exactBounds();
     }
     {
         KisHLineConstIteratorPixel srcIt = dev->createHLineConstIterator(r.x(), r.top(), r.width(), selection);
@@ -590,10 +661,7 @@ QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* sele
 
             while (!srcIt.isDone()) {
                 if (srcIt.isSelected()) {
-                    if (hasCurrentMemento)
-                        memcpy(dstIt.rawData(), srcIt.oldRawData(), pixelSize);
-                    else
-                        memcpy(dstIt.rawData(), srcIt.rawData(), pixelSize);
+                    memcpy(dstIt.rawData(), srcIt.rawData(), pixelSize);
                 }
                 ++srcIt;
                 --dstIt;
@@ -603,53 +671,62 @@ QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* sele
             dstIt.nextRow();
         }
     }
-    if (!hasCurrentMemento) {
-        dev->setDataManager(dst->dataManager(), dst->colorSpace());
+    KisPainter gc(dev);
+
+    if (selection) {
+        dev->clearSelection(const_cast<KisSelection*>(selection));
     }
+    else {
+        dev->clear(r);
+    }
+    gc.setCompositeOp(COMPOSITE_OVER);
+    gc.bitBlt(r.topLeft(), dst, r);
 
     return r;
 }
 
 QRect KisTransformWorker::mirrorY(KisPaintDeviceSP dev, const KisSelection* selection)
 {
-    KisPaintDeviceSP dst = dev;
-    bool hasCurrentMemento = dev->dataManager()->hasCurrentMemento();
-
     int pixelSize = dev->pixelSize();
-
-    if (!hasCurrentMemento) {
-        dst = new KisPaintDevice(dev->colorSpace());
-    }
+    KisPaintDeviceSP dst = new KisPaintDevice(dev->colorSpace());
 
     /* Read a line from bottom to top and and from top to bottom and write their values to each other */
     QRect r;
     if (selection) {
         r = selection->selectedExactRect();
     } else {
-        r = dev->exactBounds();
+        KoColor defaultPixel(dev->defaultPixel(), dev->colorSpace());
+        if (defaultPixel.opacityU8() != OPACITY_TRANSPARENT_U8)
+            r = dev->dataManager()->extent();
+        else
+            r = dev->exactBounds();
     }
     {
         qint32 y1, y2;
         for (y1 = r.top(), y2 = r.bottom(); y1 <= r.bottom(); ++y1, --y2) {
 
-            KisHLineIteratorPixel itTop = dst->createHLineIterator(r.x(), y1, r.width(), selection);
-            KisHLineConstIteratorPixel itBottom = dev->createHLineConstIterator(r.x(), y2, r.width());
+            KisHLineConstIteratorPixel itTop = dev->createHLineConstIterator(r.x(), y1, r.width(), selection);
+            KisHLineIteratorPixel itBottom = dst->createHLineIterator(r.x(), y2, r.width());
 
             while (!itTop.isDone() && !itBottom.isDone()) {
-                if (itBottom.isSelected()) {
-                    if (hasCurrentMemento)
-                        memcpy(itTop.rawData(), itBottom.oldRawData(), pixelSize);
-                    else
-                        memcpy(itTop.rawData(), itBottom.rawData(), pixelSize);
+                if (itTop.isSelected()) {
+                    memcpy(itBottom.rawData(), itTop.rawData(), pixelSize);
                 }
                 ++itBottom;
                 ++itTop;
             }
         }
     }
-    if (!hasCurrentMemento) {
-        dev->setDataManager(dst->dataManager(), dst->colorSpace());
+    KisPainter gc(dev);
+
+    if (selection) {
+        dev->clearSelection(const_cast<KisSelection*>(selection));
     }
+    else {
+        dev->clear(r);
+    }
+    gc.setCompositeOp(COMPOSITE_OVER);
+    gc.bitBlt(r.topLeft(), dst, r);
 
     return r;
 }
