@@ -30,12 +30,11 @@
 
 void KisCloneLayerTest::testCreation()
 {
-    const KoColorSpace * colorSpace = KoColorSpaceRegistry::instance()->colorSpace("RGBA", 0);
+    const KoColorSpace * colorSpace = KoColorSpaceRegistry::instance()->rgb8();
     KisImageSP image = new KisImage(0, 512, 512, colorSpace, "layer test");
-    KisLayerSP layer = new KisPaintLayer(image, "clone test", OPACITY_OPAQUE);
+    KisLayerSP layer = new KisPaintLayer(image, "clone test", OPACITY_OPAQUE_U8);
 
-    KisCloneLayer test(layer, image, "clonetest", OPACITY_OPAQUE
-                      );
+    KisCloneLayer test(layer, image, "clonetest", OPACITY_OPAQUE_U8);
 }
 
 
