@@ -38,9 +38,9 @@ void KisCropVisitorTest::testUndo()
     const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
 
     TestUtil::KisUndoAdapterDummy* undoAdapterDummy = new TestUtil::KisUndoAdapterDummy();
-    KisImageWSP img = new KisImage(undoAdapterDummy, 300, 300, cs, "test");
-    img->lock();
-    KisPaintLayerSP layer = new KisPaintLayer(img, "testlayer", OPACITY_OPAQUE);
+    KisImageSP image = new KisImage(undoAdapterDummy, 300, 300, cs, "test");
+    image->lock();
+    KisPaintLayerSP layer = new KisPaintLayer(image, "testlayer", OPACITY_OPAQUE_U8);
     KisPaintDeviceSP dev = layer->paintDevice();
 
     KisFillPainter painter(dev);
@@ -60,6 +60,7 @@ void KisCropVisitorTest::testUndo()
     if (!TestUtil::compareQImages(errpoint, image1, dev->convertToQImage(0, 0, 0, 300, 300))) {
         QFAIL(QString("Failed to create identical image, first different pixel: %1,%2 ").arg(errpoint.x()).arg(errpoint.y()).toAscii());
     }
+    delete undoAdapterDummy;
 }
 
 
