@@ -34,16 +34,29 @@
 
 #include "testutil.h"
 
+#define IMAGE_WIDTH 1000
+#define IMAGE_HEIGHT 1000
+
 void KisFilterMaskTest::testCreation()
 {
     KisFilterMaskSP mask = new KisFilterMask();
 }
 
+#define initImage(image, layer, device, mask) do {                      \
+    image = new KisImage(0, IMAGE_WIDTH, IMAGE_HEIGHT, 0, "tests");     \
+    layer = new KisPaintLayer(image, "", 100, device);         \
+    image->addNode(layer);                                              \
+    image->addNode(mask, layer);                                        \
+    } while(0)
+
 void KisFilterMaskTest::testProjectionNotSelected()
 {
-    const KoColorSpace * cs = KoColorSpaceRegistry::instance()->colorSpace("RGBA", 0);
+    KisImageSP image;
+    KisPaintLayerSP layer;
 
-    QImage qimg(QString(FILES_DATA_DIR) + QDir::separator() + "hakonepa.png");
+    const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
+
+    QImage qimage(QString(FILES_DATA_DIR) + QDir::separator() + "hakonepa.png");
     QImage inverted(QString(FILES_DATA_DIR) + QDir::separator() + "inverted_hakonepa.png");
 
     KisFilterSP f = KisFilterRegistry::instance()->value("invert");
@@ -56,23 +69,28 @@ void KisFilterMaskTest::testProjectionNotSelected()
 
     // Check basic apply(). Shouldn't do anything, since nothing is selected yet.
     KisPaintDeviceSP projection = new KisPaintDevice(cs);
-    projection->convertFromQImage(qimg, 0, 0, 0);
+    initImage(image, layer, projection, mask);
+
+    projection->convertFromQImage(qimage, 0, 0, 0);
     mask->createNodeProgressProxy();
-    mask->apply(projection, QRect(0, 0, qimg.width(), qimg.height()));
-    mask->select(qimg.rect(), MIN_SELECTED);
+    mask->select(qimage.rect(), MIN_SELECTED);
+    mask->apply(projection, QRect(0, 0, qimage.width(), qimage.height()));
 
     QPoint errpoint;
-    if (!TestUtil::compareQImages(errpoint, qimg, projection->convertToQImage(0, 0, 0, qimg.width(), qimg.height()))) {
-        projection->convertToQImage(0, 0, 0, qimg.width(), qimg.height()).save("filtermasktest1.png");
+    if (!TestUtil::compareQImages(errpoint, qimage, projection->convertToQImage(0, 0, 0, qimage.width(), qimage.height()))) {
+        projection->convertToQImage(0, 0, 0, qimage.width(), qimage.height()).save("filtermasktest1.png");
         QFAIL(QString("Failed to create identical image, first different pixel: %1,%2 ").arg(errpoint.x()).arg(errpoint.y()).toAscii());
     }
 }
 
 void KisFilterMaskTest::testProjectionSelected()
 {
-    const KoColorSpace * cs = KoColorSpaceRegistry::instance()->colorSpace("RGBA", 0);
+    KisImageSP image;
+    KisPaintLayerSP layer;
 
-    QImage qimg(QString(FILES_DATA_DIR) + QDir::separator() + "hakonepa.png");
+    const KoColorSpace * cs = KoColorSpaceRegistry::instance()->rgb8();
+
+    QImage qimage(QString(FILES_DATA_DIR) + QDir::separator() + "hakonepa.png");
     QImage inverted(QString(FILES_DATA_DIR) + QDir::separator() + "inverted_hakonepa.png");
 
     KisFilterSP f = KisFilterRegistry::instance()->value("invert");
@@ -85,14 +103,16 @@ void KisFilterMaskTest::testProjectionSelected()
     mask->createNodeProgressProxy();
 
     KisPaintDeviceSP projection = new KisPaintDevice(cs);
-    projection->convertFromQImage(qimg, 0, 0, 0);
-    mask->select(qimg.rect(), MAX_SELECTED);
-    mask->apply(projection, QRect(0, 0, qimg.width(), qimg.height()));
-    QCOMPARE(mask->exactBounds(), QRect(0, 0, qimg.width(), qimg.height()));
+    initImage(image, layer, projection, mask);
+    projection->convertFromQImage(qimage, 0, 0, 0);
+
+    mask->select(qimage.rect(), MAX_SELECTED);
+    mask->apply(projection, QRect(0, 0, qimage.width(), qimage.height()));
+    QCOMPARE(mask->exactBounds(), QRect(0, 0, IMAGE_WIDTH, IMAGE_HEIGHT));
 
     QPoint errpoint;
-    if (!TestUtil::compareQImages(errpoint, inverted, projection->convertToQImage(0, 0, 0, qimg.width(), qimg.height()))) {
-        projection->convertToQImage(0, 0, 0, qimg.width(), qimg.height()).save("filtermasktest2.png");
+    if (!TestUtil::compareQImages(errpoint, inverted, projection->convertToQImage(0, 0, 0, qimage.width(), qimage.height()))) {
+        projection->convertToQImage(0, 0, 0, qimage.width(), qimage.height()).save("filtermasktest2.png");
         QFAIL(QString("Failed to create inverted image, first different pixel: %1,%2 ").arg(errpoint.x()).arg(errpoint.y()).toAscii());
     }
 
