@@ -24,6 +24,7 @@
 #include <KoColorSpaceRegistry.h>
 #include <KoColorSpace.h>
 #include <KoColorProfile.h>
+#include <KoColor.h>
 
 #include "kis_iterators_pixel.h"
 #include "kis_random_accessor.h"
@@ -31,20 +32,18 @@
 
 
 #include "kis_paint_device.h"
+#include <kis_repeat_iterators_pixel.h>
 
 void KisIteratorTest::allCsApplicator(void (KisIteratorTest::* funcPtr)(const KoColorSpace*cs))
 {
-    QList<QString> csIds = KoColorSpaceRegistry::instance()->keys();
-    int counter = 0;
-    foreach(QString csId, csIds) {
+    QList<const KoColorSpace*> colorsapces = KoColorSpaceRegistry::instance()->allColorSpaces(KoColorSpaceRegistry::AllColorSpaces, KoColorSpaceRegistry::OnlyDefaultProfile);
 
-        qDebug() << "Testing with" << csId;
+    foreach(const KoColorSpace* cs, colorsapces) {
 
-        QList<const KoColorProfile*> profiles = KoColorSpaceRegistry::instance()->profilesFor(csId);
-        if (profiles.size() > 0) {
-            const KoColorSpace * cs = KoColorSpaceRegistry::instance()->colorSpace(csId, profiles[0]);
-            if (cs)(this->*funcPtr)(cs);
-        }
+        qDebug() << "Testing with" << cs->id();
+
+        if (cs->id() != "GRAYU16") // No point in testing extend for GRAYU16
+            (this->*funcPtr)(cs);
 
     }
 }
@@ -55,7 +54,7 @@ void KisIteratorTest::writeBytes(const KoColorSpace * colorSpace)
 
     KisPaintDevice dev(colorSpace);
 
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     // Check allocation on tile boundaries
 
@@ -67,25 +66,25 @@ void KisIteratorTest::writeBytes(const KoColorSpace * colorSpace)
     dev.writeBytes(bytes, 0, 0, 5 * 64, 2 * 64);
 
     // Covers
-    QVERIFY(dev.extent() == QRect(0, 0, 64 * 5, 64 * 2));
-    QVERIFY(dev.exactBounds() == QRect(0, 0, 64 * 5, 64 * 2));
+    QCOMPARE(dev.extent(), QRect(0, 0, 64 * 5, 64 * 2));
+    QCOMPARE(dev.exactBounds(), QRect(0, 0, 64 * 5, 64 * 2));
 
     dev.clear();
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     dev.clear();
     // Covers three by three tiles
     dev.writeBytes(bytes, 10, 10, 130, 130);
 
-    QVERIFY(dev.extent() == QRect(0, 0, 64 * 3, 64 * 3));
-    QVERIFY(dev.exactBounds() == QRect(10, 10, 130, 130));
+    QCOMPARE(dev.extent(), QRect(0, 0, 64 * 3, 64 * 3));
+    QCOMPARE(dev.exactBounds(), QRect(10, 10, 130, 130));
 
     dev.clear();
     // Covers 11 x 2 tiles
     dev.writeBytes(bytes, -10, -10, 10 * 64, 64);
 
-    QVERIFY(dev.extent() == QRect(-64, -64, 64 * 11, 64 * 2));
-    QVERIFY(dev.exactBounds() == QRect(-10, -10, 640, 64));
+    QCOMPARE(dev.extent(), QRect(-64, -64, 64 * 11, 64 * 2));
+    QCOMPARE(dev.exactBounds(), QRect(-10, -10, 640, 64));
 
     delete[] bytes;
 }
@@ -95,24 +94,24 @@ void KisIteratorTest::fill(const KoColorSpace * colorSpace)
 
     KisPaintDevice dev(colorSpace);
 
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     quint8 * bytes = colorSpace->allocPixelBuffer(1);
     memset(bytes, 128, colorSpace->pixelSize());
 
     dev.fill(0, 0, 5, 5, bytes);
-    QVERIFY(dev.extent() == QRect(0, 0, 64, 64));
-    QVERIFY(dev.exactBounds() == QRect(0, 0, 5, 5));
+    QCOMPARE(dev.extent(), QRect(0, 0, 64, 64));
+    QCOMPARE(dev.exactBounds(), QRect(0, 0, 5, 5));
 
     dev.clear();
     dev.fill(5, 5, 5, 5, bytes);
-    QVERIFY(dev.extent() == QRect(0, 0, 64, 64));
-    QVERIFY(dev.exactBounds() == QRect(5, 5, 5, 5));
+    QCOMPARE(dev.extent(), QRect(0, 0, 64, 64));
+    QCOMPARE(dev.exactBounds(), QRect(5, 5, 5, 5));
 
     dev.clear();
     dev.fill(5, 5, 500, 500, bytes);
-    QVERIFY(dev.extent() == QRect(0, 0, 8 * 64, 8 * 64));
-    QVERIFY(dev.exactBounds() == QRect(5, 5, 500, 500));
+    QCOMPARE(dev.extent(), QRect(0, 0, 8 * 64, 8 * 64));
+    QCOMPARE(dev.exactBounds(), QRect(5, 5, 500, 500));
 
     dev.clear();
     dev.fill(33, -10, 348, 1028, bytes);
@@ -130,15 +129,15 @@ void KisIteratorTest::rectIter(const KoColorSpace * colorSpace)
     quint8 * bytes = colorSpace->allocPixelBuffer(1);
     memset(bytes, 128, colorSpace->pixelSize());
 
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     {
         // Const does not extend the extent
 
         KisRectConstIteratorPixel cit = dev.createRectConstIterator(0, 0, 128, 128);
         while (!cit.isDone()) ++cit;
-        QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
-        QVERIFY(dev.exactBounds() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+        QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
+        QCOMPARE(dev.exactBounds(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
         // Non-const does
 
@@ -147,8 +146,8 @@ void KisIteratorTest::rectIter(const KoColorSpace * colorSpace)
             memcpy(it.rawData(), bytes, colorSpace->pixelSize());
             ++it;
         }
-        QVERIFY(dev.extent() == QRect(0, 0, 128, 128));
-        QVERIFY(dev.exactBounds() == QRect(0, 0, 128, 128));
+        QCOMPARE(dev.extent(), QRect(0, 0, 128, 128));
+        QCOMPARE(dev.exactBounds(), QRect(0, 0, 128, 128));
     }
 
     dev.clear();
@@ -159,8 +158,22 @@ void KisIteratorTest::rectIter(const KoColorSpace * colorSpace)
             memcpy(it.rawData(), bytes, colorSpace->pixelSize());
             ++it;
         }
-        QVERIFY(dev.extent() == QRect(0, 0, 3 * 64, 3 * 64));
-        QVERIFY(dev.exactBounds() == QRect(10, 10, 128, 128));
+        QCOMPARE(dev.extent(), QRect(0, 0, 3 * 64, 3 * 64));
+        QCOMPARE(dev.exactBounds(), QRect(10, 10, 128, 128));
+    }
+
+    dev.clear();
+    dev.setX(10);
+    dev.setY(-15);
+
+    {
+        KisRectIteratorPixel it = dev.createRectIterator(10, 10, 128, 128);
+        while (!it.isDone()) {
+            memcpy(it.rawData(), bytes, colorSpace->pixelSize());
+            ++it;
+        }
+        QCOMPARE(dev.extent(), QRect(10, -15, 128, 192));
+        QCOMPARE(dev.exactBounds(), QRect(10, 10, 128, 128));
     }
 
     delete[] bytes;
@@ -173,12 +186,12 @@ void KisIteratorTest::hLineIter(const KoColorSpace * colorSpace)
     quint8 * bytes = colorSpace->allocPixelBuffer(1);
     memset(bytes, 128, colorSpace->pixelSize());
 
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     KisHLineConstIteratorPixel cit = dev.createHLineConstIterator(0, 0, 128);
     while (!cit.isDone()) ++cit;
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
-    QVERIFY(dev.exactBounds() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.exactBounds(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     {
 
@@ -203,8 +216,8 @@ void KisIteratorTest::hLineIter(const KoColorSpace * colorSpace)
             ++it;
         }
 
-        QVERIFY(dev.extent() == QRect(0, 0, 128, 64));
-        QVERIFY(dev.exactBounds() == QRect(0, 1, 128, 1));
+        QCOMPARE(dev.extent(), QRect(0, 0, 128, 64));
+        QCOMPARE(dev.exactBounds(), QRect(0, 1, 128, 1));
     }
 
     dev.clear();
@@ -216,8 +229,23 @@ void KisIteratorTest::hLineIter(const KoColorSpace * colorSpace)
             ++it;
         }
 
-        QVERIFY(dev.extent() == QRect(0, 0, 192, 64));
-        QVERIFY(dev.exactBounds() == QRect(10, 10, 128, 1));
+        QCOMPARE(dev.extent(), QRect(0, 0, 192, 64));
+        QCOMPARE(dev.exactBounds(), QRect(10, 10, 128, 1));
+    }
+
+    dev.clear();
+    dev.setX(10);
+    dev.setY(-15);
+
+    {
+        KisHLineIteratorPixel it = dev.createHLineIterator(10, 10, 128);
+        while (!it.isDone()) {
+            memcpy(it.rawData(), bytes, colorSpace->pixelSize());
+            ++it;
+        }
+
+        QCOMPARE(dev.extent(), QRect(10, -15, 128, 64));
+        QCOMPARE(dev.exactBounds(), QRect(10, 10, 128, 1));
     }
 
     delete[] bytes;
@@ -230,12 +258,12 @@ void KisIteratorTest::vLineIter(const KoColorSpace * colorSpace)
     quint8 * bytes = colorSpace->allocPixelBuffer(1);
     memset(bytes, 128, colorSpace->pixelSize());
 
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     KisVLineConstIteratorPixel cit = dev.createVLineConstIterator(0, 0, 128);
     while (!cit.isDone()) ++cit;
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
-    QVERIFY(dev.exactBounds() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.exactBounds(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     {
         KisVLineIteratorPixel it = dev.createVLineIterator(0, 0, 128);
@@ -259,6 +287,21 @@ void KisIteratorTest::vLineIter(const KoColorSpace * colorSpace)
         QCOMPARE(dev.extent(), QRect(0, 0, 64, 192));
         QCOMPARE(dev.exactBounds(), QRect(10, 10, 1, 128));
     }
+    
+    dev.clear();
+    dev.setX(10);
+    dev.setY(-15);
+
+    {
+        KisVLineIteratorPixel it = dev.createVLineIterator(10, 10, 128);
+        while (!it.isDone()) {
+            memcpy(it.rawData(), bytes, colorSpace->pixelSize());
+            ++it;
+        }
+
+        QCOMPARE(dev.extent(), QRect(10, -15, 64, 192));
+        QCOMPARE(dev.exactBounds(), QRect(10, 10, 1, 128));
+    }
 
     delete[] bytes;
 
@@ -271,7 +314,7 @@ void KisIteratorTest::randomAccessor(const KoColorSpace * colorSpace)
     quint8 * bytes = colorSpace->allocPixelBuffer(1);
     memset(bytes, 128, colorSpace->pixelSize());
 
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     KisRandomConstAccessorPixel acc = dev.createRandomConstAccessor(0, 0);
     for (int y = 0; y < 128; ++y) {
@@ -279,7 +322,7 @@ void KisIteratorTest::randomAccessor(const KoColorSpace * colorSpace)
             acc.moveTo(x, y);
         }
     }
-    QVERIFY(dev.extent() == QRect(qint32_MAX, qint32_MAX, 0, 0));
+    QCOMPARE(dev.extent(), QRect(qint32_MAX, qint32_MAX, 0, 0));
 
     KisRandomAccessorPixel ac = dev.createRandomAccessor(0, 0);
     for (int y = 0; y < 128; ++y) {
@@ -288,12 +331,60 @@ void KisIteratorTest::randomAccessor(const KoColorSpace * colorSpace)
             memcpy(ac.rawData(), bytes, colorSpace->pixelSize());
         }
     }
-    QVERIFY(dev.extent() == QRect(0, 0, 128, 128));
-    QVERIFY(dev.exactBounds() == QRect(0, 0, 128, 128));
+    QCOMPARE(dev.extent(), QRect(0, 0, 128, 128));
+    QCOMPARE(dev.exactBounds(), QRect(0, 0, 128, 128));
+    
+    dev.clear();
+    dev.setX(10);
+    dev.setY(-15);
 
+    {
+        KisRandomAccessorPixel ac = dev.createRandomAccessor(0, 0);
+        for (int y = 0; y < 128; ++y) {
+            for (int x = 0; x < 128; ++x) {
+                ac.moveTo(x, y);
+                memcpy(ac.rawData(), bytes, colorSpace->pixelSize());
+            }
+        }
+        QCOMPARE(dev.extent(), QRect(-54, -15, 192, 192));
+        QCOMPARE(dev.exactBounds(), QRect(0, 0, 128, 128));
+    }
     delete[] bytes;
 }
 
+void KisIteratorTest::repeatHLineIter(const KoColorSpace* cs)
+{
+    KoColor color(Qt::green, cs);
+    
+    KisPaintDeviceSP dev = new KisPaintDevice(cs);
+    dev->fill(5, 5, 10, 10, color.data());
+
+    KisRepeatHLineConstIteratorPixel iter = dev->createRepeatHLineConstIterator(0, 0, 20, QRect(5, 5, 10, 10));
+    for(int i = 0; i < 20; i++) {
+        while(!iter.isDone()) {
+            QVERIFY(!memcmp(color.data(), iter.oldRawData(), cs->pixelSize()));
+            ++iter;
+        }
+        iter.nextRow();
+    }
+}
+
+void KisIteratorTest::repeatVLineIter(const KoColorSpace* cs)
+{
+    KoColor color(Qt::green, cs);
+    
+    KisPaintDeviceSP dev = new KisPaintDevice(cs);
+    dev->fill(5, 5, 10, 10, color.data());
+
+    KisRepeatVLineConstIteratorPixel iter = dev->createRepeatVLineConstIterator(0, 0, 20, QRect(5, 5, 10, 10));
+    for(int i = 0; i < 20; i++) {
+        while(!iter.isDone()) {
+            QVERIFY(!memcmp(color.data(), iter.oldRawData(), cs->pixelSize()));
+            ++iter;
+        }
+        iter.nextCol();
+    }
+}
 
 void KisIteratorTest::writeBytes()
 {
@@ -325,5 +416,15 @@ void KisIteratorTest::randomAccessor()
     allCsApplicator(&KisIteratorTest::randomAccessor);
 }
 
+void KisIteratorTest::repeatHLineIter()
+{
+    allCsApplicator(&KisIteratorTest::repeatHLineIter);
+}
+
+void KisIteratorTest::repeatVLineIter()
+{
+    allCsApplicator(&KisIteratorTest::repeatVLineIter);
+}
+
 QTEST_KDEMAIN(KisIteratorTest, NoGUI)
 #include "kis_iterator_test.moc"
