@@ -25,6 +25,7 @@ class KisPaintDeviceTest : public QObject
 {
     Q_OBJECT
 
+private slots:
 
     void testCreation();
     void testStore();
@@ -32,17 +33,19 @@ class KisPaintDeviceTest : public QObject
     void testClear();
     void testCrop();
     void testThumbnail();
+    void testCaching();
+    void testRegion();
     void testPixel();
     void testRoundtripReadWrite();
     void testPlanarReadWrite();
     void testRoundtripConversion();
+    void testFastBitBlt();
+    void testMakeClone();
     void testBltPerformance();
-
-
-private slots:
-
-
     void testColorSpaceConversion();
+    void testDeviceDuplication();
+    void testTranslate();
+    void testOpacity();
 };
 
 #endif
