@@ -21,6 +21,8 @@
 
 #include <QtTest/QtTest>
 
+#include "kis_cubic_curve.h"
+
 class KisPropertiesConfiguration;
 
 class KisPropertiesConfigurationTest : public QObject
@@ -32,6 +34,7 @@ private slots:
     void testSetGet();
     void testSerialization();
     void testDefaultValues();
+    void testNotSavedValues();
 private:
     KisPropertiesConfiguration* createConfig();
     void testConfig(KisPropertiesConfiguration* config);
@@ -40,6 +43,7 @@ private:
     QString v2;
     double v3;
     bool v4;
+    KisCubicCurve v5;
 };
 
 #endif
