@@ -27,9 +27,19 @@ class RulerAssistant : public KisPaintingAssistant
 public:
     RulerAssistant();
     virtual QPointF adjustPosition(const QPointF& point) const;
-    virtual void drawAssistant(QPainter& gc, const QPoint& documentOffset,  const QRect& area, const KoViewConverter &converter) const;
+    void drawAssistant(QPainter& gc, const QRectF& updateRect, const KisCoordinatesConverter *converter);
 private:
     QPointF project(const QPointF& pt) const;
 };
 
+class RulerAssistantFactory : public KisPaintingAssistantFactory
+{
+public:
+    RulerAssistantFactory();
+    virtual ~RulerAssistantFactory();
+    virtual QString id() const;
+    virtual QString name() const;
+    virtual KisPaintingAssistant* paintingAssistant(const QRectF& imageArea) const;
+};
+
 #endif
