@@ -34,7 +34,7 @@
 #include <kmessagebox.h>
 #include <kstandarddirs.h>
 #include <kis_debug.h>
-#include <kgenericfactory.h>
+#include <kpluginfactory.h>
 #include <knuminput.h>
 
 #include <kis_doc2.h>
@@ -59,7 +59,7 @@
 #include <kis_convolution_painter.h>
 #include <commands/kis_node_commands.h>
 #include <kis_layer_manager.h>
-
+#include <kis_node_commands_adapter.h>
 
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
 #define MAX(a, b) (((a) > (b)) ? (a) : (b))
@@ -79,20 +79,20 @@ void KisDropshadow::dropshadow(KoUpdater * progressUpdater,
 {
     QColor color = c;
 
+    if(!m_view) return;
+    
     KisImageWSP image = m_view->image();
     if (!image) return;
 
     KisLayerSP src = m_view->activeLayer();
     if (!src) return;
 
-    KisPaintDeviceSP dev = m_view->activeDevice();
+    KisPaintDeviceSP dev = src->projection();
     if (!dev) return;
 
-    if (image->undo()) {
-        image->undoAdapter()->beginMacro(i18n("Add Drop Shadow"));
-    }
+    image->undoAdapter()->beginMacro(i18n("Add Drop Shadow"));
 
-    KisPaintDeviceSP shadowDev = new KisPaintDevice(KoColorSpaceRegistry::instance()->colorSpace("RGBA", 0));
+    KisPaintDeviceSP shadowDev = new KisPaintDevice(KoColorSpaceRegistry::instance()->rgb8());
     KisPaintDeviceSP bShadowDev;
     const KoColorSpace *rgb8cs = shadowDev->colorSpace();
 
@@ -106,7 +106,7 @@ void KisDropshadow::dropshadow(KoUpdater * progressUpdater,
             while (! srcIt.isDone()) {
                 if (srcIt.isSelected()) {
                     //set the shadow color
-                    quint8 alpha = dev->colorSpace()->alpha(srcIt.rawData());
+                    quint8 alpha = dev->colorSpace()->opacityU8(srcIt.rawData());
                     color.setAlpha(alpha);
                     rgb8cs->fromQColor(color, dstIt.rawData());
                 }
@@ -119,7 +119,7 @@ void KisDropshadow::dropshadow(KoUpdater * progressUpdater,
         }
     }
     if (blurradius > 0) {
-        bShadowDev = new KisPaintDevice(KoColorSpaceRegistry::instance()->colorSpace("RGBA", 0));
+        bShadowDev = new KisPaintDevice(KoColorSpaceRegistry::instance()->rgb8());
         gaussianblur(progressUpdater, shadowDev, bShadowDev, rect, blurradius, blurradius, BLUR_RLE, progressUpdater);
         shadowDev = bShadowDev;
     }
@@ -127,12 +127,9 @@ void KisDropshadow::dropshadow(KoUpdater * progressUpdater,
     if (!progressUpdater->interrupted()) {
         shadowDev->move(xoffset, yoffset);
 
-        KisGroupLayerSP parent = image->rootLayer();
-        if (m_view)
-            parent = dynamic_cast<KisGroupLayer*>(m_view->activeLayer()->parent().data());
-
         KisPaintLayerSP l = KisPaintLayerSP(new KisPaintLayer(image.data(), i18n("Drop Shadow"), opacity, shadowDev));
-        image->addNode(l.data(), parent.data(), src->nextSibling());
+        KisNodeCommandsAdapter commandAdapter(m_view);
+        commandAdapter.addNode(l, src->parent(), src->prevSibling());
 
         if (allowResize) {
             QRect shadowBounds = shadowDev->exactBounds();
@@ -155,7 +152,7 @@ void KisDropshadow::dropshadow(KoUpdater * progressUpdater,
                     }
 
                     QUndoCommand *moveCommand = new KisNodeMoveCommand(image->rootLayer(),
-                            QPoint(image->rootLayer()->x(), image->rootLayer()->y()), QPoint(newRootX, newRootY));
+                            QPoint(image->rootLayer()->x(), image->rootLayer()->y()), QPoint(newRootX, newRootY), image);
                     Q_ASSERT(moveCommand != 0);
 
                     m_view->document()->addCommand(moveCommand);
@@ -166,9 +163,7 @@ void KisDropshadow::dropshadow(KoUpdater * progressUpdater,
         m_view->image()->setModified();
     }
 
-    if (image->undo()) {
-        image->undoAdapter()->endMacro();
-    }
+    image->undoAdapter()->endMacro();
 
     progressUpdater->setProgress(100);
 }
