@@ -22,9 +22,10 @@
 #ifndef PAINTERLY_MIXER_H
 #define PAINTERLY_MIXER_H
 
+#include <QVariant>
+
 #include <kparts/plugin.h>
 
-class KisView2;
 class KisPainterlyMixerDockerFactory;
 
 class PainterlyMixer : public KParts::Plugin
@@ -32,12 +33,7 @@ class PainterlyMixer : public KParts::Plugin
     Q_OBJECT
 
 public:
-    PainterlyMixer(QObject *parent, const QStringList &);
-    virtual ~PainterlyMixer();
-
-private:
-    KisView2* m_view;
-    KisPainterlyMixerDockerFactory* m_factory;
+    PainterlyMixer(QObject *parent, const QVariantList &);
 };
 
 #endif // PAINTERLY_MIXER_H
