@@ -18,6 +18,8 @@
 
 #include "kis_simple_noise_reducer.h"
 
+#include <QUndoCommand>
+
 #include <KoColorSpace.h>
 #include <KoCompositeOp.h>
 #include <KoProgressUpdater.h>
@@ -38,7 +40,6 @@ KisSimpleNoiseReducer::KisSimpleNoiseReducer()
         : KisFilter(id(), categoryEnhance(), i18n("&Gaussian Noise Reduction"))
 {
     setSupportsPainting(false);
-    setSupportsPreview(true);
     setSupportsIncrementalPainting(false);
 }
 
@@ -99,15 +100,16 @@ void KisSimpleNoiseReducer::process(KisConstProcessingInformation srcInfo,
     const KoColorSpace* cs = src->colorSpace();
 
     // Compute the blur mask
-    KisCircleMaskGenerator* kas = new KisCircleMaskGenerator(2*windowsize + 1, 2*windowsize + 1, windowsize, windowsize);
+    KisCircleMaskGenerator* kas = new KisCircleMaskGenerator(2*windowsize + 1, 1, windowsize, windowsize, 2);
 
-    KisConvolutionKernelSP kernel = KisConvolutionKernel::kernelFromMaskGenerator(kas);
+    KisConvolutionKernelSP kernel = KisConvolutionKernel::fromMaskGenerator(kas);
     delete kas;
 
     KisPaintDeviceSP interm = new KisPaintDevice(*src); // TODO no need for a full copy and then a transaction
     KisConvolutionPainter painter(interm);
     painter.beginTransaction("bouuh");
     painter.applyMatrix(kernel, interm, srcTopLeft, dstTopLeft, size, BORDER_REPEAT);
+    painter.deleteTransaction();
 
     if (progressUpdater && progressUpdater->interrupted()) {
         return;
