@@ -21,15 +21,17 @@
 #ifndef NOISEFILTER_H
 #define NOISEFILTER_H
 
-#include <kparts/plugin.h>
+#include <QObject>
+#include <QVariant>
 #include "filter/kis_filter.h"
 
 class KisConfigWidget;
 
-class KritaNoiseFilter : public KParts::Plugin
+class KritaNoiseFilter : public QObject
 {
+    Q_OBJECT
 public:
-    KritaNoiseFilter(QObject *parent, const QStringList &);
+    KritaNoiseFilter(QObject *parent, const QVariantList &);
     virtual ~KritaNoiseFilter();
 };
 
