@@ -50,51 +50,43 @@
 #include <kis_paintop.h>
 #include <kis_properties_configuration.h>
 #include <kis_selection.h>
-#include <kis_brush_option.h>
+#include <kis_brush_option_widget.h>
 #include <kis_paintop_options_widget.h>
-#include <kis_pressure_darken_option.h>
-#include <kis_pressure_opacity_option.h>
-#include <kis_pressure_size_option.h>
 #include <kis_paint_action_type_option.h>
 #include <kis_bidirectional_mixing_option.h>
 
 #include "kis_complexop_settings.h"
 #include "kis_complexop_settings_widget.h"
 
-KisComplexOp::KisComplexOp(const KisComplexOpSettings *settings, KisPainter *painter)
-        : KisBrushBasedPaintOp(painter)
+KisComplexOp::KisComplexOp(const KisComplexOpSettings *settings, KisPainter *painter, KisImageWSP image)
+        : KisBrushBasedPaintOp(settings, painter)
         , settings(settings)
 {
+    Q_UNUSED(image);
     Q_ASSERT(settings);
     Q_ASSERT(painter);
-    if (settings && settings->m_options) {
-        Q_ASSERT(settings->m_options->m_brushOption);
-        m_brush = settings->m_options->m_brushOption->brush();
-    }
+    m_sizeOption.readOptionSetting(settings);
+    m_darkenOption.readOptionSetting(settings);
+    m_opacityOption.readOptionSetting(settings);
+    m_bidiOption.readOptionSetting(settings);
 }
 
 KisComplexOp::~KisComplexOp()
 {
 }
 
-void KisComplexOp::paintAt(const KisPaintInformation& info)
+qreal KisComplexOp::paintAt(const KisPaintInformation& info)
 {
-    if (!painter()->device()) return;
+    if (!painter()->device()) return 1.0;
 
     KisBrushSP brush = m_brush;
-    if (!m_brush) {
-        if (settings->m_options) {
-            m_brush = settings->m_options->m_brushOption->brush();
-            brush = m_brush;
-        } else {
-            return;
-        }
-    }
+    if (!m_brush)
+        return 1.0;
 
     if (! brush->canPaintFor(info))
-        return;
+        return 1.0;
 
-    double scale = KisPaintOp::scaleForPressure(settings->m_options->m_sizeOption->apply(info));
+    qreal scale = KisPaintOp::scaleForPressure(m_sizeOption.apply(info));
 
     KisPaintDeviceSP device = painter()->device();
 
@@ -105,25 +97,20 @@ void KisComplexOp::paintAt(const KisPaintInformation& info)
     // is where the dab will be positioned and the fractional part determines
     // the sub-pixel positioning.
     qint32 x;
-    double xFraction;
+    qreal xFraction;
     qint32 y;
-    double yFraction;
+    qreal yFraction;
 
     splitCoordinate(pt.x(), &x, &xFraction);
     splitCoordinate(pt.y(), &y, &yFraction);
 
-    quint8 origOpacity = settings->m_options->m_opacityOption->apply(painter(), info);
-    KoColor origColor = settings->m_options->m_darkenOption->apply(painter(), info);
+    quint8 origOpacity = m_opacityOption.apply(painter(), info);
+    KoColor origColor = m_darkenOption.apply(painter(), info);
 
     QRect dabRect = QRect(0, 0, brush->maskWidth(scale, 0.0), brush->maskHeight(scale, 0.0));
     QRect dstRect = QRect(x, y, dabRect.width(), dabRect.height());
 
-
-    if (painter()->bounds().isValid()) {
-        dstRect &= painter()->bounds();
-    }
-
-    if (dstRect.isNull() || dstRect.isEmpty() || !dstRect.isValid()) return;
+    if (dstRect.isNull() || dstRect.isEmpty() || !dstRect.isValid()) return 1;
 
     qint32 sx = dstRect.x() - x;
     qint32 sy = dstRect.y() - y;
@@ -132,7 +119,7 @@ void KisComplexOp::paintAt(const KisPaintInformation& info)
 
     KisFixedPaintDeviceSP dab;
     if (brush->brushType() == IMAGE || brush->brushType() == PIPE_IMAGE) {
-        dab = brush->image(device->colorSpace(), scale, 0.0, info, xFraction, yFraction);
+        dab = brush->paintDevice(device->colorSpace(), scale, 0.0, info, xFraction, yFraction);
     } else {
         dab = cachedDab();
         KoColor color = painter()->paintColor();
@@ -140,11 +127,12 @@ void KisComplexOp::paintAt(const KisPaintInformation& info)
         brush->mask(dab, color, scale, scale, 0.0, info, xFraction, yFraction);
     }
 
-    settings->m_options->m_bidiOption->applyFixed(dab, device, painter(), sx, sy, sw, sh, scale, dstRect);
+    m_bidiOption.applyFixed(dab, device, painter(), sx, sy, sw, sh, scale, dstRect);
 
     painter()->bltFixed(dstRect.x(), dstRect.y(), dab, sx, sy, sw, sh);
 
     painter()->setOpacity(origOpacity);
     painter()->setPaintColor(origColor);
 
+    return spacing(info.pressure());
 }
