@@ -23,47 +23,23 @@
 #ifndef KIS_COMPLEXOP_SETTINGS_H_
 #define KIS_COMPLEXOP_SETTINGS_H_
 
-#include <kis_paintop_settings.h>
+#include <kis_brush_based_paintop_settings.h>
 #include <kis_types.h>
 #include "kis_complexop_settings_widget.h"
 
 class QDomElement;
+class QPointF;
+class QPainter;
 
-class KisComplexOpSettings : public QObject, public KisPaintOpSettings
+class KisComplexOpSettings : public KisBrushBasedPaintOpSettings
 {
-    Q_OBJECT
-
+    
 public:
-    using KisPaintOpSettings::fromXML;
-    using KisPaintOpSettings::toXML;
 
     KisComplexOpSettings();
     virtual ~KisComplexOpSettings();
 
     bool paintIncremental();
-
-    void fromXML(const QDomElement& elt);
-    void toXML(QDomDocument& doc, QDomElement& rootElt) const;
-
-    KisPaintOpSettingsSP clone() const;
-
-    // XXX: Hack!
-    void setOptionsWidget(KisPaintOpSettingsWidget* widget) {
-        if (m_options != 0 && m_options->property("owned by settings").toBool()) {
-            delete m_options;
-        }
-        if (!widget) {
-            m_options = 0;
-        } else {
-            m_options = qobject_cast<KisComplexOpSettingsWidget*>(widget);
-            m_options->writeConfiguration(this);
-        }
-    }
-
-public:
-
-    KisComplexOpSettingsWidget *m_options;
-
 };
 
 
