@@ -41,6 +41,7 @@
 #include <KoColorSpace.h>
 #include <KoCompositeOp.h>
 #include <KoInputDevice.h>
+#include <KoColorSpaceRegistry.h>
 
 #include <kis_brush.h>
 #include <kis_datamanager.h>
@@ -50,11 +51,10 @@
 #include <kis_paintop.h>
 #include <kis_properties_configuration.h>
 #include <kis_selection.h>
-#include <kis_brush_option.h>
+#include <kis_brush_option_widget.h>
 #include <kis_paintop_options_widget.h>
 #include <kis_pressure_darken_option.h>
 #include <kis_pressure_opacity_option.h>
-#include <kis_pressure_size_option.h>
 #include <kis_paint_action_type_option.h>
 #include <kis_perspective_grid.h>
 #include <kis_random_sub_accessor.h>
@@ -62,88 +62,82 @@
 
 #include "kis_duplicateop_settings.h"
 #include "kis_duplicateop_settings_widget.h"
+#include "kis_duplicateop_option.h"
 
 
 KisDuplicateOp::KisDuplicateOp(const KisDuplicateOpSettings *settings, KisPainter *painter)
-        : KisBrushBasedPaintOp(painter)
+        : KisBrushBasedPaintOp(settings, painter)
         , settings(settings)
 {
     Q_ASSERT(settings);
     Q_ASSERT(painter);
-    if (settings && settings->m_options) {
-        Q_ASSERT(settings->m_options->m_brushOption);
-        m_brush = settings->m_options->m_brushOption->brush();
-    }
+    m_sizeOption.readOptionSetting(settings);
+    m_healing = settings->getBool(DUPLICATE_HEALING);
+    m_perspectiveCorrection = settings->getBool(DUPLICATE_CORRECT_PERSPECTIVE);
 }
 
 KisDuplicateOp::~KisDuplicateOp()
 {
 }
 
-double KisDuplicateOp::minimizeEnergy(const double* m, double* sol, int w, int h)
+qreal KisDuplicateOp::minimizeEnergy(const qreal* m, qreal* sol, int w, int h)
 {
     int rowstride = 3 * w;
-    double err = 0;
-    memcpy(sol, m, 3* sizeof(double) * w);
+    qreal err = 0;
+    memcpy(sol, m, 3* sizeof(qreal) * w);
     m += rowstride;
     sol += rowstride;
     for (int i = 1; i < h - 1; i++) {
-        memcpy(sol, m, 3* sizeof(double));
+        memcpy(sol, m, 3* sizeof(qreal));
         m += 3; sol += 3;
         for (int j = 3; j < rowstride - 3; j++) {
-            double tmp = *sol;
+            qreal tmp = *sol;
             *sol = ((*(m - 3) + *(m + 3) + *(m - rowstride) + *(m + rowstride)) + 2 * *m) / 6;
-            double diff = *sol - tmp;
+            qreal diff = *sol - tmp;
             err += diff * diff;
             m ++; sol ++;
         }
-        memcpy(sol, m, 3* sizeof(double));
+        memcpy(sol, m, 3* sizeof(qreal));
         m += 3; sol += 3;
     }
-    memcpy(sol, m, 3* sizeof(double) * w);
+    memcpy(sol, m, 3* sizeof(qreal) * w);
     return err;
 }
 
 #define CLAMP(x,l,u) ((x)<(l)?(l):((x)>(u)?(u):(x)))
 
 
-void KisDuplicateOp::paintAt(const KisPaintInformation& info)
+qreal KisDuplicateOp::paintAt(const KisPaintInformation& info)
 {
-    if (!painter()) return;
+    if (!painter()) return 1.0;
 
     if (!m_duplicateStartIsSet) {
         m_duplicateStartIsSet = true;
         m_duplicateStart = info.pos();
     }
 
-    bool heal = settings->healing();
-
-    if (!source()) return;
+    if (!source()) return 1.0;
 
     KisBrushSP brush = m_brush;
-    if (!m_brush) {
-        if (settings->m_options) {
-            m_brush = settings->m_options->m_brushOption->brush();
-            brush = m_brush;
-        } else {
-            return;
-        }
-    }
+    if (!brush)
+        return 1.0;
 
     if (! brush->canPaintFor(info))
-        return;
+        return 1.0;
 
-    double scale = KisPaintOp::scaleForPressure(info.pressure());
+    qreal scale = KisPaintOp::scaleForPressure(m_sizeOption.apply(info));
     QPointF hotSpot = brush->hotSpot(scale, scale);
     QPointF pt = info.pos() - hotSpot;
 
+    setCurrentScale(scale);
+    
     // Split the coordinates into integer plus fractional parts. The integer
     // is where the dab will be positioned and the fractional part determines
     // the sub-pixel positioning.
     qint32 x;
-    double xFraction;
+    qreal xFraction;
     qint32 y;
-    double yFraction;
+    qreal yFraction;
 
     splitCoordinate(pt.x(), &x, &xFraction);
     splitCoordinate(pt.y(), &y, &yFraction);
@@ -169,7 +163,7 @@ void KisDuplicateOp::paintAt(const KisPaintInformation& info)
 
     // Perspective correction ?
     KisImageWSP image = settings->m_image;
-    if (settings->perspectiveCorrection() && image && image->perspectiveGrid()->countSubGrids() == 1) {
+    if (m_perspectiveCorrection && image && image->perspectiveGrid()->countSubGrids() == 1) {
         Matrix3qreal startM = Matrix3qreal::Identity();
         Matrix3qreal endM = Matrix3qreal::Identity();
 
@@ -231,22 +225,22 @@ void KisDuplicateOp::paintAt(const KisPaintInformation& info)
 
     // heal ?
 
-    if (heal) {
+    if (m_healing) {
         quint16 dataDevice[4];
         quint16 dataSrcDev[4];
-        double* matrix = new double[ 3 * sw * sh ];
+        qreal* matrix = new qreal[ 3 * sw * sh ];
         // First divide
         const KoColorSpace* deviceCs = source()->colorSpace();
         KisHLineConstIteratorPixel deviceIt = source()->createHLineConstIterator(x, y, sw);
         KisHLineIteratorPixel srcDevIt = m_srcdev->createHLineIterator(0, 0, sw);
-        double* matrixIt = &matrix[0];
+        qreal* matrixIt = &matrix[0];
         for (int j = 0; j < sh; j++) {
             for (int i = 0; !srcDevIt.isDone(); i++) {
                 deviceCs->toLabA16(deviceIt.rawData(), (quint8*)dataDevice, 1);
                 deviceCs->toLabA16(srcDevIt.rawData(), (quint8*)dataSrcDev, 1);
                 // Division
                 for (int k = 0; k < 3; k++) {
-                    matrixIt[k] = dataDevice[k] / (double)qMax((int)dataSrcDev [k], 1);
+                    matrixIt[k] = dataDevice[k] / (qreal)qMax((int)dataSrcDev [k], 1);
                 }
                 ++deviceIt;
                 ++srcDevIt;
@@ -258,11 +252,11 @@ void KisDuplicateOp::paintAt(const KisPaintInformation& info)
         // Minimize energy
         {
             int iter = 0;
-            double err;
-            double* solution = new double [ 3 * sw * sh ];
+            qreal err;
+            qreal* solution = new qreal [ 3 * sw * sh ];
             do {
                 err = minimizeEnergy(&matrix[0], &solution[0], sw, sh);
-                memcpy(&matrix[0], &solution[0], sw * sh * 3 * sizeof(double));
+                memcpy(&matrix[0], &solution[0], sw * sh * 3 * sizeof(qreal));
                 iter++;
             } while (err < 0.00001 && iter < 100);
             delete [] solution;
@@ -291,29 +285,28 @@ void KisDuplicateOp::paintAt(const KisPaintInformation& info)
         delete [] matrix;
     }
 
-    KisFixedPaintDeviceSP fixedDab = new KisFixedPaintDevice(m_srcdev->colorSpace());
-    fixedDab->setRect(QRect(0, 0, sw, sh));
-    fixedDab->initialize();
-
-    m_srcdev->readBytes(fixedDab->data(), fixedDab->bounds());
-    brush->mask(fixedDab, scale, scale, 0.0, info, xFraction, yFraction);
-    m_srcdev->writeBytes(fixedDab->data(), fixedDab->bounds());
-
-    QRect dabRect = QRect(0, 0, brush->maskWidth(scale, 0.0), brush->maskHeight(scale, 0.0));
-    QRect dstRect = QRect(x, y, dabRect.width(), dabRect.height());
-
-    if (painter()->bounds().isValid()) {
-        dstRect &= painter()->bounds();
+    KisFixedPaintDeviceSP dab = 0;
+    if (brush->brushType() == IMAGE || brush->brushType() == PIPE_IMAGE) {
+        dab = brush->paintDevice(m_srcdev->colorSpace(), scale, 0.0, info, xFraction, yFraction);
+        dab->convertTo(KoColorSpaceRegistry::instance()->alpha8());
+    } else {
+        dab = cachedDab();
+        KoColor color = painter()->paintColor();
+        color.convertTo(dab->colorSpace());
+        brush->mask(dab, color, scale, scale, 0.0, info, xFraction, yFraction);
+        dab->convertTo(KoColorSpaceRegistry::instance()->alpha8());
     }
 
-    if (dstRect.isNull() || dstRect.isEmpty() || !dstRect.isValid()) return;
+    QRect dstRect = QRect(x, y, dab->bounds().width(), dab->bounds().height());
+
+    if (dstRect.isNull() || dstRect.isEmpty() || !dstRect.isValid()) return 1.0;
 
     qint32 sx = dstRect.x() - x;
     qint32 sy = dstRect.y() - y;
     sw = dstRect.width();
     sh = dstRect.height();
 
-    painter()->bitBlt(dstRect.x(), dstRect.y(), m_srcdev, sx, sy, sw, sh);
-
+    painter()->bitBltWithFixedSelection(dstRect.x(), dstRect.y(), m_srcdev, dab, sx, sy, 0, 0, sw, sh);
 
+    return spacing(scale);
 }
