@@ -30,20 +30,17 @@
 #include <KoPointerEvent.h>
 
 #include <kis_image.h>
-#include <kis_brush_option.h>
+#include <kis_brush_option_widget.h>
 #include <kis_paintop_options_widget.h>
 #include <kis_pressure_darken_option.h>
 #include <kis_pressure_opacity_option.h>
 #include <kis_pressure_size_option.h>
 #include <kis_paint_action_type_option.h>
 #include <kis_perspective_grid.h>
-#include <KoViewConverter.h>
-
 
 KisDuplicateOpSettings::KisDuplicateOpSettings(KisImageWSP image)
         : m_image(image)
-        , m_options(0)
-        , m_isOffsetNotUptodate(true)
+        , m_isOffsetNotUptodate(false)
 {
 }
 
@@ -62,42 +59,39 @@ QPointF KisDuplicateOpSettings::offset() const
     return m_offset;
 }
 
-void KisDuplicateOpSettings::mousePressEvent(KoPointerEvent *e)
+bool KisDuplicateOpSettings::mousePressEvent(const KisPaintInformation &info, Qt::KeyboardModifiers modifiers)
 {
-    if (e->modifiers() == Qt::ShiftModifier) {
-        m_position = m_image->documentToPixel(e->point);
+    bool ignoreEvent = true;
+    if (modifiers == Qt::ControlModifier) {
+        m_position = info.pos();
         m_isOffsetNotUptodate = true;
-        e->accept();
+        ignoreEvent = false;
     } else {
         if (m_isOffsetNotUptodate) {
-            m_offset = m_image->documentToPixel(e->point) - m_position;
+            m_offset = info.pos() - m_position;
             m_isOffsetNotUptodate = false;
         }
-        e->ignore();
+        ignoreEvent = true;
     }
-
+    
+    return ignoreEvent;
 }
 
+
 void KisDuplicateOpSettings::activate()
 {
+    KisDuplicateOpSettingsWidget* options = dynamic_cast<KisDuplicateOpSettingsWidget*>(optionsWidget());
+    if(!options)
+        return;
+    
     if (m_image->perspectiveGrid()->countSubGrids() != 1) {
-        m_options->m_duplicateOption->setHealing(false);
-        m_options->m_duplicateOption->setPerspective(false);
+        options->m_duplicateOption->setHealing(false);
+        options->m_duplicateOption->setPerspective(false);
     } else {
-        m_options->m_duplicateOption->setPerspective(false);;
+        options->m_duplicateOption->setPerspective(false);
     }
 }
 
-bool KisDuplicateOpSettings::healing() const
-{
-    return m_options->m_duplicateOption->healing();
-}
-
-bool KisDuplicateOpSettings::perspectiveCorrection() const
-{
-    return m_options->m_duplicateOption->correctPerspective();
-}
-
 void KisDuplicateOpSettings::fromXML(const QDomElement& elt)
 {
     // First, call the parent class fromXML to make sure all the
@@ -107,40 +101,28 @@ void KisDuplicateOpSettings::fromXML(const QDomElement& elt)
     m_offset.setX(elt.attribute("OffsetX", "0.0").toDouble());
     m_offset.setY(elt.attribute("OffsetY", "0.0").toDouble());
     m_isOffsetNotUptodate = false;
-
-    // Then load the properties for all widgets
-    m_options->setConfiguration(this);
 }
 
 void KisDuplicateOpSettings::toXML(QDomDocument& doc, QDomElement& rootElt) const
 {
-
-    // First, make sure all the option widgets have saved their state
-    // to the property configuration
-    KisPropertiesConfiguration * settings = m_options->configuration();
-
     // Then call the parent class fromXML
-    settings->KisPropertiesConfiguration::toXML(doc, rootElt);
+    KisPropertiesConfiguration::toXML(doc, rootElt);
 
     rootElt.setAttribute("OffsetX", QString::number(m_offset.x()));
     rootElt.setAttribute("OffsetY", QString::number(m_offset.y()));
-
-
-    delete settings;
 }
 
 
 KisPaintOpSettingsSP KisDuplicateOpSettings::clone() const
 {
-
-    if (!m_options) return 0;
-    KisDuplicateOpSettings* s = dynamic_cast<KisDuplicateOpSettings*>(m_options->configuration());
+    KisPaintOpSettingsSP setting = KisPaintOpSettings::clone();
+    KisDuplicateOpSettings* s = dynamic_cast<KisDuplicateOpSettings*>(setting.data());
     s->m_image = m_image;
     s->m_offset = m_offset;
     s->m_isOffsetNotUptodate = m_isOffsetNotUptodate;
     s->m_position = m_position;
 
-    return s;
+    return setting;
 
 }
 
@@ -157,30 +139,61 @@ QRectF KisDuplicateOpSettings::duplicateOutlineRect(const QPointF& pos, KisImage
 }
 
 QRectF KisDuplicateOpSettings::paintOutlineRect(const QPointF& pos, KisImageWSP image, OutlineMode _mode) const
-{
-    QRectF dubRect = duplicateOutlineRect(pos, image);
-    if (_mode == CURSOR_IS_OUTLINE) {
-        KisBrushSP brush = m_options->m_brushOption->brush();
-        QPointF hotSpot = brush->hotSpot(1.0, 1.0);
-        QRectF rect = QRect(0, 0, brush->width(), brush->height());
-        rect.translate(pos - hotSpot - QPoint(0.5, 0.5));
-        rect = image->pixelToDocument(rect).translated(pos);
-        dubRect |= rect;
+{    
+    /*QRectF dubRect = duplicateOutlineRect(pos, image);
+    if (_mode == CursorIsOutline) {
+        dubRect |= KisBrushBasedPaintOpSettings::paintOutlineRect(pos, image, _mode);
     }
-    return dubRect;
+    return dubRect;*/
+    
+    if (_mode != CursorIsOutline) return QRectF();
+    
+    QPointF hotSpot = KisBrushBasedPaintOpSettings::brushOutline(pos,_mode).boundingRect().center();
+    QRectF boundRect = brushOutline(pos,_mode).boundingRect();
+    return image->pixelToDocument(boundRect.translated(-hotSpot)).translated(pos);
+    
 }
 
-void KisDuplicateOpSettings::paintOutline(const QPointF& pos, KisImageWSP image, QPainter &painter, const KoViewConverter &converter, OutlineMode _mode) const
+void KisDuplicateOpSettings::paintOutline(const QPointF& pos, KisImageWSP image, QPainter &painter, OutlineMode _mode) const
 {
-    KisBrushSP brush = m_options->m_brushOption->brush();
-    painter.setPen(Qt::black);
-    painter.setBackground(Qt::black);
-    if (_mode == CURSOR_IS_OUTLINE) {
-        QPointF hotSpot = brush->hotSpot(1.0, 1.0);
-        painter.drawEllipse(converter.documentToView(image->pixelToDocument(QRectF(0, 0, brush->width(), brush->height()).translated(- hotSpot - QPoint(1.0, 1.0))).translated(pos)));
-    }
-    QRectF rect2 = converter.documentToView(duplicateOutlineRect(pos, image));
+    painter.save();
+    KisBrushBasedPaintOpSettings::paintOutline(pos, image, painter, _mode);
+    painter.restore();
+    QRectF rect2 = duplicateOutlineRect(pos, image);
     painter.drawLine(rect2.topLeft(), rect2.bottomRight());
     painter.drawLine(rect2.topRight(), rect2.bottomLeft());
 
 }
+QPainterPath KisDuplicateOpSettings::brushOutline(const QPointF& pos, KisPaintOpSettings::OutlineMode mode, qreal scale, qreal rotation) const
+{
+    QPainterPath path; 
+    path = KisBrushBasedPaintOpSettings::brushOutline(QPointF(0.0,0.0),KisPaintOpSettings::CursorIsOutline, scale, rotation);
+    
+    QPainterPath copy(path);
+    QRectF rect2 = copy.boundingRect();
+    if (m_isOffsetNotUptodate) {
+        copy.translate(m_position - pos);
+    } else {
+        copy.translate(-m_offset);
+    }
+    
+    path.addPath(copy);
+    
+    QTransform m;
+    m.scale(0.5,0.5);
+    rect2 = m.mapRect(rect2);
+    
+    path.moveTo(rect2.topLeft());
+    path.lineTo(rect2.bottomRight());
+    
+    path.moveTo(rect2.topRight());
+    path.lineTo(rect2.bottomLeft());
+    
+    if (mode == CursorIsOutline){
+        return path.translated(pos);
+    } else {
+        // workaround?
+        //copy.addEllipse(QRectF(0,0,1,1));
+        return copy.translated(pos);
+    }
+}
