@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2008 Lukas Tvrdy <lukast.dev@gmail.com>
+ *  Copyright (c) 2009-2010 Lukáš Tvrdý <lukast.dev@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#ifndef _BRUSH_H_
-#define _BRUSH_H_
+#ifndef _DYNA_BRUSH_H_
+#define _DYNA_BRUSH_H_
 
 #include <QVector>
 
@@ -29,6 +29,23 @@
 
 #include "filter.h"
 
+class KisDynaProperties{
+public:
+    qreal initWidth;
+    qreal mass;
+    qreal drag;
+    qreal xAngle;
+    qreal yAngle;
+    qreal widthRange;
+    qreal lineSpacing;
+    quint8 action;
+    quint16 diameter;
+    quint16 lineCount;
+    bool enableLine;
+    bool useTwoCircles;
+    bool useFixedAngle;
+};
+
 class DynaBrush
 {
 
@@ -37,127 +54,56 @@ public:
     ~DynaBrush();
     DynaBrush(KoColor inkColor);
     void paint(KisPaintDeviceSP dev, qreal x, qreal y, const KoColor &color);
-
-    // dyna function
-    qreal flerp(qreal f0, qreal f1, qreal p) {
-        return ((f0 *(1.0 - p)) + (f1 * p));
-    }
-
+    void drawSegment(KisPainter &painter);
     int applyFilter(qreal mx, qreal my);
 
-    //setters
-    void setImage(KisImageWSP image) {
-        m_image = image;
+    void updateCursorPosition(const QPointF &point) {
+        m_cursorPos.setX(point.x() / m_canvasWidth);
+        m_cursorPos.setY(point.y() / m_canvasHeight);
     }
 
-    void setInitialWidth(qreal width) {
-        m_width = width;
+    void setProperties(KisDynaProperties * properties){
+        m_properties = properties;
     }
-
-    void setMass(qreal mass) {
-        m_curmass = mass;
+    void setCanvasSize(int width, int height) {
+        m_canvasWidth = width;
+        m_canvasHeight = height;
     }
 
-    void setDrag(qreal drag) {
-        m_curdrag = drag;
-    }
-
-    void useFixedAngle(bool useFixedAngle) {
-        m_mouse.fixedangle = useFixedAngle;
-    }
-
-    void setAngle(qreal xangle, qreal yangle) {
-        m_xangle = xangle;
-        m_yangle = yangle;
-    }
-
-    void setWidthRange(qreal widthRange) {
-        m_widthRange = widthRange;
-    }
-
-    void setAction(int action) {
-        m_action = action;
-    }
-
-    void setCircleRadius(int circleRadius) {
-        m_circleRadius = circleRadius;
-    }
-
-    void setLineCount(int lineCount) {
-        m_lineCount = lineCount;
-    }
-
-    void setLineSpacing(qreal spacing) {
-        m_lineSpacing = spacing;
-    }
-
-    void enableLine(bool enableLine) {
-        m_enableLine = enableLine;
-    }
-
-    void enableTwoCircles(bool twoCircles) {
-        m_twoCircles = twoCircles;
-    }
-
-    void initMouse(const QPointF &point) {
-        if (!m_image) return;
-        m_mousePos.setX(point.x() / m_image->width());
-        //m_fmouse.setY( (m_image->height() - point.y()) / (qreal)m_image->height() );
-        m_mousePos.setY(point.y() / m_image->height());
-    }
-
-    void drawSegment(KisPainter &painter);
-
 
 private:
     void drawCircle(KisPainter &painter, qreal x, qreal y, int radius, int steps);
-
     void drawQuad(KisPainter &painter,
                   QPointF &topRight, QPointF &topLeft,
                   QPointF &bottomLeft, QPointF &bottomRight);
-
     void drawWire(KisPainter &painter,
                   QPointF &topRight,
                   QPointF &topLeft,
                   QPointF &bottomLeft,
                   QPointF &bottomRight);
-
-
     void drawLines(KisPainter &painter,
                    QPointF &prev,
                    QPointF &now,
                    int count);
 
     KoColor m_inkColor;
-    KisImageWSP m_image;
+    
+    int m_canvasWidth;
+    int m_canvasHeight;
 
     int m_counter;
     int m_pixelSize;
 
     QVector<QPointF> m_prevPosition;
-
     qreal m_odelx, m_odely;
 
-    // mouse info
-    QPointF m_mousePos;
-    bool first;
+    // cursor position in document in relative coordinates
+    QPointF m_cursorPos;
+    // filters cursor position
+    DynaFilter m_cursorFilter;
 
-    // settings variables
-    qreal m_width;
-    qreal m_curmass;
-    qreal m_curdrag;
-    DynaFilter m_mouse;
-    qreal m_xangle;
-    qreal m_yangle;
-    qreal m_widthRange;
-
-    // drawing various primitives
-    int m_action;
-    bool m_enableLine, m_twoCircles;
-    qreal m_circleRadius;
-
-    int m_lineCount;
-    qreal m_lineSpacing;
+    bool m_initialized;
+    const KisDynaProperties * m_properties;
 
 };
 
