@@ -24,35 +24,29 @@
 #include "kis_filterop_settings.h"
 
 #include <kis_properties_configuration.h>
-#include <kis_brush_option.h>
 #include <filter/kis_filter.h>
 #include <kis_image.h>
 #include <kis_paint_device.h>
 
 #include <kis_paintop_options_widget.h>
 #include <kis_pressure_size_option.h>
+#include <kis_curve_option_widget.h>
 #include <kis_filter_option.h>
 
 KisFilterOpSettingsWidget::KisFilterOpSettingsWidget(QWidget* parent)
-        : KisPaintOpOptionsWidget(parent)
+        : KisBrushBasedPaintopOptionWidget(parent)
 {
     setObjectName("filter option widget");
 
-    m_brushOption = new KisBrushOption();
-    m_sizeOption = new KisPressureSizeOption();
     m_filterOption = new KisFilterOption();
 
-    addPaintOpOption(m_brushOption);
-    addPaintOpOption(m_sizeOption);
-    addPaintOpOption(m_filterOption);
+    addPaintOpOption(new KisCurveOptionWidget(new KisPressureSizeOption()));
+    addPaintOpOption(new KisFilterOption());
 
 }
 
 KisFilterOpSettingsWidget::~KisFilterOpSettingsWidget()
 {
-    delete m_brushOption;
-    delete m_sizeOption;
-    delete m_filterOption;
 }
 
 KisPropertiesConfiguration* KisFilterOpSettingsWidget::configuration() const
@@ -64,9 +58,4 @@ KisPropertiesConfiguration* KisFilterOpSettingsWidget::configuration() const
     return config;
 }
 
-void KisFilterOpSettingsWidget::setImage(KisImageWSP image)
-{
-    m_brushOption->setImage(image);
-}
-
 #include "kis_filterop_settings_widget.moc"
