@@ -5,6 +5,7 @@
  *  Copyright (c) 2004 Adrian Page <adrian@pagenet.plus.com>
  *  Copyright (c) 2005 Bart Coppens <kde@bartcoppens.be>
  *  Copyright (c) 2007 Cyrille Berger <cberger@cberger.net>
+ *  Copyright (c) 2010 Lukáš Tvrdý <lukast.dev@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -72,7 +73,7 @@ quint32 const GimpV2BrushMagic = ('G' << 24) + ('I' << 16) + ('M' << 8) + ('P' <
 struct KisGbrBrush::Private {
 
     QByteArray data;
-    bool ownData;
+    bool ownData;         /* seems to indicate that @ref data is owned by the brush, but in Qt4.x this is already guaranteed... so in reality it seems more to indicate wether the data is loaded from file (ownData = true) or memory (ownData = false) */
 
     bool useColorAsMask;
 
@@ -86,8 +87,8 @@ struct KisGbrBrush::Private {
 #define DEFAULT_SPACING 0.25
 
 KisGbrBrush::KisGbrBrush(const QString& filename)
-        : KisBrush(filename)
-        , d(new Private)
+    : KisBrush(filename)
+    , d(new Private)
 {
     setBrushType(INVALID);
     d->ownData = true;
@@ -99,8 +100,8 @@ KisGbrBrush::KisGbrBrush(const QString& filename)
 KisGbrBrush::KisGbrBrush(const QString& filename,
                          const QByteArray& data,
                          qint32 & dataPos)
-        : KisBrush(filename)
-        , d(new Private)
+                             : KisBrush(filename)
+                             , d(new Private)
 {
     setBrushType(INVALID);
     d->ownData = false;
@@ -115,21 +116,20 @@ KisGbrBrush::KisGbrBrush(const QString& filename,
 }
 
 KisGbrBrush::KisGbrBrush(KisPaintDeviceSP image, int x, int y, int w, int h)
-        : KisBrush()
-        , d(new Private)
+    : KisBrush()
+    , d(new Private)
 {
     setBrushType(INVALID);
     d->ownData = true;
     d->useColorAsMask = false;
     setHasColor(false);
     setSpacing(DEFAULT_SPACING);
-
     initFromPaintDev(image, x, y, w, h);
 }
 
 KisGbrBrush::KisGbrBrush(const QImage& image, const QString& name)
-        : KisBrush()
-        , d(new Private)
+    : KisBrush()
+    , d(new Private)
 {
     d->ownData = false;
     d->useColorAsMask = false;
@@ -142,8 +142,8 @@ KisGbrBrush::KisGbrBrush(const QImage& image, const QString& name)
 }
 
 KisGbrBrush::KisGbrBrush(const KisGbrBrush& rhs)
-        : KisBrush(rhs)
-        , d(new Private)
+    : KisBrush(rhs)
+    , d(new Private)
 {
     setName(rhs.name());
     *d = *rhs.d;
@@ -231,7 +231,7 @@ bool KisGbrBrush::init()
     QImage::Format imageFormat;
 
     if (bh.bytes == 1) {
-        imageFormat = QImage::Format_RGB32;
+        imageFormat = QImage::Format_Indexed8;
     } else {
         imageFormat = QImage::Format_ARGB32;
     }
@@ -244,7 +244,10 @@ bool KisGbrBrush::init()
 
     qint32 k = bh.header_size;
 
-    if (bh.bytes == 1) {
+        if (bh.bytes == 1) {
+        QVector<QRgb> table;
+        for (int i = 0; i < 256; ++i) table.append(qRgb(i, i, i));
+        m_image.setColorTable(table);
         // Grayscale
 
         if (static_cast<qint32>(k + bh.width * bh.height) > d->data.size()) {
@@ -255,9 +258,11 @@ bool KisGbrBrush::init()
         setHasColor(false);
 
         for (quint32 y = 0; y < bh.height; y++) {
+            uchar *pixel = reinterpret_cast<uchar *>(m_image.scanLine(y));
             for (quint32 x = 0; x < bh.width; x++, k++) {
                 qint32 val = 255 - static_cast<uchar>(d->data[k]);
-                m_image.setPixel(x, y, qRgb(val, val, val));
+                *pixel = val;
+                ++pixel;
             }
         }
     } else if (bh.bytes == 4) {
@@ -271,11 +276,10 @@ bool KisGbrBrush::init()
         setHasColor(true);
 
         for (quint32 y = 0; y < bh.height; y++) {
+            QRgb *pixel = reinterpret_cast<QRgb *>(m_image.scanLine(y));
             for (quint32 x = 0; x < bh.width; x++, k += 4) {
-                m_image.setPixel(x, y, qRgba(d->data[k],
-                                             d->data[k+1],
-                                             d->data[k+2],
-                                             d->data[k+3]));
+                *pixel = qRgba(d->data[k], d->data[k+1], d->data[k+2], d->data[k+3]);
+                ++pixel;
             }
         }
     } else {
@@ -327,7 +331,7 @@ bool KisGbrBrush::saveToDevice(QIODevice* dev) const
     int nameLength = qstrlen(name);
     int wrote;
 
-    bh.header_size = htonl(sizeof(GimpBrushHeader) + nameLength);
+    bh.header_size = htonl(sizeof(GimpBrushHeader) + nameLength + 1);
     bh.version = htonl(2); // Only RGBA8 data needed atm, no cinepaint stuff
     bh.width = htonl(width());
     bh.height = htonl(height());
@@ -347,7 +351,7 @@ bool KisGbrBrush::saveToDevice(QIODevice* dev) const
     if (wrote == -1)
         return false;
 
-    wrote = dev->write(name, nameLength); // No +1 for the trailing NULL it seems...
+    wrote = dev->write(name, nameLength + 1);
     if (wrote == -1)
         return false;
 
@@ -382,16 +386,16 @@ bool KisGbrBrush::saveToDevice(QIODevice* dev) const
     return true;
 }
 
-QImage KisGbrBrush::img() const
+QImage KisGbrBrush::image() const
 {
     if (hasColor() && useColorAsMask()) {
         QImage image = m_image;
-
-        for (int x = 0; x < image.width(); x++) {
-            for (int y = 0; y < image.height(); y++) {
-                QRgb c = image.pixel(x, y);
-                int a = (qGray(c) * qAlpha(c)) / 255;
-                image.setPixel(x, y, qRgba(a, 0, a, a));
+        for (int y = 0; y < image.height(); y++) {
+            QRgb *pixel = reinterpret_cast<QRgb *>(image.scanLine(y));
+            for (int x = 0; x < image.width(); x++) {
+                QRgb c = pixel[x];
+                int a = qGray(c);
+                pixel[x] = qRgba(a, a, a, qAlpha(c));
             }
         }
         return image;
@@ -411,9 +415,9 @@ enumBrushType KisGbrBrush::brushType() const
 }
 
 
-void KisGbrBrush::setImage(const QImage& img)
+void KisGbrBrush::setImage(const QImage& image)
 {
-    KisBrush::setImage(img);
+    KisBrush::setImage(image);
     setValid(true);
 }
 
@@ -433,26 +437,39 @@ void KisGbrBrush::setImage(const QImage& img)
 
 void KisGbrBrush::makeMaskImage()
 {
-    if (!hasColor())
+    if (!hasColor()) {
         return;
-
-    QImage image(width(), height(), QImage::Format_RGB32);
-
-    if (m_image.width() == image.width() && m_image.height() == image.height()) {
-        for (int x = 0; x < width(); x++) {
-            for (int y = 0; y < height(); y++) {
-                QRgb c = m_image.pixel(x, y);
-                int a = (qGray(c) * qAlpha(c)) / 255; // qGray(black) = 0
-                image.setPixel(x, y, qRgba(a, a, a, 255));
-            }
-        }
-
-        setImage(image);
     }
 
+    if (m_image.width() == width() && m_image.height() == height()) {
+        int imageWidth = width();
+        int imageHeight = height();
+        QImage image(imageWidth, imageHeight, QImage::Format_Indexed8);
+        QVector<QRgb> table;
+        for (int i = 0; i < 256; ++i) {
+            table.append(qRgb(i, i, i));
+        }
+        image.setColorTable(table);
+        
+        for (int y = 0; y < imageHeight; y++) {
+            QRgb *pixel = reinterpret_cast<QRgb *>(m_image.scanLine(y));
+            uchar * dstPixel = image.scanLine(y);
+            for (int x = 0; x < imageWidth; x++) {
+                QRgb c = pixel[x];
+                float alpha = qAlpha(c)/255.0f;
+                // linear interpolation with maximum gray value which is transparent in the mask
+                //int a = (qGray(c) * alpha) + ((1.0 - alpha) * 255);
+                // single multiplication version
+                int a = 255 + alpha*(qGray(c) - 255);
+                dstPixel[x] = (uchar)a;
+            }
+        }
+        setImage(image);
+    }
+    
     setBrushType(MASK);
     setHasColor(false);
-    d->useColorAsMask = false;
+    setUseColorAsMask(false);
     resetBoundary();
     clearScaledBrushes();
 }
@@ -465,17 +482,16 @@ KisGbrBrush* KisGbrBrush::clone() const
 void KisGbrBrush::toXML(QDomDocument& d, QDomElement& e) const
 {
     Q_UNUSED(d);
-    e.setAttribute("type", "brush"); // legacy
-    e.setAttribute("brush_type", "kis_gbr_brush");
-    e.setAttribute("name", name()); // legacy
-    e.setAttribute("filename", filename()); // legacy
-    e.setAttribute("brush_filename", filename());
-    e.setAttribute("brush_spacing", spacing());
+    e.setAttribute("type", "gbr_brush");
+    e.setAttribute("filename", shortFilename());
+    e.setAttribute("spacing", spacing());
 }
 
 void KisGbrBrush::setUseColorAsMask(bool useColorAsMask)
 {
     d->useColorAsMask = useColorAsMask;
+    resetBoundary();
+    clearScaledBrushes();
 }
 bool KisGbrBrush::useColorAsMask() const
 {
