@@ -33,9 +33,17 @@ KisBrushSP KisGbrBrushFactory::getOrCreateBrush(const QDomElement& brushDefiniti
 {
 
     KoResourceServer<KisBrush>* rServer = KisBrushServer::instance()->brushServer();
-    QString brushFileName = brushDefinition.attribute("brush_filename", "9circle.gbr");
+    QString brushFileName = brushDefinition.attribute("filename", "9circle.gbr");
     KisBrushSP brush = rServer->getResourceByFilename(brushFileName);
-    double spacing = brushDefinition.attribute("brush_spacing", "1.0").toDouble();
+    //Fallback for files that still use the old format
+    if(!brush) {
+        QFileInfo info(brushFileName);
+        brush = rServer->getResourceByFilename(info.fileName());
+    }
+    if(!brush)
+        return 0;
+    
+    double spacing = brushDefinition.attribute("spacing", "1.0").toDouble();
     brush->setSpacing(spacing);
 
     return brush;
