@@ -1,6 +1,8 @@
 /*
  *  Copyright (c) 2004 Adrian Page <adrian@pagenet.plus.com>
  *  Copyright (c) 2009 Sven Langkamp <sven.langkamp@gmail.com>
+ *  Copyright (c) 2010 Cyrille Berger <cberger@cberger.net>
+ *  Copyright (c) 2010 Lukáš Tvrdý <lukast.dev@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,7 +37,7 @@
 
 #include "kis_brush_registry.h"
 #include "kis_brush_server.h"
-#include "widgets/kis_double_widget.h"
+#include "widgets/kis_slider_spin_box.h"
 
 #include "kis_global.h"
 #include "kis_gbr_brush.h"
@@ -64,7 +66,7 @@ void KisBrushDelegate::paint(QPainter * painter, const QStyleOptionViewItem & op
         return;
 
     QRect itemRect = option.rect;
-    QImage thumbnail = brush->img();
+    QImage thumbnail = brush->image();
 
     if (thumbnail.height() > itemRect.height() || thumbnail.width() > itemRect.width()) {
         thumbnail = thumbnail.scaled(itemRect.size() , Qt::KeepAspectRatio);
@@ -89,11 +91,24 @@ KisBrushChooser::KisBrushChooser(QWidget *parent, const char *name)
 {
     setObjectName(name);
 
+    m_lbScale = new QLabel(i18n("Scale: "), this);
+    m_slScale = new KisDoubleSliderSpinBox(this);
+    m_slScale->setRange(0.0, 2.0, 2);
+    m_slScale->setValue(1.0);
+    QObject::connect(m_slScale, SIGNAL(valueChanged(qreal)), this, SLOT(slotSetItemScale(qreal)));
+
+    m_lbRotation = new QLabel(i18n("Rotation: "), this);
+    m_slRotation = new KisDoubleSliderSpinBox(this);
+    m_slRotation->setRange(0.0, 360, 2);
+    m_slRotation->setValue(0.0);
+    QObject::connect(m_slRotation, SIGNAL(valueChanged(qreal)), this, SLOT(slotSetItemRotation(qreal)));
+
     m_lbSpacing = new QLabel(i18n("Spacing: "), this);
-    m_slSpacing = new KisDoubleWidget(0.0, 10, this, "double_widget");
-    m_slSpacing->setTickPosition(QSlider::TicksBelow);
-    m_slSpacing->setTickInterval(1);
-    QObject::connect(m_slSpacing, SIGNAL(valueChanged(double)), this, SLOT(slotSetItemSpacing(double)));
+    m_slSpacing = new KisDoubleSliderSpinBox(this);
+    m_slSpacing->setRange(0.0, 10, 2);
+    m_slSpacing->setValue(0.1);
+    m_slSpacing->setExponentRatio(3.0);
+    QObject::connect(m_slSpacing, SIGNAL(valueChanged(qreal)), this, SLOT(slotSetItemSpacing(qreal)));
 
     m_chkColorMask = new QCheckBox(i18n("Use color as mask"), this);
     QObject::connect(m_chkColorMask, SIGNAL(toggled(bool)), this, SLOT(slotSetItemUseColorAsMask(bool)));
@@ -123,10 +138,15 @@ KisBrushChooser::KisBrushChooser(QWidget *parent, const char *name)
 
     mainLayout->addLayout(spacingLayout, 1);
 
-    spacingLayout->addWidget(m_lbSpacing, 0, 0);
-    spacingLayout->addWidget(m_slSpacing, 0, 1);
+    spacingLayout->addWidget(m_lbScale, 0, 0);
+    spacingLayout->addWidget(m_slScale, 0, 1);
+    spacingLayout->addWidget(m_lbRotation, 1, 0);
+    spacingLayout->addWidget(m_slRotation, 1, 1);
+    spacingLayout->addWidget(m_lbSpacing, 2, 0);
+    spacingLayout->addWidget(m_slSpacing, 2, 1);
+    spacingLayout->setColumnStretch(1, 3);
 
-    spacingLayout->addWidget(m_chkColorMask, 1, 0, 1, 2);
+    spacingLayout->addWidget(m_chkColorMask, 3, 0, 1, 2);
 
     slotActivatedBrush(m_itemChooser->currentResource());
 }
@@ -137,7 +157,12 @@ KisBrushChooser::~KisBrushChooser()
 
 void KisBrushChooser::setBrush(KisBrushSP _brush)
 {
+    KoResource *resource = static_cast<KoResource*>(_brush.data());
+    m_itemChooser->setCurrentResource( resource );
+    update( resource );
     /*
+      XXX: why is this uncommented?
+
         KisGbrBrush* brush = static_cast<KisGbrBrush*>(_brush.data());
 
         QString text = QString("%1 (%2 x %3)")
@@ -154,7 +179,32 @@ void KisBrushChooser::setBrush(KisBrushSP _brush)
     */
 }
 
-void KisBrushChooser::slotSetItemSpacing(double spacingValue)
+void KisBrushChooser::slotSetItemScale(qreal scaleValue)
+{
+    KoResource * resource = static_cast<KoResource *>(m_itemChooser->currentResource());
+
+    if (resource) {
+        KisBrush *brush = static_cast<KisBrush *>(resource);
+        brush->setScale(scaleValue);
+        slotActivatedBrush(brush);
+
+        emit sigBrushChanged();
+    }
+}
+void KisBrushChooser::slotSetItemRotation(qreal rotationValue)
+{
+    KoResource * resource = static_cast<KoResource *>(m_itemChooser->currentResource());
+
+    if (resource) {
+        KisBrush *brush = static_cast<KisBrush *>(resource);
+        brush->setAngle(rotationValue / 180.0 * M_PI);
+        slotActivatedBrush(brush);
+
+        emit sigBrushChanged();
+    }
+}
+
+void KisBrushChooser::slotSetItemSpacing(qreal spacingValue)
 {
     KoResource * resource = static_cast<KoResource *>(m_itemChooser->currentResource());
 
@@ -182,8 +232,8 @@ void KisBrushChooser::slotSetItemUseColorAsMask(bool useColorAsMask)
 
 void KisBrushChooser::update(KoResource * resource)
 {
-    KisGbrBrush* brush = static_cast<KisGbrBrush*>(resource);
-
+    KisBrush* brush = static_cast<KisBrush*>(resource);
+    
     QString text = QString("%1 (%2 x %3)")
                    .arg(i18n(brush->name().toUtf8().data()))
                    .arg(brush->width())
@@ -191,9 +241,16 @@ void KisBrushChooser::update(KoResource * resource)
 
     m_lbName->setText(text);
     m_slSpacing->setValue(brush->spacing());
-    m_chkColorMask->setChecked(brush->useColorAsMask());
+    m_slRotation->setValue(brush->angle() * 180 / M_PI);
+    m_slScale->setValue(brush->scale());
+    
+    
+    // useColorAsMask support is only in gimp brush so far
+    if (KisGbrBrush * gimpBrush = dynamic_cast<KisGbrBrush*>(resource)){
+        m_chkColorMask->setChecked(gimpBrush->useColorAsMask());
+    }
     m_chkColorMask->setEnabled(brush->hasColor());
-
+    
     emit sigBrushChanged();
 }
 
@@ -205,4 +262,19 @@ void KisBrushChooser::slotActivatedBrush(KoResource * resource)
     }
 }
 
+void KisBrushChooser::setBrushSize(qreal xPixels, qreal yPixels)
+{
+        Q_UNUSED(yPixels);
+        qreal oldWidth = m_brush->width() * m_brush->scale(); // or maybe m_slScale->value()
+        qreal newWidth = oldWidth + xPixels;
+        if (newWidth <= 0.0) {
+            return;
+        }
+        
+        qreal newScale = newWidth / m_brush->width();
+        // signal valueChanged will care about call to slotSetItemScale
+        m_slScale->setValue(newScale);
+}
+
+
 #include "kis_brush_chooser.moc"
