@@ -22,10 +22,11 @@
 #include <krita_export.h>
 #include <kis_brush.h>
 
+class KisDoubleSliderSpinBox;
 class QLabel;
 class QCheckBox;
 
-class KisDoubleWidget;
+class KisDoubleSliderSpinBox;
 class KoResourceItemChooser;
 class KoResource;
 
@@ -42,11 +43,14 @@ public:
         return m_brush;
     };
 
-    void setBrush(KisBrushSP);
+    void setBrush(KisBrushSP _brush);
+    void setBrushSize(qreal xPixels,qreal yPixels);
 
 private slots:
 
-    void slotSetItemSpacing(double);
+    void slotSetItemScale(qreal);
+    void slotSetItemRotation(qreal);
+    void slotSetItemSpacing(qreal);
     void slotSetItemUseColorAsMask(bool);
     void slotActivatedBrush(KoResource *);
     void update(KoResource *);
@@ -57,8 +61,12 @@ signals:
 
 private:
     QLabel* m_lbName;
+    QLabel* m_lbRotation;
+    QLabel* m_lbScale;
     QLabel* m_lbSpacing;
-    KisDoubleWidget* m_slSpacing;
+    KisDoubleSliderSpinBox* m_slRotation;
+    KisDoubleSliderSpinBox* m_slScale;
+    KisDoubleSliderSpinBox* m_slSpacing;
     QCheckBox* m_chkColorMask;
     KisBrushSP m_brush;
     KoResourceItemChooser* m_itemChooser;
