@@ -38,21 +38,31 @@ class QString;
 class PAINTOP_EXPORT KisPaintOpOption : public QObject
 {
     Q_OBJECT
+public:
+    static QString brushCategory();
+    static QString colorCategory();
 
 public:
 
-    KisPaintOpOption(const QString & label, bool checked = true);
+    KisPaintOpOption(const QString & label, const QString& category, bool checked );
     virtual ~KisPaintOpOption();
 
-    QString & label() const;
-    bool isCheckable() {
+    QString label() const;
+    QString category() const;
+    virtual bool isCheckable() {
         return m_checkable;
     }
-    bool isChecked() const;
+
+    virtual bool isChecked() const;
+
+    /**
+     * Reimplement this to use the image in the option widget
+     */
+    virtual void setImage(KisImageWSP image);
 
 public slots:
 
-    void setChecked(bool checked);
+    virtual void setChecked(bool checked);
 
     void setConfigurationPage(QWidget * page);
     QWidget * configurationPage() const;
@@ -71,11 +81,6 @@ public slots:
         Q_UNUSED(setting);
     }
 
-    /**
-     * re-implement this to handle actuator events.
-     */
-    //virtual void actuate(KRITA_PAINTOP_OPTION_ACTUATOR actuator);
-
 signals:
 
     /**
