@@ -24,21 +24,13 @@
 #include <KoColorSpace.h>
 
 KisPressureSizeOption::KisPressureSizeOption()
-        : KisCurveOption(i18n("Size"), "Size")
+        : KisCurveOption(i18n("Size"), "Size", KisPaintOpOption::brushCategory(), true )
 {
 }
 
 
-KisPaintInformation KisPressureSizeOption::apply(const KisPaintInformation & info) const
+double KisPressureSizeOption::apply(const KisPaintInformation & info) const
 {
-    KisPaintInformation adjustedInfo(info);
-    if (!isChecked()) {
-        adjustedInfo.setPressure(PRESSURE_DEFAULT);
-    } else {
-        if (customCurve()) {
-            adjustedInfo.setPressure(scaleToCurve(adjustedInfo.pressure()));
-        }
-    }
-
-    return adjustedInfo;
+    if (!isChecked()) return 0.5;
+    return computeValue(info);
 }
