@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2008 Boudewijn Rempt <boud@valdyas.org>
- *  Copyright (c) 2008 Lukas Tvrdy <lukast.dev@gmail.com>
+ *  Copyright (c) 2008-2010 Lukáš Tvrdý <lukast.dev@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,18 +19,17 @@
 #ifndef KIS_SPRAY_PAINTOP_H_
 #define KIS_SPRAY_PAINTOP_H_
 
-//#define BENCHMARK
-
-#include <klocale.h>
 #include <kis_paintop.h>
 #include <kis_types.h>
 
 #include "spray_brush.h"
 #include "kis_spray_paintop_settings.h"
+#include "kis_brush_option.h"
 
 class QPointF;
 class KisPainter;
 
+
 class KisSprayPaintOp : public KisPaintOp
 {
 
@@ -40,24 +38,23 @@ public:
     KisSprayPaintOp(const KisSprayPaintOpSettings *settings, KisPainter * painter, KisImageWSP image);
     virtual ~KisSprayPaintOp();
 
-    double spacing(double & xSpacing, double & ySpacing, double pressure1, double pressure2) const;
-    void paintAt(const KisPaintInformation& info);
-
-    virtual bool incremental() const {
-        return true;
-    }
+    qreal paintAt(const KisPaintInformation& info);
 
 private:
-    const KisSprayPaintOpSettings* m_settings;
-    KisImageWSP m_image;
+    KisShapeProperties m_shapeProperties;
+    KisSprayProperties m_properties;
+    KisShapeDynamicsProperties m_shapeDynamicsProperties;
+    KisColorProperties m_colorProperties;
+    KisBrushOption m_brushOption;
+    
+    const KisSprayPaintOpSettings *m_settings;
+
     KisPaintDeviceSP m_dab;
     SprayBrush m_sprayBrush;
-    double m_xSpacing, m_ySpacing, m_spacing;
-
-#ifdef BENCHMARK
-    int m_total;
-    int m_count;
-#endif
+    qreal m_xSpacing, m_ySpacing, m_spacing;
+    KisPressureRotationOption m_rotationOption;
+    KisPressureSizeOption m_sizeOption;
+    KisPressureOpacityOption m_opacityOption;
 
 };
 
