@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2008,2009 Lukáš Tvrdý <lukast.dev@gmail.com>
+ *  Copyright (c) 2008,2009,2010 Lukáš Tvrdý <lukast.dev@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,10 +21,11 @@
 
 #include <kis_paintop_options_widget.h>
 
-class KisPaintActionTypeOption;
 class KisSprayOpOption;
 class KisSprayShapeOption;
-class KisSprayColorOption;
+class KisSprayShapeDynamicsOption;
+class KisColorOption;
+class KisBrushOptionWidget;
 
 class KisSprayPaintOpSettingsWidget : public KisPaintOpOptionsWidget
 {
@@ -34,13 +35,16 @@ public:
     KisSprayPaintOpSettingsWidget(QWidget* parent = 0);
     virtual ~KisSprayPaintOpSettingsWidget();
 
+    
+    virtual void changePaintOpSize(qreal x, qreal y);
     KisPropertiesConfiguration* configuration() const;
 
 public:
-    KisPaintActionTypeOption* m_paintActionTypeOption;
     KisSprayOpOption* m_sprayOption;
     KisSprayShapeOption* m_sprayShapeOption;
-    KisSprayColorOption* m_sprayColorOption;
+    KisSprayShapeDynamicsOption * m_sprayShapeDynamicOption;
+    KisColorOption* m_ColorOption;
+    KisBrushOptionWidget * m_brushOption;
 };
 
 #endif
