@@ -49,7 +49,11 @@
 #include <kis_view2.h>
 #include <canvas/kis_canvas2.h>
 #include <widgets/kis_cmb_composite.h>
+#include <widgets/kis_slider_spin_box.h>
 #include <kis_cursor.h>
+#include <recorder/kis_recorded_fill_paint_action.h>
+#include <recorder/kis_node_query_path.h>
+#include <recorder/kis_action_recorder.h>
 
 KisToolFill::KisToolFill(KoCanvasBase * canvas)
         : KisToolPaint(canvas, KisCursor::load("tool_fill_cursor.png", 6, 6))
@@ -57,7 +61,7 @@ KisToolFill::KisToolFill(KoCanvasBase * canvas)
     setObjectName("tool_fill");
     m_painter = 0;
     m_oldColor = 0;
-    m_threshold = 15;
+    m_threshold = 80;
     m_usePattern = false;
     m_unmerged = false;
     m_fillOnlySelection = false;
@@ -70,22 +74,34 @@ KisToolFill::~KisToolFill()
 
 bool KisToolFill::flood(int startX, int startY)
 {
+    if (image()) {
+        KisNodeSP projectionNode;
+        if(m_unmerged) {
+            projectionNode = currentNode();
+        } else {
+            projectionNode = image()->root();
+        }
+        KisRecordedFillPaintAction paintAction(KisNodeQueryPath::absolutePath(currentNode()), QPoint(startX, startY), KisNodeQueryPath::absolutePath(projectionNode));
+        setupPaintAction(&paintAction);
+        paintAction.setPattern(currentPattern());
+        if(m_usePattern)
+        {
+            paintAction.setFillStyle(KisPainter::FillStylePattern);
+        }
+        image()->actionRecorder()->addAction(paintAction);
+    }
 
     KisPaintDeviceSP device = currentNode()->paintDevice();
     if (!device) return false;
     KisSelectionSP selection = currentSelection();
 
-    KisCanvas2* canvas = dynamic_cast<KisCanvas2 *>(m_canvas);
+    KisCanvas2* canvas = dynamic_cast<KisCanvas2 *>(this->canvas());
     KoProgressUpdater * updater = canvas->view()->createProgressUpdater(KoProgressUpdater::Unthreaded);
     updater->start(100, i18n("Flood Fill"));
 
     QRegion dirty;
 
     if (m_fillOnlySelection && selection) {
-#ifdef __GNUC__
-#warning Port the fixes for filling the selection from 1.6!
-#endif
-
         QRect rc = selection->selectedRect();
         KisPaintDeviceSP filled = new KisPaintDevice(device->colorSpace());
         KisFillPainter fillPainter(filled);
@@ -103,7 +119,7 @@ bool KisToolFill::flood(int startX, int startY)
                                  currentFgColor(),
                                  m_opacity);
 
-        dirty = fillPainter.dirtyRegion();
+        dirty = fillPainter.takeDirtyRegion();
 
         m_painter = new KisPainter(device, currentSelection());
         Q_CHECK_PTR(m_painter);
@@ -124,7 +140,7 @@ bool KisToolFill::flood(int startX, int startY)
             ++it;
         }
 
-        m_canvas->addCommand(m_painter->endTransaction());
+        m_painter->endTransaction(image()->undoAdapter());
 
     } else {
 
@@ -146,8 +162,8 @@ bool KisToolFill::flood(int startX, int startY)
         else
             fillPainter.fillColor(startX, startY, currentImage()->mergedImage());
 
-        dirty = fillPainter.dirtyRegion();
-        m_canvas->addCommand(fillPainter.endTransaction());
+        fillPainter.endTransaction(image()->undoAdapter());
+        dirty = fillPainter.takeDirtyRegion();
     }
     device->setDirty(dirty);
     delete updater;
@@ -156,32 +172,38 @@ bool KisToolFill::flood(int startX, int startY)
     return true;
 }
 
-void KisToolFill::mousePressEvent(KoPointerEvent *e)
+void KisToolFill::mousePressEvent(KoPointerEvent *event)
 {
-    QPointF pos = convertToPixelCoord(e);
-    m_startPos = pos;
+    if(PRESS_CONDITION(event, KisTool::HOVER_MODE,
+                       Qt::LeftButton, Qt::NoModifier)) {
+
+        setMode(KisTool::PAINT_MODE);
+
+        m_startPos = convertToPixelCoord(event).toPoint();
+    }
+    else {
+        KisToolPaint::mousePressEvent(event);
+    }
 }
 
-void KisToolFill::mouseReleaseEvent(KoPointerEvent *e)
+void KisToolFill::mouseReleaseEvent(KoPointerEvent *event)
 {
+    if(RELEASE_CONDITION(event, KisTool::PAINT_MODE, Qt::LeftButton)) {
+        setMode(KisTool::HOVER_MODE);
 
-    if (!m_canvas) return;
-    if (!currentNode()) return;
-    if (!currentImage() || !currentNode()->paintDevice()) return;
-    if (e->button() == Qt::LeftButton) {
-        int x, y;
+        if (!currentNode() || currentNode()->systemLocked() ||
+            !currentImage()->bounds().contains(m_startPos)) {
 
-        x = static_cast<int>(m_startPos.x());
-        y = static_cast<int>(m_startPos.y());
-
-        if (!currentImage()->bounds().contains(x, y)) {
             return;
         }
 
-        flood(x, y);
+        setCurrentNodeLocked(true);
+        flood(m_startPos.x(), m_startPos.y());
+        setCurrentNodeLocked(false);
         notifyModified();
-    } else {
-        KisToolPaint::mouseReleaseEvent(e);
+    }
+    else {
+        KisToolPaint::mouseReleaseEvent(event);
     }
 }
 
@@ -191,10 +213,10 @@ QWidget* KisToolFill::createOptionWidget()
     QWidget *widget = KisToolPaint::createOptionWidget();
     widget->setObjectName(toolId() + " option widget");
     m_lbThreshold = new QLabel(i18n("Threshold: "), widget);
-    m_slThreshold = new KIntNumInput(widget);
+    m_slThreshold = new KisSliderSpinBox(widget);
     m_slThreshold->setObjectName("int_widget");
     m_slThreshold->setRange(1, 100);
-    m_slThreshold->setSteps(3, 3);
+    m_slThreshold->setPageStep(3);
     m_slThreshold->setValue(m_threshold);
     connect(m_slThreshold, SIGNAL(valueChanged(int)), this, SLOT(slotSetThreshold(int)));
 
