@@ -62,7 +62,7 @@ KisToolDyna::KisToolDyna(KoCanvasBase * canvas)
     m_timer = new QTimer(this);
     Q_CHECK_PTR(m_timer);
     connect(m_timer, SIGNAL(timeout()), this, SLOT(timeoutPaint()));
-
+    
     initDyna();
 }
 
@@ -90,11 +90,11 @@ KisToolDyna::~KisToolDyna()
 
 void KisToolDyna::timeoutPaint()
 {
-    Q_ASSERT(m_painter->paintOp()->incremental());
+    Q_ASSERT(currentPaintOpPreset()->settings()->isAirbrushing());
 
     if (currentImage() && m_painter) {
         paintAt(m_previousPaintInformation);
-        QRegion r = m_painter->dirtyRegion();
+        QRegion r = m_painter->takeDirtyRegion();
         dbgPlugins << "Timeout paint dirty region:" << r;
         currentNode()->setDirty(r);
     }
@@ -104,10 +104,15 @@ void KisToolDyna::timeoutPaint()
 
 void KisToolDyna::initPaint(KoPointerEvent *e)
 {
-    //initDyna();
-    first = false;
-    initMouse(convertToPixelCoord(e->point));
-
+    m_rate = currentPaintOpPreset()->settings()->rate();
+    
+    QRectF imageSize = QRectF(QPointF(0.0,0.0),currentImage()->size());
+    QRectF documentSize = currentImage()->pixelToDocument(imageSize);
+    m_surfaceWidth = documentSize.width();
+    m_surfaceHeight = documentSize.height();
+    setMousePosition(e->point);
+    m_mouse.init(m_mousePos.x(), m_mousePos.y());
+    
     KisToolFreehand::initPaint(e);
 
     if (!m_painter) {
@@ -115,8 +120,7 @@ void KisToolDyna::initPaint(KoPointerEvent *e)
         return;
     }
 
-    m_painter->setPaintOpPreset(currentPaintOpPreset(), currentImage());
-    if (m_painter->paintOp()->incremental()) {
+    if (currentPaintOpPreset()->settings()->isAirbrushing()) {
         m_timer->start(m_rate);
     }
 }
@@ -128,48 +132,159 @@ void KisToolDyna::endPaint()
     KisToolFreehand::endPaint();
 }
 
+void KisToolDyna::mousePressEvent(KoPointerEvent *e)
+{
+    setMousePosition(e->point);
+    m_mouse.init(m_mousePos.x(), m_mousePos.y());
+    m_odelx = m_mousePos.x();
+    m_odely = m_mousePos.y();
+    
+    KisToolFreehand::mousePressEvent(e);
+}
+
 
 void KisToolDyna::mouseMoveEvent(KoPointerEvent *e)
 {
-    if (m_mode != PAINT) return;
-
-    initMouse(convertToPixelCoord(e->point));
-
-    qreal mx, my;
-    mx = m_mousePos.x();
-    my = m_mousePos.y();
-
-    if (!first) {
-        m_mouse.init(mx, my);
-        m_odelx = 0.0;
-        m_odely = 0.0;
-
-        first = true;
-        m_previousPaintInformation = KisPaintInformation(e->point);
-
+    if(!MOVE_CONDITION(event, KisTool::PAINT_MODE)) {
+        KisToolFreehand::mouseMoveEvent(e);
         return;
     }
 
-    if (applyFilter(mx, my)) {
-        drawSegment(e);
+    setMousePosition(e->point);
+
+    if (applyFilter(m_mousePos.x(), m_mousePos.y())) {
+        KoPointerEvent newEvent = filterEvent(e);
+        KisToolFreehand::mouseMoveEvent(&newEvent);
     }
 
-    //KisToolFreehand::mouseMoveEvent(e);
-    if (m_painter && m_painter->paintOp() && m_painter->paintOp()->incremental()) {
+    if (m_painter && m_painter->paintOp() && currentPaintOpPreset()->settings()->isAirbrushing()) {
         m_timer->start(m_rate);
     }
 }
 
-
-
-void KisToolDyna::slotSetSmoothness(int smoothness)
+// dyna algorithm
+int KisToolDyna::applyFilter(qreal mx, qreal my)
 {
-    m_smoothness = smoothness / (double)MAXIMUM_SMOOTHNESS;
+    /* calculate mass and drag */
+    qreal mass = flerp(1.0, 160.0, m_curmass);
+    qreal drag = flerp(0.00, 0.5, m_curdrag * m_curdrag);
+
+    /* calculate force and acceleration */
+    qreal fx = mx - m_mouse.curx;
+    qreal fy = my - m_mouse.cury;
+
+    m_mouse.acc = sqrt(fx * fx + fy * fy);
+
+    if (m_mouse.acc < 0.000001) {
+        return 0;
+    }
+
+    m_mouse.accx = fx / mass;
+    m_mouse.accy = fy / mass;
+
+    /* calculate new velocity */
+    m_mouse.velx += m_mouse.accx;
+    m_mouse.vely += m_mouse.accy;
+    m_mouse.vel = sqrt(m_mouse.velx * m_mouse.velx + m_mouse.vely * m_mouse.vely);
+    m_mouse.angx = -m_mouse.vely;
+    m_mouse.angy = m_mouse.velx;
+    if (m_mouse.vel < 0.000001) {
+        return 0;
+    }
+
+    /* calculate angle of drawing tool */
+    m_mouse.angx /= m_mouse.vel;
+    m_mouse.angy /= m_mouse.vel;
+    if (m_mouse.fixedangle) {
+        m_mouse.angx = m_xangle;
+        m_mouse.angy = m_yangle;
+    }
+
+    m_mouse.velx = m_mouse.velx * (1.0 - drag);
+    m_mouse.vely = m_mouse.vely * (1.0 - drag);
+
+    m_mouse.lastx = m_mouse.curx;
+    m_mouse.lasty = m_mouse.cury;
+    m_mouse.curx = m_mouse.curx + m_mouse.velx;
+    m_mouse.cury = m_mouse.cury + m_mouse.vely;
+
+    return 1;
 }
 
-void KisToolDyna::slotSetMagnetism(int magnetism)
+
+KoPointerEvent KisToolDyna::filterEvent(KoPointerEvent* event)
 {
-    m_magnetism = expf(magnetism / (double)MAXIMUM_MAGNETISM) / expf(1.0);
+    qreal wid = m_widthRange - m_mouse.vel;
+
+    wid = wid * m_width;
+
+    if (wid < 0.00001) {
+        wid = 0.00001;
+    }
+
+    qreal delx = m_mouse.angx * wid;
+    qreal dely = m_mouse.angy * wid;
+
+    qreal px = m_mouse.lastx;
+    qreal py = m_mouse.lasty;
+    qreal nx = m_mouse.curx;
+    qreal ny = m_mouse.cury;
+
+    QPointF prev(px , py);         // previous position
+    QPointF now(nx , ny);           // new position
+
+    QPointF prevr(px + m_odelx , py + m_odely);
+    QPointF prevl(px - m_odelx , py - m_odely);
+
+    QPointF nowl(nx - delx , ny - dely);
+    QPointF nowr(nx + delx , ny + dely);
+
+    // transform coords from float points into image points
+    prev.rx() *= m_surfaceWidth;
+    prevr.rx() *= m_surfaceWidth;
+    prevl.rx() *= m_surfaceWidth;
+    now.rx()  *= m_surfaceWidth;
+    nowl.rx() *= m_surfaceWidth;
+    nowr.rx() *= m_surfaceWidth;
+
+    prev.ry() *= m_surfaceHeight;
+    prevr.ry() *= m_surfaceHeight;
+    prevl.ry() *= m_surfaceHeight;
+    now.ry()  *= m_surfaceHeight;
+    nowl.ry() *= m_surfaceHeight;
+    nowr.ry() *= m_surfaceHeight;
+
+    qreal m_pressure;
+#if 0
+
+    qreal xTilt, yTilt;
+    qreal m_rotation;
+    qreal m_tangentialPressure;
+
+    // some funny debugging
+    dbgPlugins << "m_mouse.vel: " << m_mouse.vel;
+    dbgPlugins << "m_mouse.velx: " << m_mouse.velx;
+    dbgPlugins << "m_mouse.vely: " << m_mouse.vely;
+    dbgPlugins << "m_mouse.accx: " << m_mouse.accx;
+    dbgPlugins << "m_mouse.accy: " << m_mouse.accy;
+
+
+    dbgPlugins << "fixed: " << m_mouse.fixedangle;
+    dbgPlugins << "drag: " << m_curdrag;
+    dbgPlugins << "mass: " << m_curmass;
+    dbgPlugins << "xAngle: " << m_xangle;
+    dbgPlugins << "yAngle: " << m_yangle;
+
+#endif
+
+    m_pressure =  m_mouse.vel * 100;
+    m_pressure = qBound(0.0,m_pressure, 1.0);
+    
+    m_odelx = delx;
+    m_odely = dely;
+    
+    // how to change pressure in the KoPointerEvent???
+    return KoPointerEvent(event,now);
 }
 
 
@@ -214,36 +329,11 @@ void KisToolDyna::slotSetFixedAngle(bool fixedAngle)
     m_mouse.fixedangle = fixedAngle;
 }
 
-
 QWidget * KisToolDyna::createOptionWidget()
 {
 
     QWidget * optionWidget = KisToolFreehand::createOptionWidget();
     optionWidget->setObjectName(toolId() + "option widget");
-    m_chkSmooth = new QCheckBox(i18nc("smooth out the curves while drawing", "Smoothness"), optionWidget);
-    m_chkSmooth->setObjectName("chkSmooth");
-    m_chkSmooth->setChecked(m_smooth);
-    connect(m_chkSmooth, SIGNAL(toggled(bool)), this, SLOT(setSmooth(bool)));
-
-    m_sliderSmoothness = new QSlider(Qt::Horizontal, optionWidget);
-    m_sliderSmoothness->setMinimum(0);
-    m_sliderSmoothness->setMaximum(MAXIMUM_SMOOTHNESS);
-    m_sliderSmoothness->setEnabled(false);
-    connect(m_chkSmooth, SIGNAL(toggled(bool)), m_sliderSmoothness, SLOT(setEnabled(bool)));
-    connect(m_sliderSmoothness, SIGNAL(valueChanged(int)), SLOT(slotSetSmoothness(int)));
-    m_sliderSmoothness->setValue(m_smoothness * MAXIMUM_SMOOTHNESS);
-    // Drawing assistant configuration
-    m_chkAssistant = new QCheckBox(i18n("Assistant:"), optionWidget);
-    connect(m_chkAssistant, SIGNAL(toggled(bool)), this, SLOT(setAssistant(bool)));
-    QLabel* labelMagnetism = new QLabel(i18n("Magnetism:"), optionWidget);
-    connect(m_chkAssistant, SIGNAL(toggled(bool)), labelMagnetism, SLOT(setEnabled(bool)));
-    m_sliderMagnetism = new QSlider(Qt::Horizontal, optionWidget);
-    m_sliderMagnetism->setMinimum(0);
-    m_sliderMagnetism->setMaximum(MAXIMUM_SMOOTHNESS);
-    m_sliderMagnetism->setEnabled(false);
-    connect(m_chkAssistant, SIGNAL(toggled(bool)), m_sliderMagnetism, SLOT(setEnabled(bool)));
-    m_sliderMagnetism->setValue(m_magnetism * MAXIMUM_MAGNETISM);
-    connect(m_sliderMagnetism, SIGNAL(valueChanged(int)), SLOT(slotSetMagnetism(int)));
 
     QLabel* initWidthLbl = new QLabel(i18n("Initial width:"), optionWidget);
     QLabel* massLbl = new QLabel(i18n("Mass:"), optionWidget);
@@ -282,11 +372,6 @@ QWidget * KisToolDyna::createOptionWidget()
     m_optionLayout->setSpacing(2);
 
     KisToolFreehand::addOptionWidgetLayout(m_optionLayout);
-    m_optionLayout->addWidget(m_chkSmooth, 1, 0);
-    m_optionLayout->addWidget(m_sliderSmoothness, 1, 2);
-    m_optionLayout->addWidget(m_chkAssistant, 3, 0);
-    m_optionLayout->addWidget(labelMagnetism, 4, 0);
-    m_optionLayout->addWidget(m_sliderMagnetism, 4, 1, 1, 2);
     m_optionLayout->addWidget(initWidthLbl, 5, 0);
     m_optionLayout->addWidget(m_initWidthSPBox, 5, 1, 1, 2);
     m_optionLayout->addWidget(massLbl, 6, 0);
@@ -304,166 +389,4 @@ QWidget * KisToolDyna::createOptionWidget()
     return optionWidget;
 }
 
-// dyna algorithm
-int KisToolDyna::applyFilter(qreal mx, qreal my)
-{
-    qreal mass, drag;
-    qreal fx, fy;
-
-    /* calculate mass and drag */
-    mass = flerp(1.0, 160.0, m_curmass);
-    drag = flerp(0.00, 0.5, m_curdrag * m_curdrag);
-
-    /* calculate force and acceleration */
-    fx = mx - m_mouse.curx;
-    fy = my - m_mouse.cury;
-
-    m_mouse.acc = sqrt(fx * fx + fy * fy);
-
-    if (m_mouse.acc < 0.000001) {
-        return 0;
-    }
-
-    m_mouse.accx = fx / mass;
-    m_mouse.accy = fy / mass;
-
-    /* calculate new velocity */
-    m_mouse.velx += m_mouse.accx;
-    m_mouse.vely += m_mouse.accy;
-    m_mouse.vel = sqrt(m_mouse.velx * m_mouse.velx + m_mouse.vely * m_mouse.vely);
-    m_mouse.angx = -m_mouse.vely;
-    m_mouse.angy = m_mouse.velx;
-    if (m_mouse.vel < 0.000001) {
-        return 0;
-    }
-
-    /* calculate angle of drawing tool */
-    m_mouse.angx /= m_mouse.vel;
-    m_mouse.angy /= m_mouse.vel;
-    if (m_mouse.fixedangle) {
-        m_mouse.angx = m_xangle;
-        m_mouse.angy = m_yangle;
-    }
-
-    m_mouse.velx = m_mouse.velx * (1.0 - drag);
-    m_mouse.vely = m_mouse.vely * (1.0 - drag);
-
-    m_mouse.lastx = m_mouse.curx;
-    m_mouse.lasty = m_mouse.cury;
-    m_mouse.curx = m_mouse.curx + m_mouse.velx;
-    m_mouse.cury = m_mouse.cury + m_mouse.vely;
-
-    return 1;
-}
-
-
-void KisToolDyna::drawSegment(KoPointerEvent* event)
-{
-    qreal delx, dely;
-    qreal wid;
-    qreal px, py, nx, ny;
-
-    wid = m_widthRange - m_mouse.vel;
-
-    wid = wid * m_width;
-
-    if (wid < 0.00001) {
-        wid = 0.00001;
-    }
-
-    delx = m_mouse.angx * wid;
-    dely = m_mouse.angy * wid;
-
-    px = m_mouse.lastx;
-    py = m_mouse.lasty;
-    nx = m_mouse.curx;
-    ny = m_mouse.cury;
-
-    QPointF prev(px , py);         // previous position
-    QPointF now(nx , ny);           // new position
-
-    QPointF prevr(px + m_odelx , py + m_odely);
-    QPointF prevl(px - m_odelx , py - m_odely);
-
-    QPointF nowl(nx - delx , ny - dely);
-    QPointF nowr(nx + delx , ny + dely);
-
-    // transform coords from float points into image points
-    prev.rx() *= currentImage()->width();
-    prevr.rx() *= currentImage()->width();
-    prevl.rx() *= currentImage()->width();
-    now.rx()  *= currentImage()->width();
-    nowl.rx() *= currentImage()->width();
-    nowr.rx() *= currentImage()->width();
-
-    prev.ry() *= currentImage()->height();
-    prevr.ry() *= currentImage()->height();
-    prevl.ry() *= currentImage()->height();
-    now.ry()  *= currentImage()->height();
-    nowl.ry() *= currentImage()->height();
-    nowr.ry() *= currentImage()->height();
-
-    qreal m_pressure;
-    qreal xTilt, yTilt;
-    qreal m_rotation;
-    qreal m_tangentialPressure;
-
-#if 0
-    // some funny debugging
-    dbgPlugins << "m_mouse.vel: " << m_mouse.vel;
-    dbgPlugins << "m_mouse.velx: " << m_mouse.velx;
-    dbgPlugins << "m_mouse.vely: " << m_mouse.vely;
-    dbgPlugins << "m_mouse.accx: " << m_mouse.accx;
-    dbgPlugins << "m_mouse.accy: " << m_mouse.accy;
-
-
-    dbgPlugins << "fixed: " << m_mouse.fixedangle;
-    dbgPlugins << "drag: " << m_curdrag;
-    dbgPlugins << "mass: " << m_curmass;
-    dbgPlugins << "xAngle: " << m_xangle;
-    dbgPlugins << "yAngle: " << m_yangle;
-
-#endif
-
-    m_pressure =  m_mouse.vel * 100;
-    if (m_pressure > 1.0) m_pressure = 1.0;
-    if (m_pressure < 0.0) m_pressure = 0.0;
-
-    KisPaintInformation pi1 = KisPaintInformation(prev, m_pressure);
-    KisPaintInformation info = KisPaintInformation(now, m_pressure);
-    paintLine(pi1 , info);
-
-    m_previousPaintInformation = info;
-    m_previousPressure = m_pressure;
-
-    m_odelx = delx;
-    m_odely = dely;
-}
-
-void KisToolDyna::mousePressEvent(KoPointerEvent *e)
-{
-    if (!currentNode())
-        return;
-
-    if (!currentNode()->paintDevice())
-        return;
-
-    if (currentPaintOpPreset() && currentPaintOpPreset()->settings()) {
-        m_paintIncremental = currentPaintOpPreset()->settings()->paintIncremental();
-        currentPaintOpPreset()->settings()->mousePressEvent(e);
-        if (e->isAccepted()) {
-            return;
-        }
-    }
-
-    if (e->button() == Qt::LeftButton) {
-        initPaint(e);
-        m_previousPaintInformation = KisPaintInformation(convertToPixelCoord(e->point),
-                                     e->pressure(), e->xTilt(), e->yTilt(),
-                                     KisVector2D::Zero(),
-                                     e->rotation(), e->tangentialPressure());
-    }
-
-}
 #include "kis_tool_dyna.moc"
-
