@@ -3,6 +3,7 @@
  *
  *  Copyright (c) 2004 Michael Thaler <michael.thaler@physik.tu-muenchen.de>
  *  Copyright (c) 2009 Lukáš Tvrdý <lukast.dev@gmail.com>
+ *  Copyright (c) 2010 Cyrille Berger <cberger@cberger.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,6 +35,9 @@
 #include "KoCanvasBase.h"
 #include "KoPointerEvent.h"
 #include <KoCanvasController.h>
+#include <KoShapeController.h>
+#include <KoPathShape.h>
+#include <KoLineBorder.h>
 
 #include <kis_debug.h>
 #include <canvas/kis_canvas2.h>
@@ -41,19 +45,17 @@
 #include <kis_paintop_registry.h>
 #include <kis_cursor.h>
 #include <kis_paint_device.h>
+#include <kis_paint_information.h>
 
 #include "kis_selection.h"
+#include "widgets/kis_slider_spin_box.h"
 
-#include <config-opengl.h>
-
-#ifdef HAVE_OPENGL
-#include <GL/gl.h>
-#endif
-
+#include <recorder/kis_action_recorder.h>
+#include <recorder/kis_recorded_path_paint_action.h>
+#include <recorder/kis_node_query_path.h>
 
 KisToolStar::KisToolStar(KoCanvasBase * canvas)
-        : KisToolShape(canvas, KisCursor::load("tool_star_cursor.png", 6, 6)),
-        m_dragging(false)
+        : KisToolShape(canvas, KisCursor::load("tool_star_cursor.png", 6, 6))
 {
     setObjectName("tool_star");
     m_innerOuterRatio = 40;
@@ -65,20 +67,29 @@ KisToolStar::~KisToolStar()
 }
 void KisToolStar::mousePressEvent(KoPointerEvent *event)
 {
-    if (m_canvas && event->button() == Qt::LeftButton) {
-        m_dragging = true;
+    if(PRESS_CONDITION(event, KisTool::HOVER_MODE,
+                       Qt::LeftButton, Qt::NoModifier)) {
+
+        if (nodePaintAbility() == NONE)
+            return;
+
+        setMode(KisTool::PAINT_MODE);
+
         m_dragStart = convertToPixelCoord(event);
         m_dragEnd = convertToPixelCoord(event);
-        m_vertices = m_optWidget->verticesSpinBox->value();
-        m_innerOuterRatio = m_optWidget->ratioSpinBox->value();
+        m_vertices = m_verticesSlider->value();
+        m_innerOuterRatio = m_ratioSlider->value();
+    }
+    else {
+        KisToolShape::mousePressEvent(event);
     }
 }
 
 void KisToolStar::mouseMoveEvent(KoPointerEvent *event)
 {
-    if (m_dragging) {
+    if(MOVE_CONDITION(event, KisTool::PAINT_MODE)) {
         //Erase old lines
-        m_canvas->updateCanvas(convertToPt(boundingRect()));
+        updatePreview();
         if (event->modifiers() & Qt::AltModifier) {
             QPointF trans = convertToPixelCoord(event) - m_dragEnd;
             m_dragStart += trans;
@@ -86,17 +97,17 @@ void KisToolStar::mouseMoveEvent(KoPointerEvent *event)
         } else {
             m_dragEnd = convertToPixelCoord(event);
         }
-        m_canvas->updateCanvas(convertToPt(boundingRect()));
+        updatePreview();
+    }
+    else {
+        KisToolShape::mouseMoveEvent(event);
     }
 }
 
 void KisToolStar::mouseReleaseEvent(KoPointerEvent *event)
 {
-    if (!m_canvas)
-        return;
-
-    if (m_dragging && event->button() == Qt::LeftButton) {
-        m_dragging = false;
+    if(RELEASE_CONDITION(event, KisTool::PAINT_MODE, Qt::LeftButton)) {
+        setMode(KisTool::HOVER_MODE);
 
         if (m_dragStart == m_dragEnd)
             return;
@@ -107,26 +118,55 @@ void KisToolStar::mouseReleaseEvent(KoPointerEvent *event)
         if (!currentNode())
             return;
 
-
-        if (!currentNode()->paintDevice())
-            return;
-
-        KisPaintDeviceSP device = currentNode()->paintDevice();
-        KisPainter painter(device, currentSelection());
-        painter.beginTransaction(i18n("Star"));
-        setupPainter(&painter);
-        painter.setOpacity(m_opacity);
-        painter.setCompositeOp(m_compositeOp);
-
         vQPointF coord = starCoordinates(m_vertices, m_dragStart.x(), m_dragStart.y(), m_dragEnd.x(), m_dragEnd.y());
+        if (image()) {
+            KisRecordedPathPaintAction linePaintAction(KisNodeQueryPath::absolutePath(currentNode()), currentPaintOpPreset());
+            setupPaintAction(&linePaintAction);
+            linePaintAction.addPolyLine(coord.toList());
+            linePaintAction.addLine(KisPaintInformation(coord.last()), KisPaintInformation(coord.first()));
+            image()->actionRecorder()->addAction(linePaintAction);
+        }
 
-        painter.paintPolygon(coord);
+        if (!currentNode()->inherits("KisShapeLayer")) {
 
-        device->setDirty(painter.dirtyRegion());
-        notifyModified();
-        m_canvas->updateCanvas(convertToPt(boundingRect()));
+            if (!currentNode()->paintDevice())
+                return;
+            setCurrentNodeLocked(true);
+            
+            KisPaintDeviceSP device = currentNode()->paintDevice();
+            KisPainter painter(device, currentSelection());
+            painter.beginTransaction(i18n("Star"));
+            setupPainter(&painter);
 
-        m_canvas->addCommand(painter.endTransaction());
+            painter.paintPolygon(coord);
+
+            device->setDirty(painter.takeDirtyRegion());
+            notifyModified();
+            updatePreview();
+
+            painter.endTransaction(image()->undoAdapter());
+            setCurrentNodeLocked(false);
+        } else {
+            KoPathShape* path = new KoPathShape();
+            path->setShapeId(KoPathShapeId);
+
+            QTransform resolutionMatrix;
+            resolutionMatrix.scale(1 / currentImage()->xRes(), 1 / currentImage()->yRes());
+            path->moveTo(resolutionMatrix.map(coord[0]));
+            for (int i = 1; i < coord.count(); i++)
+                path->lineTo(resolutionMatrix.map(coord[i]));
+            path->close();
+            path->normalize();
+
+            KoLineBorder* border = new KoLineBorder(1.0, currentFgColor().toQColor());
+            path->setBorder(border);
+
+            QUndoCommand * cmd = canvas()->shapeController()->addShape(path);
+            canvas()->addCommand(cmd);
+        }
+    }
+    else {
+        KisToolShape::mouseReleaseEvent(event);
     }
 }
 
@@ -134,63 +174,19 @@ void KisToolStar::paint(QPainter& gc, const KoViewConverter &converter)
 {
     Q_UNUSED(converter);
 
-    if (!m_dragging)
-        return;
-
-    if (!m_canvas)
+    if (mode() != KisTool::PAINT_MODE)
         return;
 
     vQPointF points = starCoordinates(m_vertices, m_dragStart.x(), m_dragStart.y(), m_dragEnd.x(), m_dragEnd.y());
 
-#if defined(HAVE_OPENGL)
-    if (m_canvas->canvasController()->isCanvasOpenGL()) {
-        QPointF begin, end;
-
-        glEnable(GL_LINE_SMOOTH);
-        glEnable(GL_COLOR_LOGIC_OP);
-        glLogicOp(GL_XOR);
-        glColor3f(0.501961, 1.0, 0.501961);
-
-        glBegin(GL_LINE_LOOP);
-        for (int i = 0; i < points.count() - 1; i++) {
-            begin = pixelToView(points[i]);
-            end = pixelToView(points[i + 1]);
-
-            glVertex2f(begin.x(), begin.y());
-            glVertex2f(end.x(), end.y());
-
-        }
-        glEnd();
-
-        glDisable(GL_COLOR_LOGIC_OP);
-        glDisable(GL_LINE_SMOOTH);
-    } else
-#endif
-
-#ifdef INDEPENDENT_CANVAS
-    {
-        QPainterPath path;
-        for (int i = 0; i < points.count() - 1; i++) {
-            path.moveTo(pixelToView(points[i]));
-            path.lineTo(pixelToView(points[i + 1]));
-        }
-        path.moveTo(pixelToView(points[points.count() - 1]));
-        path.lineTo(pixelToView(points[0]));
-        paintToolOutline(&gc, path);
+    QPainterPath path;
+    for (int i = 0; i < points.count() - 1; i++) {
+        path.moveTo(pixelToView(points[i]));
+        path.lineTo(pixelToView(points[i + 1]));
     }
-#else
-    {
-        QPen pen(Qt::SolidLine);
-        gc.setPen(pen);
-
-        for (int i = 0; i < points.count() - 1; i++) {
-            gc.drawLine(pixelToView(points[i]), pixelToView(points[i + 1]));
-        }
-        gc.drawLine(pixelToView(points[points.count() - 1]), pixelToView(points[0]));
-    }
-#endif
-
-
+    path.moveTo(pixelToView(points[points.count() - 1]));
+    path.lineTo(pixelToView(points[0]));
+    paintToolOutline(&gc, path);
 }
 
 vQPointF KisToolStar::starCoordinates(int N, double mx, double my, double x, double y)
@@ -223,28 +219,31 @@ vQPointF KisToolStar::starCoordinates(int N, double mx, double my, double x, dou
     return starCoordinatesArray;
 }
 
-QRectF KisToolStar::boundingRect()
-{
+void KisToolStar::updatePreview() {
     //Calculating the radius
     double radius = sqrt((m_dragEnd.x() - m_dragStart.x()) * (m_dragEnd.x() - m_dragStart.x()) + (m_dragEnd.y() - m_dragStart.y()) * ((m_dragEnd.y() - m_dragStart.y())));
-    return QRectF(m_dragStart.x() - radius, m_dragStart.y() - radius, 2*radius, 2*radius);
+    canvas()->updateCanvas(convertToPt(QRectF(m_dragStart.x() - radius, m_dragStart.y() - radius, 2*radius, 2*radius)));
 }
 
+
 QWidget* KisToolStar::createOptionWidget()
 {
     QWidget *widget = KisToolShape::createOptionWidget();
+    widget->setObjectName(toolId() + "option widget");
 
-    m_optWidget = new WdgToolStar(widget);
-    Q_CHECK_PTR(m_optWidget);
-    m_optWidget->setObjectName(toolId() + " option widget");
+    QLabel *lblVertices = new QLabel(i18n("Vertices:"), widget);
+    m_verticesSlider = new KisSliderSpinBox(widget);
+    m_verticesSlider->setRange(2, 100);
+    m_verticesSlider->setValue(5);
+    addOptionWidgetOption(m_verticesSlider, lblVertices);
 
-    m_optWidget->ratioSpinBox->setValue(m_innerOuterRatio);
+    QLabel *lblRatio = new QLabel(i18n("Ratio:"), widget);
+    m_ratioSlider = new KisSliderSpinBox(widget);
+    m_ratioSlider->setRange(0, 100);
+    m_ratioSlider->setValue(m_innerOuterRatio);
 
-    QGridLayout *optionLayout = new QGridLayout(widget);
-    KisToolShape::addOptionWidgetLayout(optionLayout);
+    addOptionWidgetOption(m_ratioSlider, lblRatio);
 
-    optionLayout->addWidget(m_optWidget, 0, 0);
-    widget->setFixedHeight(widget->sizeHint().height());
     return widget;
 }
 
