@@ -32,7 +32,7 @@
 #include "KoColorSpace.h"
 #include "KoStore.h"
 #include <KoColorSpaceRegistry.h>
-#include <colorprofiles/KoIccColorProfile.h>
+#include <KoColorProfile.h>
 
 // kritaimage
 #include <kis_types.h>
@@ -48,7 +48,6 @@ KisClipboard::KisClipboard()
 {
     m_pushedClipboard = false;
     m_hasClip = false;
-    m_clip = 0;
 
     // Check that we don't already have a clip ready
     clipboardDataChanged();
@@ -72,9 +71,9 @@ KisClipboard* KisClipboard::instance()
     return s_instance;
 }
 
-void KisClipboard::setClip(KisPaintDeviceSP selection)
+void KisClipboard::setClip(KisPaintDeviceSP selection, const QPoint& topLeft)
 {
-    m_clip = selection;
+    dbgUI << selection->exactBounds();
 
     if (!selection)
         return;
@@ -98,9 +97,19 @@ void KisClipboard::setClip(KisPaintDeviceSP selection)
         store->close();
     }
 
+    // Coordinates
+    if (store->open("topLeft")) {
+        store->write(QString("%1 %2").arg(topLeft.x()).arg(topLeft.y()).toAscii());
+        store->close();
+    }
     // ColorSpace id of layer data
-    if (store->open("colorspace")) {
-        QString csName = selection->colorSpace()->id();
+    if (store->open("colormodel")) {
+        QString csName = selection->colorSpace()->colorModelId().id();
+        store->write(csName.toAscii(), strlen(csName.toAscii()));
+        store->close();
+    }
+    if (store->open("colordepth")) {
+        QString csName = selection->colorSpace()->colorDepthId().id();
         store->write(csName.toAscii(), strlen(csName.toAscii()));
         store->close();
     }
@@ -108,16 +117,16 @@ void KisClipboard::setClip(KisPaintDeviceSP selection)
     if (selection->colorSpace()->profile()) {
         const KoColorProfile *profile = selection->colorSpace()->profile();
         KisAnnotationSP annotation;
-        if (profile) {
-            const KoIccColorProfile* iccprofile = dynamic_cast<const KoIccColorProfile*>(profile);
-            if (iccprofile && !iccprofile->rawData().isEmpty())
-                annotation = new  KisAnnotation("icc", iccprofile->name(), iccprofile->rawData());
-        }
-        if (annotation) {
-            // save layer profile
-            if (store->open("profile.icc")) {
-                store->write(annotation->annotation());
-                store->close();
+
+        if (profile && profile->type() == "icc" && !profile->rawData().isEmpty()) {
+            annotation = new  KisAnnotation("icc", profile->name(), profile->rawData());
+
+            if (annotation) {
+                // save layer profile
+                if (store->open("profile.icc")) {
+                    store->write(annotation->annotation());
+                    store->close();
+                }
             }
         }
     }
@@ -125,18 +134,18 @@ void KisClipboard::setClip(KisPaintDeviceSP selection)
     delete store;
 
     // We also create a QImage so we can interchange with other applications
-    QImage qimg;
+    QImage qimage;
     KisConfig cfg;
     QString monitorProfileName = cfg.monitorProfile();
     const KoColorProfile *  monitorProfile = KoColorSpaceRegistry::instance()->profileByName(monitorProfileName);
-    qimg = selection->convertToQImage(monitorProfile);
+    qimage = selection->convertToQImage(monitorProfile);
 
     QMimeData *mimeData = new QMimeData;
     Q_CHECK_PTR(mimeData);
 
     if (mimeData) {
-        if (!qimg.isNull()) {
-            mimeData->setImageData(qimg);
+        if (!qimage.isNull()) {
+            mimeData->setImageData(qimage);
         }
 
         mimeData->setData(mimeType, buffer.buffer());
@@ -147,48 +156,75 @@ void KisClipboard::setClip(KisPaintDeviceSP selection)
     }
 }
 
-KisPaintDeviceSP KisClipboard::clip()
+KisPaintDeviceSP KisClipboard::clip(const QPoint& topLeftHint)
 {
+    bool customTopLeft = false;
+    QPoint topLeft = topLeftHint;
     QClipboard *cb = QApplication::clipboard();
     QByteArray mimeType("application/x-krita-selection");
     const QMimeData *cbData = cb->mimeData();
+    KisPaintDeviceSP clip;
 
     if (cbData && cbData->hasFormat(mimeType)) {
+        dbgUI << "Use clip as x-krita-selection";
         QByteArray encodedData = cbData->data(mimeType);
         QBuffer buffer(&encodedData);
         KoStore* store = KoStore::createStore(&buffer, KoStore::Read, mimeType);
-        KoColorProfile *profile = 0;
+        const KoColorProfile *profile = 0;
+
+        QString csDepth, csModel;
+
+        // topLeft
+        if (store->hasFile("topLeft")) {
+            store->open("topLeft");
+            QString str = store->read(store->size());
+            store->close();
+            QStringList list = str.split(' ');
+            if (list.size() == 2) {
+                topLeft.setX(list[0].toInt());
+                topLeft.setY(list[1].toInt());
+                customTopLeft = true;
+            }
+            dbgUI << str << topLeft;
+        }
+
+        // ColorSpace id of layer data
+        if (store->hasFile("colormodel")) {
+            store->open("colormodel");
+            csModel = QString(store->read(store->size()));
+            store->close();
+        }
+
+        if (store->hasFile("colordepth")) {
+            store->open("colordepth");
+            csDepth = QString(store->read(store->size()));
+            store->close();
+        }
 
         if (store->hasFile("profile.icc")) {
             QByteArray data;
             store->open("profile.icc");
             data = store->read(store->size());
             store->close();
-            profile = new KoIccColorProfile(data);
+            profile = KoColorSpaceRegistry::instance()->createColorProfile(csModel, csDepth, data);
+
         }
 
-        QString csName;
-        // ColorSpace id of layer data
-        if (store->hasFile("colorspace")) {
-            store->open("colorspace");
-            csName = QString(store->read(store->size()));
-            store->close();
-        }
+        const KoColorSpace *cs = KoColorSpaceRegistry::instance()->colorSpace(csModel, csDepth, profile);
 
-        const KoColorSpace *cs = KoColorSpaceRegistry::instance()->colorSpace(csName, profile);
-
-        m_clip = new KisPaintDevice(cs);
+        clip = new KisPaintDevice(cs);
 
         if (store->hasFile("layerdata")) {
             store->open("layerdata");
-            m_clip->read(store);
+            clip->read(store);
             store->close();
         }
         delete store;
     } else {
-        QImage qimg = cb->image();
+        dbgUI << "Use clip as QImage";
+        QImage qimage = cb->image();
 
-        if (qimg.isNull())
+        if (qimage.isNull())
             return KisPaintDeviceSP(0);
 
         KisConfig cfg;
@@ -205,18 +241,24 @@ KisPaintDeviceSP KisClipboard::clip()
         if (behaviour == PASTE_ASSUME_MONITOR)
             profileName = cfg.monitorProfile();
 
-        cs = KoColorSpaceRegistry::instance() ->colorSpace("RGBA", profileName);
+        cs = KoColorSpaceRegistry::instance()->rgb8(profileName);
         if (!cs) {
             cs = KoColorSpaceRegistry::instance()->rgb8();
             profileName = cs->profile()->name();
         }
 
-        m_clip = new KisPaintDevice(cs);
-        Q_CHECK_PTR(m_clip);
-        m_clip->convertFromQImage(qimg, profileName);
+        clip = new KisPaintDevice(cs);
+        Q_CHECK_PTR(clip);
+        clip->convertFromQImage(qimage, profileName);
     }
-
-    return m_clip;
+    if (!customTopLeft)
+    {
+        QRect exactBounds = clip->exactBounds();
+        topLeft -= exactBounds.topLeft() / 2;
+    }
+    clip->setX(topLeft.x());
+    clip->setY(topLeft.y());
+    return clip;
 }
 
 void KisClipboard::clipboardDataChanged()
@@ -224,14 +266,14 @@ void KisClipboard::clipboardDataChanged()
     if (!m_pushedClipboard) {
         m_hasClip = false;
         QClipboard *cb = QApplication::clipboard();
-        QImage qimg = cb->image();
+        QImage qimage = cb->image();
         const QMimeData *cbData = cb->mimeData();
         QByteArray mimeType("application/x-krita-selection");
 
         if (cbData && cbData->hasFormat(mimeType))
             m_hasClip = true;
 
-        if (!qimg.isNull())
+        if (!qimage.isNull())
             m_hasClip = true;
     }
 
@@ -257,25 +299,32 @@ QSize KisClipboard::clipSize()
         QByteArray encodedData = cbData->data(mimeType);
         QBuffer buffer(&encodedData);
         KoStore* store = KoStore::createStore(&buffer, KoStore::Read, mimeType);
-        KoColorProfile *profile = 0;
+        const KoColorProfile *profile = 0;
+        QString csDepth, csModel;
+
+        // ColorSpace id of layer data
+        if (store->hasFile("colormodel")) {
+            store->open("colormodel");
+            csModel = QString(store->read(store->size()));
+            store->close();
+        }
+
+        if (store->hasFile("colordepth")) {
+            store->open("colordepth");
+            csDepth = QString(store->read(store->size()));
+            store->close();
+        }
 
         if (store->hasFile("profile.icc")) {
             QByteArray data;
             store->open("profile.icc");
             data = store->read(store->size());
             store->close();
-            profile = new KoIccColorProfile(data);
+            profile = KoColorSpaceRegistry::instance()->createColorProfile(csModel, csDepth, data);
+
         }
 
-        QString csName;
-        // ColorSpace id of layer data
-        if (store->hasFile("colorspace")) {
-            store->open("colorspace");
-            csName = QString(store->read(store->size()));
-            store->close();
-        }
-
-        const KoColorSpace *cs = KoColorSpaceRegistry::instance()->colorSpace(csName, profile);
+        const KoColorSpace *cs = KoColorSpaceRegistry::instance()->colorSpace(csModel, csDepth, profile);
 
         clip = new KisPaintDevice(cs);
 
@@ -288,8 +337,8 @@ QSize KisClipboard::clipSize()
 
         return clip->exactBounds().size();
     } else {
-        QImage qimg = cb->image();
-        return qimg.size();
+        QImage qimage = cb->image();
+        return qimage.size();
     }
 }
 
