@@ -92,10 +92,10 @@ void KisCustomPattern::slotUpdateCurrentPattern(int)
                 if (scaledWidth == 0) scaledWidth++;
                 if (scaledHeight == 0) scaledHeight++;
 
-                QPixmap scaledPixmap = QPixmap::fromImage(m_pattern->img());
+                QPixmap scaledPixmap = QPixmap::fromImage(m_pattern->image());
                 preview->setPixmap(scaledPixmap.scaled(scaledWidth, scaledHeight));
             } else {
-                preview->setPixmap(QPixmap::fromImage(m_pattern->img()));
+                preview->setPixmap(QPixmap::fromImage(m_pattern->image()));
             }
         }
     }
@@ -148,12 +148,23 @@ void KisCustomPattern::slotUsePattern()
 
 void KisCustomPattern::createPattern()
 {
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
-    if (!img)
+    if (!image)
         return;
 
-    m_pattern = new KisPattern(img->mergedImage().data(), 0, 0, img->width(), img->height());
+    QSize size(image->width(), image->height());
+    
+    if (size.width() > 1000 || size.height() > 1000) {
+        size.scale(1000, 1000, Qt::KeepAspectRatio);
+    }
+    QImage img;
+    image->lock();
+    img = image->projection()->createThumbnail(size.width(), size.height());
+    image->unlock();
+
+
+    m_pattern = new KisPattern(img, image->objectName());
 }
 
 
