@@ -24,7 +24,6 @@
 #include <QPixmap>
 #include <QLabel>
 #include <QComboBox>
-#include <q3buttongroup.h>
 #include <QPushButton>
 #include <qscrollbar.h>
 //Added by qt3to4:
@@ -61,7 +60,7 @@ KisHistogramView::~KisHistogramView()
 {
 }
 
-void KisHistogramView::setPaintDevice(KisPaintDeviceSP dev)
+void KisHistogramView::setPaintDevice(KisPaintDeviceSP dev, const QRect &bounds)
 {
     m_cs = dev->colorSpace();
 
@@ -73,13 +72,14 @@ void KisHistogramView::setPaintDevice(KisPaintDeviceSP dev)
     m_from = m_currentProducer->viewFrom();
     m_width = m_currentProducer->viewWidth();
 
-    m_histogram = new KisHistogram(dev, m_currentProducer, LINEAR);
+    m_histogram = new KisHistogram(dev, bounds, m_currentProducer, LINEAR);
 
     updateHistogram();
 }
 
 void KisHistogramView::setHistogram(KisHistogramSP histogram)
 {
+    if (!histogram) return;
     m_cs = 0;
     m_histogram = histogram;
     m_currentProducer = m_histogram->producer();
@@ -123,11 +123,11 @@ QStringList KisHistogramView::channelStrings()
     return m_channelStrings;
 }
 
-QList<KoID> KisHistogramView::listProducers()
+QList<QString> KisHistogramView::producers()
 {
     if (m_cs)
-        return KoHistogramProducerFactoryRegistry::instance()->listKeysCompatibleWith(m_cs);
-    return QList<KoID>();
+        return KoHistogramProducerFactoryRegistry::instance()->keysCompatibleWith(m_cs);
+    return QList<QString>();
 }
 
 void KisHistogramView::setCurrentChannels(const KoID& producerID, QList<KoChannelInfo *> channels)
@@ -234,16 +234,15 @@ void KisHistogramView::setChannels()
     m_channels.clear();
     m_channelToOffset.clear();
 
-    QList<KoID> list = KoHistogramProducerFactoryRegistry::instance()->listKeysCompatibleWith(m_cs);
+    QList<QString> list = KoHistogramProducerFactoryRegistry::instance()->keysCompatibleWith(m_cs);
 
     if (list.count() == 0) {
         // XXX: No native histogram for this colorspace. Using converted RGB. We should have a warning
         KoGenericRGBHistogramProducerFactory f;
         addProducerChannels(f.generate());
     } else {
-        for (int i = 0; i < list.count(); i++) {
-            KoID id(list.at(i));
-            addProducerChannels(KoHistogramProducerFactoryRegistry::instance()->value(id.id())->generate());
+        foreach (const QString &id, list) {
+            addProducerChannels(KoHistogramProducerFactoryRegistry::instance()->value(id)->generate());
         }
     }
 
