@@ -53,11 +53,11 @@
 #include <kis_layer.h>
 #include <kis_paint_device.h>
 #include <kis_paint_layer.h>
-#include <kis_selected_transaction.h>
+#include <kis_transaction.h>
 #include <kis_selection.h>
 #include <flake/kis_shape_layer.h>
 #include <kis_shear_visitor.h>
-#include <kis_transform_worker.h>
+#include <kis_transform_visitor.h>
 #include <kis_undo_adapter.h>
 #include <kis_painter.h>
 
@@ -87,11 +87,11 @@
 KisLayerManager::KisLayerManager(KisView2 * view, KisDoc2 * doc)
         : m_view(view)
         , m_doc(doc)
-        , m_imgFlatten(0)
-        , m_imgMergeLayer(0)
+        , m_imageFlatten(0)
+        , m_imageMergeLayer(0)
         , m_layerSaveAs(0)
         , m_actLayerVis(false)
-        , m_imgResizeToLayer(0)
+        , m_imageResizeToLayer(0)
         , m_flattenLayer(0)
         , m_activeLayer(0)
         , m_commandsAdapter(new KisNodeCommandsAdapter(m_view))
@@ -129,15 +129,15 @@ void KisLayerManager::activateLayer(KisLayerSP layer)
 
 void KisLayerManager::setup(KActionCollection * actionCollection)
 {
-    m_imgFlatten  = new KAction(i18n("&Flatten image"), this);
-    actionCollection->addAction("flatten_image", m_imgFlatten);
-    m_imgFlatten->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_E));
-    connect(m_imgFlatten, SIGNAL(triggered()), this, SLOT(flattenImage()));
+    m_imageFlatten  = new KAction(i18n("&Flatten image"), this);
+    actionCollection->addAction("flatten_image", m_imageFlatten);
+    m_imageFlatten->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_E));
+    connect(m_imageFlatten, SIGNAL(triggered()), this, SLOT(flattenImage()));
 
-    m_imgMergeLayer  = new KAction(i18n("&Merge with Layer Below"), this);
-    actionCollection->addAction("merge_layer", m_imgMergeLayer);
-    m_imgMergeLayer->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_E));
-    connect(m_imgMergeLayer, SIGNAL(triggered()), this, SLOT(mergeLayer()));
+    m_imageMergeLayer  = new KAction(i18n("&Merge with Layer Below"), this);
+    actionCollection->addAction("merge_layer", m_imageMergeLayer);
+    m_imageMergeLayer->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_E));
+    connect(m_imageMergeLayer, SIGNAL(triggered()), this, SLOT(mergeLayer()));
 
     m_flattenLayer  = new KAction(i18n("&Flatten Layer"), this);
     actionCollection->addAction("flatten_layer", m_flattenLayer);
@@ -147,9 +147,9 @@ void KisLayerManager::setup(KActionCollection * actionCollection)
     actionCollection->addAction("save_layer_as_image", m_layerSaveAs);
     connect(m_layerSaveAs, SIGNAL(triggered()), this, SLOT(saveLayerAsImage()));
 
-    m_imgResizeToLayer  = new KAction(i18n("Size Canvas to Size of Current Layer"), this);
-    actionCollection->addAction("resizeimgtolayer", m_imgResizeToLayer);
-    connect(m_imgResizeToLayer, SIGNAL(triggered()), this, SLOT(imgResizeToActiveLayer()));
+    m_imageResizeToLayer  = new KAction(i18n("Size Canvas to Size of Current Layer"), this);
+    actionCollection->addAction("resizeimagetolayer", m_imageResizeToLayer);
+    connect(m_imageResizeToLayer, SIGNAL(triggered()), this, SLOT(imageResizeToActiveLayer()));
 }
 
 void KisLayerManager::addAction(QAction * action)
@@ -159,85 +159,46 @@ void KisLayerManager::addAction(QAction * action)
 
 void KisLayerManager::updateGUI()
 {
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
     KisLayerSP layer;
     qint32 nlayers = 0;
     qint32 nvisible = 0;
 
 
-    if (img) {
+    if (image) {
         layer = m_activeLayer;
-        nlayers = img->nlayers();
-        nvisible = nlayers - img->nHiddenLayers();
+        nlayers = image->nlayers();
+        nvisible = nlayers - image->nHiddenLayers();
     }
 
-    bool enable = img && layer && layer->visible() && !layer->userLocked() && !layer->systemLocked();
+    bool enable = image && layer && layer->visible() && !layer->userLocked() && !layer->systemLocked();
 
     m_layerSaveAs->setEnabled(enable);
 
-    // XXX these should be named layer instead of img
-    m_imgFlatten->setEnabled(nlayers > 1);
-    m_imgMergeLayer->setEnabled(nlayers > 1 && layer && layer->prevSibling());
+    // XXX these should be named layer instead of image
+    m_imageFlatten->setEnabled(nlayers > 1);
+    m_imageMergeLayer->setEnabled(nlayers > 1 && layer && layer->prevSibling());
     m_flattenLayer->setEnabled(nlayers > 1 && layer && layer->firstChild());
 
-    m_imgResizeToLayer->setEnabled(activeLayer());
+    m_imageResizeToLayer->setEnabled(activeLayer());
 
     if (m_view->statusBar())
-        m_view->statusBar()->setProfile(img);
+        m_view->statusBar()->setProfile(image);
 }
 
-void KisLayerManager::imgResizeToActiveLayer()
+void KisLayerManager::imageResizeToActiveLayer()
 {
     KisLayerSP layer;
     KisUndoAdapter * undoAdapter = m_view->undoAdapter();
 
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
-    if (img && (layer = activeLayer())) {
+    if (image && (layer = activeLayer())) {
 
-        if (undoAdapter && undoAdapter->undo()) {
-            undoAdapter->beginMacro(i18n("Resize Image to Size of Current Layer"));
-        }
-
-        img->lock();
-
-        QRect r = layer->exactBounds();
-        img->resize(r.width(), r.height(), r.x(), r.y(), true);
-
-        img->unlock();
-
-        if (undoAdapter && undoAdapter->undo()) {
-            undoAdapter->endMacro();
-        }
-    }
-}
-
-void KisLayerManager::layerCompositeOp(const KoCompositeOp* compositeOp)
-{
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    m_doc->addCommand(new KisLayerCompositeOpCommand(layer, layer->compositeOpId(), compositeOp->id()));
-}
-
-// range: 0 - 100
-void KisLayerManager::layerOpacity(double opacity, bool final)
-{
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    opacity = int(opacity * 2.55 + 0.5);
-    if (opacity > 255)
-        opacity = 255;
-
-    if (opacity == layer->opacity()) return;
-
-    if (!final) {
-        layer->setOpacity(static_cast<int>(opacity));
-        layer->setDirty();
-    } else {
-        m_doc->addCommand(new KisLayerOpacityCommand(layer, layer->opacity(), static_cast<int>(opacity)));
+        undoAdapter->beginMacro(i18n("Resize Image to Size of Current Layer"));
+        image->resize(layer->exactBounds(), true);
+        undoAdapter->endMacro();
     }
 }
 
@@ -271,17 +232,17 @@ void KisLayerManager::layerProperties()
         KisFilterConfiguration* config = dlg.filterConfiguration();
         QString before;
         if (config) {
-            before = config->toLegacyXML();
+            before = config->toXML();
         }
         if (dlg.exec() == QDialog::Accepted) {
 
             QString after;
+            alayer->setName(dlg.layerName());
+
             if (dlg.filterConfiguration()) {
-                after = dlg.filterConfiguration()->toLegacyXML();
+                after = dlg.filterConfiguration()->toXML();
             }
             if (after != before) {
-                alayer->setName(dlg.layerName());
-
                 KisChangeFilterCmd<KisAdjustmentLayerSP> * cmd
                 = new KisChangeFilterCmd<KisAdjustmentLayerSP>(alayer,
                         dlg.filterConfiguration(),
@@ -297,13 +258,13 @@ void KisLayerManager::layerProperties()
         KisDlgGeneratorLayer dlg(alayer->name(), m_view);
         dlg.setCaption(i18n("Generator Layer Properties"));
 
-        QString before = alayer->generator()->toLegacyXML();
+        QString before = alayer->generator()->toXML();
         dlg.setConfiguration(alayer->generator());
         dlg.resize(dlg.minimumSizeHint());
 
         if (dlg.exec() == QDialog::Accepted) {
 
-            QString after = dlg.configuration()->toLegacyXML();
+            QString after = dlg.configuration()->toXML();
             if (after != before) {
                 KisChangeGeneratorCmd<KisGeneratorLayerSP> * cmd
                 = new KisChangeGeneratorCmd<KisGeneratorLayerSP>(alayer,
@@ -360,20 +321,20 @@ void KisLayerManager::layerProperties()
 
 void KisLayerManager::layerAdd()
 {
-    KisImageWSP img = m_view->image();
-    if (img && activeLayer()) {
+    KisImageWSP image = m_view->image();
+    if (image && activeLayer()) {
         addLayer(activeLayer()->parent(), activeLayer());
-    } else if (img)
-        addLayer(img->rootLayer(), KisLayerSP(0));
+    } else if (image)
+        addLayer(image->rootLayer(), KisLayerSP(0));
 }
 
 void KisLayerManager::addLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageWSP img = m_view->image();
-    if (img) {
+    KisImageWSP image = m_view->image();
+    if (image) {
         KisConfig cfg;
         QString profilename;
-        KisLayerSP layer = new KisPaintLayer(img.data(), img->nextLayerName(), OPACITY_OPAQUE, img->colorSpace());
+        KisLayerSP layer = new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, image->colorSpace());
         if (layer) {
             layer->setCompositeOp(COMPOSITE_OVER);
             m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
@@ -387,13 +348,14 @@ void KisLayerManager::addLayer(KisNodeSP parent, KisNodeSP above)
 
 void KisLayerManager::addGroupLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageWSP img = m_view->image();
-    if (img) {
-        KisLayerSP layer = KisLayerSP(new KisGroupLayer(img.data(), img->nextLayerName(), OPACITY_OPAQUE));
+    KisImageWSP image = m_view->image();
+    if (image) {
+        KisLayerSP layer = KisLayerSP(new KisGroupLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8));
         if (layer) {
             layer->setCompositeOp(COMPOSITE_OVER);
             m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
             m_view->canvas()->update();
+            m_view->nodeManager()->activateNode(layer);
         } else {
             KMessageBox::error(m_view, i18n("Could not add layer to image."), i18n("Layer Error"));
         }
@@ -403,21 +365,21 @@ void KisLayerManager::addGroupLayer(KisNodeSP parent, KisNodeSP above)
 
 void KisLayerManager::addCloneLayer()
 {
-    KisImageWSP img = m_view->image();
-    if (img && activeLayer()) {
+    KisImageWSP image = m_view->image();
+    if (image && activeLayer()) {
         addCloneLayer(activeLayer()->parent(), activeLayer());
-    } else if (img)
-        addCloneLayer(img->rootLayer(), KisLayerSP(0));
+    } else if (image)
+        addCloneLayer(image->rootLayer(), KisLayerSP(0));
 }
 
 void KisLayerManager::addCloneLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageWSP img = m_view->image();
-    if (img) {
+    KisImageWSP image = m_view->image();
+    if (image) {
         // Check whether we are not cloning a parent layer
         if (KisGroupLayer * from = dynamic_cast<KisGroupLayer*>(m_activeLayer.data())) {
             KisNodeSP parent = parent;
-            while (parent && parent != img->root()) {
+            while (parent && parent != image->root()) {
                 if (parent.data() == from) {
                     // The chosen layer is one of our own parents -- this will
                     // lead to cyclic behaviour when updating. Don't do that!
@@ -427,12 +389,13 @@ void KisLayerManager::addCloneLayer(KisNodeSP parent, KisNodeSP above)
             }
         }
 
-        KisLayerSP layer = new KisCloneLayer(m_activeLayer, img.data(), img->nextLayerName(), OPACITY_OPAQUE);
+        KisLayerSP layer = new KisCloneLayer(m_activeLayer, image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
 
         if (layer) {
 
             layer->setCompositeOp(COMPOSITE_OVER);
             m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
+            m_view->nodeManager()->activateNode(layer);
 
             m_view->canvas()->update();
 
@@ -445,28 +408,29 @@ void KisLayerManager::addCloneLayer(KisNodeSP parent, KisNodeSP above)
 
 void KisLayerManager::addShapeLayer()
 {
-    KisImageWSP img = m_view->image();
-    if (img && activeLayer()) {
+    KisImageWSP image = m_view->image();
+    if (image && activeLayer()) {
         addShapeLayer(activeLayer()->parent(), activeLayer());
-    } else if (img)
-        addShapeLayer(img->rootLayer(), KisLayerSP(0));
+    } else if (image)
+        addShapeLayer(image->rootLayer(), KisLayerSP(0));
 }
 
 
 void KisLayerManager::addShapeLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageWSP img = m_view->image();
-    if (img) {
+    KisImageWSP image = m_view->image();
+    if (image) {
         // XXX: Make work with nodes!
         KisLayer * parentLayer = dynamic_cast<KisLayer*>(parent.data());
         KoShapeContainer * parentContainer =
             dynamic_cast<KoShapeContainer*>(m_doc->shapeForNode(parentLayer));
         if (!parentContainer) return;
 
-        KisLayerSP layer = new KisShapeLayer(parentContainer, m_doc->shapeController(), img.data(), img->nextLayerName(), OPACITY_OPAQUE);
+        KisLayerSP layer = new KisShapeLayer(parentContainer, m_doc->shapeController(), image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8);
         if (layer) {
             layer->setCompositeOp(COMPOSITE_OVER);
             m_commandsAdapter->addNode(layer.data(), parent, above.data());
+            m_view->nodeManager()->activateNode(layer);
             m_view->canvas()->update();
         } else {
             KMessageBox::error(m_view, i18n("Could not add layer to image."), i18n("Layer Error"));
@@ -484,8 +448,8 @@ void KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNodeSP above)
 {
     Q_ASSERT(parent);
 
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP l = activeLayer();
 
@@ -495,17 +459,18 @@ void KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNodeSP above)
     if (l->selection())
         selection = l->selection();
     else
-        selection = img->globalSelection();
+        selection = image->globalSelection();
 
     KisAdjustmentLayerSP adjl = addAdjustmentLayer(parent, above, QString(), 0, selection);
 
-    KisDlgAdjustmentLayer dlg(adjl, adjl.data(), dev, adjl->image(), img->nextLayerName(), i18n("New Filter Layer"), m_view, "dlgadjustmentlayer");
+    KisDlgAdjustmentLayer dlg(adjl, adjl.data(), dev, adjl->image(), image->nextLayerName(), i18n("New Filter Layer"), m_view, "dlgadjustmentlayer");
     dlg.resize(dlg.minimumSizeHint());
 
     if (dlg.exec() != QDialog::Accepted) {
-        m_view->image()->removeNode(adjl);
+        m_commandsAdapter->undoLastCommand();
     } else {
         adjl->setName(dlg.layerName());
+        m_view->nodeManager()->activateNode(adjl);
     }
 }
 
@@ -514,12 +479,12 @@ KisAdjustmentLayerSP KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNo
 {
     Q_ASSERT(parent);
 
-    KisImageWSP img = m_view->image();
-    if (!img) return 0;
+    KisImageWSP image = m_view->image();
+    if (!image) return 0;
 
-    KisAdjustmentLayerSP l = new KisAdjustmentLayer(img, name, filter, selection);
+    KisAdjustmentLayerSP l = new KisAdjustmentLayer(image, name, filter, selection);
     m_commandsAdapter->addNode(l.data(), parent, above);
-    l->setDirty(img->bounds());
+    l->setDirty(image->bounds());
     return l;
 }
 
@@ -532,10 +497,10 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP parent, KisNodeSP above)
 {
     Q_ASSERT(parent);
 
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
-    KisDlgGeneratorLayer dlg(img->nextLayerName(), m_view);
+    KisDlgGeneratorLayer dlg(image->nextLayerName(), m_view);
     dlg.resize(dlg.minimumSizeHint());
 
     if (dlg.exec() == QDialog::Accepted) {
@@ -552,24 +517,25 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP parent, KisNodeSP above, const
     Q_ASSERT(parent);
     Q_ASSERT(generator);
 
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
-    KisGeneratorLayerSP l = new KisGeneratorLayer(img, name, generator, selection);
+    KisGeneratorLayerSP l = new KisGeneratorLayer(image, name, generator, selection);
     m_commandsAdapter->addNode(l.data(), parent, above.data());
+    m_view->nodeManager()->activateNode(l);
     if (l->selection())
         l->setDirty(l->selection()->selectedExactRect());
     else
-        l->setDirty(img->bounds());
+        l->setDirty(image->bounds());
 
 }
 
 
 void KisLayerManager::layerRemove()
 {
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
-    if (img) {
+    if (image) {
         KisLayerSP layer = activeLayer();
         if (layer) {
             QRect extent = layer->extent();
@@ -588,9 +554,9 @@ void KisLayerManager::layerRemove()
 
 void KisLayerManager::layerDuplicate()
 {
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
-    if (!img)
+    if (!image)
         return;
 
     KisLayerSP active = activeLayer();
@@ -599,7 +565,6 @@ void KisLayerManager::layerDuplicate()
         return;
 
     KisLayerSP dup = dynamic_cast<KisLayer*>(active->clone().data());
-    dup->setName(i18n("Duplicate of '%1'", active->name()));
     m_commandsAdapter->addNode(dup.data(), active->parent(), active.data());
     if (dup) {
         activateLayer(dup);
@@ -612,10 +577,10 @@ void KisLayerManager::layerDuplicate()
 
 void KisLayerManager::layerRaise()
 {
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
     KisLayerSP layer;
 
-    if (!img)
+    if (!image)
         return;
 
     layer = activeLayer();
@@ -626,10 +591,10 @@ void KisLayerManager::layerRaise()
 
 void KisLayerManager::layerLower()
 {
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
     KisLayerSP layer;
 
-    if (!img)
+    if (!image)
         return;
 
     layer = activeLayer();
@@ -640,10 +605,10 @@ void KisLayerManager::layerLower()
 
 void KisLayerManager::layerFront()
 {
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
     KisLayerSP layer;
 
-    if (!img)
+    if (!image)
         return;
 
     layer = activeLayer();
@@ -653,8 +618,8 @@ void KisLayerManager::layerFront()
 
 void KisLayerManager::layerBack()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer;
 
@@ -680,20 +645,26 @@ void KisLayerManager::rotateLayerRight90()
 
 void KisLayerManager::mirrorLayerX()
 {
-    KisPaintDeviceSP dev = activeDevice();
-    if (!dev) return;
+    KisLayerSP layer = activeLayer();
+    
+    if (layer->inherits("KisShapeLayer")) {
+        m_view->image()->undoAdapter()->beginMacro(i18n("Mirror Layer X"));
 
-    KisTransaction * t = 0;
-    if (m_view->undoAdapter() && m_view->undoAdapter()->undo()) {
-        t = new KisTransaction(i18n("Mirror Layer X"), dev);
-        Q_CHECK_PTR(t);
+        KisTransformVisitor visitor(m_view->image(), -1.0, 1.0, 0.0, 0.0, 0.0, m_view->image()->width(), 0, 0, 0);
+        layer->accept(visitor);
+
+        m_view->image()->undoAdapter()->endMacro();
+    } else {
+        KisPaintDeviceSP dev = activeDevice();
+        if (!dev) return;
+
+        KisTransaction transaction(i18n("Mirror Layer X"), dev);
+
+        QRect dirty = KisTransformWorker::mirrorX(dev, m_view->selection());
+        m_activeLayer->setDirty(dirty);
+
+        transaction.commit(m_view->image()->undoAdapter());
     }
-
-    QRect dirty = KisTransformWorker::mirrorX(dev, m_view->selection());
-    m_activeLayer->setDirty(dirty);
-
-    if (t) m_view->undoAdapter()->addCommand(t);
-
     m_doc->setModified(true);
     layersUpdated();
     m_view->canvas()->update();
@@ -701,20 +672,26 @@ void KisLayerManager::mirrorLayerX()
 
 void KisLayerManager::mirrorLayerY()
 {
-    KisPaintDeviceSP dev = activeDevice();
-    if (!dev) return;
+    KisLayerSP layer = activeLayer();
+    
+    if (layer->inherits("KisShapeLayer")) {
+        m_view->image()->undoAdapter()->beginMacro(i18n("Mirror Layer Y"));
 
-    KisTransaction * t = 0;
-    if (m_view->undoAdapter() && m_view->undoAdapter()->undo()) {
-        t = new KisTransaction(i18n("Mirror Layer Y"), dev);
-        Q_CHECK_PTR(t);
+        KisTransformVisitor visitor(m_view->image(), 1.0, -1.0, 0.0, 0.0, 0.0, 0, m_view->image()->height(), 0, 0);
+        layer->accept(visitor);
+
+        m_view->image()->undoAdapter()->endMacro();
+    } else {
+        KisPaintDeviceSP dev = activeDevice();
+        if (!dev) return;
+
+        KisTransaction transaction(i18n("Mirror Layer Y"), dev);
+
+        QRect dirty = KisTransformWorker::mirrorY(dev, m_view->selection());
+        m_activeLayer->setDirty(dirty);
+
+        transaction.commit(m_view->image()->undoAdapter());
     }
-
-    QRect dirty = KisTransformWorker::mirrorY(dev, m_view->selection());
-    m_activeLayer->setDirty(dirty);
-
-    if (t) m_view->undoAdapter()->addCommand(t);
-
     m_doc->setModified(true);
     layersUpdated();
     m_view->canvas()->update();
@@ -727,19 +704,15 @@ void KisLayerManager::scaleLayer(double sx, double sy, KisFilterStrategy *filter
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    KisSelectedTransaction * t = 0;
-    if (m_view->undoAdapter() && m_view->undoAdapter()->undo()) {
-        t = new KisSelectedTransaction(i18n("Scale Layer"), layer);
-        Q_CHECK_PTR(t);
-    }
-
     KoProgressUpdater* updater = m_view->createProgressUpdater();
     KoUpdaterPtr u = updater->startSubtask();
 
-    KisTransformWorker worker(layer->paintDevice(), sx, sy, 0, 0, 0.0, 0, 0, u, filterStrategy);
-    worker.run();
+    m_view->image()->undoAdapter()->beginMacro(i18n("Scale Layer"));
 
-    if (t) m_view->undoAdapter()->addCommand(t);
+    KisTransformVisitor visitor(m_view->image(), sx, sy, 0.0, 0.0, 0.0, 0, 0, u, filterStrategy);
+    layer->accept(visitor);
+
+    m_view->image()->undoAdapter()->endMacro();
     m_doc->setModified(true);
     layersUpdated();
     m_view->canvas()->update();
@@ -754,11 +727,8 @@ void KisLayerManager::rotateLayer(double radians)
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    KisSelectedTransaction * t = 0;
-    if (m_view->undoAdapter() && m_view->undoAdapter()->undo()) {
-        t = new KisSelectedTransaction(i18n("Rotate Layer"), layer);
-        Q_CHECK_PTR(t);
-    }
+    KisUndoAdapter * undoAdapter = m_view->image()->undoAdapter();
+    undoAdapter->beginMacro(i18n("Rotate Layer"));
 
     KisFilterStrategy *filter = KisFilterStrategyRegistry::instance()->value("Triangle");
     QRect r;
@@ -771,10 +741,11 @@ void KisLayerManager::rotateLayer(double radians)
     double cy = r.y() + r.height() / 2.0;
     qint32 tx = qint32(cx * cos(radians) - cy * sin(radians) - cx + 0.5);
     qint32 ty = qint32(cy * cos(radians) + cx * sin(radians) - cy + 0.5);
-    KisTransformWorker tw(layer->paintDevice(), 1.0, 1.0, 0, 0, radians, -tx, -ty, 0, filter);
-    tw.run();
+    KisTransformVisitor visitor(m_view->image(), 1.0, 1.0, 0, 0, radians, -tx, -ty, 0, filter);
+    layer->accept(visitor);
+    layer->parent()->setDirty(r);
 
-    if (t) m_view->undoAdapter()->addCommand(t);
+    undoAdapter->endMacro();
 
     m_doc->setModified(true);
     layersUpdated();
@@ -788,20 +759,18 @@ void KisLayerManager::shearLayer(double angleX, double angleY)
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    KisUndoAdapter * undo = 0;
-    if ((undo = m_view->image()->undoAdapter())) {
-        undo->beginMacro(i18n("Shear layer"));
-    }
+    KisUndoAdapter * undoAdapter = m_view->image()->undoAdapter();
+    undoAdapter->beginMacro(i18n("Shear layer"));
 
     KoProgressUpdater* updater = m_view->statusBar()->progress()->createUpdater();
     updater->start(100, i18n("Shear layer"));
     KoUpdaterPtr up = updater->startSubtask();
 
     KisShearVisitor v(angleX, angleY, up);
-    v.setUndoAdapter(undo);
+    v.setUndoAdapter(undoAdapter);
     layer->accept(v);
 
-    if (undo) undo->endMacro();
+    undoAdapter->endMacro();
 
     m_doc->setModified(true);
     layersUpdated();
@@ -813,12 +782,12 @@ void KisLayerManager::shearLayer(double angleX, double angleY)
 
 void KisLayerManager::flattenImage()
 {
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
 
-    if (img) {
+    if (image) {
         bool doIt = true;
 
-        if (img->nHiddenLayers() > 0) {
+        if (image->nHiddenLayers() > 0) {
             int answer = KMessageBox::warningYesNo(m_view,
                                                    i18n("The image contains hidden layers that will be lost."),
                                                    i18n("Flatten Image"),
@@ -831,15 +800,15 @@ void KisLayerManager::flattenImage()
         }
 
         if (doIt) {
-            img->flatten();
+            image->flatten();
         }
     }
 }
 
 void KisLayerManager::mergeLayer()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer = activeLayer();
     if (!layer) return;
@@ -847,7 +816,7 @@ void KisLayerManager::mergeLayer()
     const KisMetaData::MergeStrategy* strategy = KisMetaDataMergeStrategyChooserWidget::showDialog(m_view);
     if (!strategy) return;
 
-    KisLayerSP  newLayer = img->mergeLayer(layer, strategy);
+    KisLayerSP  newLayer = image->mergeLayer(layer, strategy);
     if (newLayer) {
         newLayer->setDirty();
     }
@@ -858,13 +827,13 @@ void KisLayerManager::mergeLayer()
 
 void KisLayerManager::flattenLayer()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    KisLayerSP newLayer = img->flattenLayer(layer);
+    KisLayerSP newLayer = image->flattenLayer(layer);
     if (newLayer) {
         newLayer->setDirty();
     }
@@ -903,19 +872,19 @@ void KisLayerManager::saveLayerAsImage()
     if (url.isEmpty())
         return;
 
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP l = activeLayer();
     if (!l) return;
 
-    QRect r = img->bounds();
+    QRect r = image->bounds();
 
     KisDoc2 d;
     d.prepareForImport();
 
-    KisImageWSP dst = new KisImage(d.undoAdapter(), r.width(), r.height(), img->colorSpace(), l->name());
-    dst->setResolution(img->xRes(), img->yRes());
+    KisImageWSP dst = new KisImage(d.undoAdapter(), r.width(), r.height(), image->colorSpace(), l->name());
+    dst->setResolution(image->xRes(), image->yRes());
     d.setCurrentImage(dst);
     KisPaintLayer* paintLayer = new KisPaintLayer(dst, "projection", l->opacity());
     KisPainter gc(paintLayer->paintDevice());
