@@ -17,7 +17,9 @@
  */
 
 #include "kis_painting_assistant.h"
+#include "kis_debug.h"
 
+#include <kglobal.h>
 
 struct KisPaintingAssistantHandle::Private {
     QList<KisPaintingAssistant*> assistants;
@@ -30,7 +32,10 @@ KisPaintingAssistantHandle::KisPaintingAssistantHandle(QPointF p) : QPointF(p),
 {
 }
 
-KisPaintingAssistantHandle::KisPaintingAssistantHandle(const KisPaintingAssistantHandle& rhs) : QPointF(rhs), d(new Private)
+KisPaintingAssistantHandle::KisPaintingAssistantHandle(const KisPaintingAssistantHandle& rhs)
+    : QPointF(rhs)
+    , KisShared()
+    , d(new Private)
 {
 }
 
@@ -131,3 +136,30 @@ QList<KisPaintingAssistantHandleSP> KisPaintingAssistant::handles()
 {
     return d->handles;
 }
+
+KisPaintingAssistantFactory::KisPaintingAssistantFactory()
+{
+}
+
+KisPaintingAssistantFactory::~KisPaintingAssistantFactory()
+{
+}
+
+KisPaintingAssistantFactoryRegistry::KisPaintingAssistantFactoryRegistry()
+{
+}
+
+KisPaintingAssistantFactoryRegistry::~KisPaintingAssistantFactoryRegistry()
+{
+    foreach(QString id, keys()) {
+        delete get(id);
+    }
+    dbgRegistry << "deleting KisPaintingAssistantFactoryRegistry ";
+}
+
+KisPaintingAssistantFactoryRegistry* KisPaintingAssistantFactoryRegistry::instance()
+{
+    K_GLOBAL_STATIC(KisPaintingAssistantFactoryRegistry, s_instance);
+    return s_instance;
+}
+
