@@ -22,10 +22,15 @@
 // documentation of libpng available at
 // http://www.libpng.org/pub/png/libpng-1.2.5-manual.html
 
-#include <config-endian.h> // WORDS_BIGENDIAN
+#ifndef PNG_MAX_UINT // Removed in libpng 1.4
+#define PNG_MAX_UINT PNG_UINT_31_MAX
+#endif
+
+#include <KoConfig.h> // WORDS_BIGENDIAN
 
 #include <limits.h>
 #include <stdio.h>
+#include <zlib.h>
 
 #include <QBuffer>
 #include <QFile>
@@ -40,17 +45,19 @@
 #include <KoDocumentInfo.h>
 #include <KoID.h>
 #include <KoColorSpaceRegistry.h>
-#include <colorprofiles/KoIccColorProfile.h>
+#include <KoColorProfile.h>
 
 #include <kis_doc2.h>
 #include <kis_image.h>
 #include <kis_iterators_pixel.h>
 #include <kis_layer.h>
 #include <kis_paint_device.h>
+#include <kis_transaction.h>
 #include <kis_paint_layer.h>
 #include <kis_group_layer.h>
 #include <kis_meta_data_io_backend.h>
 #include <kis_meta_data_store.h>
+#include <KoColorModelStandardIds.h>
 
 namespace
 {
@@ -76,24 +83,26 @@ int getColorTypeforColorSpace(const KoColorSpace * cs , bool alpha)
 }
 
 
-QString getColorSpaceForColorType(int color_type, int color_nb_bits)
+QPair<QString, QString> getColorSpaceForColorType(int color_type, int color_nb_bits)
 {
-    if (color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
-        if (color_nb_bits == 16) {
-            return "GRAYA16";
-        } else if (color_nb_bits <= 8) {
-            return "GRAYA";
+    QPair<QString, QString> r;
+
+    if (color_type ==  PNG_COLOR_TYPE_PALETTE) {
+        r.first = RGBAColorModelID.id();
+        r.second = Integer8BitsColorDepthID.id();
+    } else {
+        if (color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
+            r.first = GrayAColorModelID.id();
+        } else if (color_type == PNG_COLOR_TYPE_RGB_ALPHA || color_type == PNG_COLOR_TYPE_RGB) {
+            r.first = RGBAColorModelID.id();
         }
-    } else if (color_type == PNG_COLOR_TYPE_RGB_ALPHA || color_type == PNG_COLOR_TYPE_RGB) {
         if (color_nb_bits == 16) {
-            return "RGBA16";
+            r.second = Integer16BitsColorDepthID.id();
         } else if (color_nb_bits <= 8) {
-            return "RGBA";
+            r.second = Integer8BitsColorDepthID.id();
         }
-    } else if (color_type ==  PNG_COLOR_TYPE_PALETTE) {
-        return "RGBA"; // <-- we will convert the index image to RGBA
     }
-    return "";
+    return r;
 }
 
 
@@ -182,17 +191,18 @@ void writeRawProfile(png_struct *ping, png_info *ping_info, QString profile_type
     png_free(ping, text[0].text);
     png_free(ping, text);
 }
+
 QByteArray png_read_raw_profile(png_textp text)
 {
     QByteArray profile;
 
-    unsigned char unhex[103] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-                                0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0,
-                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10, 11, 12,
-                                13, 14, 15
-                               };
+    static unsigned char unhex[103] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                                       0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0,
+                                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+                                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10, 11, 12,
+                                       13, 14, 15
+                                      };
 
     png_charp sp = text[0].text + 1;
     /* look for newline */
@@ -225,6 +235,7 @@ QByteArray png_read_raw_profile(png_textp text)
     }
     return profile;
 }
+
 void decode_meta_data(png_textp text, KisMetaData::Store* store, QString type, int headerSize)
 {
     dbgFile << "Decoding " << type << " " << text[0].key;
@@ -254,7 +265,7 @@ KisPNGConverter::KisPNGConverter(KisDoc2 *doc, KisUndoAdapter *adapter)
     m_adapter = adapter;
     m_stop = false;
     m_max_row = 0;
-    m_img = 0;
+    m_image = 0;
 }
 
 KisPNGConverter::~KisPNGConverter()
@@ -398,13 +409,18 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
 
     png_byte signature[8];
     iod->peek((char*)signature, 8);
+
+#if PNG_LIBPNG_VER < 10400
     if (!png_check_sig(signature, 8)) {
+#else
+    if (png_sig_cmp(signature, 0, 8) != 0) {
+#endif
         iod->close();
         return (KisImageBuilder_RESULT_BAD_FETCH);
     }
 
     // Initialize the internal structures
-    png_structp png_ptr =  png_create_read_struct(PNG_LIBPNG_VER_STRING, png_voidp_NULL, png_error_ptr_NULL, png_error_ptr_NULL);
+    png_structp png_ptr =  png_create_read_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
     if (!png_ptr) {
         iod->close();
         return (KisImageBuilder_RESULT_FAILURE);
@@ -438,17 +454,17 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     png_read_info(png_ptr, info_ptr);
 
 
-    if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY && info_ptr->bit_depth < 8) {
+    if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY && png_get_bit_depth(png_ptr, info_ptr) < 8) {
         png_set_expand(png_ptr);
     }
 
-    if (info_ptr->color_type == PNG_COLOR_TYPE_PALETTE && info_ptr->bit_depth < 8) {
+    if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE && png_get_bit_depth(png_ptr, info_ptr) < 8) {
         png_set_packing(png_ptr);
     }
 
 
-    if (info_ptr->color_type != PNG_COLOR_TYPE_PALETTE &&
-            (info_ptr->valid & PNG_INFO_tRNS)) {
+    if (png_get_color_type(png_ptr, info_ptr) != PNG_COLOR_TYPE_PALETTE &&
+            (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))) {
         png_set_expand(png_ptr);
     }
     png_read_update_info(png_ptr, info_ptr);
@@ -465,8 +481,8 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
 #endif
 
     // Determine the colorspace
-    QString csName = getColorSpaceForColorType(color_type, color_nb_bits);
-    if (csName.isEmpty()) {
+    QPair<QString, QString> csName = getColorSpaceForColorType(color_type, color_nb_bits);
+    if (csName.first.isEmpty()) {
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
         iod->close();
         return KisImageBuilder_RESULT_UNSUPPORTED_COLORSPACE;
@@ -474,19 +490,27 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     bool hasalpha = (color_type == PNG_COLOR_TYPE_RGB_ALPHA || color_type == PNG_COLOR_TYPE_GRAY_ALPHA);
 
     // Read image profile
-    png_charp profile_name, profile_data;
+    png_charp profile_name;
+#if PNG_LIBPNG_VER_MAJOR >= 1 && PNG_LIBPNG_VER_MINOR >= 5
+    png_bytep profile_data;
+#else
+    png_charp profile_data;
+#endif
     int compression_type;
     png_uint_32 proflen;
 
 
-    KoColorProfile* profile = 0;
+    const KoColorProfile* profile = 0;
+
     if (png_get_iCCP(png_ptr, info_ptr, &profile_name, &compression_type, &profile_data, &proflen)) {
+
+
         QByteArray profile_rawdata;
         // XXX: Hardcoded for icc type -- is that correct for us?
         if (QString::compare(profile_name, "icc") == 0) {
             profile_rawdata.resize(proflen);
             memcpy(profile_rawdata.data(), profile_data, proflen);
-            profile = new KoIccColorProfile(profile_rawdata);
+            profile = KoColorSpaceRegistry::instance()->createColorProfile(csName.first, csName.second, profile_rawdata);
             Q_CHECK_PTR(profile);
             if (profile) {
 //                 dbgFile << "profile name: " << profile->productName() << " profile description: " << profile->productDescription() << " information sur le produit: " << profile->productInfo();
@@ -500,14 +524,22 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     } else {
         dbgFile << "no embedded profile, will use the default profile";
     }
+    
+    // Check that the profile is used by the color space
+    if (profile && !KoColorSpaceRegistry::instance()->colorSpaceFactory(
+        KoColorSpaceRegistry::instance()->colorSpaceId(
+      csName.first, csName.second))->profileIsCompatible(profile)) {
+        warnFile << "The profile " << profile->name() << " is not compatible with the color space model " << csName.first << " " << csName.second;
+        profile = 0;
+    }
 
     // Retrieve a pointer to the colorspace
     const KoColorSpace* cs;
     if (profile && profile->isSuitableForOutput()) {
         dbgFile << "image has embedded profile: " << profile -> name() << "\n";
-        cs = KoColorSpaceRegistry::instance()->colorSpace(csName, profile);
+        cs = KoColorSpaceRegistry::instance()->colorSpace(csName.first, csName.second, profile);
     } else
-        cs = KoColorSpaceRegistry::instance()->colorSpace(KoID(csName, ""), "");
+        cs = KoColorSpaceRegistry::instance()->colorSpace(csName.first, csName.second, 0);
 
     if (cs == 0) {
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
@@ -517,27 +549,29 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     // Create the cmsTransform if needed
     KoColorTransformation* transform = 0;
     if (profile && !profile->isSuitableForOutput()) {
-        transform = KoColorSpaceRegistry::instance()->colorSpace(csName, profile)->createColorConverter( cs );
+        transform = KoColorSpaceRegistry::instance()->colorSpace(csName.first, csName.second, profile)->createColorConverter(cs);
     }
 
-    // Creating the KisImageSP
-    if (m_img == 0) {
-        m_img = new KisImage(m_adapter, width, height, cs, "built image");
-        Q_CHECK_PTR(m_img);
-        m_img->lock();
-        if (profile && !profile->isSuitableForOutput()) {
-            KisAnnotationSP annotation;
-            // XXX we hardcode icc, this is correct for lcms?
-            // XXX productName(), or just "ICC Profile"?
-            KoIccColorProfile* iccprofile = dynamic_cast<KoIccColorProfile*>(profile);
-            if (iccprofile && !iccprofile->rawData().isEmpty())
-                annotation = new  KisAnnotation("icc", iccprofile->name(), iccprofile->rawData());
-            m_img -> addAnnotation(annotation);
-        }
+    // Creating the KisImageWSP
+    if (m_image == 0) {
+        m_image = new KisImage(m_adapter, width, height, cs, "built image");
+        Q_CHECK_PTR(m_image);
+        m_image->lock();
+    }
+
+    // Read resolution
+    int unit_type;
+    png_uint_32 x_resolution, y_resolution;
+
+    png_get_pHYs(png_ptr, info_ptr, &x_resolution, &y_resolution, &unit_type);
+    if (unit_type == PNG_RESOLUTION_METER) {
+        m_image->setResolution((double) POINT_TO_CM(x_resolution) / 100.0, (double) POINT_TO_CM(y_resolution) / 100.0); // It is the "invert" macro because we convert from pointer-per-inchs to points
     }
 
     double coeff = quint8_MAX / (double)(pow(2, color_nb_bits) - 1);
-    KisPaintLayerSP layer = new KisPaintLayer(m_img.data(), m_img -> nextLayerName(), UCHAR_MAX);
+    KisPaintLayerSP layer = new KisPaintLayer(m_image.data(), m_image -> nextLayerName(), UCHAR_MAX);
+    KisTransaction("", layer -> paintDevice());
+
     // Read comments/texts...
     png_text* text_ptr;
     int num_comments;
@@ -560,6 +594,10 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
                 decode_meta_data(text_ptr + i, layer->metaData(), "iptc", 14);
             } else if (key.contains("Raw profile type xmp")) {
                 decode_meta_data(text_ptr + i, layer->metaData(), "xmp", 0);
+            } else if (key == "version") {
+                m_image->addAnnotation(new KisAnnotation("kpp_version", "version", QByteArray(text_ptr[i].text)));
+            } else if (key == "preset") {
+                m_image->addAnnotation(new KisAnnotation("kpp_preset", "preset", QByteArray(text_ptr[i].text)));
             }
         }
     }
@@ -615,7 +653,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
                 while (!it.isDone()) {
                     quint16 *d = reinterpret_cast<quint16 *>(it.rawData());
                     d[0] = *(src++);
-                    if (transform) transform->transform( reinterpret_cast<quint8*>(d), reinterpret_cast<quint8*>(d), 1);
+                    if (transform) transform->transform(reinterpret_cast<quint8*>(d), reinterpret_cast<quint8*>(d), 1);
                     if (hasalpha) {
                         d[1] = *(src++);
                     } else {
@@ -628,7 +666,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
                 while (!it.isDone()) {
                     quint8 *d = it.rawData();
                     d[0] = (quint8)(stream.nextValue() * coeff);
-                    if (transform) transform->transform( d, d, 1);
+                    if (transform) transform->transform(d, d, 1);
                     if (hasalpha) {
                         d[1] = (quint8)(stream.nextValue() * coeff);
                     } else {
@@ -650,7 +688,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
                     d[2] = *(src++);
                     d[1] = *(src++);
                     d[0] = *(src++);
-                    if (transform) transform->transform( reinterpret_cast<quint8 *>(d), reinterpret_cast<quint8*>(d), 1);
+                    if (transform) transform->transform(reinterpret_cast<quint8 *>(d), reinterpret_cast<quint8*>(d), 1);
                     if (hasalpha) d[3] = *(src++);
                     else d[3] = quint16_MAX;
                     ++it;
@@ -662,7 +700,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
                     d[2] = (quint8)(stream.nextValue() * coeff);
                     d[1] = (quint8)(stream.nextValue() * coeff);
                     d[0] = (quint8)(stream.nextValue() * coeff);
-                    if (transform) transform->transform( d, d, 1);
+                    if (transform) transform->transform(d, d, 1);
                     if (hasalpha) d[3] = (quint8)(stream.nextValue() * coeff);
                     else d[3] = UCHAR_MAX;
                     ++it;
@@ -692,7 +730,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
             return KisImageBuilder_RESULT_UNSUPPORTED;
         }
     }
-    m_img->addNode(layer.data(), m_img->rootLayer().data());
+    m_image->addNode(layer.data(), m_image->rootLayer().data());
 
     png_read_end(png_ptr, end_info);
     iod->close();
@@ -701,7 +739,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(QIODevice* iod)
     png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
 
     delete reader;
-    m_img->unlock();
+    m_image->unlock();
     return KisImageBuilder_RESULT_OK;
 
 }
@@ -712,7 +750,7 @@ KisImageBuilder_Result KisPNGConverter::buildImage(const KUrl& uri)
     if (uri.isEmpty())
         return KisImageBuilder_RESULT_NO_URI;
 
-    if (!KIO::NetAccess::exists(uri, false, qApp -> activeWindow())) {
+    if (!KIO::NetAccess::exists(uri, KIO::NetAccess::SourceSide, qApp -> activeWindow())) {
         return KisImageBuilder_RESULT_NOT_EXIST;
     }
 
@@ -725,9 +763,9 @@ KisImageBuilder_Result KisPNGConverter::buildImage(const KUrl& uri)
         uriTF.setPath(tmpFile);
 
         // open the file
-        dbgFile << QFile::encodeName(uriTF.path()) << " " << uriTF.path() << " " << uriTF;
+        dbgFile << QFile::encodeName(uriTF.toLocalFile()) << " " << uriTF.toLocalFile() << " " << uriTF;
 //         QFile *fp = new QFile(QFile::encodeName(uriTF.path()) );
-        QFile *fp = new QFile(uriTF.path());
+        QFile *fp = new QFile(uriTF.toLocalFile());
         if (fp->exists()) {
             result = buildImage(fp);
         } else {
@@ -741,13 +779,13 @@ KisImageBuilder_Result KisPNGConverter::buildImage(const KUrl& uri)
 }
 
 
-KisImageSP KisPNGConverter::image()
+KisImageWSP KisPNGConverter::image()
 {
-    return m_img;
+    return m_image;
 }
 
 
-KisImageBuilder_Result KisPNGConverter::buildFile(const KUrl& uri, KisImageSP img, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
+KisImageBuilder_Result KisPNGConverter::buildFile(const KUrl& uri, KisImageWSP image, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
 {
     dbgFile << "Start writing PNG File " << uri;
     if (uri.isEmpty())
@@ -757,14 +795,14 @@ KisImageBuilder_Result KisPNGConverter::buildFile(const KUrl& uri, KisImageSP im
         return KisImageBuilder_RESULT_NOT_LOCAL;
     // Open a QIODevice for writing
     QFile *fp = new QFile(uri.toLocalFile());
-    KisImageBuilder_Result result = buildFile(fp, img, device, annotationsStart, annotationsEnd, options, metaData);
+    KisImageBuilder_Result result = buildFile(fp, image, device, annotationsStart, annotationsEnd, options, metaData);
     delete fp;
     return result;
 // TODO: if failure do            KIO::del(uri); // async
 
 }
 
-KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageSP img, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
+KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageWSP image, KisPaintDeviceSP device, vKisAnnotationSP_it annotationsStart, vKisAnnotationSP_it annotationsEnd, KisPNGOptions options, KisMetaData::Store* metaData)
 {
     if (!iodevice->open(QIODevice::WriteOnly)) {
         dbgFile << "Failed to open PNG File for writing";
@@ -774,14 +812,14 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageS
     if (!device)
         return KisImageBuilder_RESULT_INVALID_ARG;
 
-    if (!img)
+    if (!image)
         return KisImageBuilder_RESULT_EMPTY;
 
     // Setup the writing callback of libpng
-    int height = img->height();
-    int width = img->width();
+    int height = image->height();
+    int width = image->width();
     // Initialize structures
-    png_structp png_ptr =  png_create_write_struct(PNG_LIBPNG_VER_STRING, png_voidp_NULL, png_error_ptr_NULL, png_error_ptr_NULL);
+    png_structp png_ptr =  png_create_write_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
     if (!png_ptr) {
         return (KisImageBuilder_RESULT_FAILURE);
     }
@@ -809,7 +847,6 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageS
     /* set the zlib compression level */
     png_set_compression_level(png_ptr, options.compression);
 
-
     png_set_write_fn(png_ptr, (void*)iodevice, _write_fn, _flush_fn);
 
     /* set other zlib parameters */
@@ -829,9 +866,9 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageS
     // Try to compute a table of color if the colorspace is RGB8f
     png_colorp palette = 0;
     int num_palette = 0;
-    if (!options.alpha && KoID(device->colorSpace()->id()) == KoID("RGBA")) { // png doesn't handle indexed images and alpha, and only have indexed for RGB8
+    if (!options.alpha && options.tryToSaveAsIndexed && KoID(device->colorSpace()->id()) == KoID("RGBA")) { // png doesn't handle indexed images and alpha, and only have indexed for RGB8
         palette = new png_color[255];
-        KisRectConstIteratorPixel it = device->createRectConstIterator(0, 0, img->width(), img->height());
+        KisRectConstIteratorPixel it = device->createRectConstIterator(0, 0, image->width(), image->height());
         bool toomuchcolor = false;
         while (!it.isDone()) {
             const quint8* c = it.rawData();
@@ -899,18 +936,36 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageS
         dbgFile << "Trying to store annotation of type " << (*it) -> type() << " of size " << (*it) -> annotation() . size();
 
         if ((*it) -> type().startsWith(QString("krita_attribute:"))) { //
-                                                              // Attribute
-#ifdef __GNUC__                                                 \
-    #warning "it should be possible to save krita_attributes in the \"CHUNKs\""
+            // Attribute
+#ifdef __GNUC__
+#warning "it should be possible to save krita_attributes in the \"CHUNKs\""
 #endif
             dbgFile << "cannot save this annotation : " << (*it) -> type();
-        } else { // Profile
-            char* name = new char[(*it)->type().length()+1];
-            strcpy(name, (*it)->type().toAscii());
-            png_set_iCCP(png_ptr, info_ptr, name, PNG_COMPRESSION_TYPE_BASE, (char*)(*it)->annotation().data(), (*it) -> annotation() . size());
+        } else if ((*it)->type() == "kpp_version" || (*it)->type() == "kpp_preset" ) {
+            dbgFile << "Saving preset information " << (*it)->description();
+            png_textp      text = (png_textp) png_malloc(png_ptr, (png_uint_32) sizeof(png_text));
+            
+            QByteArray keyData = (*it)->description().toLatin1();
+            text[0].key = keyData.data();
+            text[0].text = (*it)->annotation().data();
+            text[0].text_length = (*it)->annotation().size();
+            text[0].compression = -1;
+            
+            png_set_text(png_ptr, info_ptr, text, 1);
+            png_free(png_ptr, text);
         }
         ++it;
     }
+    
+    // Save the color profile
+    const KoColorProfile* colorProfile = device->colorSpace()->profile();
+    QByteArray colorProfileData = colorProfile->rawData();
+    
+#if PNG_LIBPNG_VER_MAJOR >= 1 && PNG_LIBPNG_VER_MINOR >= 5
+    png_set_iCCP(png_ptr, info_ptr, "icc", PNG_COMPRESSION_TYPE_BASE, (const png_bytep)colorProfileData.data(), colorProfileData . size());
+#else
+    png_set_iCCP(png_ptr, info_ptr, "icc", PNG_COMPRESSION_TYPE_BASE, (char*)colorProfileData.data(), colorProfileData . size());
+#endif
 
     // read comments from the document information
     if (m_doc) {
@@ -963,9 +1018,9 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageS
             writeRawProfile(png_ptr, info_ptr, "iptc", buffer.data());
         }
         // Save XMP
-        if(options.xmp)
+        if (options.xmp)
 #if 1
-        // TODO enable when XMP support is finiehsed
+            // TODO enable when XMP support is finiehsed
         {
             dbgFile << "Trying to save XMP information";
             KisMetaData::IOBackend* xmpIO = KisMetaData::IOBackendRegistry::instance()->value("xmp");
@@ -979,6 +1034,12 @@ KisImageBuilder_Result KisPNGConverter::buildFile(QIODevice* iodevice, KisImageS
         }
 #endif
     }
+#if 0 // Unimplemented?
+    // Save resolution
+    int unit_type;
+    png_uint_32 x_resolution, y_resolution;
+#endif
+    png_set_pHYs(png_ptr, info_ptr, CM_TO_POINT(image->xRes()) * 100.0, CM_TO_POINT(image->yRes()) * 100.0, PNG_RESOLUTION_METER); // It is the "invert" macro because we convert from pointer-per-inchs to points
 
     // Save the information to the file
     png_write_info(png_ptr, info_ptr);
