@@ -1,3 +1,4 @@
+
 /*
  *  Copyright (c) 2004 Boudewijn Rempt <boud@valdyas.org>
  *  Copyright (c) 2007 Sven Langkamp <sven.langkamp@gmail.com>
@@ -21,8 +22,6 @@
  */
 
 #include "kis_selection_manager.h"
-#include "dialogs/kis_dlg_apply_profile.h" // If this isn't first, I get
-// errors in qslider.h. Weird... (BSAR)
 #include <QApplication>
 #include <QClipboard>
 #include <QColor>
@@ -49,6 +48,7 @@
 #include <KoToolProxy.h>
 
 #include "kis_adjustment_layer.h"
+#include "kis_node_manager.h"
 #include "canvas/kis_canvas2.h"
 #include "kis_config.h"
 #include "kis_convolution_painter.h"
@@ -65,16 +65,14 @@
 #include "kis_paint_device.h"
 #include "kis_paint_layer.h"
 #include "kis_painter.h"
-#include "kis_selected_transaction.h"
-#include "kis_selection.h"
 #include "kis_transaction.h"
+#include "kis_selection.h"
 #include "kis_types.h"
 #include "kis_canvas_resource_provider.h"
 #include "kis_undo_adapter.h"
 #include "kis_pixel_selection.h"
 #include "flake/kis_shape_selection.h"
 #include "commands/kis_selection_commands.h"
-#include "kis_selection_transaction.h"
 #include "kis_selection_mask.h"
 #include "flake/kis_shape_layer.h"
 #include "kis_selection_decoration.h"
@@ -97,17 +95,16 @@ KisSelectionManager::KisSelectionManager(KisView2 * view, KisDoc2 * doc)
         m_selectAll(0),
         m_deselect(0),
         m_clear(0),
-        m_delete(0),
         m_reselect(0),
         m_invert(0),
         m_toNewLayer(0),
-        m_feather(0),
         m_smooth(0),
         m_load(0),
         m_save(0),
         m_fillForegroundColor(0),
         m_fillBackgroundColor(0),
-        m_fillPattern(0)
+        m_fillPattern(0),
+        m_imageResizeToSelection(0)
 {
     m_clipboard = KisClipboard::instance();
 
@@ -123,8 +120,7 @@ KisSelectionManager::KisSelectionManager(KisView2 * view, KisDoc2 * doc)
 
 KisSelectionManager::~KisSelectionManager()
 {
-    while (!m_pluginActions.isEmpty())
-        delete m_pluginActions.takeFirst();
+    qDeleteAll(m_pluginActions);
 }
 
 void KisSelectionManager::setup(KActionCollection * collection)
@@ -148,18 +144,14 @@ void KisSelectionManager::setup(KActionCollection * collection)
     m_deselect = collection->addAction(KStandardAction::Deselect,  "deselect", this, SLOT(deselect()));
 
     m_clear = collection->addAction(KStandardAction::Clear,  "clear", this, SLOT(clear()));
-
-    m_delete = new KAction(KIcon("edit-delete"), i18n("D&elete"), this);
-    collection->addAction("delete", m_delete);
-    m_delete->setShortcut(QKeySequence(Qt::Key_Delete));
-    connect(m_delete, SIGNAL(triggered()), this, SLOT(deleteSelection()));
+    m_clear->setShortcut(QKeySequence((Qt::Key_Delete)));
 
     m_reselect  = new KAction(i18n("&Reselect"), this);
     collection->addAction("reselect", m_reselect);
     m_reselect->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_D));
     connect(m_reselect, SIGNAL(triggered()), this, SLOT(reselect()));
 
-    m_invert  = new KAction(i18n("&Invert"), this);
+    m_invert  = new KAction(i18n("&Invert Selection"), this);
     collection->addAction("invert", m_invert);
     m_invert->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_I));
     connect(m_invert, SIGNAL(triggered()), this, SLOT(invert()));
@@ -174,11 +166,6 @@ void KisSelectionManager::setup(KActionCollection * collection)
     m_cutToNewLayer->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_J));
     connect(m_cutToNewLayer, SIGNAL(triggered()), this, SLOT(cutToNewLayer()));
 
-    m_feather  = new KAction(i18n("Feather"), this);
-    collection->addAction("feather", m_feather);
-    m_feather->setShortcut(QKeySequence(Qt::CTRL + Qt::ALT + Qt::Key_D));
-    connect(m_feather, SIGNAL(triggered()), this, SLOT(feather()));
-
     m_fillForegroundColor  = new KAction(i18n("Fill with Foreground Color"), this);
     collection->addAction("fill_selection_foreground_color", m_fillForegroundColor);
     m_fillForegroundColor->setShortcut(QKeySequence(Qt::ALT + Qt::Key_Backspace));
@@ -195,7 +182,7 @@ void KisSelectionManager::setup(KActionCollection * collection)
 
     m_toggleDisplaySelection  = new KToggleAction(i18n("Display Selection"), this);
     collection->addAction("toggle_display_selection", m_toggleDisplaySelection);
-    m_toggleDisplaySelection->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_H));
+    m_toggleDisplaySelection->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_H));
     connect(m_toggleDisplaySelection, SIGNAL(triggered()), this, SLOT(toggleDisplaySelection()));
 
     m_toggleDisplaySelection->setCheckedState(KGuiItem(i18n("Hide Selection")));
@@ -205,6 +192,10 @@ void KisSelectionManager::setup(KActionCollection * collection)
     collection->addAction("smooth", m_smooth);
     connect(m_smooth, SIGNAL(triggered()), this, SLOT(smooth()));
 
+    m_imageResizeToSelection  = new KAction(i18n("Size Canvas to Size of Selection"), this);
+    collection->addAction("resizeimagetoselection", m_imageResizeToSelection);
+    connect(m_imageResizeToSelection, SIGNAL(triggered()), this, SLOT(imageResizeToSelection()));
+
 //     m_load
 //         = new KAction(i18n("Load..."),
 //                   0, 0,
@@ -251,13 +242,13 @@ void KisSelectionManager::updateGUI()
         return;
     }
 
-    KisImageWSP img = m_view->image();
+    KisImageWSP image = m_view->image();
     KisLayerSP l;
     KisPaintDeviceSP dev;
 
     bool enable = false;
 
-    if (img && m_view->activeDevice() && m_view->activeLayer()) {
+    if (image && m_view->activeDevice() && m_view->activeLayer()) {
         l = m_view->activeLayer();
 
         enable = l && !l->userLocked() && l->visible();
@@ -271,19 +262,18 @@ void KisSelectionManager::updateGUI()
 #endif
                     }
 
-        m_cut->setEnabled(enable);
     m_clear->setEnabled(enable);
+    m_cut->setEnabled(enable);
     m_fillForegroundColor->setEnabled(enable);
     m_fillBackgroundColor->setEnabled(enable);
     m_fillPattern->setEnabled(enable);
 
     m_cutToNewLayer->setEnabled(enable && l->selection());
-    m_selectAll->setEnabled(!img.isNull());
+    m_selectAll->setEnabled(!image.isNull());
 
     bool hasPixelSelection = enable && l->selection() && l->selection()->hasPixelSelection()
                              && !m_view->selection()->isDeselected();
     m_invert->setEnabled(hasPixelSelection);
-    m_feather->setEnabled(hasPixelSelection);
 
     m_smooth->setEnabled(enable);
 //    m_load->setEnabled(enable);
@@ -300,11 +290,12 @@ void KisSelectionManager::updateGUI()
     else
         m_reselect->setEnabled(false);
 
+    m_imageResizeToSelection->setEnabled(m_view->selection() && !m_view->selection()->isDeselected());
     if (!m_pluginActions.isEmpty()) {
         QListIterator<QAction *> i(m_pluginActions);
 
         while (i.hasNext()) {
-            i.next()->setEnabled(!img.isNull());
+            i.next()->setEnabled(!image.isNull());
         }
     }
 
@@ -312,7 +303,7 @@ void KisSelectionManager::updateGUI()
     // the current layer is locked.
 
     enable = false;
-    if (img && l) {
+    if (image && l) {
         enable = l->selection() && l->visible();
 
 
@@ -320,7 +311,6 @@ void KisSelectionManager::updateGUI()
 
     l = m_view->activeLayer();
     KisShapeLayer * shapeLayer = dynamic_cast<KisShapeLayer*>(l.data());
-
     bool shapePasteEnable = false;
     bool shapeCopyEnable = false;
     if (shapeLayer) {
@@ -340,20 +330,22 @@ void KisSelectionManager::updateGUI()
     }
 
     m_copy->setEnabled(enable || shapeCopyEnable);
-    m_paste->setEnabled(!img.isNull() && (m_clipboard->hasClip() || shapePasteEnable));
-    m_pasteNew->setEnabled(!img.isNull() && m_clipboard->hasClip());
+    m_paste->setEnabled(!image.isNull() && (m_clipboard->hasClip() || shapePasteEnable));
+    m_pasteNew->setEnabled(!image.isNull() && m_clipboard->hasClip());
     m_toNewLayer->setEnabled(enable);
 
-    if (shapeLayer || ((m_view->selection()
-                        && m_view->selection()->hasShapeSelection()
-                       )
-                       &&   m_view->canvasBase()->shapeManager()->selection()->count() > 0
-                      )
-       ) {
-        m_delete->setEnabled(true);
-    } else
-        m_delete->setEnabled(false);
-
+    //Handle the clear action disponibility
+    
+    if (m_view->canvasBase()->shapeManager()->selection()->count() > 0) {
+        m_clear->setEnabled(true);
+    }
+    else if (shapeLayer && m_view->canvasBase()->shapeManager()->shapes().empty()){
+        m_clear->setEnabled(false);
+    }
+    else {
+        m_clear->setEnabled(true);
+    }
+        
     updateStatusBar();
 
 }
@@ -373,8 +365,8 @@ void KisSelectionManager::selectionChanged()
 
 void KisSelectionManager::cut()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer = m_view->activeLayer();
     if (!layer) return;
@@ -383,14 +375,14 @@ void KisSelectionManager::cut()
 
     copy();
 
-    KisSelectedTransaction *t = new KisSelectedTransaction(i18n("Cut"), layer);
-    Q_CHECK_PTR(t);
+    KisSelectedTransaction transaction(i18n("Cut"), layer);
 
     layer->paintDevice()->clearSelection(m_view->selection());
     QRect rect = m_view->selection()->selectedRect();
     deselect();
 
-    m_view->document()->addCommand(t);
+    transaction.commit(m_view->image()->undoAdapter());
+
     layer->setDirty(rect);
 }
 
@@ -402,17 +394,18 @@ void KisSelectionManager::copy()
     KisShapeLayer * shapeLayer = dynamic_cast<KisShapeLayer*>(layer.data());
     if (shapeLayer) {
         m_view->canvasBase()->toolProxy()->copy();
-    } else {
+    } 
+    else {
 
-        KisImageWSP img = m_view->image();
-        if (!img) return;
+        KisImageWSP image = m_view->image();
+        if (!image) return;
 
         KisPaintDeviceSP dev = m_view->activeDevice();
         if (!dev) return;
 
         KisSelectionSP selection = m_view->selection();
 
-        QRect r = (selection) ? selection->selectedExactRect() : img->bounds();
+        QRect r = (selection) ? selection->selectedExactRect() : image->bounds();
 
         KisPaintDeviceSP clip = new KisPaintDevice(dev->colorSpace());
         Q_CHECK_PTR(clip);
@@ -449,7 +442,7 @@ void KisSelectionManager::copy()
             }
         }
 
-        m_clipboard->setClip(clip);
+        m_clipboard->setClip(clip, r.topLeft());
     }
 
     selectionChanged();
@@ -458,53 +451,39 @@ void KisSelectionManager::copy()
 
 void KisSelectionManager::paste()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
-    KisPaintDeviceSP clip = m_clipboard->clip();
+    //figure out where to position the clip
+    // XXX: Fix this for internal points & zoom! (BSAR)
+    QWidget * w = m_view->canvas();
+    QPoint center = QPoint(w->width() / 2, w->height() / 2);
+    QPoint bottomright = QPoint(w->width(), w->height());
+    if (bottomright.x() > image->width())
+        center.setX(image->width() / 2);
+    if (bottomright.y() > image->height())
+        center.setY(image->height() / 2);
+
+    const KoCanvasBase* canvasBase = m_view->canvasBase();
+    const KoViewConverter* viewConverter = m_view->canvasBase()->viewConverter();
+
+    KisPaintDeviceSP clip = m_clipboard->clip(
+        QPoint(
+            viewConverter->viewToDocumentX(canvasBase->canvasController()->canvasOffsetX()) + center.x(),
+            viewConverter->viewToDocumentY(canvasBase->canvasController()->canvasOffsetY()) + center.y()));
 
     if (clip) {
-        KisPaintLayer *layer = new KisPaintLayer(img.data(), img->nextLayerName() + i18n("(pasted)"), OPACITY_OPAQUE);
+        KisPaintLayer *layer = new KisPaintLayer(image.data(), image->nextLayerName() + i18n("(pasted)"), OPACITY_OPAQUE_U8, clip);
         Q_CHECK_PTR(layer);
 
-        QRect r = clip->exactBounds();
-        KisPainter gc;
-        gc.begin(layer->paintDevice());
-        gc.setCompositeOp(COMPOSITE_COPY);
-        gc.bitBlt(0, 0, clip, r.x(), r.y(), r.width(), r.height());
-        gc.end();
-
-        //figure out where to position the clip
-        // XXX: Fix this for internal points & zoom! (BSAR)
-        QWidget * w = m_view->canvas();
-        QPoint center = QPoint(w->width() / 2, w->height() / 2);
-        QPoint bottomright = QPoint(w->width(), w->height());
-        if (bottomright.x() > img->width())
-            center.setX(img->width() / 2);
-        if (bottomright.y() > img->height())
-            center.setY(img->height() / 2);
-        center -= QPoint(r.width() / 2, r.height() / 2);
-
-        const KoCanvasBase* canvasBase = m_view->canvasBase();
-        const KoViewConverter* viewConverter = m_view->canvasBase()->viewConverter();
-
-        layer->setX(viewConverter->viewToDocumentX(canvasBase->canvasController()->canvasOffsetX() + center.x()));
-        layer->setY(viewConverter->viewToDocumentY(canvasBase->canvasController()->canvasOffsetY() + center.y()));
-
-        qDebug() << "layer x,y:" << layer->x() << ", " << layer->y();
-
-        /*XXX CBR have an idea of asking the user if he is about to paste a clip in another cs than that of
-          the image if that is what he want rather than silently converting
-          if ( ! ( *clip->colorSpace == *img ->colorSpace()) )
-          if (dlg->exec() == QDialog::Accepted)
-          layer->convertTo(img->colorSpace());
-        */
         if (m_view->activeLayer()) {
             m_adapter->addNode(layer , m_view->activeLayer()->parent(), m_view->activeLayer().data());
         } else {
-            m_adapter->addNode(layer , img->rootLayer(), 0);
+            m_adapter->addNode(layer , image->rootLayer(), 0);
         }
         layer->setDirty();
+        m_view->nodeManager()->activateNode(layer);
+
     } else
         m_view->canvasBase()->toolProxy()->paste();
 }
@@ -516,7 +495,7 @@ void KisSelectionManager::pasteAt()
 
 void KisSelectionManager::pasteNew()
 {
-    KisPaintDeviceSP clip = m_clipboard->clip();
+    KisPaintDeviceSP clip = m_clipboard->clip(QPoint(0,0));
     if (!clip) return;
 
     QRect r = clip->exactBounds();
@@ -532,21 +511,18 @@ void KisSelectionManager::pasteNew()
     if (!doc) return;
 
     Q_ASSERT(doc->undoAdapter() != 0);
-    doc->undoAdapter()->setUndo(false);
 
-    KisImageWSP img = new KisImage(doc->undoAdapter(), r.width(), r.height(),
-                                   KoColorSpaceRegistry::instance()->colorSpace(clip->colorSpace()->id(), clip->colorSpace()->profile()), "Pasted");    // TODO should be translated ?
-    KisPaintLayerSP layer = new KisPaintLayer(img.data(), clip->objectName(), OPACITY_OPAQUE, clip->colorSpace());
+    KisImageWSP image = new KisImage(doc->undoAdapter(), r.width(), r.height(),
+                                   KoColorSpaceRegistry::instance()->colorSpace(clip->colorSpace()->colorModelId().id(), clip->colorSpace()->colorDepthId().id(), clip->colorSpace()->profile()), "Pasted");    // TODO should be translated ?
+    KisPaintLayerSP layer = new KisPaintLayer(image.data(), clip->objectName(), OPACITY_OPAQUE_U8, clip->colorSpace());
 
     KisPainter p(layer->paintDevice());
     p.setCompositeOp(COMPOSITE_COPY);
     p.bitBlt(0, 0, clip, r.x(), r.y(), r.width(), r.height());
     p.end();
 
-    img->addNode(layer.data(), img->rootLayer());
-    doc->setCurrentImage(img);
-
-    doc->undoAdapter()->setUndo(true);
+    image->addNode(layer.data(), image->rootLayer());
+    doc->setCurrentImage(image);
 
     KoMainWindow *win = new KoMainWindow(doc->componentData());
     win->show();
@@ -555,27 +531,34 @@ void KisSelectionManager::pasteNew()
 
 void KisSelectionManager::selectAll()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
-    QUndoCommand* selectionCmd = new QUndoCommand(i18n("Select All"));
+    KisUndoAdapter *undoAdapter = m_view->image()->undoAdapter();
+    undoAdapter->beginMacro(i18n("Select All"));
 
-    if (!m_view->selection())
-        new KisSetGlobalSelectionCommand(img, selectionCmd);
-    KisSelectionSP selection = m_view->selection();
+    if (!image->globalSelection()) {
+        QUndoCommand *globalSelectionCommand =
+            new KisSetGlobalSelectionCommand(image, 0, 0);
 
-    new KisSelectionTransaction(QString(), img, selection, selectionCmd);
+        undoAdapter->addCommand(globalSelectionCommand);
+    }
 
-    selection->getOrCreatePixelSelection()->select(img->bounds());
+    KisSelectionSP selection = image->globalSelection();
+
+    KisSelectionTransaction transaction(QString(), image, selection);
+    selection->getOrCreatePixelSelection()->select(image->bounds());
+    transaction.commit(undoAdapter);
+
+    undoAdapter->endMacro();
 
     m_view->selectionManager()->selectionChanged();
-    m_view->document()->addCommand(selectionCmd);
 }
 
 void KisSelectionManager::deselect()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisSelectionSP sel = m_view->selection();
     if (!sel) return;
@@ -583,52 +566,54 @@ void KisSelectionManager::deselect()
     KisLayerSP layer = m_view->activeLayer();
     if (!layer) return;
 
-    if (layer->selectionMask()) {
-        KisDeselectLocalSelectionCommand* cmd = new KisDeselectLocalSelectionCommand(img, layer->selectionMask());
+    if (image->globalSelection()) {
+        KisDeselectGlobalSelectionCommand* cmd = new KisDeselectGlobalSelectionCommand(image);
         m_view->document()->addCommand(cmd);
-    } else {
-        if (img->globalSelection()) {
-            KisDeselectGlobalSelectionCommand* cmd = new KisDeselectGlobalSelectionCommand(img);
-            m_view->document()->addCommand(cmd);
-        }
     }
 }
 
 
 void KisSelectionManager::clear()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisPaintDeviceSP dev = m_view->activeDevice();
-    if (!dev) return;
 
-    KisSelectionSP sel = m_view->selection();
+    if(m_view->canvasBase()->shapeManager()->selection()->count()){
+        deleteSelection();
+    }else if(dev){
+        
+        KisSelectionSP sel = m_view->selection();
 
-    KisTransaction * t = new KisTransaction(i18n("Clear"), dev);
+        KisTransaction transaction(i18n("Clear"), dev);
 
-    if (sel)
-        dev->clearSelection(sel);
-    else {
-        dev->clear();
-        dev->setDirty();
+        if (sel){
+            dev->clearSelection(sel);
+        }else{
+            dev->clear();
+            dev->setDirty();
+        }
+
+        updateGUI();
+
+        transaction.commit(image->undoAdapter());
+        dev->setDirty(image->bounds());
     }
-
-    dev->setDirty(img->bounds());
-    img->undoAdapter()->addCommand(t);
 }
 
 void KisSelectionManager::deleteSelection()
 {
-    if (m_view->canvasBase()->shapeManager()->selection())
-        m_view->canvasBase()->toolProxy()->deleteSelection();
+    if (m_view->canvasBase()->shapeManager()->selection()){
+        m_view->canvasBase()->toolProxy()->deleteSelection();  
+    }
     updateGUI();
 }
 
 void KisSelectionManager::fill(const KoColor& color, bool fillWithPattern, const QString& transactionText)
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisPaintDeviceSP dev = m_view->activeDevice();
     if (!dev) return;
@@ -640,25 +625,21 @@ void KisSelectionManager::fill(const KoColor& color, bool fillWithPattern, const
     KisFillPainter painter(filled);
 
     if (fillWithPattern) {
-        painter.fillRect(0, 0, img->width(), img->height(),
+        painter.fillRect(0, 0, image->width(), image->height(),
                          m_view->resourceProvider()->currentPattern());
     } else {
-        painter.fillRect(0, 0, img->width(), img->height(), color);
+        painter.fillRect(0, 0, image->width(), image->height(), color);
     }
 
     painter.end();
 
     KisPainter painter2(dev, selection);
 
-    if (img->undo()) painter2.beginTransaction(transactionText);
-
-    painter2.bitBlt(0, 0, filled, 0, 0, img->width(), img->height());
+    painter2.beginTransaction(transactionText);
+    painter2.bitBlt(0, 0, filled, 0, 0, image->width(), image->height());
+    painter2.endTransaction(image->undoAdapter());
 
     dev->setDirty();
-
-    if (img->undo()) {
-        img->undoAdapter()->addCommand(painter2.endTransaction());
-    }
 }
 
 void KisSelectionManager::fillForegroundColor()
@@ -678,48 +659,41 @@ void KisSelectionManager::fillPattern()
 
 void KisSelectionManager::reselect()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer = m_view->activeLayer();
     if (!layer) return;
 
-    if (layer->selectionMask() && layer->selectionMask()->deleselectedSelection()) {
-        KisReselectLocalSelectionCommand* cmd = new KisReselectLocalSelectionCommand(img, layer->selectionMask());
-        m_view->document()->addCommand(cmd);
-    } else {
-        if (img->globalSelection()) {
-            KisReselectGlobalSelectionCommand* cmd = new KisReselectGlobalSelectionCommand(img);
-            m_view->document()->addCommand(cmd);
-        }
+    if (image->globalSelection()) {
+        KisReselectGlobalSelectionCommand* cmd = new KisReselectGlobalSelectionCommand(image);
+        image->undoAdapter()->addCommand(cmd);
     }
 }
 
 
 void KisSelectionManager::invert()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisSelectionSP selection = m_view->selection();
     if (!selection) return;
 
-    KisPixelSelectionSP s = selection->getOrCreatePixelSelection();
+    KisPixelSelectionSP pixelSelection = selection->getOrCreatePixelSelection();
 
-    KisSelectionTransaction * t = new KisSelectionTransaction(i18n("Invert"), img, selection);
-    Q_CHECK_PTR(t);
-
-    s->invert();
-    s->setDirty(img->bounds());
+    KisSelectionTransaction transaction(i18n("Invert"), image, selection);
+    pixelSelection->invert();
+    transaction.commit(image->undoAdapter());
 
+    pixelSelection->setDirty(image->bounds());
     selectionChanged();
-    m_view->document()->addCommand(t);
 }
 
 void KisSelectionManager::copySelectionToNewLayer()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisLayerSP layer = m_view->activeLayer();
     if (!layer) return;
@@ -730,8 +704,8 @@ void KisSelectionManager::copySelectionToNewLayer()
 
 void KisSelectionManager::cutToNewLayer()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisPaintDeviceSP dev = m_view->activeDevice();
     if (!dev) return;
@@ -740,50 +714,6 @@ void KisSelectionManager::cutToNewLayer()
     paste();
 }
 
-
-void KisSelectionManager::feather()
-{
-    KisImageWSP img = m_view->image();
-    if (!img) return;
-    if (!m_view->selection())
-        return;
-
-    KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
-    KisSelectionTransaction * t = new KisSelectionTransaction(i18n("Feather..."), img, m_view->selection());
-    Q_CHECK_PTR(t);
-
-
-    // XXX: we should let gaussian blur & others influence alpha channels as well
-    // (on demand of the caller)
-
-    KisConvolutionPainter painter(selection);
-
-    KisConvolutionKernelSP k = new KisConvolutionKernel(3, 3, 0, 16);
-
-    k->data()[0] = 1;
-    k->data()[1] = 2;
-    k->data()[2] = 1;
-    k->data()[3] = 2;
-    k->data()[4] = 4;
-    k->data()[5] = 2;
-    k->data()[6] = 1;
-    k->data()[7] = 2;
-    k->data()[8] = 1;
-
-    QRect rect = selection->selectedExactRect();
-    // Make sure we've got enough space around the edges.
-    rect = QRect(rect.x() - 3, rect.y() - 3, rect.width() + 6, rect.height() + 6);
-    rect &= QRect(0, 0, img->width(), img->height());
-    painter.setChannelFlags(selection->colorSpace()->channelFlags(false, true, false, false));
-    painter.applyMatrix(k, selection, rect.topLeft(), rect.topLeft(), rect.size(), BORDER_AVOID);
-    painter.end();
-
-    m_view->document()->addCommand(t);
-    m_view->selection()->setDirty(img->bounds());
-    selectionChanged();
-
-}
-
 void KisSelectionManager::toggleDisplaySelection()
 {
     KisCanvasDecoration* decoration = m_view->canvasBase()->decoration("selection");
@@ -801,14 +731,14 @@ bool KisSelectionManager::displaySelection()
 
 void KisSelectionManager::grow(qint32 xradius, qint32 yradius)
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     if (!m_view->selection()) return;
     KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
 
     //determine the layerSize
-    QRect layerSize = img->bounds();
+    QRect layerSize = image->bounds();
 
     /*
       Any bugs in this function are probably also in thin_region
@@ -820,7 +750,7 @@ void KisSelectionManager::grow(qint32 xradius, qint32 yradius)
     if (xradius <= 0 || yradius <= 0)
         return;
 
-    KisSelectionTransaction *t = new KisSelectionTransaction(i18n("Grow"), img, m_view->selection());
+    KisSelectionTransaction transaction(i18n("Grow Selection"), image, m_view->selection());
 
     max = new quint8* [layerSize.width() + 2 * xradius];
     buf = new quint8* [yradius + 1];
@@ -918,25 +848,25 @@ void KisSelectionManager::grow(qint32 xradius, qint32 yradius)
     delete[] buf;
     delete[] out;
 
-    m_view->document()->addCommand(t);
-    m_view->selection()->setDirty(img->bounds());
+    transaction.commit(image->undoAdapter());
+
+    m_view->selection()->setDirty(image->bounds());
     selectionChanged();
 }
 
 void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
 {
 
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     if (!m_view->selection()) return;
     KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
 
-    KisSelectionTransaction *t = new KisSelectionTransaction(i18n("Shrink"), img, m_view->selection());
-    Q_CHECK_PTR(t);
+    KisSelectionTransaction transaction(i18n("Shrink Selection"), image, m_view->selection());
 
     //determine the layerSize
-    QRect layerSize = img->bounds();
+    QRect layerSize = image->bounds();
     /*
       pretty much the same as fatten_region only different
       blame all bugs in this function on jaycox@gimp.org
@@ -1068,8 +998,9 @@ void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
     delete[] buf;
     delete[] out;
 
-    m_view->document()->addCommand(t);
-    m_view->selection()->setDirty(img->bounds());
+    transaction.commit(image->undoAdapter());
+
+    m_view->selection()->setDirty(image->bounds());
     selectionChanged();
 }
 
@@ -1077,8 +1008,8 @@ void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
 
 void KisSelectionManager::smooth()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     if (!m_view->selection()) return;
     if (!m_view->activeLayer()) return;
@@ -1131,7 +1062,7 @@ void KisSelectionManager::smooth()
 
     delete[] out;
 #if 0
-    dev->setDirty(img->bounds());
+    dev->setDirty(image->bounds());
 #endif
 }
 
@@ -1139,8 +1070,8 @@ void KisSelectionManager::smooth()
 
 void KisSelectionManager::erode()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisSelectionSP selection = m_view->selection();
     if (!selection) return;
@@ -1207,8 +1138,8 @@ void KisSelectionManager::erode()
 
 void KisSelectionManager::dilate()
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     KisSelectionSP selection = m_view->selection();
     if (!selection) return;
@@ -1272,17 +1203,16 @@ void KisSelectionManager::dilate()
 
 void KisSelectionManager::border(qint32 xradius, qint32 yradius)
 {
-    KisImageWSP img = m_view->image();
-    if (!img) return;
+    KisImageWSP image = m_view->image();
+    if (!image) return;
 
     if (!m_view->selection()) return;
     KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
 
     //determine the layerSize
-    QRect layerSize = img->bounds();
+    QRect layerSize = image->bounds();
 
-    KisSelectionTransaction *t = new KisSelectionTransaction(i18n("Border"), img, m_view->selection());
-    Q_CHECK_PTR(t);
+    KisSelectionTransaction transaction(i18n("Border Selection"), image, m_view->selection());
 
     quint8  *buf[3];
     quint8 **density;
@@ -1320,8 +1250,9 @@ void KisSelectionManager::border(qint32 xradius, qint32 yradius)
             delete[] source[i];
         delete[] transition;
 
-        m_view->document()->addCommand(t);
-        m_view->selection()->setDirty(img->bounds());
+        transaction.commit(image->undoAdapter());
+
+        m_view->selection()->setDirty(image->bounds());
         selectionChanged();
         return;
     }
@@ -1485,9 +1416,57 @@ void KisSelectionManager::border(qint32 xradius, qint32 yradius)
     density -= xradius;
     delete[] density;
 
+    transaction.commit(image->undoAdapter());
 
-    m_view->document()->addCommand(t);
-    m_view->selection()->setDirty(img->bounds());
+    m_view->selection()->setDirty(image->bounds());
+    selectionChanged();
+}
+
+void KisSelectionManager::feather(qint32 radius)
+{
+    KisImageWSP image = m_view->image();
+    if (!image) return;
+    if (!m_view->selection())
+        return;
+
+    KisPixelSelectionSP selection = m_view->selection()->getOrCreatePixelSelection();
+    KisSelectionTransaction transaction(i18n("Feather Selection"), image, m_view->selection());
+
+    // compute horizontal kernel
+    const uint kernelSize = radius * 2 + 1;
+    Matrix<qreal, Dynamic, Dynamic> gaussianMatrix(1, kernelSize);
+
+    const qreal multiplicand = 1 / (2 * M_PI * radius * radius);
+    const qreal exponentMultiplicand = 1 / (2 * radius * radius);
+
+    for (uint x = 0; x < kernelSize; x++)
+    {
+        uint xDistance = qAbs((int)radius - (int)x);
+        gaussianMatrix(0, x) = multiplicand * exp( -(qreal)((xDistance * xDistance) + (radius * radius)) * exponentMultiplicand );
+    }
+
+    KisConvolutionKernelSP kernelHoriz = KisConvolutionKernel::fromMatrix(gaussianMatrix, 0, gaussianMatrix.sum());
+    KisConvolutionKernelSP kernelVertical = KisConvolutionKernel::fromMatrix(gaussianMatrix.transpose(), 0, gaussianMatrix.sum());
+
+    QRect rect = selection->selectedExactRect();
+    // Make sure we've got enough space around the edges.
+    rect.adjust(-kernelSize, -kernelSize, kernelSize, kernelSize);
+    rect &= QRect(0, 0, image->width(), image->height());
+
+    KisPaintDeviceSP interm = new KisPaintDevice(selection->colorSpace());
+    KisConvolutionPainter horizPainter(interm);
+    horizPainter.setChannelFlags(interm->colorSpace()->channelFlags(false, true, false, false));
+    horizPainter.applyMatrix(kernelHoriz, selection, rect.topLeft(), rect.topLeft(), rect.size(), BORDER_AVOID);
+    horizPainter.end();
+
+    KisConvolutionPainter verticalPainter(selection);
+    verticalPainter.setChannelFlags(selection->colorSpace()->channelFlags(false, true, false, false));
+    verticalPainter.applyMatrix(kernelVertical, interm, rect.topLeft(), rect.topLeft(), rect.size(), BORDER_AVOID);
+    verticalPainter.end();
+
+    transaction.commit(image->undoAdapter());
+
+    m_view->selection()->setDirty(image->bounds());
     selectionChanged();
 }
 
@@ -1583,4 +1562,19 @@ void KisSelectionManager::shapeSelectionChanged()
     }
 }
 
+void KisSelectionManager::imageResizeToSelection()
+{
+    KisSelectionSP selection = m_view->selection();
+    KisUndoAdapter * undoAdapter = m_view->undoAdapter();
+
+    KisImageWSP image = m_view->image();
+
+    if (image && selection) {
+
+        undoAdapter->beginMacro(i18n("Resize Image to Size of Selection"));
+        image->resize(selection->selectedExactRect(), true);
+        undoAdapter->endMacro();
+    }
+}
+
 #include "kis_selection_manager.moc"
