@@ -74,6 +74,7 @@ KisStatusBar::KisStatusBar(KisView2 * view)
     m_pointerPositionLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
     m_pointerPositionLabel->setMinimumWidth(100);
     view->addStatusBarItem(m_pointerPositionLabel);
+    m_pointerPositionLabel->setVisible(false);
 
     m_progress = new KisProgressWidget(view);
     m_progress->setMaximumWidth(225);
@@ -85,6 +86,12 @@ KisStatusBar::KisStatusBar(KisView2 * view)
 
 KisStatusBar::~KisStatusBar()
 {
+    m_view->removeStatusBarItem(m_selectionStatusLabel);
+    m_view->removeStatusBarItem(m_statusBarStatusLabel);
+    m_view->removeStatusBarItem(m_statusBarProfileLabel);
+    m_view->removeStatusBarItem(m_imageSizeLabel);
+    m_view->removeStatusBarItem(m_pointerPositionLabel);
+    m_view->removeStatusBarItem(m_progress);
 }
 
 #define EPSILON 1e-6
@@ -115,9 +122,9 @@ void KisStatusBar::imageSizeChanged(qint32 w, qint32 h)
     m_imageSizeLabel->setText(QString("%1 x %2").arg(w).arg(h));
 }
 
-void KisStatusBar::setSelection(KisImageWSP img)
+void KisStatusBar::setSelection(KisImageWSP image)
 {
-    Q_UNUSED(img);
+    Q_UNUSED(image);
 
     KisSelectionSP selection = m_view->selection();
     if (selection && !selection->isDeselected()) {
@@ -131,18 +138,18 @@ void KisStatusBar::setSelection(KisImageWSP img)
     m_selectionStatusLabel->setToolTip(i18n("No Selection"));
 }
 
-void KisStatusBar::setProfile(KisImageWSP img)
+void KisStatusBar::setProfile(KisImageWSP image)
 {
     if (m_statusBarProfileLabel == 0) {
         return;
     }
 
-    if (!img) return;
+    if (!image) return;
 
-    if (img->profile() == 0) {
+    if (image->profile() == 0) {
         m_statusBarProfileLabel->setText(i18n("No profile"));
     } else {
-        m_statusBarProfileLabel->setText(img->colorSpace()->name() + "  " + img->profile()->name());
+        m_statusBarProfileLabel->setText(image->colorSpace()->name() + "  " + image->profile()->name());
     }
 
 }
@@ -163,5 +170,4 @@ KisProgressWidget* KisStatusBar::progress()
     return m_progress;
 }
 
-
 #include "kis_statusbar.moc"
