@@ -5,7 +5,7 @@
  *                1999 Michael Koch    <koch@kde.org>
  *                1999 Carsten Pfeiffer <pfeiffer@kde.org>
  *                2002 Patrick Julien <freak@codepimps.org>
- *                2003-2007 Boudewijn Rempt <boud@valdyas.org>
+ *                2003-20010 Boudewijn Rempt <boud@valdyas.org>
  *                2004 Clarence Dang <dang@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
+#include <stdio.h>
+
 #include "kis_view2.h"
 #include <qprinter.h>
 
@@ -36,7 +38,6 @@
 #include <QPrintDialog>
 #include <QObject>
 
-#include <k3urldrag.h>
 #include <kaction.h>
 #include <klocale.h>
 #include <kmenu.h>
@@ -51,9 +52,10 @@
 #include <kxmlguiwindow.h>
 #include <kxmlguifactory.h>
 #include <kmessagebox.h>
+#include <ktemporaryfile.h>
+#include <kactioncollection.h>
 
 #include <KoMainWindow.h>
-#include <KoCanvasController.h>
 #include <KoSelection.h>
 #include <KoToolBoxFactory.h>
 #include <KoZoomHandler.h>
@@ -64,8 +66,9 @@
 #include <KoDockRegistry.h>
 #include <KoResourceServerProvider.h>
 #include <KoCompositeOp.h>
-
-#include <kactioncollection.h>
+#include <KoTemplateCreateDia.h>
+#include <KoCanvasControllerWidget.h>
+#include <KoDocumentEntry.h>
 
 #include <kis_image.h>
 #include <kis_undo_adapter.h>
@@ -75,6 +78,8 @@
 #include "kis_config_notifier.h"
 #include "kis_statusbar.h"
 #include "canvas/kis_canvas2.h"
+#include "canvas/kis_canvas_controller.h"
+#include "kis_coordinates_converter.h"
 #include "kis_doc2.h"
 #include "kis_factory2.h"
 #include "kis_filter_manager.h"
@@ -102,7 +107,29 @@
 #include <kis_paint_layer.h>
 #include "kis_progress_widget.h"
 
+#include <QDebug>
+#include <QPoint>
+#include "kis_paintop_box.h"
 #include "kis_node_commands_adapter.h"
+#include <kis_paintop_preset.h>
+#include "ko_favorite_resource_manager.h"
+#include "kis_paintop_box.h"
+
+class BlockingUserInputEventFilter : public QObject
+{
+    bool eventFilter(QObject *watched, QEvent *event)
+    {
+        Q_UNUSED(watched);
+        if(dynamic_cast<QWheelEvent*>(event)
+            || dynamic_cast<QKeyEvent*>(event)
+            || dynamic_cast<QMouseEvent*>(event)) {
+            return true;
+        }
+        else {
+            return false;
+        }
+    }
+};
 
 class KisView2::KisView2Private
 {
@@ -110,21 +137,22 @@ class KisView2::KisView2Private
 public:
 
     KisView2Private()
-            : canvas(0)
-            , doc(0)
-            , viewConverter(0)
-            , canvasController(0)
-            , resourceProvider(0)
-            , filterManager(0)
-            , statusBar(0)
-            , selectionManager(0)
-            , controlFrame(0)
-            , nodeManager(0)
-            , zoomManager(0)
-            , imageManager(0)
-            , gridManager(0)
-            , perspectiveGridManager(0)
-            , paintingAssistantManager(0) {
+        : canvas(0)
+        , doc(0)
+        , viewConverter(0)
+        , canvasController(0)
+        , resourceProvider(0)
+        , filterManager(0)
+        , statusBar(0)
+        , selectionManager(0)
+        , controlFrame(0)
+        , nodeManager(0)
+        , zoomManager(0)
+        , imageManager(0)
+        , gridManager(0)
+        , perspectiveGridManager(0)
+        , paintingAssistantManager(0) {
+
     }
 
     ~KisView2Private() {
@@ -141,6 +169,7 @@ public:
         delete perspectiveGridManager;
         delete paintingAssistantManager;
         delete viewConverter;
+        delete statusBar;
     }
 
 public:
@@ -153,6 +182,8 @@ public:
     KisFilterManager * filterManager;
     KisStatusBar * statusBar;
     KAction * totalRefresh;
+    KAction* mirrorCanvas;
+    KAction* createTemplate;
     KisSelectionManager *selectionManager;
     KisControlFrame * controlFrame;
     KisNodeManager * nodeManager;
@@ -161,13 +192,16 @@ public:
     KisGridManager * gridManager;
     KisPerspectiveGridManager * perspectiveGridManager;
     KisPaintingAssistantsManager* paintingAssistantManager;
+    KoFavoriteResourceManager* favoriteResourceManager;
+    BlockingUserInputEventFilter blockingEventFilter;
 };
 
 
 KisView2::KisView2(KisDoc2 * doc, QWidget * parent)
-        : KoView(doc, parent),
-        m_d(new KisView2Private())
+    : KoView(doc, parent),
+    m_d(new KisView2Private())
 {
+
     setFocusPolicy(Qt::NoFocus);
 
     m_d->totalRefresh = new KAction(i18n("Total Refresh"), this);
@@ -175,6 +209,14 @@ KisView2::KisView2(KisDoc2 * doc, QWidget * parent)
     m_d->totalRefresh->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_R));
     connect(m_d->totalRefresh, SIGNAL(triggered()), this, SLOT(slotTotalRefresh()));
 
+    m_d->createTemplate = new KAction( i18n( "&Create Template From Image..." ), this);
+    actionCollection()->addAction("createTemplate", m_d->createTemplate);
+    connect(m_d->createTemplate, SIGNAL(triggered()), this, SLOT(slotCreateTemplate()));
+
+    KAction *firstRun = new KAction( i18n( "Load Tutorial"), this);
+    actionCollection()->addAction("first_run", firstRun);
+    connect(firstRun, SIGNAL(triggered()), this, SLOT(slotFirstRun()));
+
     setComponentData(KisFactory2::componentData(), false);
 
     if (!doc->isReadWrite()) {
@@ -190,16 +232,23 @@ KisView2::KisView2(KisDoc2 * doc, QWidget * parent)
     m_d->doc = doc;
     m_d->viewConverter = new KoZoomHandler();
 
-    m_d->canvasController = new KoCanvasController(this);
-    m_d->canvasController->setDrawShadow(false);
-    m_d->canvasController->setMargin(10);
-    m_d->canvasController->setCanvasMode(KoCanvasController::Infinite);
+    KoCanvasControllerWidget *canvasController = new KisCanvasController(this);
+    canvasController->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+    canvasController->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+    canvasController->setDrawShadow(false);
+    canvasController->setCanvasMode(KoCanvasController::Infinite);
+    KisConfig cfg;
+    canvasController->setZoomWithWheel(cfg.zoomWithWheel());
+    canvasController->setVastScrolling(cfg.vastScrolling());
+
+    m_d->canvasController = canvasController;
+
 
     m_d->canvas = new KisCanvas2(m_d->viewConverter, this, doc->shapeController());
     m_d->canvasController->setCanvas(m_d->canvas);
 
     m_d->resourceProvider = new KisCanvasResourceProvider(this);
-    m_d->resourceProvider->setCanvasResourceProvider(m_d->canvas->resourceProvider());
+    m_d->resourceProvider->setResourceManager(m_d->canvas->resourceManager());
 
     Q_ASSERT(m_d->canvasController);
     KoToolManager::instance()->addController(m_d->canvasController);
@@ -211,21 +260,39 @@ KisView2::KisView2(KisDoc2 * doc, QWidget * parent)
     createManagers();
     createActions();
 
+    m_d->mirrorCanvas = new KToggleAction(i18n("Mirror Image"), this);
+    m_d->mirrorCanvas->setChecked(false);
+    actionCollection()->addAction("mirror_canvas", m_d->mirrorCanvas);
 
-    KoToolBoxFactory toolBoxFactory(m_d->canvasController, i18n("Tools"));
-    createDockWidget(&toolBoxFactory);
+    m_d->mirrorCanvas->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_I));
+    connect(m_d->mirrorCanvas, SIGNAL(toggled(bool)),m_d->canvas, SLOT(mirrorCanvas(bool)));
 
-    KoDockerManager *dockerMng = dockerManager();
-    if (!dockerMng) {
-        dockerMng = new KoDockerManager(this);
-        setDockerManager(dockerMng);
+    KAction *rotateCanvasRight = new KAction(i18n("Rotate Canvas Right"), this);
+    actionCollection()->addAction("rotate_canvas_right", rotateCanvasRight);
+    rotateCanvasRight->setShortcut(QKeySequence("Ctrl+]"));
+    connect(rotateCanvasRight, SIGNAL(triggered()),m_d->canvas, SLOT(rotateCanvasRight15()));
+
+    KAction *rotateCanvasLeft = new KAction(i18n("Rotate Canvas Left"), this);
+    actionCollection()->addAction("rotate_canvas_left", rotateCanvasLeft);
+    rotateCanvasLeft->setShortcut(QKeySequence("Ctrl+["));
+    connect(rotateCanvasLeft, SIGNAL(triggered()),m_d->canvas, SLOT(rotateCanvasLeft15()));
+
+    KAction *resetCanvasTransformations = new KAction(i18n("Reset Canvas Transformations"), this);
+    actionCollection()->addAction("reset_canvas_transformations", resetCanvasTransformations);
+    resetCanvasTransformations->setShortcut(QKeySequence("Ctrl+'"));
+    connect(resetCanvasTransformations, SIGNAL(triggered()),m_d->canvas, SLOT(resetCanvasTransformations()));
+
+    if (shell())
+    {
+        KoToolBoxFactory toolBoxFactory(m_d->canvasController, i18n("Tools"));
+        shell()->createDockWidget(&toolBoxFactory);
+
+        connect(canvasController, SIGNAL(toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*)),
+                shell()->dockerManager(), SLOT(newOptionWidgets(const  QMap<QString, QWidget *> &, QWidget*)));
     }
 
-    connect(m_d->canvasController, SIGNAL(toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*)),
-            dockerMng, SLOT(newOptionWidgets(const  QMap<QString, QWidget *> &, QWidget*)));
-
     m_d->statusBar = new KisStatusBar(this);
-    connect(m_d->canvasController, SIGNAL(documentMousePositionChanged(const QPointF &)),
+    connect(m_d->canvasController->proxyObject, SIGNAL(documentMousePositionChanged(const QPointF &)),
             m_d->statusBar, SLOT(documentMousePositionChanged(const QPointF &)));
 
     connect(m_d->nodeManager, SIGNAL(sigNodeActivated(KisNodeSP)),
@@ -255,6 +322,7 @@ KisView2::KisView2(KisDoc2 * doc, QWidget * parent)
 
     // canvas sends signal that origin is changed
     connect(m_d->canvas, SIGNAL(documentOriginChanged()), m_d->zoomManager, SLOT(pageOffsetChanged()));
+    connect(m_d->canvas, SIGNAL(scrollAreaSizeChanged()), m_d->zoomManager, SLOT(slotScrollAreaSizeChanged()));
 
     setAcceptDrops(true);
 }
@@ -271,9 +339,7 @@ void KisView2::dragEnterEvent(QDragEnterEvent *event)
     dbgUI << "KisView2::dragEnterEvent";
     // Only accept drag if we're not busy, particularly as we may
     // be showing a progress bar and calling qApp->processEvents().
-    if (K3URLDrag::canDecode(event) && QApplication::overrideCursor() == 0) {
-        event->accept();
-    } else if (event->mimeData()->hasImage()) {
+    if (event->mimeData()->hasImage()) {
         event->accept();
     } else {
         event->ignore();
@@ -283,22 +349,22 @@ void KisView2::dragEnterEvent(QDragEnterEvent *event)
 void KisView2::dropEvent(QDropEvent *event)
 {
     if (event->mimeData()->hasImage()) {
-        QImage qimg = qvariant_cast<QImage>(event->mimeData()->imageData());
-        KisImageWSP img = image();
+        QImage qimage = qvariant_cast<QImage>(event->mimeData()->imageData());
+        KisImageWSP kisimage = image();
 
-        if (img) {
+        if (kisimage) {
             KisPaintDeviceSP device = new KisPaintDevice(KoColorSpaceRegistry::instance()->rgb8());
-            device->convertFromQImage(qimg, "");
-            KisLayerSP layer = new KisPaintLayer(img.data(), img->nextLayerName(), OPACITY_OPAQUE, device);
+            device->convertFromQImage(qimage, "");
+            KisLayerSP layer = new KisPaintLayer(kisimage.data(), kisimage->nextLayerName(), OPACITY_OPAQUE_U8, device);
 
-            QPointF pos = img->documentToIntPixel(m_d->viewConverter->viewToDocument(event->pos() + m_d->canvas->documentOffset() - m_d->canvas->documentOrigin()));
+            QPointF pos = kisimage->documentToIntPixel(m_d->viewConverter->viewToDocument(event->pos() + m_d->canvas->documentOffset() - m_d->canvas->documentOrigin()));
             layer->setX(pos.x());
             layer->setY(pos.y());
 
             if (layer) {
                 KisNodeCommandsAdapter adapter(this);
                 if (!m_d->nodeManager->layerManager()->activeLayer()) {
-                    adapter.addNode(layer.data(), img->rootLayer().data() , 0);
+                    adapter.addNode(layer.data(), kisimage->rootLayer().data() , 0);
                 } else {
                     adapter.addNode(layer.data(), m_d->nodeManager->layerManager()->activeLayer()->parent().data(), m_d->nodeManager->layerManager()->activeLayer().data());
                 }
@@ -310,55 +376,53 @@ void KisView2::dropEvent(QDropEvent *event)
         return;
     }
 
-    KUrl::List urls;
-    if (K3URLDrag::decode(event, urls)) {
-        if (urls.count() > 0) {
+    QList<QUrl> urls = event->mimeData()->urls();
+    if (urls.length() > 0) {
 
-            KMenu popup;
-            popup.setObjectName("drop_popup");
+        KMenu popup;
+        popup.setObjectName("drop_popup");
 
-            QAction *insertAsNewLayer = new QAction(i18n("Insert as New Layer"), &popup);
-            QAction *insertAsNewLayers = new QAction(i18n("Insert as New Layers"), &popup);
+        QAction *insertAsNewLayer = new QAction(i18n("Insert as New Layer"), &popup);
+        QAction *insertAsNewLayers = new QAction(i18n("Insert as New Layers"), &popup);
 
-            QAction *openInNewDocument = new QAction(i18n("Open in New Document"), &popup);
-            QAction *openInNewDocuments = new QAction(i18n("Open in New Documents"), &popup);
+        QAction *openInNewDocument = new QAction(i18n("Open in New Document"), &popup);
+        QAction *openInNewDocuments = new QAction(i18n("Open in New Documents"), &popup);
 
-            QAction *cancel = new QAction(i18n("Cancel"), &popup);
+        QAction *cancel = new QAction(i18n("Cancel"), &popup);
 
-            if (urls.count() == 1) {
-                if (!image().isNull()) {
-                    popup.addAction(insertAsNewLayer);
-                }
-                popup.addAction(openInNewDocument);
-            } else {
-                if (!image().isNull()) {
-                    popup.addAction(insertAsNewLayers);
-                }
-                popup.addAction(openInNewDocuments);
+        if (urls.count() == 1) {
+            if (!image().isNull()) {
+                popup.addAction(insertAsNewLayer);
             }
+            popup.addAction(openInNewDocument);
+        } else {
+            if (!image().isNull()) {
+                popup.addAction(insertAsNewLayers);
+            }
+            popup.addAction(openInNewDocuments);
+        }
 
-            popup.addSeparator();
-            popup.addAction(cancel);
+        popup.addSeparator();
+        popup.addAction(cancel);
 
-            QAction *action = popup.exec(QCursor::pos());
+        QAction *action = popup.exec(QCursor::pos());
 
-            if (action != 0 && action != cancel) {
-                for (KUrl::List::ConstIterator it = urls.constBegin(); it != urls.constEnd(); ++it) {
-                    KUrl url = *it;
+        if (action != 0 && action != cancel) {
+            foreach(QUrl url, urls) {
 
-                    if (action == insertAsNewLayer || action == insertAsNewLayers) {
-                        m_d->imageManager->importImage(url);
-                    } else {
-                        Q_ASSERT(action == openInNewDocument || action == openInNewDocuments);
+                if (action == insertAsNewLayer || action == insertAsNewLayers) {
+                    m_d->imageManager->importImage(KUrl(url));
+                } else {
+                    Q_ASSERT(action == openInNewDocument || action == openInNewDocuments);
 
-                        if (shell() != 0) {
-                            shell()->openDocument(url);
-                        }
+                    if (shell() != 0) {
+                        shell()->openDocument(url);
                     }
                 }
             }
         }
     }
+
 }
 
 KoZoomController *KisView2::zoomController() const
@@ -482,45 +546,49 @@ KisUndoAdapter * KisView2::undoAdapter()
 
 void KisView2::slotLoadingFinished()
 {
-    KisImageWSP img = image();
-
-    img->refreshGraph();
-
+    /**
+     * Cold-start of image-size signals
+     */
     slotImageSizeChanged();
-
     if (m_d->statusBar) {
-        m_d->statusBar->imageSizeChanged(img->width(), img->height());
+        m_d->statusBar->imageSizeChanged(image()->width(), image()->height());
+    }
+    if (m_d->resourceProvider) {
+        m_d->resourceProvider->slotImageSizeChanged();
+    }
+    if (m_d->nodeManager) {
+        m_d->nodeManager->nodesUpdated();
     }
-    m_d->resourceProvider->slotImageSizeChanged();
-
-    m_d->nodeManager->nodesUpdated();
 
     connectCurrentImage();
 
-    if (img->locked()) {
+    if (image()->locked()) {
         // If this is the first view on the image, the image will have been locked
         // so unlock it.
-        img->blockSignals(false);
-        img->unlock();
+        image()->blockSignals(false);
+        image()->unlock();
     }
 
-    if (KisNodeSP node = img->rootLayer()->firstChild()) {
+    if (KisNodeSP node = image()->rootLayer()->firstChild()) {
         m_d->nodeManager->activateNode(node);
     }
 
     /**
      * Dirty hack alert
      */
-    m_d->viewConverter->setZoomMode(KoZoomMode::ZOOM_PAGE);
-    m_d->zoomManager->zoomController()->setAspectMode(true);
+    if (m_d->viewConverter)
+        m_d->viewConverter->setZoomMode(KoZoomMode::ZOOM_PAGE);
+    if (m_d->zoomManager && m_d->zoomManager->zoomController())
+        m_d->zoomManager->zoomController()->setAspectMode(true);
 
     updateGUI();
+
+    emit sigLoadingFinished();
 }
 
 
 void KisView2::createActions()
 {
-    actionCollection()->addAction(KStandardAction::FullScreen, "full_screen", this, SLOT(slotUpdateFullScreen(bool)));
     actionCollection()->addAction(KStandardAction::Preferences,  "preferences", this, SLOT(slotPreferences()));
 
     KAction* action = new KAction(i18n("Edit Palette..."), this);
@@ -584,18 +652,17 @@ void KisView2::updateGUI()
 
 void KisView2::connectCurrentImage()
 {
-    KisImageWSP img = image();
-    if (img) {
+    if (image()) {
         if (m_d->statusBar) {
-            connect(img.data(), SIGNAL(sigColorSpaceChanged(const KoColorSpace *)), m_d->statusBar, SLOT(updateStatusBarProfileLabel()));
-            connect(img.data(), SIGNAL(sigProfileChanged(KoColorProfile *)), m_d->statusBar, SLOT(updateStatusBarProfileLabel()));
-            connect(img.data(), SIGNAL(sigSizeChanged(qint32, qint32)), m_d->statusBar, SLOT(imageSizeChanged(qint32, qint32)));
+            connect(image(), SIGNAL(sigColorSpaceChanged(const KoColorSpace *)), m_d->statusBar, SLOT(updateStatusBarProfileLabel()));
+            connect(image(), SIGNAL(sigProfileChanged(const KoColorProfile *)), m_d->statusBar, SLOT(updateStatusBarProfileLabel()));
+            connect(image(), SIGNAL(sigSizeChanged(qint32, qint32)), m_d->statusBar, SLOT(imageSizeChanged(qint32, qint32)));
 
         }
-        connect(img.data(), SIGNAL(sigSizeChanged(qint32, qint32)), m_d->resourceProvider, SLOT(slotImageSizeChanged()));
-        connect(img.data(), SIGNAL(sigSizeChanged(qint32, qint32)), this, SLOT(slotImageSizeChanged()));
-        connect(img.data(), SIGNAL(sigResolutionChanged(double, double)), this, SLOT(slotImageSizeChanged()));
-        connect(img->undoAdapter(), SIGNAL(selectionChanged()), selectionManager(), SLOT(selectionChanged()));
+        connect(image(), SIGNAL(sigSizeChanged(qint32, qint32)), m_d->resourceProvider, SLOT(slotImageSizeChanged()));
+        connect(image(), SIGNAL(sigSizeChanged(qint32, qint32)), this, SLOT(slotImageSizeChanged()));
+        connect(image(), SIGNAL(sigResolutionChanged(double, double)), this, SLOT(slotImageSizeChanged()));
+        connect(image()->undoAdapter(), SIGNAL(selectionChanged()), selectionManager(), SLOT(selectionChanged()));
 
     }
 
@@ -605,48 +672,32 @@ void KisView2::connectCurrentImage()
             m_d->nodeManager, SLOT(activateNode(KisNodeSP)));
 
     if (m_d->controlFrame) {
-        connect(img.data(), SIGNAL(sigColorSpaceChanged(const KoColorSpace *)), m_d->controlFrame->paintopBox(), SLOT(colorSpaceChanged(const KoColorSpace*)));
+        connect(image(), SIGNAL(sigColorSpaceChanged(const KoColorSpace *)), m_d->controlFrame->paintopBox(), SLOT(colorSpaceChanged(const KoColorSpace*)));
     }
 
 }
 
 void KisView2::disconnectCurrentImage()
 {
-    KisImageWSP img = image();
+    if (image()) {
 
-    if (img) {
-
-        img->disconnect(this);
-        img->disconnect(m_d->nodeManager);
-        img->disconnect(m_d->selectionManager);
+        image()->disconnect(this);
+        image()->disconnect(m_d->nodeManager);
+        image()->disconnect(m_d->selectionManager);
         if (m_d->statusBar)
-            img->disconnect(m_d->statusBar);
+            image()->disconnect(m_d->statusBar);
 
         m_d->canvas->disconnectCurrentImage();
     }
 }
 
-void KisView2::slotUpdateFullScreen(bool toggle)
-{
-    if (KoView::shell()) {
-
-        Qt::WindowStates newState = KoView::shell()->windowState();
-
-        if (toggle) {
-            newState |= Qt::WindowFullScreen;
-        } else {
-            newState &= ~Qt::WindowFullScreen;
-        }
-
-        KoView::shell()->setWindowState(newState);
-    }
-}
-
 void KisView2::slotPreferences()
 {
     if (KisDlgPreferences::editPreferences()) {
         KisConfigNotifier::instance()->notifyConfigChanged();
         m_d->resourceProvider->resetDisplayProfile();
+        KisConfig cfg;
+        static_cast<KoCanvasControllerWidget*>(m_d->canvasController)->setZoomWithWheel(cfg.zoomWithWheel());
 
         // Update the settings for all nodes -- they don't query
         // KisConfig directly because they need the settings during
@@ -677,11 +728,14 @@ void KisView2::slotImageSizeChanged()
     m_d->zoomManager->zoomController()->setPageSize(size);
     m_d->zoomManager->zoomController()->setDocumentSize(size);
 
-    QSize documentSize(int(ceil(m_d->viewConverter->documentToViewX(image()->width()  / image()->xRes()))),
-                       int(ceil(m_d->viewConverter->documentToViewY(image()->height() / image()->yRes()))));
-    m_d->canvasController->setDocumentSize(documentSize);
+    canvasBase()->notifyZoomChanged();
+    QSize widgetSize = canvasBase()->coordinatesConverter()->imageRectInWidgetPixels().toAlignedRect().size();
+    m_d->canvasController->updateDocumentSize(widgetSize, true);
 
     m_d->zoomManager->updateGUI();
+
+    //update view
+    canvas()->update();
 }
 
 
@@ -689,21 +743,20 @@ void KisView2::loadPlugins()
 {
     // Load all plugins
     KService::List offers = KServiceTypeTrader::self()->query(QString::fromLatin1("Krita/ViewPlugin"),
-                            QString::fromLatin1("(Type == 'Service') and "
-                                                "([X-Krita-Version] == 3)"));
+                                                              QString::fromLatin1("(Type == 'Service') and "
+                                                                                  "([X-Krita-Version] == 4)"));
     KService::List::ConstIterator iter;
     for (iter = offers.constBegin(); iter != offers.constEnd(); ++iter) {
-
         KService::Ptr service = *iter;
-        int errCode = 0;
+        dbgUI << "Load plugin " << service->name();
+        QString error;
+
         KParts::Plugin* plugin =
-            KService::createInstance<KParts::Plugin> (service, this, QStringList(), &errCode);
+                dynamic_cast<KParts::Plugin*>(service->createInstance<QObject>(this, QVariantList(), &error));
         if (plugin) {
             insertChildClient(plugin);
         } else {
-            if (errCode == KLibLoader::ErrNoLibrary) {
-                warnKrita << " Error loading plugin was : ErrNoLibrary" << KLibLoader::self()->lastErrorMessage();
-            }
+            errKrita << "Fail to create an instance for " << service->name() << " " << error;
         }
     }
 }
@@ -740,7 +793,72 @@ KisPaintingAssistantsManager* KisView2::paintingAssistantManager()
 
 void KisView2::slotTotalRefresh()
 {
-    m_d->canvas->resetCanvas();
+    KisConfig cfg;
+    m_d->canvas->resetCanvas(cfg.useOpenGL());
+}
+
+void KisView2::slotCreateTemplate()
+{
+    int width = 60;
+    int height = 60;
+    QPixmap pix = m_d->doc->generatePreview(QSize(width, height));
+
+    KTemporaryFile tempFile;
+    tempFile.setSuffix(".kra");
+
+    //Check that creation of temp file was successful
+    if (!tempFile.open()) {
+        qWarning("Creation of temporary file to store template failed.");
+        return;
+    }
+
+    m_d->doc->saveNativeFormat(tempFile.fileName());
+
+    KoTemplateCreateDia::createTemplate("krita_template", KisFactory2::componentData(),
+                                        tempFile.fileName(), pix, this);
+
+    KisFactory2::componentData().dirs()->addResourceType("krita_template", "data", "krita/templates/");
+
+}
+
+void KisView2::disableControls()
+{
+    // prevents possible crashes, if somebody changes the paintop during dragging by using the mousewheel
+    // this is for Bug 250944
+    // the solution blocks all wheel, mouse and key event, while dragging with the freehand tool
+    // see KisToolFreehand::initPaint() and endPaint()
+    m_d->controlFrame->paintopBox()->installEventFilter(&m_d->blockingEventFilter);
+    foreach(QObject* child, m_d->controlFrame->paintopBox()->children()) {
+        child->installEventFilter(&m_d->blockingEventFilter);
+    }
+}
+
+void KisView2::enableControls()
+{
+    m_d->controlFrame->paintopBox()->removeEventFilter(&m_d->blockingEventFilter);
+    foreach(QObject* child, m_d->controlFrame->paintopBox()->children()) {
+        child->removeEventFilter(&m_d->blockingEventFilter);
+    }
+}
+
+void KisView2::slotFirstRun()
+{
+    QString fname = KisFactory2::componentData().dirs()->findResource("kis_images", "krita_first_start.kra");
+    if (!fname.isEmpty()) {
+        KoDocumentEntry entry = KoDocumentEntry(KoDocument::readNativeService());
+        QString errorMsg;
+        KoDocument* doc = entry.createDoc(&errorMsg);
+        if (!doc) return;
+        KoMainWindow *shell = new KoMainWindow(doc->componentData());
+        shell->show();
+        QObject::connect(doc, SIGNAL(sigProgress(int)), shell, SLOT(slotProgress(int)));
+        // for initDoc to fill in the recent docs list
+        // and for KoDocument::slotStarted
+        doc->addShell(shell);
+        doc->showStartUpWidget(shell, true);
+        doc->openUrl(fname);
+    }
+
 }
 
 #include "kis_view2.moc"
