@@ -60,6 +60,7 @@ public:
 private slots:
 
     void slotZoomChanged(KoZoomMode::Mode mode, qreal zoom);
+    void slotScrollAreaSizeChanged();
     void toggleShowRulers(bool show);
     void mousePositionChanged(const QPoint &pos);
     void changeAspectMode(bool aspectMode);
@@ -81,6 +82,7 @@ private:
     KAction * m_actualPixels;
     KAction * m_actualSize;
     KAction * m_fitToCanvas;
+    QWidget * m_zoomActionWidget;
 };
 
 #endif
