@@ -65,8 +65,18 @@ KisMetaData::Value exivValueToKMDValue(const Exiv2::Value::AutoPtr value, bool f
     case Exiv2::comment: // look at kexiv2 for the problem about decoding correctly that tag
         return KisMetaData::Value(value->toString().c_str());
     case Exiv2::unsignedRational:
+        if(value->size() < 2)
+        {
+          dbgFile << "Invalid size :" << value->size() << " value =" << value->toString().c_str();
+          return KisMetaData::Value();
+        }
         return KisMetaData::Value(KisMetaData::Rational(value->toRational().first , value->toRational().second));
     case Exiv2::signedRational:
+        if(value->size() < 2)
+        {
+          dbgFile << "Invalid size :" << value->size() << " value =" << value->toString().c_str();
+          return KisMetaData::Value();
+        }
         return KisMetaData::Value(KisMetaData::Rational(value->toRational().first , value->toRational().second));
     case Exiv2::date:
     case Exiv2::time:
@@ -218,7 +228,20 @@ Exiv2::Value* kmdValueToExivXmpValue(const KisMetaData::Value& value)
     case KisMetaData::Value::AlternativeArray:
     case KisMetaData::Value::OrderedArray:
     case KisMetaData::Value::UnorderedArray: {
-        Exiv2::Value* arrV = new Exiv2::XmpArrayValue;
+        Exiv2::XmpArrayValue* arrV = new Exiv2::XmpArrayValue;
+        switch (value.type()) {
+        case KisMetaData::Value::OrderedArray:
+            arrV->setXmpArrayType(Exiv2::XmpValue::xaSeq);
+            break;
+        case KisMetaData::Value::UnorderedArray:
+            arrV->setXmpArrayType(Exiv2::XmpValue::xaBag);
+            break;
+        case KisMetaData::Value::AlternativeArray:
+            arrV->setXmpArrayType(Exiv2::XmpValue::xaAlt);
+            break;
+        default:
+            qFatal("can't happen");
+        }
         foreach(const KisMetaData::Value& v, value.asArray()) {
             Exiv2::Value* ev = kmdValueToExivXmpValue(v);
             arrV->read(ev->toString());
