@@ -24,7 +24,7 @@
 
 #include <KoStore.h>
 #include <KoColorSpaceRegistry.h>
-#include <KoIccColorProfile.h>
+#include <KoColorProfile.h>
 #include <KoDocumentInfo.h>
 
 #include "kis_doc2.h"
@@ -46,7 +46,6 @@
 #include <kis_image.h>
 #include <kis_layer.h>
 #include <kis_name_server.h>
-#include <kis_paint_device_action.h>
 #include <kis_paint_layer.h>
 #include <kis_selection.h>
 #include <kis_selection_mask.h>
@@ -90,7 +89,7 @@ KisImageWSP KisKraLoader::loadXML(const KoXmlElement& element)
     QString attr;
     KoXmlNode node;
     KoXmlNode child;
-    KisImageWSP img = 0;
+    KisImageWSP image = 0;
     QString name;
     qint32 width;
     qint32 height;
@@ -141,11 +140,14 @@ KisImageWSP KisKraLoader::loadXML(const KoXmlElement& element)
         if (colorspacename  == "Grayscale + Alpha")
             colorspacename  = "GRAYA";
 
+        QString colorspaceModel = KoColorSpaceRegistry::instance()->colorSpaceColorModelId(colorspacename).id();
+        QString colorspaceDepth = KoColorSpaceRegistry::instance()->colorSpaceColorDepthId(colorspacename).id();
+
         if ((profileProductName = element.attribute(PROFILE)).isNull()) {
             // no mention of profile so get default profile
-            cs = KoColorSpaceRegistry::instance()->colorSpace(colorspacename, "");
+            cs = KoColorSpaceRegistry::instance()->colorSpace(colorspaceModel, colorspaceDepth, "");
         } else {
-            cs = KoColorSpaceRegistry::instance()->colorSpace(colorspacename, profileProductName);
+            cs = KoColorSpaceRegistry::instance()->colorSpace(colorspaceModel, colorspaceDepth, profileProductName);
         }
 
         if (cs == 0) {
@@ -153,36 +155,30 @@ KisImageWSP KisKraLoader::loadXML(const KoXmlElement& element)
             return KisImageWSP(0);
         }
 
-        img = new KisImage(m_d->document->undoAdapter(), width, height, cs, name);
+        image = new KisImage(m_d->document->undoAdapter(), width, height, cs, name);
+        image->lock();
+        image->setResolution(xres, yres);
 
-#ifdef __GNUC__
-#warning "KisKraLoader::loadXML: check whether image->lock still works with the top-down update "
-#endif
-        img->lock();
+        loadNodes(element, image, const_cast<KisGroupLayer*>(image->rootLayer().data()));
 
-        img->setResolution(xres, yres);
-
-
-        loadNodes(element, img, const_cast<KisGroupLayer*>(img->rootLayer().data()));
-
-        img->unlock();
+        image->unlock();
 
     }
 
-    return img;
+    return image;
 }
 
-void KisKraLoader::loadBinaryData(KoStore * store, KisImageWSP img, const QString & uri, bool external)
+void KisKraLoader::loadBinaryData(KoStore * store, KisImageWSP image, const QString & uri, bool external)
 {
     // Load the layers data
-    KisKraLoadVisitor visitor(img, store, m_d->layerFilenames, m_d->imageName, m_d->syntaxVersion);
+    KisKraLoadVisitor visitor(image, store, m_d->layerFilenames, m_d->imageName, m_d->syntaxVersion);
 
     if (external)
         visitor.setExternalUri(uri);
 
-    img->lock();
+    image->lock();
 
-    img->rootLayer()->accept(visitor);
+    image->rootLayer()->accept(visitor);
 
 
     // annotations
@@ -194,7 +190,7 @@ void KisKraLoader::loadBinaryData(KoStore * store, KisImageWSP img, const QStrin
         store->open(location);
         data = store->read(store->size());
         store->close();
-        img->addAnnotation(KisAnnotationSP(new KisAnnotation("exif", "", data)));
+        image->addAnnotation(KisAnnotationSP(new KisAnnotation("exif", "", data)));
     }
 
     // icc profile
@@ -205,7 +201,7 @@ void KisKraLoader::loadBinaryData(KoStore * store, KisImageWSP img, const QStrin
         QByteArray data; data.resize(store->size());
         store->read(data.data(), store->size());
         store->close();
-        img->setProfile(new KoIccColorProfile(data));
+        image->assignImageProfile(KoColorSpaceRegistry::instance()->createColorProfile(image->colorSpace()->colorModelId().id(), image->colorSpace()->colorDepthId().id(), data));
     }
 
 
@@ -214,10 +210,10 @@ void KisKraLoader::loadBinaryData(KoStore * store, KisImageWSP img, const QStrin
     if (m_d->document->documentInfo()->aboutInfo("comment").isNull())
         m_d->document->documentInfo()->setAboutInfo("comment", m_d->imageComment);
 
-    img->unlock();
+    image->unlock();
 }
 
-KisNode* KisKraLoader::loadNodes(const KoXmlElement& element, KisImageWSP img, KisNode* parent)
+KisNode* KisKraLoader::loadNodes(const KoXmlElement& element, KisImageWSP image, KisNode* parent)
 {
 
     KoXmlNode node = element.lastChild();
@@ -231,17 +227,16 @@ KisNode* KisKraLoader::loadNodes(const KoXmlElement& element, KisImageWSP img, K
 
             if (node.nodeName().toUpper() == LAYERS.toUpper() || node.nodeName().toUpper() == MASKS.toUpper()) {
                 for (child = node.lastChild(); !child.isNull(); child = child.previousSibling()) {
-                    KisNode* node = loadNode(child.toElement(), img);
+                    KisNode* node = loadNode(child.toElement(), image);
                     if (!node) {
-                        qDebug() << ">>>>>>>>>>>>>>>>>>> Could not load node";
 #ifdef __GNUC__
 #warning "KisKraLoader::loadNodes: report node load failures back to the user!"
 #endif
                     } else {
-                        img->nextLayerName(); // Make sure the nameserver is current with the number of nodes.
-                        img->addNode(node, parent);
+                        image->nextLayerName(); // Make sure the nameserver is current with the number of nodes.
+                        image->addNode(node, parent);
                         if (node->inherits("KisLayer") && child.childNodesCount() > 0) {
-                            loadNodes(child.toElement(), img, node);
+                            loadNodes(child.toElement(), image, node);
                         }
 
                     }
@@ -253,7 +248,7 @@ KisNode* KisKraLoader::loadNodes(const KoXmlElement& element, KisImageWSP img, K
     return parent;
 }
 
-KisNode* KisKraLoader::loadNode(const KoXmlElement& element, KisImageWSP img)
+KisNode* KisKraLoader::loadNode(const KoXmlElement& element, KisImageWSP image)
 {
     // Nota bene: If you add new properties to layers, you should
     // ALWAYS define a default value in case the property is not
@@ -265,16 +260,23 @@ KisNode* KisKraLoader::loadNode(const KoXmlElement& element, KisImageWSP img)
     qint32 x = element.attribute(X, "0").toInt();
     qint32 y = element.attribute(Y, "0").toInt();
 
-    qint32 opacity = element.attribute(OPACITY, QString::number(OPACITY_OPAQUE)).toInt();
-    if (opacity < OPACITY_TRANSPARENT) opacity = OPACITY_TRANSPARENT;
-    if (opacity > OPACITY_OPAQUE) opacity = OPACITY_OPAQUE;
+    qint32 opacity = element.attribute(OPACITY, QString::number(OPACITY_OPAQUE_U8)).toInt();
+    if (opacity < OPACITY_TRANSPARENT_U8) opacity = OPACITY_TRANSPARENT_U8;
+    if (opacity > OPACITY_OPAQUE_U8) opacity = OPACITY_OPAQUE_U8;
 
     const KoColorSpace* colorSpace = 0;
-    if ((element.attribute(COLORSPACE_NAME)).isNull())
-        colorSpace = img->colorSpace();
-    else
+    if ((element.attribute(COLORSPACE_NAME)).isNull()) {
+        dbgFile << "No attribute color space for layer: " << name;
+        colorSpace = image->colorSpace();
+    } else {
+        QString colorspacename = element.attribute(COLORSPACE_NAME);
+        QString colorspaceModel = KoColorSpaceRegistry::instance()->colorSpaceColorModelId(colorspacename).id();
+        QString colorspaceDepth = KoColorSpaceRegistry::instance()->colorSpaceColorDepthId(colorspacename).id();
+
         // use default profile - it will be replaced later in completeLoading
-        colorSpace = KoColorSpaceRegistry::instance()->colorSpace(element.attribute(COLORSPACE_NAME), "");
+        colorSpace = KoColorSpaceRegistry::instance()->colorSpace(colorspaceModel, colorspaceDepth, "");
+        dbgFile << "Using color space: " << colorspacename << colorspaceModel << colorspaceDepth << " " << colorSpace << " for layer: " << name;
+    }
 
     bool visible = element.attribute(VISIBLE, "1") == "0" ? false : true;
     bool locked = element.attribute(LOCKED, "0") == "0" ? false : true;
@@ -298,17 +300,17 @@ KisNode* KisKraLoader::loadNode(const KoXmlElement& element, KisImageWSP img)
     KisNode* node = 0;
 
     if (nodeType == PAINT_LAYER)
-        node = loadPaintLayer(element, img, name, colorSpace, opacity);
+        node = loadPaintLayer(element, image, name, colorSpace, opacity);
     else if (nodeType == GROUP_LAYER)
-        node = loadGroupLayer(element, img, name, colorSpace, opacity);
+        node = loadGroupLayer(element, image, name, colorSpace, opacity);
     else if (nodeType == ADJUSTMENT_LAYER)
-        node = loadAdjustmentLayer(element, img, name, colorSpace, opacity);
+        node = loadAdjustmentLayer(element, image, name, colorSpace, opacity);
     else if (nodeType == SHAPE_LAYER)
-        node = loadShapeLayer(element, img, name, colorSpace, opacity);
+        node = loadShapeLayer(element, image, name, colorSpace, opacity);
     else if (nodeType == GENERATOR_LAYER)
-        node = loadGeneratorLayer(element, img, name, colorSpace, opacity);
+        node = loadGeneratorLayer(element, image, name, colorSpace, opacity);
     else if (nodeType == CLONE_LAYER)
-        node = loadCloneLayer(element, img, name, colorSpace, opacity);
+        node = loadCloneLayer(element, image, name, colorSpace, opacity);
     else if (nodeType == FILTER_MASK)
         node = loadFilterMask(element);
     else if (nodeType == TRANSPARENCY_MASK)
@@ -316,7 +318,7 @@ KisNode* KisKraLoader::loadNode(const KoXmlElement& element, KisImageWSP img)
     else if (nodeType == TRANSFORMATION_MASK)
         node = loadTransformationMask(element);
     else if (nodeType == SELECTION_MASK)
-        node = loadSelectionMask(img, element);
+        node = loadSelectionMask(image, element);
     else
         warnKrita << "Trying to load layer of unsupported type " << nodeType;
 
@@ -355,16 +357,18 @@ KisNode* KisKraLoader::loadNode(const KoXmlElement& element, KisImageWSP img)
 }
 
 
-KisNode* KisKraLoader::loadPaintLayer(const KoXmlElement& element, KisImageWSP img,
+KisNode* KisKraLoader::loadPaintLayer(const KoXmlElement& element, KisImageWSP image,
                                       const QString& name, const KoColorSpace* cs, quint32 opacity)
 {
+    Q_UNUSED(element);
+
     QString attr;
     KisPaintLayer* layer;
 
     QString colorspacename;
     QString profileProductName;
 
-    layer = new KisPaintLayer(img, name, opacity, cs);
+    layer = new KisPaintLayer(image, name, opacity, cs);
     Q_CHECK_PTR(layer);
 
     // Load exif info
@@ -383,24 +387,26 @@ KisNode* KisKraLoader::loadPaintLayer(const KoXmlElement& element, KisImageWSP i
 
 }
 
-KisNode* KisKraLoader::loadGroupLayer(const KoXmlElement& element, KisImageWSP img,
+KisNode* KisKraLoader::loadGroupLayer(const KoXmlElement& element, KisImageWSP image,
                                       const QString& name, const KoColorSpace* cs, quint32 opacity)
 {
+    Q_UNUSED(element);
+    Q_UNUSED(cs);
     QString attr;
     KisGroupLayer* layer;
 
-    layer = new KisGroupLayer(img, name, opacity);
+    layer = new KisGroupLayer(image, name, opacity);
     Q_CHECK_PTR(layer);
 
     return layer;
 
 }
 
-KisNode* KisKraLoader::loadAdjustmentLayer(const KoXmlElement& element, KisImageWSP img,
+KisNode* KisKraLoader::loadAdjustmentLayer(const KoXmlElement& element, KisImageWSP image,
         const QString& name, const KoColorSpace* cs, quint32 opacity)
 {
     // XXX: do something with filterversion?
-
+    Q_UNUSED(cs);
     QString attr;
     KisAdjustmentLayer* layer;
     QString filtername;
@@ -420,7 +426,7 @@ KisNode* KisKraLoader::loadAdjustmentLayer(const KoXmlElement& element, KisImage
     KisFilterConfiguration * kfc = f->defaultConfiguration(0);
 
     // We'll load the configuration and the selection later.
-    layer = new KisAdjustmentLayer(img, name, kfc, 0);
+    layer = new KisAdjustmentLayer(image, name, kfc, 0);
     Q_CHECK_PTR(layer);
 
     layer->setOpacity(opacity);
@@ -430,12 +436,16 @@ KisNode* KisKraLoader::loadAdjustmentLayer(const KoXmlElement& element, KisImage
 }
 
 
-KisNode* KisKraLoader::loadShapeLayer(const KoXmlElement& element, KisImageWSP img,
+KisNode* KisKraLoader::loadShapeLayer(const KoXmlElement& element, KisImageWSP image,
                                       const QString& name, const KoColorSpace* cs, quint32 opacity)
 {
+
+    Q_UNUSED(element);
+    Q_UNUSED(cs);
+
     QString attr;
 
-    KisShapeLayer* layer = new KisShapeLayer(0, m_d->document->shapeController(), img, name, opacity);
+    KisShapeLayer* layer = new KisShapeLayer(0, m_d->document->shapeController(), image, name, opacity);
     Q_CHECK_PTR(layer);
 
     return layer;
@@ -443,9 +453,10 @@ KisNode* KisKraLoader::loadShapeLayer(const KoXmlElement& element, KisImageWSP i
 }
 
 
-KisNode* KisKraLoader::loadGeneratorLayer(const KoXmlElement& element, KisImageWSP img,
+KisNode* KisKraLoader::loadGeneratorLayer(const KoXmlElement& element, KisImageWSP image,
         const QString& name, const KoColorSpace* cs, quint32 opacity)
 {
+    Q_UNUSED(cs);
     // XXX: do something with generator version?
     KisGeneratorLayer* layer;
     QString generatorname = element.attribute(GENERATOR_NAME);
@@ -465,7 +476,7 @@ KisNode* KisKraLoader::loadGeneratorLayer(const KoXmlElement& element, KisImageW
     KisFilterConfiguration * kgc = generator->defaultConfiguration(0);
 
     // We'll load the configuration and the selection later.
-    layer = new KisGeneratorLayer(img, name, kgc, 0);
+    layer = new KisGeneratorLayer(image, name, kgc, 0);
     Q_CHECK_PTR(layer);
 
     layer->setOpacity(opacity);
@@ -474,10 +485,12 @@ KisNode* KisKraLoader::loadGeneratorLayer(const KoXmlElement& element, KisImageW
 
 }
 
-KisNode* KisKraLoader::loadCloneLayer(const KoXmlElement& element, KisImageWSP img,
+KisNode* KisKraLoader::loadCloneLayer(const KoXmlElement& element, KisImageWSP image,
                                       const QString& name, const KoColorSpace* cs, quint32 opacity)
 {
-    KisCloneLayer* layer = new KisCloneLayer(0, img, name, opacity);
+    Q_UNUSED(cs);
+
+    KisCloneLayer* layer = new KisCloneLayer(0, image, name, opacity);
 
     if ((element.attribute(CLONE_FROM)).isNull()) {
         return 0;
@@ -527,6 +540,7 @@ KisNode* KisKraLoader::loadFilterMask(const KoXmlElement& element)
 
 KisNode* KisKraLoader::loadTransparencyMask(const KoXmlElement& element)
 {
+    Q_UNUSED(element);
     KisTransparencyMask* mask = new KisTransparencyMask();
     Q_CHECK_PTR(mask);
 
@@ -557,9 +571,10 @@ KisNode* KisKraLoader::loadTransformationMask(const KoXmlElement& element)
     return mask;
 }
 
-KisNode* KisKraLoader::loadSelectionMask(KisImageWSP img, const KoXmlElement& element)
+KisNode* KisKraLoader::loadSelectionMask(KisImageWSP image, const KoXmlElement& element)
 {
-    KisSelectionMask* mask = new KisSelectionMask(img);
+    Q_UNUSED(element);
+    KisSelectionMask* mask = new KisSelectionMask(image);
     Q_CHECK_PTR(mask);
 
     return mask;
