@@ -37,12 +37,12 @@ public:
     KisSelectionToolHelper(KisCanvas2* canvas, KisNodeSP node, const QString& name);
     virtual ~KisSelectionToolHelper();
 
-    QUndoCommand* selectPixelSelection(KisPixelSelectionSP selection, selectionAction action);
+    void selectPixelSelection(KisPixelSelectionSP selection, selectionAction action);
     void addSelectionShape(KoShape* shape);
 
 private:
     KisCanvas2* m_canvas;
-    KisImageSP m_image;
+    KisImageWSP m_image;
     KisLayerSP m_layer;
     QString m_name;
 };
