@@ -33,7 +33,7 @@ FreehandPaintJob::FreehandPaintJob(KisToolFreehand* toolFreeHand,
                                    const FreehandPaintJob* previousPaintJob) :
         m_toolFreeHand(toolFreeHand),
         m_painter(painter),
-        m_dragDist(-1.0),
+        m_dragDist(-1.0, -1.0),
         m_pi1(pi1),
         m_pi2(pi2),
         m_previousPaintJob(previousPaintJob)
@@ -60,9 +60,9 @@ FreehandPaintAtJob::~FreehandPaintAtJob()
 
 void FreehandPaintAtJob::run()
 {
-    m_dragDist = 0.0;
+    m_dragDist = KisDistanceInformation(0,0);
     m_painter->paintAt(KisPaintInformation(m_pi1));
-    m_toolFreeHand->setDirty(m_painter->dirtyRegion());
+    m_toolFreeHand->setDirty(m_painter->takeDirtyRegion());
 }
 
 
@@ -82,9 +82,9 @@ FreehandPaintLineJob::~FreehandPaintLineJob()
 
 void FreehandPaintLineJob::run()
 {
-    m_dragDist = (m_previousPaintJob) ? m_dragDist = m_previousPaintJob->dragDist() : 0.0;
+    m_dragDist = (m_previousPaintJob) ? m_previousPaintJob->dragDist() : KisDistanceInformation(0.0, 0.0);
     m_dragDist = m_painter->paintLine(m_pi1, m_pi2, m_dragDist);
-    m_toolFreeHand->setDirty(m_painter->dirtyRegion());
+    m_toolFreeHand->setDirty(m_painter->takeDirtyRegion());
 }
 
 FreehandPaintBezierJob::FreehandPaintBezierJob(KisToolFreehand* toolFreeHand,
@@ -106,8 +106,8 @@ FreehandPaintBezierJob::~FreehandPaintBezierJob()
 
 void FreehandPaintBezierJob::run()
 {
-    m_dragDist = (m_previousPaintJob) ? m_dragDist = m_previousPaintJob->dragDist() : 0.0;
+    m_dragDist = (m_previousPaintJob) ? m_previousPaintJob->dragDist() : KisDistanceInformation(0.0, 0.0);
     m_dragDist = m_painter->paintBezierCurve(m_pi1, m_control1, m_control2, m_pi2, m_dragDist);
-    m_toolFreeHand->setDirty(m_painter->dirtyRegion());
+    m_toolFreeHand->setDirty(m_painter->takeDirtyRegion());
 }
 
