@@ -80,10 +80,8 @@ KisFilterSelectorWidget::KisFilterSelectorWidget(QWidget* parent) : d(new Privat
 
 KisFilterSelectorWidget::~KisFilterSelectorWidget()
 {
-    KisConfig cfg;
-    QList<int> sizes = d->uiFilterSelector.splitter->sizes();
-    cfg.setShowFilterGallery(sizes[0] > 0);
     delete d->filtersModel;
+    delete d->currentBookmarkedFilterConfigurationsModel;
     delete d->currentCentralWidget;
     delete d->widgetLayout;
     delete d;
@@ -96,7 +94,7 @@ void KisFilterSelectorWidget::setPaintDevice(KisPaintDeviceSP _paintDevice)
     d->paintDevice = _paintDevice;
     d->thumb = d->paintDevice->createThumbnailDevice(100, 100);
     d->filtersModel = new KisFiltersModel(d->thumb);
-    d->uiFilterSelector.filtersSelector->setModel(d->filtersModel);
+    d->uiFilterSelector.filtersSelector->setFilterModel(d->filtersModel);
     d->uiFilterSelector.filtersSelector->header()->setVisible(false);
 }
 
@@ -105,17 +103,23 @@ void KisFilterSelectorWidget::setImage(KisImageWSP _image)
     d->image = _image;
 }
 
-void KisFilterSelectorWidget::showSelector(bool visible)
+void KisFilterSelectorWidget::showFilterGallery(bool visible)
 {
     QList<int> sizes;
+    int currentCentralWidgetWidth = d->currentCentralWidget ? d->currentCentralWidget->width() : 0;
     if (visible) {
-        sizes << 200;
+        sizes << d->uiFilterSelector.filtersSelector->sizeHint().width() << currentCentralWidgetWidth;
     } else {
-        sizes << 0;
+        sizes << 0 << currentCentralWidgetWidth;
     }
     d->uiFilterSelector.splitter->setSizes(sizes);
 }
 
+bool KisFilterSelectorWidget::isFilterGalleryVisible() const
+{
+    return d->uiFilterSelector.splitter->sizes()[0] > 0;
+}
+
 void KisFilterSelectorWidget::setFilter(KisFilterSP f)
 {
     Q_ASSERT(f);
@@ -141,8 +145,10 @@ void KisFilterSelectorWidget::setFilter(KisFilterSP f)
     } else {
         d->currentFilterConfigurationWidget = widget;
         d->currentCentralWidget = widget;
+        d->currentFilterConfigurationWidget->blockSignals(true);
         d->currentFilterConfigurationWidget->setConfiguration(
             d->currentFilter->defaultConfiguration(d->paintDevice));
+        d->currentFilterConfigurationWidget->blockSignals(false);
         connect(d->currentFilterConfigurationWidget, SIGNAL(sigConfigurationUpdated()), this, SIGNAL(configurationChanged()));
     }
 
@@ -154,18 +160,7 @@ void KisFilterSelectorWidget::setFilter(KisFilterSP f)
     // Add the widget to the layout
     d->currentCentralWidget->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
     d->widgetLayout->addWidget(d->currentCentralWidget, 0 , 0);
-
-    KisConfig cfg;
-    QList<int> sizes;
-    if (cfg.showFilterGallery()) {
-
-        sizes << d->uiFilterSelector.filtersSelector->sizeHint().width()
-        << d->currentCentralWidget->sizeHint().width();
-    } else {
-        sizes << 0 << d->currentCentralWidget->sizeHint().width();
-    }
-    d->uiFilterSelector.splitter->setSizes(sizes);
-
+    showFilterGallery(isFilterGalleryVisible());
 }
 
 void KisFilterSelectorWidget::setFilterIndex(const QModelIndex& idx)
