@@ -58,11 +58,14 @@ class Filter::And : public AbstractCondition
 public:
     And() {}
     And(const And& other);
-    virtual ~And() { qDeleteAll(list); }
-    virtual Type type() const { return AbstractCondition::And; }
+    virtual ~And() {
+        qDeleteAll(list);
+    }
+    virtual Type type() const {
+        return AbstractCondition::And;
+    }
     virtual bool loadOdf(const KoXmlElement& parent);
-    virtual void saveOdf(KoXmlWriter& xmlWriter)
-    {
+    virtual void saveOdf(KoXmlWriter& xmlWriter) {
         if (!list.count())
             return;
         xmlWriter.startElement("table:filter-and");
@@ -70,29 +73,26 @@ public:
             list[i]->saveOdf(xmlWriter);
         xmlWriter.endElement();
     }
-    virtual bool evaluate(const Database& database, int index) const
-    {
-        for (int i = 0; i < list.count(); ++i)
-        {
+    virtual bool evaluate(const Database& database, int index) const {
+        for (int i = 0; i < list.count(); ++i) {
             // lazy evaluation, stop on first false
             if (!list[i]->evaluate(database, index))
                 return false;
         }
         return true;
     }
-    virtual bool isEmpty() const { return list.isEmpty(); }
-    virtual QHash<QString, Filter::Comparison> conditions(int fieldNumber) const
-    {
+    virtual bool isEmpty() const {
+        return list.isEmpty();
+    }
+    virtual QHash<QString, Filter::Comparison> conditions(int fieldNumber) const {
         QHash<QString, Filter::Comparison> result;
         for (int i = 0; i < list.count(); ++i)
             result.unite(list[i]->conditions(fieldNumber));
         return result;
     }
-    virtual void removeConditions(int fieldNumber)
-    {
+    virtual void removeConditions(int fieldNumber) {
         QList<AbstractCondition*> newList;
-        for (int i = 0; i < list.count(); ++i)
-        {
+        for (int i = 0; i < list.count(); ++i) {
             list[i]->removeConditions(fieldNumber);
             if (!list[i]->isEmpty())
                 newList.append(list[i]);
@@ -101,15 +101,12 @@ public:
         }
         list = newList;
     }
-    bool operator!=(const And& other) const
-    {
+    bool operator!=(const And& other) const {
         return list != other.list;
     }
-    virtual QString dump() const
-    {
+    virtual QString dump() const {
         QString result = "\t";
-        for (int i = 0; i < list.count(); ++i)
-        {
+        for (int i = 0; i < list.count(); ++i) {
             if (i)
                 result += "AND\t";
             result += list[i]->dump();
@@ -129,11 +126,14 @@ class Filter::Or : public AbstractCondition
 public:
     Or() {}
     Or(const Or& other);
-    virtual ~Or() { qDeleteAll(list); }
-    virtual Type type() const { return AbstractCondition::Or; }
+    virtual ~Or() {
+        qDeleteAll(list);
+    }
+    virtual Type type() const {
+        return AbstractCondition::Or;
+    }
     virtual bool loadOdf(const KoXmlElement& element);
-    virtual void saveOdf(KoXmlWriter& xmlWriter)
-    {
+    virtual void saveOdf(KoXmlWriter& xmlWriter) {
         if (!list.count())
             return;
         xmlWriter.startElement("table:filter-or");
@@ -141,29 +141,26 @@ public:
             list[i]->saveOdf(xmlWriter);
         xmlWriter.endElement();
     }
-    virtual bool evaluate(const Database& database, int index) const
-    {
-        for (int i = 0; i < list.count(); ++i)
-        {
+    virtual bool evaluate(const Database& database, int index) const {
+        for (int i = 0; i < list.count(); ++i) {
             // lazy evaluation, stop on first true
             if (list[i]->evaluate(database, index))
                 return true;
         }
         return false;
     }
-    virtual bool isEmpty() const { return list.isEmpty(); }
-    virtual QHash<QString, Filter::Comparison> conditions(int fieldNumber) const
-    {
+    virtual bool isEmpty() const {
+        return list.isEmpty();
+    }
+    virtual QHash<QString, Filter::Comparison> conditions(int fieldNumber) const {
         QHash<QString, Filter::Comparison> result;
         for (int i = 0; i < list.count(); ++i)
             result.unite(list[i]->conditions(fieldNumber));
         return result;
     }
-    virtual void removeConditions(int fieldNumber)
-    {
+    virtual void removeConditions(int fieldNumber) {
         QList<AbstractCondition*> newList;
-        for (int i = 0; i < list.count(); ++i)
-        {
+        for (int i = 0; i < list.count(); ++i) {
             list[i]->removeConditions(fieldNumber);
             if (!list[i]->isEmpty())
                 newList.append(list[i]);
@@ -172,15 +169,12 @@ public:
         }
         list = newList;
     }
-    bool operator!=(const Or& other) const
-    {
+    bool operator!=(const Or& other) const {
         return list != other.list;
     }
-    virtual QString dump() const
-    {
+    virtual QString dump() const {
         QString result = "\t";
-        for (int i = 0; i < list.count(); ++i)
-        {
+        for (int i = 0; i < list.count(); ++i) {
             if (i)
                 result += "OR\t";
             result += list[i]->dump();
@@ -199,37 +193,34 @@ class Filter::Condition : public AbstractCondition
 {
 public:
     Condition()
-        : fieldNumber(-1)
-        , operation(Match)
-        , caseSensitivity(Qt::CaseInsensitive)
-        , dataType(Text)
-    {
+            : fieldNumber(-1)
+            , operation(Match)
+            , caseSensitivity(Qt::CaseInsensitive)
+            , dataType(Text) {
     }
     Condition(int _fieldNumber, Comparison _comparison, const QString& _value,
               Qt::CaseSensitivity _caseSensitivity, Mode _mode)
-        : fieldNumber(_fieldNumber)
-        , value(_value)
-        , operation(_comparison)
-        , caseSensitivity(_caseSensitivity)
-        , dataType(_mode)
-    {
+            : fieldNumber(_fieldNumber)
+            , value(_value)
+            , operation(_comparison)
+            , caseSensitivity(_caseSensitivity)
+            , dataType(_mode) {
     }
     Condition(const Condition& other)
-        : AbstractCondition()
-        , fieldNumber(other.fieldNumber)
-        , value(other.value)
-        , operation(other.operation)
-        , caseSensitivity(other.caseSensitivity)
-        , dataType(other.dataType)
-    {
+            : AbstractCondition()
+            , fieldNumber(other.fieldNumber)
+            , value(other.value)
+            , operation(other.operation)
+            , caseSensitivity(other.caseSensitivity)
+            , dataType(other.dataType) {
     }
     virtual ~Condition() {}
 
-    virtual Type type() const { return AbstractCondition::Condition; }
-    virtual bool loadOdf(const KoXmlElement& element)
-    {
-        if (element.hasAttributeNS(KoXmlNS::table, "field-number"))
-        {
+    virtual Type type() const {
+        return AbstractCondition::Condition;
+    }
+    virtual bool loadOdf(const KoXmlElement& element) {
+        if (element.hasAttributeNS(KoXmlNS::table, "field-number")) {
             bool ok = false;
             fieldNumber = element.attributeNS(KoXmlNS::table, "field-number", QString()).toInt(&ok);
             if (!ok || fieldNumber < 0)
@@ -237,8 +228,7 @@ public:
         }
         if (element.hasAttributeNS(KoXmlNS::table, "value"))
             value = element.attributeNS(KoXmlNS::table, "value", QString());
-        if (element.hasAttributeNS(KoXmlNS::table, "operator"))
-        {
+        if (element.hasAttributeNS(KoXmlNS::table, "operator")) {
             const QString string = element.attributeNS(KoXmlNS::table, "operator", QString());
             if (string == "match")
                 operation = Match;
@@ -268,21 +258,18 @@ public:
                 operation = TopPercent;
             else if (string == "bottom percent")
                 operation = BottomPercent;
-            else
-            {
-                kDebug() <<"table:operator: unknown value";
+            else {
+                kDebug() << "table:operator: unknown value";
                 return false;
             }
         }
-        if (element.hasAttributeNS(KoXmlNS::table, "case-sensitive"))
-        {
+        if (element.hasAttributeNS(KoXmlNS::table, "case-sensitive")) {
             if (element.attributeNS(KoXmlNS::table, "case-sensitive", "false") == "true")
                 caseSensitivity = Qt::CaseSensitive;
             else
                 caseSensitivity = Qt::CaseInsensitive;
         }
-        if (element.hasAttributeNS(KoXmlNS::table, "data-type"))
-        {
+        if (element.hasAttributeNS(KoXmlNS::table, "data-type")) {
             if (element.attributeNS(KoXmlNS::table, "data-type", "text") == "number")
                 dataType = Number;
             else
@@ -290,57 +277,55 @@ public:
         }
         return true;
     }
-    virtual void saveOdf(KoXmlWriter& xmlWriter)
-    {
+    virtual void saveOdf(KoXmlWriter& xmlWriter) {
         if (fieldNumber < 0)
             return;
         xmlWriter.startElement("table:filter-condition");
         xmlWriter.addAttribute("table:field-number", fieldNumber);
         xmlWriter.addAttribute("table:value", value);
-        switch (operation)
-        {
-            case Match:
-                xmlWriter.addAttribute("table:operator", "match");
-                break;
-            case NotMatch:
-                xmlWriter.addAttribute("table:operator", "!match");
-                break;
-            case Equal:
-                xmlWriter.addAttribute("table:operator", "=");
-                break;
-            case NotEqual:
-                xmlWriter.addAttribute("table:operator", "!=");
-                break;
-            case Less:
-                xmlWriter.addAttribute("table:operator", "<");
-                break;
-            case Greater:
-                xmlWriter.addAttribute("table:operator", ">");
-                break;
-            case LessOrEqual:
-                xmlWriter.addAttribute("table:operator", "<=");
-                break;
-            case GreaterOrEqual:
-                xmlWriter.addAttribute("table:operator", ">=");
-                break;
-            case Empty:
-                xmlWriter.addAttribute("table:operator", "empty");
-                break;
-            case NotEmpty:
-                xmlWriter.addAttribute("table:operator", "!empty");
-                break;
-            case TopValues:
-                xmlWriter.addAttribute("table:operator", "top values");
-                break;
-            case BottomValues:
-                xmlWriter.addAttribute("table:operator", "bottom values");
-                break;
-            case TopPercent:
-                xmlWriter.addAttribute("table:operator", "top percent");
-                break;
-            case BottomPercent:
-                xmlWriter.addAttribute("table:operator", "bottom percent");
-                break;
+        switch (operation) {
+        case Match:
+            xmlWriter.addAttribute("table:operator", "match");
+            break;
+        case NotMatch:
+            xmlWriter.addAttribute("table:operator", "!match");
+            break;
+        case Equal:
+            xmlWriter.addAttribute("table:operator", "=");
+            break;
+        case NotEqual:
+            xmlWriter.addAttribute("table:operator", "!=");
+            break;
+        case Less:
+            xmlWriter.addAttribute("table:operator", "<");
+            break;
+        case Greater:
+            xmlWriter.addAttribute("table:operator", ">");
+            break;
+        case LessOrEqual:
+            xmlWriter.addAttribute("table:operator", "<=");
+            break;
+        case GreaterOrEqual:
+            xmlWriter.addAttribute("table:operator", ">=");
+            break;
+        case Empty:
+            xmlWriter.addAttribute("table:operator", "empty");
+            break;
+        case NotEmpty:
+            xmlWriter.addAttribute("table:operator", "!empty");
+            break;
+        case TopValues:
+            xmlWriter.addAttribute("table:operator", "top values");
+            break;
+        case BottomValues:
+            xmlWriter.addAttribute("table:operator", "bottom values");
+            break;
+        case TopPercent:
+            xmlWriter.addAttribute("table:operator", "top percent");
+            break;
+        case BottomPercent:
+            xmlWriter.addAttribute("table:operator", "bottom percent");
+            break;
         }
         if (caseSensitivity == Qt::CaseSensitive)
             xmlWriter.addAttribute("table:case-sensitive", true);
@@ -348,8 +333,7 @@ public:
             xmlWriter.addAttribute("table:data-type", "number");
         xmlWriter.endElement();
     }
-    virtual bool evaluate(const Database& database, int index) const
-    {
+    virtual bool evaluate(const Database& database, int index) const {
         const Sheet* sheet = database.range().lastSheet();
         const QRect range = database.range().lastRange();
         const int start = database.orientation() == Qt::Vertical ? range.left() : range.top();
@@ -358,47 +342,42 @@ public:
                             ? sheet->cellStorage()->value(start + fieldNumber, index)
                             : sheet->cellStorage()->value(index, start + fieldNumber);
         const QString testString = sheet->map()->converter()->asString(value).asString();
-        switch (operation)
-        {
-            case Match:
-            {
-                const bool result = QString::compare(this->value, testString, caseSensitivity) == 0;
+        switch (operation) {
+        case Match: {
+            const bool result = QString::compare(this->value, testString, caseSensitivity) == 0;
 //                 kDebug() <<"Match" << this->value <<"?" << testString <<"" << result;
-                if (result)
-                    return true;
-                break;
-            }
-            case NotMatch:
-            {
-                const bool result = QString::compare(this->value, testString, caseSensitivity) != 0;
+            if (result)
+                return true;
+            break;
+        }
+        case NotMatch: {
+            const bool result = QString::compare(this->value, testString, caseSensitivity) != 0;
 //                 kDebug() <<"Not Match" << this->value <<"?" << testString <<"" << result;
-                if (result)
-                    return true;
-                break;
-            }
-            default:
-                break;
+            if (result)
+                return true;
+            break;
+        }
+        default:
+            break;
         }
         return false;
     }
-    virtual bool isEmpty() const { return fieldNumber == -1; }
-    virtual QHash<QString, Filter::Comparison> conditions(int fieldNumber) const
-    {
+    virtual bool isEmpty() const {
+        return fieldNumber == -1;
+    }
+    virtual QHash<QString, Filter::Comparison> conditions(int fieldNumber) const {
         QHash<QString, Filter::Comparison> result;
         if (this->fieldNumber == fieldNumber)
             result.insert(value, operation);
         return result;
     }
-    virtual void removeConditions(int fieldNumber)
-    {
-        if (this->fieldNumber == fieldNumber)
-        {
+    virtual void removeConditions(int fieldNumber) {
+        if (this->fieldNumber == fieldNumber) {
 //             kDebug() <<"removing condition for fieldNumber" << fieldNumber;
             this->fieldNumber = -1;
         }
     }
-    bool operator!=(const Condition& other) const
-    {
+    bool operator!=(const Condition& other) const {
         if (fieldNumber == other.fieldNumber)
             return false;
         if (value == other.value)
@@ -411,14 +390,12 @@ public:
             return false;
         return true;
     }
-    virtual QString dump() const
-    {
+    virtual QString dump() const {
         QString result = QString("fieldNumber: %1 ").arg(fieldNumber);
-        switch (operation)
-        {
-            case Match:     result += "Match"; break;
-            case NotMatch:  result += "Not Match"; break;
-            default:        break;
+        switch (operation) {
+        case Match:     result += "Match"; break;
+        case NotMatch:  result += "Not Match"; break;
+        default:        break;
         }
         return result + " value: " + value + '\n';
     }
@@ -432,10 +409,9 @@ public:
 };
 
 Filter::And::And(const And& other)
-    : AbstractCondition()
+        : AbstractCondition()
 {
-    for (int i = 0; i < other.list.count(); ++i)
-    {
+    for (int i = 0; i < other.list.count(); ++i) {
         if (!other.list[i])
             continue;
         else if (other.list[i]->type() == AbstractCondition::And)
@@ -451,8 +427,7 @@ bool Filter::And::loadOdf(const KoXmlElement& parent)
 {
     KoXmlElement element;
     AbstractCondition* condition;
-    forEachElement(element, parent)
-    {
+    forEachElement(element, parent) {
         if (element.namespaceURI() != KoXmlNS::table)
             continue;
         if (element.localName() == "filter-or")
@@ -470,10 +445,9 @@ bool Filter::And::loadOdf(const KoXmlElement& parent)
 }
 
 Filter::Or::Or(const Or& other)
-    : AbstractCondition()
+        : AbstractCondition()
 {
-    for (int i = 0; i < other.list.count(); ++i)
-    {
+    for (int i = 0; i < other.list.count(); ++i) {
         if (!other.list[i])
             continue;
         else if (other.list[i]->type() == AbstractCondition::And)
@@ -489,8 +463,7 @@ bool Filter::Or::loadOdf(const KoXmlElement& parent)
 {
     KoXmlElement element;
     AbstractCondition* condition;
-    forEachElement(element, parent)
-    {
+    forEachElement(element, parent) {
         if (element.namespaceURI() != KoXmlNS::table)
             continue;
         if (element.localName() == "filter-and")
@@ -512,10 +485,9 @@ class Filter::Private
 {
 public:
     Private()
-        : condition( 0 )
-        , conditionSource(Self)
-        , displayDuplicates(true)
-    {
+            : condition(0)
+            , conditionSource(Self)
+            , displayDuplicates(true) {
     }
 
     AbstractCondition* condition;
@@ -526,12 +498,12 @@ public:
 };
 
 Filter::Filter()
-    : d(new Private)
+        : d(new Private)
 {
 }
 
 Filter::Filter(const Filter& other)
-    : d(new Private)
+        : d(new Private)
 {
     if (!other.d->condition)
         d->condition = 0;
@@ -558,32 +530,21 @@ void Filter::addCondition(Composition composition,
                           Qt::CaseSensitivity caseSensitivity, Mode mode)
 {
     Condition* condition = new Condition(fieldNumber, comparison, value, caseSensitivity, mode);
-    if (!d->condition)
-    {
+    if (!d->condition) {
         d->condition = condition;
-    }
-    else if (composition == AndComposition)
-    {
-        if (d->condition->type() == AbstractCondition::And)
-        {
+    } else if (composition == AndComposition) {
+        if (d->condition->type() == AbstractCondition::And) {
             static_cast<And*>(d->condition)->list.append(condition);
-        }
-        else
-        {
+        } else {
             And* andComposition = new And();
             andComposition->list.append(d->condition);
             andComposition->list.append(condition);
             d->condition = andComposition;
         }
-    }
-    else // composition == OrComposition
-    {
-        if (d->condition->type() == AbstractCondition::Or)
-        {
+    } else { // composition == OrComposition
+        if (d->condition->type() == AbstractCondition::Or) {
             static_cast<Or*>(d->condition)->list.append(condition);
-        }
-        else
-        {
+        } else {
             Or* orComposition = new Or();
             orComposition->list.append(d->condition);
             orComposition->list.append(condition);
@@ -594,8 +555,7 @@ void Filter::addCondition(Composition composition,
 
 void Filter::addSubFilter(Composition composition, const Filter& filter)
 {
-    if (!d->condition)
-    {
+    if (!d->condition) {
         if (!filter.d->condition)
             d->condition = 0;
         else if (filter.d->condition->type() == AbstractCondition::And)
@@ -604,20 +564,15 @@ void Filter::addSubFilter(Composition composition, const Filter& filter)
             d->condition = new Or(*static_cast<Or*>(filter.d->condition));
         else // if (filter.d->condition->type() == AbstractCondition::Condition)
             d->condition = new Condition(*static_cast<Condition*>(filter.d->condition));
-    }
-    else if (composition == AndComposition)
-    {
-        if (filter.d->condition && d->condition->type() == AbstractCondition::And)
-        {
+    } else if (composition == AndComposition) {
+        if (filter.d->condition && d->condition->type() == AbstractCondition::And) {
             if (filter.d->condition->type() == AbstractCondition::And)
                 static_cast<And*>(d->condition)->list += static_cast<And*>(filter.d->condition)->list;
             else if (filter.d->condition->type() == AbstractCondition::Or)
                 static_cast<And*>(d->condition)->list.append(new Or(*static_cast<Or*>(filter.d->condition)));
             else // if (filter.d->condition->type() == AbstractCondition::Condition)
                 static_cast<And*>(d->condition)->list.append(new Condition(*static_cast<Condition*>(filter.d->condition)));
-        }
-        else if (filter.d->condition)
-        {
+        } else if (filter.d->condition) {
             And* andComposition = new And();
             andComposition->list.append(d->condition);
             if (filter.d->condition->type() == AbstractCondition::And)
@@ -628,20 +583,15 @@ void Filter::addSubFilter(Composition composition, const Filter& filter)
                 andComposition->list.append(new Condition(*static_cast<Condition*>(filter.d->condition)));
             d->condition = andComposition;
         }
-    }
-    else // composition == OrComposition
-    {
-        if (filter.d->condition && d->condition->type() == AbstractCondition::Or)
-        {
+    } else { // composition == OrComposition
+        if (filter.d->condition && d->condition->type() == AbstractCondition::Or) {
             if (filter.d->condition->type() == AbstractCondition::And)
                 static_cast<Or*>(d->condition)->list.append(new And(*static_cast<And*>(filter.d->condition)));
             else if (filter.d->condition->type() == AbstractCondition::Or)
                 static_cast<Or*>(d->condition)->list += static_cast<Or*>(filter.d->condition)->list;
             else // if (filter.d->condition->type() == AbstractCondition::Condition)
                 static_cast<Or*>(d->condition)->list.append(new Condition(*static_cast<Condition*>(filter.d->condition)));
-        }
-        else if (filter.d->condition)
-        {
+        } else if (filter.d->condition) {
             Or* orComposition = new Or();
             orComposition->list.append(d->condition);
             if (filter.d->condition->type() == AbstractCondition::And)
@@ -662,8 +612,7 @@ QHash<QString, Filter::Comparison> Filter::conditions(int fieldNumber) const
 
 void Filter::removeConditions(int fieldNumber)
 {
-    if (fieldNumber == -1)
-    {
+    if (fieldNumber == -1) {
 //         kDebug() <<"removing all conditions";
         delete d->condition;
         d->condition = 0;
@@ -672,8 +621,7 @@ void Filter::removeConditions(int fieldNumber)
     if (!d->condition)
         return;
     d->condition->removeConditions(fieldNumber);
-    if (d->condition->isEmpty())
-    {
+    if (d->condition->isEmpty()) {
         delete d->condition;
         d->condition = 0;
     }
@@ -691,57 +639,46 @@ bool Filter::evaluate(const Database& database, int index) const
 
 bool Filter::loadOdf(const KoXmlElement& element, const Map* map)
 {
-    if (element.hasAttributeNS(KoXmlNS::table, "target-range-address"))
-    {
+    if (element.hasAttributeNS(KoXmlNS::table, "target-range-address")) {
         const QString address = element.attributeNS(KoXmlNS::table, "target-range-address", QString());
         // only absolute addresses allowed; no fallback sheet needed
         d->targetRangeAddress = Region(Region::loadOdf(address), map);
         if (!d->targetRangeAddress.isValid())
             return false;
     }
-    if (element.hasAttributeNS(KoXmlNS::table, "condition-source"))
-    {
+    if (element.hasAttributeNS(KoXmlNS::table, "condition-source")) {
         if (element.attributeNS(KoXmlNS::table, "condition-source", "self") == "cell-range")
             d->conditionSource = Private::CellRange;
         else
             d->conditionSource = Private::Self;
     }
-    if (element.hasAttributeNS(KoXmlNS::table, "condition-source-range-address"))
-    {
+    if (element.hasAttributeNS(KoXmlNS::table, "condition-source-range-address")) {
         const QString address = element.attributeNS(KoXmlNS::table, "condition-source-range-address", QString());
         // only absolute addresses allowed; no fallback sheet needed
         d->conditionSourceRangeAddress = Region(Region::loadOdf(address), map);
     }
-    if (element.hasAttributeNS(KoXmlNS::table, "display-duplicates"))
-    {
+    if (element.hasAttributeNS(KoXmlNS::table, "display-duplicates")) {
         if (element.attributeNS(KoXmlNS::table, "display-duplicates", "true") == "false")
             d->displayDuplicates = false;
         else
             d->displayDuplicates = true;
     }
     KoXmlElement conditionElement;
-    forEachElement(conditionElement, element)
-    {
-        if (conditionElement.localName() == "filter-and")
-        {
+    forEachElement(conditionElement, element) {
+        if (conditionElement.localName() == "filter-and") {
             d->condition = new And();
             break;
-        }
-        else if (conditionElement.localName() == "filter-or")
-        {
+        } else if (conditionElement.localName() == "filter-or") {
             d->condition = new Or();
             break;
-        }
-        else if (conditionElement.localName() == "filter-condition")
-        {
+        } else if (conditionElement.localName() == "filter-condition") {
             d->condition = new Condition();
             break;
         }
     }
     if (!d->condition)
         return false;
-    if (!d->condition->loadOdf(conditionElement.toElement()))
-    {
+    if (!d->condition->loadOdf(conditionElement.toElement())) {
         delete d->condition;
         d->condition = 0;
         return false;
@@ -781,13 +718,13 @@ bool Filter::operator==(const Filter& other) const
     if (d->condition->type() != other.d->condition->type())
         return false;
     if (d->condition->type() == AbstractCondition::And &&
-        *static_cast<And*>(d->condition) != *static_cast<And*>(other.d->condition))
+            *static_cast<And*>(d->condition) != *static_cast<And*>(other.d->condition))
         return false;
     if (d->condition->type() == AbstractCondition::Or &&
-        *static_cast<Or*>(d->condition) != *static_cast<Or*>(other.d->condition))
+            *static_cast<Or*>(d->condition) != *static_cast<Or*>(other.d->condition))
         return false;
     if (d->condition->type() == AbstractCondition::Condition &&
-        *static_cast<Condition*>(d->condition) != *static_cast<Condition*>(other.d->condition))
+            *static_cast<Condition*>(d->condition) != *static_cast<Condition*>(other.d->condition))
         return false;
     return true;
 }
@@ -795,7 +732,7 @@ bool Filter::operator==(const Filter& other) const
 void Filter::dump() const
 {
     if (d->condition)
-        kDebug() <<"Condition:" + d->condition->dump();
+        kDebug() << "Condition:" + d->condition->dump();
     else
-        kDebug() <<"Condition: 0";
+        kDebug() << "Condition: 0";
 }
