@@ -38,7 +38,7 @@
 #include <QPointF>
 #include <KoView.h>
 #include <KoZoomMode.h>
-#include "kspread_export.h"
+#include <kspread/kspread_export.h>
 
 class QAction;
 class QKeyEvent;
@@ -52,7 +52,7 @@ class KoZoomHandler;
 namespace KSpread
 {
 class Cell;
-class ColumnHeader;
+class ColumnHeaderWidget;
 class Damage;
 class Sheet;
 class Canvas;
@@ -60,119 +60,107 @@ class Doc;
 class Region;
 class Selection;
 class SheetView;
-class RowHeader;
+class RowHeaderWidget;
 class View;
 class TabBar;
 
-/** @class View
- *
+/**
+ * @class View
  * @brief The View class displays a KSpread document.
- *
- *
  * View is used to display a spreadsheet document and provide
  * the interface for the user to perform editing and data analysis.
  *
  * A view consists of several parts:
  *  \li canvas to display cells in a sheet
- *  \li line editor to display and edit cell contents
- *  \li location editor to show marker and/or selection
  *  \li column header to show columns
  *  \li row header to show rows
+ *  \li corner button to select all cells
  *  \li horizontal and vertical scrollbars for navigation
  *  \li tab bar to select active worksheet
- *
+ *  \li status bar to show status messages
  */
 class KSPREAD_EXPORT View : public KoView
 {
     Q_OBJECT
 
 public:
-
-    /** Creates a new view */
-    View( QWidget *_parent, Doc *doc );
+    /** Creates a new view displaying \p document and with \p parent as parent. */
+    View(QWidget *parent, Doc *document);
 
     /** Destroys the view */
     ~View();
 
+    /** \return the document this view displays. */
     Doc* doc() const;
 
-    /** Returns the canvas of the view */
+    /** \return the canvas of the view */
     Canvas* canvasWidget() const;
 
-    /** Returns the canvas controller of the view */
+    /** \return the canvas controller of the view */
     KoCanvasController* canvasController() const;
 
-    /** Returns the column header */
-    ColumnHeader* columnHeader() const;
+    /** \return the column header */
+    ColumnHeaderWidget* columnHeader() const;
 
-    /** Returns the row header */
-    RowHeader* rowHeader() const;
+    /** \return the row header */
+    RowHeaderWidget* rowHeader() const;
 
-    /** Returns the horizontal scrollbar */
+    /** \return the horizontal scrollbar */
     QScrollBar* horzScrollBar() const;
 
-    /** Returns the vertical scrollbar */
+    /** \return the vertical scrollbar */
     QScrollBar* vertScrollBar() const;
 
-    /** Returns the tab bar */
+    /** \return the tab bar */
     TabBar* tabBar() const;
 
     /** \return the zoom handler */
     KoZoomHandler* zoomHandler() const;
 
-    void setZoom( int zoom, bool updateViews ); // change the zoom value
-
-    void addSheet( Sheet *_t );
-    //void removesheet( Sheet *_t );
-    void removeAllSheets();
-
-    const Sheet* activeSheet() const;
-    Sheet* activeSheet();
+    /** \return the sheet, that is currently displayed */
+    Sheet* activeSheet() const;
 
     /**
      * \return the SheetView for \p sheet
      */
-    SheetView* sheetView( const Sheet* sheet ) const;
-
-    virtual QWidget *canvas() const;
+    SheetView* sheetView(const Sheet* sheet) const;
 
+    /** Loads the view settings. */
     void initConfig();
 
-
+    /** Initializes the on-the-fly calculation method. */
     void initCalcMenu();
 
+    /** Changes the maximum number of entries in the recent files menu. */
     void changeNbOfRecentFiles(int _nb);
 
+    /** Updates the state of the 'Show Page Borders' action (for DBus). */
     void updateBorderButton();
 
+    /** \return the color for cell borders \deprecated */
     QColor borderColor() const;
 
+    /**
+     * \return the current cell selection.
+     */
     Selection* selection() const;
 
+    /**
+     * Updates the action, that unhides a hidden sheet.
+     * If there are no hidden sheets or the map is protected,
+     * the show sheet action will be disabled.
+     */
     void updateShowSheetMenu();
 
-    /**
-     * Mark all selected cells / regions of cells as 'dirty' (ie. requiring a repaint)
-     * They will be repainted on the next call to paintUpdates()
-     */
-    void markSelectionAsDirty();
-
-    /**
-     * Repaint any cell with the paintDirty flag that is visible in this view
-     */
-    void paintUpdates();
-
-    bool showSheet(const QString& sheetName);
-
     /**
      * @return marker for @p sheet
      */
-    QPoint markerFromSheet( Sheet* sheet ) const;
+    QPoint markerFromSheet(Sheet* sheet) const;
 
     /**
      * @return scroll offset for @p sheet
      */
-    QPointF offsetFromSheet( Sheet* sheet ) const;
+    QPointF offsetFromSheet(Sheet* sheet) const;
 
     /**
      * Save current sheet selection.
@@ -181,76 +169,71 @@ public:
     void saveCurrentSheetSelection();
 
     /**
-     * @return @c true if document is being loaded. It is useful to suppress scrolling
-     * while the "View loading" process.
+     * The state of the 'View loading' process.
+     * Useful to suppress the painting of the canvas and setting the initial
+     * scrolling and selection positions, which need a fully built View.
+     * \return \c true if the view is not being fully created yet.
      */
     bool isLoading() const;
 
-    virtual QMatrix matrix() const;
-
-    virtual KoZoomController *zoomController() const;
-
-
 public Q_SLOTS:
-    /**
-    * refresh view when you hide/show vertical scrollbar
-    */
-    void refreshView();
+    /** Clears all visual cached data. */
     void refreshSheetViews();
+
+    /** Invalidates all visual cached data for the cells in \p region. */
     void refreshSelection(const Region& region);
-    void aboutToModify(const Region& region);
-    void initialPosition();
+
+    void finishLoading();
     /**
-     * Actions
+     * Prepares this view for a modification of \p region, e.g. closes the cell
+     * editor.
      */
+    void aboutToModify(const Region& region);
+
+    /** Sets the initial scrolling offset and selection after loading. */
+    void initialPosition();
+
+    //BEGIN Actions
     void createTemplate();
     void recalcWorkBook();
     void recalcWorkSheet();
     void paperLayoutDlg();
     void resetPrintRange();
-    void togglePageBorders( bool );
-    void toggleProtectSheet( bool );
-    void toggleProtectDoc( bool );
-    void viewZoom( KoZoomMode::Mode mode, qreal zoom );
+    void togglePageBorders(bool);
+    void toggleProtectSheet(bool);
+    void toggleProtectDoc(bool);
+    void viewZoom(KoZoomMode::Mode mode, qreal zoom);
 
     void insertSheet();
     void duplicateSheet();
     void deleteSheet();
     void hideSheet();
     void showSheet();
-    void helpUsing();
-
-    void setSelectionTextColor(const QColor &txtColor);
-    void setSelectionBackgroundColor(const QColor &bgColor);
-    void setSelectionBorderColor(const QColor &bdColor);
-
-    void setSelectionLeftBorderColor(const QColor &color);
-    void setSelectionRightBorderColor(const QColor &color);
-    void setSelectionTopBorderColor(const QColor &color);
-    void setSelectionBottomBorderColor(const QColor &color);
-    void setSelectionAllBorderColor(const QColor &color);
-    void setSelectionOutlineBorderColor(const QColor &color);
 
     void optionsNotifications();
     void preference();
-    void setSelectionComment(const QString& comment);
 
-    void copyAsText();
+    void copyAsText(); // DBus related
 
-    void moveSheet( unsigned sheet, unsigned target );
+    void moveSheet(unsigned sheet, unsigned target);
 
     /**
      * Shows the sheet properties dialog.
      */
     void sheetProperties();
+    //END Actions
 
-    void setActiveSheet(Sheet* sheet, bool updatesheet = true);
+    /**
+     * Sets the currently displayed \p sheet.
+     * \param updateTabBar if \c true, updates the tab bar
+     */
+    void setActiveSheet(Sheet* sheet, bool updateTabBar = true);
 
     /**
      * Switch the active sheet to the name. This slot is connected to the tab bar
      * and activated when the user selects a new sheet in the tab bar.
      */
-    void changeSheet( const QString& _name );
+    void changeSheet(const QString& _name);
 
     /**
      * Switch the active sheet to the next visible sheet. Does nothing if the current
@@ -283,7 +266,10 @@ public Q_SLOTS:
      */
     void setShapeAnchoring(const QString& mode);
 
+    /** Shows the context menu for the on-the-fly calculation method. */
     void statusBarClicked(const QPoint& pos);
+
+    /** Updates the on-the-fly calculation method menu. */
     void menuCalc(bool);
 
     /**
@@ -309,21 +295,23 @@ public Q_SLOTS:
     /**
      * Shows the status bar if b is true, otherwise the status bar will be hidden.
      */
-    void showStatusBar( bool b );
+    void showStatusBar(bool b);
 
     /**
      * Shows the tab bar if b is true, otherwise the tab bar will be hidden.
      */
-    void showTabBar( bool b );
+    void showTabBar(bool b);
 
     /**
      * Shows context menu when tabbar is double-clicked.
      */
-    void popupTabBarMenu( const QPoint& );
+    void popupTabBarMenu(const QPoint&);
 
-    void handleDamages( const QList<Damage*>& damages );
-
-    void initialiseMarkerFromSheet( Sheet *_sheet, const QPoint &point );
+    /**
+     * \ingroup Damages
+     * Handles damages that need visual updates.
+     */
+    void handleDamages(const QList<Damage*>& damages);
 
     /**
      * write in statusBar result of calc (Min, or Max, average, sum, count)
@@ -331,66 +319,65 @@ public Q_SLOTS:
     void calcStatusBarOp();
 
 protected slots:
+    /** Renames the active sheet. */
     void slotRename();
 
 public slots:
-    // Document signals
-    void slotRefreshView();
-    void slotUpdateView( Sheet *_sheet );
-    void slotUpdateView( Sheet *_sheet, const Region& );
-    void slotUpdateColumnHeader( Sheet *_sheet );
-    void slotUpdateRowHeader( Sheet *_sheet );
+    /** Reacts on selection changes. */
     void slotChangeSelection(const Region&);
+
+    /** Reacts on reference selection changes. */
     void slotScrollChoice(const Region&);
-    void slotAddSheet( Sheet *_sheet );
-    void slotSheetRenamed( Sheet* sheet, const QString& old_name );
-    void slotSheetHidden( Sheet*_sheet );
-    void slotSheetShown( Sheet*_sheet );
-    void slotSheetRemoved( Sheet*_sheet );
+
+    /** Updates the shape anchoring action. */
     void shapeSelectionChanged();
+
+    /** Calls KoToolProxy::deleteSelection(). */
     void editDeleteSelection();
 
-public:
+public: // reimplementations
+    // KoView interface
+    virtual QWidget *canvas() const;
     virtual int leftBorder() const;
     virtual int rightBorder() const;
     virtual int topBorder() const;
     virtual int bottomBorder() const;
 
-protected:
-
-    virtual void keyPressEvent ( QKeyEvent * _ev );
-    virtual void resizeEvent( QResizeEvent *_ev );
-
-    /**
-     * Activates the formula editor for the current cell.
-     * This function is usually called if the user presses
-     * a button in the formula toolbar.
-     */
-    void activateFormulaEditor();
-
-    virtual void updateReadWrite( bool readwrite );
-
-    virtual void guiActivateEvent( KParts::GUIActivateEvent *ev );
-
+protected: // reimplementations
+    // QWidget interface
+    virtual void keyPressEvent(QKeyEvent * _ev);
+    // KoView interface
+    virtual void updateReadWrite(bool readwrite);
+    virtual void guiActivateEvent(KParts::GUIActivateEvent *ev);
     virtual KoPrintJob * createPrintJob();
+public:
+    virtual KoZoomController *zoomController() const;
 
 Q_SIGNALS:
+    /** Indicates that the document's read/write state has changed. */
     void documentReadWriteToggled(bool readwrite);
+
+    /** Indicates that the sheet's protection state has changed. */
     void sheetProtectionToggled(bool protect);
 
 private Q_SLOTS:
+    /** Adds \p sheet to the displayed sheets. */
+    void addSheet(Sheet *sheet);
+
+    /** Removes \p sheet from the displayed sheets. */
     void removeSheet(Sheet* sheet);
-    void reviveSheet(Sheet* sheet);
 
 private:
-    Q_DISABLE_COPY( View )
+    Q_DISABLE_COPY(View)
 
     class Private;
     Private * const d;
 
+    /** Creates and initializes the canvas and other child widgets. */
     void initView();
 
-    friend class Private;
+    /** Sets the column/row headers minima according to the zoom level. */
+    void setHeaderMinima();
 };
 
 } // namespace KSpread
