@@ -52,7 +52,6 @@
 #include "ApplicationSettings.h"
 #include "CalculationSettings.h"
 #include "part/Doc.h"
-#include "Editors.h"
 #include "part/Factory.h"
 #include "FunctionModuleRegistry.h"
 #include "Localization.h"
@@ -80,6 +79,7 @@ public:
     MethodOfCalc oldFunction;
     KoUnit oldUnit;
     double oldIndentationStep;
+    bool oldCaptureAllArrowKeys;
     QColor oldGridColor;
     QColor oldPageBorderColor;
 
@@ -149,6 +149,13 @@ void PreferenceDialog::Private::applyInterfaceOptions()
         oldIndentationStep = value;
     }
 
+    const bool capture = interfaceOptions.m_captureAllArrowKeys->isChecked();
+    if (capture != view->doc()->map()->settings()->captureAllArrowKeys()) {
+        view->doc()->map()->settings()->setCaptureAllArrowKeys(capture);
+        config->group("Editor").writeEntry("CaptureAllArrowKeys", capture);
+        oldCaptureAllArrowKeys = capture;
+    }
+
     const QColor gridColor = interfaceOptions.m_gridColor->color();
     if (gridColor != view->doc()->map()->settings()->gridColor()) {
         view->doc()->map()->settings()->setGridColor(gridColor);
@@ -163,41 +170,30 @@ void PreferenceDialog::Private::applyInterfaceOptions()
         oldPageBorderColor = pageBorderColor;
     }
 
-#if 0 // UNDOREDOLIMIT
-    int const newUndo=m_undoRedoLimit->value();
-    if( newUndo!=m_oldNbRedo )
-    {
-        config->group( "Misc" ).writeEntry( "UndoRedo", newUndo );
-        view->doc()->map()->settings()->setUndoRedoLimit(newUndo);
-        m_oldNbRedo=newUndo;
-    }
-#endif
 #if 0 // KSPREAD_COMPLETION_MODE_SETTING
-    KGlobalSettings::Completion tmpCompletion=KGlobalSettings::CompletionNone;
-    switch(typeCompletion->currentIndex())
-    {
-        case 0:
-            tmpCompletion=KGlobalSettings::CompletionNone;
-            break;
-        case 1:
-            tmpCompletion=KGlobalSettings::CompletionShell;
-            break;
-        case 2:
-            tmpCompletion=KGlobalSettings::CompletionPopup;
-            break;
-        case 3:
-            tmpCompletion=KGlobalSettings::CompletionAuto;
-            break;
-        case 4:
-            tmpCompletion=KGlobalSettings::CompletionMan;
-            break;
+    KGlobalSettings::Completion tmpCompletion = KGlobalSettings::CompletionNone;
+    switch (typeCompletion->currentIndex()) {
+    case 0:
+        tmpCompletion = KGlobalSettings::CompletionNone;
+        break;
+    case 1:
+        tmpCompletion = KGlobalSettings::CompletionShell;
+        break;
+    case 2:
+        tmpCompletion = KGlobalSettings::CompletionPopup;
+        break;
+    case 3:
+        tmpCompletion = KGlobalSettings::CompletionAuto;
+        break;
+    case 4:
+        tmpCompletion = KGlobalSettings::CompletionMan;
+        break;
     }
 
 
-    if(comboChanged)
-    {
+    if (comboChanged) {
         view->doc()->map()->settings()->setCompletionMode(tmpCompletion);
-        parameterGroup.writeEntry( "Completion Mode", (int)tmpCompletion);
+        parameterGroup.writeEntry("Completion Mode", (int)tmpCompletion);
     }
 #endif
 }
@@ -208,11 +204,9 @@ void PreferenceDialog::Private::defaultInterfaceOptions()
     interfaceOptions.m_statusBarFunction->setCurrentIndex(0);
     interfaceOptions.m_unit->setCurrentIndex(0);
     interfaceOptions.m_indentationStep->changeValue(10.0);
+    interfaceOptions.m_captureAllArrowKeys->setChecked(true);
     interfaceOptions.m_gridColor->setColor(Qt::lightGray);
     interfaceOptions.m_pageBorderColor->setColor(Qt::red);
-#if 0 // UNDOREDOLIMIT
-    m_undoRedoLimit->setValue(30);
-#endif
 #if 0 // KSPREAD_COMPLETION_MODE_SETTING
     typeCompletion->setCurrentIndex(3);
 #endif
@@ -227,7 +221,7 @@ void PreferenceDialog::Private::resetInterfaceOptions()
     oldFunction = view->doc()->map()->settings()->getTypeOfCalc();
     oldUnit = view->doc()->unit();
     oldIndentationStep = view->doc()->map()->settings()->indentValue();
-//     m_oldNbRedo = config->group( "Misc" ).readEntry( "UndoRedo", m_oldNbRedo );
+    oldCaptureAllArrowKeys = view->doc()->map()->settings()->captureAllArrowKeys();
 
     const KConfigGroup colorGroup = config->group("KSpread Color");
     oldGridColor = colorGroup.readEntry("GridColor", QColor(Qt::lightGray));
@@ -239,21 +233,22 @@ void PreferenceDialog::Private::resetInterfaceOptions()
     interfaceOptions.m_statusBarFunction->setCurrentIndex(functionIndex);
     interfaceOptions.m_unit->setCurrentIndex(oldUnit.indexInList());
     interfaceOptions.m_indentationStep->changeValue(oldIndentationStep);
+    interfaceOptions.m_captureAllArrowKeys->setChecked(oldCaptureAllArrowKeys);
     interfaceOptions.m_gridColor->setColor(oldGridColor);
     interfaceOptions.m_pageBorderColor->setColor(oldPageBorderColor);
 }
 
 void PreferenceDialog::Private::applyOpenSaveOptions()
 {
-    KSharedConfigPtr config = Factory::global().config();;
+    KSharedConfigPtr config = Factory::global().config();
     KConfigGroup parameterGroup = config->group("Parameters");
     Doc* doc = view->doc();
 
     int value = fileOptions.m_recentFilesEntries->value();
     if (value != oldRecentFilesEntries) {
-       parameterGroup.writeEntry("NbRecentFile", value);
-       view->changeNbOfRecentFiles(value);
-       oldRecentFilesEntries = value;
+        parameterGroup.writeEntry("NbRecentFile", value);
+        view->changeNbOfRecentFiles(value);
+        oldRecentFilesEntries = value;
     }
 
     value = fileOptions.m_autoSaveDelay->value();
@@ -295,8 +290,8 @@ void PreferenceDialog::Private::resetOpenSaveOptions()
 
 
 PreferenceDialog::PreferenceDialog(View* view)
-    : KPageDialog(view)
-    , d(new Private)
+        : KPageDialog(view)
+        , d(new Private)
 {
     setObjectName("PreferenceDialog");
     setCaption(i18n("Configure KSpread"));
@@ -306,8 +301,8 @@ PreferenceDialog::PreferenceDialog(View* view)
 
     d->view = view;
 
-    connect(this, SIGNAL(okClicked()), this,SLOT(slotApply()));
-    connect(this, SIGNAL(defaultClicked()), this,SLOT(slotDefault()));
+    connect(this, SIGNAL(okClicked()), this, SLOT(slotApply()));
+    connect(this, SIGNAL(defaultClicked()), this, SLOT(slotDefault()));
     connect(this, SIGNAL(resetClicked()), this, SLOT(slotReset()));
 
     QWidget* widget = 0;
@@ -365,8 +360,8 @@ PreferenceDialog::PreferenceDialog(View* view)
 
     // Plugin Options Widget
     d->pluginSelector = new KPluginSelector(this);
-    const QString serviceType = QString::fromLatin1("KSpread/Plugin");
-    const QString query = QString::fromLatin1("[X-KSpread-Version] >= 2");
+    const QString serviceType = QLatin1String("KSpread/Plugin");
+    const QString query = QLatin1String("([X-KSpread-InterfaceVersion] == 0)");
     const KService::List offers = KServiceTypeTrader::self()->query(serviceType, query);
     const QList<KPluginInfo> pluginInfoList = KPluginInfo::fromServices(offers);
     d->pluginSelector->addPlugins(pluginInfoList, KPluginSelector::ReadConfigFile,
@@ -413,12 +408,13 @@ void PreferenceDialog::slotApply()
 
     // Plugin Options
     d->pluginSelector->save();
-    FunctionModuleRegistry::instance()->loadFunctions();
+    FunctionModuleRegistry::instance()->loadFunctionModules();
 
     d->spellCheckPage->save();
 
-    d->view->doc()->refreshInterface();
-    d->view->slotUpdateView(d->view->activeSheet());
+    // The changes affect the document, not just a single view,
+    // so all user interfaces have to be updated.
+    d->view->doc()->updateAllViews();
 }
 
 void PreferenceDialog::slotDefault()
@@ -454,30 +450,22 @@ void PreferenceDialog::unitChanged(int index)
 }
 
 
-#if 0 // UNDOREDOLIMIT
-  m_oldNbRedo = config->group( "Misc" ).readEntry( "UndoRedo", m_oldNbRedo );
-
-  m_undoRedoLimit=new KIntNumInput( m_oldNbRedo, tmpQGroupBox );
-  m_undoRedoLimit->setLabel(i18n("Undo/redo limit:"));
-  m_undoRedoLimit->setRange(10, 60, 1);
-#endif
-
 #if 0 // KSPREAD_COMPLETION_MODE_SETTING
-  QLabel *label=new QLabel(i18n("&Completion mode:"), tmpQGroupBox);
+QLabel *label = new QLabel(i18n("&Completion mode:"), tmpQGroupBox);
 
-  typeCompletion=new KComboBox(tmpQGroupBox);
-  label->setBuddy(typeCompletion);
-  typeCompletion->setWhatsThis( i18n( "Lets you choose the (auto) text completion mode from a range of options in the drop down selection box." ) );
+typeCompletion = new KComboBox(tmpQGroupBox);
+label->setBuddy(typeCompletion);
+typeCompletion->setWhatsThis(i18n("Lets you choose the (auto) text completion mode from a range of options in the drop down selection box."));
 
-  listType+=i18n("None");
-  listType+=i18n("Manual");
-  listType+=i18n("Popup");
-  listType+=i18n("Automatic");
-  listType+=i18n("Semi-Automatic");
-  typeCompletion->insertItems( 0,listType);
-  typeCompletion->setCurrentIndex(0);
-  comboChanged=false;
-  connect(typeCompletion,SIGNAL(activated( const QString & )),this,SLOT(slotTextComboChanged(const QString &)));
+listType += i18n("None");
+listType += i18n("Manual");
+listType += i18n("Popup");
+listType += i18n("Automatic");
+listType += i18n("Semi-Automatic");
+typeCompletion->insertItems(0, listType);
+typeCompletion->setCurrentIndex(0);
+comboChanged = false;
+connect(typeCompletion, SIGNAL(activated(const QString &)), this, SLOT(slotTextComboChanged(const QString &)));
 #endif
 
 #include "PreferenceDialog.moc"
