@@ -28,21 +28,29 @@
 namespace KSpread
 {
 
+class Doc;
+class Sheet;
+
 class TestLogicFunctions: public QObject
 {
     Q_OBJECT
 
 private slots:
+    void init();
+    void cleanup();
+
     void initTestCase();
-  void testAND();
-  void testFALSE();
-  void testIF();
-  void testNOT();
-  void testOR();
-  void testTRUE();
-  void testXOR();
+    void testAND();
+    void testFALSE();
+    void testIF();
+    void testNOT();
+    void testOR();
+    void testTRUE();
+    void testXOR();
 private:
-  Value evaluate(const QString&, Value& ex);
+    Doc* m_doc;
+    Sheet* m_sheet;
+    Value evaluate(const QString&, Value& ex);
 };
 
 } // namespace KSpread
