@@ -31,13 +31,16 @@
 
 #include <KoInteractionTool.h>
 
-#include "Cell.h"
-#include "kspread_export.h"
+#include <kspread/Cell.h>
+#include <kspread/kspread_export.h>
 #include "Selection.h"
 
+class KoColor;
+
 namespace KSpread
 {
 class SheetView;
+class CellEditorBase;
 
 /**
  * Abstract tool providing actions acting on cell ranges.
@@ -47,6 +50,14 @@ class KSPREAD_EXPORT CellToolBase : public KoInteractionTool
     Q_OBJECT
 
 public:
+    /**
+     * The editor type.
+     */
+    enum Editor {
+        EmbeddedEditor,  ///< the embedded editor appearing in a cell
+        ExternalEditor   ///< the external editor located in the tool options
+    };
+
     /**
      * Constructor.
      */
@@ -70,26 +81,46 @@ public:
 
     virtual Selection* selection() = 0;
 
-    bool createEditor(bool clear = true, bool focus = true);
-    CellEditor* editor() const;
+    virtual void deleteSelection();
+
+    virtual bool createEditor(bool clear = true, bool focus = true);
+    virtual CellEditorBase* editor() const;
+
+    /**
+     * Sets the editor \p type, which had the focus at last.
+     */
+    void setLastEditorWithFocus(Editor type);
+
+    /**
+     * Scrolls to the cell located at \p location.
+     */
+    void scrollToCell(const QPoint &location);
 
 public Q_SLOTS:
-    virtual void activate(bool temporary = false);
+    virtual void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes);
     virtual void deactivate();
 
-    void deleteEditor(bool saveChanges, bool expandMatrix = false);
+    virtual void deleteEditor(bool saveChanges, bool expandMatrix = false);
 
 protected:
     void init();
     virtual QWidget* createOptionWidget();
-    void applyUserInput(bool expandMatrix = false);
+    void applyUserInput(const QString &userInput, bool expandMatrix = false);
     virtual KoInteractionStrategy* createStrategy(KoPointerEvent* event);
+
+    /**
+     * The shape offset in document coordinates.
+     */
     virtual QPointF offset() const = 0;
     virtual QSizeF size() const = 0;
+
+    /**
+     * The canvas scrolling offset in document coordinates.
+     */
+    virtual QPointF canvasOffset() const = 0;
     virtual int maxCol() const = 0;
     virtual int maxRow() const = 0;
     virtual SheetView* sheetView(const Sheet* sheet) const = 0;
-    bool eventFilter(QObject *obj, QEvent *event);
 
 protected Q_SLOTS:
     void selectionChanged(const Region&);
@@ -114,7 +145,7 @@ protected Q_SLOTS:
     void fontSize(int size);
     void increaseFontSize();
     void decreaseFontSize();
-    void changeTextColor();
+    void changeTextColor(const KoColor &);
     // -- horizontal alignment actions --
     void alignLeft(bool enable);
     void alignRight(bool enable);
@@ -131,7 +162,7 @@ protected Q_SLOTS:
     void borderAll();
     void borderRemove();
     void borderOutline();
-    void borderColor();
+    void borderColor(const KoColor &);
     // -- text layout actions --
     void wrapText(bool enable);
     void verticalText(bool enable);
@@ -147,7 +178,7 @@ protected Q_SLOTS:
     void toUpperCase();
     void toLowerCase();
     void firstLetterToUpperCase();
-    void changeBackgroundColor();
+    void changeBackgroundColor(const KoColor &);
     // -- cell merging actions --
     void mergeCells();
     void mergeCellsHorizontal();
@@ -245,6 +276,8 @@ protected Q_SLOTS:
     void listChoosePopupMenu();
     void listChooseItemSelected(QAction*);
     void documentSettingsDialog();
+    void breakBeforeColumn(bool);
+    void breakBeforeRow(bool);
 
 private:
     Q_DISABLE_COPY(CellToolBase)
