@@ -31,8 +31,6 @@
 
 // KSpread
 #include "CellToolBase.h"
-#include "Editors.h"
-#include "CharSelectDia.h"
 
 // dialogs
 #include "dialogs/FindDialog.h"
@@ -50,8 +48,12 @@ class QUndoCommand;
 class KFind;
 class KReplace;
 
+
 namespace KSpread
 {
+class CharacterSelectDialog;
+class CellEditor;
+class CellToolOptionWidget;
 class Sheet;
 
 class CellToolBase::Private
@@ -62,20 +64,17 @@ public:
     // Inline editor
     CellEditor* cellEditor;
     // Insert special character dialog
-    CharSelectDia* specialCharDialog;
+    CharacterSelectDialog* specialCharDialog;
     // Option widget elements
-    QGridLayout *widgetLayout;
-    ExternalEditor *userInput;
+    CellToolOptionWidget *optionWidget;
     QPointer<FormulaDialog> formulaDialog;
-    QToolButton *formulaButton, *applyButton, *cancelButton;
-    LocationComboBox* locationComboBox;
-    bool hasWideLayout;
     // Actions with extended names for the popup menu
     QHash<QString, QAction*> popupMenuActions;
     // Initialization flag.
     bool initialized;
     // For "Selection List..." action
     QMenu* popupListChoose;
+    Editor lastEditorWithFocus;
 
     // Find and Replace context. We remember the options and
     // the strings used previously.
@@ -103,7 +102,6 @@ public:
 
 public:
     void updateEditor(const Cell& cell);
-    void updateLocationComboBox();
     void updateActions(const Cell& cell);
     void setProtectedActionsEnabled(bool enable);
 
@@ -114,9 +112,6 @@ public:
     bool processEndKey(QKeyEvent *event);
     bool processPriorKey(QKeyEvent *event);
     bool processNextKey(QKeyEvent *event);
-    void processDeleteKey(QKeyEvent *event);
-    void processF2Key(QKeyEvent *event);
-    void processF4Key(QKeyEvent *event);
     void processOtherKey(QKeyEvent *event);
     bool processControlArrowKey(QKeyEvent *event);
     bool formatKeyPress(QKeyEvent *event);
@@ -156,8 +151,17 @@ public:
     QList<QAction*> popupActionList() const;
     void createPopupMenuActions();
 
-    /** Relayout the tool option docker. */
-    void relayoutDocker (bool wide);
+
+    /**
+     * \ingroup UI
+     * Tests for text values in the selection cursor's column, that are
+     * different to the one at the selection cursor.
+     * Used to create a popup menu consisting of text values to choose from.
+     * \param selection the selection of cells to work on
+     * \return \c true if there are text values in the selection cursor's column
+     */
+    bool testListChoose(Selection *selection) const;
+
 private:
     CellToolBase* q;
 };
