@@ -36,39 +36,48 @@
 #include <KoXmlReader.h>
 #include <KoXmlNS.h>
 #include <KoShapeRegistry.h>
-#include <KoShapeFactory.h>
+#include <KoShapeFactoryBase.h>
 #include <KoTextShapeData.h>
-#include <KoDataCenter.h>
 #include <KoShapeLoadingContext.h>
 #include <KoStyleManager.h>
 #include <KoOdfLoadingContext.h>
+#include <KoUpdater.h>
+#include <KoProgressUpdater.h>
 
 // KDE + Qt includes
 #include <QTextCursor>
 #include <KDebug>
 
+#include <rdf/KoDocumentRdfBase.h>
+
 KWOdfLoader::KWOdfLoader(KWDocument *document)
         : QObject(document),
         m_document(document)
 {
-    connect(this, SIGNAL(sigProgress(int)), m_document, SIGNAL(sigProgress(int)));
+    connect(this, SIGNAL(progressUpdate(int)), m_document, SIGNAL(sigProgress(int)));
 }
 
 KWOdfLoader::~KWOdfLoader()
 {
 }
 
-KWDocument* KWOdfLoader::document() const
+KWDocument *KWOdfLoader::document() const
 {
     return m_document;
 }
 
 //1.6: KWDocument::loadOasis
-bool KWOdfLoader::load(KoOdfReadStore & odfStore)
+bool KWOdfLoader::load(KoOdfReadStore &odfStore)
 {
-    emit sigProgress(0);
     //kDebug(32001) << "========================> KWOdfLoader::load START";
 
+    QPointer<KoUpdater> updater;
+    if (m_document->progressUpdater()) {
+        updater = m_document->progressUpdater()->startSubtask(1,
+                                                           "KWOdfLoader::load");
+        updater->setProgress(0);
+    }
+
     KoXmlElement content = odfStore.contentDoc().documentElement();
     KoXmlElement realBody(KoXml::namedItemNS(content, KoXmlNS::office, "body"));
     if (realBody.isNull()) {
@@ -91,6 +100,8 @@ bool KWOdfLoader::load(KoOdfReadStore & odfStore)
         return false;
     }
 
+    if (updater) updater->setProgress(20);
+
     // TODO check versions and mimetypes etc.
 
     bool hasMainText = false;
@@ -110,28 +121,32 @@ bool KWOdfLoader::load(KoOdfReadStore & odfStore)
     }
 
     KoOdfLoadingContext odfContext(odfStore.styles(), odfStore.store(), m_document->componentData());
-    KoShapeLoadingContext sc(odfContext, m_document->dataCenterMap());
+    KoShapeLoadingContext sc(odfContext, m_document->resourceManager());
 
     // Load all styles before the corresponding paragraphs try to use them!
-    KWOdfSharedLoadingData * sharedData = new KWOdfSharedLoadingData(this);
-    KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>(m_document->dataCenterMap()["StyleManager"]);
-    Q_ASSERT(styleManager);
-    sharedData->loadOdfStyles(odfContext, styleManager);
+    KWOdfSharedLoadingData *sharedData = new KWOdfSharedLoadingData(this);
     sc.addSharedData(KOTEXT_SHARED_LOADING_ID, sharedData);
+    KoStyleManager *styleManager = m_document->resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
+    Q_ASSERT(styleManager);
+    sharedData->loadOdfStyles(sc, styleManager);
+
+    if (updater) updater->setProgress(40);
 
-    KoTextLoader * loader = new KoTextLoader(sc);
-    Q_UNUSED(loader);
     KoOdfLoadingContext context(odfStore.styles(), odfStore.store(), m_document->componentData());
 
     loadMasterPageStyles(context, hasMainText);
 
+    // add page background frame set
+    KWFrameSet *pageBackgroundFrameSet = new KWFrameSet(KWord::BackgroundFrameSet);
+    m_document->addFrameSet(pageBackgroundFrameSet);
+
 #if 0 //1.6:
     KWOasisLoader oasisLoader(this);
     // <text:page-sequence> oasis extension for DTP (2003-10-27 post by Daniel)
     m_processingType = (!KoXml::namedItemNS(body, KoXmlNS::text, "page-sequence").isNull()) ? DTP : WP;
     m_hasTOC = false;
     m_tabStop = MM_TO_POINT(15);
-    const KoXmlElement* defaultParagStyle = styles.defaultStyle("paragraph");
+    const KoXmlElement *defaultParagStyle = styles.defaultStyle("paragraph");
     if (defaultParagStyle) {
         KoStyleStack stack;
         stack.push(*defaultParagStyle);
@@ -163,15 +178,15 @@ bool KWOdfLoader::load(KoOdfReadStore & odfStore)
     /*
     // We always needs at least one valid default paragraph style
     KoParagraphStyle *defaultParagraphStyle = m_document->styleManager()->defaultParagraphStyle();
-    //const KoXmlElement* defaultParagraphStyle = context.stylesReader().defaultStyle("paragraph");
-    //if( ! defaultParagraphStyle ) {
+    //const KoXmlElement *defaultParagraphStyle = context.stylesReader().defaultStyle("paragraph");
+    //if(! defaultParagraphStyle) {
     KoParagraphStyle *parastyle = new KoParagraphStyle();
     parastyle->setName("Standard");
     m_document->styleManager()->add(parastyle);
-    context.styleStack().setTypeProperties( "paragraph" ); // load all style attributes from "style:paragraph-properties"
+    context.styleStack().setTypeProperties("paragraph"); // load all style attributes from "style:paragraph-properties"
     parastyle->loadOasis(context.styleStack()); // load the KoParagraphStyle from the stylestack
     KoCharacterStyle *charstyle = parastyle->characterStyle();
-    context.styleStack().setTypeProperties( "text" ); // load all style attributes from "style:text-properties"
+    context.styleStack().setTypeProperties("text"); // load all style attributes from "style:text-properties"
     charstyle->loadOasis(context.styleStack()); // load the KoCharacterStyle from the stylestack
     //}
     */
@@ -190,24 +205,36 @@ bool KWOdfLoader::load(KoOdfReadStore & odfStore)
         }
     }
 
+    if (updater) updater->setProgress(60);
+
     KoTextShapeData textShapeData;
     if (hasMainText) {
         KWTextFrameSet *mainFs = new KWTextFrameSet(m_document, KWord::MainTextFrameSet);
         mainFs->setAllowLayout(false);
+        mainFs->setPageStyle(m_document->pageManager()->pageStyle("Standard"));
         m_document->addFrameSet(mainFs);
         textShapeData.setDocument(mainFs->document(), false);
     }
+
+    if (updater) updater->setProgress(80);
+
     // Let the TextShape handle loading the body element.
-    textShapeData.loadOdf(body, sc);
+    textShapeData.loadOdf(body, sc, m_document->documentRdfBase());
+
+    // Grab weak references to all the Rdf stuff that was loaded
+    if (KoDocumentRdfBase *rdf = m_document->documentRdfBase()) {
+        rdf->updateInlineRdfStatements(textShapeData.document());
+    }
+
+    if (updater) updater->setProgress(90);
 
     loadSettings(odfStore.settingsDoc());
 
-    //kDebug(32001) << "========================> KWOdfLoader::load END";
-    emit sigProgress(100);
+    if (updater) updater->setProgress(100);
     return true;
 }
 
-void KWOdfLoader::loadSettings(const KoXmlDocument& settingsDoc)
+void KWOdfLoader::loadSettings(const KoXmlDocument &settingsDoc)
 {
     if (settingsDoc.isNull())
         return;
@@ -223,19 +250,19 @@ void KWOdfLoader::loadSettings(const KoXmlDocument& settingsDoc)
     if (!configurationSettings.isNull()) {
         const QString ignorelist = configurationSettings.parseConfigItemString("SpellCheckerIgnoreList");
         kDebug(32001) << "Ignorelist:" << ignorelist;
-        //1.6: m_document->setSpellCheckIgnoreList( QStringList::split( ',', ignorelist ) );
+        //1.6: m_document->setSpellCheckIgnoreList(QStringList::split(',', ignorelist));
     }
-    //1.6: m_document->variableCollection()->variableSetting()->loadOasis( settings );
+    //1.6: m_document->variableCollection()->variableSetting()->loadOasis(settings);
 }
 
-void KWOdfLoader::loadMasterPageStyles(KoOdfLoadingContext& context, bool hasMainText)
+void KWOdfLoader::loadMasterPageStyles(KoOdfLoadingContext &context, bool hasMainText)
 {
     kDebug(32001) << " !!!!!!!!!!!!!! loadMasterPageStyles called !!!!!!!!!!!!!!";
     kDebug(32001) << "Number of items :" << context.stylesReader().masterPages().size();
 
     //TODO probably we should introduce more logic to handle the "standard" even
     //in faulty documents. See also bugreport #129585 as example.
-    const KoOdfStylesReader& styles = context.stylesReader();
+    const KoOdfStylesReader &styles = context.stylesReader();
     QHashIterator<QString, KoXmlElement *> it(styles.masterPages());
     while (it.hasNext()) {
         it.next();
@@ -247,9 +274,9 @@ void KWOdfLoader::loadMasterPageStyles(KoOdfLoadingContext& context, bool hasMai
         const KoXmlElement *masterNode = it.value();
         const KoXmlElement *masterPageStyle = masterNode ? styles.findStyle(masterNode->attributeNS(KoXmlNS::style, "page-layout-name", QString())) : 0;
         if (masterPageStyle) {
-            masterPage.loadOdf(*masterPageStyle);
-            loadHeaderFooter(context, masterPage, *masterNode, *masterPageStyle, LoadHeader);
-            loadHeaderFooter(context, masterPage, *masterNode, *masterPageStyle, LoadFooter);
+            masterPage.loadOdf(context, *masterNode, *masterPageStyle, m_document->resourceManager());
+            loadHeaderFooter(context, masterPage, *masterNode, LoadHeader);
+            loadHeaderFooter(context, masterPage, *masterNode, LoadFooter);
         }
         masterPage.setHasMainTextFrame(hasMainText);
         if (!alreadyExists)
@@ -258,7 +285,7 @@ void KWOdfLoader::loadMasterPageStyles(KoOdfLoadingContext& context, bool hasMai
 }
 
 // helper function to create a KWTextFrameSet for a header/footer.
-void KWOdfLoader::loadHeaderFooterFrame(KoOdfLoadingContext& context, const KWPageStyle &pageStyle, const KoXmlElement& elem, KWord::HeaderFooterType hfType, KWord::TextFrameSetType fsType)
+void KWOdfLoader::loadHeaderFooterFrame(KoOdfLoadingContext &context, const KWPageStyle &pageStyle, const KoXmlElement &elem, KWord::TextFrameSetType fsType)
 {
     KWTextFrameSet *fs = new KWTextFrameSet(m_document, fsType);
     fs->setPageStyle(pageStyle);
@@ -270,8 +297,14 @@ void KWOdfLoader::loadHeaderFooterFrame(KoOdfLoadingContext& context, const KWPa
     // use auto-styles from styles.xml, not those from content.xml
     context.setUseStylesAutoStyles(true);
 
-    KoShapeLoadingContext ctxt(context, m_document->dataCenterMap());
-    KoTextLoader loader(ctxt);
+    KoShapeLoadingContext ctxt(context, m_document->resourceManager());
+    KWOdfSharedLoadingData *sharedData = new KWOdfSharedLoadingData(this);
+    KoStyleManager *styleManager = m_document->resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
+    Q_ASSERT(styleManager);
+    sharedData->loadOdfStyles(ctxt, styleManager);
+    ctxt.addSharedData(KOTEXT_SHARED_LOADING_ID, sharedData);
+
+    KoTextLoader loader(ctxt, m_document->documentRdfBase());
     QTextCursor cursor(fs->document());
     loader.loadBody(elem, cursor);
 
@@ -280,7 +313,7 @@ void KWOdfLoader::loadHeaderFooterFrame(KoOdfLoadingContext& context, const KWPa
 }
 
 //1.6: KWOasisLoader::loadOasisHeaderFooter
-void KWOdfLoader::loadHeaderFooter(KoOdfLoadingContext& context, KWPageStyle &pageStyle, const KoXmlElement& masterPage, const KoXmlElement& masterPageStyle, HFLoadType headerFooter)
+void KWOdfLoader::loadHeaderFooter(KoOdfLoadingContext &context, KWPageStyle &pageStyle, const KoXmlElement &masterPage, HFLoadType headerFooter)
 {
     // The actual content of the header/footer.
     KoXmlElement elem = KoXml::namedItemNS(masterPage, KoXmlNS::style, headerFooter == LoadHeader ? "header" : "footer");
@@ -291,11 +324,11 @@ void KWOdfLoader::loadHeaderFooter(KoOdfLoadingContext& context, KWPageStyle &pa
     KWord::HeaderFooterType hfType = elem.isNull() ? KWord::HFTypeNone : leftElem.isNull() ? KWord::HFTypeUniform : KWord::HFTypeEvenOdd;
 
     if (! leftElem.isNull()) {   // header-left and footer-left
-        loadHeaderFooterFrame(context, pageStyle, leftElem, hfType, headerFooter == LoadHeader ? KWord::EvenPagesHeaderTextFrameSet : KWord::EvenPagesFooterTextFrameSet);
+        loadHeaderFooterFrame(context, pageStyle, leftElem, headerFooter == LoadHeader ? KWord::EvenPagesHeaderTextFrameSet : KWord::EvenPagesFooterTextFrameSet);
     }
 
     if (! elem.isNull()) {   // header and footer
-        loadHeaderFooterFrame(context, pageStyle, elem, hfType, headerFooter == LoadHeader ? KWord::OddPagesHeaderTextFrameSet : KWord::OddPagesFooterTextFrameSet);
+        loadHeaderFooterFrame(context, pageStyle, elem, headerFooter == LoadHeader ? KWord::OddPagesHeaderTextFrameSet : KWord::OddPagesFooterTextFrameSet);
     }
 
     if (headerFooter == LoadHeader) {
