@@ -25,8 +25,6 @@
 #include "KWDocument.h"
 #include "KWPage.h"
 
-#include <ktemporaryfile.h>
-
 #include "frames/KWTextFrameSet.h"
 #include "frames/KWTextFrame.h"
 #include <KoXmlWriter.h>
@@ -43,12 +41,14 @@
 #include <KoTextSharedSavingData.h>
 
 #include <KoStoreDevice.h>
+#include <rdf/KoDocumentRdfBase.h>
 
 #include <QBuffer>
 #include <QTextCursor>
 #include <KDebug>
+#include <ktemporaryfile.h>
 
-QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGenStyles& mainStyles, KoGenChanges & changes, KWTextFrameSet* fs)
+QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver &embeddedSaver, KoGenStyles &mainStyles, KoGenChanges  &changes, KWTextFrameSet *fs)
 {
     QByteArray tag;
     switch (fs->textFrameSetType()) {
@@ -75,16 +75,16 @@ QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver& embeddedS
     Q_ASSERT(shapedata);
 
     writer.startElement(tag);
-    shapedata->saveOdf(context, 0, -1);
+    shapedata->saveOdf(context, m_document->documentRdfBase());
     writer.endElement();
 
     return content;
 }
 
 // rename to save pages ?
-void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGenStyles& mainStyles, KoGenChanges& changes)
+void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver &embeddedSaver, KoGenStyles &mainStyles, KoGenChanges &changes)
 {
-    //kDebug(32001 )<< "START saveHeaderFooter ############################################";
+    //kDebug(32001)<< "START saveHeaderFooter ############################################";
     // first get all the framesets in a nice quick-to-access data structure
     // this avoids iterating till we drop
     QHash<KWPageStyle, QHash<int, KWTextFrameSet*> > data;
@@ -107,10 +107,10 @@ void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGen
         if (data.contains(pageStyle))
             continue;
 
-        KoGenStyle masterStyle(KoGenStyle::StyleMaster);
+        KoGenStyle masterStyle(KoGenStyle::MasterPageStyle);
         KoGenStyle layoutStyle = pageStyle.saveOdf();
-        masterStyle.addProperty("style:page-layout-name", mainStyles.lookup(layoutStyle, "pm"));
-        QString name = mainStyles.lookup(masterStyle, pageStyle.name(), KoGenStyles::DontForceNumbering);
+        masterStyle.addProperty("style:page-layout-name", mainStyles.insert(layoutStyle, "pm"));
+        QString name = mainStyles.insert(masterStyle, pageStyle.name(), KoGenStyles::DontAddNumberToName);
         m_masterPages.insert(pageStyle, name);
     }
 
@@ -122,10 +122,10 @@ void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGen
           << KWord::EvenPagesFooterTextFrameSet;
 
     foreach (KWPageStyle pageStyle, data.keys()) {
-        KoGenStyle masterStyle(KoGenStyle::StyleMaster);
+        KoGenStyle masterStyle(KoGenStyle::MasterPageStyle);
         //masterStyle.setAutoStyleInStylesDotXml(true);
         KoGenStyle layoutStyle = pageStyle.saveOdf();
-        masterStyle.addProperty("style:page-layout-name", mainStyles.lookup(layoutStyle, "pm"));
+        masterStyle.addProperty("style:page-layout-name", mainStyles.insert(layoutStyle, "pm"));
 
         QHash<int, KWTextFrameSet*> headersAndFooters = data.value(pageStyle);
         int index = 0;
@@ -142,17 +142,17 @@ void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGen
             if (content.isNull())
                 continue;
 
-            masterStyle.addChildElement(QString::number(++index), content);
+            masterStyle.addChildElement(QString::number(++index), QString::fromUtf8(content));
         }
         // append the headerfooter-style to the main-style
         if (! masterStyle.isEmpty()) {
-            QString name = mainStyles.lookup(masterStyle, pageStyle.name(), KoGenStyles::DontForceNumbering);
+            QString name = mainStyles.insert(masterStyle, pageStyle.name(), KoGenStyles::DontAddNumberToName);
             m_masterPages.insert(pageStyle, name);
         }
     }
 
-    //foreach (KoGenStyles::NamedStyle s, mainStyles.styles(KoGenStyle::StyleAuto))
-    //    mainStyles.markStyleForStylesXml( s.name );
+    //foreach (KoGenStyles::NamedStyle s, mainStyles.styles(KoGenStyle::ParagraphAutoStyle))
+    //    mainStyles.markStyleForStylesXml(s.name);
 
     //kDebug(32001) << "END saveHeaderFooter ############################################";
 }
@@ -169,11 +169,11 @@ KWOdfWriter::~KWOdfWriter()
 }
 
 // 1.6: KWDocument::saveOasisHelper()
-bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & embeddedSaver)
+bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embeddedSaver)
 {
     //kDebug(32001) << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!";
 
-    KoStore * store = odfStore.store();
+    KoStore *store = odfStore.store();
 
     if (! store->open("settings.xml")) {
         return false;
@@ -184,11 +184,11 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
     if (!store->close())
         return false;
 
-    KoXmlWriter * manifestWriter = odfStore.manifestWriter();
+    KoXmlWriter *manifestWriter = odfStore.manifestWriter();
 
     manifestWriter->addManifestEntry("settings.xml", "text/xml");
 
-    KoXmlWriter* contentWriter = odfStore.contentWriter();
+    KoXmlWriter *contentWriter = odfStore.contentWriter();
     if (!contentWriter)
         return false;
 
@@ -209,7 +209,7 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
     KoGenChanges changes;
 
     // Save the named styles
-    KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>(m_document->dataCenterMap()["StyleManager"]);
+    KoStyleManager *styleManager = m_document->resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
     styleManager->saveOdf(mainStyles);
 
     // TODO get the pagestyle for the first page and store that as 'style:default-page-layout'
@@ -240,8 +240,13 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
         //  3) frames that are not anchored but freely positioned somewhere on the page.
         //     in ODF terms those frames are page-anchored.
 
-        if (fs->frameCount() == 1 && fs->frames().first()->shape()->parent())
-            continue; // is a frame that is anchored to text, don't save those here.
+        if (fs->frameCount() == 1) {
+            KoShape *shape = fs->frames().first()->shape();
+            // may be a frame that is anchored to text, don't save those here.
+            // but first check since clipped shapes look similar, but are not anchored to text
+            if (shape->parent() && !shape->parent()->isClipped(shape))
+                continue;
+        }
 
         KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
         if (tfs) {
@@ -254,7 +259,6 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
             }
         }
 
-        KWFrame *lastNonCopyFrame = 0;
         int counter = 1;
         QSet<QString> uniqueNames;
         foreach (KWFrame *frame, fs->frames()) { // make sure all shapes have names.
@@ -268,95 +272,14 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
         const QList<KWFrame*> frames = fs->frames();
         for (int i = 0; i < frames.count(); ++i) {
             KWFrame *frame = frames.at(i);
-            KoShape * shape = frame->shape();
-            // frame properties first
-            shape->setAdditionalStyleAttribute("fo:margin", QString::number(frame->runAroundDistance()) + "pt");
-            shape->setAdditionalStyleAttribute("style:horizontal-pos", "from-left");
-            shape->setAdditionalStyleAttribute("style:horizontal-rel", "page");
-            shape->setAdditionalStyleAttribute("style:vertical-pos", "from-top");
-            shape->setAdditionalStyleAttribute("style:vertical-rel", "page");
-            QString value;
-            switch (frame->textRunAround()) {
-            case KWord::RunAround:
-                switch (frame->runAroundSide()) {
-                case KWord::BiggestRunAroundSide: value = "biggest"; break;
-                case KWord::LeftRunAroundSide: value = "left"; break;
-                case KWord::RightRunAroundSide: value = "right"; break;
-                case KWord::AutoRunAroundSide: value = "dynamic"; break;
-                case KWord::BothRunAroundSide: value = "parallel"; break;
-                }
-                break;
-            case KWord::RunThrough:
-                value = "run-through";
-                break;
-            case KWord::NoRunAround:
-                value = "none";
-                break;
-            }
-            shape->setAdditionalStyleAttribute("style:wrap", value);
-
-            switch (frame->frameBehavior()) {
-            case KWord::AutoCreateNewFrameBehavior:
-                value = "auto-create-new-frame";
-                break;
-            case KWord::IgnoreContentFrameBehavior:
-                value = "clip";
-                break;
-            case KWord::AutoExtendFrameBehavior:
-                // the third case, AutoExtendFrame is handled by min-height
-                value.clear();
-                KWTextFrame *tf = dynamic_cast<KWTextFrame*>(frame);
-                if (tf && tf->minimumFrameHeight() > 1)
-                    shape->setAdditionalAttribute("fo:min-height", QString::number(tf->minimumFrameHeight()) + "pt");
-                break;
-            }
-            if (!value.isEmpty())
-                shape->setAdditionalStyleAttribute("style:overflow-behavior", value);
-
-            if (frame->frameBehavior() != KWord::IgnoreContentFrameBehavior) {
-                switch (frame->newFrameBehavior()) {
-                case KWord::ReconnectNewFrame: value = "followup"; break;
-                case KWord::NoFollowupFrame: value.clear(); break; // "none" is the default
-                case KWord::CopyNewFrame: value = "copy"; break;
-                }
-                if (!value.isEmpty()) {
-                    shape->setAdditionalStyleAttribute("koffice:frame-behavior-on-new-page", value);
-                    if (! frame->frameOnBothSheets())
-                        shape->setAdditionalAttribute("koffice:frame-copy-to-facing-pages", "true");
-                }
-            }
-
-            if (frame->isCopy()) {
-                Q_ASSERT(lastNonCopyFrame);
-                shape->setAdditionalAttribute("draw:copy-of", lastNonCopyFrame->shape()->name());
-            } else {
-                lastNonCopyFrame = frame;
-            }
-
-            // shape properties
-            KWPage page = m_document->pageManager()->page(shape);
-            const qreal pagePos = page.offsetInDocument();
-
-            const int effectiveZIndex = shape->zIndex() + m_zIndexOffsets.value(page);
-            shape->setAdditionalAttribute("draw:z-index", QString::number(effectiveZIndex));
-            shape->setAdditionalAttribute("text:anchor-type", "page");
-            shape->setAdditionalAttribute("text:anchor-page-number", QString::number(page.pageNumber()));
-            context.addShapeOffset(shape, QMatrix(1, 0, 0 , 1, 0, -pagePos));
-            shape->saveOdf(context);
-            context.removeShapeOffset(shape);
-            shape->removeAdditionalAttribute("draw:copy-of");
-            shape->removeAdditionalAttribute("draw:z-index");
-            shape->removeAdditionalAttribute("fo:min-height");
-            shape->removeAdditionalAttribute("koffice:frame-copy-to-facing-pages");
-            shape->removeAdditionalAttribute("text:anchor-page-number");
-            shape->removeAdditionalAttribute("text:anchor-page-number");
-            shape->removeAdditionalAttribute("text:anchor-type");
+            KWPage page = m_document->pageManager()->page(frame->shape());
+            frame->saveOdf(context, page, m_zIndexOffsets.value(page));
         }
     }
 
     if (mainTextFrame) {
         if (! mainTextFrame->frames().isEmpty() && mainTextFrame->frames().first()) {
-            KoTextShapeData * shapeData = qobject_cast<KoTextShapeData *>(mainTextFrame->frames().first()->shape()->userData());
+            KoTextShapeData *shapeData = qobject_cast<KoTextShapeData *>(mainTextFrame->frames().first()->shape()->userData());
             if (shapeData) {
                 KWPageManager *pm = m_document->pageManager();
                 if (pm->pageCount()) { // make the first page refer to our page master
@@ -366,13 +289,13 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
                     tbf.setProperty(KoParagraphStyle::MasterPageName, m_masterPages.value(style));
                     cursor.mergeBlockFormat(tbf);
                 }
-                shapeData->saveOdf(context);
+                shapeData->saveOdf(context, m_document->documentRdfBase());
             }
         }
     }
 
     //we save the changes before starting the page sequence element because odf validator insist on having <tracked-changes> right after the <office:text> tag
-    mainStyles.saveOdfAutomaticStyles(contentWriter, false);
+    mainStyles.saveOdfStyles(KoGenStyles::DocumentAutomaticStyles, contentWriter);
 
     changes.saveOdfChanges(changeWriter);
 
@@ -382,14 +305,20 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
     tmpChangeFile.close();
     bodyWriter->addCompleteElement(&tmpChangeFile);
 
-    bodyWriter->startElement("text:page-sequence");
-    foreach (KWPage page, m_document->pageManager()->pages()) {
-        Q_ASSERT(m_masterPages.contains(page.pageStyle()));
-        bodyWriter->startElement("text:page");
-        bodyWriter->addAttribute("text:master-page-name", m_masterPages.value(page.pageStyle()));
-        bodyWriter->endElement(); // text:page
+    // Do not write out text:page-sequence, if there is a maintTextFrame
+    // The ODF specification does not allow text:page-sequence in office:text
+    // if there is e.g. text:p or text:h there
+    if (!mainTextFrame) {
+        bodyWriter->startElement("text:page-sequence");
+        foreach (KWPage page, m_document->pageManager()->pages()) {
+            Q_ASSERT(m_masterPages.contains(page.pageStyle()));
+            bodyWriter->startElement("text:page");
+            bodyWriter->addAttribute("text:master-page-name",
+                    m_masterPages.value(page.pageStyle()));
+            bodyWriter->endElement(); // text:page
+        }
+        bodyWriter->endElement(); // text:page-sequence
     }
-    bodyWriter->endElement(); // text:page-sequence
 
     delete tmpBodyWriter;
     tmpBodyWriter = 0;
@@ -405,6 +334,12 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
     // add manifest line for content.xml
     manifestWriter->addManifestEntry("content.xml", "text/xml");
 
+    // update references to xml:id to be to new xml:id
+    // in the external Rdf
+    if (KoDocumentRdfBase *rdf = m_document->documentRdfBase()) {
+        QMap<QString, QString> m = sharedData->getRdfIdMapping();
+        rdf->updateXmlIdReferences(m);
+    }
     // save the styles.xml
     if (!mainStyles.saveOdfStylesDotXml(store, manifestWriter))
         return false;
@@ -419,7 +354,7 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
 bool KWOdfWriter::saveOdfSettings(KoStore *store)
 {
     KoStoreDevice settingsDev(store);
-    KoXmlWriter * settingsWriter = KoOdfWriteStore::createOasisXmlWriter(&settingsDev, "office:document-settings");
+    KoXmlWriter *settingsWriter = KoOdfWriteStore::createOasisXmlWriter(&settingsDev, "office:document-settings");
 
     // add this so that OOo reads guides lines and grid data from ooo:view-settings
     settingsWriter->addAttribute("xmlns:ooo", "http://openoffice.org/2004/office");
@@ -459,7 +394,7 @@ void KWOdfWriter::calculateZindexOffsets()
 {
     Q_ASSERT(m_zIndexOffsets.isEmpty()); // call this method only once, please.
     foreach (KWFrameSet *fs, m_document->frameSets()) {
-        if (KWord::isAutoGenerated(dynamic_cast<KWTextFrameSet*>(fs)))
+        if (KWord::isAutoGenerated(fs))
             continue;
         foreach (KWFrame *frame, fs->frames()) {
             addShapeToTree(frame->shape());
@@ -481,9 +416,9 @@ void KWOdfWriter::addShapeToTree(KoShape *shape)
         m_shapeTree.insert(shape->boundingRect(), shape);
 
     // add the children of a KoShapeContainer
-    KoShapeContainer* container = dynamic_cast<KoShapeContainer*>(shape);
+    KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(shape);
     if (container) {
-        foreach(KoShape* containerShape, container->childShapes()) {
+        foreach(KoShape *containerShape, container->shapes()) {
             addShapeToTree(containerShape);
         }
     }
