@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
- * Copyright (C) 2008-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2008-2010 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,14 +26,19 @@
 #include <QMap>
 
 class QPoint;
-class KStatusBar;
-class KWView;
-class QLabel;
-class KSqueezedTextLabel;
-class KoCanvasController;
-class KWDocument;
-class KWCanvas;
 class QAction;
+class QLabel;
+
+class KStatusBar;
+class KSqueezedTextLabel;
+
+class KoCanvasController;
+class KoCanvasControllerProxyObject;
+class KoCanvasBase;
+
+class KWDocument;
+class KWView;
+
 
 /**
  * The KWStatusBar class implements an extended statusbar for KWord.
@@ -49,10 +55,8 @@ public:
 
     static void addViewControls(KStatusBar *statusBar, KWView *view);
 
-    void addView(KWView *view);
-
 public slots:
-    void setText(const QString& text);
+    void setText(const QString &text);
 
 private slots:
     void setModified(bool modified);
@@ -65,6 +69,7 @@ private slots:
     void showModified(bool visible);
     void showMouse(bool visible);
     void showZoom(bool visible);
+    void removeView(QObject *);
 
 private:
     /**
@@ -75,15 +80,14 @@ private:
     * \param view The KWord view instance the statusbar belongs to. Each
     * KWStatusBar instance belongs to exactly one view.
     */
-    KWStatusBar(KStatusBar* statusBar, KWView* view);
+    KWStatusBar(KStatusBar *statusBar, KWView* view);
 
-    void setCurrentCanvas(KWCanvas *view);
+    void setCurrentView(KWView *view);
 
     KStatusBar *m_statusbar;
     QPointer<KWView> m_currentView;
     QMap<KWView*, QWidget*> m_zoomWidgets;
-    KWDocument *const m_document;
-    QPointer<KoCanvasController> m_controller;
+    QPointer<KoCanvasControllerProxyObject> m_controller;
     int m_currentPageNumber;
     QAction *m_zoomAction;
 
