@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
- * Copyright (C) 2005-2007, 2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2005-2007, 2009, 2010 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -35,9 +36,12 @@ class KWCanvas;
 class KWFrame;
 class KWGui;
 
+class KoCanvasBase;
 class KoZoomController;
-class KToggleAction;
+class KoRdfSemanticItem;
+class KActionMenu;
 
+class KToggleAction;
 /**
  * KWords view class. Following the broad model-view-controller idea this class
  * shows you one view on the document. There can be multiple views of the same document each
@@ -57,7 +61,7 @@ public:
      * @param document the document we show.
      * @param parent a parent widget we show ourselves in.
      */
-    KWView(const QString& viewMode, KWDocument *document, QWidget *parent);
+    KWView(const QString &viewMode, KWDocument *document, QWidget *parent);
     virtual ~KWView();
 
     /**
@@ -82,7 +86,7 @@ public:
     /**
      * Return the current canvas; much like canvas(), but this one does not downcast.
      */
-    KWCanvas *kwcanvas() const;
+    KoCanvasBase *canvasBase() const;
 
     /// Return the view converter for this view.
     KoViewConverter *viewConverter() {
@@ -95,10 +99,14 @@ public:
     const KWPage currentPage() const;
     void setCurrentPage(const KWPage &page);
 
+    /// go to page
+    void goToPage(const KWPage &page);
+
     virtual KoZoomController *zoomController() const { return m_zoomController; }
 
 public slots:
     void offsetInDocumentMoved(int yOffset);
+    void variableChanged();
 
 protected:
     /// reimplemented method from superclass
@@ -149,6 +157,8 @@ private slots:
     void toggleViewFrameBorders(bool on);
     /// displays the KWPageSettingsDialog that allows to change properties of the entire page
     void formatPage();
+    /// displays libs/main/rdf/SemanticStylesheetsEditor to edit Rdf stylesheets
+    void editSemanticStylesheets();
     /// convert current frame to an inline frame
     void inlineFrame();
     /// called if the zoom changed
@@ -163,6 +173,8 @@ private slots:
     void showStatusBar(bool);
     /// delete the current page
     void deletePage();
+    /// insert a new page
+    void insertPage();
     /// toggle the display of non-printing characters
     void setShowFormattingChars(bool on);
     /// selects all frames
@@ -171,12 +183,25 @@ private slots:
     void editDeleteSelection();
     /// create a KWOutlineShape for the selected frame(s).
     void createCustomOutline();
+    /// Wrap the selected frames into a clipping shape container.
+    void createFrameClipping();
+    /// unwrap the selected frames into a clipping shape container.
+    void removeFrameClipping();
     /** decide if we enable or disable the action "delete_page" uppon m_document->page_count() */
     void handleDeletePageAction();
     /// set the status of the show-statusbar action to reflect the current setting.
     void updateStatusBarAction();
     /// insert image
     void insertImage();
+    /// show guides menu option uses this
+    void setGuideVisibility(bool on);
+    /// go to previous page
+    void goToPreviousPage();
+    /// go to next page
+    void goToNextPage();
+    /// A semantic item was updated and should have it's text refreshed.
+    void semanticObjectViewSiteUpdated(KoRdfSemanticItem *item, const QString &xmlid);
+    void createTextOnShape();
 
 private:
     /// helper method for the raiseFrame/lowerFrame/bringToFront/sendToBack methods
@@ -210,6 +235,8 @@ private:
     KToggleAction *m_actionViewFooter;
     KToggleAction *m_actionViewSnapToGrid;
 
+    KActionMenu* m_actionMenu;
+
     bool m_snapToGrid;
 };
 
