@@ -62,15 +62,17 @@ QList<KWViewMode::ViewMap> KWViewModeNormal::clipRectToDocument(const QRect &vie
     while (index > 1) { // 1 since we might hit a pagespread in the binary search, so start one page early
         page = page.next();
         --index;
+        if (page.pageSide() == KWPage::PageSpread)
+            --index;
     }
 
     int emptyPages = 0;
     while (page.isValid()) {
 #ifndef NDEBUG
         if (page.pageNumber() - pageOffset >= m_pageTops.count()) {
-            kWarning(32003) << "KWViewModeNormal ERROR; pagemanager has more pages than viewmode ("
-            << m_pageManager->pageCount() << ">" << m_pageTops.count()
-            << "). Make sure you add pages via the document!";
+            kWarning(32003) << "ERROR; pagemanager has more pages than viewmode ("
+                << m_pageManager->pageCount() << ">" << m_pageTops.count()
+                << "). Make sure you add pages via the document!";
             break;
         }
 #endif
@@ -191,6 +193,7 @@ QPointF KWViewModeNormal::documentToView(const QPointF & point) const
     QPointF offsetInPage(point.x(),  + point.y() - page.offsetInDocument());
     Q_ASSERT(pageIndex >= 0 && pageIndex < m_pageTops.count());
     QPointF translated(x, m_pageTops[pageIndex]);
+    Q_ASSERT(m_viewConverter);
     return m_viewConverter->documentToView(translated + offsetInPage);
 }
 
