@@ -22,8 +22,9 @@
 
 #define TextShape_SHAPEID "TextShapeID"
 
-#include <KoCanvasResourceProvider.h>
+#include <KoResourceManager.h>
 
+class KWFrameSet;
 class KWTextFrameSet;
 
 /// The kword-global namespace for all KWord related things.
@@ -61,11 +62,17 @@ namespace KWord
         RunThrough  ///< The text will completely ignore the frame and layout as if it was not there
     };
 
+    enum FrameSetType {
+        BackgroundFrameSet,
+        TextFrameSet,
+        OtherFrameSet
+    };
+
     /**
      * Returns true if the passed frameset is a known type that the frameLayout auto-creates.
      * @param fs the frameset to check. 0 is allowed.
      */
-    bool isAutoGenerated(KWTextFrameSet *fs);
+    bool isAutoGenerated(KWFrameSet *fs);
     /**
      * Returns true if the passed frameset is a header or a footer frameset.
      * @param fs the frameset to check. 0 is allowed.
@@ -96,7 +103,7 @@ namespace KWord
         FootNoteSeparatorRight  ///< Right of page
     };
 
-    /// Specifies the KWord specific resources that you can put and get from the KoCanvasResourceProvider
+    /// Specifies the KWord specific resources that you can put and get from the KoResourceManager
     enum KWordCanvasResources {
         // lets try to put Current in front of everything for ease of reading.
         CurrentFrame = KoCanvasResource::KWordStart,
