@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2007, 2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,24 +24,23 @@
 #include "KWPageManager.h"
 #include "KWPage.h"
 #include "KWView.h"
-#include "KWCanvas.h"
 #include "frames/KWTextFrameSet.h"
 #include "frames/KWFrame.h"
 
 #include <KoInsets.h>
 #include <KoShapeManager.h>
+#include "KoCanvasBase.h"
 
 #include <QApplication>
 #include <QTextBlock>
 #include <QTextLayout>
 #include <QTextDocument>
 
-KWPrintingDialog::KWPrintingDialog(KWView *view)
-        : KoPrintingDialog(view),
-        m_document(view->kwdocument()),
-        m_clipToPage(false)
+KWPrintingDialog::KWPrintingDialog(KWDocument *document, KoShapeManager *shapeManager, QWidget *parent)
+        : KoPrintingDialog(parent),
+        m_document(document)
 {
-    setShapeManager(view->kwcanvas()->shapeManager());
+    setShapeManager(shapeManager);
 
     while (! m_document->layoutFinishedAtleastOnce()) {
         QCoreApplication::processEvents();
@@ -72,37 +72,34 @@ KWPrintingDialog::~KWPrintingDialog()
 QRectF KWPrintingDialog::preparePage(int pageNumber)
 {
     const int resolution = printer().resolution();
-    KoInsets bleed = m_document->pageManager()->padding();
-    const int bleedOffset = (int)(m_clipToPage ? 0 : POINT_TO_INCH(-bleed.left * resolution));
-    const int bleedWidth = (int)(m_clipToPage ? 0 : POINT_TO_INCH((bleed.left + bleed.right) * resolution));
-    const int bleedHeigt = (int)(m_clipToPage ? 0 : POINT_TO_INCH((bleed.top + bleed.bottom) * resolution));
-
     KWPage page = m_document->pageManager()->page(pageNumber);
     if (! page.isValid())
         return QRectF();
-    QRectF pageRect = page.rect(pageNumber);
-    pageRect.adjust(-bleed.left, -bleed.top, bleed.right, bleed.bottom);
+    printer().setPaperSize(page.rect(pageNumber).size(), QPrinter::Point);
 
-    printer().setPaperSize(pageRect.size(), QPrinter::Point);
-    const qreal offsetInDocument = page.offsetInDocument();
-
-    const int pageOffset = qRound(POINT_TO_INCH(resolution * offsetInDocument));
+    KoInsets bleed = m_document->pageManager()->padding();
+    const int bleedOffsetX = qRound(POINT_TO_INCH(bleed.left * resolution));
+    const int bleedOffsetY = qRound(POINT_TO_INCH(bleed.top * resolution));
+    const int bleedWidth = qRound(POINT_TO_INCH((bleed.left + bleed.right) * resolution));
+    const int bleedHeight = qRound(POINT_TO_INCH((bleed.top + bleed.bottom) * resolution));
 
+    const int pageOffset = qRound(POINT_TO_INCH(resolution * page.offsetInDocument()));
     painter().translate(0, -pageOffset);
+
     qreal width = page.width();
-    int clipHeight = (int) POINT_TO_INCH(resolution * page.height());
+    const int clipHeight = (int) POINT_TO_INCH(resolution * page.height());
     int clipWidth = (int) POINT_TO_INCH(resolution * page.width());
-    int offset = bleedOffset;
+    int offsetX = -bleedOffsetX;
     if (page.pageSide() == KWPage::PageSpread) {
         width /= 2;
         clipWidth /= 2;
         if (pageNumber != page.pageNumber()) { // right side
-            offset += clipWidth;
+            offsetX += clipWidth;
             painter().translate(-clipWidth, 0);
         }
     }
 
-    return QRectF(offset, pageOffset, clipWidth + bleedWidth, clipHeight + bleedHeigt);
+    return QRectF(offsetX, pageOffset - bleedOffsetY, clipWidth + bleedWidth, clipHeight + bleedHeight);
 }
 
 QList<KoShape*> KWPrintingDialog::shapesOnPage(int pageNumber)
@@ -129,7 +126,8 @@ int KWPrintingDialog::documentFirstPage() const
 
 int KWPrintingDialog::documentLastPage() const
 {
-    return m_document->pageManager()->last().pageNumber();
+    KWPage lastPage = m_document->pageManager()->last();
+    return lastPage.pageNumber() + (lastPage.pageSide() == KWPage::PageSpread ? 1 : 0);
 }
 
 QAbstractPrintDialog::PrintDialogOptions KWPrintingDialog::printDialogOptions() const
