@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,7 +29,7 @@ KWStatisticsDialog::KWStatisticsDialog(KWView *parent)
     setCaption(i18n("Statistics"));
     setButtons(KDialog::Close);
     setDefaultButton(KDialog::Close);
-    setMainWidget(new KWStatistics(parent->kwcanvas()->resourceProvider(), parent->kwdocument(), 0, this));
+    setMainWidget(new KWStatistics(parent->canvasBase()->resourceManager(), parent->kwdocument(), 0, this));
 }
 
 KWStatisticsDialog::~KWStatisticsDialog()
