@@ -19,6 +19,8 @@
 #ifndef KWCOPYSHAPE_H
 #define KWCOPYSHAPE_H
 
+#include "kword_export.h"
+
 #include <KoShape.h>
 
 class KWPageManager;
@@ -29,7 +31,7 @@ class KWPageManager;
  * reused from the original, as passed on the constructor.  The shapes appearance
  * will be the same as the original, just in a different place in the document.
  */
-class KWCopyShape : public KoShape
+class KWORD_TEST_EXPORT KWCopyShape : public KoShape
 {
 public:
     /**
@@ -46,10 +48,11 @@ public:
     /// reimplemented from KoShape
     QPainterPath outline() const;
     /// reimplemented
-    virtual void saveOdf(KoShapeSavingContext & context) const;
+    virtual void saveOdf(KoShapeSavingContext &context) const;
     /// reimplemented
-    virtual bool loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context);
+    virtual bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context);
 
+    KoShape *original() {return m_original;}
 private:
     KoShape *m_original;
     const KWPageManager *m_pageManager;
