@@ -31,6 +31,7 @@ class KoTextAnchor;
 class KWFrameSet;
 class KoViewConverter;
 class KWOutlineShape;
+class KWPage;
 
 /**
  * This class represents a single frame.
@@ -124,9 +125,7 @@ public:
      * This property specifies how text from another textframe will behave when this frame
      * intersects with it.
      */
-    void setTextRunAround(KWord::TextRunAround runAround) {
-        m_runAround = runAround;
-    }
+    void setTextRunAround(KWord::TextRunAround runAround);
 
     /**
      * The space between this frames edge and the text when that text runs around this frame.
@@ -185,25 +184,19 @@ public:
      * Copy all the settings from the parameter frame and apply them to this frame.
      * @param frame the frame to use as original
      */
-    virtual void copySettings(const KWFrame *frame);
+    void copySettings(const KWFrame *frame);
 
     /**
      * Save the frame as ODF
      * @param context the context for saving.
      */
-    void saveOdf(KoShapeSavingContext & context);
+    void saveOdf(KoShapeSavingContext &context, const KWPage &page, int pageZIndexOffset = 0) const;
 
     KWOutlineShape *outlineShape() const {
         return m_outline;
     }
     void setOutlineShape(KWOutlineShape *outline);
 
-    void attachAnchor(KoTextAnchor *anchor);
-    KoTextAnchor *getAnchor();
-protected:
-    /// replace the current set shape with the argument one.
-    void setShape(KoShape *shape);
-
 private:
     KoShape *m_shape;
     KWord::FrameBehavior m_frameBehavior;
@@ -212,7 +205,6 @@ private:
     KWord::RunAroundSide m_runAroundSide;
     KWord::TextRunAround m_runAround;
     qreal m_runAroundDistance;
-    KoTextAnchor *m_anchor;
     // The page number is only used during loading.
     // It is set to the page number if the frame contains a page anchored frame.
     // In all other cases it is set to -1.
