@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
- * Copyright (C) 2008 Pierre Ducroquet <pinaraf@pinaraf.info>
+ * Copyright (C) 2006-2007, 2010 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2010 Ko Gmbh <casper.boemann@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,11 +22,12 @@
 #include "KWTextFrameSet.h"
 #include "KWTextFrame.h"
 #include "KWCopyShape.h"
-#include "KWDocument.h"
-#include "KWPage.h"
-#include "KWPageTextInfo.h"
-#include "frames/KWAnchorStrategy.h"
-#include "frames/KWOutlineShape.h"
+#include "../KWDocument.h"
+#include "../KWPage.h"
+#include "../KWPageTextInfo.h"
+#include "KWAnchorStrategy.h"
+#include "KWOutlineShape.h"
+#include "Outline.h"
 
 #include <KoTextShapeData.h>
 #include <KoShapeContainer.h>
@@ -39,6 +40,7 @@
 #include <QTextBlock>
 
 // #define DEBUG_TEXT
+// #define DEBUG_ANCHORS
 
 #ifdef DEBUG_TEXT
 #define TDEBUG kDebug(32002)
@@ -46,13 +48,13 @@
 #define TDEBUG if(0) kDebug(32002)
 #endif
 
+#ifdef DEBUG_ANCHORS
+#define ADEBUG kDebug(32002)
+#else
+#define ADEBUG if(0) kDebug(32002)
+#endif
+
 // helper methods
-static qreal xAtY(const QLineF &line, qreal y)
-{
-    if (line.dx() == 0)
-        return line.x1();
-    return line.x1() + (y - line.y1()) / line.dy() * line.dx();
-}
 
 static qreal yAtX(const QLineF &line, qreal x)
 {
@@ -70,7 +72,7 @@ static QList<QPointF> intersect(const QRectF &rect, const QLineF &line)
     // top edge
     if ((startOfLine.y() <= rect.top() && endOfLine.y() >= rect.top()) ||
             (startOfLine.y() >= rect.top() && endOfLine.y() <= rect.top())) {
-        qreal x = xAtY(line, rect.top());
+        qreal x = Outline::xAtY(line, rect.top());
         if (x >= rect.left() && x <= rect.right() && x)
             answer.append(QPointF(x, rect.top()));
     }
@@ -86,7 +88,7 @@ static QList<QPointF> intersect(const QRectF &rect, const QLineF &line)
     // bottom edge
     if ((startOfLine.y() <= rect.bottom() && endOfLine.y() >= rect.bottom()) ||
             (startOfLine.y() >= rect.bottom() && endOfLine.y() <= rect.bottom())) {
-        qreal x = xAtY(line, rect.bottom());
+        qreal x = Outline::xAtY(line, rect.bottom());
         if (x >= rect.left() && x <= rect.right())
             answer.append(QPointF(x, rect.bottom()));
     }
@@ -102,117 +104,6 @@ static QList<QPointF> intersect(const QRectF &rect, const QLineF &line)
     return answer;
 }
 
-// ----------------- Class that allows us with the runaround of QPainterPaths ----------------
-class Outline
-{
-public:
-    Outline(KWFrame *frame, const QMatrix &matrix) : m_side(None) {
-        init(matrix, frame->outlineShape() ? frame->outlineShape() : frame->shape(), frame->runAroundDistance());
-        if (frame->textRunAround() == KWord::NoRunAround)
-            m_side = Empty;
-        else {
-            if (frame->runAroundSide() == KWord::LeftRunAroundSide)
-                m_side = Right;
-            else if (frame->runAroundSide() == KWord::RightRunAroundSide)
-                m_side = Left;
-        }
-    }
-
-    Outline(KoShape *shape, const QMatrix &matrix) : m_side(None) {
-        init(matrix, shape, 0);
-    }
-
-    void init(const QMatrix &matrix, KoShape *shape, qreal distance) {
-        QPainterPath path =  matrix.map(shape->outline());
-        m_bounds = path.boundingRect();
-        if (distance >= 0.0) {
-            QMatrix grow = matrix;
-            grow.translate(m_bounds.width() / 2.0, m_bounds.height() / 2.0);
-            const qreal scaleX = (m_bounds.width() + distance) / m_bounds.width();
-            const qreal scaleY = (m_bounds.height() + distance) / m_bounds.height();
-            grow.scale(scaleX, scaleY);
-            grow.translate(-m_bounds.width() / 2.0, -m_bounds.height() / 2.0);
-
-            path =  grow.map(shape->outline());
-            // kDebug() <<"Grow" << distance <<", Before:" << m_bounds <<", after:" << path.boundingRect();
-            m_bounds = path.boundingRect();
-        }
-
-        QPolygonF poly = path.toFillPolygon();
-
-        QPointF prev = *(poly.begin());
-        foreach (const QPointF &vtx, poly) { //initialized edges
-            if (vtx.x() == prev.x() && vtx.y() == prev.y())
-                continue;
-            QLineF line;
-            if (prev.y() < vtx.y()) // Make sure the vector lines all point downwards.
-                line = QLineF(prev, vtx);
-            else
-                line = QLineF(vtx, prev);
-            m_edges.insert(line.y1(), line);
-            prev = vtx;
-        }
-    }
-
-    QRectF limit(const QRectF &content) {
-        if (m_side == Empty) {
-            if (content.intersects(m_bounds))
-                return QRectF();
-            return content;
-        }
-
-        if (m_side == None) { // first time for this text;
-            qreal insetLeft = m_bounds.right() - content.left();
-            qreal insetRight = content.right() - m_bounds.left();
-
-            if (insetLeft < insetRight)
-                m_side = Left;
-            else
-                m_side = Right;
-        }
-        if (!m_bounds.intersects(content))
-            return content;
-
-        // two points, as we are checking a rect, not a line.
-        qreal points[2] = { content.top(), content.bottom() };
-        QRectF answer = content;
-        for (int i = 0; i < 2; i++) {
-            const qreal y = points[i];
-            qreal x = m_side == Left ? answer.left() : answer.right();
-            bool first = true;
-            QMap<qreal, QLineF>::const_iterator iter = m_edges.constBegin();
-            for (;iter != m_edges.constEnd(); ++iter) {
-                QLineF line = iter.value();
-                if (line.y2() < y) // not a section that will intersect with ou Y yet
-                    continue;
-                if (line.y1() > y) // section is below our Y, so abort loop
-                    break;
-                if (qAbs(line.dy()) < 1E-10)  // horizontal lines don't concern us.
-                    continue;
-
-                qreal intersect = xAtY(iter.value(), y);
-                if (first) {
-                    x = intersect;
-                    first = false;
-                } else if ((m_side == Left && intersect > x) || (m_side == Right && intersect < x))
-                    x = intersect;
-            }
-            if (m_side == Left)
-                answer.setLeft(qMax(answer.left(), x));
-            else
-                answer.setRight(qMin(answer.right(), x));
-        }
-
-        return answer;
-    }
-
-private:
-    enum Side { None, Left, Right, Empty };
-    Side m_side;
-    QMultiMap<qreal, QLineF> m_edges; //sorted with y-coord
-    QRectF m_bounds;
-};
-
 class KWTextDocumentLayout::DummyShape : public KoShape
 {
 public:
@@ -239,7 +130,7 @@ KWTextDocumentLayout::KWTextDocumentLayout(KWTextFrameSet *frameSet)
         m_lastKnownFrameCount(0)
 {
     if (m_frameSet->frameCount()) {
-        KoTextShapeData *data = dynamic_cast<KoTextShapeData*>(m_frameSet->frames().first()->shape()->userData());
+        KoTextShapeData *data = qobject_cast<KoTextShapeData*>(m_frameSet->frames().first()->shape()->userData());
         if (data) { // reset layout.
             data->setEndPosition(-1);
             data->foul();
@@ -268,22 +159,20 @@ void KWTextDocumentLayout::relayout()
 {
     if (! m_frameSet->allowLayout())
         return;
-
     const QList<KWFrame*> frames = m_frameSet->frames();
     QList<KWFrame*> dirtyFrames = frames;
-    bool foundADirtyOne = false;
     KWFrame *firstDirtyFrame = 0;
     foreach (KWFrame *frame, frames) {
-        KoTextShapeData *data = dynamic_cast<KoTextShapeData*>(frame->shape()->userData());
+        KoTextShapeData *data = qobject_cast<KoTextShapeData*>(frame->shape()->userData());
         if (!firstDirtyFrame && data && data->isDirty())
             firstDirtyFrame = frame;
-        if (! firstDirtyFrame)
+        if (!firstDirtyFrame)
             dirtyFrames.removeAll(frame);
     }
 
-    qSort(m_frameSet->m_frames.begin(), m_frameSet->m_frames.end(), KWTextFrameSet::sortTextFrames); // make sure the ordering is proper
-
-    if (foundADirtyOne) {
+    // make sure the ordering is proper
+    m_frameSet->sortFrames();
+    if (firstDirtyFrame) {
         // if the dirty frame has been resorted to no longer be the first one, then we should
         // mark dirty any frame that were previously later in the flow, but are now before it.
         foreach (KWFrame *frame, frames) {
@@ -315,14 +204,20 @@ void KWTextDocumentLayout::positionInlineObject(QTextInlineObject item, int posi
             if (strategy->anchor() == anchor)
                 return;
         }
-        TDEBUG << "new anchor";
+        ADEBUG << "new anchor";
         m_newAnchors.append(new KWAnchorStrategy(anchor));
     }
 }
 
 void KWTextDocumentLayout::layout()
 {
-    TDEBUG << "starting layout pass";
+    TDEBUG << "starting layout pass" << ((void*)document())
+#ifdef DEBUG_ANCHORS
+     << "m_newAnchors" << m_newAnchors.count() << "m_activeAnchors" << m_activeAnchors.count();
+#else
+    ;
+#endif
+
     QList<Outline*> outlines;
     class End
     {
@@ -345,8 +240,22 @@ void KWTextDocumentLayout::layout()
     qreal bottomOfText = 0.0;
     bool newParagraph = true;
     bool requestFrameResize = false, firstParagraph = true;
+    int startOfBlock = -1; // the first position in a block (aka paragraph)
+    int startOfBlockText = -1; // the first position of text in a block. Will be different only if anchors preceed text.
     KoShape *currentShape = 0;
     while (m_state->shape) {
+        ADEBUG << "> loop.... layout has" << m_state->layout->lineCount() << "lines, we have" << m_activeAnchors.count() << "+" << m_newAnchors.count() <<"anchors";
+#ifndef DEBUG_ANCHORS
+        for (int i = 0; i < m_state->layout->lineCount(); ++i) {
+            QTextLine line = m_state->layout->lineAt(i);
+            ADEBUG << i << "]" << (line.isValid() ? QString("%1 - %2").arg(line.textStart()).arg(line.textLength()) : QString("invalid"));
+        }
+#endif
+        if (m_state->layout->lineCount() == 0 && startOfBlock != m_state->cursorPosition()) {  // new paragraph
+            startOfBlock = m_state->cursorPosition();
+            startOfBlockText = m_state->cursorPosition();
+        }
+
         class Line
         {
         public:
@@ -357,31 +266,64 @@ void KWTextDocumentLayout::layout()
                 return line.isValid();
             }
             void tryFit() {
-                QRectF rect(m_state->x(), m_state->y(), m_state->width(), 1.);
+                const qreal leftIndent = m_state->x();
+                const qreal width = m_state->width();
+                QRectF rect(leftIndent, m_state->y(), width, 1.);
                 line.setLineWidth(rect.width());
-                if (rect.width() <= 0. || line.textLength() == 0) { // margin so small that the text can't fit.
-                    line.setNumColumns(1);
+                if (rect.width() <= 0. || line.textLength() == 0) {
+                    // margin so small that the text can't fit.
+                    if (m_state->layout->lineCount() > 1
+                            || m_state->layout->text().length() > 0) // parag not empty
+                        line.setNumColumns(1); // allow at least one char.
                     line.setPosition(QPointF(rect.x(), rect.y()));
                     return;
                 }
                 rect = limit(rect);
 
+                qreal movedDown = 0;
+                qreal eligableTop = rect.top();
                 while (true) {
-                    if (m_state->numColumns() > 0)
-                        line.setNumColumns(m_state->numColumns());
-                    else
-                        line.setLineWidth(rect.width());
+                    if (m_state->numColumns() > 0) {
+                        line.setNumColumns(m_state->numColumns(),rect.width());
+                        line.setPosition(QPointF(line.x(), rect.y()));
+                    } else {
+                        qreal x = rect.x();
+                        qreal effectiveLineWidth = rect.width();
+                        if (leftIndent < x) { // limit moved the left edge, keep the indent.
+                            x += leftIndent;
+                            effectiveLineWidth -= leftIndent;
+                        }
+                        line.setLineWidth(effectiveLineWidth);
+                        line.setPosition(QPointF(x, rect.y()));
+                    }
                     rect.setHeight(line.height());
+
                     QRectF newLine = limit(rect);
-                    if (newLine.width() <= 0.)
-                        // TODO be more intelligent than just moving down 10 pt
-                        rect = QRectF(m_state->x(), rect.top() + 10, m_state->width(), rect.height());
-                    else if (qAbs(newLine.left() - rect.left()) < 1E-10 && qAbs(newLine.right() - rect.right()) < 1E-10)
+                    if (newLine.width() <= 0 || line.textLength() == 0
+                            || line.cursorToX(line.textStart()+1) > newLine.right()) {
+                        const int Move = 10;
+                        movedDown += Move;
+                        if (movedDown > m_state->shape->size().height() * 1.3) {
+                            // TODO the magic number of shape height needs some history
+                            // investigation here.
+                            rect = QRectF(leftIndent, eligableTop, width, rect.height());
+                            break;
+                        }
+                        if (newLine.width() > 0) {
+                            // at least one char fits
+                            eligableTop = rect.top();
+                        }
+
+                        rect = QRectF(leftIndent, rect.top() + Move, width, rect.height());
+                    } else if (qAbs(newLine.left() - rect.left()) < 1E-10 && qAbs(newLine.right()
+                                - rect.right()) < 1E-10) {
                         break;
-                    else
+                    } else if (newLine.isValid()) {
                         rect = newLine;
+                    } else {
+                        break;
+                    }
                 }
-                line.setPosition(QPointF(rect.x(), rect.y()));
             }
             void setOutlines(const QList<Outline*> &outlines) {
                 m_outlines = &outlines;
@@ -392,11 +334,13 @@ void KWTextDocumentLayout::layout()
             QRectF limit(const QRectF &rect) {
                 QRectF answer = rect;
                 foreach (Outline *outline, *m_outlines)
-                answer = outline->limit(answer);
+                    answer = outline->limit(answer);
+
                 return answer;
             }
             KoTextDocumentLayout::LayoutState *m_state;
             const QList<Outline*> *m_outlines;
+            qreal m_initialIndent;
         };
 
         if (m_state->shape != currentShape) { // next shape
@@ -422,8 +366,9 @@ void KWTextDocumentLayout::layout()
                         if (frame->outlineShape()) {
                             if (frame->outlineShape()->zIndex() <= currentShape->zIndex())
                                 continue;
-                        } else if (frame->shape()->zIndex() <= currentShape->zIndex())
+                        } else if (frame->shape()->zIndex() <= currentShape->zIndex()) {
                             continue;
+                        }
                         if (! bounds.intersects(frame->shape()->boundingRect()))
                             continue;
                         bool isChild = false;
@@ -435,7 +380,7 @@ void KWTextDocumentLayout::layout()
                         }
                         if (isChild)
                             continue;
-                        QMatrix matrix = (frame->outlineShape()
+                        QTransform matrix = (frame->outlineShape()
                                 ? frame->outlineShape()
                                 : frame->shape())->absoluteTransformation(0);
                         matrix = matrix * currentShape->absoluteTransformation(0).inverted();
@@ -446,7 +391,7 @@ void KWTextDocumentLayout::layout()
                 // set the page for the shape.
                 KWPage page = m_frameSet->pageManager()->page(currentShape);
                 Q_ASSERT(page.isValid());
-                KoTextShapeData *data = dynamic_cast<KoTextShapeData*>(currentShape->userData());
+                KoTextShapeData *data = qobject_cast<KoTextShapeData*>(currentShape->userData());
                 Q_ASSERT(data);
                 data->setPageDirection(page.directionHint());
                 data->setPage(new KWPageTextInfo(page));
@@ -456,34 +401,74 @@ void KWTextDocumentLayout::layout()
         // anchors might require us to do some layout again, give it the chance to 'do as it will'
         bool restartLine = false;
         foreach (KWAnchorStrategy *strategy, m_activeAnchors + m_newAnchors) {
-            TDEBUG << "checking anchor";
-            if (strategy->checkState(m_state)) {
-                TDEBUG << "  restarting line";
+            ADEBUG << "checking anchor";
+            QPointF old;
+            if (strategy->anchoredShape())
+                old = strategy->anchoredShape()->position();
+            if (strategy->checkState(m_state, startOfBlock, startOfBlockText, m_frameSet)) {
+                ADEBUG << "  restarting line";
                 restartLine = true;
-                break;
             }
+            if (strategy->anchoredShape() && old != strategy->anchoredShape()->position()) {
+                // refresh outlines in case the shape moved.
+                foreach (Outline *outline, outlines) {
+                    if (outline->shape() == strategy->anchoredShape()) {
+                        ADEBUG << "  refreshing outline";
+                        QTransform matrix = strategy->anchoredShape()->absoluteTransformation(0);
+                        matrix = matrix * currentShape->absoluteTransformation(0).inverted();
+                        matrix.translate(0, m_state->documentOffsetInShape());
+                        outline->changeMatrix(matrix);
+                        break;
+                    }
+                }
+            }
+            if (restartLine)
+                break;
         }
+
         if (restartLine)
             continue;
         foreach (KWAnchorStrategy *strategy, m_activeAnchors + m_newAnchors) {
+            ADEBUG << " + isFinished?"<< strategy->isFinished();
             if (strategy->isFinished() && strategy->anchor()->positionInDocument() < m_state->cursorPosition()) {
-                TDEBUG << "  is finished";
+                ADEBUG << "  is finished";
                 m_activeAnchors.removeAll(strategy);
                 m_newAnchors.removeAll(strategy);
                 delete strategy;
             }
         }
-
         foreach (KWAnchorStrategy *strategy, m_newAnchors) {
-            if (strategy->anchoredShape() != 0) {
-                QMatrix matrix = strategy->anchoredShape()->absoluteTransformation(0);
-                matrix = matrix * currentShape->absoluteTransformation(0).inverted();
-                matrix.translate(0, m_state->documentOffsetInShape());
-                outlines.append(new Outline(strategy->anchoredShape(), matrix));
+            ADEBUG << "  migrating strategy!";
+            if (strategy->anchoredShape()) {
+                KWFrame *frame = dynamic_cast<KWFrame*>(strategy->anchoredShape()->applicationData());
+                if (! frame || frame->textRunAround() != KWord::RunThrough) {
+                    QTransform matrix = strategy->anchoredShape()->absoluteTransformation(0);
+                    matrix = matrix * currentShape->absoluteTransformation(0).inverted();
+                    matrix.translate(0, m_state->documentOffsetInShape());
+                    outlines.append(new Outline(strategy->anchoredShape(), matrix));
+                 }
+                // if the anchor occupies the first character of our block,
+                // create one line for it. This can theoretically change currentshape=0
+                // midway through the loop causing crashes. However the textshape makes
+                // sure that a line like this always have height so infinite that it
+                // doesn't happen.
+                const int cursorPosition = m_state->cursorPosition();
+                if (cursorPosition == startOfBlockText && strategy->anchor()->positionInDocument() == cursorPosition && !strategy->anchor()->isPositionedInline()) {
+                    ADEBUG << "   creating line for anchor";
+                    QTextLine line = m_state->layout->createLine();
+                    line.setNumColumns(1);
+                    line.setPosition(QPointF(m_state->x(), m_state->y()));
+                    m_state->addLine(line);
+                    ++startOfBlockText;
+                }
+                restartLine = true;
+                ADEBUG << "adding child outline";
             }
             m_activeAnchors.append(strategy);
         }
         m_newAnchors.clear();
+        if (restartLine)
+            continue;
 
         Line line(m_state);
         if (!line.isValid()) { // end of parag
@@ -491,7 +476,7 @@ void KWTextDocumentLayout::layout()
             if (firstParagraph) {
                 // start counting after the resumed paragraph
                 firstParagraph = false;
-                endPos = posY + m_state->shape->size().height() * 2;
+                endPos = posY + m_state->shape->size().height() * 2.2;
             }
             bool moreText = m_state->nextParag();
             if (m_state->shape && m_state->y() > posY)
@@ -514,7 +499,7 @@ void KWTextDocumentLayout::layout()
                         iter--;
                         lastFrame = dynamic_cast<KWTextFrame*>(*iter);
                     } while (lastFrame == 0);
-                    KoTextShapeData *data = dynamic_cast<KoTextShapeData*>(lastFrame->shape()->userData());
+                    KoTextShapeData *data = qobject_cast<KoTextShapeData*>(lastFrame->shape()->userData());
                     Q_ASSERT(data);
                     qreal spaceLeft = lastFrame->shape()->size().height() - bottomOfText + data->documentOffset();
                     data->wipe();
@@ -525,6 +510,8 @@ void KWTextDocumentLayout::layout()
                 }
 
                 m_frameSet->layoutDone();
+                cleanupAnchors();
+                emit finishedLayout();
                 return; // done!
             } else if (m_state->shape == 0) {
                 TDEBUG << "encountered an 'end of page' break, we need an extra page to honor that!";
@@ -537,7 +524,7 @@ void KWTextDocumentLayout::layout()
             newParagraph = true;
             continue;
         }
-        if (m_state->interrupted() || (newParagraph && m_state->y() > endPos)) {
+        if (m_state->isInterrupted() || (newParagraph && m_state->y() > endPos)) {
             // enough for now. Try again later.
             TDEBUG << "schedule a next layout due to having done a layout of quite some space";
             scheduleLayoutWithoutInterrupt();
@@ -545,27 +532,37 @@ void KWTextDocumentLayout::layout()
         }
         newParagraph = false;
         line.setOutlines(outlines);
+        const int anchorCount = m_newAnchors.count();
         line.tryFit();
+        if (m_state->layout->lineCount() == 1 && anchorCount != m_newAnchors.count()) {
+            // start parag over so we can correctly take the just found anchors into account.
+            m_state->layout->endLayout();
+            m_state->layout->beginLayout();
+            continue;
+        }
 #ifdef DEBUG_TEXT
         if (line.line.isValid()) {
             QTextBlock b = document()->findBlock(m_state->cursorPosition());
             if (b.isValid()) {
                 TDEBUG << "fitted line" << b.text().mid(line.line.textStart(), line.line.textLength());
                 TDEBUG << "       1 @ " << line.line.position() << " from parag at pos " << b.position();
+                TDEBUG << "         y " << line.line.y() << " h" <<  line.line.height();
             }
         }
 #endif
 
         bottomOfText = line.line.y() + line.line.height();
-        if (bottomOfText > m_state->shape->size().height() && document()->blockCount() == 1) {
+        if (bottomOfText > m_state->shape->size().height() && document()->blockCount() == 1 && KWord::isHeaderFooter(m_frameSet)) {
+            TDEBUG << "requestMoreFrames" << (bottomOfText - m_state->shape->size().height());
             m_frameSet->requestMoreFrames(bottomOfText - m_state->shape->size().height());
+            cleanupAnchors();
             return;
         }
 
         while (m_state->addLine(line.line)) {
             if (m_state->shape == 0) { // no more shapes to put the text in!
                 TDEBUG << "no more shape for our text; bottom is" << m_state->y();
-                line.line.setPosition(QPointF(0, m_state->y() + 20));
+                line.line.setPosition(QPointF(0, m_state->y() + 10000)); // move it away from any place that we're likely going to paint
 
                 if (requestFrameResize) { // plenty more text, but first lets resize the shape.
                     TDEBUG << "  we need more space; we require at least:" << m_dummyShape->size().height();
@@ -581,9 +578,10 @@ void KWTextDocumentLayout::layout()
 
                 KWFrame *lastFrame = m_frameSet->frames().last();
                 if (lastFrame->frameBehavior() == KWord::IgnoreContentFrameBehavior
-                        || dynamic_cast<KWCopyShape*> (lastFrame)) {
+                        || dynamic_cast<KWCopyShape*>(lastFrame)) {
                     m_state->clearTillEnd();
                     m_frameSet->layoutDone();
+                    emit finishedLayout();
                     return; // done!
                 }
 
@@ -608,13 +606,13 @@ void KWTextDocumentLayout::layout()
                 if (list.count() > 0)
                     down2 = QLineF(down2.p1(), list.last());
                 const qreal maxFrameLength = qMin(down.length(), down2.length());
-                if (maxFrameLength <= currentShape->size().height()) {
+                if (qAbs(maxFrameLength - currentShape->size().height()) < 1) {
                     m_state->clearTillEnd();
                     TDEBUG << "  we need another page";
                     m_frameSet->requestMoreFrames(0); // new page, please.
                     return;
                 }
-                KoTextShapeData *data = dynamic_cast<KoTextShapeData*>(lastFrame->shape()->userData());
+                KoTextShapeData *data = qobject_cast<KoTextShapeData*>(lastFrame->shape()->userData());
                 Q_ASSERT(data);
 
                 m_dummyShape->setSize(QSizeF(currentShape->size().width(), maxFrameLength - currentShape->size().height()));
@@ -622,6 +620,7 @@ void KWTextDocumentLayout::layout()
                 if (! m_state->setFollowupShape(m_dummyShape)) { // if I can't render into a dummy shape
                     m_state->clearTillEnd();
                     m_frameSet->layoutDone();
+                    emit finishedLayout();
                     return; // done!
                 }
                 requestFrameResize = true;
@@ -637,15 +636,17 @@ void KWTextDocumentLayout::layout()
             }
 #endif
         }
-
-        QRectF repaintRect = line.line.rect();
-        repaintRect.moveTop(repaintRect.y() - m_state->docOffsetInShape());
-        repaintRect.setX(0.0); // just take full width since we can't force a repaint of
-        repaintRect.setWidth(m_state->shape->size().width()); // where lines were before layout.
-        m_state->shape->update(repaintRect);
     }
     if (requestFrameResize) {
         TDEBUG << "  requestFrameResize" << m_dummyShape->size().height();
         m_frameSet->requestMoreFrames(m_dummyShape->size().height());
     }
 }
+
+void KWTextDocumentLayout::cleanupAnchors()
+{
+    qDeleteAll(m_activeAnchors);
+    m_activeAnchors.clear();
+    qDeleteAll(m_newAnchors);
+    m_newAnchors.clear();
+}
