@@ -1,5 +1,5 @@
 /* This file is part of the KOffice project
- * Copyright (C) 2005,2008 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2005,2008,2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,6 +20,7 @@
 
 #include <KWPageManager.h>
 #include <KWPage.h>
+#include <KoColorBackground.h>
 
 #include <kdebug.h>
 
@@ -223,8 +224,6 @@ void TestPageManager::removePages()
     pageManager->removePage(page2);
     QCOMPARE(page1.pageNumber(), 1);
     QCOMPARE(page4.pageNumber(), 2);
-
-    /* todo: bool tryRemovingPages(); */
 }
 
 void TestPageManager::pageInfo()
@@ -277,21 +276,21 @@ void TestPageManager::pageInfo()
     QCOMPARE(pageManager->bottomOfPage(4), 1000.0);
 
     layout = pageManager->defaultPageStyle().pageLayout();
-    layout.top = 5;
-    layout.left = 6;
-    layout.bottom = 7;
-    layout.right = 8;
+    layout.topMargin = 5;
+    layout.leftMargin = 6;
+    layout.bottomMargin = 7;
+    layout.rightMargin = 8;
     pageManager->defaultPageStyle().setPageLayout(layout);
 
     layout = pageStylePage2.pageLayout();
-    layout.top = 9;
-    layout.left = 10;
-    layout.bottom = 11;
-    layout.right = 12;
+    layout.topMargin = 9;
+    layout.leftMargin = 10;
+    layout.bottomMargin = 11;
+    layout.rightMargin = 12;
     pageStylePage2.setPageLayout(layout);
 
     layout = page1l.pageStyle().pageLayout(); //layout is valid for page1l and page1r
-    layout.right = 14.0;
+    layout.rightMargin = 14.0;
     page1l.pageStyle().setPageLayout(layout);
     QCOMPARE(page1l.rightMargin(), 14.0);
     QCOMPARE(page1r.rightMargin(), 14.0);
@@ -329,8 +328,8 @@ void TestPageManager::pageInfo()
     QCOMPARE(page2.rightMargin(), 16.0);
     QCOMPARE(page2.leftMargin(), 17.0);
 
-    layout.left = 18;
-    layout.right = 19;
+    layout.leftMargin = 18;
+    layout.rightMargin = 19;
     pageStylePage2.setPageLayout(layout);
     QCOMPARE(page2.rightMargin(), 16.0);
     QCOMPARE(page2.leftMargin(), 17.0);
@@ -503,7 +502,7 @@ void TestPageManager::testSetPageStyle()
     KWPage page = manager.appendPage();
 
     KWPageStyle style("myStyle");
-    KoPageLayout layout = KoPageLayout::standardLayout();
+    KoPageLayout layout;
     layout.height = 100;
     style.setPageLayout(layout);
     page.setPageStyle(style);
@@ -533,9 +532,12 @@ void TestPageManager::testPageSpreadLayout()
     KoPageLayout layout = page.pageStyle().pageLayout();
     layout.width = 450;
     layout.height = 150;
-    page.setPageSide(KWPage::PageSpread);
+    QCOMPARE(page.pageNumber(), 1);
+    page.setPageSide(KWPage::PageSpread); // makes it page 2 (and 3)
+    QCOMPARE(page.pageNumber(), 2);
     page.pageStyle().setPageLayout(layout);
     KWPage page2 = manager.appendPage();
+    QCOMPARE(page2.pageNumber(), 4);
     page2.setDirectionHint(KoText::LeftRightTopBottom);
     layout.width = 200;
     page2.pageStyle().setPageLayout(layout);
@@ -545,6 +547,16 @@ void TestPageManager::testPageSpreadLayout()
 
     QCOMPARE(manager.pageNumber(QPointF(10, 200)), 4);
     QCOMPARE(manager.pageNumber(QPointF(10, 151)), 4);
+
+
+    KWPage four = page.next();
+    QCOMPARE(four.pageNumber(), 4);
+    KWPage invalid = four.next();
+    QVERIFY(!invalid.isValid());
+    QVERIFY(invalid.pageNumber() != 4);
+    KWPage copy = invalid;
+    QVERIFY(!copy.isValid());
+    QVERIFY(copy.pageNumber() != 4);
 }
 
 void TestPageManager::testInsertPage()
@@ -593,6 +605,8 @@ void TestPageManager::testPadding()
     QCOMPARE(padding2.right, padding.right);
 
     KoPageLayout lay;
+    QVERIFY(lay.leftMargin >= 0);
+    QVERIFY(lay.bindingSide == -1);
     lay.width = 100;
     lay.height = 50;
     KWPageStyle style("testStyle");
@@ -639,5 +653,118 @@ void TestPageManager::testPageOffset()
     QCOMPARE(page.offsetInDocument(), (qreal) 400 * 49);
 }
 
+void TestPageManager::testBackgroundRefCount()
+{
+    KWPageStyle ps1("test");
+    QVERIFY(ps1.background() == 0);
+    KoColorBackground *background = new KoColorBackground(QColor(Qt::red));
+    QVERIFY(background->ref());
+    QCOMPARE(background->useCount(), 1);
+
+    ps1.setBackground(background);
+    QCOMPARE(background->useCount(), 2);
+
+    {
+        KWPageStyle ps2("test2");
+        QCOMPARE(background->useCount(), 2);
+        ps2 = ps1;
+        QCOMPARE(background->useCount(), 2);
+    }
+    QCOMPARE(background->useCount(), 2);
+
+    ps1 = ps1;
+    QCOMPARE(background->useCount(), 2);
+    ps1.setBackground(0);
+    QCOMPARE(background->useCount(), 1);
+    delete background;
+}
+
+void TestPageManager::testAppendPageSpread()
+{
+    KWPageManager manager;
+
+    KWPageStyle style = manager.addPageStyle("pagestyle1");
+    KoPageLayout layout = style.pageLayout();
+    layout.leftMargin = -1;
+    layout.rightMargin = -1;
+    layout.pageEdge = 7;
+    layout.bindingSide = 13;
+    style.setPageLayout(layout);
+
+    KWPage page1 = manager.appendPage(style);
+    QCOMPARE(page1.pageNumber(), 1);
+    QCOMPARE(page1.pageSide(), KWPage::Right);
+    QCOMPARE(page1.width(), style.pageLayout().width);
+    KWPage test = page1.next();
+    QVERIFY(!test.isValid());
+
+    KWPage page2 = manager.appendPage(style);
+    QCOMPARE(page1.pageNumber(), 1);
+    QCOMPARE(page1.pageSide(), KWPage::Right);
+    QCOMPARE(page1.width(), style.pageLayout().width);
+    QCOMPARE(page2.pageNumber(), 2);
+    QCOMPARE(page2.pageSide(), KWPage::PageSpread);
+    QCOMPARE(page2.width(), style.pageLayout().width * 2);
+    test = page2.next();
+    QVERIFY(!test.isValid());
+
+    KWPage page3 = manager.insertPage(2, style);
+    QCOMPARE(page1.pageNumber(), 1);
+    QCOMPARE(page1.pageSide(), KWPage::Right);
+    QCOMPARE(page1.width(), style.pageLayout().width);
+    QCOMPARE(page2.pageNumber(), 4);
+    QCOMPARE(page2.pageSide(), KWPage::PageSpread);
+    QCOMPARE(page2.width(), style.pageLayout().width * 2);
+    QCOMPARE(page3.pageNumber(), 2);
+    QCOMPARE(page3.pageSide(), KWPage::PageSpread);
+    QCOMPARE(page3.width(), style.pageLayout().width * 2);
+    test = page2.next();
+    QVERIFY(!test.isValid());
+
+    KWPage realPage3 = manager.page(3);
+    QVERIFY(realPage3.isValid());
+    QCOMPARE(realPage3.pageNumber(), 2); // its still a pagespread
+}
+
+void TestPageManager::testRemovePageSpread()
+{
+    KWPageManager manager;
+
+    KoPageLayout layout = manager.defaultPageStyle().pageLayout();
+    layout.leftMargin = -1;
+    layout.rightMargin = -1;
+    layout.pageEdge = 7;
+    layout.bindingSide = 13;
+    manager.defaultPageStyle().setPageLayout(layout);
+
+    KWPage page1 = manager.appendPage();
+    KWPage pageSpread1 = manager.appendPage();
+    KWPage pageSpread2 = manager.appendPage();
+    QCOMPARE(pageSpread1.pageSide(), KWPage::PageSpread);
+    QCOMPARE(pageSpread2.pageSide(), KWPage::PageSpread);
+    QCOMPARE(manager.pageCount(), 5);
+
+    manager.removePage(pageSpread2); // remove from end
+    QVERIFY(pageSpread1.isValid());
+    QCOMPARE(pageSpread1.pageSide(), KWPage::PageSpread);
+    QCOMPARE(manager.pageCount(), 3);
+    QVERIFY(!pageSpread2.isValid());
+    QVERIFY(pageSpread2.pageNumber() < 0);
+
+    // re-add so we can remove something in the middle
+    pageSpread2 = manager.appendPage();
+    QCOMPARE(pageSpread1.pageSide(), KWPage::PageSpread);
+    QCOMPARE(pageSpread2.pageSide(), KWPage::PageSpread);
+    QCOMPARE(manager.pageCount(), 5);
+
+
+    manager.removePage(pageSpread1); // remove pages 2&3
+    QVERIFY(!pageSpread1.isValid());
+    QVERIFY(pageSpread1.pageNumber() < 0);
+    QCOMPARE(pageSpread2.pageSide(), KWPage::PageSpread);
+    QCOMPARE(manager.pageCount(), 3);
+    QVERIFY(pageSpread2.isValid());
+}
+
 QTEST_KDEMAIN(TestPageManager, GUI)
-#include "TestPageManager.moc"
+#include <TestPageManager.moc>
