@@ -49,7 +49,7 @@ KoLineBorder::KoLineBorder()
     d->pen.setWidthF(1.0);
 }
 
-KoLineBorder::KoLineBorder(const KoLineBorder & other)
+KoLineBorder::KoLineBorder(const KoLineBorder &other)
         : KoShapeBorderModel(), d(new Private())
 {
     d->color = other.d->color;
@@ -70,7 +70,7 @@ KoLineBorder::~KoLineBorder()
     delete d;
 }
 
-KoLineBorder& KoLineBorder::operator = (const KoLineBorder & rhs)
+KoLineBorder &KoLineBorder::operator = (const KoLineBorder &rhs)
 {
     if (this == &rhs)
         return *this;
@@ -82,7 +82,7 @@ KoLineBorder& KoLineBorder::operator = (const KoLineBorder & rhs)
     return *this;
 }
 
-void KoLineBorder::fillStyle(KoGenStyle &style, KoShapeSavingContext &context)
+void KoLineBorder::fillStyle(KoGenStyle &style, KoShapeSavingContext &context) const
 {
     QPen pen = d->pen;
     if (d->brush.gradient())
@@ -92,7 +92,7 @@ void KoLineBorder::fillStyle(KoGenStyle &style, KoShapeSavingContext &context)
     KoOdfGraphicStyles::saveOdfStrokeStyle(style, context.mainStyles(), pen);
 }
 
-void KoLineBorder::borderInsets(const KoShape *shape, KoInsets &insets)
+void KoLineBorder::borderInsets(const KoShape *shape, KoInsets &insets) const
 {
     Q_UNUSED(shape);
     qreal lineWidth = d->pen.widthF();
@@ -101,8 +101,8 @@ void KoLineBorder::borderInsets(const KoShape *shape, KoInsets &insets)
     lineWidth *= 0.5; // since we draw a line half inside, and half outside the object.
 
     // if we have square cap, we need a little more space
-    // -> sqrt( (0.5*penWidth)^2 + (0.5*penWidth)^2 )
-    if( capStyle() == Qt::SquareCap )
+    // -> sqrt((0.5*penWidth)^2 + (0.5*penWidth)^2)
+    if (capStyle() == Qt::SquareCap)
         lineWidth *= M_SQRT2;
 
     insets.top = lineWidth;
@@ -111,12 +111,12 @@ void KoLineBorder::borderInsets(const KoShape *shape, KoInsets &insets)
     insets.right = lineWidth;
 }
 
-bool KoLineBorder::hasTransparency()
+bool KoLineBorder::hasTransparency() const
 {
     return d->color.alpha() > 0;
 }
 
-void KoLineBorder::paintBorder(KoShape *shape, QPainter &painter, const KoViewConverter &converter)
+void KoLineBorder::paint(KoShape *shape, QPainter &painter, const KoViewConverter &converter)
 {
     KoShape::applyConversion(painter, converter);
 
@@ -127,18 +127,18 @@ void KoLineBorder::paintBorder(KoShape *shape, QPainter &painter, const KoViewCo
     else
         pen.setColor(d->color);
 
-    if(!pen.isCosmetic())
+    if (!pen.isCosmetic())
         painter.strokePath(shape->outline(), pen);
 }
 
-void KoLineBorder::paintBorder(KoShape *shape, QPainter &painter, const KoViewConverter &converter, const QColor & color )
+void KoLineBorder::paint(KoShape *shape, QPainter &painter, const KoViewConverter &converter, const QColor &color)
 {
     KoShape::applyConversion(painter, converter);
 
     QPen pen = d->pen;
     pen.setColor(color);
 
-    if(!pen.isCosmetic()) {
+    if (!pen.isCosmetic()) {
         painter.strokePath(shape->outline(), pen);
     }
 }
@@ -183,12 +183,12 @@ qreal KoLineBorder::miterLimit() const
     return d->pen.miterLimit();
 }
 
-const QColor & KoLineBorder::color() const
+QColor KoLineBorder::color() const
 {
     return d->color;
 }
 
-void KoLineBorder::setColor(const QColor & color)
+void KoLineBorder::setColor(const QColor &color)
 {
     d->color = color;
 }
@@ -221,7 +221,7 @@ qreal KoLineBorder::dashOffset() const
     return d->pen.dashOffset();
 }
 
-void KoLineBorder::setLineBrush(const QBrush & brush)
+void KoLineBorder::setLineBrush(const QBrush &brush)
 {
     d->brush = brush;
 }
