@@ -32,6 +32,7 @@
 #include "KoShapeBackground.h"
 #include "KoShapeContainer.h"
 #include "KoFilterEffectStack.h"
+#include "KoTextOnShapeContainer.h"
 
 #include <KoXmlReader.h>
 #include <KoXmlWriter.h>
@@ -44,12 +45,53 @@
 #include <KDebug>
 #include <QtGui/QPainter>
 
+#ifndef QT_NO_DEBUG
+#include <qnumeric.h> // for qIsNaN
+static bool qIsNaNPoint(const QPointF &p) {
+    return qIsNaN(p.x()) || qIsNaN(p.y());
+}
+#endif
+
 KoPathShapePrivate::KoPathShapePrivate(KoPathShape *q)
     : KoShapePrivate(q),
     fillRule(Qt::OddEvenFill)
 {
 }
 
+QRectF KoPathShapePrivate::handleRect(const QPointF &p, qreal radius) const
+{
+    return QRectF(p.x() - radius, p.y() - radius, 2*radius, 2*radius);
+}
+
+void KoPathShapePrivate::applyViewboxTransformation(const KoXmlElement &element)
+{
+    Q_Q(KoPathShape);
+    // apply viewbox transformation
+    QRectF viewBox = q->loadOdfViewbox(element);
+    if (! viewBox.isEmpty()) {
+        // load the desired size
+        QSizeF size;
+        size.setWidth(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "width", QString())));
+        size.setHeight(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "height", QString())));
+
+        // load the desired position
+        QPointF pos;
+        pos.setX(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "x", QString())));
+        pos.setY(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "y", QString())));
+
+        // create matrix to transform original path data into desired size and position
+        QTransform viewMatrix;
+        viewMatrix.translate(-viewBox.left(), -viewBox.top());
+        viewMatrix.scale(size.width() / viewBox.width(), size.height() / viewBox.height());
+        viewMatrix.translate(pos.x(), pos.y());
+
+        // transform the path data
+        map(viewMatrix);
+    }
+}
+
+
+/////////////////////////
 KoPathShape::KoPathShape()
     :KoShape(*(new KoPathShapePrivate(this)))
 {
@@ -67,18 +109,22 @@ KoPathShape::~KoPathShape()
 
 void KoPathShape::saveOdf(KoShapeSavingContext & context) const
 {
+    Q_D(const KoPathShape);
     context.xmlWriter().startElement("draw:path");
     saveOdfAttributes(context, OdfAllAttributes | OdfViewbox);
 
-    context.xmlWriter().addAttribute("svg:d", toString(QMatrix()));
-    context.xmlWriter().addAttribute("koffice:nodeTypes", nodeTypes());
+    context.xmlWriter().addAttribute("svg:d", toString());
+    context.xmlWriter().addAttribute("koffice:nodeTypes", d->nodeTypes());
 
     saveOdfCommonChildElements(context);
+    if (parent())
+        parent()->saveOdfChildElements(context);
     context.xmlWriter().endElement();
 }
 
 bool KoPathShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context)
 {
+    Q_D(KoPathShape);
     loadOdfAttributes(element, context, OdfMandatories | OdfAdditionalAttributes | OdfCommonChildElements);
 
     // first clear the path data from the default path
@@ -116,12 +162,12 @@ bool KoPathShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &c
     } else { // path loading
         KoPathShapeLoader loader(this);
         loader.parseSvg(element.attributeNS(KoXmlNS::svg, "d"), true);
-        loadNodeTypes(element);
+        d->loadNodeTypes(element);
     }
 
-    applyViewboxTransformation(element);
+    d->applyViewboxTransformation(element);
     QPointF pos = normalize();
-    setTransformation(QMatrix());
+    setTransformation(QTransform());
 
     if (element.hasAttributeNS(KoXmlNS::svg, "x") || element.hasAttributeNS(KoXmlNS::svg, "y")) {
         pos.setX(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "x", QString())));
@@ -131,6 +177,7 @@ bool KoPathShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &c
     setPosition(pos);
 
     loadOdfAttributes(element, context, OdfTransformation);
+    KoTextOnShapeContainer::tryWrapShape(this, element, context);
 
     return true;
 }
@@ -149,21 +196,12 @@ void KoPathShape::loadStyle(const KoXmlElement & element, KoShapeLoadingContext
     KoShape::loadStyle(element, context);
 
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
-    styleStack.save();
-
-    // fill the style stack with the shapes style
-    if (element.hasAttributeNS(KoXmlNS::draw, "style-name")) {
-        context.odfLoadingContext().fillStyleStack(element, KoXmlNS::draw, "style-name", "graphic");
-    } else if (element.hasAttributeNS(KoXmlNS::presentation, "style-name")) {
-        context.odfLoadingContext().fillStyleStack(element, KoXmlNS::presentation, "style-name", "presentation");
-    }
     styleStack.setTypeProperties("graphic");
 
     if (styleStack.hasProperty(KoXmlNS::svg, "fill-rule")) {
         QString rule = styleStack.property(KoXmlNS::svg, "fill-rule");
         d->fillRule = rule == "nonzero" ?  Qt::WindingFill : Qt::OddEvenFill;
     }
-    styleStack.restore();
 }
 
 QRectF KoPathShape::loadOdfViewbox(const KoXmlElement & element) const
@@ -183,32 +221,6 @@ QRectF KoPathShape::loadOdfViewbox(const KoXmlElement & element) const
     return viewbox;
 }
 
-void KoPathShape::applyViewboxTransformation(const KoXmlElement & element)
-{
-    // apply viewbox transformation
-    QRectF viewBox = loadOdfViewbox(element);
-    if (! viewBox.isEmpty()) {
-        // load the desired size
-        QSizeF size;
-        size.setWidth(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "width", QString())));
-        size.setHeight(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "height", QString())));
-
-        // load the desired position
-        QPointF pos;
-        pos.setX(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "x", QString())));
-        pos.setY(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "y", QString())));
-
-        // create matrix to transform original path data into desired size and position
-        QMatrix viewMatrix;
-        viewMatrix.translate(-viewBox.left(), -viewBox.top());
-        viewMatrix.scale(size.width() / viewBox.width(), size.height() / viewBox.height());
-        viewMatrix.translate(pos.x(), pos.y());
-
-        // transform the path data
-        map(viewMatrix);
-    }
-}
-
 void KoPathShape::clear()
 {
     foreach(KoSubpath *subpath, m_subpaths) {
@@ -228,19 +240,20 @@ void KoPathShape::paint(QPainter &painter, const KoViewConverter &converter)
 
     if (background())
         background()->paint(painter, path);
-    //paintDebug( painter );
+    //paintDebug(painter);
 }
 
 #ifndef NDEBUG
-void KoPathShape::paintDebug(QPainter &painter)
+void KoPathShapePrivate::paintDebug(QPainter &painter)
 {
-    KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
+    Q_Q(KoPathShape);
+    KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
     int i = 0;
 
     QPen pen(Qt::black);
     painter.save();
     painter.setPen(pen);
-    for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+    for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
         for (; it != (*pathIt)->constEnd(); ++it) {
             ++i;
@@ -269,10 +282,11 @@ void KoPathShape::paintDebug(QPainter &painter)
     kDebug(30006) << "nop =" << i;
 }
 
-void KoPathShape::debugPath()
+void KoPathShapePrivate::debugPath() const
 {
-    KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
-    for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+    Q_Q(const KoPathShape);
+    KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
+    for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
         for (; it != (*pathIt)->constEnd(); ++it) {
             kDebug(30006) << "p:" << (*pathIt) << "," << *it << "," << (*it)->point() << "," << (*it)->properties() << "," << (*it)->group();
@@ -283,11 +297,12 @@ void KoPathShape::debugPath()
 
 void KoPathShape::paintPoints(QPainter &painter, const KoViewConverter &converter, int handleRadius)
 {
+    Q_D(KoPathShape);
     applyConversion(painter, converter);
 
     KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
 
-    QRectF handle = converter.viewToDocument(handleRect(QPoint(0, 0), handleRadius));
+    QRectF handle = converter.viewToDocument(d->handleRect(QPoint(0, 0), handleRadius));
 
     for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
@@ -296,11 +311,6 @@ void KoPathShape::paintPoints(QPainter &painter, const KoViewConverter &converte
     }
 }
 
-QRectF KoPathShape::handleRect(const QPointF &p, qreal radius) const
-{
-    return QRectF(p.x() - radius, p.y() - radius, 2*radius, 2*radius);
-}
-
 QPainterPath KoPathShape::outline() const
 {
     QPainterPath path;
@@ -308,33 +318,43 @@ QPainterPath KoPathShape::outline() const
         KoPathPoint * lastPoint = subpath->first();
         bool activeCP = false;
         foreach(KoPathPoint * currPoint, *subpath) {
-            KoPathPoint::KoPointProperties currProperties = currPoint->properties();
+            KoPathPoint::PointProperties currProperties = currPoint->properties();
             if (currPoint == subpath->first()) {
                 if (currProperties & KoPathPoint::StartSubpath) {
+                    Q_ASSERT(!qIsNaNPoint(currPoint->point()));
                     path.moveTo(currPoint->point());
                 }
             } else if (activeCP && currPoint->activeControlPoint1()) {
+                Q_ASSERT(!qIsNaNPoint(lastPoint->controlPoint2()));
+                Q_ASSERT(!qIsNaNPoint(currPoint->controlPoint1()));
+                Q_ASSERT(!qIsNaNPoint(currPoint->point()));
                 path.cubicTo(
                     lastPoint->controlPoint2(),
                     currPoint->controlPoint1(),
                     currPoint->point());
-            } else if( activeCP || currPoint->activeControlPoint1()) {
+            } else if (activeCP || currPoint->activeControlPoint1()) {
+                Q_ASSERT(!qIsNaNPoint(lastPoint->controlPoint2()));
+                Q_ASSERT(!qIsNaNPoint(currPoint->controlPoint1()));
                 path.quadTo(
                     activeCP ? lastPoint->controlPoint2() : currPoint->controlPoint1(),
                     currPoint->point());
             } else {
+                Q_ASSERT(!qIsNaNPoint(currPoint->point()));
                 path.lineTo(currPoint->point());
             }
             if (currProperties & KoPathPoint::CloseSubpath && currProperties & KoPathPoint::StopSubpath) {
                 // add curve when there is a curve on the way to the first point
                 KoPathPoint * firstPoint = subpath->first();
+                Q_ASSERT(!qIsNaNPoint(firstPoint->point()));
                 if (currPoint->activeControlPoint2() && firstPoint->activeControlPoint1()) {
                     path.cubicTo(
                         currPoint->controlPoint2(),
                         firstPoint->controlPoint1(),
                         firstPoint->point());
                 }
-                else if(currPoint->activeControlPoint2() || firstPoint->activeControlPoint1()) {
+                else if (currPoint->activeControlPoint2() || firstPoint->activeControlPoint1()) {
+                    Q_ASSERT(!qIsNaNPoint(currPoint->point()));
+                    Q_ASSERT(!qIsNaNPoint(currPoint->controlPoint1()));
                     path.quadTo(
                         currPoint->activeControlPoint2() ? currPoint->controlPoint2() : firstPoint->controlPoint1(),
                         firstPoint->point());
@@ -355,7 +375,7 @@ QPainterPath KoPathShape::outline() const
 
 QRectF KoPathShape::boundingRect() const
 {
-    QMatrix transform = absoluteTransformation(0);
+    QTransform transform = absoluteTransformation(0);
     // calculate the bounding rect of the transformed outline
     QRectF bb(transform.map(outline()).boundingRect());
     if (border()) {
@@ -374,7 +394,7 @@ QRectF KoPathShape::boundingRect() const
     }
     if (shadow()) {
         KoInsets insets;
-        shadow()->insets(this, insets);
+        shadow()->insets(insets);
         bb.adjust(-insets.left, -insets.top, insets.right, insets.bottom);
     }
     if (filterEffectStack()) {
@@ -393,13 +413,14 @@ QSizeF KoPathShape::size() const
 
 void KoPathShape::setSize(const QSizeF &newSize)
 {
-    QMatrix matrix(resizeMatrix(newSize));
+    Q_D(KoPathShape);
+    QTransform matrix(resizeMatrix(newSize));
 
     KoShape::setSize(newSize);
-    map(matrix);
+    d->map(matrix);
 }
 
-QMatrix KoPathShape::resizeMatrix( const QSizeF & newSize ) const
+QTransform KoPathShape::resizeMatrix(const QSizeF & newSize) const
 {
     QSizeF oldSize = size();
     if (oldSize.width() == 0.0) {
@@ -417,7 +438,7 @@ QMatrix KoPathShape::resizeMatrix( const QSizeF & newSize ) const
         sizeNew.setHeight(0.000001);
     }
 
-    return QMatrix(sizeNew.width() / oldSize.width(), 0, 0, sizeNew.height() / oldSize.height(), 0, 0);
+    return QTransform(sizeNew.width() / oldSize.width(), 0, 0, sizeNew.height() / oldSize.height(), 0, 0);
 }
 
 KoPathPoint * KoPathShape::moveTo(const QPointF &p)
@@ -431,23 +452,25 @@ KoPathPoint * KoPathShape::moveTo(const QPointF &p)
 
 KoPathPoint * KoPathShape::lineTo(const QPointF &p)
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty()) {
         moveTo(QPointF(0, 0));
     }
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StopSubpath);
     KoPathPoint * lastPoint = m_subpaths.last()->last();
-    updateLast(&lastPoint);
+    d->updateLast(&lastPoint);
     m_subpaths.last()->push_back(point);
     return point;
 }
 
 KoPathPoint * KoPathShape::curveTo(const QPointF &c1, const QPointF &c2, const QPointF &p)
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty()) {
         moveTo(QPointF(0, 0));
     }
     KoPathPoint * lastPoint = m_subpaths.last()->last();
-    updateLast(&lastPoint);
+    d->updateLast(&lastPoint);
     lastPoint->setControlPoint2(c1);
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StopSubpath);
     point->setControlPoint1(c2);
@@ -457,11 +480,12 @@ KoPathPoint * KoPathShape::curveTo(const QPointF &c1, const QPointF &c2, const Q
 
 KoPathPoint * KoPathShape::curveTo(const QPointF &c, const QPointF &p)
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty())
         moveTo(QPointF(0, 0));
 
     KoPathPoint * lastPoint = m_subpaths.last()->last();
-    updateLast(&lastPoint);
+    d->updateLast(&lastPoint);
     lastPoint->setControlPoint2(c);
     KoPathPoint * point = new KoPathPoint(this, p, KoPathPoint::StopSubpath);
     m_subpaths.last()->push_back(point);
@@ -553,26 +577,29 @@ int KoPathShape::arcToCurve(qreal rx, qreal ry, qreal startAngle, qreal sweepAng
 
 void KoPathShape::close()
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty()) {
         return;
     }
-    closeSubpath(m_subpaths.last());
+    d->closeSubpath(m_subpaths.last());
 }
 
 void KoPathShape::closeMerge()
 {
+    Q_D(KoPathShape);
     if (m_subpaths.empty()) {
         return;
     }
-    closeMergeSubpath(m_subpaths.last());
+    d->closeMergeSubpath(m_subpaths.last());
 }
 
 QPointF KoPathShape::normalize()
 {
+    Q_D(KoPathShape);
     QPointF tl(outline().boundingRect().topLeft());
-    QMatrix matrix;
+    QTransform matrix;
     matrix.translate(-tl.x(), -tl.y());
-    map(matrix);
+    d->map(matrix);
 
     // keep the top left point of the object
     applyTransformation(matrix.inverted());
@@ -580,10 +607,11 @@ QPointF KoPathShape::normalize()
     return tl;
 }
 
-void KoPathShape::map(const QMatrix &matrix)
+void KoPathShapePrivate::map(const QTransform &matrix)
 {
-    KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
-    for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+    Q_Q(KoPathShape);
+    KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
+    for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
         for (; it != (*pathIt)->constEnd(); ++it) {
             (*it)->map(matrix);
@@ -591,13 +619,14 @@ void KoPathShape::map(const QMatrix &matrix)
     }
 }
 
-void KoPathShape::updateLast(KoPathPoint ** lastPoint)
+void KoPathShapePrivate::updateLast(KoPathPoint **lastPoint)
 {
+    Q_Q(KoPathShape);
     // check if we are about to add a new point to a closed subpath
     if ((*lastPoint)->properties() & KoPathPoint::StopSubpath
             && (*lastPoint)->properties() & KoPathPoint::CloseSubpath) {
         // get the first point of the subpath
-        KoPathPoint * subpathStart = m_subpaths.last()->first();
+        KoPathPoint *subpathStart = q->m_subpaths.last()->first();
         // clone the first point of the subpath...
         KoPathPoint * newLastPoint = new KoPathPoint(*subpathStart);
         // ... and make it a normal point
@@ -613,7 +642,7 @@ void KoPathShape::updateLast(KoPathPoint ** lastPoint)
         // now start a new subpath with the cloned start point
         KoSubpath *path = new KoSubpath;
         path->push_back(newLastPoint);
-        m_subpaths.push_back(path);
+        q->m_subpaths.push_back(path);
         *lastPoint = newLastPoint;
     } else {
         // the subpath was not closed so the formerly last point
@@ -682,7 +711,8 @@ KoPathPointIndex KoPathShape::pathPointIndex(const KoPathPoint *point) const
 
 KoPathPoint * KoPathShape::pointByIndex(const KoPathPointIndex &pointIndex) const
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(const KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath == 0 || pointIndex.second < 0 || pointIndex.second >= subpath->size())
         return 0;
@@ -692,9 +722,10 @@ KoPathPoint * KoPathShape::pointByIndex(const KoPathPointIndex &pointIndex) cons
 
 KoPathSegment KoPathShape::segmentByIndex(const KoPathPointIndex &pointIndex) const
 {
+    Q_D(const KoPathShape);
     KoPathSegment segment(0, 0);
 
-    KoSubpath * subpath = subPath(pointIndex.first);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath != 0 && pointIndex.second >= 0 && pointIndex.second < subpath->size()) {
         KoPathPoint * point = subpath->at(pointIndex.second);
@@ -728,9 +759,10 @@ int KoPathShape::subpathCount() const
     return m_subpaths.count();
 }
 
-int KoPathShape::pointCountSubpath(int subpathIndex) const
+int KoPathShape::subpathPointCount(int subpathIndex) const
 {
-    KoSubpath * subpath = subPath(subpathIndex);
+    Q_D(const KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath == 0)
         return -1;
@@ -740,7 +772,8 @@ int KoPathShape::pointCountSubpath(int subpathIndex) const
 
 bool KoPathShape::isClosedSubpath(int subpathIndex)
 {
-    KoSubpath * subpath = subPath(subpathIndex);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath == 0)
         return false;
@@ -753,12 +786,13 @@ bool KoPathShape::isClosedSubpath(int subpathIndex)
 
 bool KoPathShape::insertPoint(KoPathPoint* point, const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath == 0 || pointIndex.second < 0 || pointIndex.second > subpath->size())
         return false;
 
-    KoPathPoint::KoPointProperties properties = point->properties();
+    KoPathPoint::PointProperties properties = point->properties();
     properties &= ~KoPathPoint::StartSubpath;
     properties &= ~KoPathPoint::StopSubpath;
     properties &= ~KoPathPoint::CloseSubpath;
@@ -793,15 +827,20 @@ bool KoPathShape::insertPoint(KoPathPoint* point, const KoPathPointIndex &pointI
 
 KoPathPoint * KoPathShape::removePoint(const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (subpath == 0 || pointIndex.second < 0 || pointIndex.second >= subpath->size())
         return 0;
 
     KoPathPoint * point = subpath->takeAt(pointIndex.second);
 
+    //don't do anything (not even crash), if there was only one point
+    if (pointCount()==0) {
+        return point;
+    }
     // check if we removed the first point
-    if (pointIndex.second == 0) {
+    else if (pointIndex.second == 0) {
         // first point removed, set new StartSubpath
         subpath->first()->setProperty(KoPathPoint::StartSubpath);
         // check if path was closed
@@ -826,7 +865,8 @@ KoPathPoint * KoPathShape::removePoint(const KoPathPointIndex &pointIndex)
 
 bool KoPathShape::breakAfter(const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (!subpath || pointIndex.second < 0 || pointIndex.second > subpath->size() - 2
         || isClosedSubpath(pointIndex.first))
@@ -851,8 +891,9 @@ bool KoPathShape::breakAfter(const KoPathPointIndex &pointIndex)
 
 bool KoPathShape::join(int subpathIndex)
 {
-    KoSubpath * subpath = subPath(subpathIndex);
-    KoSubpath * nextSubpath = subPath(subpathIndex + 1);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
+    KoSubpath *nextSubpath = d->subPath(subpathIndex + 1);
 
     if (!subpath || !nextSubpath || isClosedSubpath(subpathIndex)
             || isClosedSubpath(subpathIndex+1))
@@ -878,7 +919,8 @@ bool KoPathShape::join(int subpathIndex)
 
 bool KoPathShape::moveSubpath(int oldSubpathIndex, int newSubpathIndex)
 {
-    KoSubpath * subpath = subPath(oldSubpathIndex);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(oldSubpathIndex);
 
     if (subpath == 0 || newSubpathIndex >= m_subpaths.size())
         return false;
@@ -894,7 +936,8 @@ bool KoPathShape::moveSubpath(int oldSubpathIndex, int newSubpathIndex)
 
 KoPathPointIndex KoPathShape::openSubpath(const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (!subpath || pointIndex.second < 0 || pointIndex.second >= subpath->size()
             || !isClosedSubpath(pointIndex.first))
@@ -920,7 +963,8 @@ KoPathPointIndex KoPathShape::openSubpath(const KoPathPointIndex &pointIndex)
 
 KoPathPointIndex KoPathShape::closeSubpath(const KoPathPointIndex &pointIndex)
 {
-    KoSubpath * subpath = subPath(pointIndex.first);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(pointIndex.first);
 
     if (!subpath || pointIndex.second < 0 || pointIndex.second >= subpath->size()
         || isClosedSubpath(pointIndex.first))
@@ -939,13 +983,14 @@ KoPathPointIndex KoPathShape::closeSubpath(const KoPathPointIndex &pointIndex)
     subpath->first()->setProperty(KoPathPoint::StartSubpath);
     subpath->last()->setProperty(KoPathPoint::StopSubpath);
 
-    closeSubpath(subpath);
+    d->closeSubpath(subpath);
     return pathPointIndex(oldStartPoint);
 }
 
 bool KoPathShape::reverseSubpath(int subpathIndex)
 {
-    KoSubpath * subpath = subPath(subpathIndex);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath == 0)
         return false;
@@ -961,8 +1006,8 @@ bool KoPathShape::reverseSubpath(int subpathIndex)
     KoPathPoint *first = subpath->first();
     KoPathPoint *last = subpath->last();
 
-    KoPathPoint::KoPointProperties firstProps = first->properties();
-    KoPathPoint::KoPointProperties lastProps = last->properties();
+    KoPathPoint::PointProperties firstProps = first->properties();
+    KoPathPoint::PointProperties lastProps = last->properties();
 
     firstProps |= KoPathPoint::StartSubpath;
     firstProps &= ~KoPathPoint::StopSubpath;
@@ -980,7 +1025,8 @@ bool KoPathShape::reverseSubpath(int subpathIndex)
 
 KoSubpath * KoPathShape::removeSubpath(int subpathIndex)
 {
-    KoSubpath * subpath = subPath(subpathIndex);
+    Q_D(KoPathShape);
+    KoSubpath *subpath = d->subPath(subpathIndex);
 
     if (subpath != 0)
         m_subpaths.removeAt(subpathIndex);
@@ -1003,8 +1049,8 @@ bool KoPathShape::combine(KoPathShape *path)
     if (! path)
         return false;
 
-    QMatrix pathMatrix = path->absoluteTransformation(0);
-    QMatrix myMatrix = absoluteTransformation(0).inverted();
+    QTransform pathMatrix = path->absoluteTransformation(0);
+    QTransform myMatrix = absoluteTransformation(0).inverted();
 
     foreach(KoSubpath* subpath, path->m_subpaths) {
         KoSubpath *newSubpath = new KoSubpath();
@@ -1027,7 +1073,7 @@ bool KoPathShape::separate(QList<KoPathShape*> & separatedPaths)
     if (! m_subpaths.size())
         return false;
 
-    QMatrix myMatrix = absoluteTransformation(0);
+    QTransform myMatrix = absoluteTransformation(0);
 
     foreach(KoSubpath* subpath, m_subpaths) {
         KoPathShape *shape = new KoPathShape();
@@ -1050,7 +1096,7 @@ bool KoPathShape::separate(QList<KoPathShape*> & separatedPaths)
     return true;
 }
 
-void KoPathShape::closeSubpath(KoSubpath *subpath)
+void KoPathShapePrivate::closeSubpath(KoSubpath *subpath)
 {
     if (! subpath)
         return;
@@ -1059,7 +1105,7 @@ void KoPathShape::closeSubpath(KoSubpath *subpath)
     subpath->first()->setProperty(KoPathPoint::CloseSubpath);
 }
 
-void KoPathShape::closeMergeSubpath(KoSubpath *subpath)
+void KoPathShapePrivate::closeMergeSubpath(KoSubpath *subpath)
 {
     if (! subpath || subpath->size() < 2)
         return;
@@ -1086,12 +1132,13 @@ void KoPathShape::closeMergeSubpath(KoSubpath *subpath)
     }
 }
 
-KoSubpath * KoPathShape::subPath(int subpathIndex) const
+KoSubpath *KoPathShapePrivate::subPath(int subpathIndex) const
 {
-    if (subpathIndex < 0 || subpathIndex >= m_subpaths.size())
+    Q_Q(const KoPathShape);
+    if (subpathIndex < 0 || subpathIndex >= q->m_subpaths.size())
         return 0;
 
-    return m_subpaths.at(subpathIndex);
+    return q->m_subpaths.at(subpathIndex);
 }
 
 QString KoPathShape::pathShapeId() const
@@ -1099,7 +1146,7 @@ QString KoPathShape::pathShapeId() const
     return KoPathShapeId;
 }
 
-QString KoPathShape::toString(const QMatrix &matrix) const
+QString KoPathShape::toString(const QTransform &matrix) const
 {
     QString d;
 
@@ -1157,7 +1204,7 @@ QString KoPathShape::toString(const QMatrix &matrix) const
     return d;
 }
 
-char nodeType( const KoPathPoint * point )
+char nodeType(const KoPathPoint * point)
 {
     if (point->properties() & KoPathPoint::IsSmooth) {
         return 's';
@@ -1170,31 +1217,32 @@ char nodeType( const KoPathPoint * point )
     }
 }
 
-QString KoPathShape::nodeTypes() const
+QString KoPathShapePrivate::nodeTypes() const
 {
+    Q_Q(const KoPathShape);
     QString types;
-    KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
-    for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+    KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
+    for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
         KoSubpath::const_iterator it((*pathIt)->constBegin());
         for (; it != (*pathIt)->constEnd(); ++it) {
             if (it == (*pathIt)->constBegin()) {
-                types.append( 'c' );
+                types.append('c');
             }
             else {
-                types.append( nodeType( *it ) );
+                types.append(nodeType(*it));
             }
 
             if ((*it)->properties() & KoPathPoint::StopSubpath
                 && (*it)->properties() & KoPathPoint::CloseSubpath) {
                 KoPathPoint * firstPoint = (*pathIt)->first();
-                types.append( nodeType( firstPoint ) );
+                types.append(nodeType(firstPoint));
             }
         }
     }
     return types;
 }
 
-void updateNodeType( KoPathPoint * point, const QChar & nodeType )
+void updateNodeType(KoPathPoint * point, const QChar & nodeType)
 {
     if (nodeType == 's') {
         point->setProperty(KoPathPoint::IsSmooth);
@@ -1204,13 +1252,14 @@ void updateNodeType( KoPathPoint * point, const QChar & nodeType )
     }
 }
 
-void KoPathShape::loadNodeTypes(const KoXmlElement & element)
+void KoPathShapePrivate::loadNodeTypes(const KoXmlElement &element)
 {
+    Q_Q(KoPathShape);
     if (element.hasAttributeNS(KoXmlNS::koffice, "nodeTypes")) {
         QString nodeTypes = element.attributeNS(KoXmlNS::koffice, "nodeTypes");
         QString::const_iterator nIt(nodeTypes.constBegin());
-        KoSubpathList::const_iterator pathIt(m_subpaths.constBegin());
-        for (; pathIt != m_subpaths.constEnd(); ++pathIt) {
+        KoSubpathList::const_iterator pathIt(q->m_subpaths.constBegin());
+        for (; pathIt != q->m_subpaths.constEnd(); ++pathIt) {
             KoSubpath::const_iterator it((*pathIt)->constBegin());
             for (; it != (*pathIt)->constEnd(); ++it, nIt++) {
                 // be sure not to crash if there are not enough nodes in nodeTypes
@@ -1245,7 +1294,7 @@ void KoPathShape::setFillRule(Qt::FillRule fillRule)
     d->fillRule = fillRule;
 }
 
-KoPathShape * KoPathShape::fromQPainterPath(const QPainterPath &path)
+KoPathShape * KoPathShape::createShapeFromPainterPath(const QPainterPath &path)
 {
     KoPathShape * shape = new KoPathShape();
 
@@ -1275,7 +1324,7 @@ KoPathShape * KoPathShape::fromQPainterPath(const QPainterPath &path)
 
 bool KoPathShape::hitTest(const QPointF &position) const
 {
-    if (parent() && parent()->childClipped(this) && ! parent()->hitTest(position))
+    if (parent() && parent()->isClipped(this) && ! parent()->hitTest(position))
         return false;
 
     QPointF point = absoluteTransformation(0).inverted().map(position);
@@ -1283,12 +1332,12 @@ bool KoPathShape::hitTest(const QPointF &position) const
     if (border()) {
         KoInsets insets;
         border()->borderInsets(this, insets);
-        QRectF roi( QPointF(-insets.left, -insets.top), QPointF(insets.right, insets.bottom) );
-        roi.moveCenter( point );
-        if( outlinePath.intersects( roi ) || outlinePath.contains( roi ) )
+        QRectF roi(QPointF(-insets.left, -insets.top), QPointF(insets.right, insets.bottom));
+        roi.moveCenter(point);
+        if (outlinePath.intersects(roi) || outlinePath.contains(roi))
             return true;
     } else {
-        if( outlinePath.contains( point ) )
+        if (outlinePath.contains(point))
             return true;
     }
 
