@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2006, 2008 Casper Boemann <cbr@boemann.dk>
-   Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+   Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
    Copyright (C) 2007-2009 Jan Hambrecht <jaham@gmx.net>
 
    This library is free software; you can redistribute it and/or
@@ -26,7 +26,7 @@
 #include "KoInsets.h"
 #include "KoFlake.h"
 
-#include <QMatrix>
+#include <QTransform>
 #include <QVector>
 #include <QSet>
 #include <QMap>
@@ -34,7 +34,7 @@
 #include <QMetaType>
 
 #include <KoXmlReaderForward.h>
-#include <KoSnapData.h>
+//#include <KoSnapData.h>
 
 #include "flake_export.h"
 
@@ -54,9 +54,12 @@ class KoCanvasBase;
 class KoShapeLoadingContext;
 class KoGenStyle;
 class KoShapeControllerBase;
-class KoDataCenter;
+class KoDataCenterBase;
 class KoShapeShadow;
 class KoEventAction;
+class KoShapePrivate;
+class KoFilterEffectStack;
+class KoSnapData;
 
 /**
  *
@@ -108,8 +111,8 @@ public:
     enum ChangeType {
         PositionChanged, ///< used after a setPosition()
         RotationChanged, ///< used after a setRotation()
-        ScaleChanged,   ///< used after a setScale()
-        ShearChanged,   ///< used after a setShear()
+        ScaleChanged,   ///< used after a scale()
+        ShearChanged,   ///< used after a shear()
         SizeChanged,    ///< used after a setSize()
         GenericMatrixChange,    ///< used after the matrix was changed without knowing which property explicitly changed
         ParentChanged,   ///< used after a setParent()
@@ -118,7 +121,9 @@ public:
         BorderChanged, ///< the shapes border has changed
         BackgroundChanged, ///< the shapes background has changed
         ShadowChanged, ///< the shapes shadow has changed
-        ParameterChanged ///< the shapes parameter has changed (KoParameterShape only)
+        ParameterChanged, ///< the shapes parameter has changed (KoParameterShape only)
+        ContentChanged, ///< the content of the shape changed e.g. a new image inside a pixmap/text change inside a textshape
+        ChildChanged ///< a child of a container was changed/removed. This is propagated to all parents
     };
 
     /**
@@ -155,7 +160,7 @@ public:
      * @param painter used for painting the shape
      * @param converter to convert between internal and view coordinates.
      * @param canvas the canvas that requested this paint.  This can be used to retrieve canvas specific properties
-     *      like selection and get a reference to the KoCanvasResourceProvider.
+     *      like selection and get a reference to the KoResourceManager.
      */
     virtual void paintDecorations(QPainter &painter, const KoViewConverter &converter, const KoCanvasBase *canvas);
 
@@ -167,15 +172,15 @@ public:
      *
      * @return false if loading failed
      */
-    virtual bool loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context) = 0;
+    virtual bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context) = 0;
 
     /**
      * @brief store the shape data as ODF XML.
-     * This is the method that will be called when saving a shape as a described in¬
+     * This is the method that will be called when saving a shape as a described in
      * OpenDocument 9.2 Drawing Shapes.
      * @see saveOdfAttributes
      */
-    virtual void saveOdf(KoShapeSavingContext & context) const = 0;
+    virtual void saveOdf(KoShapeSavingContext &context) const = 0;
 
     /**
      * This method can be used while saving the shape as ODF to add the data
@@ -194,13 +199,6 @@ public:
      */
     void saveOdfCommonChildElements(KoShapeSavingContext &context) const;
 
-    /**
-     * After the shape has been created this method is called so it can get access to any DataCenter it
-     * might want.
-     * The default implementation does nothing.
-     */
-    virtual void init(const QMap<QString, KoDataCenter *> & dataCenterMap);
-
     /**
      * @brief Scale the shape using the zero-point which is the top-left corner.
      * @see position()
@@ -208,7 +206,7 @@ public:
      * @param sx scale in x direction
      * @param sy scale in y direction
      */
-    void setScale(qreal sx, qreal sy);
+    void scale(qreal sx, qreal sy);
 
     /**
      * @brief Rotate the shape (relative)
@@ -233,7 +231,7 @@ public:
      * @param sx shear in x direction
      * @param sy shear in y direction
      */
-    void setShear(qreal sx, qreal sy);
+    void shear(qreal sx, qreal sy);
 
     /**
      * @brief Resize the shape
@@ -251,7 +249,7 @@ public:
      *
      * The size is in shape coordinates.
      *
-     * @return the size of the shape as set by resize()
+     * @return the size of the shape as set by setSize()
      */
     virtual QSizeF size() const;
 
@@ -267,7 +265,7 @@ public:
      *
      * @return the position of the shape
      */
-    virtual QPointF position() const;
+    QPointF position() const;
 
     /**
      * @brief Check if the shape is hit on position
@@ -299,6 +297,7 @@ public:
 
     /**
      * Return a list of the connection points that have been added to this shape.
+     * All the points are relative to the shape position, see absolutePosition().
      * @return a list of the connectors that have been added to this shape.
      */
     QList<QPointF> connectionPoints() const;
@@ -306,17 +305,17 @@ public:
     /**
      * Add a event action
      */
-    void addEventAction(KoEventAction * action);
+    void addEventAction(KoEventAction *action);
 
     /**
      * Remove a event action
      */
-    void removeEventAction(KoEventAction * action);
+    void removeEventAction(KoEventAction *action);
 
     /**
      * Get all event actions
      */
-    QList<KoEventAction *> eventActions() const;
+    QSet<KoEventAction *> eventActions() const;
 
     /**
      * Set the background of the shape.
@@ -326,7 +325,8 @@ public:
      * if it is transparent or not.
      * @param background the new shape background.
      */
-    void setBackground(KoShapeBackground * background);
+    void setBackground(KoShapeBackground *background);
+
     /**
      * return the brush used to paint te background of this shape with.
      * A QBrush can have a plain color, be fully transparent or have a complex fill.
@@ -334,7 +334,7 @@ public:
      * will be able to tell if its transparent or not.
      * @return the background-brush
      */
-    KoShapeBackground * background() const;
+    KoShapeBackground *background() const;
 
     /**
      * Returns true if there is some transparency, false if the shape is fully opaque.
@@ -342,7 +342,19 @@ public:
      * you should override it and always return true if your shape is not square.
      * @return if the shape is (partly) transparent.
      */
-    virtual bool hasTransparency();
+    virtual bool hasTransparency() const;
+
+    /**
+     * Sets shape level transparency.
+     * @param transparency the new shape level transparency
+     */
+    void setTransparency(qreal transparency);
+
+    /**
+     * Returns the shape level transparency.
+     * @param recursive when true takes the parents transparency into account
+     */
+    qreal transparency(bool recursive=false) const;
 
     /**
      * Retrieve the z-coordinate of this shape.
@@ -371,6 +383,7 @@ public:
      * Being visible means being painted, as well as being used for
      *   things like guidelines or searches.
      * @param on when true; set the shape to be visible.
+     * @see setGeometryProtected(), setContentProtected(), setSelectable()
      */
     void setVisible(bool on);
     /**
@@ -379,6 +392,7 @@ public:
      *   things like guidelines or searches.
      * @param recursive when true, checks visibility recursively
      * @return current visibility state of this shape.
+     * @see isGeometryProtected(), isContentProtected(), isSelectable()
      */
     bool isVisible(bool recursive = false) const;
 
@@ -405,37 +419,48 @@ public:
      * Makes it possible for the user to select this shape.
      * This parameter defaults to true.
      * @param selectable when true; set the shape to be selectable by the user.
+     * @see setGeometryProtected(), setContentProtected(), setVisible()
      */
     void setSelectable(bool selectable);
+
     /**
      * Returns if this shape can be selected by the user.
      * @return true only when the object is selectable.
+     * @see isGeometryProtected(), isContentProtected(), isVisible()
      */
     bool isSelectable() const;
 
     /**
-     * Changes the Shape to be locked in place.
-     * Being locked means the user can not change shape or position of the shape.
-     * @param locked when true; set the shape to be locked.
+     * Tells the shape to have its position/rotation and size protected from user-changes.
+     * The geometry being protected means the user can not change shape or position of the
+     * shape. This includes any matrix operation such as rotation.
+     * @param on when true; set the shape to have its geometry protected.
+     * @see setContentProtected(), setSelectable(), setVisible()
      */
-    void setLocked(bool locked);
+    void setGeometryProtected(bool on);
+
     /**
-     * Returns current locked state of this shape.
-     * Being locked means the user can not change shape or position of the shape.
-     * @return current locked state of this shape.
+     * Returns current geometry protection state of this shape.
+     * The geometry being protected means the user can not change shape or position of the
+     * shape. This includes any matrix operation such as rotation.
+     * @return current geometry protection state of this shape.
+     * @see isContentProtected(), isSelectable(), isVisible()
      */
-    bool isLocked() const;
+    bool isGeometryProtected() const;
 
     /**
      * Marks the shape to have its content protected against editing.
      * Content protection is a hint for tools to disallow the user editing the content.
      * @param protect when true set the shapes content to be protected from user modification.
+     * @see setGeometryProtected(), setSelectable(), setVisible()
      */
     void setContentProtected(bool protect);
+
     /**
      * Returns current content protection state of this shape.
      * Content protection is a hint for tools to disallow the user editing the content.
      * @return current content protection state of this shape.
+     * @see isGeometryProtected(), isSelectable(), isVisible()
      */
     bool isContentProtected() const;
 
@@ -469,7 +494,7 @@ public:
      * will be merged into an appropriate repaint action.
      * @param shape the rectangle (in pt) to queue for repaint.
      */
-    void update(const QRectF &shape) const;
+    virtual void update(const QRectF &shape) const;
 
     /**
      * This is a method used to sort a list using the STL sorting methods.
@@ -485,7 +510,7 @@ public:
      * on, for example.
      * @returns the outline of the shape in the form of a path.
      */
-    virtual const QPainterPath outline() const;
+    virtual QPainterPath outline() const;
 
     /**
      * Returns the currently set border, or 0 if there is no border.
@@ -506,10 +531,10 @@ public:
     KoInsets borderInsets() const;
 
     /// Sets the new shadow, removing the old one
-    void setShadow(KoShapeShadow * shadow);
+    void setShadow(KoShapeShadow *shadow);
 
-    /// Returns the currently set shadow or 0 if there is now shadow set
-    KoShapeShadow * shadow() const;
+    /// Returns the currently set shadow or 0 if there is no shadow set
+    KoShapeShadow *shadow() const;
 
     /**
      * Setting the shape to keep its aspect-ratio has the effect that user-scaling will
@@ -570,6 +595,7 @@ public:
      * @param applicationData the new application data, or 0 to delete the current one.
      */
     void setApplicationData(KoShapeApplicationData *applicationData);
+
     /**
      * Return the current applicationData.
      */
@@ -577,14 +603,15 @@ public:
 
     /**
      * Return the Id of this shape, identifying the type of shape by the id of the factory.
-     * @see KoShapeFactory::shapeId()
+     * @see KoShapeFactoryBase::shapeId()
      * @return the id of the shape-type
      */
-    const QString & shapeId() const;
+    QString shapeId() const;
+
     /**
      * Set the Id of this shape.  A shapeFactory is expected to set the Id at creation
      * so applications can find out what kind of shape this is.
-     * @see KoShapeFactory::shapeId()
+     * @see KoShapeFactoryBase::shapeId()
      * @param id the ID from the factory that created this shape
      */
     void setShapeId(const QString &id);
@@ -598,7 +625,7 @@ public:
      * @param converter if not null, this method uses the converter to mark the right
      *        offsets in the current view.
      */
-    QMatrix absoluteTransformation(const KoViewConverter *converter) const;
+    QTransform absoluteTransformation(const KoViewConverter *converter) const;
 
     /**
      * Applies a transformation to this shape.
@@ -609,16 +636,16 @@ public:
      *
      * @param matrix the transformation matrix to apply
      */
-    void applyAbsoluteTransformation(const QMatrix &matrix);
+    void applyAbsoluteTransformation(const QTransform &matrix);
 
     /**
      * Sets a new transformation matrix describing the local transformations on this shape.
      * @param matrix the new transformation matrix
      */
-    void setTransformation(const QMatrix &matrix);
+    void setTransformation(const QTransform &matrix);
 
     /// Returns the shapes local transformation matrix
-    QMatrix transformation() const;
+    QTransform transformation() const;
 
     /**
      * Applies a transformation to this shape.
@@ -627,7 +654,7 @@ public:
      *
      * @param matrix the transformation matrix to apply
      */
-    void applyTransformation(const QMatrix &matrix);
+    void applyTransformation(const QTransform &matrix);
 
     /**
      * Copy all the settings from the parameter shape and apply them to this shape.
@@ -635,7 +662,7 @@ public:
      * is a notable exclusion.
      * @param shape the shape to use as original
      */
-    virtual void copySettings(const KoShape *shape);
+    void copySettings(const KoShape *shape);
 
     /**
      * Convenience method that allows people implementing paint() to use the shape
@@ -684,7 +711,7 @@ public:
      * Sets the name of the shape.
      * @param name the new shape name
      */
-    void setName(const QString & name);
+    void setName(const QString &name);
 
     /**
      * Update the position of the shape in the tree of the KoShapeManager.
@@ -693,14 +720,17 @@ public:
 
     /**
      * A shape can be in a state that it is doing processing data like loading or text layout.
-     * In which case
-     * it can be shown on screen just fine (probably partially) but it should really not be printed
+     * In this case it can be shown on screen probably partially but it should really not be printed
      * until it is fully done processing.
-     * Flake will call this method from a non-main thread and only start printing it when the
-     * method returned.
-     * Warning!  This method can be blocking for a long time, never call it on the gui-thread!
+     * Warning! This method can be blocking for a long time
+     * @param asynchronous If set to true the processing will can take place in a different thread and the 
+     *                     function will not block until the shape is finised. 
+     *                     In case of printing Flake will call this method from a non-main thread and only 
+     *                     start printing it when the in case of printing method returned.
+     *                     If set to false the processing needs to be done synchronously and will 
+     *                     block until the result is finished.
      */
-    virtual void waitUntilReady() const {}
+    virtual void waitUntilReady(const KoViewConverter &converter, bool asynchronous = true) const;
 
     /**
      * Schedule the shape for thread-safe deletion.
@@ -723,17 +753,26 @@ public:
 
     /**
      * Adds a shape which depends on this shape.
+     * Making a shape dependent on this one means it will get shapeChanged() called
+     * on each update of this shape.
      *
      * If this shape already depends on the given shape, establishing the
      * dependency is refused to prevent circular dependencies.
      *
      * @param shape the shape which depends on this shape
-     * @return true if dependency could be established, else false
+     * @return true if dependency could be established, otherwise false
+     * @see removeDependee(), hasDependee()
      */
-    bool addDependee(KoShape * shape);
+    bool addDependee(KoShape *shape);
 
-    /// Removes as shape depending on this shape
-    void removeDependee(KoShape * shape);
+    /**
+     * Removes as shape depending on this shape.
+     * @see addDependee(), hasDependee()
+     */
+    void removeDependee(KoShape *shape);
+
+    /// Returns if the given shape is dependent on this shape
+    bool hasDependee(KoShape *shape) const;
 
     /// Returns additional snap data the shape wants to have snapping to
     virtual KoSnapData snapData() const;
@@ -747,14 +786,14 @@ public:
      * @param name The name of the attribute in the following form prefix:tag e.g. presentation:placeholder
      * @param value The value of the attribute
      */
-    void setAdditionalAttribute(const char * name, const QString & value);
+    void setAdditionalAttribute(const QString &name, const QString &value);
 
     /**
      * Remove additional attribute
      *
      * @param name The name of the attribute in the following form prefix:tag e.g. presentation:placeholder
      */
-    void removeAdditionalAttribute(const char * name);
+    void removeAdditionalAttribute(const QString &name);
 
     /**
      * Check if additional attribute is set
@@ -763,7 +802,7 @@ public:
      *
      * @return true if there is a attribute with prefix:tag set, false otherwise
      */
-    bool hasAdditionalAttribute(const char * name) const;
+    bool hasAdditionalAttribute(const QString &name) const;
 
     /**
      * Get additional attribute
@@ -772,13 +811,63 @@ public:
      *
      * @return The value of the attribute if it exists or a null string if not found.
      */
-    QString additionalAttribute(const char * name) const;
+    QString additionalAttribute(const QString &name) const;
 
-    void setAdditionalStyleAttribute(const char * name, const QString & value);
+    void setAdditionalStyleAttribute(const char *name, const QString &value);
 
-    void removeAdditionalStyleAttribute(const char * name);
+    void removeAdditionalStyleAttribute(const char *name);
+
+    /**
+     * Returns the filter effect stack of the shape
+     *
+     * @return the list of filter effects applied on the shape when rendering.
+     */
+    KoFilterEffectStack *filterEffectStack() const;
+
+    /// Sets the new filter effect stack, removing the old one
+    void setFilterEffectStack(KoFilterEffectStack *filterEffectStack);
+
+    /**
+     * Set the property collision detection.
+     * Setting this to true will result in calls to shapeChanged() with the CollisionDetected
+     * parameter whenever either this or another shape is moved/rotated etc and intersects this shape.
+     * @param detect if true detect collisions.
+     */
+    void setCollisionDetection(bool detect);
+
+    /**
+     * get the property collision detection.
+     * @returns true if collision detection is on.
+     */
+    bool collisionDetection();
+
+    /**
+     * Return the tool delegates for this shape.
+     * In Flake a shape being selected will cause the tool manager to make available all tools that
+     * can edit the selected shapes.  In some cases selecting one shape should allow the tool to
+     * edit a related shape be available too.  The tool delegates allows this to happen by taking
+     * all the shapes in the set into account on tool selection.
+     * Notice that if the set is non-empty 'this' shape is no longer looked at. You can choose
+     * to add itself to the set too.
+     */
+    QSet<KoShape*> toolDelegates() const;
+
+    /**
+     * Set the tool delegates.
+     * @param delegates the new delegates.
+     * @see toolDelegates()
+     */
+    void setToolDelegates(const QSet<KoShape*> &delegates);
+
+    /**
+     * \internal
+     * Returns the private object for use within the flake lib
+     */
+    KoShapePrivate *priv();
 
 protected:
+    /// constructor
+    KoShape(KoShapePrivate &);
 
     /* ** loading saving helper methods */
     /// attributes from ODF 1.1 chapter 9.2.15 Common Drawing Shape Attributes
@@ -810,14 +899,14 @@ protected:
      * @param element element which represents the shape in odf
      * @param attributes a number of OdfAttribute items to state which attributes to load.
      */
-    bool loadOdfAttributes(const KoXmlElement & element, KoShapeLoadingContext &context, int attributes);
+    bool loadOdfAttributes(const KoXmlElement &element, KoShapeLoadingContext &context, int attributes);
 
     /**
      * Parses the transformation attribute from the given string
      * @param transform the transform attribute string
      * @return the resulting transformation matrix
      */
-    QMatrix parseOdfTransform(const QString &transform);
+    QTransform parseOdfTransform(const QString &transform);
 
     /**
      * @brief Saves the style used for the shape
@@ -838,72 +927,30 @@ protected:
      * @param element the xml element to  load the style from
      * @param context the loading context used for loading
      */
-    virtual void loadStyle(const KoXmlElement & element, KoShapeLoadingContext &context);
-
-    /// Loads the fill style
-    //QBrush loadOdfFill( const KoXmlElement & element, KoShapeLoadingContext & context );
-    KoShapeBackground * loadOdfFill(const KoXmlElement & element, KoShapeLoadingContext & context);
+    virtual void loadStyle(const KoXmlElement &element, KoShapeLoadingContext &context);
 
     /// Loads the stroke style
-    KoShapeBorderModel * loadOdfStroke(const KoXmlElement & element, KoShapeLoadingContext & context);
+    KoShapeBorderModel *loadOdfStroke(const KoXmlElement &element, KoShapeLoadingContext &context) const;
 
     /// Loads the shadow style
-    KoShapeShadow * loadOdfShadow(const KoXmlElement & element, KoShapeLoadingContext & context);
-
-    /**
-     * Fills the style stack and returns the value of the given style property (e.g fill, stroke).
-     */
-    QString getStyleProperty(const char *property, const KoXmlElement & element, KoShapeLoadingContext & context);
+    KoShapeBackground *loadOdfFill(KoShapeLoadingContext &context) const;
 
     /* ** end loading saving */
 
-
     /**
      * A hook that allows inheriting classes to do something after a KoShape property changed
      * This is called whenever the shape, position rotation or scale properties were altered.
      * @param type an indicator which type was changed.
      */
-    virtual void shapeChanged(ChangeType type) {
-        Q_UNUSED(type);
-    }
-
-    /// This is called whenever a shape this shape depends on has changed
-    virtual void notifyShapeChanged(KoShape * shape, ChangeType type);
-
-    /**
-     * Set the property collision detection.
-     * Setting this to true will result in calls to shapeChanged() with the CollisionDetected
-     * parameter whenever either this or another shape is moved/rotated etc and intersects this shape.
-     * @param detect if true detect collisions.
-     */
-    void setCollisionDetection(bool detect);
-
-    /**
-     * get the property collision detection.
-     * @returns true if collision detection is on.
-     */
-    bool collisionDetection();
+    virtual void shapeChanged(ChangeType type, KoShape *shape = 0);
 
     /// return the current matrix that contains the rotation/scale/position of this shape
-    const QMatrix& matrix() const;
+    QTransform transform() const;
 
-    /**
-     * Notify the shape that a change was done. To be used by inheriting shapes.
-     * @param type the change type
-     */
-    void notifyChangedShape(ChangeType type);
-    
-    /// Returns if the given shape is dependent on this shape
-    bool hasDependee(KoShape * shape) const;
+    KoShapePrivate *d_ptr;
 
 private:
-    friend class KoShapeManager;
-    void addShapeManager(KoShapeManager * manager);
-    void removeShapeManager(KoShapeManager * manager);
-
-    class Private;
-    friend class Private;
-    Private * const d;
+    Q_DECLARE_PRIVATE(KoShape)
 };
 
 Q_DECLARE_METATYPE(KoShape*)
