@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2007 Jan Hambrecht <jaham@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
@@ -21,7 +21,8 @@
 #include "KoShapeContainer_p.h"
 #include "KoShapeContainerModel.h"
 #include "KoShapeBorderModel.h"
-#include "KoChildrenData.h"
+#include "KoShapeContainerDefaultModel.h"
+#include "KoShapeSavingContext.h"
 
 #include <QPointF>
 #include <QPainter>
@@ -29,13 +30,13 @@
 
 KoShapeContainerPrivate::KoShapeContainerPrivate(KoShapeContainer *q)
     : KoShapePrivate(q),
-    children(0)
+    model(0)
 {
 }
 
 KoShapeContainerPrivate::~KoShapeContainerPrivate()
 {
-    delete children;
+    delete model;
 }
 
 KoShapeContainer::KoShapeContainer()
@@ -47,42 +48,47 @@ KoShapeContainer::KoShapeContainer(KoShapeContainerModel *model)
         : KoShape(*(new KoShapeContainerPrivate(this)))
 {
     Q_D(KoShapeContainer);
-    d->children = model;
+    d->model = model;
+}
+
+KoShapeContainer::KoShapeContainer(KoShapeContainerPrivate &dd)
+    : KoShape(dd)
+{
 }
 
 KoShapeContainer::~KoShapeContainer()
 {
     Q_D(KoShapeContainer);
-    if (d->children) {
-        foreach(KoShape *shape, d->children->childShapes())
+    if (d->model) {
+        foreach(KoShape *shape, d->model->shapes())
             shape->setParent(0);
     }
 }
 
-void KoShapeContainer::addChild(KoShape *shape)
+void KoShapeContainer::addShape(KoShape *shape)
 {
     Q_D(KoShapeContainer);
     Q_ASSERT(shape);
-    if (shape->parent() == this && childShapes().contains(shape))
+    if (shape->parent() == this && shapes().contains(shape))
         return;
-    if (d->children == 0)
-        d->children = new KoChildrenData();
+    if (d->model == 0)
+        d->model = new KoShapeContainerDefaultModel();
     if (shape->parent() && shape->parent() != this)
-        shape->parent()->removeChild(shape);
-    d->children->add(shape);
+        shape->parent()->removeShape(shape);
+    d->model->add(shape);
     shape->setParent(this);
-    childCountChanged();
+    shapeCountChanged();
 }
 
-void KoShapeContainer::removeChild(KoShape *shape)
+void KoShapeContainer::removeShape(KoShape *shape)
 {
     Q_D(KoShapeContainer);
     Q_ASSERT(shape);
-    if (d->children == 0)
+    if (d->model == 0)
         return;
-    d->children->remove(shape);
+    d->model->remove(shape);
     shape->setParent(0);
-    childCountChanged();
+    shapeCountChanged();
 
     KoShapeContainer * grandparent = parent();
     if (grandparent) {
@@ -90,28 +96,44 @@ void KoShapeContainer::removeChild(KoShape *shape)
     }
 }
 
-int  KoShapeContainer::childCount() const
+int  KoShapeContainer::shapeCount() const
 {
     Q_D(const KoShapeContainer);
-    if (d->children == 0)
+    if (d->model == 0)
         return 0;
-    return d->children->count();
+    return d->model->count();
 }
 
 bool KoShapeContainer::isChildLocked(const KoShape *child) const
 {
     Q_D(const KoShapeContainer);
-    if (d->children == 0)
+    if (d->model == 0)
         return false;
-    return d->children->isChildLocked(child);
+    return d->model->isChildLocked(child);
 }
 
-void KoShapeContainer::setClipping(const KoShape *child, bool clipping)
+void KoShapeContainer::setClipped(const KoShape *child, bool clipping)
 {
     Q_D(KoShapeContainer);
-    if (d->children == 0)
+    if (d->model == 0)
         return;
-    d->children->setClipping(child, clipping);
+    d->model->setClipped(child, clipping);
+}
+
+void KoShapeContainer::setInheritsTransform(const KoShape *shape, bool inherit)
+{
+    Q_D(KoShapeContainer);
+    if (d->model == 0)
+        return;
+    d->model->setInheritsTransform(shape, inherit);
+}
+
+bool KoShapeContainer::inheritsTransform(const KoShape *shape) const
+{
+    Q_D(const KoShapeContainer);
+    if (d->model == 0)
+        return false;
+    return d->model->inheritsTransform(shape);
 }
 
 void KoShapeContainer::paint(QPainter &painter, const KoViewConverter &converter)
@@ -120,43 +142,53 @@ void KoShapeContainer::paint(QPainter &painter, const KoViewConverter &converter
     painter.save();
     paintComponent(painter, converter);
     painter.restore();
-    if (d->children == 0 || d->children->count() == 0)
+    if (d->model == 0 || d->model->count() == 0)
         return;
 
-    QList<KoShape*> sortedObjects = d->children->childShapes();
+    QList<KoShape*> sortedObjects = d->model->shapes();
     qSort(sortedObjects.begin(), sortedObjects.end(), KoShape::compareShapeZIndex);
 
-    QMatrix baseMatrix = absoluteTransformation(0).inverted() * painter.matrix();
+    // Do the following to revert the absolute transformation of the container
+    // that is re-applied in shape->absoluteTransformation() later on. The transformation matrix
+    // of the container has already been applied once before this function is called.
+    QTransform baseMatrix = absoluteTransformation(&converter).inverted() * painter.transform();
 
     // clip the children to the parent outline.
-    QMatrix m;
+    QTransform m;
     qreal zoomX, zoomY;
     converter.zoom(&zoomX, &zoomY);
     m.scale(zoomX, zoomY);
     painter.setClipPath(m.map(outline()));
 
+    QRectF toPaintRect = converter.viewToDocument(painter.clipRegion().boundingRect());
+    toPaintRect = transform().mapRect(toPaintRect);
+    // We'll use this clipRect to see if our child shapes lie within it.
+    // Because shape->boundingRect() uses absoluteTransformation(0) we'll
+    // use that as well to have the same (absolute) reference transformation
+    // of our and the child's bounding boxes.
+    QTransform absTrans = absoluteTransformation(0);
+    QRectF clipRect = absTrans.map(outline()).boundingRect();
+
+
     foreach(KoShape *shape, sortedObjects) {
         //kDebug(30006) <<"KoShapeContainer::painting shape:" << shape->shapeId() <<"," << shape->boundingRect();
-        if (! shape->isVisible())
+        if (!shape->isVisible())
             continue;
-        if (! childClipped(shape))  // the shapeManager will have to draw those, or else we can't do clipRects
+        if (!isClipped(shape))  // the shapeManager will have to draw those, or else we can't do clipRects
+            continue;
+        // don't try to draw a child shape that is not in the clipping rect of the painter.
+        if (!clipRect.intersects(shape->boundingRect()))
+
             continue;
-        if (painter.hasClipping()) {
-            QRectF rect = converter.viewToDocument(painter.clipRegion().boundingRect());
-            rect = matrix().mapRect(rect);
-            // don't try to draw a child shape that is not in the clipping rect of the painter.
-            if (! rect.intersects(shape->boundingRect()))
-                continue;
-        }
 
         painter.save();
-        painter.setMatrix(shape->absoluteTransformation(&converter) * baseMatrix);
+        painter.setTransform(shape->absoluteTransformation(&converter) * baseMatrix);
         shape->paint(painter, converter);
         painter.restore();
         if (shape->border()) {
             painter.save();
-            painter.setMatrix(shape->absoluteTransformation(&converter) * baseMatrix);
-            shape->border()->paintBorder(shape, painter, converter);
+            painter.setTransform(shape->absoluteTransformation(&converter) * baseMatrix);
+            shape->border()->paint(shape, painter, converter);
             painter.restore();
         }
     }
@@ -166,45 +198,49 @@ void KoShapeContainer::shapeChanged(ChangeType type, KoShape *shape)
 {
     Q_D(KoShapeContainer);
     Q_UNUSED(shape);
-    if (d->children == 0)
+    if (d->model == 0)
         return;
     if (!(type == RotationChanged || type == ScaleChanged || type == ShearChanged
-            || type == SizeChanged || type == PositionChanged))
+            || type == SizeChanged || type == PositionChanged || type == GenericMatrixChange))
         return;
-    d->children->containerChanged(this);
-    foreach(KoShape *shape, d->children->childShapes())
+    d->model->containerChanged(this, type);
+    foreach(KoShape *shape, d->model->shapes())
         shape->notifyChanged();
 }
 
-bool KoShapeContainer::childClipped(const KoShape *child) const
+bool KoShapeContainer::isClipped(const KoShape *child) const
 {
     Q_D(const KoShapeContainer);
-    if (d->children == 0) // throw exception??
+    if (d->model == 0) // throw exception??
         return false;
-    return d->children->childClipped(child);
+    return d->model->isClipped(child);
 }
 
 void KoShapeContainer::update() const
 {
     Q_D(const KoShapeContainer);
     KoShape::update();
-    if (d->children)
-        foreach(KoShape *shape, d->children->childShapes())
+    if (d->model)
+        foreach(KoShape *shape, d->model->shapes())
             shape->update();
 }
 
-QList<KoShape*> KoShapeContainer::childShapes() const
+QList<KoShape*> KoShapeContainer::shapes() const
 {
     Q_D(const KoShapeContainer);
-    if (d->children == 0)
+    if (d->model == 0)
         return QList<KoShape*>();
 
-    return d->children->childShapes();
+    return d->model->shapes();
 }
 
 KoShapeContainerModel *KoShapeContainer::model() const
 {
     Q_D(const KoShapeContainer);
-    return d->children;
+    return d->model;
 }
 
+void KoShapeContainer::saveOdfChildElements(KoShapeSavingContext &context) const
+{
+    Q_UNUSED(context);
+}
