@@ -49,7 +49,7 @@ QRectF KoShapeLayer::boundingRect() const
 {
     QRectF bb;
 
-    foreach(KoShape* shape, childShapes()) {
+    foreach(KoShape* shape, shapes()) {
         if (bb.isEmpty())
             bb = shape->boundingRect();
         else
@@ -61,10 +61,7 @@ QRectF KoShapeLayer::boundingRect() const
 
 void KoShapeLayer::saveOdf(KoShapeSavingContext & context) const
 {
-    // save later according to parag 9.1.3
-    context.addLayerForSaving(this);
-
-    QList<KoShape*> shapes = childShapes();
+    QList<KoShape*> shapes = this->shapes();
     qSort(shapes.begin(), shapes.end(), KoShape::compareShapeZIndex);
 
     foreach(KoShape* shape, shapes) {
@@ -86,3 +83,7 @@ bool KoShapeLayer::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &
 
     return true;
 }
+
+void KoShapeLayer::paintComponent(QPainter &, const KoViewConverter &)
+{
+}
