@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2008-2009 Jan Hambrecht <jaham@gmx.net>
+ * Copyright (C) 2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -68,10 +69,10 @@ void KoShapeShadow::paint(KoShape *shape, QPainter &painter, const KoViewConvert
         return;
 
     // calculate the shadow offset independent of shape transformation
-    QMatrix tm;
+    QTransform tm;
     tm.translate(d->offset.x(), d->offset.y());
-    QMatrix tr = shape->absoluteTransformation(&converter);
-    QMatrix offsetMatrix = tr * tm * tr.inverted();
+    QTransform tr = shape->absoluteTransformation(&converter);
+    QTransform offsetMatrix = tr * tm * tr.inverted();
 
     if (shape->background()) {
         painter.save();
@@ -80,7 +81,7 @@ void KoShapeShadow::paint(KoShape *shape, QPainter &painter, const KoViewConvert
 
         // the shadow direction is independent of the shapes transformation
         // please only change if you know what you are doing
-        painter.setMatrix(offsetMatrix * painter.matrix());
+        painter.setTransform(offsetMatrix * painter.transform());
         painter.setBrush(QBrush(d->color));
 
         QPainterPath path(shape->outline());
@@ -92,19 +93,19 @@ void KoShapeShadow::paint(KoShape *shape, QPainter &painter, const KoViewConvert
     }
 
     if (shape->border()) {
-        QMatrix oldPainterMatrix = painter.matrix();
+        QTransform oldPainterMatrix = painter.transform();
         KoShape::applyConversion(painter, converter);
-        QMatrix newPainterMatrix = painter.matrix();
+        QTransform newPainterMatrix = painter.transform();
 
         // the shadow direction is independent of the shapes transformation
         // please only change if you know what you are doing
-        painter.setMatrix(offsetMatrix * painter.matrix());
+        painter.setTransform(offsetMatrix * painter.transform());
 
-        // compensate applyConversion call in paintBorder
-        QMatrix scaleMatrix = newPainterMatrix * oldPainterMatrix.inverted();
-        painter.setMatrix(scaleMatrix.inverted() * painter.matrix());
+        // compensate applyConversion call in paint
+        QTransform scaleMatrix = newPainterMatrix * oldPainterMatrix.inverted();
+        painter.setTransform(scaleMatrix.inverted() * painter.transform());
 
-        shape->border()->paintBorder(shape, painter, converter, d->color);
+        shape->border()->paint(shape, painter, converter, d->color);
     }
 }
 
@@ -128,7 +129,7 @@ QColor KoShapeShadow::color() const
     return d->color;
 }
 
-void KoShapeShadow::setVisibility(bool visible)
+void KoShapeShadow::setVisible(bool visible)
 {
     d->visible = visible;
 }
@@ -138,12 +139,15 @@ bool KoShapeShadow::isVisible() const
     return d->visible;
 }
 
-void KoShapeShadow::insets(const KoShape *shape, KoInsets &insets)
+void KoShapeShadow::insets(KoInsets &insets) const
 {
-    if (! d->visible)
+    if (!d->visible) {
+        insets.top = 0;
+        insets.bottom = 0;
+        insets.left = 0;
+        insets.right = 0;
         return;
-
-    Q_UNUSED(shape);
+    }
 
     insets.left = (d->offset.x() < 0.0) ? qAbs(d->offset.x()) : 0.0;
     insets.top = (d->offset.y() < 0.0) ? qAbs(d->offset.y()) : 0.0;
@@ -151,12 +155,12 @@ void KoShapeShadow::insets(const KoShape *shape, KoInsets &insets)
     insets.bottom = (d->offset.y() > 0.0) ? d->offset.y() : 0.0;
 }
 
-void KoShapeShadow::addUser()
+bool KoShapeShadow::ref()
 {
-    d->refCount.ref();
+    return d->refCount.ref();
 }
 
-bool KoShapeShadow::removeUser()
+bool KoShapeShadow::deref()
 {
     return d->refCount.deref();
 }
