@@ -18,6 +18,7 @@
  */
 
 #include "KoSnapStrategy.h"
+#include "KoSnapProxy.h"
 #include "KoSnapGuide.h"
 #include <KoPathShape.h>
 #include <KoPathPoint.h>
@@ -31,7 +32,7 @@
 #include <math.h>
 
 
-KoSnapStrategy::KoSnapStrategy(KoSnapStrategy::SnapType type)
+KoSnapStrategy::KoSnapStrategy(KoSnapGuide::Strategy type)
         : m_snapType(type)
 {
 }
@@ -46,7 +47,7 @@ void KoSnapStrategy::setSnappedPosition(const QPointF &position)
     m_snappedPosition = position;
 }
 
-KoSnapStrategy::SnapType KoSnapStrategy::type() const
+KoSnapGuide::Strategy KoSnapStrategy::type() const
 {
     return m_snapType;
 }
@@ -64,7 +65,7 @@ qreal KoSnapStrategy::scalarProduct(const QPointF &p1, const QPointF &p2)
 }
 
 OrthogonalSnapStrategy::OrthogonalSnapStrategy()
-        : KoSnapStrategy(KoSnapStrategy::Orthogonal)
+        : KoSnapStrategy(KoSnapGuide::OrthogonalSnapping)
 {
 }
 
@@ -77,7 +78,7 @@ bool OrthogonalSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy * pr
     QList<KoShape*> shapes = proxy->shapes();
     foreach(KoShape * shape, shapes) {
         QList<QPointF> points = proxy->pointsFromShape(shape);
-        foreach(const QPointF & point, points) {
+        foreach (const QPointF &point, points) {
             qreal dx = fabs(point.x() - mousePosition.x());
             if (dx < minHorzDist && dx < maxSnapDistance) {
                 minHorzDist = dx;
@@ -130,7 +131,7 @@ QPainterPath OrthogonalSnapStrategy::decoration(const KoViewConverter &converter
 }
 
 NodeSnapStrategy::NodeSnapStrategy()
-        : KoSnapStrategy(KoSnapStrategy::Node)
+        : KoSnapStrategy(KoSnapGuide::NodeSnapping)
 {
 }
 
@@ -145,7 +146,7 @@ bool NodeSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy * proxy, q
 
     QPointF snappedPoint = mousePosition;
 
-    foreach(const QPointF & point, points) {
+    foreach (const QPointF &point, points) {
         qreal distance = squareDistance(mousePosition, point);
         if (distance < maxDistance && distance < minDistance) {
             snappedPoint = point;
@@ -168,7 +169,7 @@ QPainterPath NodeSnapStrategy::decoration(const KoViewConverter &converter) cons
 }
 
 ExtensionSnapStrategy::ExtensionSnapStrategy()
-        : KoSnapStrategy(KoSnapStrategy::Extension)
+        : KoSnapStrategy(KoSnapGuide::ExtensionSnapping)
 {
 }
 
@@ -186,14 +187,14 @@ bool ExtensionSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy * pro
         if (! path)
             continue;
 
-        QMatrix matrix = path->absoluteTransformation(0);
+        QTransform matrix = path->absoluteTransformation(0);
 
         int subpathCount = path->subpathCount();
         for (int subpathIndex = 0; subpathIndex < subpathCount; ++subpathIndex) {
             if (path->isClosedSubpath(subpathIndex))
                 continue;
 
-            int pointCount = path->pointCountSubpath(subpathIndex);
+            int pointCount = path->subpathPointCount(subpathIndex);
 
             // check the extension from the start point
             KoPathPoint * first = path->pointByIndex(KoPathPointIndex(subpathIndex, 0));
@@ -246,30 +247,30 @@ bool ExtensionSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy * pro
     m_lines.clear();
     // if we have to extension near our mouse position, they might have an intersection
     // near our mouse position which we want to use as the snapped position
-    if (minDistances[0] < HUGE_VAL && minDistances[1] < HUGE_VAL ) {
+    if (minDistances[0] < HUGE_VAL && minDistances[1] < HUGE_VAL) {
         // check if intersection of extension lines is near mouse position
-        KoPathSegment s1( startPoints[0], snappedPoints[0] + snappedPoints[0]-startPoints[0] );
-        KoPathSegment s2( startPoints[1], snappedPoints[1] + snappedPoints[1]-startPoints[1] );
-        QList<QPointF> isects = s1.intersections( s2 );
+        KoPathSegment s1(startPoints[0], snappedPoints[0] + snappedPoints[0]-startPoints[0]);
+        KoPathSegment s2(startPoints[1], snappedPoints[1] + snappedPoints[1]-startPoints[1]);
+        QList<QPointF> isects = s1.intersections(s2);
         if (isects.count() == 1 && squareDistance(isects[0], mousePosition) < maxDistance) {
             // add both extension lines
-            m_lines.append( QLineF(startPoints[0], isects[0]) );
-            m_lines.append( QLineF(startPoints[1], isects[0]) );
+            m_lines.append(QLineF(startPoints[0], isects[0]));
+            m_lines.append(QLineF(startPoints[1], isects[0]));
             setSnappedPosition(isects[0]);
         }
         else {
             // only add nearest extension line of both
             uint index = minDistances[0] < minDistances[1] ? 0 : 1;
-            m_lines.append( QLineF(startPoints[index], snappedPoints[index]) );
+            m_lines.append(QLineF(startPoints[index], snappedPoints[index]));
             setSnappedPosition(snappedPoints[index]);
         }
     }
     else  if (minDistances[0] < HUGE_VAL) {
-        m_lines.append( QLineF(startPoints[0], snappedPoints[0]) );
+        m_lines.append(QLineF(startPoints[0], snappedPoints[0]));
         setSnappedPosition(snappedPoints[0]);
     }
     else if (minDistances[1] < HUGE_VAL) {
-        m_lines.append( QLineF(startPoints[1], snappedPoints[1]) );
+        m_lines.append(QLineF(startPoints[1], snappedPoints[1]));
         setSnappedPosition(snappedPoints[1]);
     }
     else {
@@ -285,14 +286,14 @@ QPainterPath ExtensionSnapStrategy::decoration(const KoViewConverter &converter)
     Q_UNUSED(converter);
 
     QPainterPath decoration;
-    foreach( const QLineF & line, m_lines ) {
+    foreach (const QLineF &line, m_lines) {
         decoration.moveTo(line.p1());
         decoration.lineTo(line.p2());
     }
     return decoration;
 }
 
-bool ExtensionSnapStrategy::snapToExtension(QPointF &position, KoPathPoint * point, const QMatrix &matrix)
+bool ExtensionSnapStrategy::snapToExtension(QPointF &position, KoPathPoint * point, const QTransform &matrix)
 {
     QPointF direction = extensionDirection(point, matrix);
     if (direction.isNull())
@@ -322,7 +323,7 @@ qreal ExtensionSnapStrategy::project(const QPointF &lineStart, const QPointF &li
     return scalar /= diffLength;
 }
 
-QPointF ExtensionSnapStrategy::extensionDirection(KoPathPoint * point, const QMatrix &matrix)
+QPointF ExtensionSnapStrategy::extensionDirection(KoPathPoint * point, const QTransform &matrix)
 {
     KoPathShape * path = point->parent();
     KoPathPointIndex index = path->pathPointIndex(point);
@@ -356,11 +357,11 @@ QPointF ExtensionSnapStrategy::extensionDirection(KoPathPoint * point, const QMa
 }
 
 IntersectionSnapStrategy::IntersectionSnapStrategy()
-        : KoSnapStrategy(KoSnapStrategy::Intersection)
+        : KoSnapStrategy(KoSnapGuide::IntersectionSnapping)
 {
 }
 
-bool IntersectionSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy * proxy, qreal maxSnapDistance)
+bool IntersectionSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy *proxy, qreal maxSnapDistance)
 {
     qreal maxDistance = maxSnapDistance * maxSnapDistance;
     qreal minDistance = HUGE_VAL;
@@ -374,7 +375,7 @@ bool IntersectionSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy *
 
     int segmentCount = segments.count();
     for (int i = 0; i < segmentCount; ++i) {
-        const KoPathSegment& s1 = segments[i];
+        const KoPathSegment &s1 = segments[i];
         for (int j = i + 1; j < segmentCount; ++j) {
             QList<QPointF> isects = s1.intersections(segments[j]);
             //kDebug() << isects.count() << "intersections found";
@@ -405,11 +406,11 @@ QPainterPath IntersectionSnapStrategy::decoration(const KoViewConverter &convert
 }
 
 GridSnapStrategy::GridSnapStrategy()
-        : KoSnapStrategy(KoSnapStrategy::Grid)
+        : KoSnapStrategy(KoSnapGuide::GridSnapping)
 {
 }
 
-bool GridSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy * proxy, qreal maxSnapDistance)
+bool GridSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy *proxy, qreal maxSnapDistance)
 {
     if (! proxy->canvas()->snapToGrid())
         return false;
@@ -468,11 +469,11 @@ QPainterPath GridSnapStrategy::decoration(const KoViewConverter &converter) cons
 }
 
 BoundingBoxSnapStrategy::BoundingBoxSnapStrategy()
-        : KoSnapStrategy(KoSnapStrategy::BoundingBox)
+        : KoSnapStrategy(KoSnapGuide::BoundingBoxSnapping)
 {
 }
 
-bool BoundingBoxSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy * proxy, qreal maxSnapDistance)
+bool BoundingBoxSnapStrategy::snap(const QPointF &mousePosition, KoSnapProxy *proxy, qreal maxSnapDistance)
 {
     qreal maxDistance = maxSnapDistance * maxSnapDistance;
     qreal minDistance = HUGE_VAL;
@@ -553,7 +554,7 @@ QPainterPath BoundingBoxSnapStrategy::decoration(const KoViewConverter &converte
 }
 
 LineGuideSnapStrategy::LineGuideSnapStrategy()
-        : KoSnapStrategy(KoSnapStrategy::GuideLine)
+        : KoSnapStrategy(KoSnapGuide::GuideLineSnapping)
 {
 }
 
