@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
- * Copyright (c) 2006 Boudewijn Rempt <boud@valdyas.org>
+ * Copyright (c) 2006-2010 Boudewijn Rempt <boud@valdyas.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,11 +18,12 @@
  * Boston, MA 02110-1301, USA.
  */
 #include "KoToolProxy.h"
+#include "KoToolProxy_p.h"
 
-#include "KoTool.h"
+#include "KoToolBase.h"
 #include "KoPointerEvent.h"
 #include "KoInputDevice.h"
-#include "KoToolManager.h"
+#include "KoToolManager_p.h"
 #include "KoToolSelection.h"
 #include "KoCanvasBase.h"
 #include "KoCanvasController.h"
@@ -32,135 +33,75 @@
 
 #include <kdebug.h>
 #include <QTimer>
-#include <QPointer>
 
-/* Unused for now.
-#if 0
-namespace
+KoToolProxyPrivate::KoToolProxyPrivate(KoToolProxy *p)
+    : activeTool(0),
+    tabletPressed(false),
+    hasSelection(false),
+    controller(0),
+    parent(p)
 {
-
-    // Helper class to determine when the user might switch between
-    // tablet and mouse.
-    class TabletProximityFilter {
-    public:
-
-        TabletProximityFilter( KoToolManager * manager )
-            : m_manager( manager )
-            {
-            }
-
-        virtual ~TabletProximityFilter();
-
-        bool eventFilter( QObject * object,  QEvent * event ) {
-
-            if ( object == qApp ) {
-                switch( event->type() ) {
-                    case QEvent::TabletEnterProximity:
-                        break;
-                    case QEvent::TabletLeaveProximity:
-                        break;
-                    default:
-                        break;
-                }
-            }
-            return false;
-        }
-
-    private:
-
-        KoToolManager * m_manager;
-    };
-
+    scrollTimer.setInterval(100);
+    mouseLeaveWorkaround = false;
 }
-#endif */
 
-
-class KoToolProxy::Private
+void KoToolProxyPrivate::timeout() // Auto scroll the canvas
 {
-public:
-    Private(KoToolProxy *p)
-        : activeTool(0),
-        tabletPressed(false),
-        hasSelection(false),
-        controller(0),
-        parent(p)
-    {
-        scrollTimer.setInterval(100);
-        mouseLeaveWorkaround = false;
-    }
+    Q_ASSERT(controller);
+    int offsetX = controller->canvasOffsetX();
+    int offsetY = controller->canvasOffsetY();
+    QRectF mouseArea(scrollEdgePoint, QSizeF(10, 10));
+    mouseArea.setTopLeft(mouseArea.center());
 
-    void timeout() // Auto scroll the canvas
-    {
-        Q_ASSERT(controller);
-        int offsetX = controller->canvasOffsetX();
-        int offsetY = controller->canvasOffsetY();
-        // get the points version of 10 pixels offset.
-        QPointF offset = controller->canvas()->viewConverter()->viewToDocument(QPointF(10, 10));
-        QRectF mouseArea(scrollEdgePoint, QSizeF(offset.x(), offset.y()));
-        mouseArea.setTopLeft(mouseArea.center());
+    controller->ensureVisible(mouseArea, true);
 
-        controller->ensureVisible(mouseArea, true);
+    QPoint moved(offsetX - controller->canvasOffsetX(), offsetY - controller->canvasOffsetY());
+    if (moved.x() == 0 && moved.y() == 0)
+        return;
+    scrollEdgePoint += moved;
 
-        QPoint moved(offsetX - controller->canvasOffsetX(), offsetY - controller->canvasOffsetY());
-        if (moved.x() == 0 && moved.y() == 0)
-            return;
-        scrollEdgePoint += controller->canvas()->viewConverter()->viewToDocument(moved);
+    QMouseEvent event(QEvent::MouseMove, scrollEdgePoint, Qt::LeftButton, Qt::LeftButton, 0);
+    KoPointerEvent ev(&event, controller->canvas()->viewConverter()->viewToDocument(scrollEdgePoint));
+    activeTool->mouseMoveEvent(&ev);
+}
 
-        QMouseEvent event(QEvent::MouseMove, scrollEdgePoint.toPoint(), Qt::LeftButton, Qt::LeftButton, 0);
-        KoPointerEvent ev(&event, scrollEdgePoint);
-        activeTool->mouseMoveEvent(&ev);
-    }
+void KoToolProxyPrivate::checkAutoScroll(const KoPointerEvent &event)
+{
+    if (controller == 0) return;
+    if (!activeTool) return;
+    if (!activeTool->wantsAutoScroll()) return;
+    if (!event.isAccepted()) return;
+    if (event.buttons() != Qt::LeftButton) return;
+    scrollEdgePoint = controller->canvas()->viewConverter()->documentToView(event.point).toPoint();
+    if (! scrollTimer.isActive())
+        scrollTimer.start();
+}
 
-    void checkAutoScroll(const KoPointerEvent &event)
-    {
-        if (controller == 0) return;
-        if (!activeTool) return;
-        if (!activeTool->wantsAutoScroll()) return;
-        if (!event.isAccepted()) return;
-        if (event.buttons() != Qt::LeftButton) return;
-        scrollEdgePoint = event.point;
-        if (! scrollTimer.isActive())
-            scrollTimer.start();
-    }
+void KoToolProxyPrivate::selectionChanged(bool newSelection)
+{
+    if (hasSelection == newSelection)
+        return;
+    hasSelection = newSelection;
+    emit parent->selectionChanged(hasSelection);
+}
 
-    void selectionChanged(bool newSelection)
-    {
-        if (hasSelection == newSelection)
-            return;
-        hasSelection = newSelection;
-        emit parent->selectionChanged(hasSelection);
-    }
+bool KoToolProxyPrivate::isActiveLayerEditable()
+{
+    if (!activeTool)
+        return false;
 
-    bool isActiveLayerEditable()
-    {
-        if( ! activeTool )
-            return false;
-
-        KoShapeManager * shapeManager = activeTool->canvas()->shapeManager();
-        KoShapeLayer * activeLayer = shapeManager->selection()->activeLayer();
-        if( activeLayer && ! activeLayer->isEditable() )
-            return false;
-        return true;
-    }
-
-    QPointer<KoTool> activeTool;
-    bool tabletPressed;
-    bool hasSelection;
-    QTimer scrollTimer;
-    QPointF scrollEdgePoint;
-    KoCanvasController *controller;
-    KoToolProxy *parent;
-
-    QPoint mouseDownPoint; // used to determine if the mouse-release is after a drag or a simple click
-
-    bool mouseLeaveWorkaround; // up until at least 4.3.0 we get a mouse move event when the tablet leaves the canvas.
-};
+    KoShapeManager * shapeManager = activeTool->canvas()->shapeManager();
+    KoShapeLayer * activeLayer = shapeManager->selection()->activeLayer();
+    if (activeLayer && !activeLayer->isEditable())
+        return false;
+    return true;
+}
 
 KoToolProxy::KoToolProxy(KoCanvasBase *canvas, QObject *parent)
         : QObject(parent),
-        d(new Private(this))
+        d(new KoToolProxyPrivate(this))
 {
-    KoToolManager::instance()->registerToolProxy(this, canvas);
+    KoToolManager::instance()->priv()->registerToolProxy(this, canvas);
 
     connect(&d->scrollTimer, SIGNAL(timeout()), this, SLOT(timeout()));
 }
@@ -180,16 +121,30 @@ void KoToolProxy::repaintDecorations()
     if (d->activeTool) d->activeTool->repaintDecorations();
 }
 
+#include <KDebug>
 void KoToolProxy::tabletEvent(QTabletEvent *event, const QPointF &point)
 {
+    // don't process tablet events for stylus middle and right mouse button
+    // they will be re-send as mouse events with the correct button. there is no possibility to get the button from the QTabletEvent.
+    if(qFuzzyIsNull(event->pressure()) && d->tabletPressed==false && event->type()!=QEvent::TabletMove) {
+        //kDebug()<<"don't accept tablet event: "<< point;
+        return;
+    }
+    else {
+        // Accept the tablet events as they are useless to parent widgets and they will
+        // get re-send as mouseevents if we don't accept them.
+        //kDebug()<<"accept tablet event: "<< point;
+        event->accept();
+    }
+
     KoInputDevice id(event->device(), event->pointerType(), event->uniqueId());
-    KoToolManager::instance()->switchInputDevice(id);
+    KoToolManager::instance()->priv()->switchInputDevice(id);
 
     KoPointerEvent ev(event, point);
     switch (event->type()) {
     case QEvent::TabletPress:
         ev.setTabletButton(Qt::LeftButton);
-        if (! d->tabletPressed && d->activeTool)
+        if (!d->tabletPressed && d->activeTool)
             d->activeTool->mousePressEvent(&ev);
         d->tabletPressed = true;
         break;
@@ -197,19 +152,19 @@ void KoToolProxy::tabletEvent(QTabletEvent *event, const QPointF &point)
         ev.setTabletButton(Qt::LeftButton);
         d->tabletPressed = false;
         d->scrollTimer.stop();
-        if (d->activeTool) d->activeTool->mouseReleaseEvent(&ev);
+        if (d->activeTool)
+            d->activeTool->mouseReleaseEvent(&ev);
         break;
     case QEvent::TabletMove:
-        if (d->tabletPressed) ev.setTabletButton(Qt::LeftButton);
-        if (d->activeTool) d->activeTool->mouseMoveEvent(&ev);
+        if (d->tabletPressed)
+            ev.setTabletButton(Qt::LeftButton);
+        if (d->activeTool)
+            d->activeTool->mouseMoveEvent(&ev);
         d->checkAutoScroll(ev);
     default:
         ; // ignore the rest.
     }
 
-    // Always accept tablet events as they are useless to parent widgets and they will
-    // get re-send as mouseevents if we don't accept them.
-    event->accept();
     d->mouseLeaveWorkaround = true;
 }
 
@@ -217,22 +172,40 @@ void KoToolProxy::mousePressEvent(QMouseEvent *event, const QPointF &point)
 {
     d->mouseLeaveWorkaround = false;
     KoInputDevice id;
-    KoToolManager::instance()->switchInputDevice(id);
+    KoToolManager::instance()->priv()->switchInputDevice(id);
     d->mouseDownPoint = event->pos();
 
     if (d->tabletPressed) // refuse to send a press unless there was a release first.
         return;
 
     KoPointerEvent ev(event, point);
-    if (d->activeTool) d->activeTool->mousePressEvent(&ev);
-    else event->ignore();
+    if (d->activeTool)
+        d->activeTool->mousePressEvent(&ev);
+    else
+        event->ignore();
+}
+
+void KoToolProxy::mousePressEvent(KoPointerEvent *event)
+{
+    d->mouseLeaveWorkaround = false;
+    KoInputDevice id;
+    KoToolManager::instance()->priv()->switchInputDevice(id);
+    d->mouseDownPoint = event->pos();
+
+    if (d->tabletPressed) // refuse to send a press unless there was a release first.
+        return;
+
+    if (d->activeTool)
+        d->activeTool->mousePressEvent(event);
+    else
+        event->ignore();
 }
 
 void KoToolProxy::mouseDoubleClickEvent(QMouseEvent *event, const QPointF &point)
 {
     d->mouseLeaveWorkaround = false;
     KoInputDevice id;
-    KoToolManager::instance()->switchInputDevice(id);
+    KoToolManager::instance()->priv()->switchInputDevice(id);
     if (d->activeTool == 0) {
         event->ignore();
         return;
@@ -241,7 +214,22 @@ void KoToolProxy::mouseDoubleClickEvent(QMouseEvent *event, const QPointF &point
     KoPointerEvent ev(event, point);
     d->activeTool->mouseDoubleClickEvent(&ev);
     if (! event->isAccepted())
-        d->activeTool->m_canvas->shapeManager()->suggestChangeTool(&ev);
+        d->activeTool->canvas()->shapeManager()->suggestChangeTool(&ev);
+}
+
+void KoToolProxy::mouseDoubleClickEvent(KoPointerEvent *event)
+{
+    d->mouseLeaveWorkaround = false;
+    KoInputDevice id;
+    KoToolManager::instance()->priv()->switchInputDevice(id);
+    if (d->activeTool == 0) {
+        event->ignore();
+        return;
+    }
+
+    d->activeTool->mouseDoubleClickEvent(event);
+    if (!event->isAccepted())
+        d->activeTool->canvas()->shapeManager()->suggestChangeTool(event);
 }
 
 void KoToolProxy::mouseMoveEvent(QMouseEvent *event, const QPointF &point)
@@ -251,7 +239,7 @@ void KoToolProxy::mouseMoveEvent(QMouseEvent *event, const QPointF &point)
         return;
     }
     KoInputDevice id;
-    KoToolManager::instance()->switchInputDevice(id);
+    KoToolManager::instance()->priv()->switchInputDevice(id);
     if (d->activeTool == 0) {
         event->ignore();
         return;
@@ -263,11 +251,29 @@ void KoToolProxy::mouseMoveEvent(QMouseEvent *event, const QPointF &point)
     d->checkAutoScroll(ev);
 }
 
+void KoToolProxy::mouseMoveEvent(KoPointerEvent *event)
+{
+    if (d->mouseLeaveWorkaround) {
+        d->mouseLeaveWorkaround = false;
+        return;
+    }
+    KoInputDevice id;
+    KoToolManager::instance()->priv()->switchInputDevice(id);
+    if (d->activeTool == 0) {
+        event->ignore();
+        return;
+    }
+
+    d->activeTool->mouseMoveEvent(event);
+
+    d->checkAutoScroll(*event);
+}
+
 void KoToolProxy::mouseReleaseEvent(QMouseEvent *event, const QPointF &point)
 {
     d->mouseLeaveWorkaround = false;
     KoInputDevice id;
-    KoToolManager::instance()->switchInputDevice(id);
+    KoToolManager::instance()->priv()->switchInputDevice(id);
     d->scrollTimer.stop();
 
     KoPointerEvent ev(event, point);
@@ -278,8 +284,8 @@ void KoToolProxy::mouseReleaseEvent(QMouseEvent *event, const QPointF &point)
                 && qAbs(d->mouseDownPoint.x() - event->x()) < 5
                 && qAbs(d->mouseDownPoint.y() - event->y()) < 5) {
             // we potentially will change the selection
-            Q_ASSERT(d->activeTool->m_canvas);
-            KoShapeManager *manager = d->activeTool->m_canvas->shapeManager();
+            Q_ASSERT(d->activeTool->canvas());
+            KoShapeManager *manager = d->activeTool->canvas()->shapeManager();
             Q_ASSERT(manager);
             // only change the selection if that will not lead to losing a complex selection
             if (manager->selection()->count() <= 1) {
@@ -299,23 +305,72 @@ void KoToolProxy::mouseReleaseEvent(QMouseEvent *event, const QPointF &point)
     }
 }
 
+void KoToolProxy::mouseReleaseEvent(KoPointerEvent* event)
+{
+    d->mouseLeaveWorkaround = false;
+    KoInputDevice id;
+    KoToolManager::instance()->priv()->switchInputDevice(id);
+    d->scrollTimer.stop();
+
+    if (d->activeTool) {
+        d->activeTool->mouseReleaseEvent(event);
+
+        if (!event->isAccepted() && event->button() == Qt::LeftButton && event->modifiers() == 0
+                && qAbs(d->mouseDownPoint.x() - event->x()) < 5
+                && qAbs(d->mouseDownPoint.y() - event->y()) < 5) {
+            // we potentially will change the selection
+            Q_ASSERT(d->activeTool->canvas());
+            KoShapeManager *manager = d->activeTool->canvas()->shapeManager();
+            Q_ASSERT(manager);
+            // only change the selection if that will not lead to losing a complex selection
+            if (manager->selection()->count() <= 1) {
+                KoShape *shape = manager->shapeAt(event->point);
+                if (shape && !manager->selection()->isSelected(shape)) { // make the clicked shape the active one
+                    manager->selection()->deselectAll();
+                    manager->selection()->select(shape);
+                    QList<KoShape*> shapes;
+                    shapes << shape;
+                    QString tool = KoToolManager::instance()->preferredToolForSelection(shapes);
+                    KoToolManager::instance()->switchToolRequested(tool);
+                }
+            }
+        }
+    } else {
+        event->ignore();
+    }
+}
+
 void KoToolProxy::keyPressEvent(QKeyEvent *event)
 {
-    if (d->activeTool) d->activeTool->keyPressEvent(event);
-    else event->ignore();
+    if (d->activeTool)
+        d->activeTool->keyPressEvent(event);
+    else
+        event->ignore();
 }
 
 void KoToolProxy::keyReleaseEvent(QKeyEvent *event)
 {
-    if (d->activeTool) d->activeTool->keyReleaseEvent(event);
-    else event->ignore();
+    if (d->activeTool)
+        d->activeTool->keyReleaseEvent(event);
+    else
+        event->ignore();
 }
 
-void KoToolProxy::wheelEvent(QWheelEvent * event, const QPointF &point)
+void KoToolProxy::wheelEvent(QWheelEvent *event, const QPointF &point)
 {
     KoPointerEvent ev(event, point);
-    if (d->activeTool) d->activeTool->wheelEvent(&ev);
-    else event->ignore();
+    if (d->activeTool)
+        d->activeTool->wheelEvent(&ev);
+    else
+        event->ignore();
+}
+
+void KoToolProxy::wheelEvent(KoPointerEvent *event)
+{
+    if (d->activeTool)
+        d->activeTool->wheelEvent(event);
+    else
+        event->ignore();
 }
 
 QVariant KoToolProxy::inputMethodQuery(Qt::InputMethodQuery query, const KoViewConverter &converter) const
@@ -337,7 +392,7 @@ KoToolSelection* KoToolProxy::selection()
     return 0;
 }
 
-void KoToolProxy::setActiveTool(KoTool *tool)
+void KoToolProxy::setActiveTool(KoToolBase *tool)
 {
     if (d->activeTool)
         disconnect(d->activeTool, SIGNAL(selectionChanged(bool)), this, SLOT(selectionChanged(bool)));
@@ -349,9 +404,9 @@ void KoToolProxy::setActiveTool(KoTool *tool)
     }
 }
 
-void KoToolProxy::setCanvasController(KoCanvasController *controller)
+void KoToolProxyPrivate::setCanvasController(KoCanvasController *c)
 {
-    d->controller = controller;
+    controller = c;
 }
 
 QHash<QString, KAction*> KoToolProxy::actions() const
@@ -401,4 +456,19 @@ void KoToolProxy::deleteSelection()
         return d->activeTool->deleteSelection();
 }
 
+void KoToolProxy::processEvent(QEvent *e) const
+{
+    if(e->type()==QEvent::ShortcutOverride
+       && d->activeTool
+       && d->activeTool->isInTextMode()
+       && static_cast<QKeyEvent*>(e)->modifiers()==Qt::NoModifier) {
+        e->accept();
+    }
+}
+
+KoToolProxyPrivate *KoToolProxy::priv()
+{
+    return d;
+}
+
 #include <KoToolProxy.moc>
