@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2007, 2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,8 +32,8 @@ public:
             return;
         m_members.append(child);
     }
-    void setClipping(const KoShape *, bool) { }
-    bool childClipped(const KoShape *) const {
+    void setClipped(const KoShape *, bool) { }
+    bool isClipped(const KoShape *) const {
         return false;
     }
     void remove(KoShape *child) {
@@ -42,14 +42,18 @@ public:
     int count() const {
         return m_members.count();
     }
-    QList<KoShape*> childShapes() const {
+    QList<KoShape*> shapes() const {
         return QList<KoShape*>(m_members);
     }
-    void containerChanged(KoShapeContainer *) { }
+    void containerChanged(KoShapeContainer *, KoShape::ChangeType) { }
     bool isChildLocked(const KoShape *child) const {
         Q_ASSERT(child->parent());
         return child->isGeometryProtected() || child->parent()->isGeometryProtected();
     }
+    void setInheritsTransform(const KoShape *, bool ) { }
+    bool inheritsTransform(const KoShape *) const {
+        return false;
+    }
 
 private: // members
     QList <KoShape *> m_members;
