@@ -31,22 +31,22 @@ public:
     ~Private()
     {
         foreach(KoShapeBorderModel* border, oldBorders) {
-            if (border && ! border->removeUser())
+            if (border && !border->deref())
                 delete border;
         }
     }
 
-    void addOldBorder( KoShapeBorderModel * oldBorder )
+    void addOldBorder(KoShapeBorderModel * oldBorder)
     {
         if (oldBorder)
-            oldBorder->addUser();
+            oldBorder->ref();
         oldBorders.append(oldBorder);
     }
 
-    void addNewBorder( KoShapeBorderModel * newBorder )
+    void addNewBorder(KoShapeBorderModel * newBorder)
     {
         if (newBorder)
-            newBorder->addUser();
+            newBorder->ref();
         newBorders.append(newBorder);
     }
 
@@ -83,7 +83,7 @@ KoShapeBorderCommand::KoShapeBorderCommand(const QList<KoShape*> &shapes,
     // save old borders
     foreach(KoShape *shape, shapes)
         d->addOldBorder(shape->border());
-    foreach(KoShapeBorderModel * border, borders )
+    foreach (KoShapeBorderModel * border, borders)
         d->addNewBorder(border);
 
     setText(i18n("Set border"));
