@@ -26,46 +26,64 @@
 #include <kdebug.h>
 #include <limits.h>
 
-KoShapeReorderCommand::KoShapeReorderCommand(const QList<KoShape*> &shapes, QList<int> &newIndexes, QUndoCommand *parent)
-        : QUndoCommand(parent)
-        , m_shapes(shapes)
-        , m_newIndexes(newIndexes)
+
+class KoShapeReorderCommandPrivate
 {
-    Q_ASSERT(m_shapes.count() == m_newIndexes.count());
-    foreach(KoShape *shape, shapes)
-        m_previousIndexes.append(shape->zIndex());
+public:
+    KoShapeReorderCommandPrivate(const QList<KoShape*> &s, QList<int> &ni)
+        : shapes(s), newIndexes(ni)
+    {
+    }
+
+    QList<KoShape*> shapes;
+    QList<int> previousIndexes;
+    QList<int> newIndexes;
+};
+
+KoShapeReorderCommand::KoShapeReorderCommand(const QList<KoShape*> &shapes, QList<int> &newIndexes, QUndoCommand *parent)
+    : QUndoCommand(parent),
+    d(new KoShapeReorderCommandPrivate(shapes, newIndexes))
+{
+    Q_ASSERT(shapes.count() == newIndexes.count());
+    foreach (KoShape *shape, shapes)
+        d->previousIndexes.append(shape->zIndex());
 
     setText(i18n("Reorder shapes"));
 }
 
+KoShapeReorderCommand::~KoShapeReorderCommand()
+{
+    delete d;
+}
+
 void KoShapeReorderCommand::redo()
 {
     QUndoCommand::redo();
-    for (int i = 0; i < m_shapes.count(); i++) {
-        m_shapes.at(i)->update();
-        m_shapes.at(i)->setZIndex(m_newIndexes.at(i));
-        m_shapes.at(i)->update();
+    for (int i = 0; i < d->shapes.count(); i++) {
+        d->shapes.at(i)->update();
+        d->shapes.at(i)->setZIndex(d->newIndexes.at(i));
+        d->shapes.at(i)->update();
     }
 }
 
 void KoShapeReorderCommand::undo()
 {
     QUndoCommand::undo();
-    for (int i = 0; i < m_shapes.count(); i++) {
-        m_shapes.at(i)->update();
-        m_shapes.at(i)->setZIndex(m_previousIndexes.at(i));
-        m_shapes.at(i)->update();
+    for (int i = 0; i < d->shapes.count(); i++) {
+        d->shapes.at(i)->update();
+        d->shapes.at(i)->setZIndex(d->previousIndexes.at(i));
+        d->shapes.at(i)->update();
     }
 }
 
-void KoShapeReorderCommand::prepare(KoShape * s, QMap<KoShape*, QList<KoShape*> > & newOrder, KoShapeManager * manager, MoveShapeType move)
+static void prepare(KoShape *s, QMap<KoShape*, QList<KoShape*> > &newOrder, KoShapeManager *manager, KoShapeReorderCommand::MoveShapeType move)
 {
-    KoShapeContainer * parent = s->parent();
-    QMap<KoShape*, QList<KoShape*> >::iterator it( newOrder.find( parent ) );
-    if ( it == newOrder.end() ) {
+    KoShapeContainer *parent = s->parent();
+    QMap<KoShape*, QList<KoShape*> >::iterator it(newOrder.find(parent));
+    if (it == newOrder.end()) {
         QList<KoShape*> children;
-        if ( parent != 0 ) {
-            children = parent->childShapes();
+        if (parent != 0) {
+            children = parent->shapes();
         }
         else {
             // get all toplevel shapes
@@ -81,23 +99,23 @@ void KoShapeReorderCommand::prepare(KoShape * s, QMap<KoShape*, QList<KoShape*>
     int index = shapes.indexOf(s);
     if (index != -1) {
         shapes.removeAt(index);
-        switch ( move ) {
-            case BringToFront:
-                index = shapes.size();
-                break;
-            case RaiseShape:
-                if (index < shapes.size()) {
-                    ++index;
-                }
-                break;
-            case LowerShape:
-                if (index > 0) {
-                    --index;
-                }
-                break;
-            case SendToBack:
-                index = 0;
-                break;
+        switch (move) {
+        case KoShapeReorderCommand::BringToFront:
+            index = shapes.size();
+            break;
+        case KoShapeReorderCommand::RaiseShape:
+            if (index < shapes.size()) {
+                ++index;
+            }
+            break;
+        case KoShapeReorderCommand::LowerShape:
+            if (index > 0) {
+                --index;
+            }
+            break;
+        case KoShapeReorderCommand::SendToBack:
+            index = 0;
+            break;
         }
         shapes.insert(index,s);
     }
@@ -111,13 +129,13 @@ KoShapeReorderCommand *KoShapeReorderCommand::createCommand(const QList<KoShape*
     QMap<KoShape*, QList<KoShape*> > newOrder;
     QList<KoShape*> sortedShapes(shapes);
     qSort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
-    if ( move == BringToFront || move == LowerShape ) {
-        for ( int i = 0; i < sortedShapes.size(); ++i ) {
+    if (move == BringToFront || move == LowerShape) {
+        for (int i = 0; i < sortedShapes.size(); ++i) {
             prepare(sortedShapes.at(i), newOrder, manager, move);
         }
     }
     else {
-        for ( int i = sortedShapes.size() - 1; i >= 0; --i ) {
+        for (int i = sortedShapes.size() - 1; i >= 0; --i) {
             prepare(sortedShapes.at(i), newOrder, manager, move);
         }
     }
@@ -125,7 +143,7 @@ KoShapeReorderCommand *KoShapeReorderCommand::createCommand(const QList<KoShape*
 
     QMap<KoShape*, QList<KoShape*> >::iterator newIt(newOrder.begin());
     for (; newIt!= newOrder.end(); ++newIt) {
-        QList<KoShape*> order( newIt.value() );
+        QList<KoShape*> order(newIt.value());
         order.removeAll(0);
         int index = -2^13;
         int pos = 0;
