@@ -29,22 +29,22 @@ public:
     Private() {}
     ~Private() {
         foreach(KoShapeShadow* shadow, oldShadows) {
-            if (shadow && ! shadow->removeUser())
+            if (shadow && !shadow->deref())
                 delete shadow;
         }
     }
 
-    void addOldShadow( KoShapeShadow * oldShadow )
+    void addOldShadow(KoShapeShadow * oldShadow)
     {
         if (oldShadow)
-            oldShadow->addUser();
+            oldShadow->ref();
         oldShadows.append(oldShadow);
     }
 
-    void addNewShadow( KoShapeShadow * newShadow )
+    void addNewShadow(KoShapeShadow * newShadow)
     {
         if (newShadow)
-            newShadow->addUser();
+            newShadow->ref();
         newShadows.append(newShadow);
     }
 
