@@ -18,7 +18,7 @@
 */
 #include "TestShapeBorderCommand.h"
 
-#include "MockShapes.h"
+#include <MockShapes.h>
 #include "KoShapeBorderModel.h"
 #include "KoLineBorder.h"
 #include "KoShapeBorderCommand.h"
@@ -56,7 +56,8 @@ void TestShapeBorderCommand::refCounting()
     QVERIFY(shape1->border() == whiteBorder);
 
     // if white is deleted when deleting cmd1 this will crash
-    whiteBorder->borderInsets(shape1);
+    KoInsets insets;
+    whiteBorder->borderInsets(shape1, insets);
 
     delete cmd2;
     delete shape1;
