@@ -19,31 +19,32 @@
  */
 
 #include "KoCreateShapeStrategy.h"
+#include "KoShapeRubberSelectStrategy_p.h"
 #include "KoCreateShapesTool.h"
+#include "KoShape.h"
 #include "KoShapeRegistry.h"
 #include "KoShapeManager.h"
 #include "KoCanvasBase.h"
-#include "KoShapeFactory.h"
+#include "KoSelection.h"
+#include "KoShapeFactoryBase.h"
 #include "KoShapeController.h"
 
-#include <kdebug.h>
-#include <QUndoCommand>
+#include <QPainter>
 
-KoCreateShapeStrategy::KoCreateShapeStrategy(KoCreateShapesTool *tool, KoCanvasBase *canvas, const QPointF &clicked)
-        : KoShapeRubberSelectStrategy(tool, canvas, clicked, canvas->snapToGrid())
+#include <kdebug.h>
+
+KoCreateShapeStrategy::KoCreateShapeStrategy(KoCreateShapesTool *tool, const QPointF &clicked)
+        : KoShapeRubberSelectStrategy(tool, clicked, tool->canvas()->snapToGrid())
 {
-    KoCreateShapesTool *parent = static_cast<KoCreateShapesTool*>(m_parent);
-    KoShapeFactory *factory = KoShapeRegistry::instance()->value(parent->shapeId());
+    KoCreateShapesTool *parent = static_cast<KoCreateShapesTool*>(d_ptr->tool);
+    KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value(parent->shapeId());
     if (factory) {
-        QMap<QString, KoDataCenter *> dataCenterMap;
         const KoProperties *props = parent->shapeProperties();
         KoShape *shape;
         if (props) {
-            // it is ok that the data center map is empty as the shape is never added to the document
-            shape = factory->createShapeAndInit(props, dataCenterMap);
+            shape = factory->createShape(props);
         } else {
-            // it is ok that the data center map is empty as the shape is never added to the document
-            shape = factory->createDefaultShapeAndInit(dataCenterMap);
+            shape = factory->createDefaultShape();
         }
 
         m_outline = shape->outline();
@@ -54,8 +55,9 @@ KoCreateShapeStrategy::KoCreateShapeStrategy(KoCreateShapesTool *tool, KoCanvasB
 
 QUndoCommand* KoCreateShapeStrategy::createCommand()
 {
-    KoCreateShapesTool *parent = static_cast<KoCreateShapesTool*>(m_parent);
-    KoShapeFactory *factory = KoShapeRegistry::instance()->value(parent->shapeId());
+    Q_D(KoShapeRubberSelectStrategy);
+    KoCreateShapesTool *parent = static_cast<KoCreateShapesTool*>(d_ptr->tool);
+    KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value(parent->shapeId());
     if (! factory) {
         kWarning(30006) << "Application requested a shape that is not registered" << parent->shapeId();
         return 0;
@@ -64,12 +66,12 @@ QUndoCommand* KoCreateShapeStrategy::createCommand()
     const KoProperties *props = parent->shapeProperties();
     KoShape *shape;
     if (props)
-        shape = factory->createShapeAndInit(props, parent->m_canvas->shapeController()->dataCenterMap());
+        shape = factory->createShape(props, parent->canvas()->shapeController()->resourceManager());
     else
-        shape = factory->createDefaultShapeAndInit(parent->m_canvas->shapeController()->dataCenterMap());
+        shape = factory->createDefaultShape(parent->canvas()->shapeController()->resourceManager());
     if (shape->shapeId().isEmpty())
         shape->setShapeId(factory->id());
-    QRectF rect = selectRect();
+    QRectF rect = d->selectedRect();
     shape->setPosition(rect.topLeft());
     QSizeF newSize = rect.size();
     // if the user has dragged when creating the shape,
@@ -77,9 +79,9 @@ QUndoCommand* KoCreateShapeStrategy::createCommand()
     if (newSize.width() > 1.0 && newSize.height() > 1.0)
         shape->setSize(newSize);
 
-    QUndoCommand * cmd = parent->m_canvas->shapeController()->addShape(shape);
+    QUndoCommand * cmd = parent->canvas()->shapeController()->addShape(shape);
     if (cmd) {
-        KoSelection *selection = parent->m_canvas->shapeManager()->selection();
+        KoSelection *selection = parent->canvas()->shapeManager()->selection();
         selection->deselectAll();
         selection->select(shape);
     }
@@ -89,11 +91,13 @@ QUndoCommand* KoCreateShapeStrategy::createCommand()
 void KoCreateShapeStrategy::finishInteraction(Qt::KeyboardModifiers modifiers)
 {
     Q_UNUSED(modifiers);
-    m_canvas->updateCanvas(selectRect());
+    Q_D(KoShapeRubberSelectStrategy);
+    d->tool->canvas()->updateCanvas(d->selectedRect());
 }
 
 void KoCreateShapeStrategy::paint(QPainter &painter, const KoViewConverter &converter)
 {
+    Q_D(KoShapeRubberSelectStrategy);
     if (m_outline.isEmpty())
         KoShapeRubberSelectStrategy::paint(painter, converter);
     else {
@@ -105,11 +109,11 @@ void KoCreateShapeStrategy::paint(QPainter &painter, const KoViewConverter &conv
         QBrush sb(selectColor, Qt::SolidPattern);
         painter.setPen(QPen(sb, 0));
         painter.setBrush(sb);
-        QRectF paintRect = converter.documentToView(selectRect());
+        QRectF paintRect = converter.documentToView(d->selectedRect());
 
         qreal xscale = paintRect.width() / m_outlineBoundingRect.width();
         qreal yscale = paintRect.height() / m_outlineBoundingRect.height();
-        QMatrix matrix;
+        QTransform matrix;
         matrix.translate(-m_outlineBoundingRect.left(), -m_outlineBoundingRect.top());
         matrix.scale(xscale, yscale);
         painter.translate(paintRect.left(), paintRect.top());
@@ -117,7 +121,7 @@ void KoCreateShapeStrategy::paint(QPainter &painter, const KoViewConverter &conv
         if (painter.hasClipping())
             paintRect = paintRect.intersect(painter.clipRegion().boundingRect());
 
-        painter.setMatrix(matrix, true);
+        painter.setTransform(matrix, true);
         painter.drawPath(m_outline);
         painter.restore();
     }
@@ -125,7 +129,8 @@ void KoCreateShapeStrategy::paint(QPainter &painter, const KoViewConverter &conv
 
 void KoCreateShapeStrategy::handleMouseMove(const QPointF &point, Qt::KeyboardModifiers modifiers)
 {
+    Q_D(KoShapeRubberSelectStrategy);
     KoShapeRubberSelectStrategy::handleMouseMove(point, modifiers);
     if (! m_outline.isEmpty())
-        m_canvas->updateCanvas(selectRect());
+        d->tool->canvas()->updateCanvas(d->selectedRect());
 }
