@@ -24,16 +24,16 @@
 #include <klocale.h>
 
 KoPanToolFactory::KoPanToolFactory(QObject *parent)
-        : KoToolFactory(parent, KoPanTool_ID, i18n("Pan Tool"))
+        : KoToolFactoryBase(parent, KoPanTool_ID)
 {
     setToolTip(i18n("Pan"));
-    setToolType(mainToolType());
+    setToolType(navigationToolType());
     setPriority(5);
     setIcon("hand");
     setActivationShapeId("flake/always");
 }
 
-KoTool* KoPanToolFactory::createTool(KoCanvasBase *canvas)
+KoToolBase* KoPanToolFactory::createTool(KoCanvasBase *canvas)
 {
     return new KoPanTool(canvas);
 }
