@@ -24,10 +24,9 @@
 #define KOPATHTOOL_H
 
 #include "KoPathShape.h"
-#include "KoTool.h"
+#include "KoToolBase.h"
 #include "KoPathToolSelection.h"
-
-#include "QMap"
+#include <QMap>
 
 class QButtonGroup;
 class KoCanvasBase;
@@ -37,7 +36,7 @@ class KoPathToolHandle;
 class KAction;
 
 /// The tool for editing a KoPathShape or a KoParameterShape
-class FLAKE_TEST_EXPORT KoPathTool : public KoTool
+class FLAKE_TEST_EXPORT KoPathTool : public KoToolBase
 {
     Q_OBJECT
 public:
@@ -63,7 +62,7 @@ public:
     /// reimplemented
     virtual void mouseDoubleClickEvent(KoPointerEvent *event);
     /// reimplemented
-    virtual void activate(bool temporary = false);
+    virtual void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes);
     /// reimplemented
     virtual void deactivate();
 
@@ -104,6 +103,7 @@ private slots:
     void updateActions();
     void pointToLine();
     void pointToCurve();
+    void activate();
 
 private:
 
@@ -138,6 +138,8 @@ private:
     KAction *m_actionConvertToPath;
     QCursor m_selectCursor;
     QCursor m_moveCursor;
+
+    Q_DECLARE_PRIVATE(KoToolBase)
 };
 
 #endif
