@@ -24,7 +24,7 @@
 #include <klocale.h>
 
 KoPathToolFactory::KoPathToolFactory(QObject *parent)
-        : KoToolFactory(parent, "PathToolFactoryId", i18n("Path tool"))
+        : KoToolFactoryBase(parent, "PathToolFactoryId")
 {
     setToolTip(i18n("Path editing tool"));
     setToolType(dynamicToolType());
@@ -37,9 +37,9 @@ KoPathToolFactory::~KoPathToolFactory()
 {
 }
 
-KoTool * KoPathToolFactory::createTool(KoCanvasBase *canvas)
+KoToolBase * KoPathToolFactory::createTool(KoCanvasBase *canvas)
 {
     return new KoPathTool(canvas);
 }
 
-#include "KoPathToolFactory.moc"
+#include <KoPathToolFactory.moc>
