@@ -24,16 +24,16 @@
 #include <klocale.h>
 
 KoZoomToolFactory::KoZoomToolFactory(QObject *parent)
-        : KoToolFactory(parent, "KoZoomToolId", i18n("Zoom Tool"))
+        : KoToolFactoryBase(parent, "KoZoomToolId")
 {
     setToolTip(i18n("Zoom"));
-    setToolType(mainToolType());
+    setToolType(navigationToolType());
     setPriority(5);
     setIcon("zoom-original");
     setActivationShapeId("flake/always");
 }
 
-KoTool* KoZoomToolFactory::createTool(KoCanvasBase *canvas)
+KoToolBase *KoZoomToolFactory::createTool(KoCanvasBase *canvas)
 {
     return new KoZoomTool(canvas);
 }
