@@ -21,7 +21,7 @@
 #include "KoPADocument.h"
 
 #include <KoStore.h>
-#include <KoUndoStack.h>
+#include <KoResourceManager.h>
 #include <KoXmlWriter.h>
 #include <KoXmlReader.h>
 #include <KoOdfStylesReader.h>
@@ -41,7 +41,8 @@
 #include <KoPathShape.h>
 #include <KoLineBorder.h>
 #include <KoXmlNS.h>
-#include <KoDataCenter.h>
+#include <KoProgressUpdater.h>
+#include <KoUpdater.h>
 
 #include "KoPACanvas.h"
 #include "KoPAView.h"
@@ -66,7 +67,7 @@ public:
     QList<KoPAPageBase*> masterPages;
     KoInlineTextObjectManager *inlineTextObjectManager;
     bool rulersVisible;
-    QMap<QString, KoDataCenter *>  dataCenterMap;
+    KoPAPageProvider *pageProvider;
 };
 
 KoPADocument::KoPADocument( QWidget* parentWidget, QObject* parent, bool singleViewMode )
@@ -74,17 +75,14 @@ KoPADocument::KoPADocument( QWidget* parentWidget, QObject* parent, bool singleV
     d(new Private())
 {
     d->inlineTextObjectManager = new KoInlineTextObjectManager(this);
+    d->rulersVisible = false;
 
-    // Ask every shapefactory to populate the dataCenterMap
-    foreach(const QString & id, KoShapeRegistry::instance()->keys())
-    {
-        KoShapeFactory *shapeFactory = KoShapeRegistry::instance()->value(id);
-        shapeFactory->populateDataCenterMap(d->dataCenterMap);
-    }
-
-    //Populate the document undoStack in the dataCenterMap. This can be used later by shapes for their undo/redo mechanism.
-    d->dataCenterMap["UndoStack"] = undoStack();
-    d->dataCenterMap[KoPageProvider::ID] = new KoPAPageProvider();
+    resourceManager()->setUndoStack(undoStack());
+    resourceManager()->setOdfDocument(this);
+    QVariant variant;
+    d->pageProvider = new KoPAPageProvider();
+    variant.setValue<void*>(d->pageProvider);
+    resourceManager()->setResource(KoText::PageProvider, variant);
     loadConfig();
 }
 
@@ -93,7 +91,7 @@ KoPADocument::~KoPADocument()
     saveConfig();
     qDeleteAll( d->pages );
     qDeleteAll( d->masterPages );
-    qDeleteAll( d->dataCenterMap );
+    delete d->pageProvider;
     delete d;
 }
 
@@ -114,11 +112,15 @@ bool KoPADocument::loadXML( const KoXmlDocument & doc, KoStore * )
     return true;
 }
 
-bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
+bool KoPADocument::loadOdf( KoOdfReadStore & odfStore)
 {
-    emit sigProgress( 0 );
+    QPointer<KoUpdater> updater;
+    if (progressUpdater()) {
+        updater = progressUpdater()->startSubtask(1, "KoPADocument::loadOdf");
+        updater->setProgress(0);
+    }
     KoOdfLoadingContext loadingContext( odfStore.styles(), odfStore.store(), componentData() );
-    KoPALoadingContext paContext( loadingContext, dataCenterMap() );
+    KoPALoadingContext paContext(loadingContext, resourceManager());
 
     KoXmlElement content = odfStore.contentDoc().documentElement();
     KoXmlElement realBody ( KoXml::namedItemNS( content, KoXmlNS::office, "body" ) );
@@ -137,11 +139,15 @@ bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
 
     // Load text styles before the corresponding text shapes try to use them!
     KoTextSharedLoadingData * sharedData = new KoTextSharedLoadingData();
-    KoStyleManager * styleManager = dynamic_cast<KoStyleManager *>( dataCenterMap()["StyleManager"] );
-    sharedData->loadOdfStyles(loadingContext, styleManager);
     paContext.addSharedData( KOTEXT_SHARED_LOADING_ID, sharedData );
+    KoStyleManager *styleManager = resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
+
+    sharedData->loadOdfStyles(paContext, styleManager);
 
     d->masterPages = loadOdfMasterPages( odfStore.styles().masterPages(), paContext );
+    if ( !loadOdfProlog( body, paContext ) ) {
+        return false;
+    }
     d->pages = loadOdfPages( body, paContext );
 
     // create pages if there are none
@@ -164,20 +170,10 @@ bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
 
     updatePageCount();
 
-    emit sigProgress( -1 );
+    if (updater) updater->setProgress(100);
     return true;
 }
 
-bool KoPADocument::completeLoading( KoStore* store )
-{
-    bool ok=true;
-    foreach(KoDataCenter *dataCenter, d->dataCenterMap)
-    {
-        ok = ok && dataCenter->completeLoading(store);
-    }
-    return ok;
-}
-
 bool KoPADocument::saveOdf( SavingContext & documentContext )
 {
     KoXmlWriter* contentWriter = documentContext.odfStore.contentWriter();
@@ -187,13 +183,17 @@ bool KoPADocument::saveOdf( SavingContext & documentContext )
     KoGenStyles mainStyles;
     KoXmlWriter * bodyWriter = documentContext.odfStore.bodyWriter();
 
-    KoPASavingContext paContext( *bodyWriter, mainStyles, documentContext.embeddedSaver, 1, KoShapeSavingContext::Store );
+    KoPASavingContext paContext(*bodyWriter, mainStyles, documentContext.embeddedSaver, 1);
 
     saveOdfDocumentStyles( paContext );
 
     bodyWriter->startElement( "office:body" );
     bodyWriter->startElement( odfTagName( true ) );
 
+    if ( !saveOdfProlog( paContext ) ) {
+        return false;
+    }
+
     if ( !saveOdfPages( paContext, d->pages, d->masterPages ) ) {
         return false;
     }
@@ -205,7 +205,7 @@ bool KoPADocument::saveOdf( SavingContext & documentContext )
     bodyWriter->endElement(); // office:odfTagName()
     bodyWriter->endElement(); // office:body
 
-    mainStyles.saveOdfAutomaticStyles( contentWriter, false );
+    mainStyles.saveOdfStyles( KoGenStyles::DocumentAutomaticStyles, contentWriter );
 
     documentContext.odfStore.closeContentWriter();
 
@@ -234,12 +234,6 @@ bool KoPADocument::saveOdf( SavingContext & documentContext )
     return paContext.saveDataCenter( documentContext.odfStore.store(), documentContext.odfStore.manifestWriter() );
 }
 
-bool KoPADocument::completeSaving( KoStore* store)
-{
-    Q_UNUSED(store)
-    return true;
-}
-
 QList<KoPAPageBase *> KoPADocument::loadOdfMasterPages( const QHash<QString, KoXmlElement*> masterStyles, KoPALoadingContext & context )
 {
     context.odfLoadingContext().setUseStylesAutoStyles( true );
@@ -284,6 +278,13 @@ bool KoPADocument::loadOdfEpilogue( const KoXmlElement & body, KoPALoadingContex
     return true;
 }
 
+bool KoPADocument::loadOdfProlog( const KoXmlElement & body, KoPALoadingContext & context )
+{
+    Q_UNUSED( body );
+    Q_UNUSED( context );
+    return true;
+}
+
 bool KoPADocument::saveOdfPages( KoPASavingContext &paContext, QList<KoPAPageBase *> &pages, QList<KoPAPageBase *> &masterPages )
 {
     paContext.addOption( KoPASavingContext::DrawId );
@@ -308,6 +309,12 @@ bool KoPADocument::saveOdfPages( KoPASavingContext &paContext, QList<KoPAPageBas
     return true;
 }
 
+bool KoPADocument::saveOdfProlog( KoPASavingContext & paContext )
+{
+    Q_UNUSED( paContext );
+    return true;
+}
+
 bool KoPADocument::saveOdfEpilogue( KoPASavingContext & paContext )
 {
     Q_UNUSED( paContext );
@@ -372,7 +379,7 @@ void KoPADocument::loadOdfSettings(  const KoXmlDocument & settingsDoc )
 
 void KoPADocument::saveOdfDocumentStyles( KoPASavingContext & context )
 {
-    KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>( dataCenterMap()["StyleManager"] );
+    KoStyleManager *styleManager = resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
     Q_ASSERT( styleManager );
     styleManager->saveOdf( context.mainStyles() );
 }
@@ -508,11 +515,6 @@ void KoPADocument::pageRemoved( KoPAPageBase * page, QUndoCommand * parent )
     Q_UNUSED( parent );
 }
 
-QMap<QString, KoDataCenter *> KoPADocument::dataCenterMap() const
-{
-    return d->dataCenterMap;
-}
-
 KoPAPageBase * KoPADocument::pageByShape( KoShape * shape ) const
 {
     KoShape * parent = shape;
@@ -531,13 +533,13 @@ void KoPADocument::updateViews(KoPAPageBase *page)
     foreach (KoView *view, views()) {
         KoPAView *paView = static_cast<KoPAView *>(view);
         if ( paView->activePage() == page ) {
-            paView->updateActivePage( page );
+            paView->viewMode()->updateActivePage( page );
         }
         else if ( dynamic_cast<KoPAMasterPage *>( page ) ) {
             // if the page changed is a master page, we need to check whether it is the current page's master page
             KoPAPage *activePage = dynamic_cast<KoPAPage *>( paView->activePage() );
             if ( activePage && activePage->masterPage() == page ) {
-                paView->updateActivePage( activePage );
+                paView->viewMode()->updateActivePage( activePage );
             }
         }
     }
@@ -551,7 +553,7 @@ KoPageApp::PageType KoPADocument::pageType() const
 QPixmap KoPADocument::pageThumbnail(KoPAPageBase* page, const QSize& size)
 {
     int pageNumber = pageIndex(page) + 1;
-    static_cast<KoPAPageProvider*>(d->dataCenterMap[KoPAPageProvider::ID])->setMasterPageNumber(pageNumber);
+    d->pageProvider->setPageData(pageNumber, page);
     return page->thumbnail(size);
 }
 
@@ -642,7 +644,7 @@ int KoPADocument::takePage( KoPAPageBase *page )
         {
             KoPAView * kopaView = static_cast<KoPAView*>( view );
             if ( page == kopaView->activePage() ) {
-                kopaView->updateActivePage( newActivePage );
+                kopaView->viewMode()->updateActivePage( newActivePage );
             }
         }
         updatePageCount();
@@ -766,16 +768,11 @@ int KoPADocument::pageCount() const
 
 void KoPADocument::updatePageCount()
 {
-    KoInlineTextObjectManager * om = dynamic_cast<KoInlineTextObjectManager*>( dataCenterMap()["InlineTextObjectManager"] );
-
-    if ( om ) {
+    if (resourceManager()->hasResource(KoText::InlineTextObjectManager)) {
+        QVariant var = resourceManager()->resource(KoText::InlineTextObjectManager);
+        KoInlineTextObjectManager *om = var.value<KoInlineTextObjectManager*>();
         om->setProperty( KoInlineObject::PageCount, pageCount() );
     }
 }
 
-void KoPADocument::insertIntoDataCenterMap(QString key, KoDataCenter *dc)
-{
-    d->dataCenterMap[key] = dc;
-}
-
-#include "KoPADocument.moc"
+#include <KoPADocument.moc>
