@@ -54,7 +54,9 @@ void KoPAPage::saveOdf( KoShapeSavingContext & context ) const
 
     paContext.xmlWriter().startElement( "draw:page" );
     paContext.xmlWriter().addAttribute( "draw:name", paContext.pageName( this ) );
-    paContext.xmlWriter().addAttribute( "koffice:name", name() );
+    if (!name().isEmpty() && name() != paContext.pageName( this )) {
+        paContext.xmlWriter().addAttribute( "koffice:name", name() );
+    }
     paContext.xmlWriter().addAttribute( "draw:id", "page" + QString::number( paContext.page() ) );
     paContext.xmlWriter().addAttribute( "draw:master-page-name", paContext.masterPageName( m_masterPage ) );
     paContext.xmlWriter().addAttribute( "draw:style-name", saveOdfPageStyle( paContext ) );
@@ -155,6 +157,12 @@ void KoPAPage::setDisplayMasterBackground( bool display )
     }
 }
 
+bool KoPAPage::displayShape(KoShape *shape) const
+{
+    Q_UNUSED(shape);
+    return true;
+}
+
 QPixmap KoPAPage::generateThumbnail( const QSize& size )
 {
     // don't paint null pixmap
@@ -182,11 +190,11 @@ void KoPAPage::paintPage( QPainter & painter, KoZoomHandler & zoomHandler )
 
     KoShapePainter shapePainter( getPaintingStrategy() );
     if ( displayMasterShapes() ) {
-        shapePainter.setShapes( masterPage()->childShapes() );
-        shapePainter.paintShapes( painter, zoomHandler );
+        shapePainter.setShapes( masterPage()->shapes() );
+        shapePainter.paint(painter, zoomHandler);
     }
-    shapePainter.setShapes( childShapes() );
-    shapePainter.paintShapes( painter, zoomHandler );
+    shapePainter.setShapes( shapes() );
+    shapePainter.paint(painter, zoomHandler);
 }
 
 void KoPAPage::saveOdfPageStyleData( KoGenStyle &style, KoPASavingContext &paContext ) const
