@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2006-2009 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2006-2010 Thorsten Zachmann <zachmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -47,7 +47,7 @@ KoPAPageBase::KoPAPageBase()
 {
     // Add a default layer
     KoShapeLayer* layer = new KoShapeLayer;
-    addChild(layer);
+    addShape(layer);
 }
 
 KoPAPageBase::~KoPAPageBase()
@@ -79,16 +79,33 @@ void KoPAPageBase::paintBackground( QPainter & painter, const KoViewConverter &
 
 void KoPAPageBase::saveOdfPageContent( KoPASavingContext & paContext ) const
 {
+    saveOdfLayers(paContext);
     saveOdfShapes( paContext );
     saveOdfAnimations( paContext );
     saveOdfPresentationNotes( paContext );
-    paContext.saveLayerSet( paContext.xmlWriter() );
+}
+
+void KoPAPageBase::saveOdfLayers(KoPASavingContext &paContext) const
+{
+    QList<KoShape*> shapes(this->shapes());
+    qSort(shapes.begin(), shapes.end(), KoShape::compareShapeZIndex);
+    foreach(KoShape* shape, shapes) {
+        KoShapeLayer *layer = dynamic_cast<KoShapeLayer*>(shape);
+        if (layer) {
+            paContext.addLayerForSaving(layer);
+        }
+        else {
+            Q_ASSERT(layer);
+            kWarning(30010) << "Page contains non layer where a layer is expected";
+        }
+    }
+    paContext.saveLayerSet(paContext.xmlWriter());
     paContext.clearLayers();
 }
 
 void KoPAPageBase::saveOdfShapes( KoShapeSavingContext &context ) const
 {
-    QList<KoShape*> shapes( childShapes() );
+    QList<KoShape*> shapes(this->shapes());
     QList<KoShape*> tlshapes( shapes );
 
     qSort( tlshapes.begin(), tlshapes.end(), KoShape::compareShapeZIndex );
@@ -100,7 +117,7 @@ void KoPAPageBase::saveOdfShapes( KoShapeSavingContext &context ) const
 
 QString KoPAPageBase::saveOdfPageStyle( KoPASavingContext &paContext ) const
 {
-    KoGenStyle style( KoGenStyle::StyleDrawingPage, "drawing-page" );
+    KoGenStyle style( KoGenStyle::DrawingPageAutoStyle, "drawing-page" );
 
     if ( paContext.isSet( KoShapeSavingContext::AutoStyleInStyleXml ) ) {
         style.setAutoStyleInStylesDotXml( true );
@@ -108,7 +125,7 @@ QString KoPAPageBase::saveOdfPageStyle( KoPASavingContext &paContext ) const
 
     saveOdfPageStyleData( style, paContext );
 
-    return paContext.mainStyles().lookup( style, "dp" );
+    return paContext.mainStyles().insert( style, "dp" );
 }
 
 void KoPAPageBase::saveOdfPageStyleData( KoGenStyle &style, KoPASavingContext &paContext ) const
@@ -116,12 +133,6 @@ void KoPAPageBase::saveOdfPageStyleData( KoGenStyle &style, KoPASavingContext &p
     KoShapeBackground * bg = background();
     if( bg )
         bg->fillStyle( style, paContext );
-
-    if ( paContext.isSet( KoShapeSavingContext::AutoStyleInStyleXml ) ) {
-        style.setAutoStyleInStylesDotXml( true );
-    }
-
-    paContext.mainStyles().lookup( style, paContext.isSet( KoShapeSavingContext::PresentationShape ) ? "pr" : "gr" );
 }
 
 bool KoPAPageBase::saveOdfAnimations( KoPASavingContext & paContext ) const
@@ -148,7 +159,7 @@ bool KoPAPageBase::loadOdf( const KoXmlElement &element, KoShapeLoadingContext &
     loadOdfPageTag(element, paContext);
     styleStack.restore();
 
-    // load layers and shapes 
+    // load layers and shapes
     const KoXmlElement & pageLayerSet = KoXml::namedItemNS( element, KoXmlNS::draw, "layer-set" );
 
     const KoXmlElement & usedPageLayerSet = pageLayerSet.isNull() ? loadingContext.odfLoadingContext().stylesReader().layerSet(): pageLayerSet;
@@ -160,12 +171,12 @@ bool KoPAPageBase::loadOdf( const KoXmlElement &element, KoShapeLoadingContext &
         KoShapeLayer * layer = 0;
         if ( first ) {
             first = false;
-            layer = dynamic_cast<KoShapeLayer *>( childShapes().first() );
+            layer = dynamic_cast<KoShapeLayer *>( shapes().first() );
             Q_ASSERT( layer );
         }
         else {
             layer = new KoShapeLayer();
-            addChild( layer );
+            addShape( layer );
         }
         if ( layer ) {
             layer->setZIndex( layerZIndex++ );
@@ -173,7 +184,7 @@ bool KoPAPageBase::loadOdf( const KoXmlElement &element, KoShapeLoadingContext &
         }
     }
 
-    KoShapeLayer * layer = dynamic_cast<KoShapeLayer *>( childShapes().first() );
+    KoShapeLayer * layer = dynamic_cast<KoShapeLayer *>( shapes().first() );
     if ( layer )
     {
         KoXmlElement child;
@@ -184,7 +195,7 @@ bool KoPAPageBase::loadOdf( const KoXmlElement &element, KoShapeLoadingContext &
             KoShape * shape = KoShapeRegistry::instance()->createShapeFromOdf( child, loadingContext );
             if ( shape ) {
                 if( ! shape->parent() ) {
-                    layer->addChild( shape );
+                    layer->addShape( shape );
                 }
             }
         }
@@ -198,10 +209,11 @@ bool KoPAPageBase::loadOdf( const KoXmlElement &element, KoShapeLoadingContext &
 void KoPAPageBase::loadOdfPageTag( const KoXmlElement &element,
                                    KoPALoadingContext &loadingContext )
 {
+    Q_UNUSED(element);
     KoStyleStack& styleStack = loadingContext.odfLoadingContext().styleStack();
 
     if ( styleStack.hasProperty( KoXmlNS::draw, "fill" ) ) {
-        setBackground( loadOdfFill( element, loadingContext ) );
+        setBackground(loadOdfFill(loadingContext));
     }
 }
 
@@ -226,7 +238,7 @@ QRectF KoPAPageBase::boundingRect() const
 QRectF KoPAPageBase::contentRect() const
 {
     QRectF bb;
-    foreach (KoShape* layer, childShapes()) {
+    foreach (KoShape* layer, shapes()) {
         if (bb.isNull()) {
             bb = layer->boundingRect();
         }
