@@ -46,7 +46,7 @@ KoPABackgroundToolWidget::KoPABackgroundToolWidget( KoPABackgroundTool *tool, QW
     connect( widget.useMasterBackground, SIGNAL( stateChanged( int ) ), this, SLOT( useMasterBackground( int ) ) );
     connect( widget.displayMasterShapes, SIGNAL( stateChanged( int ) ), this, SLOT( displayMasterShapes( int ) ) );
 
-    connect( m_tool->view(), SIGNAL( activePageChanged() ), this, SLOT( slotActivePageChanged() ) );
+    connect( m_tool->view()->proxyObject, SIGNAL( activePageChanged() ), this, SLOT( slotActivePageChanged() ) );
 
     slotActivePageChanged();
 }
@@ -76,9 +76,9 @@ void KoPABackgroundToolWidget::slotActivePageChanged()
 void KoPABackgroundToolWidget::setBackgroundImage()
 {
     // TODO only make images selectable
-    KoImageCollection * collection = dynamic_cast<KoImageCollection *>( m_tool->canvas()->shapeController()->dataCenter( "ImageCollection" ) );
+    KoImageCollection *collection = m_tool->canvas()->shapeController()->resourceManager()->imageCollection();
     Q_ASSERT( collection );
-    KoShape * page = m_tool->canvas()->resourceProvider()->koShapeResource( KoPageApp::CurrentPage );
+    KoShape * page = m_tool->canvas()->resourceManager()->koShapeResource( KoPageApp::CurrentPage );
     Q_ASSERT( page );
     if ( !collection || !page ) {
         return;
@@ -121,7 +121,7 @@ void KoPABackgroundToolWidget::setBackgroundImage()
 
 void KoPABackgroundToolWidget::useMasterBackground( int state )
 {
-    KoPAPage * page = dynamic_cast<KoPAPage *>( m_tool->canvas()->resourceProvider()->koShapeResource( KoPageApp::CurrentPage ) );
+    KoPAPage * page = dynamic_cast<KoPAPage *>( m_tool->canvas()->resourceManager()->koShapeResource( KoPageApp::CurrentPage ) );
     if ( page ) {
         KoPADisplayMasterBackgroundCommand * cmd = new KoPADisplayMasterBackgroundCommand( page, state == Qt::Checked );
         m_tool->canvas()->addCommand( cmd );
@@ -130,11 +130,11 @@ void KoPABackgroundToolWidget::useMasterBackground( int state )
 
 void KoPABackgroundToolWidget::displayMasterShapes( int state )
 {
-    KoPAPage * page = dynamic_cast<KoPAPage *>( m_tool->canvas()->resourceProvider()->koShapeResource( KoPageApp::CurrentPage ) );
+    KoPAPage * page = dynamic_cast<KoPAPage *>( m_tool->canvas()->resourceManager()->koShapeResource( KoPageApp::CurrentPage ) );
     if ( page ) {
         KoPADisplayMasterShapesCommand * cmd = new KoPADisplayMasterShapesCommand( page, state == Qt::Checked );
         m_tool->canvas()->addCommand( cmd );
     }
 }
 
-#include "KoPABackgroundToolWidget.moc"
+#include <KoPABackgroundToolWidget.moc>
