@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C)  2006 Thomas Zander <zander@kde.org>
+ * Copyright (C)  2006, 2010 Thomas Zander <zander@kde.org>
  * Copyright (C)  2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
  * This library is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
 
 #include "kotext_export.h"
 
-
+#include <KoResourceManager.h>
 
 #include <QtCore/QStringList>
 #include <QtCore/QChar>
@@ -41,9 +41,11 @@ KOTEXT_EXPORT QStringList underlineTypeList();
 KOTEXT_EXPORT QStringList underlineStyleList();
 KOTEXT_EXPORT Qt::Alignment alignmentFromString(const QString &align);
 KOTEXT_EXPORT QString alignmentToString(Qt::Alignment align);
+KOTEXT_EXPORT Qt::Alignment valignmentFromString(const QString &align);
+KOTEXT_EXPORT QString valignmentToString(Qt::Alignment align);
 
-/// This enum contains values to be used as keys in the KoCanvasResourceProvider
-enum Options {
+/// This enum contains values to be used as keys in the canvas KoResourceManager
+enum CanvasResource {
     ShowTextFrames =  278622039, ///< boolean that enables painting of frame outlines
     ShowSpaces,         ///< boolean that enables painting of spaces
     ShowTabs,           ///< boolean that enables painting of tabs
@@ -54,13 +56,16 @@ enum Options {
     CurrentTextAnchor = 341899485,   ///<  used by the text plugin whenever the anchor-position is changed
     SelectedTextPosition = 21314576,   ///<  used by the text plugin whenever the alternative selection is changed
     ///  used by the text plugin whenever the alternative selection anchor-position is changed
-    SelectedTextAnchor = 3344189
+    SelectedTextAnchor = 3344189,
+    /** The KoDocumentRdf for the document,
+     this will be a KoDocumentRdfBase when Soprano support is not compiled in. */
+    DocumentRdf
 };
 
 /// For paragraphs each tab definition is represented by this struct.
 struct KOTEXT_EXPORT Tab {
     Tab();
-    qreal position;    ///< distance in point from the start of the text-shape
+    qreal position;    ///< distance in ps-points from the edge of the text-shape
     QTextOption::TabType type;       ///< Determine which type is used.
     QChar delimiter;    ///< If type is DelimitorTab; tab until this char was found in the text.
     KoCharacterStyle::LineType leaderType; // none/single/double
@@ -84,6 +89,21 @@ enum Direction {
     InheritDirection    ///< Direction is unspecified and should come from the container
 };
 
+/**
+ * Text resources per koffice-document.
+ * \sa KoResourceManager KoShapeCollection::resourceManager()
+ */
+enum DocumentResource {
+    ChangeTracker = KoDocumentResource::KoTextStart + 1, ///< KoChangeTracker
+    InlineTextObjectManager, ///< The KoText inline-text-object manager. KoInlineTextObjectManager
+    StyleManager,           ///< The KoStyleManager
+    PageProvider            ///< The KoPageProvider
+};
+
+enum KoTextFrameProperty {
+    TableOfContents = QTextFormat::UserProperty + 1,
+};
+
 /// convert the string version of directions (as specified in XSL and ODF) to the Direction enum
 KOTEXT_EXPORT Direction directionFromString(const QString &direction);
 }
