@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2007, 2009 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,6 +22,8 @@
 
 #include "kotext_export.h"
 
+#include "KoTextDocument.h"
+
 #include <QAbstractTextDocumentLayout>
 #include <QRectF>
 #include <QSizeF>
@@ -35,6 +37,7 @@ class KoStyleManager;
 class QTextLayout;
 class KoInlineTextObjectManager;
 class KoViewConverter;
+class KoImageCollection;
 
 /**
  * KWords text layouter that allows text to flow in multiple frames and around
@@ -48,11 +51,13 @@ public:
 
     /// This struct is a helper for painting of kotext texts.
     struct PaintContext {
-        PaintContext() : viewConverter(0) { }
+        PaintContext() : viewConverter(0), imageCollection(0) { }
         /// the QText context
         QAbstractTextDocumentLayout::PaintContext textContext;
         /// A view converter, when set, is used to find out when the zoom is so low that painting of text is unneeded
         const KoViewConverter *viewConverter;
+
+        KoImageCollection *imageCollection;
     };
 
     /// constructor
@@ -113,6 +118,9 @@ public:
     /// stop layouting the text document until manually restarted.
     void interruptLayout();
 
+    /// get the isInterrupted state from the LayoutState
+    bool isInterrupted() const;
+
     /// Add a shape to the list of shapes that the text can run into.
     void addShape(KoShape *shape);
 
@@ -135,7 +143,7 @@ public:
         /// Asks the layout to stop and restart from the beginning.
         virtual void reset() = 0;
         /// returns true if reset has been called.
-        virtual bool interrupted() = 0;
+        virtual bool isInterrupted() const = 0;
         /// return the number of columns of the line to be layouted
         virtual int numColumns() {
             return 0;
@@ -181,7 +189,7 @@ public:
 
         /// the index in the list of shapes (or frameset) of the shape we are currently layouting.
         int shapeNumber;
-        /// the shape that is currently being layed out
+        /// the shape that is currently being laid out
         KoShape *shape;
         /// The current paragraph layout.
         QTextLayout *layout;
@@ -198,6 +206,23 @@ public:
     /// reimplemented from QAbstractTextDocumentLayout
     virtual void documentChanged(int position, int charsRemoved, int charsAdded);
 
+    /**
+     * Sets the document's resizing method. @see KoTextDocument::setResizeMethod
+     */
+    void setResizeMethod(KoTextDocument::ResizeMethod mode);
+
+    /**
+     * Returns the document's resizing method. @see KoTextDocument::resizeMethod
+     */
+    KoTextDocument::ResizeMethod resizeMethod() const;
+
+signals:
+    void shapeAdded(KoShape *shape);
+    /**
+     * Signal is emitted every time a layout run has finished and all text is positioned.
+     */
+    void finishedLayout();
+
 public slots:
     /// make sure we start a layout run (returns immediately)
     void scheduleLayout();
@@ -225,7 +250,10 @@ protected:
 
 
 private:
+    // Takes care of auto-resizing the text shape
+    Q_PRIVATE_SLOT(d, void adjustSize())
     Q_PRIVATE_SLOT(d, void relayoutPrivate())
+    Q_PRIVATE_SLOT(d, void postLayoutHook())
 
     class Private;
     Private * const d;
