@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2009 Pierre Stirnweiss <pstirnweiss@googlemail.com>
- * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,7 +30,6 @@
 #include "KoTextLocator.h"
 #include "KoTextOdfSaveHelper.h"
 #include "KoTextPaste.h"
-#include "KoUndoStack.h"
 #include "changetracker/KoChangeTracker.h"
 #include "changetracker/KoChangeTrackerElement.h"
 #include "changetracker/KoDeleteChangeMarker.h"
@@ -40,6 +39,7 @@
 #include "styles/KoTableCellStyle.h"
 
 #include <KLocale>
+#include <KUndoStack>
 
 #include <QApplication>
 #include <QFontDatabase>
@@ -107,19 +107,22 @@ void KoTextEditor::Private::documentCommandAdded()
         {}
 
         void undo() {
-            if (m_document.isNull())
+            QTextDocument *doc = m_document.data();
+            if (doc == 0)
                 return;
-            m_document->undo(KoTextDocument(m_document).textEditor()->cursor());
+            doc->undo(KoTextDocument(doc).textEditor()->cursor());
         }
 
         void redo() {
-            if (m_document.isNull())
+            QTextDocument *doc = m_document.data();
+            if (doc == 0)
                 return;
-            m_document->redo(KoTextDocument(m_document).textEditor()->cursor());
+            doc->redo(KoTextDocument(doc).textEditor()->cursor());
         }
 
-        QPointer<QTextDocument> m_document;
+        QWeakPointer<QTextDocument> m_document;
     };
+
     //kDebug() << "editor state: " << editorState << " headcommand: " << headCommand;
     if (!headCommand || editorState == NoOp) {
         headCommand = new QUndoCommand(commandTitle);
@@ -209,21 +212,19 @@ return found;
 
 void KoTextEditor::Private::deleteSelection()
 {
+#ifndef NDEBUG
     KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(document->documentLayout());
     Q_ASSERT(layout);
-    Q_ASSERT(layout->inlineTextObjectManager());
-
+#endif
     QTextCursor delText = QTextCursor(caret);
     if (!delText.hasSelection())
         delText.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor);
+    // XXX: is there are reason for these two unused variables? Side effects? (boud)
     QString text = delText.selectedText();
+    Q_UNUSED(text);
     QTextDocumentFragment selection = delText.selection();
-    if (KoTextDocument(document).changeTracker() && KoTextDocument(document).changeTracker()->isEnabled()) {
-        QTextCharFormat format = caret.charFormat();
-        q->registerTrackedChange(caret, KoGenChange::deleteChange, i18n("Delete"), format, format); //this will replace the text to be deleted by a marker.
-    }
-    else
-        caret.deleteChar();
+    Q_UNUSED(selection);
+    caret.deleteChar();
 }
 
 void KoTextEditor::Private::runDirectionUpdater()
@@ -318,23 +319,66 @@ void KoTextEditor::addCommand(QUndoCommand *command)
     d->updateState(KoTextEditor::Private::Custom, (!command->text().isEmpty())?command->text():i18n("Text"));
     //kDebug() << "will push the custom command: " << command->text();
     d->headCommand = command;
-    KoTextDocument(d->document).undoStack()->push(command);
+    QUndoStack *stack = KoTextDocument(d->document).undoStack();
+    if (stack)
+        stack->push(command);
+    else
+        command->redo();
     //kDebug() << "custom command pushed";
     d->updateState(KoTextEditor::Private::NoOp);
 }
 
 void KoTextEditor::registerTrackedChange(QTextCursor &selection, KoGenChange::Type changeType, QString title, QTextFormat& format, QTextFormat& prevFormat, bool applyToWholeBlock)
 {
+    if (!KoTextDocument(d->document).changeTracker() || !KoTextDocument(d->document).changeTracker()->recordChanges()) {
+        // clear the ChangeTrackerId from the passed in selection, without recursively registring
+        // change tracking again  ;)
+        int start = qMin(selection.position(), selection.anchor());
+        int end = qMax(selection.position(), selection.anchor());
+
+        QTextBlock block = selection.block();
+        if (block.position() > start)
+            block = block.document()->findBlock(start);
+
+        while (block.isValid() && block.position() < end) {
+            QTextBlock::iterator iter = block.begin();
+            while (! iter.atEnd()) {
+                QTextFragment fragment = iter.fragment();
+                if (fragment.position() > end)
+                    break;
+                if (fragment.position() + fragment.length() <= start) {
+                    iter++;
+                    continue;
+                }
+
+                QTextCursor cursor(block);
+                cursor.setPosition(fragment.position());
+                QTextCharFormat fm = cursor.charFormat();
+                if (fm.hasProperty(KoCharacterStyle::ChangeTrackerId)) {
+                    fm.clearProperty(KoCharacterStyle::ChangeTrackerId);
+                    int to = qMin(end, fragment.position() + fragment.length());
+                    cursor.setPosition(to, QTextCursor::KeepAnchor);
+                    cursor.setCharFormat(fm);
+
+                    //Changing the format invalidates the iterator. So, reset the iterator and start from the beginning.
+                    iter = block.begin();
+                    continue;
+                }
+                iter++;
+            }
+            block = block.next();
+        }
+
+
+        return;
+    }
+#ifndef NDEBUG
     KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(d->document->documentLayout());
     Q_ASSERT(layout);
     Q_ASSERT(layout->inlineTextObjectManager());
+#endif
 
-    if (!KoTextDocument(d->document).changeTracker() || !KoTextDocument(d->document).changeTracker()->isEnabled()) {
-        d->clearCharFormatProperty(KoCharacterStyle::ChangeTrackerId);
-        return;
-    }
-
-    if (changeType != KoGenChange::deleteChange) {
+    if (changeType != KoGenChange::DeleteChange) {
         //first check if there already is an identical change registered just before or just after the selection. If so, merge appropriatly.
         //TODO implement for format change. handle the prevFormat/newFormat check.
         QTextCursor checker = QTextCursor(selection);
@@ -352,16 +396,16 @@ void KoTextEditor::registerTrackedChange(QTextCursor &selection, KoGenChange::Ty
         }
         changeId = (idBefore)?idBefore:idAfter;
 
-        switch (changeType) {
-            case KoGenChange::insertChange:
+        switch (changeType) {//TODO: this whole thing actually needs to be done like a visitor. If the selection contains several change regions, the parenting needs to be individualised.
+            case KoGenChange::InsertChange:
                 if (!changeId)
                     changeId = KoTextDocument(d->document).changeTracker()->getInsertChangeId(title, selection.charFormat().property( KoCharacterStyle::ChangeTrackerId ).toInt());
                 break;
-            case KoGenChange::formatChange:
+            case KoGenChange::FormatChange:
                 if (!changeId)
                     changeId = KoTextDocument(d->document).changeTracker()->getFormatChangeId(title, format, prevFormat, d->caret.charFormat().property( KoCharacterStyle::ChangeTrackerId ).toInt());
                 break;
-            case KoGenChange::deleteChange:
+            case KoGenChange::DeleteChange:
                 //this should never be the case
                 break;
         }
@@ -376,62 +420,7 @@ void KoTextEditor::registerTrackedChange(QTextCursor &selection, KoGenChange::Ty
         selection.mergeCharFormat(f);
     }
     else {
-        QTextCursor checker = QTextCursor(selection);
-        KoDeleteChangeMarker *deleteChangemarker = 0;
-        KoDeleteChangeMarker *testMarker;
-        bool backwards = (checker.anchor() > checker.position());
-        int selectionBegin = qMin(checker.anchor(), checker.position());
-        int selectionEnd = qMax(checker.anchor(), checker.position());
-        int changeId;
-        QString prefix;
-        QString sufix;
-        QString delText;
-        checker.setPosition(selectionBegin);
-        testMarker = dynamic_cast<KoDeleteChangeMarker*>(layout->inlineTextObjectManager()->inlineTextObject(checker));
-        if (testMarker) {
-            prefix =  KoTextDocument(d->document).changeTracker()->elementById(testMarker->changeId())->getDeleteData();
-        }
-        checker.setPosition(selectionEnd);
-        if (!checker.atEnd()) {
-            checker.movePosition(QTextCursor::NextCharacter);
-            testMarker = dynamic_cast<KoDeleteChangeMarker*>(layout->inlineTextObjectManager()->inlineTextObject(checker));
-            if (testMarker) {
-                sufix =  KoTextDocument(d->document).changeTracker()->elementById(testMarker->changeId())->getDeleteData();
-            }
-        }
-        checker.setPosition(selectionBegin);
-        while (checker.position() < selectionEnd && !checker.atEnd()) {
-            checker.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
-            if (layout->inlineTextObjectManager()->inlineTextObject(checker)) {
-                testMarker = dynamic_cast<KoDeleteChangeMarker*>(layout->inlineTextObjectManager()->inlineTextObject(checker));
-                if (testMarker) {
-                    delText = delText + KoTextDocument(d->document).changeTracker()->elementById(testMarker->changeId())->getDeleteData();
-                }
-            }
-            else {
-                delText = delText + checker.selectedText();
-            }
-            checker.setPosition(checker.position());
-        }
-
-        QTextDocumentFragment deletedFragment = selection.selection();
-        changeId = KoTextDocument(d->document).changeTracker()->getDeleteChangeId(i18n("Delete"), deletedFragment, selection.charFormat().property( KoCharacterStyle::ChangeTrackerId ).toInt());
-        deleteChangemarker = new KoDeleteChangeMarker(KoTextDocument(d->document).changeTracker());
-        deleteChangemarker->setChangeId(changeId);
-        if (!sufix.isEmpty()) {
-            selection.setPosition(selectionBegin);
-            selection.setPosition(selectionEnd+1, QTextCursor::KeepAnchor);
-            selectionEnd++;
-        }
-        if (!prefix.isEmpty()) {
-            selection.setPosition(selectionBegin-1);
-            selection.setPosition(selectionEnd, QTextCursor::KeepAnchor);
-            selectionBegin--;
-        }
-        layout->inlineTextObjectManager()->insertInlineObject(selection, deleteChangemarker);
-        KoTextDocument(d->document).changeTracker()->elementById(deleteChangemarker->changeId())->setDeleteData(prefix + delText + sufix);
-        if (backwards)
-            selection.movePosition(QTextCursor::PreviousCharacter);
+        //Handled in DeleteCommand
     }
 }
 
@@ -442,7 +431,7 @@ void KoTextEditor::bold(bool bold)
     format.setFontWeight(bold ? QFont::Bold : QFont::Normal);
 
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Bold"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Bold"), format, prevFormat, false);
     d->caret.mergeCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -454,7 +443,7 @@ void KoTextEditor::italic(bool italic)
     format.setFontItalic(italic);
 
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Italic"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Italic"), format, prevFormat, false);
     d->caret.mergeCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -472,7 +461,7 @@ void KoTextEditor::underline(bool underline)
     }
 
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Underline"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Underline"), format, prevFormat, false);
     d->caret.mergeCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -489,7 +478,7 @@ void KoTextEditor::strikeOut(bool strikeout)
         format.setProperty(KoCharacterStyle::StrikeOutStyle, KoCharacterStyle::NoLineStyle);
     }
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Strike Out"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Strike Out"), format, prevFormat, false);
     d->caret.mergeCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -524,7 +513,7 @@ void KoTextEditor::setVerticalTextAlignment(Qt::Alignment align)
     QTextCharFormat format;
     format.setVerticalAlignment(charAlign);
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Set Vertical Alignment"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Vertical Alignment"), format, prevFormat, false);
     d->caret.mergeCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -611,7 +600,7 @@ void KoTextEditor::setFontFamily(const QString &font)
     QTextCharFormat format;
     format.setFontFamily(font);
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Set Font"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Font"), format, prevFormat, false);
     d->caret.mergeCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -622,7 +611,7 @@ void KoTextEditor::setFontSize(int size)
     QTextCharFormat format;
     format.setFontPointSize(size);
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Set Font Size"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Font Size"), format, prevFormat, false);
     d->caret.mergeCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -633,7 +622,7 @@ void KoTextEditor::setTextBackgroundColor(const QColor &color)
     QTextCharFormat format;
     format.setBackground(QBrush(color));
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Set Background Color"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Background Color"), format, prevFormat, false);
     d->caret.mergeCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -644,7 +633,7 @@ void KoTextEditor::setTextColor(const QColor &color)
     QTextCharFormat format;
     format.setForeground(QBrush(color));
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Set Text Color"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Text Color"), format, prevFormat, false);
     d->caret.mergeCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -656,7 +645,7 @@ void KoTextEditor::setStyle(KoCharacterStyle *style)
     QTextCharFormat format;
     style->applyStyle(format);
     QTextCharFormat prevFormat(d->caret.charFormat());
-    registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Set Character Style"), format, prevFormat, false);
+    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Character Style"), format, prevFormat, false);
     d->caret.setCharFormat(format);
     d->updateState(KoTextEditor::Private::NoOp);
 }
@@ -689,7 +678,7 @@ void KoTextEditor::setDefaultFormat()
         QTextCharFormat format;
         defaultCharStyle->applyStyle(format);
         QTextCharFormat prevFormat(d->caret.charFormat());
-        registerTrackedChange(d->caret, KoGenChange::formatChange, i18n("Set default format"), format, prevFormat, false);
+        registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set default format"), format, prevFormat, false);
         d->caret.setCharFormat(format);
     }
     d->updateState(KoTextEditor::Private::NoOp);
@@ -759,28 +748,27 @@ void KoTextEditor::insertInlineObject(KoInlineObject *inliner)
     KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(d->document->documentLayout());
     Q_ASSERT(layout);
     Q_ASSERT(layout->inlineTextObjectManager());
-    layout->inlineTextObjectManager()->insertInlineObject(d->caret, inliner);
+    layout->inlineTextObjectManager()->insertInlineObject(d->caret, inliner, d->caret.charFormat());
     d->updateState(KoTextEditor::Private::NoOp);
 }
 
 void KoTextEditor::insertFrameBreak()
-{//TODO split newLine method in two.
+{
     d->updateState(KoTextEditor::Private::KeyPress, i18n("Insert Break"));
     QTextBlock block = d->caret.block();
-    /*
-    if(d->caret->position() == block.position() && block.length() > 0) { // start of parag
-        QTextBlockFormat bf = d->caret->blockFormat();
-        bf.setPageBreakPolicy(QTextFormat::PageBreak_AlwaysAfter);
-        d->caret->setBlockFormat(bf);
-} else { */
-    QTextBlockFormat bf = d->caret.blockFormat();
-    //       if(d->caret->position() != block.position() + block.length() -1 ||
-    //               bf.pageBreakPolicy() != QTextFormat::PageBreak_Auto) // end of parag or already a pagebreak
-    newLine();
-    bf = d->caret.blockFormat();
-    bf.setProperty(KoParagraphStyle::BreakBefore, true);
-    d->caret.setBlockFormat(bf);
-    //}
+    if (d->caret.position() == block.position() && block.length() > 0) { // start of parag
+        QTextBlockFormat bf = d->caret.blockFormat();
+        bf.setPageBreakPolicy(QTextFormat::PageBreak_AlwaysBefore);
+        d->caret.insertBlock(bf);
+        if (block.textList())
+            block.textList()->remove(block);
+    } else {
+        QTextBlockFormat bf = d->caret.blockFormat();
+        newLine();
+        bf = d->caret.blockFormat();
+        bf.setPageBreakPolicy(QTextFormat::PageBreak_AlwaysBefore); // TODO we should create an autostyle instead
+        d->caret.setBlockFormat(bf);
+    }
     d->updateState(KoTextEditor::Private::NoOp);
 }
 
@@ -951,7 +939,7 @@ void KoTextEditor::insertText(const QString &text)
     if (!d->caret.hasSelection() && d->caret.charFormat().hasProperty(KoCharacterStyle::InlineInstanceId))
         d->clearCharFormatProperty(KoCharacterStyle::InlineInstanceId);
     QTextCharFormat format = d->caret.charFormat();
-    registerTrackedChange(d->caret, KoGenChange::insertChange, i18n("Key Press"), format, format, false);
+    registerTrackedChange(d->caret, KoGenChange::InsertChange, i18n("Key Press"), format, format, false);
     int blockNumber = d->caret.blockNumber();
     d->caret.insertText(text);
 
@@ -1021,7 +1009,6 @@ void KoTextEditor::newLine()
     bf.clearProperty(KoParagraphStyle::UnnumberedListItem);
     bf.clearProperty(KoParagraphStyle::IsListHeader);
     bf.clearProperty(KoParagraphStyle::MasterPageName);
-    bf.clearProperty(KoParagraphStyle::BreakBefore);
     d->caret.setBlockFormat(bf);
     if (nextStyle) {
         QTextBlock block = d->caret.block();
@@ -1056,13 +1043,13 @@ int KoTextEditor::position() const
 
 void KoTextEditor::removeSelectedText()
 {
-//TODO
+    d->caret.removeSelectedText();
 }
 
 void KoTextEditor::select(QTextCursor::SelectionType selection)
 {
     //TODO add selection of previous/next char, and option about hasSelection
-    d->caret.select (selection);
+    d->caret.select(selection);
 }
 
 QString KoTextEditor::selectedText() const
@@ -1123,4 +1110,29 @@ bool KoTextEditor::isBidiDocument() const
     return d->isBidiDocument;
 }
 
-#include "KoTextEditor.moc"
+void KoTextEditor::beginEditBlock()
+{
+    d->updateState(KoTextEditor::Private::Custom);
+    d->caret.beginEditBlock();
+}
+
+void KoTextEditor::endEditBlock()
+{
+    d->caret.endEditBlock();
+    d->updateState(KoTextEditor::Private::NoOp);
+}
+
+void KoTextEditor::finishedLoading()
+{
+    QTextBlock block = d->document->begin();
+    while (!d->isBidiDocument && block.isValid()) {
+        bool rtl = isRightToLeft(block.text());
+        if ((QApplication::isLeftToRight() && rtl) || (QApplication::isRightToLeft() && !rtl)) {
+            d->isBidiDocument = true;
+            emit isBidiUpdated();
+        }
+        block = block.next();
+    }
+}
+
+#include <KoTextEditor.moc>
