@@ -26,18 +26,24 @@
 #include <KoShapeSavingContext.h>
 
 #include <opendocument/KoTextSharedSavingData.h>
+#include "KoTextSopranoRdfModel_p.h"
 
 struct KoTextOdfSaveHelper::Private {
     Private(KoTextShapeData *shapeData, int from, int to)
-            :shapeData(shapeData),
-            from(from),
-            to(to) {}
+        : shapeData(shapeData),
+        from(from),
+        to(to),
+        rdfModel(0)
+    {
+    }
 
     KoShapeSavingContext *context;
     KoTextShapeData *shapeData;
 
     int from;
     int to;
+
+    Soprano::Model *rdfModel; //< This is so cut/paste can serialize the relevant RDF to the clipboard
 };
 
 
@@ -60,7 +66,7 @@ bool KoTextOdfSaveHelper::writeBody()
     bodyWriter.startElement("office:body");
     bodyWriter.startElement(KoOdf::bodyContentElement(KoOdf::Text, true));
 
-    d->shapeData->saveOdf(*d->context, d->from, d->to);
+    d->shapeData->saveOdf(*d->context, 0, d->from, d->to);
 
     bodyWriter.endElement(); // office:element
     bodyWriter.endElement(); // office:body
@@ -74,3 +80,14 @@ KoShapeSavingContext * KoTextOdfSaveHelper::context(KoXmlWriter * bodyWriter, Ko
     d->context = new KoShapeSavingContext(*bodyWriter, mainStyles, embeddedSaver);
     return d->context;
 }
+
+void KoTextOdfSaveHelper::setRdfModel(Soprano::Model *m)
+{
+    d->rdfModel = m;
+}
+
+Soprano::Model *KoTextOdfSaveHelper::rdfModel() const
+{
+    return d->rdfModel;
+}
+
