@@ -43,21 +43,25 @@ public:
     /// reimplemented from KoShapeContainerModel
     virtual void remove(KoShape *child);
     /// reimplemented from KoShapeContainerModel
-    virtual void setClipping(const KoShape *child, bool clipping);
+    virtual void setClipped(const KoShape *child, bool clipping);
     /// reimplemented from KoShapeContainerModel
-    virtual bool childClipped(const KoShape *child) const;
+    virtual bool isClipped(const KoShape *child) const;
     /// reimplemented from KoShapeContainerModel
     virtual int count() const;
     /// reimplemented from KoShapeContainerModel
-    virtual QList<KoShape*> childShapes() const;
+    virtual QList<KoShape*> shapes() const;
     /// reimplemented from KoShapeContainerModel
-    virtual void containerChanged(KoShapeContainer *container);
+    virtual void containerChanged(KoShapeContainer *container, KoShape::ChangeType type);
     /// reimplemented from KoShapeContainerModel
     virtual void proposeMove(KoShape *child, QPointF &move);
     /// reimplemented from KoShapeContainerModel
     virtual void childChanged(KoShape *child, KoShape::ChangeType type);
     /// reimplemented from KoShapeContainerModel
     virtual bool isChildLocked(const KoShape *child) const;
+    /// reimplemented from KoShapeContainerModel
+    virtual void setInheritsTransform(const KoShape *shape, bool inherit);
+    /// reimplemented from KoShapeContainerModel
+    virtual bool inheritsTransform(const KoShape *shape) const;
 
     /// each child that is added due to being anchored in the text has an anchor; register it for rules based placement.
     void addAnchor(KoTextAnchor *anchor);
