@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006, 2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006, 2009-2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
@@ -20,6 +20,8 @@
  */
 
 #include "KoTextShapeData.h"
+#include <KoTextShapeDataBase.h>
+#include <KoTextShapeDataBase_p.h>
 #include "KoTextDocument.h"
 #include "KoTextEditor.h"
 #include "KoTextDocumentLayout.h"
@@ -30,7 +32,6 @@
 #include <QUrl>
 #include <QTextDocument>
 #include <QTextBlock>
-#include <QPointer>
 #include <QMetaObject>
 #include <QTextCursor>
 
@@ -38,6 +39,7 @@
 #include <KoGenStyles.h>
 #include <KoShapeLoadingContext.h>
 #include <KoShapeSavingContext.h>
+#include <rdf/KoDocumentRdfBase.h>
 
 #include <KoXmlWriter.h>
 
@@ -46,14 +48,11 @@
 #include "opendocument/KoTextLoader.h"
 #include "opendocument/KoTextWriter.h"
 
-#include "KoTextDebug.h"
-
-class KoTextShapeDataPrivate
+class KoTextShapeDataPrivate : public KoTextShapeDataBasePrivate
 {
 public:
     KoTextShapeDataPrivate()
-            : document(0),
-            ownsDocument(true),
+            : ownsDocument(true),
             dirty(true),
             offset(0.0),
             position(-1),
@@ -63,32 +62,36 @@ public:
     {
     }
 
-    QTextDocument *document;
-    bool ownsDocument, dirty;
+    virtual ~KoTextShapeDataPrivate()
+    {
+        if (ownsDocument) {
+            delete document;
+        }
+        delete textpage;
+    }
+
+    bool ownsDocument;
+    bool dirty;
     qreal offset;
     int position, endPosition;
-    KoInsets margins;
     KoText::Direction direction;
     KoTextPage *textpage;
 };
 
 
 KoTextShapeData::KoTextShapeData()
-        : d(new KoTextShapeDataPrivate())
+    : KoTextShapeDataBase(*(new KoTextShapeDataPrivate()))
 {
     setDocument(new QTextDocument, true);
 }
 
 KoTextShapeData::~KoTextShapeData()
 {
-    delete d->textpage;
-    if (d->ownsDocument)
-        delete d->document;
-    delete d;
 }
 
 void KoTextShapeData::setDocument(QTextDocument *document, bool transferOwnership)
 {
+    Q_D(KoTextShapeData);
     Q_ASSERT(document);
     if (d->ownsDocument && document != d->document)
         delete d->document;
@@ -118,53 +121,57 @@ void KoTextShapeData::setDocument(QTextDocument *document, bool transferOwnershi
         kodoc.setTextEditor(new KoTextEditor(d->document));
 }
 
-QTextDocument *KoTextShapeData::document()
-{
-    return d->document;
-}
-
 qreal KoTextShapeData::documentOffset() const
 {
+    Q_D(const KoTextShapeData);
     return d->offset;
 }
 
 void KoTextShapeData::setDocumentOffset(qreal offset)
 {
+    Q_D(KoTextShapeData);
     d->offset = offset;
 }
 
 int KoTextShapeData::position() const
 {
+    Q_D(const KoTextShapeData);
     return d->position;
 }
 
 void KoTextShapeData::setPosition(int position)
 {
+    Q_D(KoTextShapeData);
     d->position = position;
 }
 
 int KoTextShapeData::endPosition() const
 {
+    Q_D(const KoTextShapeData);
     return d->endPosition;
 }
 
 void KoTextShapeData::setEndPosition(int position)
 {
+    Q_D(KoTextShapeData);
     d->endPosition = position;
 }
 
 void KoTextShapeData::foul()
 {
+    Q_D(KoTextShapeData);
     d->dirty = true;
 }
 
 void KoTextShapeData::wipe()
 {
+    Q_D(KoTextShapeData);
     d->dirty = false;
 }
 
 bool KoTextShapeData::isDirty() const
 {
+    Q_D(const KoTextShapeData);
     return d->dirty;
 }
 
@@ -173,64 +180,55 @@ void KoTextShapeData::fireResizeEvent()
     emit relayout();
 }
 
-void KoTextShapeData::setShapeMargins(const KoInsets &margins)
-{
-    d->margins = margins;
-}
-
-KoInsets KoTextShapeData::shapeMargins() const
-{
-    return d->margins;
-}
-
 void KoTextShapeData::setPageDirection(KoText::Direction direction)
 {
+    Q_D(KoTextShapeData);
     d->direction = direction;
 }
 
 KoText::Direction KoTextShapeData::pageDirection() const
 {
+    Q_D(const KoTextShapeData);
     return d->direction;
 }
 
-void KoTextShapeData::setPage(KoTextPage* textpage)
+void KoTextShapeData::setPage(KoTextPage *textpage)
 {
+    Q_D(KoTextShapeData);
     delete d->textpage;
     d->textpage = textpage;
 }
 
 KoTextPage* KoTextShapeData::page() const
 {
+    Q_D(const KoTextShapeData);
     return d->textpage;
 }
 
-bool KoTextShapeData::loadOdf(const KoXmlElement & element, KoShapeLoadingContext & context)
+bool KoTextShapeData::loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context, KoDocumentRdfBase *rdfData, KoShape *shape)
 {
-    KoTextLoader loader(context);
+    KoTextLoader loader(context, rdfData, shape);
 
     QTextCursor cursor(document());
-#if 0
-    // TODO if this is really needed we need to keep the format when removing the selection
-    // maybe it should be done in the text shape instead
-    cursor.select(QTextCursor::Document);
-    cursor.removeSelectedText();
-#endif
     loader.loadBody(element, cursor);   // now let's load the body from the ODF KoXmlElement.
     KoTextEditor *editor = KoTextDocument(document()).textEditor();
     if (editor) // at one point we have to get the position from the odf doc instead.
         editor->setPosition(0);
+    editor->finishedLoading();
 
     return true;
 }
 
-void KoTextShapeData::saveOdf(KoShapeSavingContext &context, int from, int to) const
+void KoTextShapeData::saveOdf(KoShapeSavingContext &context, KoDocumentRdfBase *rdfData, int from, int to) const
 {
-    KoTextWriter writer(context);
+    Q_D(const KoTextShapeData);
+    KoTextWriter writer(context, rdfData);
     writer.write(d->document, from, to);
 }
 
 void KoTextShapeData::relayoutFor(KoTextPage &textPage)
 {
+    Q_D(KoTextShapeData);
     KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(d->document->documentLayout());
     if (layout == 0)
         return;
@@ -242,4 +240,4 @@ void KoTextShapeData::relayoutFor(KoTextPage &textPage)
     d->textpage = oldPage;
 }
 
-#include "KoTextShapeData.moc"
+#include <KoTextShapeData.moc>
