@@ -136,7 +136,7 @@ void KoVariable::variableMoved(const KoShape *shape, const QTextDocument *docume
     Q_UNUSED(posInDocument);
 }
 
-const QString &KoVariable::value() const
+QString KoVariable::value() const
 {
     Q_D(const KoVariable);
     return d->value;
@@ -147,8 +147,3 @@ int KoVariable::positionInDocument() const
     Q_D(const KoVariable);
     return d->lastPositionInDocument;
 }
-
-bool KoVariable::loadOdf(const KoXmlElement&, KoShapeLoadingContext&)
-{
-    return true;
-}
