@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2004-2006 David Faure <faure@kde.org>
- * Copyright (C) 2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007, 2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
  * Copyright (C) 2007 Pierre Ducroquet <pinaraf@gmail.com>
  * Copyright (C) 2007-2008 Thorsten Zachmann <zachmann@kde.org>
@@ -33,6 +33,7 @@
 #include <KoXmlNS.h>
 #include <KoOdfStylesReader.h>
 #include <KoOdfLoadingContext.h>
+#include <KoShapeLoadingContext.h>
 
 #include "styles/KoStyleManager.h"
 #include "styles/KoParagraphStyle.h"
@@ -43,18 +44,24 @@
 #include "styles/KoTableColumnStyle.h"
 #include "styles/KoTableRowStyle.h"
 #include "styles/KoTableCellStyle.h"
+#include "styles/KoSectionStyle.h"
 
 class KoTextSharedLoadingData::Private
 {
 public:
+    Private()
+    : applicationDefaultStyle(0)
+    {}
     ~Private() {
         qDeleteAll(paragraphStylesToDelete);
         qDeleteAll(characterStylesToDelete);
         qDeleteAll(listStylesToDelete);
         qDeleteAll(tableStylesToDelete);
         qDeleteAll(tableCellStylesToDelete);
-        // NOTE: Table column/row styles are managed by KoTableColumnAndRowStyleManager
-        //       and are not deleted here.
+        qDeleteAll(tableColumnStylesToDelete);
+        qDeleteAll(tableRowStylesToDelete);
+        qDeleteAll(sectionStylesToDelete);
+        delete applicationDefaultStyle;
     }
 
     // It is possible that automatic-styles in content.xml and styles.xml have the same name
@@ -68,6 +75,7 @@ public:
     QHash<QString, KoTableColumnStyle *>      tableColumnContentDotXmlStyles;
     QHash<QString, KoTableRowStyle *>      tableRowContentDotXmlStyles;
     QHash<QString, KoTableCellStyle *>      tableCellContentDotXmlStyles;
+    QHash<QString, KoSectionStyle *>      sectionContentDotXmlStyles;
     QHash<QString, KoParagraphStyle *> paragraphStylesDotXmlStyles;
     QHash<QString, KoCharacterStyle *> characterStylesDotXmlStyles;
     QHash<QString, KoListStyle *>      listStylesDotXmlStyles;
@@ -75,13 +83,19 @@ public:
     QHash<QString, KoTableColumnStyle *>      tableColumnStylesDotXmlStyles;
     QHash<QString, KoTableRowStyle *>      tableRowStylesDotXmlStyles;
     QHash<QString, KoTableCellStyle *>      tableCellStylesDotXmlStyles;
+    QHash<QString, KoSectionStyle *>      sectionStylesDotXmlStyles;
 
     QList<KoParagraphStyle *> paragraphStylesToDelete;
     QList<KoCharacterStyle *> characterStylesToDelete;
     QList<KoListStyle *> listStylesToDelete;
     QList<KoTableStyle *> tableStylesToDelete;
     QList<KoTableCellStyle *> tableCellStylesToDelete;
+    QList<KoTableColumnStyle *> tableColumnStylesToDelete;
+    QList<KoTableRowStyle *> tableRowStylesToDelete;
+    QList<KoSectionStyle *> sectionStylesToDelete;
     QHash<QString, KoParagraphStyle*> namedParagraphStyles;
+
+    KoCharacterStyle *applicationDefaultStyle;
 };
 
 KoTextSharedLoadingData::KoTextSharedLoadingData()
@@ -94,7 +108,7 @@ KoTextSharedLoadingData::~KoTextSharedLoadingData()
     delete d;
 }
 
-static void addDefaultParagraphStyle(KoOdfLoadingContext &context, const KoXmlElement *styleElem, const KoXmlElement *appDefault, KoStyleManager *styleManager)
+void KoTextSharedLoadingData::addDefaultParagraphStyle(KoShapeLoadingContext &context, const KoXmlElement *styleElem, const KoXmlElement *appDefault, KoStyleManager *styleManager)
 {
     if (styleManager && styleElem) {
         styleManager->defaultParagraphStyle()->loadOdf(styleElem, context);
@@ -104,27 +118,29 @@ static void addDefaultParagraphStyle(KoOdfLoadingContext &context, const KoXmlEl
     }
 }
 
-void KoTextSharedLoadingData::loadOdfStyles(KoOdfLoadingContext &context, KoStyleManager *styleManager)
+void KoTextSharedLoadingData::loadOdfStyles(KoShapeLoadingContext &shapeContext, KoStyleManager *styleManager)
 {
-    addCharacterStyles(context, context.stylesReader().autoStyles("text").values(), ContentDotXml);
-    addCharacterStyles(context, context.stylesReader().autoStyles("text", true).values(), StylesDotXml);
+    KoOdfLoadingContext &context = shapeContext.odfLoadingContext();
+
     // only add styles of office:styles to the style manager
-    addCharacterStyles(context, context.stylesReader().customStyles("text").values(), ContentDotXml | StylesDotXml, styleManager);
+    addCharacterStyles(shapeContext, context.stylesReader().customStyles("text").values(), ContentDotXml | StylesDotXml, styleManager);
+    addCharacterStyles(shapeContext, context.stylesReader().autoStyles("text", true).values(), StylesDotXml);
+    addCharacterStyles(shapeContext, context.stylesReader().autoStyles("text").values(), ContentDotXml);
 
-    addListStyles(context, context.stylesReader().autoStyles("list").values(), ContentDotXml);
-    addListStyles(context, context.stylesReader().autoStyles("list", true).values(), StylesDotXml);
-    addListStyles(context, context.stylesReader().customStyles("list").values(), ContentDotXml | StylesDotXml, styleManager);
+    addListStyles(shapeContext, context.stylesReader().autoStyles("list").values(), ContentDotXml);
+    addListStyles(shapeContext, context.stylesReader().autoStyles("list", true).values(), StylesDotXml);
+    addListStyles(shapeContext, context.stylesReader().customStyles("list").values(), ContentDotXml | StylesDotXml, styleManager);
 
-    addDefaultParagraphStyle(context, context.stylesReader().defaultStyle("paragraph"), context.defaultStylesReader().defaultStyle("paragraph"), styleManager);
+    addDefaultParagraphStyle(shapeContext, context.stylesReader().defaultStyle("paragraph"), context.defaultStylesReader().defaultStyle("paragraph"), styleManager);
     // adding all the styles in order of dependency; automatic styles can have a parent in the named styles, so load the named styles first.
 
     // add office:styles from styles.xml to paragraphContentDotXmlStyles, paragraphStylesDotXmlStyles and styleManager
     // now all styles referencable from the body in content.xml is in paragraphContentDotXmlStyles
-    addParagraphStyles(context, context.stylesReader().customStyles("paragraph").values(), ContentDotXml | StylesDotXml, styleManager);
+    addParagraphStyles(shapeContext, context.stylesReader().customStyles("paragraph").values(), ContentDotXml | StylesDotXml, styleManager);
     // add office:automatic-styles in styles.xml to paragraphStylesDotXmlStyles
-    addParagraphStyles(context, context.stylesReader().autoStyles("paragraph", true).values(), StylesDotXml);
+    addParagraphStyles(shapeContext, context.stylesReader().autoStyles("paragraph", true).values(), StylesDotXml);
     // add office:automatic-styles in content.xml to paragraphContentDotXmlStyles
-    addParagraphStyles(context, context.stylesReader().autoStyles("paragraph").values(), ContentDotXml);
+    addParagraphStyles(shapeContext, context.stylesReader().autoStyles("paragraph").values(), ContentDotXml);
 
     addTableStyles(context, context.stylesReader().autoStyles("table").values(), ContentDotXml);
     addTableStyles(context, context.stylesReader().autoStyles("table", true).values(), StylesDotXml);
@@ -142,13 +158,17 @@ void KoTextSharedLoadingData::loadOdfStyles(KoOdfLoadingContext &context, KoStyl
     addTableCellStyles(context, context.stylesReader().autoStyles("table-cell", true).values(), StylesDotXml);
     addTableCellStyles(context, context.stylesReader().customStyles("table-cell").values(), ContentDotXml | StylesDotXml, styleManager);
 
-    addOutlineStyle(context, styleManager);
+    addSectionStyles(context, context.stylesReader().autoStyles("section").values(), ContentDotXml);
+    addSectionStyles(context, context.stylesReader().autoStyles("section", true).values(), StylesDotXml);
+    addSectionStyles(context, context.stylesReader().customStyles("section").values(), ContentDotXml | StylesDotXml, styleManager);
+
+    addOutlineStyle(shapeContext, styleManager);
 
     kDebug(32500) << "content.xml: paragraph styles" << d->paragraphContentDotXmlStyles.count() << "character styles" << d->characterContentDotXmlStyles.count();
     kDebug(32500) << "styles.xml:  paragraph styles" << d->paragraphStylesDotXmlStyles.count() << "character styles" << d->characterStylesDotXmlStyles.count();
 }
 
-void KoTextSharedLoadingData::addParagraphStyles(KoOdfLoadingContext &context, QList<KoXmlElement*> styleElements,
+void KoTextSharedLoadingData::addParagraphStyles(KoShapeLoadingContext &context, QList<KoXmlElement*> styleElements,
         int styleTypes, KoStyleManager *styleManager)
 {
     QList<QPair<QString, KoParagraphStyle *> > paragraphStyles(loadParagraphStyles(context, styleElements, styleTypes, styleManager));
@@ -164,7 +184,7 @@ void KoTextSharedLoadingData::addParagraphStyles(KoOdfLoadingContext &context, Q
     }
 }
 
-QList<QPair<QString, KoParagraphStyle *> > KoTextSharedLoadingData::loadParagraphStyles(KoOdfLoadingContext &context, QList<KoXmlElement*> styleElements,
+QList<QPair<QString, KoParagraphStyle *> > KoTextSharedLoadingData::loadParagraphStyles(KoShapeLoadingContext &context, QList<KoXmlElement*> styleElements,
         int styleTypes, KoStyleManager *styleManager)
 {
     QList<QPair<QString, KoParagraphStyle *> > paragraphStyles;
@@ -216,33 +236,41 @@ QList<QPair<QString, KoParagraphStyle *> > KoTextSharedLoadingData::loadParagrap
     return paragraphStyles;
 }
 
-void KoTextSharedLoadingData::addCharacterStyles(KoOdfLoadingContext &context, QList<KoXmlElement*> styleElements,
+void KoTextSharedLoadingData::addCharacterStyles(KoShapeLoadingContext &context, QList<KoXmlElement*> styleElements,
         int styleTypes, KoStyleManager *styleManager)
 {
-    QList<QPair<QString, KoCharacterStyle *> > characterStyles(loadCharacterStyles(context, styleElements));
+    QList<OdfCharStyle> characterStyles(loadCharacterStyles(context, styleElements));
 
-    QList<QPair<QString, KoCharacterStyle *> >::iterator it(characterStyles.begin());
-    for (; it != characterStyles.end(); ++it) {
+    foreach (const OdfCharStyle &odfStyle, characterStyles) {
         if (styleTypes & ContentDotXml) {
-            d->characterContentDotXmlStyles.insert(it->first, it->second);
+            d->characterContentDotXmlStyles.insert(odfStyle.odfName, odfStyle.style);
         }
         if (styleTypes & StylesDotXml) {
-            d->characterStylesDotXmlStyles.insert(it->first, it->second);
+            d->characterStylesDotXmlStyles.insert(odfStyle.odfName, odfStyle.style);
         }
 
-        // TODO check if it a know style set the styleid so that the custom styles are kept during copy and paste
-        // in case styles are not added to the style manager they have to be deleted after loading to avoid leaking memeory
         if (styleManager) {
-            styleManager->add(it->second);
+            styleManager->add(odfStyle.style);
         } else {
-            d->characterStylesToDelete.append(it->second);
+            if (!odfStyle.parentStyle.isEmpty()) { // an auto style with a parent.
+                // lets find the parent and set the styleId of that one on the auto-style too.
+                // this will have the effect that whereever the autostyle is applied, it will
+                // cause the parent style-id to be applied. So we don't loose this info.
+                KoCharacterStyle *parent = characterStyle(odfStyle.parentStyle, false);
+                if (!parent)
+                    parent = characterStyle(odfStyle.parentStyle, true); // try harder
+                if (parent)
+                    odfStyle.style->setStyleId(parent->styleId());
+            }
+            d->characterStylesToDelete.append(odfStyle.style);
         }
     }
 }
 
-QList<QPair<QString, KoCharacterStyle *> > KoTextSharedLoadingData::loadCharacterStyles(KoOdfLoadingContext &context, QList<KoXmlElement*> styleElements)
+QList<KoTextSharedLoadingData::OdfCharStyle> KoTextSharedLoadingData::loadCharacterStyles(KoShapeLoadingContext &shapeContext, QList<KoXmlElement*> styleElements)
 {
-    QList<QPair<QString, KoCharacterStyle *> > characterStyles;
+    QList<OdfCharStyle> characterStyles;
+    KoOdfLoadingContext &context = shapeContext.odfLoadingContext();
 
     foreach(KoXmlElement *styleElem, styleElements) {
         Q_ASSERT(styleElem);
@@ -253,6 +281,7 @@ QList<QPair<QString, KoCharacterStyle *> > KoTextSharedLoadingData::loadCharacte
         if (displayName.isEmpty()) {
             displayName = name;
         }
+        QString parent = styleElem->attributeNS(KoXmlNS::style, "parent-style-name");
 
         kDebug(32500) << "styleName =" << name << "styleDisplayName =" << displayName;
 
@@ -263,16 +292,19 @@ QList<QPair<QString, KoCharacterStyle *> > KoTextSharedLoadingData::loadCharacte
 
         KoCharacterStyle *characterStyle = new KoCharacterStyle();
         characterStyle->setName(displayName);
-        characterStyle->loadOdf(context);
-
+        characterStyle->loadOdf(shapeContext);
         context.styleStack().restore();
 
-        characterStyles.append(QPair<QString, KoCharacterStyle *>(name, characterStyle));
+        OdfCharStyle answer;
+        answer.odfName = name;
+        answer.parentStyle = parent;
+        answer.style = characterStyle;
+        characterStyles.append(answer);
     }
     return characterStyles;
 }
 
-void KoTextSharedLoadingData::addListStyles(KoOdfLoadingContext &context, QList<KoXmlElement*> styleElements,
+void KoTextSharedLoadingData::addListStyles(KoShapeLoadingContext &context, QList<KoXmlElement*> styleElements,
                                             int styleTypes, KoStyleManager *styleManager)
 {
     QList<QPair<QString, KoListStyle *> > listStyles(loadListStyles(context, styleElements));
@@ -286,7 +318,7 @@ void KoTextSharedLoadingData::addListStyles(KoOdfLoadingContext &context, QList<
             d->listStylesDotXmlStyles.insert(it->first, it->second);
         }
         // TODO check if it a know style set the styleid so that the custom styles are kept during copy and paste
-        // in case styles are not added to the style manager they have to be deleted after loading to avoid leaking memeory
+        // in case styles are not added to the style manager they have to be deleted after loading to avoid leaking memory
         if (styleManager) {
             styleManager->add(it->second);
         } else {
@@ -295,7 +327,7 @@ void KoTextSharedLoadingData::addListStyles(KoOdfLoadingContext &context, QList<
     }
 }
 
-QList<QPair<QString, KoListStyle *> > KoTextSharedLoadingData::loadListStyles(KoOdfLoadingContext &context, QList<KoXmlElement*> styleElements)
+QList<QPair<QString, KoListStyle *> > KoTextSharedLoadingData::loadListStyles(KoShapeLoadingContext &context, QList<KoXmlElement*> styleElements)
 {
     QList<QPair<QString, KoListStyle *> > listStyles;
 
@@ -365,10 +397,11 @@ void KoTextSharedLoadingData::addTableColumnStyles(KoOdfLoadingContext &context,
             d->tableColumnStylesDotXmlStyles.insert(it->first, it->second);
         }
         // TODO check if it a know style set the styleid so that the custom styles are kept during copy and paste
-        // in case styles are not added to the style manager they will be managed by the
-        // tableColumnAndRowStyleManager instead
+        // in case styles are not added to the style manager they have to be deleted after loading to avoid leaking memeory
         if (styleManager) {
             styleManager->add(it->second);
+        } else {
+            d->tableColumnStylesToDelete.append(it->second);
         }
     }
 }
@@ -404,10 +437,11 @@ void KoTextSharedLoadingData::addTableRowStyles(KoOdfLoadingContext &context, QL
             d->tableRowStylesDotXmlStyles.insert(it->first, it->second);
         }
         // TODO check if it a know style set the styleid so that the custom styles are kept during copy and paste
-        // in case styles are not added to the style manager they will be managed by the
-        // tableColumnAndRowStyleManager instead
+        // in case styles are not added to the style manager they have to be deleted after loading to avoid leaking memeory
         if (styleManager) {
             styleManager->add(it->second);
+        } else {
+            d->tableRowStylesToDelete.append(it->second);
         }
     }
 }
@@ -469,10 +503,50 @@ QList<QPair<QString, KoTableCellStyle *> > KoTextSharedLoadingData::loadTableCel
     return tableCellStyles;
 }
 
-void KoTextSharedLoadingData::addOutlineStyle(KoOdfLoadingContext &context, KoStyleManager *styleManager)
+void KoTextSharedLoadingData::addSectionStyles(KoOdfLoadingContext &context, QList<KoXmlElement*> styleElements,
+                                            int styleTypes, KoStyleManager *styleManager)
+{
+    QList<QPair<QString, KoSectionStyle *> > sectionStyles(loadSectionStyles(context, styleElements));
+
+    QList<QPair<QString, KoSectionStyle *> >::iterator it(sectionStyles.begin());
+    for (; it != sectionStyles.end(); ++it) {
+        if (styleTypes & ContentDotXml) {
+            d->sectionContentDotXmlStyles.insert(it->first, it->second);
+        }
+        if (styleTypes & StylesDotXml) {
+            d->sectionStylesDotXmlStyles.insert(it->first, it->second);
+        }
+        // TODO check if it a know style set the styleid so that the custom styles are kept during copy and paste
+        // in case styles are not added to the style manager they have to be deleted after loading to avoid leaking memeory
+        if (styleManager) {
+            styleManager->add(it->second);
+        } else {
+            d->sectionStylesToDelete.append(it->second);
+        }
+    }
+}
+
+QList<QPair<QString, KoSectionStyle *> > KoTextSharedLoadingData::loadSectionStyles(KoOdfLoadingContext &context, QList<KoXmlElement*> styleElements)
+{
+    QList<QPair<QString, KoSectionStyle *> > sectionStyles;
+
+    foreach(KoXmlElement *styleElem, styleElements) {
+        Q_ASSERT(styleElem);
+        Q_ASSERT(!styleElem->isNull());
+
+        QString name = styleElem->attributeNS(KoXmlNS::style, "name", QString());
+        // nah don't think this is it: context.fillStyleStack(*styleElem, KoXmlNS::style, "style-name", "table");
+        KoSectionStyle *sectionstyle = new KoSectionStyle();
+        sectionstyle->loadOdf(styleElem, context);
+        sectionStyles.append(QPair<QString, KoSectionStyle *>(name, sectionstyle));
+    }
+    return sectionStyles;
+}
+
+void KoTextSharedLoadingData::addOutlineStyle(KoShapeLoadingContext &context, KoStyleManager *styleManager)
 {
     // outline-styles used e.g. for headers
-    KoXmlElement outlineStyleElem = KoXml::namedItemNS(context.stylesReader().officeStyle(), KoXmlNS::text, "outline-style");
+    KoXmlElement outlineStyleElem = KoXml::namedItemNS(context.odfLoadingContext().stylesReader().officeStyle(), KoXmlNS::text, "outline-style");
     if (styleManager && outlineStyleElem.isElement()) {
         KoListStyle *outlineStyle = new KoListStyle();
         outlineStyle->loadOdf(context, outlineStyleElem);
@@ -480,41 +554,61 @@ void KoTextSharedLoadingData::addOutlineStyle(KoOdfLoadingContext &context, KoSt
     }
 }
 
-KoParagraphStyle *KoTextSharedLoadingData::paragraphStyle(const QString &name, bool stylesDotXml)
+KoParagraphStyle *KoTextSharedLoadingData::paragraphStyle(const QString &name, bool stylesDotXml) const
 {
     return stylesDotXml ? d->paragraphStylesDotXmlStyles.value(name) : d->paragraphContentDotXmlStyles.value(name);
 }
 
-KoCharacterStyle *KoTextSharedLoadingData::characterStyle(const QString &name, bool stylesDotXml)
+KoCharacterStyle *KoTextSharedLoadingData::characterStyle(const QString &name, bool stylesDotXml) const
 {
     return stylesDotXml ? d->characterStylesDotXmlStyles.value(name) : d->characterContentDotXmlStyles.value(name);
 }
 
-KoListStyle *KoTextSharedLoadingData::listStyle(const QString &name, bool stylesDotXml)
+QList<KoCharacterStyle*> KoTextSharedLoadingData::characterStyles(bool stylesDotXml) const
+{
+    return stylesDotXml ? d->characterStylesDotXmlStyles.values() : d->characterContentDotXmlStyles.values();
+}
+
+KoListStyle *KoTextSharedLoadingData::listStyle(const QString &name, bool stylesDotXml) const
 {
     return stylesDotXml ? d->listStylesDotXmlStyles.value(name) : d->listContentDotXmlStyles.value(name);
 }
 
-KoTableStyle *KoTextSharedLoadingData::tableStyle(const QString &name, bool stylesDotXml)
+KoTableStyle *KoTextSharedLoadingData::tableStyle(const QString &name, bool stylesDotXml) const
 {
     return stylesDotXml ? d->tableStylesDotXmlStyles.value(name) : d->tableContentDotXmlStyles.value(name);
 }
 
-KoTableColumnStyle *KoTextSharedLoadingData::tableColumnStyle(const QString &name, bool stylesDotXml)
+KoTableColumnStyle *KoTextSharedLoadingData::tableColumnStyle(const QString &name, bool stylesDotXml) const
 {
     return stylesDotXml ? d->tableColumnStylesDotXmlStyles.value(name) : d->tableColumnContentDotXmlStyles.value(name);
 }
 
-KoTableRowStyle *KoTextSharedLoadingData::tableRowStyle(const QString &name, bool stylesDotXml)
+KoTableRowStyle *KoTextSharedLoadingData::tableRowStyle(const QString &name, bool stylesDotXml) const
 {
     return stylesDotXml ? d->tableRowStylesDotXmlStyles.value(name) : d->tableRowContentDotXmlStyles.value(name);
 }
 
-KoTableCellStyle *KoTextSharedLoadingData::tableCellStyle(const QString &name, bool stylesDotXml)
+KoTableCellStyle *KoTextSharedLoadingData::tableCellStyle(const QString &name, bool stylesDotXml) const
 {
     return stylesDotXml ? d->tableCellStylesDotXmlStyles.value(name) : d->tableCellContentDotXmlStyles.value(name);
 }
 
+KoSectionStyle *KoTextSharedLoadingData::sectionStyle(const QString &name, bool stylesDotXml) const
+{
+    return stylesDotXml ? d->sectionStylesDotXmlStyles.value(name) : d->sectionContentDotXmlStyles.value(name);
+}
+
+void KoTextSharedLoadingData::setApplicationDefaultStyle(KoCharacterStyle *applicationDefaultStyle)
+{
+    d->applicationDefaultStyle = applicationDefaultStyle;
+}
+
+KoCharacterStyle *KoTextSharedLoadingData::applicationDefaultStyle() const
+{
+    return d->applicationDefaultStyle;
+}
+
 void KoTextSharedLoadingData::shapeInserted(KoShape *shape, const KoXmlElement &element, KoShapeLoadingContext &/*context*/)
 {
     Q_UNUSED(shape);
