@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
  * This library is free software; you can redistribute it and/or
@@ -27,9 +28,12 @@
 
 #include <KoXmlNS.h>
 #include <KoOdfLoadingContext.h>
+#include <KoShapeLoadingContext.h>
 #include <KoXmlWriter.h>
 #include <KoUnit.h>
 #include <KoText.h>
+#include <KoImageCollection.h>
+#include <KoImageData.h>
 
 class KoListLevelProperties::Private
 {
@@ -80,6 +84,22 @@ int KoListLevelProperties::propertyInt(int key) const
     return variant.toInt();
 }
 
+uint KoListLevelProperties::propertyUInt(int key) const
+{
+    QVariant variant = d->stylesPrivate.value(key);
+    if (variant.isNull())
+        return 0;
+    return variant.toUInt();
+}
+
+qulonglong KoListLevelProperties::propertyULongLong(int key) const
+{
+    QVariant variant = d->stylesPrivate.value(key);
+    if (variant.isNull())
+        return 0;
+    return variant.toULongLong();
+}
+
 qreal KoListLevelProperties::propertyDouble(int key) const
 {
     QVariant variant = d->stylesPrivate.value(key);
@@ -233,20 +253,48 @@ qreal KoListLevelProperties::minimumWidth() const
     return propertyDouble(KoListStyle::MinimumWidth);
 }
 
+void KoListLevelProperties::setWidth(qreal width)
+{
+    setProperty(KoListStyle::Width, width);
+}
+
+qreal KoListLevelProperties::width() const
+{
+    return propertyDouble(KoListStyle::Width);
+}
+
+void KoListLevelProperties::setHeight(qreal height)
+{
+    setProperty(KoListStyle::Height, height);
+}
+
+qreal KoListLevelProperties::height() const
+{
+    return propertyDouble(KoListStyle::Height);
+}
+
+void KoListLevelProperties::setBulletImage(KoImageData *imageData)
+{
+    setProperty(KoListStyle::BulletImageKey, imageData->key());
+}
+
 KoListLevelProperties & KoListLevelProperties::operator=(const KoListLevelProperties & other)
 {
     d->copy(other.d);
     return *this;
 }
 
-void KoListLevelProperties::setListId(const QString &listId)
+void KoListLevelProperties::setListId(KoListStyle::ListIdType listId)
 {
     setProperty(KoListStyle::ListId, listId);
 }
 
-QString KoListLevelProperties::listId() const
+KoListStyle::ListIdType KoListLevelProperties::listId() const
 {
-    return propertyString(KoListStyle::ListId);
+    if (sizeof(KoListStyle::ListIdType) == sizeof(uint))
+        return propertyUInt(KoListStyle::ListId);
+    else
+        return propertyULongLong(KoListStyle::ListId);
 }
 
 bool KoListLevelProperties::letterSynchronization() const
@@ -301,9 +349,9 @@ KoListLevelProperties KoListLevelProperties::fromTextList(QTextList *list)
     return llp;
 }
 
-void KoListLevelProperties::loadOdf(KoOdfLoadingContext& context, const KoXmlElement& style)
+void KoListLevelProperties::loadOdf(KoShapeLoadingContext& scontext, const KoXmlElement& style)
 {
-    Q_UNUSED(context);
+    KoOdfLoadingContext &context = scontext.odfLoadingContext();
 
     // The text:level attribute specifies the level of the number list
     // style. It can be used on all list-level styles.
@@ -325,7 +373,7 @@ void KoListLevelProperties::loadOdf(KoOdfLoadingContext& context, const KoXmlEle
             switch (bulletChar[0].unicode()) {
             case 0x2022: // bullet, a small disc -> circle
                 //TODO use BulletSize to differ between small and large discs
-                setStyle(KoListStyle::CircleItem);
+                setStyle(KoListStyle::DiscItem);
                 break;
             case 0x25CF: // black circle, large disc -> disc
             case 0xF0B7: // #113361
@@ -403,18 +451,33 @@ void KoListLevelProperties::loadOdf(KoOdfLoadingContext& context, const KoXmlEle
 
         //The style:num-prefix and style:num-suffix attributes specify what to display before and after the number.
         const QString prefix = style.attributeNS(KoXmlNS::style, "num-prefix", QString());
-        if (! prefix.isNull())
+        if (! prefix.isEmpty())
             setListItemPrefix(prefix);
         const QString suffix = style.attributeNS(KoXmlNS::style, "num-suffix", QString());
-        if (! suffix.isNull())
+        if (! suffix.isEmpty())
             setListItemSuffix(suffix);
         const QString startValue = style.attributeNS(KoXmlNS::text, "start-value", QString("1"));
         setStartValue(startValue.toInt());
     }
-    // TODO implement bitmap for now just use an empty buller char
     else if (style.localName() == "list-level-style-image") {   // list with image
-        setStyle(KoListStyle::CustomCharItem);
-        setBulletCharacter(QChar());
+        setStyle(KoListStyle::ImageItem);
+        KoImageCollection *imageCollection = scontext.imageCollection();
+        const QString href = style.attribute("href");
+        if(imageCollection) {
+            if (!href.isEmpty()) {
+                KoStore *store = context.store();
+                setBulletImage(imageCollection->createImageData(href, store));
+            } else {
+                // check if we have an office:binary data element containing the image data
+                const KoXmlElement &binaryData(KoXml::namedItemNS(style, KoXmlNS::office, "binary-data"));
+                if (!binaryData.isNull()) {
+                    QImage image;
+                    if (image.loadFromData(QByteArray::fromBase64(binaryData.text().toLatin1()))) {
+                        setBulletImage(imageCollection->createImageData(image));
+                    }
+                }
+            }
+        }
     }
     else { // if not defined, we have do nothing
         kDebug(32500) << "stylename else:" << style.localName() << "level=" << level << "displayLevel=" << displayLevel;
@@ -423,7 +486,7 @@ void KoListLevelProperties::loadOdf(KoOdfLoadingContext& context, const KoXmlEle
     }
 
     setLevel(level);
-    if (!displayLevel.isNull())
+    if (!displayLevel.isEmpty())
         setDisplayLevel(displayLevel.toInt());
 
     KoXmlElement property;
@@ -432,17 +495,29 @@ void KoListLevelProperties::loadOdf(KoOdfLoadingContext& context, const KoXmlEle
             continue;
         const QString localName = property.localName();
         if (localName == "list-level-properties") {
-            if (property.hasAttributeNS(KoXmlNS::text, "space-before"))
-                setIndent(KoUnit::parseValue(property.attributeNS(KoXmlNS::text, "space-before")));
+            QString spaceBefore(property.attributeNS(KoXmlNS::text, "space-before"));
+            if (!spaceBefore.isEmpty())
+                setIndent(KoUnit::parseValue(spaceBefore));
 
-            if (property.hasAttributeNS(KoXmlNS::text, "min-label-width"))
-                setMinimumWidth(KoUnit::parseValue(property.attributeNS(KoXmlNS::text, "min-label-width")));
+            QString minLableWidth(property.attributeNS(KoXmlNS::text, "min-label-width"));
+            if (!minLableWidth.isEmpty())
+                setMinimumWidth(KoUnit::parseValue(minLableWidth));
 
-            if (property.hasAttributeNS(KoXmlNS::fo, "text-align"))
-                setAlignment(KoText::alignmentFromString(property.attributeNS(KoXmlNS::fo, "text-align")));
+            QString textAlign(property.attributeNS(KoXmlNS::fo, "text-align"));
+            if (!textAlign.isEmpty())
+                setAlignment(KoText::alignmentFromString(textAlign));
 
-            if (property.hasAttributeNS(KoXmlNS::text, "min-label-distance"))
-                setMinimumWidth(KoUnit::parseValue(property.attributeNS(KoXmlNS::text, "min-label-distance")));
+            QString minLableDistance(property.attributeNS(KoXmlNS::text, "min-label-distance"));
+            if (!minLableDistance.isEmpty())
+                setMinimumDistance(KoUnit::parseValue(minLableDistance));
+
+            QString width(property.attributeNS(KoXmlNS::fo, "width"));
+            if (!width.isEmpty())
+                setWidth(KoUnit::parseValue(width));
+
+            QString height(property.attributeNS(KoXmlNS::fo, "height"));
+            if (!height.isEmpty())
+                setHeight(KoUnit::parseValue(height));
         } else if (localName == "text-properties") {
             // TODO
         }
@@ -504,14 +579,13 @@ void KoListLevelProperties::saveOdf(KoXmlWriter *writer) const
             bullet = d->stylesPrivate.value(KoListStyle::BulletCharacter).toInt();
         } else { // try to determine the bullet character from the style
             switch (style()) {
-            case KoListStyle::CircleItem:           bullet = 0x2022; break;
+            case KoListStyle::DiscItem:             bullet = 0x2022; break;
             case KoListStyle::RhombusItem:          bullet = 0xE00C; break;
             case KoListStyle::SquareItem:           bullet = 0xE00A; break;
             case KoListStyle::RightArrowHeadItem:   bullet = 0x27A2; break;
             case KoListStyle::RightArrowItem:       bullet = 0x2794; break;
             case KoListStyle::HeavyCheckMarkItem:   bullet = 0x2714; break;
             case KoListStyle::BallotXItem:          bullet = 0x2717; break;
-            case KoListStyle::DiscItem: // intentional fall through
             default:                                bullet = 0x25CF; break;
             }
         }
