@@ -1,5 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007,2008 Sebastian Sauer <mail@dipe.org>
+ * Copyright (C) 2007,ducroquet Pierre Ducroquet <pinaraf@gmail.com>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
@@ -38,7 +40,7 @@ class KoStyleStack;
 class KoGenStyle;
 class KoGenStyles;
 #include "KoXmlReaderForward.h"
-class KoOdfLoadingContext;
+class KoShapeLoadingContext;
 #include "KoBorder.h"
 
 /**
@@ -68,10 +70,8 @@ public:
         DropCapsLength, ///< Number of glyphs to show as drop-caps
         DropCapsLines,  ///< Number of lines that the drop-caps span
         DropCapsDistance,   ///< Distance between drop caps and text
-        DropCapsTextStyle,  ///< Text style of dropped chars. Not supported yet.
+        DropCapsTextStyle,  ///< Text style of dropped chars.
         FollowDocBaseline,  ///< If true the baselines will be aligned with the doc-wide grid
-        BreakBefore,    ///< If true, insert a frame break before this paragraph
-        BreakAfter,     ///< If true, insert a frame break after this paragraph
 
         // border stuff
         HasLeftBorder,  ///< If true, paint a border on the left
@@ -123,7 +123,8 @@ public:
 
         MasterPageName,         ///< Optional name of the master-page
 
-        OutlineLevel            ///< Outline level for headings
+        OutlineLevel,            ///< Outline level for headings
+        DefaultOutlineLevel
 // do 15.5.24
 // continue at 15.5.28
     };
@@ -262,7 +263,7 @@ public:
      */
     void setDropCapsLines(int lines);
     /**
-     * set dropCapsLines
+     * The dropCapsLines
      * @see setDropCapsLines
      */
     int dropCapsLines() const;
@@ -274,10 +275,23 @@ public:
      */
     void setDropCapsDistance(qreal distance);
     /**
-     * Set dropCaps distance
+     * The dropCaps distance
      * @see setDropCapsDistance
      */
     qreal dropCapsDistance() const;
+
+    /**
+     * Set the style id of the text style used for dropcaps
+     * @see setDropCapsDistance
+     */
+    void setDropCapsTextStyleId(int id);
+
+    /**
+     * The style id of the text style used for dropcaps
+     * @see setDropCapsTextStyleId
+     */
+    int dropCapsTextStyleId() const;
+
     /**
      * If true the baselines will be aligned with the doc-wide grid
      */
@@ -465,6 +479,17 @@ public:
      * Change this block outline level
      */
     void setOutlineLevel(int outline);
+    
+    /**
+     * Return the default outline level of this style, or 0 if there is none.
+     */
+    int defaultOutlineLevel() const;
+
+    /**
+     * Change the default outline level for this style.
+     */
+    void setDefaultOutlineLevel(int outline);
+
 
     /// copy all the properties from the other style to this style, effectively duplicating it.
     void copyProperties(const KoParagraphStyle *style);
@@ -523,7 +548,7 @@ public:
      * @param context the odf loading context
      * @param element the element containing the
      */
-    void loadOdf(const KoXmlElement *element, KoOdfLoadingContext &context);
+    void loadOdf(const KoXmlElement *element, KoShapeLoadingContext &context);
 
     void saveOdf(KoGenStyle &style, KoGenStyles &mainStyles);
 
@@ -558,7 +583,7 @@ private:
      * Load the style from the \a KoStyleStack style stack using the
      * OpenDocument format.
      */
-    void loadOdfProperties(KoStyleStack &styleStack);
+    void loadOdfProperties(KoShapeLoadingContext &scontext);
     qreal propertyDouble(int key) const;
     int propertyInt(int key) const;
     bool propertyBoolean(int key) const;
