@@ -1,5 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006, 2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
+ * Copyright (C) 2007 Pierre Ducroquet <pinaraf@gmail.com>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  * Copyright (C) 2009 KO GmbH <cbo@kogmbh.com>
  *
@@ -28,6 +30,7 @@
 #include "KoTableColumnStyle.h"
 #include "KoTableRowStyle.h"
 #include "KoTableCellStyle.h"
+#include "KoSectionStyle.h"
 #include "ChangeFollower.h"
 #include "KoTextDocument.h"
 
@@ -56,6 +59,7 @@ public:
     QHash<int, KoTableColumnStyle *> tableColumnStyles;
     QHash<int, KoTableRowStyle *> tableRowStyles;
     QHash<int, KoTableCellStyle *> tableCellStyles;
+    QHash<int, KoSectionStyle *> sectionStyles;
     QList<ChangeFollower*> documentUpdaterProxies;
 
     bool updateTriggered;
@@ -96,10 +100,10 @@ KoStyleManager::~KoStyleManager()
 
 void KoStyleManager::saveOdfDefaultStyles(KoGenStyles &mainStyles)
 {
-    KoGenStyle style(KoGenStyle::StyleUser, "paragraph");
+    KoGenStyle style(KoGenStyle::ParagraphStyle, "paragraph");
     style.setDefaultStyle(true);
     d->defaultParagraphStyle->saveOdf(style, mainStyles);
-    mainStyles.lookup(style);
+    mainStyles.insert(style);
 }
 
 void KoStyleManager::saveOdf(KoGenStyles& mainStyles)
@@ -115,12 +119,12 @@ void KoStyleManager::saveOdf(KoGenStyles& mainStyles)
 
         QString name(QString(QUrl::toPercentEncoding(paragraphStyle->name(), "", " ")).replace('%', '_'));
         if (name.isEmpty()) {
-            name = "P";
+            name = 'P';
         }
 
-        KoGenStyle style(KoGenStyle::StyleUser, "paragraph");
+        KoGenStyle style(KoGenStyle::ParagraphStyle, "paragraph");
         paragraphStyle->saveOdf(style, mainStyles);
-        QString newName = mainStyles.lookup(style, name, KoGenStyles::DontForceNumbering);
+        QString newName = mainStyles.insert(style, name, KoGenStyles::DontAddNumberToName);
         savedNames.insert(paragraphStyle, newName);
         characterParagraphStyles.insert(paragraphStyle->characterStyle());
     }
@@ -140,12 +144,12 @@ void KoStyleManager::saveOdf(KoGenStyles& mainStyles)
 
         QString name(QString(QUrl::toPercentEncoding(characterStyle->name(), "", " ")).replace('%', '_'));
         if (name.isEmpty()) {
-            name = "T";
+            name = 'T';
         }
 
-        KoGenStyle style(KoGenStyle::StyleUser, "text");
+        KoGenStyle style(KoGenStyle::ParagraphStyle, "text");
         characterStyle->saveOdf(style);
-        mainStyles.lookup(style, name, KoGenStyles::DontForceNumbering);
+        mainStyles.insert(style, name, KoGenStyles::DontAddNumberToName);
     }
 
     foreach(KoListStyle *listStyle, d->listStyles) {
@@ -153,41 +157,41 @@ void KoStyleManager::saveOdf(KoGenStyles& mainStyles)
             continue;
         QString name(QString(QUrl::toPercentEncoding(listStyle->name(), "", " ")).replace('%', '_'));
         if (name.isEmpty())
-            name = "L";
+            name = 'L';
 
-        KoGenStyle style(KoGenStyle::StyleList);
+        KoGenStyle style(KoGenStyle::ListStyle);
         listStyle->saveOdf(style);
-        mainStyles.lookup(style, name, KoGenStyles::DontForceNumbering);
+        mainStyles.insert(style, name, KoGenStyles::DontAddNumberToName);
     }
 
     foreach(KoTableStyle *tableStyle, d->tableStyles) {
         QString name(QString(QUrl::toPercentEncoding(tableStyle->name(), "", " ")).replace('%', '_'));
         if (name.isEmpty())
-            name = "T"; //TODO is this correct?
+            name = 'T'; //TODO is this correct?
 
-        KoGenStyle style(KoGenStyle::StyleTable);
+        KoGenStyle style(KoGenStyle::TableStyle);
         tableStyle->saveOdf(style);
-        mainStyles.lookup(style, name, KoGenStyles::DontForceNumbering);
+        mainStyles.insert(style, name, KoGenStyles::DontAddNumberToName);
     }
 
     foreach(KoTableColumnStyle *tableColumnStyle, d->tableColumnStyles) {
         QString name(QString(QUrl::toPercentEncoding(tableColumnStyle->name(), "", " ")).replace('%', '_'));
         if (name.isEmpty())
-            name = "T"; //TODO is this correct?
+            name = 'T'; //TODO is this correct?
 
-        KoGenStyle style(KoGenStyle::StyleTableColumn);
+        KoGenStyle style(KoGenStyle::TableColumnStyle);
         tableColumnStyle->saveOdf(style);
-        mainStyles.lookup(style, name, KoGenStyles::DontForceNumbering);
+        mainStyles.insert(style, name, KoGenStyles::DontAddNumberToName);
     }
 
     foreach(KoTableRowStyle *tableRowStyle, d->tableRowStyles) {
         QString name(QString(QUrl::toPercentEncoding(tableRowStyle->name(), "", " ")).replace('%', '_'));
         if (name.isEmpty())
-            name = "T"; //TODO is this correct?
+            name = 'T'; //TODO is this correct?
 
-        KoGenStyle style(KoGenStyle::StyleTableRow);
+        KoGenStyle style(KoGenStyle::TableRowStyle);
         tableRowStyle->saveOdf(style);
-        mainStyles.lookup(style, name, KoGenStyles::DontForceNumbering);
+        mainStyles.insert(style, name, KoGenStyles::DontAddNumberToName);
     }
 
     foreach(KoTableCellStyle *tableCellStyle, d->tableCellStyles) {
@@ -195,9 +199,19 @@ void KoStyleManager::saveOdf(KoGenStyles& mainStyles)
         if (name.isEmpty())
             name = "T."; //TODO is this correct?
 
-        KoGenStyle style(KoGenStyle::StyleTableCell);
+        KoGenStyle style(KoGenStyle::TableCellStyle);
         tableCellStyle->saveOdf(style);
-        mainStyles.lookup(style, name, KoGenStyles::DontForceNumbering);
+        mainStyles.insert(style, name, KoGenStyles::DontAddNumberToName);
+    }
+
+    foreach(KoSectionStyle *sectionStyle, d->sectionStyles) {
+        QString name(QString(QUrl::toPercentEncoding(sectionStyle->name(), "", " ")).replace('%', '_'));
+        if (name.isEmpty())
+            name = "T."; //TODO is this correct?
+
+        KoGenStyle style(KoGenStyle::SectionStyle);
+        sectionStyle->saveOdf(style);
+        mainStyles.insert(style, name, KoGenStyles::DontAddNumberToName);
     }
 }
 
@@ -261,7 +275,6 @@ void KoStyleManager::add(KoTableColumnStyle *style)
 {
     if (d->tableColumnStyles.key(style, -1) != -1)
         return;
-    style->setParent(this);
     style->setStyleId(d->s_stylesNumber);
     d->tableColumnStyles.insert(d->s_stylesNumber++, style);
     emit styleAdded(style);
@@ -271,7 +284,6 @@ void KoStyleManager::add(KoTableRowStyle *style)
 {
     if (d->tableRowStyles.key(style, -1) != -1)
         return;
-    style->setParent(this);
     style->setStyleId(d->s_stylesNumber);
     d->tableRowStyles.insert(d->s_stylesNumber++, style);
     emit styleAdded(style);
@@ -287,6 +299,16 @@ void KoStyleManager::add(KoTableCellStyle *style)
     emit styleAdded(style);
 }
 
+void KoStyleManager::add(KoSectionStyle *style)
+{
+    if (d->sectionStyles.key(style, -1) != -1)
+        return;
+    style->setParent(this);
+    style->setStyleId(d->s_stylesNumber);
+    d->sectionStyles.insert(d->s_stylesNumber++, style);
+    emit styleAdded(style);
+}
+
 void KoStyleManager::remove(KoCharacterStyle *style)
 {
     if (d->charStyles.remove(style->styleId()))
@@ -329,6 +351,12 @@ void KoStyleManager::remove(KoTableCellStyle *style)
         emit styleRemoved(style);
 }
 
+void KoStyleManager::remove(KoSectionStyle *style)
+{
+    if (d->sectionStyles.remove(style->styleId()))
+        emit styleRemoved(style);
+}
+
 void KoStyleManager::remove(ChangeFollower *cf)
 {
     d->documentUpdaterProxies.removeAll(cf);
@@ -431,6 +459,19 @@ void KoStyleManager::alteredStyle(const KoTableCellStyle *style)
     requestFireUpdate();
 }
 
+void KoStyleManager::alteredStyle(const KoSectionStyle *style)
+{
+    Q_ASSERT(style);
+    int id = style->styleId();
+    if (id <= 0) {
+        kWarning(32500) << "alteredStyle received from a non registered style!";
+        return;
+    }
+    if (!d->updateQueue.contains(id))
+        d->updateQueue.append(id);
+    requestFireUpdate();
+}
+
 void KoStyleManager::updateAlteredStyles()
 {
     foreach(ChangeFollower *cf, d->documentUpdaterProxies) {
@@ -522,6 +563,10 @@ KoTableCellStyle *KoStyleManager::tableCellStyle(int id) const
     return d->tableCellStyles.value(id);
 }
 
+KoSectionStyle *KoStyleManager::sectionStyle(int id) const
+{
+    return d->sectionStyles.value(id);
+}
 
 KoCharacterStyle *KoStyleManager::characterStyle(const QString &name) const
 {
@@ -586,6 +631,15 @@ KoTableCellStyle *KoStyleManager::tableCellStyle(const QString &name) const
     return 0;
 }
 
+KoSectionStyle *KoStyleManager::sectionStyle(const QString &name) const
+{
+    foreach(KoSectionStyle *style, d->sectionStyles) {
+        if (style->name() == name)
+            return style;
+    }
+    return 0;
+}
+
 KoParagraphStyle *KoStyleManager::defaultParagraphStyle() const
 {
     return d->defaultParagraphStyle;
@@ -644,14 +698,9 @@ QList<KoTableCellStyle*> KoStyleManager::tableCellStyles() const
     return d->tableCellStyles.values();
 }
 
-bool KoStyleManager::completeLoading(KoStore *)
+QList<KoSectionStyle*> KoStyleManager::sectionStyles() const
 {
-    return true;
+    return d->sectionStyles.values();
 }
 
-bool KoStyleManager::completeSaving(KoStore *, KoXmlWriter *, KoShapeSavingContext *)
-{
-    return true;
-}
-
-#include "KoStyleManager.moc"
+#include <KoStyleManager.moc>
