@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  * Copyright (C) 2009 KO GmbH <cbo@kogmbh.com>
@@ -27,7 +28,6 @@
 
 #include <QObject>
 #include <QMetaType>
-#include <KoDataCenter.h>
 
 class QTextDocument;
 class KoCharacterStyle;
@@ -37,6 +37,7 @@ class KoTableStyle;
 class KoTableColumnStyle;
 class KoTableRowStyle;
 class KoTableCellStyle;
+class KoSectionStyle;
 class KoXmlWriter;
 class ChangeFollower;
 class KoGenStyles;
@@ -46,7 +47,7 @@ class KoTextShapeData;
  * Manages all character, paragraph, table and table cell styles for any number
  * of documents.
  */
-class KOTEXT_EXPORT KoStyleManager : public QObject, public KoDataCenter
+class KOTEXT_EXPORT KoStyleManager : public QObject
 {
     Q_OBJECT
 public:
@@ -101,6 +102,10 @@ public:
      * Add a new table cell style, automatically giving it a new styleId.
      */
     void add(KoTableCellStyle *style);
+    /**
+     * Add a new sewction style, automatically giving it a new styleId.
+     */
+    void add(KoSectionStyle *style);
 
     /**
      * Remove a style.
@@ -130,6 +135,10 @@ public:
      * Remove a table cell style.
      */
     void remove(KoTableCellStyle *style);
+    /**
+     * Remove a section style.
+     */
+    void remove(KoSectionStyle *style);
 
     /**
      * Add a document for which the styles will be applied.
@@ -199,6 +208,15 @@ public:
      */
     KoTableCellStyle *tableCellStyle(int id) const;
 
+    /**
+     * Return a sectionStyle by its id.
+     * From documents you can retrieve the id out of each QTextFrameFormat
+     * by requesting the KoSectionStyle::StyleId property.
+     * @param id the unique Id to search for.
+     * @see KoSectionStyle::styleId()
+     */
+    KoSectionStyle *sectionStyle(int id) const;
+
     /**
      * Return the first characterStyle with the param user-visible-name.
      * Since the name does not have to be unique there can be multiple
@@ -258,6 +276,15 @@ public:
      */
     KoTableCellStyle *tableCellStyle(const QString &name) const;
 
+    /**
+     * Return the first sectionStyle with the param user-visible-name.
+     * Since the name does not have to be unique there can be multiple
+     * styles registered with that name, only the first is returned
+     * @param name the name of the style.
+     * @see sectionStyle(id);
+     */
+    KoSectionStyle *sectionStyle(const QString &name) const;
+
      /**
      * Return the default paragraph style that will always be present in each
      * document. You can alter the style, but you can never delete it.
@@ -304,11 +331,8 @@ public:
     /// return all the tableCellStyles registered.
     QList<KoTableCellStyle*> tableCellStyles() const;
 
-    /// reimplemented
-    virtual bool completeLoading(KoStore *store);
-
-    /// reimplemented
-    virtual bool completeSaving(KoStore *store, KoXmlWriter * manifestWriter, KoShapeSavingContext * context);
+    /// return all the sectionStyles registered.
+    QList<KoSectionStyle*> sectionStyles() const;
 
 signals:
     void styleAdded(KoParagraphStyle*);
@@ -318,6 +342,7 @@ signals:
     void styleAdded(KoTableColumnStyle*);
     void styleAdded(KoTableRowStyle*);
     void styleAdded(KoTableCellStyle*);
+    void styleAdded(KoSectionStyle*);
     void styleRemoved(KoParagraphStyle*);
     void styleRemoved(KoCharacterStyle*);
     void styleRemoved(KoListStyle*);
@@ -325,6 +350,7 @@ signals:
     void styleRemoved(KoTableColumnStyle*);
     void styleRemoved(KoTableRowStyle*);
     void styleRemoved(KoTableCellStyle*);
+    void styleRemoved(KoSectionStyle*);
 
 public slots:
     /**
@@ -375,6 +401,13 @@ public slots:
      */
     void alteredStyle(const KoTableCellStyle *style);
 
+   /**
+     * Slot that should be called whenever a style is changed. This will update
+     * all documents with the style.
+     * Note that successive calls are aggregated.
+     */
+    void alteredStyle(const KoSectionStyle *style);
+
 private slots:
     void updateAlteredStyles(); // for the QTimer::singleshot
 
