@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Roopesh Chander <roop@forwardbias.in>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
@@ -39,7 +39,7 @@
 #include <KoXmlWriter.h>
 
 struct Edge {
-    Edge() : spacing(0.0) { }
+    Edge() : innerPen(), outerPen(), spacing(0.0) { }
     QPen innerPen;
     QPen outerPen;
     qreal spacing;
@@ -57,7 +57,8 @@ public:
         stylesPrivate.add(key, value);
     }
 
-    Edge edges[4];
+    Edge edges[6];
+    BorderStyle borderstyle[6];
     QString name;
     KoTableCellStyle *parentStyle;
     int next;
@@ -70,18 +71,39 @@ KoTableCellStyle::KoTableCellStyle(QObject *parent)
     //d->edges[Top].outerPen = format.penProperty(TopBorderOuterPen);
     d->edges[Top].spacing = 0;
    // d->edges[Top].innerPen = format.penProperty(TopBorderInnerPen);
+    d->borderstyle[Top] = BorderNone;
 
    // d->edges[Left].outerPen = format.penProperty(LeftBorderOuterPen);
     d->edges[Left].spacing = 0;
    // d->edges[Left].innerPen = format.penProperty(LeftBorderInnerPen);
+    d->borderstyle[Left] = BorderNone;
 
     //d->edges[Bottom].outerPen =format.penProperty(BottomBorderOuterPen);
     d->edges[Bottom].spacing = 0;
     //d->edges[Bottom].innerPen = format.penProperty(BottomBorderInnerPen);
+    d->borderstyle[Bottom] = BorderNone;
 
     //d->edges[Right].outerPen = format.penProperty(RightBorderOuterPen);
     d->edges[Right].spacing = 0;
     //d->edges[Right].innerPen = format.penProperty(RightBorderInnerPen);
+    d->borderstyle[Right] = BorderNone;
+
+    d->edges[TopLeftToBottomRight].spacing = 0;
+    d->borderstyle[TopLeftToBottomRight] = BorderNone;
+
+    d->edges[BottomLeftToTopRight].spacing = 0;
+    d->borderstyle[BottomLeftToTopRight] = BorderNone;
+}
+
+bool KoTableCellStyle::isDrawn(BorderStyle style) const
+{
+    if (style == BorderWave)
+        return true;
+    if (style == BorderDoubleWave)
+        return true;
+    if (style == BorderSlash)
+        return true;
+    return false;
 }
 
 KoTableCellStyle::KoTableCellStyle(const QTextTableCellFormat &format, QObject *parent)
@@ -89,22 +111,36 @@ KoTableCellStyle::KoTableCellStyle(const QTextTableCellFormat &format, QObject *
         d(new Private())
 {
     d->stylesPrivate = format.properties();
-      
+
     d->edges[Top].outerPen = format.penProperty(TopBorderOuterPen);
     d->edges[Top].spacing = format.doubleProperty(TopBorderSpacing);
     d->edges[Top].innerPen = format.penProperty(TopBorderInnerPen);
+    d->borderstyle[Top] = BorderStyle(format.intProperty(TopBorderStyle));
 
     d->edges[Left].outerPen = format.penProperty(LeftBorderOuterPen);
     d->edges[Left].spacing = format.doubleProperty(LeftBorderSpacing);
     d->edges[Left].innerPen = format.penProperty(LeftBorderInnerPen);
+    d->borderstyle[Left] = BorderStyle(format.intProperty(LeftBorderStyle));
 
     d->edges[Bottom].outerPen =format.penProperty(BottomBorderOuterPen);
     d->edges[Bottom].spacing = format.doubleProperty(BottomBorderSpacing);
     d->edges[Bottom].innerPen = format.penProperty(BottomBorderInnerPen);
+    d->borderstyle[Bottom] = BorderStyle(format.intProperty(BottomBorderStyle));
 
     d->edges[Right].outerPen = format.penProperty(RightBorderOuterPen);
     d->edges[Right].spacing = format.doubleProperty(RightBorderSpacing);
     d->edges[Right].innerPen = format.penProperty(RightBorderInnerPen);
+    d->borderstyle[Right] = BorderStyle(format.intProperty(RightBorderStyle));
+
+    d->edges[TopLeftToBottomRight].outerPen = format.penProperty(TopLeftToBottomRightBorderOuterPen);
+    d->edges[TopLeftToBottomRight].spacing = format.doubleProperty(TopLeftToBottomRightBorderSpacing);
+    d->edges[TopLeftToBottomRight].innerPen = format.penProperty(TopLeftToBottomRightBorderInnerPen);
+    d->borderstyle[TopLeftToBottomRight] = BorderStyle(format.intProperty(TopLeftToBottomRightBorderStyle));
+
+    d->edges[BottomLeftToTopRight].outerPen = format.penProperty(BottomLeftToTopRightBorderOuterPen);
+    d->edges[BottomLeftToTopRight].spacing = format.doubleProperty(BottomLeftToTopRightBorderSpacing);
+    d->edges[BottomLeftToTopRight].innerPen = format.penProperty(BottomLeftToTopRightBorderInnerPen);
+    d->borderstyle[BottomLeftToTopRight] = BorderStyle(format.intProperty(BottomLeftToTopRightBorderStyle));
 }
 
 KoTableCellStyle *KoTableCellStyle::fromTableCell(const QTextTableCell &tableCell, QObject *parent)
@@ -141,7 +177,9 @@ QRectF KoTableCellStyle::boundingRect(const QRectF &contentRect) const
 void KoTableCellStyle::setEdge(Side side, BorderStyle style, qreal width, QColor color)
 {
     Edge edge;
-    qreal innerWidth = 0.0, space = 0.0;
+    qreal innerWidth = 0;
+    qreal middleWidth = 0;
+    qreal space = 0;
     switch (style) {
     case BorderNone:
         width = 0.0;
@@ -157,8 +195,37 @@ void KoTableCellStyle::setEdge(Side side, BorderStyle style, qreal width, QColor
     case BorderDashed:
         edge.outerPen.setStyle(Qt::DashLine);
         break;
+    case BorderDashedLong: {
+        QVector<qreal> dashes;
+        dashes << 6 << 6;
+        edge.outerPen.setDashPattern(dashes);
+        break;
+    }
+    case BorderTriple:
+        innerWidth = middleWidth = space = width/6;
+        width -= (space + innerWidth);
+        edge.outerPen.setStyle(Qt::SolidLine);
+        break;
+    case BorderDashDot:
+        edge.outerPen.setStyle(Qt::DashDotLine);
+        break;
+    case BorderDashDotDot:
+        edge.outerPen.setStyle(Qt::DashDotDotLine);
+        break;
+    case BorderWave:
+        edge.outerPen.setStyle(Qt::SolidLine);
+        break;
+    case BorderSlash:
+        edge.outerPen.setStyle(Qt::SolidLine);
+        break;
+    case BorderDoubleWave:
+        innerWidth = space = width/4; //some nice default look
+        width -= (space + innerWidth);
+        edge.outerPen.setStyle(Qt::SolidLine);
+        break;
     default:
         edge.outerPen.setStyle(Qt::SolidLine);
+        break;
     }
     edge.outerPen.setColor(color);
     edge.outerPen.setJoinStyle(Qt::MiterJoin);
@@ -168,14 +235,18 @@ void KoTableCellStyle::setEdge(Side side, BorderStyle style, qreal width, QColor
     edge.spacing = space;
     edge.innerPen = edge.outerPen;
     edge.innerPen.setWidthF(innerWidth);
+    QPen middlePen;
+    middlePen = edge.outerPen;
+    middlePen.setWidthF(middleWidth);
 
     d->edges[side] = edge;
+    d->borderstyle[side] = style;
 }
 
 void KoTableCellStyle::setEdgeDoubleBorderValues(Side side, qreal innerWidth, qreal space)
 {
     qreal totalWidth = d->edges[side].outerPen.widthF() + d->edges[side].spacing + d->edges[side].innerPen.widthF();
-    if(d->edges[side].innerPen.widthF() > 0.0) {
+    if (d->edges[side].innerPen.widthF() > 0.0) {
         d->edges[side].outerPen.setWidthF(totalWidth - innerWidth - space);
         d->edges[side].spacing = space;
         d->edges[side].innerPen.setWidthF(innerWidth);
@@ -184,7 +255,7 @@ void KoTableCellStyle::setEdgeDoubleBorderValues(Side side, qreal innerWidth, qr
 
 bool KoTableCellStyle::hasBorders() const
 {
-    for (int i = Top; i <= Right; i++)
+    for (int i = Top; i <= BottomLeftToTopRight; i++)
         if (d->edges[i].outerPen.widthF() > 0.0)
             return true;
     return false;
@@ -194,11 +265,51 @@ void KoTableCellStyle::paintBackground(QPainter &painter, const QRectF &bounds)
 {
     QRectF innerBounds = bounds;
 
-    if(hasProperty(CellBackgroundBrush)) {
+    if (hasProperty(CellBackgroundBrush)) {
         painter.fillRect(bounds, background());
     }
 }
 
+void KoTableCellStyle::drawHorizontalWave(BorderStyle style, QPainter &painter, qreal x, qreal w, qreal t) const
+{
+    QPen pen = painter.pen();
+    const qreal linewidth = pen.width();
+    const qreal penwidth = linewidth/6;
+    pen.setWidth(penwidth);
+    painter.setPen(pen);
+    if (style == BorderSlash) {
+        for (qreal sx=x; sx<x+w-linewidth; sx+=linewidth*0.5) {
+            painter.drawLine(QLineF(sx, t-penwidth*2, sx+linewidth, t+penwidth*2));
+        }
+    } else {
+        for (qreal sx=x; sx<x+w-2*linewidth; sx+=linewidth) {
+            painter.drawLine(QLineF(sx, t-penwidth*2, sx+linewidth, t+penwidth*2));
+            sx+=linewidth;
+            painter.drawLine(QLineF(sx, t+penwidth*2, sx+linewidth, t-penwidth*2));
+        }
+    }
+}
+
+void KoTableCellStyle::drawVerticalWave(BorderStyle style, QPainter &painter, qreal y, qreal h, qreal t) const
+{
+    QPen pen = painter.pen();
+    const qreal linewidth = pen.width();
+    const qreal penwidth = linewidth/6;
+    pen.setWidth(penwidth);
+    painter.setPen(pen);
+    if (style == BorderSlash) {
+        for (qreal sy=y; sy<y+h-linewidth; sy+=linewidth*0.5) {
+            painter.drawLine(QLineF(t-penwidth*2, sy, t+penwidth*2, sy+linewidth));
+        }
+    } else {
+        for (qreal sy=y; sy<y+h-2*linewidth; sy+=linewidth) {
+            painter.drawLine(QLineF(t-penwidth*2, sy, t+penwidth*2, sy+linewidth));
+            sy+=linewidth;
+            painter.drawLine(QLineF(t+penwidth*2, sy, t-penwidth*2, sy+linewidth));
+        }
+    }
+}
+
 void KoTableCellStyle::paintBorders(QPainter &painter, const QRectF &bounds) const
 {
     QRectF innerBounds = bounds;
@@ -209,7 +320,15 @@ void KoTableCellStyle::paintBorders(QPainter &painter, const QRectF &bounds) con
         painter.setPen(pen);
         const qreal t = bounds.top() + pen.widthF() / 2.0;
         innerBounds.setTop(bounds.top() + d->edges[Top].spacing + pen.widthF());
-        painter.drawLine(QLineF(bounds.left(), t, bounds.right(), t));
+        if(isDrawn(d->borderstyle[Top])) {
+#if 0 // Unfinished code?
+            const qreal width = pen.widthF()/6;
+            qreal x;
+            //for (
+#endif
+        } else {
+            painter.drawLine(QLineF(bounds.left(), t, bounds.right(), t));
+        }
     }
     if (d->edges[Bottom].outerPen.widthF() > 0) {
         QPen pen = d->edges[Bottom].outerPen;
@@ -232,6 +351,8 @@ void KoTableCellStyle::paintBorders(QPainter &painter, const QRectF &bounds) con
         innerBounds.setRight(bounds.right() - d->edges[Right].spacing - pen.widthF());
         painter.drawLine(QLineF(r, bounds.top() + d->edges[Top].outerPen.widthF(), r, bounds.bottom() - d->edges[Bottom].outerPen.widthF()));
     }
+    paintDiagonalBorders(painter, bounds);
+
     // inner lines
     if (d->edges[Top].innerPen.widthF() > 0) {
         QPen pen = d->edges[Top].innerPen;
@@ -259,7 +380,41 @@ void KoTableCellStyle::paintBorders(QPainter &painter, const QRectF &bounds) con
     }
 }
 
-void KoTableCellStyle::drawTopHorizontalBorder(QPainter &painter, qreal x, qreal y, qreal w) const
+void KoTableCellStyle::paintDiagonalBorders(QPainter &painter, const QRectF &bounds) const
+{
+    if (d->edges[TopLeftToBottomRight].outerPen.widthF() > 0) {
+        QPen diagonalPen = d->edges[TopLeftToBottomRight].outerPen;
+        painter.setPen(diagonalPen);
+
+        QPen topPen = d->edges[Top].outerPen;
+        const qreal top = bounds.top() + topPen.widthF() / 2.0;
+        QPen leftPen = d->edges[Left].outerPen;
+        const qreal left = bounds.left() + leftPen.widthF() / 2.0;
+        QPen bottomPen = d->edges[Bottom].outerPen;
+        const qreal bottom = bounds.bottom() - bottomPen.widthF() / 2.0;
+        QPen rightPen = d->edges[Right].outerPen;
+        const qreal right = bounds.right() - rightPen.widthF() / 2.0;
+
+        painter.drawLine(QLineF(left, top, right, bottom));
+    }
+    if (d->edges[BottomLeftToTopRight].outerPen.widthF() > 0) {
+        QPen pen = d->edges[BottomLeftToTopRight].outerPen;
+        painter.setPen(pen);
+
+        QPen topPen = d->edges[Top].outerPen;
+        const qreal top = bounds.top() + topPen.widthF() / 2.0;
+        QPen leftPen = d->edges[Left].outerPen;
+        const qreal left = bounds.left() + leftPen.widthF() / 2.0;
+        QPen bottomPen = d->edges[Bottom].outerPen;
+        const qreal bottom = bounds.bottom() - bottomPen.widthF() / 2.0;
+        QPen rightPen = d->edges[Right].outerPen;
+        const qreal right = bounds.right() - rightPen.widthF() / 2.0;
+
+        painter.drawLine(QLineF(left, bottom, right, top));
+    }
+}
+
+void KoTableCellStyle::drawTopHorizontalBorder(QPainter &painter, qreal x, qreal y, qreal w, QPainterPath *accumulatedBlankBorders) const
 {
     qreal t=y;
     if (d->edges[Top].outerPen.widthF() > 0) {
@@ -267,52 +422,82 @@ void KoTableCellStyle::drawTopHorizontalBorder(QPainter &painter, qreal x, qreal
 
         painter.setPen(pen);
         t += pen.widthF() / 2.0;
-        painter.drawLine(QLineF(x, t, x+w, t));
+        if(isDrawn(d->borderstyle[Top])) {
+                drawHorizontalWave(d->borderstyle[Top], painter,x,w,t);
+        } else {
+            painter.drawLine(QLineF(x, t, x+w, t));
+        }
         t = y + d->edges[Top].spacing + pen.widthF();
+    } else if (accumulatedBlankBorders) {
+        // No border but we'd like to draw one for user convenience when on screen
+        accumulatedBlankBorders->moveTo(x, t);
+        accumulatedBlankBorders->lineTo(x+w, t);
     }
+
     // inner line
     if (d->edges[Top].innerPen.widthF() > 0) {
         QPen pen = d->edges[Top].innerPen;
         painter.setPen(pen);
         t += pen.widthF() / 2.0;
-        painter.drawLine(QLineF(x, t, x+w, t));
+        if(isDrawn(d->borderstyle[Top])) {
+                drawHorizontalWave(d->borderstyle[Top], painter,x,w,t);
+        } else {
+            painter.drawLine(QLineF(x, t, x+w, t));
+        }
     }
 }
 
-void KoTableCellStyle::drawSharedHorizontalBorder(QPainter &painter, const KoTableCellStyle &styleBelow,  qreal x, qreal y, qreal w) const
+void KoTableCellStyle::drawSharedHorizontalBorder(QPainter &painter, const KoTableCellStyle &styleBelow,  qreal x, qreal y, qreal w, QPainterPath *accumulatedBlankBorders) const
 {
     // First determine which style "wins" by comparing total width
     qreal thisWidth = d->edges[Bottom].outerPen.widthF() + d->edges[Bottom].spacing + d->edges[Bottom].innerPen.widthF();
     qreal thatWidth = styleBelow.d->edges[Top].outerPen.widthF() + styleBelow.d->edges[Top].spacing
                                     + styleBelow.d->edges[Top].innerPen.widthF();
-                                    
     if(thisWidth >= thatWidth) {
-        // top style wins
+        // bottom style wins
        qreal t=y;
         if (d->edges[Bottom].outerPen.widthF() > 0) {
             QPen pen = d->edges[Bottom].outerPen;
+            const qreal linewidth = pen.widthF();
 
             painter.setPen(pen);
-            t += pen.widthF() / 2.0;
-            painter.drawLine(QLineF(x, t, x+w, t));
-            t = y + d->edges[Bottom].spacing + pen.widthF();
+            t += linewidth / 2.0;
+            if(isDrawn(d->borderstyle[Bottom])) {
+                drawHorizontalWave(d->borderstyle[Bottom], painter,x,w,t);
+            } else {
+                painter.drawLine(QLineF(x, t, x+w, t));
+            }
+            t = y + d->edges[Bottom].spacing + linewidth;
+        } else if (accumulatedBlankBorders) {
+            // No border but we'd like to draw one for user convenience when on screen
+            accumulatedBlankBorders->moveTo(x, t);
+            accumulatedBlankBorders->lineTo(x+w, t);
+
         }
         // inner line
         if (d->edges[Bottom].innerPen.widthF() > 0) {
             QPen pen = d->edges[Bottom].innerPen;
             painter.setPen(pen);
             t += pen.widthF() / 2.0;
-            painter.drawLine(QLineF(x, t, x+w, t));
+            if(isDrawn(d->borderstyle[Bottom])) {
+                drawHorizontalWave(d->borderstyle[Bottom], painter,x,w,t);
+            } else {
+                painter.drawLine(QLineF(x, t, x+w, t));
+            }
         }
     } else {
-        // bottom style wins
+        // top style wins
         qreal t=y;
         if (styleBelow.d->edges[Top].outerPen.widthF() > 0) {
             QPen pen = styleBelow.d->edges[Top].outerPen;
 
             painter.setPen(pen);
             t += pen.widthF() / 2.0;
-            painter.drawLine(QLineF(x, t, x+w, t));
+            if(isDrawn(d->borderstyle[Top])) {
+                drawHorizontalWave(d->borderstyle[Top], painter,x,w,t);
+            } else {
+                painter.drawLine(QLineF(x, t, x+w, t));
+            }
             t = y + styleBelow.d->edges[Top].spacing + pen.widthF();
         }
         // inner line
@@ -320,12 +505,16 @@ void KoTableCellStyle::drawSharedHorizontalBorder(QPainter &painter, const KoTab
             QPen pen = styleBelow.d->edges[Top].innerPen;
             painter.setPen(pen);
             t += pen.widthF() / 2.0;
-            painter.drawLine(QLineF(x, t, x+w, t));
+            if(isDrawn(d->borderstyle[Top])) {
+                drawHorizontalWave(d->borderstyle[Top], painter,x,w,t);
+            } else {
+                painter.drawLine(QLineF(x, t, x+w, t));
+            }
         }
-    }                               
+    }
 }
 
-void KoTableCellStyle::drawBottomHorizontalBorder(QPainter &painter, qreal x, qreal y, qreal w) const
+void KoTableCellStyle::drawBottomHorizontalBorder(QPainter &painter, qreal x, qreal y, qreal w, QPainterPath *accumulatedBlankBorders) const
 {
     qreal t=y;
     if (d->edges[Bottom].outerPen.widthF() > 0) {
@@ -333,49 +522,77 @@ void KoTableCellStyle::drawBottomHorizontalBorder(QPainter &painter, qreal x, qr
 
         painter.setPen(pen);
         t -= pen.widthF() / 2.0;
-        painter.drawLine(QLineF(x, t, x+w, t));
+        if(isDrawn(d->borderstyle[Bottom])) {
+            drawHorizontalWave(d->borderstyle[Bottom], painter,x,w,t);
+        } else {
+            painter.drawLine(QLineF(x, t, x+w, t));
+        }
         t = y - d->edges[Bottom].spacing - pen.widthF();
+    } else if (accumulatedBlankBorders) {
+        // No border but we'd like to draw one for user convenience when on screen
+        accumulatedBlankBorders->moveTo(x, t);
+        accumulatedBlankBorders->lineTo(x+w, t);
+
     }
+
     // inner line
     if (d->edges[Bottom].innerPen.widthF() > 0) {
         QPen pen = d->edges[Bottom].innerPen;
         painter.setPen(pen);
         t -= pen.widthF() / 2.0;
-        painter.drawLine(QLineF(x, t, x+w, t));
+        if(isDrawn(d->borderstyle[Bottom])) {
+            drawHorizontalWave(d->borderstyle[Bottom], painter,x,w,t);
+        } else {
+            painter.drawLine(QLineF(x, t, x+w, t));
+        }
     }
 }
 
-void KoTableCellStyle::drawLeftmostVerticalBorder(QPainter &painter, qreal x, qreal y, qreal h) const
+void KoTableCellStyle::drawLeftmostVerticalBorder(QPainter &painter, qreal x, qreal y, qreal h, QPainterPath *accumulatedBlankBorders) const
 {
     qreal thisWidth = d->edges[Left].outerPen.widthF() + d->edges[Left].spacing + d->edges[Left].innerPen.widthF();
     qreal l = x - thisWidth / 2.0;
-    
+
     if (d->edges[Left].outerPen.widthF() > 0) {
         QPen pen = d->edges[Left].outerPen;
 
         painter.setPen(pen);
         l += pen.widthF() / 2.0;
-        painter.drawLine(QLineF(l, y, l, y+h));
+        if(isDrawn(d->borderstyle[Left])) {
+            drawVerticalWave(d->borderstyle[Left], painter,y,h,l);
+        } else {
+            painter.drawLine(QLineF(l, y, l, y+h));
+        }
         l += d->edges[Left].spacing + pen.widthF() / 2.0;
+    } else if (accumulatedBlankBorders) {
+        // No border but we'd like to draw one for user convenience when on screen
+        accumulatedBlankBorders->moveTo(l, y);
+        accumulatedBlankBorders->lineTo(l, y+h);
+
     }
+
     // inner line
     if (d->edges[Left].innerPen.widthF() > 0) {
         QPen pen = d->edges[Left].innerPen;
         painter.setPen(pen);
         l += pen.widthF() / 2.0;
-        painter.drawLine(QLineF(l, y, l, y+h));
+        if(isDrawn(d->borderstyle[Left])) {
+            drawVerticalWave(d->borderstyle[Left], painter,y,h,l);
+        } else {
+            painter.drawLine(QLineF(l, y, l, y+h));
+        }
     }
 }
 
-void KoTableCellStyle::drawSharedVerticalBorder(QPainter &painter, const KoTableCellStyle &styleRight,  qreal x, qreal y, qreal h) const
+void KoTableCellStyle::drawSharedVerticalBorder(QPainter &painter, const KoTableCellStyle &styleRight,  qreal x, qreal y, qreal h, QPainterPath *accumulatedBlankBorders) const
 {
     // First determine which style "wins" by comparing total width
     qreal thisWidth = d->edges[Right].outerPen.widthF() + d->edges[Right].spacing + d->edges[Right].innerPen.widthF();
     qreal thatWidth = styleRight.d->edges[Left].outerPen.widthF() + styleRight.d->edges[Left].spacing
                                     + styleRight.d->edges[Left].innerPen.widthF();
-                                    
+
     qreal l=x;
-    
+
     if(thisWidth >= thatWidth) {
         // left style wins
         l -= thisWidth / 2.0;
@@ -384,15 +601,29 @@ void KoTableCellStyle::drawSharedVerticalBorder(QPainter &painter, const KoTable
 
             painter.setPen(pen);
             l += pen.widthF() / 2.0;
-            painter.drawLine(QLineF(l, y, l, y+h));
+            if(isDrawn(d->borderstyle[Right])) {
+                drawVerticalWave(d->borderstyle[Right], painter,y,h,l);
+            } else {
+                painter.drawLine(QLineF(l, y, l, y+h));
+            }
             l += d->edges[Right].spacing + pen.widthF() / 2.0;
+        } else if (accumulatedBlankBorders) {
+            // No border but we'd like to draw one for user convenience when on screen
+            accumulatedBlankBorders->moveTo(l, y);
+            accumulatedBlankBorders->lineTo(l, y+h);
+
         }
+
         // inner line
         if (d->edges[Right].innerPen.widthF() > 0) {
             QPen pen = d->edges[Right].innerPen;
             painter.setPen(pen);
             l += pen.widthF() / 2.0;
-            painter.drawLine(QLineF(l, y, l, y+h));
+            if(isDrawn(d->borderstyle[Right])) {
+                drawVerticalWave(d->borderstyle[Right], painter,y,h,l);
+            } else {
+                painter.drawLine(QLineF(l, y, l, y+h));
+            }
         }
     } else {
         // right style wins
@@ -402,7 +633,11 @@ void KoTableCellStyle::drawSharedVerticalBorder(QPainter &painter, const KoTable
 
             painter.setPen(pen);
             l += pen.widthF() / 2.0;
-            painter.drawLine(QLineF(l, y, l, y+h));
+            if(isDrawn(d->borderstyle[Left])) {
+                drawVerticalWave(d->borderstyle[Left], painter,y,h,l);
+            } else {
+                painter.drawLine(QLineF(l, y, l, y+h));
+            }
             l += styleRight.d->edges[Left].spacing + pen.widthF() / 2.0;
         }
         // inner line
@@ -410,30 +645,48 @@ void KoTableCellStyle::drawSharedVerticalBorder(QPainter &painter, const KoTable
             QPen pen = styleRight.d->edges[Left].innerPen;
             painter.setPen(pen);
             l += pen.widthF() / 2.0;
-            painter.drawLine(QLineF(l, y, l, y+h));
+            if(isDrawn(d->borderstyle[Left])) {
+                drawVerticalWave(d->borderstyle[Left], painter,y,h,l);
+            } else {
+                painter.drawLine(QLineF(l, y, l, y+h));
+            }
         }
-    }                               
+    }
 }
 
-void KoTableCellStyle::drawRightmostVerticalBorder(QPainter &painter, qreal x, qreal y, qreal h) const
+void KoTableCellStyle::drawRightmostVerticalBorder(QPainter &painter, qreal x, qreal y, qreal h, QPainterPath *accumulatedBlankBorders) const
 {
     qreal thisWidth = d->edges[Right].outerPen.widthF() + d->edges[Right].spacing + d->edges[Right].innerPen.widthF();
     qreal l = x - thisWidth / 2.0;
-    
+
     if (d->edges[Right].outerPen.widthF() > 0) {
         QPen pen = d->edges[Right].outerPen;
 
         painter.setPen(pen);
         l += pen.widthF() / 2.0;
-        painter.drawLine(QLineF(l, y, l, y+h));
+        if(isDrawn(d->borderstyle[Right])) {
+            drawVerticalWave(d->borderstyle[Right], painter,y,h,l);
+        } else {
+            painter.drawLine(QLineF(l, y, l, y+h));
+        }
         l += d->edges[Right].spacing - pen.widthF() / 2.0;
+    } else if (accumulatedBlankBorders) {
+        // No border but we'd like to draw one for user convenience when on screen
+        accumulatedBlankBorders->moveTo(l, y);
+        accumulatedBlankBorders->lineTo(l, y+h);
+
     }
+
     // inner line
     if (d->edges[Right].innerPen.widthF() > 0) {
         QPen pen = d->edges[Right].innerPen;
         painter.setPen(pen);
         l += pen.widthF() / 2.0;
-        painter.drawLine(QLineF(l, y, l, y+h));
+        if(isDrawn(d->borderstyle[Right])) {
+            drawVerticalWave(d->borderstyle[Right], painter,y,h,l);
+        } else {
+            painter.drawLine(QLineF(l, y, l, y+h));
+        }
     }
 }
 
@@ -447,6 +700,18 @@ KoTableCellStyle::BorderStyle KoTableCellStyle::oasisBorderStyle(const QString &
         return BorderDotted;
     if (borderstyle == "dashed")
         return BorderDashed;
+    if (borderstyle == "dash-largegap")
+        return BorderDashedLong;
+    if (borderstyle == "dot-dash") // not offficially odf, but we suppport it anyway
+        return BorderDashDot;
+    if (borderstyle == "dot-dot-dash") // not offficially odf, but we suppport it anyway
+        return BorderDashDotDot;
+    if (borderstyle == "slash") // not offficially odf, but we suppport it anyway
+        return BorderSlash;
+    if (borderstyle == "wave") // not offficially odf, but we suppport it anyway
+        return BorderWave;
+    if (borderstyle == "double-wave") // not offficially odf, but we suppport it anyway
+        return BorderDoubleWave;
     return BorderSolid; // not needed to handle "solid" since it's the default
 }
 
@@ -492,6 +757,51 @@ void KoTableCellStyle::setBottomPadding(qreal padding)
     setProperty(QTextFormat::TableCellBottomPadding, padding);
 }
 
+qreal KoTableCellStyle::leftPadding() const
+{
+    return propertyDouble(QTextFormat::TableCellLeftPadding);
+}
+
+qreal KoTableCellStyle::rightPadding() const
+{
+    return propertyDouble(QTextFormat::TableCellRightPadding);
+}
+
+qreal KoTableCellStyle::topPadding() const
+{
+    return propertyDouble(QTextFormat::TableCellTopPadding);
+}
+
+qreal KoTableCellStyle::bottomPadding() const
+{
+    return propertyDouble(QTextFormat::TableCellBottomPadding);
+}
+
+qreal KoTableCellStyle::leftBorderWidth() const
+{
+    const Edge &edge = d->edges[Left];
+    return edge.spacing + edge.innerPen.widthF() + edge.outerPen.widthF();
+}
+
+qreal KoTableCellStyle::rightBorderWidth() const
+{
+    const Edge &edge = d->edges[Right];
+    return edge.spacing + edge.innerPen.widthF() + edge.outerPen.widthF();
+}
+
+qreal KoTableCellStyle::topBorderWidth() const
+{
+    const Edge &edge = d->edges[Top];
+    return edge.spacing + edge.innerPen.widthF() + edge.outerPen.widthF();
+}
+
+qreal KoTableCellStyle::bottomBorderWidth() const
+{
+    const Edge &edge = d->edges[Bottom];
+    return edge.spacing + edge.innerPen.widthF() + edge.outerPen.widthF();
+}
+
+
 void KoTableCellStyle::setPadding(qreal padding)
 {
     setBottomPadding(padding);
@@ -558,8 +868,7 @@ QColor KoTableCellStyle::propertyColor(int key) const
 {
     QVariant variant = value(key);
     if (variant.isNull()) {
-        QColor color;
-        return color;
+        return QColor();
     }
     return qvariant_cast<QColor>(variant);
 }
@@ -578,15 +887,27 @@ void KoTableCellStyle::applyStyle(QTextTableCellFormat &format) const
     format.setProperty(TopBorderOuterPen, d->edges[Top].outerPen);
     format.setProperty(TopBorderSpacing,  d->edges[Top].spacing);
     format.setProperty(TopBorderInnerPen, d->edges[Top].innerPen);
+    format.setProperty(TopBorderStyle, d->borderstyle[Top]);
     format.setProperty(LeftBorderOuterPen, d->edges[Left].outerPen);
     format.setProperty(LeftBorderSpacing,  d->edges[Left].spacing);
     format.setProperty(LeftBorderInnerPen, d->edges[Left].innerPen);
+    format.setProperty(LeftBorderStyle, d->borderstyle[Left]);
     format.setProperty(BottomBorderOuterPen, d->edges[Bottom].outerPen);
     format.setProperty(BottomBorderSpacing,  d->edges[Bottom].spacing);
     format.setProperty(BottomBorderInnerPen, d->edges[Bottom].innerPen);
+    format.setProperty(BottomBorderStyle, d->borderstyle[Bottom]);
     format.setProperty(RightBorderOuterPen, d->edges[Right].outerPen);
     format.setProperty(RightBorderSpacing,  d->edges[Right].spacing);
     format.setProperty(RightBorderInnerPen, d->edges[Right].innerPen);
+    format.setProperty(RightBorderStyle, d->borderstyle[Right]);
+    format.setProperty(TopLeftToBottomRightBorderOuterPen, d->edges[TopLeftToBottomRight].outerPen);
+    format.setProperty(TopLeftToBottomRightBorderSpacing,  d->edges[TopLeftToBottomRight].spacing);
+    format.setProperty(TopLeftToBottomRightBorderInnerPen, d->edges[TopLeftToBottomRight].innerPen);
+    format.setProperty(TopLeftToBottomRightBorderStyle, d->borderstyle[TopLeftToBottomRight]);
+    format.setProperty(BottomLeftToTopRightBorderOuterPen, d->edges[BottomLeftToTopRight].outerPen);
+    format.setProperty(BottomLeftToTopRightBorderSpacing,  d->edges[BottomLeftToTopRight].spacing);
+    format.setProperty(BottomLeftToTopRightBorderInnerPen, d->edges[BottomLeftToTopRight].innerPen);
+    format.setProperty(BottomLeftToTopRightBorderStyle, d->borderstyle[BottomLeftToTopRight]);
 }
 
 void KoTableCellStyle::setBackground(const QBrush &brush)
@@ -604,12 +925,21 @@ QBrush KoTableCellStyle::background() const
     QVariant variant = d->stylesPrivate.value(CellBackgroundBrush);
 
     if (variant.isNull()) {
-        QBrush brush;
-        return brush;
+        return QBrush();
     }
     return qvariant_cast<QBrush>(variant);
 }
 
+void KoTableCellStyle::setAlignment(Qt::Alignment alignment)
+{
+    setProperty(QTextFormat::BlockAlignment, (int) alignment);
+}
+
+Qt::Alignment KoTableCellStyle::alignment() const
+{
+    return static_cast<Qt::Alignment>(propertyInt(VerticalAlignment));
+}
+
 KoTableCellStyle *KoTableCellStyle::parentStyle() const
 {
     return d->parentStyle;
@@ -657,7 +987,7 @@ void KoTableCellStyle::loadOdf(const KoXmlElement *element, KoOdfLoadingContext
         d->name = element->attributeNS(KoXmlNS::style, "name", QString());
 
     QString masterPage = element->attributeNS(KoXmlNS::style, "master-page-name", QString());
-    if (! masterPage.isNull()) {
+    if (! masterPage.isEmpty()) {
         setMasterPageName(masterPage);
     }
     context.styleStack().save();
@@ -686,27 +1016,60 @@ void KoTableCellStyle::loadOdfProperties(KoStyleStack &styleStack)
     // Borders
     if (styleStack.hasProperty(KoXmlNS::fo, "border", "left")) {
         QString border = styleStack.property(KoXmlNS::fo, "border", "left");
+        QString style = border.section(' ', 1, 1);
+        if (styleStack.hasProperty(KoXmlNS::koffice, "specialborder", "left")) {
+            style = styleStack.property(KoXmlNS::koffice, "specialborder", "left");
+        }
         if (!border.isEmpty() && border != "none" && border != "hidden") {
-            setEdge(Left, oasisBorderStyle(border.section(' ', 1, 1)), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
+            setEdge(Left, oasisBorderStyle(style), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
         }
     }
     if (styleStack.hasProperty(KoXmlNS::fo, "border", "top")) {
         QString border = styleStack.property(KoXmlNS::fo, "border", "top");
+        QString style = border.section(' ', 1, 1);
+        if (styleStack.hasProperty(KoXmlNS::koffice, "specialborder", "top")) {
+            style = styleStack.property(KoXmlNS::koffice, "specialborder", "top");
+        }
         if (!border.isEmpty() && border != "none" && border != "hidden") {
-            setEdge(Top, oasisBorderStyle(border.section(' ', 1, 1)), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
+            setEdge(Top, oasisBorderStyle(style), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
         }
     }
+
     if (styleStack.hasProperty(KoXmlNS::fo, "border", "right")) {
         QString border = styleStack.property(KoXmlNS::fo, "border", "right");
+        QString style = border.section(' ', 1, 1);
+        if (styleStack.hasProperty(KoXmlNS::koffice, "specialborder", "right")) {
+            style = styleStack.property(KoXmlNS::koffice, "specialborder", "right");
+        }
         if (!border.isEmpty() && border != "none" && border != "hidden") {
-            setEdge(Right, oasisBorderStyle(border.section(' ', 1, 1)), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
+            setEdge(Right, oasisBorderStyle(style), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
         }
     }
     if (styleStack.hasProperty(KoXmlNS::fo, "border", "bottom")) {
         QString border = styleStack.property(KoXmlNS::fo, "border", "bottom");
-        if (!border.isEmpty() && border != "none" && border != "hidden") {
-            setEdge(Bottom, oasisBorderStyle(border.section(' ', 1, 1)), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
+        QString style = border.section(' ', 1, 1);
+        if (styleStack.hasProperty(KoXmlNS::koffice, "specialborder", "bottom")) {
+            style = styleStack.property(KoXmlNS::koffice, "specialborder", "bottom");
         }
+        if (!border.isEmpty() && border != "none" && border != "hidden") {
+            setEdge(Bottom, oasisBorderStyle(style), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
+        }
+    }
+    if (styleStack.hasProperty(KoXmlNS::style, "diagonal-tl-br")) {
+        QString border = styleStack.property(KoXmlNS::style, "diagonal-tl-br");
+        QString style = border.section(' ', 1, 1);
+        if (styleStack.hasProperty(KoXmlNS::koffice, "specialborder", "tl-br")) {
+            style = styleStack.property(KoXmlNS::koffice, "specialborder", "tl-br");
+        }
+        setEdge(TopLeftToBottomRight, oasisBorderStyle(style), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
+    }
+    if (styleStack.hasProperty(KoXmlNS::style, "diagonal-bl-tr")) {
+        QString border = styleStack.property(KoXmlNS::style, "diagonal-bl-tr");
+        QString style = border.section(' ', 1, 1);
+        if (styleStack.hasProperty(KoXmlNS::koffice, "specialborder", "bl-tr")) {
+            style = styleStack.property(KoXmlNS::koffice, "specialborder", "bl-tr");
+        }
+        setEdge(BottomLeftToTopRight, oasisBorderStyle(style), KoUnit::parseValue(border.section(' ', 0, 0), 1.0),QColor(border.section(' ', 2, 2)));
     }
 
     if (styleStack.hasProperty(KoXmlNS::style, "border-line-width", "left")) {
@@ -737,6 +1100,20 @@ void KoTableCellStyle::loadOdfProperties(KoStyleStack &styleStack)
             setEdgeDoubleBorderValues(Bottom, KoUnit::parseValue(blw[0], 1.0), KoUnit::parseValue(blw[1], 0.1));
         }
     }
+    if (styleStack.hasProperty(KoXmlNS::style, "diagonal-tl-br-widths")) {
+        QString borderLineWidth = styleStack.property(KoXmlNS::style, "diagonal-tl-br-widths");
+        if (!borderLineWidth.isEmpty() && borderLineWidth != "none" && borderLineWidth != "hidden") {
+            QStringList blw = borderLineWidth.split(' ', QString::SkipEmptyParts);
+            setEdgeDoubleBorderValues(TopLeftToBottomRight, KoUnit::parseValue(blw[0], 1.0), KoUnit::parseValue(blw[1], 0.1));
+        }
+    }
+    if (styleStack.hasProperty(KoXmlNS::style, "diagonal-bl-tr-widths")) {
+        QString borderLineWidth = styleStack.property(KoXmlNS::style, "diagonal-bl-tr-widths");
+        if (!borderLineWidth.isEmpty() && borderLineWidth != "none" && borderLineWidth != "hidden") {
+            QStringList blw = borderLineWidth.split(' ', QString::SkipEmptyParts);
+            setEdgeDoubleBorderValues(BottomLeftToTopRight, KoUnit::parseValue(blw[0], 1.0), KoUnit::parseValue(blw[1], 0.1));
+        }
+    }
 
     // The fo:background-color attribute specifies the background color of a cell.
     if (styleStack.hasProperty(KoXmlNS::fo, "background-color")) {
@@ -751,6 +1128,12 @@ void KoTableCellStyle::loadOdfProperties(KoStyleStack &styleStack)
             setBackground(brush);
         }
     }
+
+    // Alignment
+    const QString verticalAlign(styleStack.property(KoXmlNS::style, "vertical-align"));
+    if (!verticalAlign.isEmpty()) {
+        setAlignment(KoText::valignmentFromString(verticalAlign));
+    }
 }
 
 void KoTableCellStyle::copyProperties(const KoTableCellStyle *style)
@@ -782,6 +1165,7 @@ void KoTableCellStyle::removeDuplicates(const KoTableCellStyle &other)
 
 void KoTableCellStyle::saveOdf(KoGenStyle &style)
 {
+    Q_UNUSED(style);
 /*
     QList<int> keys = d->stylesPrivate.keys();
     foreach(int key, keys) {
@@ -876,4 +1260,4 @@ void KoTableCellStyle::saveOdf(KoGenStyle &style)
 */
 }
 
-#include "KoTableCellStyle.moc"
+#include <KoTableCellStyle.moc>
