@@ -58,6 +58,7 @@ KoDetailsPane::KoDetailsPane(QWidget* parent, const KComponentData &_componentDa
     m_documentList->installEventFilter(this);
     m_documentList->setIconSize(QSize(64, 64));
     m_documentList->setModel(d->m_model);
+    m_splitter->setSizes(QList<int>() << 2 << 1);
 
     changePalette();
 
@@ -132,4 +133,4 @@ QStandardItemModel* KoDetailsPane::model() const
     return d->m_model;
 }
 
-#include "KoDetailsPane.moc"
+#include <KoDetailsPane.moc>
