@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (c) 2000 Simon Hausmann <hausmann@kde.org>
 
-   $Id: KoDocInfoPropsFactory.cpp 853603 2008-08-28 00:56:29Z girish $
+   $Id: KoDocInfoPropsFactory.cpp 1192731 2010-11-03 21:41:15Z dfaure $
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -21,7 +21,15 @@
 
 #include <KoDocumentInfoPropsPage.h>
 
-#include <kgenericfactory.h>
+#include <kpluginfactory.h>
 
-typedef KGenericFactory<KoDocumentInfoPropsPage, KPropertiesDialog> PropsDlgFactory;
-K_EXPORT_COMPONENT_FACTORY(kodocinfopropspage, PropsDlgFactory("koffice"))
+static QObject* createDocInfoPropsPage(QWidget* w, QObject* parent, const QVariantList& args)
+{
+    Q_UNUSED(w);
+    KPropertiesDialog* props = qobject_cast<KPropertiesDialog *>(parent);
+    Q_ASSERT(props);
+    return new KoDocumentInfoPropsPage(props, args);
+}
+
+K_PLUGIN_FACTORY(PropsDlgFactory, registerPlugin<KoDocumentInfoPropsPage>(QString(), createDocInfoPropsPage);)
+K_EXPORT_PLUGIN(PropsDlgFactory("koffice"))
