@@ -2,6 +2,7 @@
  * Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
  * Copyright (C) 2000-2005 David Faure <faure@kde.org>
  * Copyright (C) 2007-2008 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,6 +29,12 @@
 #include "KoMainWindow.h"
 #include "KoFilterManager.h"
 #include "KoDocumentInfo.h"
+#include "KoCanvasController.h"
+#include "KoCanvasControllerWidget.h"
+#include "rdf/KoDocumentRdfBase.h"
+#ifdef SHOULD_BUILD_RDF
+#include "rdf/KoDocumentRdf.h"
+#endif
 #include "KoOdfStylesReader.h"
 #include "KoOdfReadStore.h"
 #include "KoOdfWriteStore.h"
@@ -35,12 +42,15 @@
 #include "KoXmlNS.h"
 #include "KoOpenPane.h"
 #include "KoApplication.h"
-#include "KoUndoStack.h"
+#include <KoProgressProxy.h>
+#include <KoProgressUpdater.h>
+#include <KoUpdater.h>
 
 #include <KoDpi.h>
 #include <KoXmlWriter.h>
 
 #include <kdialog.h>
+#include <KUndoStack>
 #include <kfileitem.h>
 #include <kio/job.h>
 #include <kio/jobuidelegate.h>
@@ -65,6 +75,8 @@
 #include <QtGui/QApplication>
 #include <QtGui/QPrinter>
 #include <QtGui/QPrintDialog>
+#include <QGraphicsScene>
+#include <QGraphicsProxyWidget>
 
 // Define the protocol used here for embedded documents' URL
 // This used to "store" but KUrl didn't like it,
@@ -98,6 +110,8 @@ class KoDocument::Private
 {
 public:
     Private() :
+            progressUpdater(0),
+            profileStream(0),
             filterManager(0),
             specialOutputFlag(0),   // default is native format
             isImporting(false),
@@ -114,7 +128,9 @@ public:
             storeInternal(false),
             bLoading(false),
             startUpWidget(0),
-            undoStack(0)
+            undoStack(0),
+            canvasItem(0)
+
     {
         confirmNonNativeSave[0] = true;
         confirmNonNativeSave[1] = true;
@@ -138,10 +154,18 @@ public:
 
     KoViewWrapperWidget *wrapperWidget;
     KoDocumentInfo *docInfo;
+#ifdef SHOULD_BUILD_RDF
+    KoDocumentRdf *docRdf;
+#else
+    KoDocumentRdfBase *docRdf;
+#endif
+    KoProgressUpdater *progressUpdater;
+    QTextStream *profileStream;
+    QTime profileReferenceTime;
 
     KoUnit unit;
 
-    KoFilterManager * filterManager; // The filter-manager to use when loading/saving [for the options]
+    KoFilterManager *filterManager; // The filter-manager to use when loading/saving [for the options]
 
     QByteArray mimeType; // The actual mimetype of the document
     QByteArray outputMimeType; // The mimetype to use when saving
@@ -172,7 +196,7 @@ public:
     QString templateType;
     QList<KoVersionInfo> versionInfo;
 
-    KoUndoStack* undoStack;
+    KUndoStack *undoStack;
 
     KoGridData gridData;
     KoGuidesData guidesData;
@@ -182,6 +206,8 @@ public:
     bool bEmpty;
 
     KoPageLayout pageLayout;
+
+    QGraphicsItem *canvasItem;
 };
 
 // Used in singleViewMode
@@ -214,18 +240,18 @@ public:
     }
 
     // Called by openFile()
-    void setKoView(KoView * view) {
+    void setKoView(KoView *view) {
         m_view = view;
         setFocusProxy(m_view);
     }
-    KoView * koView() const {
+    KoView *koView() const {
         return m_view;
     }
 private:
-    KoView* m_view;
+    KoView *m_view;
 };
 
-KoBrowserExtension::KoBrowserExtension(KoDocument * doc)
+KoBrowserExtension::KoBrowserExtension(KoDocument *doc)
         : KParts::BrowserExtension(doc)
 {
     emit enableAction("print", true);
@@ -235,9 +261,9 @@ void KoBrowserExtension::print()
 {
     kDebug(30003) << "implement; KoBrowserExtension::print";
     /*
-        KoDocument * doc = static_cast<KoDocument *>( parent() );
-        KoViewWrapperWidget * wrapper = static_cast<KoViewWrapperWidget *>( doc->widget() );
-        KoView * view = wrapper->koView();
+        KoDocument *doc = static_cast<KoDocument *>( parent() );
+        KoViewWrapperWidget *wrapper = static_cast<KoViewWrapperWidget *>( doc->widget() );
+        KoView *view = wrapper->koView();
         // TODO remove code duplication (KoMainWindow), by moving this to KoView
         QPrinter printer;
         QPrintDialog *printDialog = KdePrint::createPrintDialog(&printer, view->printDialogPages(), view);
@@ -248,7 +274,59 @@ void KoBrowserExtension::print()
     */
 }
 
-KoDocument::KoDocument(QWidget * parentWidget, QObject* parent, bool singleViewMode)
+namespace {
+    KoMainWindow *currentShell(KoDocument *doc)
+    {
+        if (!doc) return 0;
+
+        QWidget *widget = qApp->activeWindow();
+        KoMainWindow *shell = qobject_cast<KoMainWindow*>(widget);
+        while (!shell && widget) {
+            widget = widget->parentWidget();
+            shell = qobject_cast<KoMainWindow*>(widget);
+        }
+
+        if (!shell && doc && doc->shells().size() > 0) {
+            shell = doc->shells().first();
+        }
+        return shell;
+    }
+    class DocumentProgressProxy : public KoProgressProxy {
+    public:
+        KoDocument *const m_doc;
+
+        DocumentProgressProxy(KoDocument *doc)
+            : m_doc(doc)
+        {
+        }
+
+        ~DocumentProgressProxy() {
+            // signal that the job is done
+            setValue(-1);
+        }
+
+        int maximum() const {
+            return 100;
+        }
+
+        void setValue(int value) {
+            KoMainWindow *mainWindow = currentShell(m_doc);
+            if (mainWindow) {
+                mainWindow->slotProgress(value);
+            }
+        }
+
+        void setRange(int /*minimum*/, int /*maximum*/) {
+
+        }
+
+        void setFormat(const QString &/*format*/) {
+
+        }
+    };
+}
+
+KoDocument::KoDocument(QWidget *parentWidget, QObject *parent, bool singleViewMode)
         : KParts::ReadWritePart(parent)
         , d(new Private)
 {
@@ -282,15 +360,19 @@ KoDocument::KoDocument(QWidget * parentWidget, QObject* parent, bool singleViewM
     }
 
     d->docInfo = new KoDocumentInfo(this);
+    d->docRdf = 0;
+#ifdef SHOULD_BUILD_RDF
+    d->docRdf  = new KoDocumentRdf(this);
+#endif
 
     d->pageLayout.width = 0;
     d->pageLayout.height = 0;
-    d->pageLayout.top = 0;
-    d->pageLayout.bottom = 0;
-    d->pageLayout.left = 0;
-    d->pageLayout.right = 0;
+    d->pageLayout.topMargin = 0;
+    d->pageLayout.bottomMargin = 0;
+    d->pageLayout.leftMargin = 0;
+    d->pageLayout.rightMargin = 0;
 
-    d->undoStack = new KoUndoStack(this);
+    d->undoStack = new KUndoStack(this);
     d->undoStack->createUndoAction(actionCollection());
     d->undoStack->createRedoAction(actionCollection());
 
@@ -310,7 +392,7 @@ KoDocument::~KoDocument()
 
     // Tell our views that the document is already destroyed and
     // that they shouldn't try to access it.
-    foreach(KoView* view, d->views) {
+    foreach(KoView *view, d->views) {
         view->setDocumentDeleted();
     }
     delete d->startUpWidget;
@@ -321,6 +403,7 @@ KoDocument::~KoDocument()
     }
 
     delete d->filterManager;
+
     delete d;
     s_documentList->removeOne(this);
     // last one?
@@ -592,7 +675,7 @@ void KoDocument::slotAutoSave()
 QAction *KoDocument::action(const QDomElement &element) const
 {
     // First look in the document itself
-    QAction* act = KParts::ReadWritePart::action(element);
+    QAction *act = KParts::ReadWritePart::action(element);
     if (act)
         return act;
 
@@ -626,10 +709,10 @@ void KoDocument::setReadWrite(bool readwrite)
 {
     KParts::ReadWritePart::setReadWrite(readwrite);
 
-    foreach(KoView* view, d->views) {
+    foreach(KoView *view, d->views) {
         view->updateReadWrite(readwrite);
     }
-    foreach(KoMainWindow* mainWindow, d->shells) {
+    foreach(KoMainWindow *mainWindow, d->shells) {
         mainWindow->setReadWrite(readwrite);
     }
     setAutoSave(d->autoSaveDelay);
@@ -653,7 +736,7 @@ void KoDocument::addView(KoView *view)
     view->updateReadWrite(isReadWrite());
 
     if (d->views.size() == 1) {
-        KoApplication* app = qobject_cast<KoApplication*>(KApplication::kApplication());
+        KoApplication *app = qobject_cast<KoApplication*>(KApplication::kApplication());
         if (0 != app) {
             emit app->documentOpened('/'+objectName());
         }
@@ -665,7 +748,7 @@ void KoDocument::removeView(KoView *view)
     d->views.removeAll(view);
 
     if (d->views.isEmpty()) {
-        KoApplication* app = qobject_cast<KoApplication*>(KApplication::kApplication());
+        KoApplication *app = qobject_cast<KoApplication*>(KApplication::kApplication());
         if (0 != app) {
             emit app->documentClosed('/'+objectName());
         }
@@ -684,7 +767,7 @@ int KoDocument::viewCount() const
 
 KParts::Part *KoDocument::hitTest(QWidget *widget, const QPoint &globalPos)
 {
-    foreach(KoView* view, d->views) {
+    foreach(KoView *view, d->views) {
         if (static_cast<QWidget *>(view) == widget) {
             QPoint canvasPos(view->canvas()->mapFromGlobal(globalPos));
             canvasPos.rx() += view->canvasXOffset();
@@ -704,6 +787,20 @@ KoDocumentInfo *KoDocument::documentInfo() const
     return d->docInfo;
 }
 
+KoDocumentRdf *KoDocument::documentRdf() const
+{
+#ifdef SHOULD_BUILD_RDF
+    return d->docRdf;
+#endif
+    return 0;
+}
+
+KoDocumentRdfBase *KoDocument::documentRdfBase() const
+{
+    return d->docRdf;
+}
+
+
 void KoDocument::paintEverything(QPainter &painter, const QRect &rect, KoView *view)
 {
     Q_UNUSED(view);
@@ -758,10 +855,11 @@ bool KoDocument::saveNativeFormat(const QString & file)
     // OLD: QCString mimeType = oasis ? nativeOasisMimeType() : nativeFormatMimeType();
     QByteArray mimeType = d->outputMimeType;
     QByteArray nativeOasisMime = nativeOasisMimeType();
-    bool oasis = !mimeType.isEmpty() && (mimeType == nativeOasisMime || mimeType == nativeOasisMime + "-template");
+    bool oasis = !mimeType.isEmpty() && (mimeType == nativeOasisMime || mimeType == nativeOasisMime + "-template" || mimeType.startsWith("application/vnd.oasis.opendocument"));
+
     // TODO: use std::auto_ptr or create store on stack [needs API fixing],
     // to remove all the 'delete store' in all the branches
-    KoStore* store = KoStore::createStore(file, KoStore::Write, mimeType, backend);
+    KoStore *store = KoStore::createStore(file, KoStore::Write, mimeType, backend);
     if (d->specialOutputFlag == SaveEncrypted && !d->password.isNull())
         store->setPassword(d->password);
     if (store->bad()) {
@@ -771,141 +869,158 @@ bool KoDocument::saveNativeFormat(const QString & file)
     }
 
     if (oasis) {
-        kDebug(30003) << "Saving to OASIS format";
-        // Tell KoStore not to touch the file names
-        store->disallowNameExpansion();
-        KoOdfWriteStore odfStore(store);
-        KoXmlWriter* manifestWriter = odfStore.manifestWriter(mimeType);
-        KoEmbeddedDocumentSaver embeddedSaver;
-        SavingContext documentContext(odfStore, embeddedSaver);
-
-        if (!saveOdf(documentContext)) {
-            kDebug(30003) << "saveOdf failed";
-            delete store;
-            return false;
-        }
-
-        // Save embedded objects
-        if (!embeddedSaver.saveEmbeddedDocuments(documentContext)) {
-            kDebug(30003) << "save embedded documents failed";
-            delete store;
-            return false;
-        }
-
-        if (store->open("meta.xml")) {
-            if (!d->docInfo->saveOasis(store) || !store->close()) {
-                delete store;
-                return false;
-            }
-            manifestWriter->addManifestEntry("meta.xml", "text/xml");
-        } else {
-            d->lastErrorMessage = i18n("Not able to write '%1'. Partition full?", QString("meta.xml"));
-            delete store;
-            return false;
-        }
-
-        if (store->open("Thumbnails/thumbnail.png")) {
-            if (!saveOasisPreview(store, manifestWriter) || !store->close()) {
-                d->lastErrorMessage = i18n("Error while trying to write '%1'. Partition full?", QString("Thumbnails/thumbnail.png"));
-                delete store;
-                return false;
-            }
-            // No manifest entry!
-        } else {
-            d->lastErrorMessage = i18n("Not able to write '%1'. Partition full?", QString("Thumbnails/thumbnail.png"));
-            delete store;
-            return false;
-        }
-
-        if (!d->versionInfo.isEmpty()) {
-            if (store->open("VersionList.xml")) {
-                KoStoreDevice dev(store);
-                KoXmlWriter* xmlWriter = KoOdfWriteStore::createOasisXmlWriter(&dev,
-                                         "VL:version-list");
-                for (int i = 0; i < d->versionInfo.size(); ++i) {
-                    KoVersionInfo *version = &d->versionInfo[i];
-                    xmlWriter->startElement("VL:version-entry");
-                    xmlWriter->addAttribute("VL:title", version->title);
-                    xmlWriter->addAttribute("VL:comment", version->comment);
-                    xmlWriter->addAttribute("VL:creator", version->saved_by);
-                    xmlWriter->addAttribute("dc:date-time", version->date.toString(Qt::ISODate));
-                    xmlWriter->endElement();
-                }
-                xmlWriter->endElement(); // root element
-                xmlWriter->endDocument();
-                delete xmlWriter;
-                store->close();
-                manifestWriter->addManifestEntry("VersionList.xml", "text/xml");
-
-                for (int i = 0; i < d->versionInfo.size(); ++i) {
-                    KoVersionInfo *version = &d->versionInfo[i];
-                    store->addDataToFile(version->data, "Versions/" + version->title);
-                }
-            } else {
-                d->lastErrorMessage = i18n("Not able to write '%1'. Partition full?", QString("VersionList.xml"));
-                delete store;
-                return false;
-            }
-        }
-
-        // Write out manifest file
-        if (!odfStore.closeManifestWriter()) {
-            d->lastErrorMessage = i18n("Error while trying to write '%1'. Partition full?", QString("META-INF/manifest.xml"));
-            delete store;
-            return false;
-        }
-
-        // Remember the given password, if necessary
-        if (store->isEncrypted() && !d->isExporting)
-            d->password = store->password();
-
-        delete store;
+        return saveNativeFormatODF(store, mimeType);
     } else {
+        return saveNativeFormatKOffice(store);
+    }
+}
 
-        kDebug(30003) << "Saving root";
-        if (store->open("root")) {
+bool KoDocument::saveNativeFormatODF(KoStore *store, const QByteArray &mimeType)
+{
+    kDebug(30003) << "Saving to OASIS format";
+    // Tell KoStore not to touch the file names
+    store->disallowNameExpansion();
+    KoOdfWriteStore odfStore(store);
+    KoXmlWriter *manifestWriter = odfStore.manifestWriter(mimeType);
+    KoEmbeddedDocumentSaver embeddedSaver;
+    SavingContext documentContext(odfStore, embeddedSaver);
+
+    if (!saveOdf(documentContext)) {
+        kDebug(30003) << "saveOdf failed";
+        delete store;
+        return false;
+    }
+
+    // Save embedded objects
+    if (!embeddedSaver.saveEmbeddedDocuments(documentContext)) {
+        kDebug(30003) << "save embedded documents failed";
+        delete store;
+        return false;
+    }
+
+    if (store->open("meta.xml")) {
+        if (!d->docInfo->saveOasis(store) || !store->close()) {
+            delete store;
+            return false;
+        }
+        manifestWriter->addManifestEntry("meta.xml", "text/xml");
+    } else {
+        d->lastErrorMessage = i18n("Not able to write '%1'. Partition full?", QString("meta.xml"));
+        delete store;
+        return false;
+    }
+
+    if (d->docRdf && !d->docRdf->saveOasis(store, manifestWriter)) {
+        d->lastErrorMessage = i18n("Not able to write RDF metadata. Partition full?");
+        delete store;
+        return false;
+    }
+
+    if (store->open("Thumbnails/thumbnail.png")) {
+        if (!saveOasisPreview(store, manifestWriter) || !store->close()) {
+            d->lastErrorMessage = i18n("Error while trying to write '%1'. Partition full?", QString("Thumbnails/thumbnail.png"));
+            delete store;
+            return false;
+        }
+        // No manifest entry!
+    } else {
+        d->lastErrorMessage = i18n("Not able to write '%1'. Partition full?", QString("Thumbnails/thumbnail.png"));
+        delete store;
+        return false;
+    }
+
+    if (!d->versionInfo.isEmpty()) {
+        if (store->open("VersionList.xml")) {
             KoStoreDevice dev(store);
-            if (!saveToStream(&dev) || !store->close()) {
-                kDebug(30003) << "saveToStream failed";
-                delete store;
-                return false;
+            KoXmlWriter *xmlWriter = KoOdfWriteStore::createOasisXmlWriter(&dev,
+                                     "VL:version-list");
+            for (int i = 0; i < d->versionInfo.size(); ++i) {
+                KoVersionInfo *version = &d->versionInfo[i];
+                xmlWriter->startElement("VL:version-entry");
+                xmlWriter->addAttribute("VL:title", version->title);
+                xmlWriter->addAttribute("VL:comment", version->comment);
+                xmlWriter->addAttribute("VL:creator", version->saved_by);
+                xmlWriter->addAttribute("dc:date-time", version->date.toString(Qt::ISODate));
+                xmlWriter->endElement();
+            }
+            xmlWriter->endElement(); // root element
+            xmlWriter->endDocument();
+            delete xmlWriter;
+            store->close();
+            manifestWriter->addManifestEntry("VersionList.xml", "text/xml");
+
+            for (int i = 0; i < d->versionInfo.size(); ++i) {
+                KoVersionInfo *version = &d->versionInfo[i];
+                store->addDataToFile(version->data, "Versions/" + version->title);
             }
         } else {
-            d->lastErrorMessage = i18n("Not able to write '%1'. Partition full?", QString("maindoc.xml"));
+            d->lastErrorMessage = i18n("Not able to write '%1'. Partition full?", QString("VersionList.xml"));
             delete store;
             return false;
         }
-        if (store->open("documentinfo.xml")) {
-            QDomDocument doc = d->docInfo->save();
-            KoStoreDevice dev(store);
-
-            QByteArray s = doc.toByteArray(); // this is already Utf8!
-            (void)dev.write(s.data(), s.size());
-            (void)store->close();
-        }
-
-        if (store->open("preview.png")) {
-            // ### TODO: missing error checking (The partition could be full!)
-            savePreview(store);
-            (void)store->close();
-        }
-
-        if (!completeSaving(store)) {
-            delete store;
-            return false;
-        }
-        kDebug(30003) << "Saving done of url:" << url().url();
-        if (!store->finalize()) {
-            delete store;
-            return false;
-        }
-        // Success
-        delete store;
     }
+
+    // Write out manifest file
+    if (!odfStore.closeManifestWriter()) {
+        d->lastErrorMessage = i18n("Error while trying to write '%1'. Partition full?", QString("META-INF/manifest.xml"));
+        delete store;
+        return false;
+    }
+
+    // Remember the given password, if necessary
+    if (store->isEncrypted() && !d->isExporting)
+        d->password = store->password();
+
+    delete store;
+
     return true;
 }
 
-bool KoDocument::saveToStream(QIODevice * dev)
+bool KoDocument::saveNativeFormatKOffice(KoStore *store)
+{
+    kDebug(30003) << "Saving root";
+    if (store->open("root")) {
+        KoStoreDevice dev(store);
+        if (!saveToStream(&dev) || !store->close()) {
+            kDebug(30003) << "saveToStream failed";
+            delete store;
+            return false;
+        }
+    } else {
+        d->lastErrorMessage = i18n("Not able to write '%1'. Partition full?", QString("maindoc.xml"));
+        delete store;
+        return false;
+    }
+    if (store->open("documentinfo.xml")) {
+        QDomDocument doc = d->docInfo->save();
+        KoStoreDevice dev(store);
+
+        QByteArray s = doc.toByteArray(); // this is already Utf8!
+        (void)dev.write(s.data(), s.size());
+        (void)store->close();
+    }
+
+    if (store->open("preview.png")) {
+        // ### TODO: missing error checking (The partition could be full!)
+        savePreview(store);
+        (void)store->close();
+    }
+
+    if (!completeSaving(store)) {
+        delete store;
+        return false;
+    }
+    kDebug(30003) << "Saving done of url:" << url().url();
+    if (!store->finalize()) {
+        delete store;
+        return false;
+    }
+    // Success
+    delete store;
+    return true;
+}
+
+bool KoDocument::saveToStream(QIODevice *dev)
 {
     QDomDocument doc = saveXML();
     // Save to buffer
@@ -918,7 +1033,7 @@ bool KoDocument::saveToStream(QIODevice * dev)
 }
 
 // Called for embedded documents
-bool KoDocument::saveToStore(KoStore* _store, const QString & _path)
+bool KoDocument::saveToStore(KoStore *_store, const QString & _path)
 {
     kDebug(30003) << "Saving document to store" << _path;
 
@@ -950,7 +1065,7 @@ bool KoDocument::saveToStore(KoStore* _store, const QString & _path)
     return true;
 }
 
-bool KoDocument::saveOasisPreview(KoStore* store, KoXmlWriter* manifestWriter)
+bool KoDocument::saveOasisPreview(KoStore *store, KoXmlWriter *manifestWriter)
 {
     const QPixmap pix = generatePreview(QSize(128, 128));
     QImage preview(pix.toImage().convertToFormat(QImage::Format_ARGB32, Qt::ColorOnly));
@@ -967,7 +1082,7 @@ bool KoDocument::saveOasisPreview(KoStore* store, KoXmlWriter* manifestWriter)
     return true;
 }
 
-bool KoDocument::savePreview(KoStore* store)
+bool KoDocument::savePreview(KoStore *store)
 {
     QPixmap pix = generatePreview(QSize(256, 256));
     // Reducing to 8bpp reduces file sizes quite a lot.
@@ -1142,10 +1257,15 @@ bool KoDocument::openUrl(const KUrl & _url)
         //if ( d->shells.isEmpty() )
         //    kWarning(30003) << "no shell yet !";
         // Add to recent actions list in our shells
-        foreach(KoMainWindow* mainWindow, d->shells) {
+        foreach(KoMainWindow *mainWindow, d->shells) {
             mainWindow->addRecentURL(_url);
         }
     }
+    if (ret) {
+        // Detect readonly local-files; remote files are assumed to be writable, unless we add a KIO::stat here (async).
+        KFileItem file(url, mimeType(), KFileItem::Unknown);
+        setReadWrite(file.isWritable());
+    }
     return ret;
 }
 
@@ -1201,9 +1321,17 @@ bool KoDocument::openFile()
 
     QString importedFile = localFilePath();
 
+    // create the main progress monitoring object for loading, this can
+    // contain subtasks for filtering and loading
+    DocumentProgressProxy proxyProgress(this);
+    d->progressUpdater = new KoProgressUpdater(&proxyProgress,
+                                               KoProgressUpdater::Threaded,
+                                               d->profileStream);
+    d->progressUpdater->setReferenceTime(d->profileReferenceTime);
+
     if (!isNativeFormat(typeName.toLatin1(), ForImport)) {
         if (!d->filterManager)
-            d->filterManager = new KoFilterManager(this);
+            d->filterManager = new KoFilterManager(this, d->progressUpdater);
         KoFilter::ConversionStatus status;
         importedFile = d->filterManager->importDocument(localFilePath(), status);
         if (status != KoFilter::OK) {
@@ -1264,6 +1392,8 @@ bool KoDocument::openFile()
             }
 
             d->bLoading = false;
+            delete d->progressUpdater;
+            d->progressUpdater = 0;
             return false;
         }
         d->bEmpty = false;
@@ -1312,13 +1442,13 @@ bool KoDocument::openFile()
 
     if (ok && d->bSingleViewMode) {
         // See addClient below
-        KXMLGUIFactory* guiFactory = factory();
+        KXMLGUIFactory *guiFactory = factory();
         if (guiFactory)  // 0 when splitting views in konq, for some reason
             guiFactory->removeClient(this);
 
         if (!d->views.isEmpty()) {
             // We already had a view (this happens when doing reload in konqueror)
-            KoView* v = d->views.first();
+            KoView *v = d->views.first();
             if (guiFactory)
                 guiFactory->removeClient(v);
             removeView(v);
@@ -1346,10 +1476,24 @@ bool KoDocument::openFile()
         deleteOpenPane();
     }
     d->bLoading = false;
+
+    QPointer<KoUpdater> updater
+            = progressUpdater()->startSubtask(1, "clear undo stack");
+    updater->setProgress(0);
     undoStack()->clear();
+    updater->setProgress(100);
+
+    delete d->progressUpdater;
+    d->progressUpdater = 0;
+
     return ok;
 }
 
+KoProgressUpdater *KoDocument::progressUpdater() const
+{
+    return d->progressUpdater;
+}
+
 // shared between openFile and koMainWindow's "create new empty document" code
 void KoDocument::setMimeTypeAfterLoading(const QString& mimeType)
 {
@@ -1363,7 +1507,7 @@ void KoDocument::setMimeTypeAfterLoading(const QString& mimeType)
 }
 
 // The caller must call store->close() if loadAndParse returns true.
-bool KoDocument::oldLoadAndParse(KoStore* store, const QString& filename, KoXmlDocument& doc)
+bool KoDocument::oldLoadAndParse(KoStore *store, const QString& filename, KoXmlDocument& doc)
 {
     //kDebug(30003) <<"Trying to open" << filename;
 
@@ -1477,7 +1621,7 @@ bool KoDocument::loadNativeFormat(const QString & file_)
 bool KoDocument::loadNativeFormatFromStore(const QString& file)
 {
     KoStore::Backend backend = (d->specialOutputFlag == SaveAsDirectoryStore) ? KoStore::Directory : KoStore::Auto;
-    KoStore * store = KoStore::createStore(file, KoStore::Read, "", backend);
+    KoStore *store = KoStore::createStore(file, KoStore::Read, "", backend);
 
     if (store->bad()) {
         d->lastErrorMessage = i18n("Not a valid KOffice file: %1", file);
@@ -1506,7 +1650,7 @@ bool KoDocument::loadNativeFormatFromStore(QByteArray &data)
     bool succes;
     KoStore::Backend backend = (d->specialOutputFlag == SaveAsDirectoryStore) ? KoStore::Directory : KoStore::Auto;
     QBuffer buffer(&data);
-    KoStore * store = KoStore::createStore(&buffer, KoStore::Read, "", backend);
+    KoStore *store = KoStore::createStore(&buffer, KoStore::Read, "", backend);
 
     if (store->bad())
         return false;
@@ -1526,9 +1670,14 @@ bool KoDocument::loadNativeFormatFromStore(QByteArray &data)
     return succes;
 }
 
-bool KoDocument::loadNativeFormatFromStoreInternal(KoStore * store)
+bool KoDocument::loadNativeFormatFromStoreInternal(KoStore *store)
 {
     bool oasis = true;
+
+    if (oasis && store->hasFile("manifest.rdf") && d->docRdf) {
+        d->docRdf->loadOasis(store);
+    }
+
     // OASIS/OOo file format?
     if (store->hasFile("content.xml")) {
         store->disallowNameExpansion();
@@ -1577,7 +1726,11 @@ bool KoDocument::loadNativeFormatFromStoreInternal(KoStore * store)
     }
 
     if (oasis && store->hasFile("VersionList.xml")) {
-        KMessageBox::information(0, i18n("This document contains several versions. Go to File->Versions to open an old version."), QString(), "informVersionsAtLoading");
+        KNotification *notify = new KNotification("DocumentHasVersions");
+        notify->setText(i18n("Document <i>%1</i> contains several versions. Go to File->Versions to open an old version.", store->urlOfStore().url()));
+        notify->addContext("url", store->urlOfStore().url());
+        QTimer::singleShot(0, notify, SLOT(sendEvent()));
+
         KoXmlDocument versionInfo;
         KoOdfReadStore oasisStore(store);
         if (oasisStore.loadAndParse("VersionList.xml", versionInfo, d->lastErrorMessage)) {
@@ -1604,7 +1757,7 @@ bool KoDocument::loadNativeFormatFromStoreInternal(KoStore * store)
 }
 
 // For embedded documents
-bool KoDocument::loadFromStore(KoStore* _store, const QString& url)
+bool KoDocument::loadFromStore(KoStore *_store, const QString& url)
 {
     if (_store->open(url)) {
         KoXmlDocument doc;
@@ -1635,7 +1788,7 @@ bool KoDocument::loadFromStore(KoStore* _store, const QString& url)
     return result;
 }
 
-bool KoDocument::loadOasisFromStore(KoStore* store)
+bool KoDocument::loadOasisFromStore(KoStore *store)
 {
     KoOdfReadStore odfStore(store);
     if (! odfStore.loadAndParse(d->lastErrorMessage)) {
@@ -1663,7 +1816,7 @@ bool KoDocument::addVersion(const QString& comment)
     // to remove all the 'delete store' in all the branches
     QByteArray data;
     QBuffer buffer(&data);
-    KoStore* store = KoStore::createStore(&buffer/*file*/, KoStore::Write, mimeType, backend);
+    KoStore *store = KoStore::createStore(&buffer/*file*/, KoStore::Write, mimeType, backend);
     if (store->bad()) {
         delete store;
         return false;
@@ -1674,7 +1827,7 @@ bool KoDocument::addVersion(const QString& comment)
     store->disallowNameExpansion();
     KoOdfWriteStore odfStore(store);
 
-    KoXmlWriter* manifestWriter = odfStore.manifestWriter(mimeType);
+    KoXmlWriter *manifestWriter = odfStore.manifestWriter(mimeType);
     Q_UNUSED(manifestWriter); // XXX why?
 
     KoEmbeddedDocumentSaver embeddedSaver;
@@ -1792,7 +1945,7 @@ void KoDocument::setTitleModified(const QString &caption, bool mod)
         return;
     }
     // we must be root doc so update caption in all related windows
-    foreach(KoMainWindow* mainWindow, d->shells) {
+    foreach(KoMainWindow *mainWindow, d->shells) {
         mainWindow->updateCaption(caption, mod);
         mainWindow->updateReloadFileAction(this);
         mainWindow->updateVersionsFileAction(this);
@@ -1817,7 +1970,7 @@ QString KoDocument::caption() const
     if (documentInfo()) {
         c = documentInfo()->aboutInfo("title");
     }
-    const QString url(prettyPathOrUrl());
+    const QString url(this->url().fileName());
     if (!c.isEmpty() && !url.isEmpty()) {
         c = QString("%1 - %2").arg(c).arg(url);
     }
@@ -2019,7 +2172,7 @@ void KoDocument::addShell(KoMainWindow *shell)
 void KoDocument::removeShell(KoMainWindow *shell)
 {
     //kDebug(30003) <<"shell" << (void*)shell <<"removed from doc" << this;
-    d->shells.removeAt(d->shells.indexOf(shell));
+    d->shells.removeAll(shell);
 }
 
 const QList<KoMainWindow*>& KoDocument::shells() const
@@ -2032,7 +2185,7 @@ int KoDocument::shellCount() const
     return d->shells.count();
 }
 
-// DCOPObject * KoDocument::dcopObject()
+// DCOPObject *KoDocument::dcopObject()
 // {
 //     if ( !d->dcopObject )
 //         d->dcopObject = new KoDocumentIface( this );
@@ -2168,7 +2321,7 @@ bool KoDocument::hasExternURL() const
     return !url().protocol().isEmpty() && url().protocol() != STORE_PROTOCOL && url().protocol() != INTERNAL_PROTOCOL;
 }
 
-void KoDocument::slotStarted(KIO::Job* job)
+void KoDocument::slotStarted(KIO::Job *job)
 {
     if (job && job->ui()) {
         job->ui()->setWindow(currentShell());
@@ -2176,8 +2329,8 @@ void KoDocument::slotStarted(KIO::Job* job)
 }
 
 static const struct {
-    const char* localName;
-    const char* documentType;
+    const char *localName;
+    const char *documentType;
 } TN2DTArray[] = {
     { "text", I18N_NOOP("a word processing") },
     { "spreadsheet", I18N_NOOP("a spreadsheet") },
@@ -2210,7 +2363,7 @@ KoUnit KoDocument::unit() const
     return d->unit;
 }
 
-void KoDocument::setUnit(KoUnit unit)
+void KoDocument::setUnit(const KoUnit &unit)
 {
     if (d->unit != unit) {
         d->unit = unit;
@@ -2218,7 +2371,7 @@ void KoDocument::setUnit(KoUnit unit)
     }
 }
 
-void KoDocument::saveUnitOdf(KoXmlWriter* settingsWriter) const
+void KoDocument::saveUnitOdf(KoXmlWriter *settingsWriter) const
 {
     settingsWriter->addConfigItem("unit", unitName());
 }
@@ -2228,7 +2381,7 @@ QString KoDocument::unitName() const
     return KoUnit::unitName(unit());
 }
 
-void KoDocument::showStartUpWidget(KoMainWindow* parent, bool alwaysShow)
+void KoDocument::showStartUpWidget(KoMainWindow *parent, bool alwaysShow)
 {
 #ifndef NDEBUG
     if (d->templateType.isEmpty())
@@ -2294,13 +2447,13 @@ void KoDocument::startCustomDocument()
     deleteOpenPane();
 }
 
-KoOpenPane* KoDocument::createOpenPane(QWidget* parent, const KComponentData &componentData,
+KoOpenPane *KoDocument::createOpenPane(QWidget *parent, const KComponentData &componentData,
                                        const QString& templateType)
 {
     const QStringList mimeFilter = KoFilterManager::mimeFilter(KoDocument::readNativeFormatMimeType(),
                                                                KoFilterManager::Import, KoDocument::readExtraNativeMimeTypes());
 
-    KoOpenPane* openPane = new KoOpenPane(parent, componentData, mimeFilter, templateType);
+    KoOpenPane *openPane = new KoOpenPane(parent, componentData, mimeFilter, templateType);
     QList<CustomDocumentWidgetItem> widgetList = createCustomDocumentWidgets(openPane);
     foreach(const CustomDocumentWidgetItem & item, widgetList) {
         openPane->addCustomDocumentWidget(item.widget, item.title, item.icon);
@@ -2326,15 +2479,16 @@ QString KoDocument::templateType() const
     return d->templateType;
 }
 
-void KoDocument::deleteOpenPane()
+void KoDocument::deleteOpenPane(bool closing)
 {
     if (d->startUpWidget) {
         d->startUpWidget->hide();
         d->startUpWidget->deleteLater();
 
-        shells().first()->factory()->container("mainToolBar",
-                                               shells().first())->show();
-        shells().first()->setRootDocument(this);
+	if (!closing) {
+	    shells().first()->setRootDocument(this);
+	    shells().first()->factory()->container("mainToolBar", shells().first())->show();
+	}
     } else {
         emit closeEmbedInitDialog();
     }
@@ -2345,11 +2499,11 @@ QList<KoDocument::CustomDocumentWidgetItem> KoDocument::createCustomDocumentWidg
     return QList<CustomDocumentWidgetItem>();
 }
 
-bool KoDocument::showEmbedInitDialog(QWidget* parent)
+bool KoDocument::showEmbedInitDialog(QWidget *parent)
 {
     KDialog dlg(parent);
     dlg.setCaption(i18n("Embedding Object"));
-    KoOpenPane* pane = createOpenPane(&dlg, componentData(), templateType());
+    KoOpenPane *pane = createOpenPane(&dlg, componentData(), templateType());
     pane->layout()->setMargin(0);
     dlg.setMainWidget(pane);
     KConfigGroup cfg = KSharedConfig::openConfig("EmbedInitDialog")->group(QString());
@@ -2368,12 +2522,12 @@ QList<KoVersionInfo> & KoDocument::versionList()
     return d->versionInfo;
 }
 
-KoUndoStack* KoDocument::undoStack()
+KUndoStack *KoDocument::undoStack()
 {
     return d->undoStack;
 }
 
-void KoDocument::addCommand(QUndoCommand* command)
+void KoDocument::addCommand(QUndoCommand *command)
 {
     if (command)
         d->undoStack->push(command);
@@ -2395,6 +2549,16 @@ void KoDocument::setDocumentClean(bool clean)
     setModified(!clean);
 }
 
+void KoDocument::setProfileStream(QTextStream *profilestream)
+{
+    d->profileStream = profilestream;
+}
+
+void KoDocument::setProfileReferenceTime(const QTime& referenceTime)
+{
+    d->profileReferenceTime = referenceTime;
+}
+
 void KoDocument::clearUndoHistory()
 {
     d->undoStack->clear();
@@ -2410,20 +2574,9 @@ KoGuidesData &KoDocument::guidesData()
     return d->guidesData;
 }
 
-KoMainWindow* KoDocument::currentShell()
+KoMainWindow *KoDocument::currentShell()
 {
-    QWidget* widget = qApp->activeWindow();
-    if (!widget) return 0;
-
-    while(qobject_cast<KoMainWindow*>(widget) == 0 && widget->parent() && widget->parent()->inherits("QWidget")) {
-        widget = qobject_cast<QWidget*>(widget->parent());
-    }
-
-    KoMainWindow* shell = qobject_cast<KoMainWindow*>(widget);
-    if (!shell) {
-        shell = d->shells.first();
-    }
-    return shell;
+    return ::currentShell(this);
 }
 
 bool KoDocument::isEmpty() const
@@ -2436,11 +2589,28 @@ void KoDocument::setEmpty()
     d->bEmpty = true;
 }
 
+QGraphicsItem *KoDocument::canvasItem()
+{
+    if (!d->canvasItem) {
+        d->canvasItem = createCanvasItem();
+    }
+    return d->canvasItem;
+}
+
+QGraphicsItem *KoDocument::createCanvasItem()
+{
+    KoView *view = createView();
+    QGraphicsProxyWidget *proxy = new QGraphicsProxyWidget();
+    QWidget *canvasController = view->findChild<KoCanvasControllerWidget*>();
+    proxy->setWidget(canvasController);
+    return proxy;
+}
+
 // static
 int KoDocument::defaultAutoSave()
 {
     return 300;
 }
 
-#include "KoDocument_p.moc"
-#include "KoDocument.moc"
+#include <KoDocument_p.moc>
+#include <KoDocument.moc>
