@@ -2,6 +2,7 @@
    Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
    Copyright (C) 2000-2005 David Faure <faure@kde.org>
    Copyright (C) 2007 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -23,15 +24,15 @@
 #define KODOCUMENT_H
 
 #include <QtCore/QDateTime>
-#include <QtGui/QMatrix>
+#include <QtGui/QTransform>
 #include <QList>
 
 #include <kparts/part.h>
 #include <kservice.h>
 #include <kcomponentdata.h>
 
-#include "KoUnit.h"
-#include "KoPageLayout.h"
+#include <KoUnit.h>
+#include <KoPageLayout.h>
 #include "komain_export.h"
 #include "KoGridData.h"
 #include "KoGuidesData.h"
@@ -39,6 +40,7 @@
 #include <KoOdfDocument.h>
 
 class QUndoCommand;
+class QGraphicsItem;
 
 class KoStore;
 class KoOdfReadStore;
@@ -46,8 +48,12 @@ class KoOdfWriteStore;
 class KoMainWindow;
 class KoView;
 class KoDocumentInfo;
+class KoDocumentRdf;
+class KoDocumentRdfBase;
 class KoOpenPane;
-class KoUndoStack;
+class KUndoStack;
+class KoTextEditor;
+class KoProgressUpdater;
 
 class KoVersionInfo
 {
@@ -76,21 +82,21 @@ class KOMAIN_EXPORT KoDocument : public KParts::ReadWritePart, public KoOdfDocum
 public:
 
     /**
-     *  Constructor.
-     * The first 4 arguments are the same as the ones passed to KParts::Factory::createPart.
+     * Constructor.
      *
      * @param parentWidget the parent widget, in case we create a wrapper widget
      *        (in single view mode).
-     *        Usually the first argument passed by KParts::Factory::createPart.
+     *        Usually passed by KPluginFactory::create.
      * @param parent may be another KoDocument, or anything else.
-     *        Usually the third argument of KParts::Factory::createPart.
-     * @param singleViewMode determines whether the document may only have one view. In this case
-     *        the @p parent must be a QWidget derived class. KoDocument will then create a wrapper widget
-     *        (KoViewWrapperWidget) which is a child of @p parentWidget.
+     *        Usually passed by KPluginFactory::create.
+     * @param singleViewMode determines whether the document may only have one view.
+     *        Usually passed by the derived factory, when KPluginFactory::create says the requested interface is not KoDocument.
+     *        In this case the @p parent must be a QWidget derived class.
+     *        KoDocument will then create a wrapper widget (KoViewWrapperWidget) which is a child of @p parentWidget.
      *        This widget can be retrieved by calling widget().
      */
-    KoDocument(QWidget* parentWidget,
-               QObject* parent,
+    KoDocument(QWidget *parentWidget,
+               QObject *parent,
                bool singleViewMode = false);
 
     /**
@@ -355,6 +361,14 @@ public:
      */
     int viewCount() const;
 
+    /**
+     * @return a QGraphicsItem canvas displaying this document. The QGraphicsItem
+     * is created on first call. There is only one QGraphicsItem canvas that can
+     * be shown by many QGraphicsView subclasses (those should reimplement KoCanvasController
+     * as well).
+     */
+    QGraphicsItem *canvasItem();
+
     /**
      * Reimplemented from KParts::Part
      */
@@ -397,7 +411,12 @@ public:
      * @param parent the KoMainWindow used as parent for the widget.
      * @param alwaysShow always show the widget even if the user has configured it to not show.
      */
-    virtual void showStartUpWidget(KoMainWindow* parent, bool alwaysShow = false);
+    virtual void showStartUpWidget(KoMainWindow *parent, bool alwaysShow = false);
+
+    /**
+     * Removes the startupWidget shown at application start up.
+     */
+    void deleteOpenPane(bool closing = false);
 
     /**
      *  Tells the document that its title has been modified, either because
@@ -429,40 +448,27 @@ public:
      *  @param store The store to load from
      *  @param url An internal url, like tar:/1/2
      */
-    virtual bool loadFromStore(KoStore* store, const QString& url);
+    virtual bool loadFromStore(KoStore *store, const QString& url);
 
     /**
      *  @brief Loads an OASIS document from a store.
      *  This is used for both the main document and embedded objects.
      */
-    virtual bool loadOasisFromStore(KoStore* store);
+    virtual bool loadOasisFromStore(KoStore *store);
 
     /**
      *  @brief Saves a document to a store.
      *
      *  You should not have to reimplement this - but call it in saveChildren().
      */
-    virtual bool saveToStore(KoStore* store, const QString& path);
+    virtual bool saveToStore(KoStore *store, const QString& path);
 
     /**
      *  Reimplement this method to load the contents of your KOffice document,
      *  from the XML document. This is for the pre-Oasis file format (maindoc.xml).
      */
-    virtual bool loadXML(const KoXmlDocument & doc, KoStore * store) = 0;
+    virtual bool loadXML(const KoXmlDocument & doc, KoStore *store) = 0;
 
-    /**
-     *  Reimplement this method to load the contents of your %KOffice document,
-     *  from the XML document ("content.xml"). The styles have been parsed already,
-     *  you can find them in the odfStore.styles(). The store can be used
-     *  to load images and embedded documents.
-     */
-    virtual bool loadOdf(KoOdfReadStore & odfStore) = 0;
-
-    /**
-     *  Reimplement this method to save the contents of your %KOffice document,
-     *  using the ODF format.
-     */
-    virtual bool saveOdf(SavingContext & documentContext) = 0;
 
     /**
      *  Reimplement this to save the contents of the %KOffice document into
@@ -475,9 +481,8 @@ public:
      *  including processing instruction, complete DOCTYPE tag (with systemId and publicId), and root element.
      *  @param tagName the name of the tag for the root element
      *  @param version the DTD version (usually the application's version).
-     *  @deprecated use KoOdfWriteStore::createOasisXmlWriter instead
      */
-    KDE_DEPRECATED QDomDocument createDomDocument(const QString& tagName, const QString& version) const;
+    QDomDocument createDomDocument(const QString& tagName, const QString& version) const;
 
     /**
      *  Return a correctly created QDomDocument for an old (1.3-style) %KOffice document,
@@ -502,7 +507,7 @@ public:
      *  saveXML(). This method exists only for applications that
      *  don't use QDomDocument for saving, i.e. kword and kpresenter.
      */
-    virtual bool saveToStream(QIODevice * dev);
+    virtual bool saveToStream(QIODevice *dev);
 
     /**
      *  Loads a document in the native format from a given URL.
@@ -519,6 +524,8 @@ public:
      */
     virtual bool saveNativeFormat(const QString & file);
 
+public:
+
     /**
      * Activate/deactivate/configure the autosave feature.
      * @param delay in seconds, 0 to disable
@@ -561,6 +568,26 @@ public:
      */
     KoDocumentInfo *documentInfo() const;
 
+    /**
+     * @return the Rdf metadata for this document.
+     * This method should only be used by code that links to
+     * the RDF system and needs full access to the KoDocumentRdf object.
+     * @see KoDocumentRdf
+     */
+    KoDocumentRdf *documentRdf() const;
+    /**
+     * @return the Rdf metadata for this document.
+     * @see KoDocumentRdf
+     */
+    KoDocumentRdfBase *documentRdfBase() const;
+
+    /**
+     * @return the object to report progress to.
+     * One can add more KoUpdaters to it to make the progress reporting more
+     * accurate. If no active progress reporter is present, 0 is returned.
+     **/
+    KoProgressUpdater *progressUpdater() const;
+
     /**
      * Appends the shell to the list of shells which show this
      * document as their root document.
@@ -593,13 +620,6 @@ public:
         return s_documentList;
     }
 
-    /**
-     * Signal the progress of operations such as loading or saving.
-     */
-    void emitProgress(int value) {
-        emit sigProgress(value);
-    }
-
     /**
      * Return true if url() is a real filename, false if url() is
      * an internal url in the store, like "tar:/..."
@@ -714,14 +734,14 @@ public:
     /**
      * Sets the unit used to display all measures/distances.
      */
-    void setUnit(KoUnit u);
+    void setUnit(const KoUnit &unit);
 
     /**
      * Save the unit to the settings writer
      *
      * @param settingsWriter
      */
-    void saveUnitOdf(KoXmlWriter* settingsWriter) const;
+    void saveUnitOdf(KoXmlWriter *settingsWriter) const;
 
     /**
      * Returns the name of the unit used to display all measures/distances.
@@ -745,7 +765,7 @@ public:
      * Shows the init dialog when embeding
      * @param parent the parent widget
      */
-    virtual bool showEmbedInitDialog(QWidget* parent);
+    virtual bool showEmbedInitDialog(QWidget *parent);
 
 
     QList<KoVersionInfo> &versionList();
@@ -780,12 +800,12 @@ public slots:
     /**
      * Returns the global undo stack
      */
-    KoUndoStack *undoStack();
+    KUndoStack *undoStack();
     /**
      * Adds a command to the undo stack and executes it by calling the redo() function.
      * @param command command to add to the undo stack
      */
-    virtual void addCommand(QUndoCommand* command);
+    virtual void addCommand(QUndoCommand *command);
 
     /**
      * Begins recording of a macro command. At the end endMacro needs to be called.
@@ -810,6 +830,15 @@ public slots:
      */
     virtual void setDocumentClean(bool clean);
 
+    /**
+     * Set the output stream to report profile information to.
+     */
+    void setProfileStream(QTextStream *profilestream);
+
+    /**
+     * Set the output stream to report profile information to.
+     */
+    void setProfileReferenceTime(const QTime& referenceTime);
 signals:
 
     /**
@@ -817,7 +846,7 @@ signals:
      * It is common to connect views to it, in order to change the displayed units
      * (e.g. in the rulers)
      */
-    void unitChanged(KoUnit);
+    void unitChanged(const KoUnit &unit);
 
     /**
      * Progress info while loading or saving. The value is in percents (i.e. a number between 0 and 100)
@@ -863,7 +892,7 @@ protected:
      */
     struct CustomDocumentWidgetItem {
         /// Pointer to the custom document widget
-        QWidget* widget;
+        QWidget *widget;
         /// title used in the sidebar. If left empty it will be displayed as "Custom Document"
         QString title;
         /// icon used in the sidebar. If left empty it will use the unknown icon
@@ -879,6 +908,12 @@ protected:
 
     virtual KoView *createViewInstance(QWidget *parent) = 0;
 
+    /**
+     * Override this to create a QGraphicsItem that does not rely
+     * on proxying a KoCanvasController.
+     */
+    virtual QGraphicsItem *createCanvasItem();
+
     /**
      *  Loads a document from KReadOnlyPart::m_file (KParts takes care of downloading
      *  remote documents).
@@ -917,7 +952,7 @@ protected:
      *  from a store. This function is called after loadXML()
      *  and after loadChildren() have been called.
      */
-    virtual bool completeLoading(KoStore* store);
+    virtual bool completeLoading(KoStore *store);
 
     /**
      *  If you want to write additional files to a store,
@@ -929,7 +964,7 @@ protected:
      *  But do this ONLY if the document is not stored extern (see isStoredExtern() ).
      *  If it is, then the pictures should be saved to tar:/pictures.
      */
-    virtual bool completeSaving(KoStore* store);
+    virtual bool completeSaving(KoStore *store);
 
 
     /** @internal */
@@ -959,13 +994,13 @@ protected:
      * @param instance the KComponentData to be used for KConfig data
      * @param templateType the template-type (group) that should be selected on creation.
      */
-    KoOpenPane* createOpenPane(QWidget* parent, const KComponentData &instance,
+    KoOpenPane *createOpenPane(QWidget *parent, const KComponentData &instance,
                                const QString& templateType = QString());
 
 
 
     /// to satisfy KoOdfDocument where it overlaps with kparts
-    virtual KUrl getOdfUrl() const {
+    virtual KUrl odfUrl() const {
         return url();
     }
 
@@ -979,28 +1014,26 @@ private slots:
     void slotStarted(KIO::Job*);
     void startCustomDocument();
 
-    /**
-     * Removes the open widget shown at application start up.
-     */
-    void deleteOpenPane();
-
 private:
 
+    bool saveNativeFormatODF(KoStore *store, const QByteArray &mimeType);
+    bool saveNativeFormatKOffice(KoStore *store);
+
     /// @return the current KoMainWindow shell
-    KoMainWindow* currentShell();
+    KoMainWindow *currentShell();
 
     KService::Ptr nativeService();
-    bool oldLoadAndParse(KoStore* store, const QString& filename, KoXmlDocument& doc);
+    bool oldLoadAndParse(KoStore *store, const QString& filename, KoXmlDocument& doc);
     bool loadNativeFormatFromStore(const QString& file);
-    bool loadNativeFormatFromStoreInternal(KoStore * store);
+    bool loadNativeFormatFromStoreInternal(KoStore *store);
 
-    bool savePreview(KoStore* store);
-    bool saveOasisPreview(KoStore* store, KoXmlWriter* manifestWriter);
+    bool savePreview(KoStore *store);
+    bool saveOasisPreview(KoStore *store, KoXmlWriter *manifestWriter);
 
     QString prettyPathOrUrl() const;
 
     class Private;
-    Private * const d;
+    Private *const d;
 
 
     static QList<KoDocument*> *s_documentList;
