@@ -26,13 +26,16 @@
 #include "KoDocumentInfo.h"
 #include "KoDocument.h"
 #include "KoMainWindow.h"
+#include "rdf/KoDocumentRdfEditWidgetBase.h"
+#ifdef SHOULD_BUILD_RDF
+#include "rdf/KoDocumentRdfEditWidget.h"
+#endif
 
 #include <kmimetype.h>
 #include <klocale.h>
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
-#include <kstdguiitem.h>
 #include <kconfiggroup.h>
 
 #ifdef KDEPIMLIBS_FOUND
@@ -54,26 +57,35 @@ class KoDocumentInfoDlg::KoDocumentInfoDlgPrivate
 {
 public:
     KoDocumentInfoDlgPrivate() :
+            m_rdf(0),
+            m_rdfEditWidget(0),
             m_toggleEncryption(false),
             m_applyToggleEncryption(false),
             m_documentSaved(false) {}
     ~KoDocumentInfoDlgPrivate() {}
 
     KoDocumentInfo* m_info;
+    QList<KPageWidgetItem*> m_pages;
     Ui::KoDocumentInfoAboutWidget* m_aboutUi;
     Ui::KoDocumentInfoAuthorWidget* m_authorUi;
-
+    KoDocumentRdf* m_rdf;
+#ifdef SHOULD_BUILD_RDF
+    KoDocumentRdfEditWidget* m_rdfEditWidget;
+#else
+    KoDocumentRdfEditWidgetBase* m_rdfEditWidget;
+#endif
     bool m_toggleEncryption;
     bool m_applyToggleEncryption;
     bool m_documentSaved;
 };
 
 
-KoDocumentInfoDlg::KoDocumentInfoDlg(QWidget* parent, KoDocumentInfo* docInfo)
+KoDocumentInfoDlg::KoDocumentInfoDlg(QWidget* parent, KoDocumentInfo* docInfo, KoDocumentRdf* docRdf)
         : KPageDialog(parent)
         , d(new KoDocumentInfoDlgPrivate)
 {
     d->m_info = docInfo;
+    d->m_rdf = docRdf;
 
     setCaption(i18n("Document Information"));
     setInitialSize(QSize(500, 500));
@@ -92,12 +104,17 @@ KoDocumentInfoDlg::KoDocumentInfoDlg(QWidget* parent, KoDocumentInfo* docInfo)
     }
     KPageWidgetItem *page = new KPageWidgetItem(infodlg, i18n("General"));
     page->setHeader(i18n("General"));
+
+    // Ugly hack, the mimetype should be a parameter, instead
     KoDocument* doc = dynamic_cast< KoDocument* >(d->m_info->parent());
-    KMimeType::Ptr mime = KMimeType::mimeType(doc->mimeType());
-    if (! mime)
-        mime = KMimeType::defaultMimeTypePtr();
-    page->setIcon(KIcon(KIconLoader::global()->loadMimeTypeIcon(mime->iconName(), KIconLoader::Desktop, 48)));
+    if (doc) {
+        KMimeType::Ptr mime = KMimeType::mimeType(doc->mimeType());
+        if (! mime)
+            mime = KMimeType::defaultMimeTypePtr();
+        page->setIcon(KIcon(KIconLoader::global()->loadMimeTypeIcon(mime->iconName(), KIconLoader::Desktop, 48)));
+    }
     addPage(page);
+    d->m_pages.append(page);
 
     initAboutTab();
 
@@ -108,12 +125,25 @@ KoDocumentInfoDlg::KoDocumentInfoDlg(QWidget* parent, KoDocumentInfo* docInfo)
     page->setHeader(i18n("Author"));
     page->setIcon(KIcon("user-identity"));
     addPage(page);
+    d->m_pages.append(page);
 
     initAuthorTab();
 
-    connect(this, SIGNAL(okClicked()), this, SLOT(slotApply()));
     // Saving encryption implies saving the document, this is done after closing the dialog
     connect(this, SIGNAL(hidden()), this, SLOT(slotSaveEncryption()));
+
+    if (d->m_rdf) {
+        d->m_rdfEditWidget = 0;
+
+#ifdef SHOULD_BUILD_RDF
+        d->m_rdfEditWidget = new KoDocumentRdfEditWidget(this, d->m_rdf);
+        page = new KPageWidgetItem(d->m_rdfEditWidget->widget(), i18n("Rdf"));
+        page->setHeader(i18n("Rdf"));
+        page->setIcon(KIcon("text-rdf"));
+        addPage(page);
+        d->m_pages.append(page);
+#endif
+    }
 }
 
 KoDocumentInfoDlg::~KoDocumentInfoDlg()
@@ -123,6 +153,23 @@ KoDocumentInfoDlg::~KoDocumentInfoDlg()
     delete d;
 }
 
+void KoDocumentInfoDlg::slotButtonClicked(int button)
+{
+    emit buttonClicked(static_cast<KDialog::ButtonCode>(button));
+    switch (button) {
+    case Ok:
+        if (d->m_rdfEditWidget) {
+            if (d->m_rdfEditWidget->shouldDialogCloseBeVetoed()) {
+                return;
+            }
+        }
+        slotApply();
+        accept();
+        return;
+    }
+    KPageDialog::slotButtonClicked(button);
+}
+
 bool KoDocumentInfoDlg::isDocumentSaved()
 {
     return d->m_documentSaved;
@@ -237,6 +284,9 @@ void KoDocumentInfoDlg::slotApply()
 {
     saveAboutData();
     saveAuthorData();
+    if (d->m_rdfEditWidget) {
+        d->m_rdfEditWidget->apply();
+    }
 }
 
 void KoDocumentInfoDlg::saveAboutData()
@@ -481,4 +531,23 @@ void KoDocumentInfoDlg::slotSaveEncryption()
     d->m_documentSaved = !doc->url().isEmpty();
 }
 
-#include "KoDocumentInfoDlg.moc"
+QList<KPageWidgetItem*> KoDocumentInfoDlg::pages() const
+{
+    return d->m_pages;
+}
+
+void KoDocumentInfoDlg::setReadOnly(bool ro)
+{
+    d->m_aboutUi->meComments->setReadOnly(ro);
+
+    Q_FOREACH(KPageWidgetItem* page, d->m_pages) {
+        Q_FOREACH(QLineEdit* le, page->widget()->findChildren<QLineEdit *>()) {
+            le->setReadOnly(ro);
+        }
+        Q_FOREACH(QPushButton* le, page->widget()->findChildren<QPushButton *>()) {
+            le->setDisabled(ro);
+        }
+    }
+}
+
+#include <KoDocumentInfoDlg.moc>
