@@ -31,6 +31,7 @@ Boston, MA 02110-1301, USA.
 
 #include "komain_export.h"
 class KoDocument;
+class KoProgressUpdater;
 
 /**
  *  @brief The class managing all the filters.
@@ -57,7 +58,8 @@ public:
     /**
      * Create a filter manager for a document
      */
-    explicit KoFilterManager(KoDocument* document);
+    explicit KoFilterManager(KoDocument *document,
+                             KoProgressUpdater *progressUpdater = 0);
 
     /**
      * Create a filter manager for the Shape Collection docker.
@@ -80,7 +82,7 @@ public:
      *        to allow embedding for filters. Most likely you do not have to care.
      */
     explicit KoFilterManager(const QString& url, const QByteArray& mimetypeHint = "",
-                             KoFilterChain* const parentChain = 0);
+                             KoFilterChain * const parentChain = 0);
 
     virtual ~KoFilterManager();
 
@@ -91,7 +93,8 @@ public:
      * If the QString which is returned isEmpty() and the status is OK,
      * then we imported the file directly into the document.
      */
-    QString importDocument(const QString& url, KoFilter::ConversionStatus& status);
+    QString importDocument(const QString& url,
+                           KoFilter::ConversionStatus& status);
 
     /**
      * @brief Exports the given file/document to the specified URL/mimetype.
@@ -142,8 +145,10 @@ public:
      */
     bool getBatchMode(void) const;
 
-signals:
-    void sigProgress(int);
+    /**
+     * Return the KoProgressUpdater or NULL if there is none.
+     **/
+    KoProgressUpdater *progressUpdater() const;
 
 private:
     // === API for KoFilterChains === (internal)
@@ -158,16 +163,16 @@ private:
     QString exportFile() const {
         return m_exportUrl;
     }
-    friend KoDocument* KoFilterChain::filterManagerKoDocument() const;
-    KoDocument* document() const {
+    friend KoDocument *KoFilterChain::filterManagerKoDocument() const;
+    KoDocument *document() const {
         return m_document;
     }
     friend int KoFilterChain::filterManagerDirection() const;
     int direction() const {
         return static_cast<int>(m_direction);
     }
-    friend KoFilterChain* KoFilterChain::filterManagerParentChain() const;
-    KoFilterChain* parentChain() const {
+    friend KoFilterChain *KoFilterChain::filterManagerParentChain() const;
+    KoFilterChain *parentChain() const {
         return m_parentChain;
     }
 
@@ -177,8 +182,8 @@ private:
 
     void importErrorHelper(const QString& mimeType, const bool suppressDialog = false);
 
-    KoDocument* m_document;
-    KoFilterChain* const m_parentChain;
+    KoDocument *m_document;
+    KoFilterChain *const m_parentChain;
     QString m_importUrl, m_exportUrl;
     QByteArray m_importUrlMimetypeHint;  ///< suggested mimetype
     KOfficeFilter::Graph m_graph;
