@@ -23,10 +23,12 @@ Boston, MA 02110-1301, USA.
 
 #include <QString>
 #include <QStringList>
-
+#include <QWeakPointer>
 #include <KUrl>
 #include <KDialog>
 
+#include <KoProgressUpdater.h>
+
 class QListWidget;
 
 class KoFilterManager::Private
@@ -34,6 +36,13 @@ class KoFilterManager::Private
 public:
     bool batch;
     QByteArray importMimeType;
+    QWeakPointer<KoProgressUpdater> progressUpdater;
+
+    Private(KoProgressUpdater *progressUpdater_ = 0)
+        : progressUpdater(progressUpdater_)
+    {
+    }
+
 };
 
 class KoFilterChooser : public KDialog
