@@ -2,6 +2,7 @@
    Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
    Copyright (C) 2000-2006 David Faure <faure@kde.org>
    Copyright (C) 2007, 2009 Thomas zander <zander@kde.org>
+   Copyright (C) 2010 Benjamin Port <port.benjamin@gmail.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -28,10 +29,11 @@
 #include "KoDocumentInfoDlg.h"
 #include "KoFileDialog.h"
 #include "KoVersionDialog.h"
-#include "KoDockFactory.h"
+#include "KoDockFactoryBase.h"
 #include "KoDockWidgetTitleBar.h"
 #include "KoPrintJob.h"
 #include "KoDocumentEntry.h"
+#include "KoDockerManager.h"
 
 #include <krecentfilesaction.h>
 #include <kaboutdata.h>
@@ -72,8 +74,6 @@
 
 #include "kofficeversion.h"
 
-class KoDockerManager : public QObject { }; // little hack to be able to use this class as a qobject
-
 class KoPartManager : public KParts::PartManager
 {
 public:
@@ -84,7 +84,7 @@ public:
         setIgnoreScrollBars(true);
     }
     virtual bool eventFilter(QObject *obj, QEvent *ev) {
-        if (!obj->isWidgetType() )
+        if (!obj->isWidgetType())
             return false;
         return KParts::PartManager::eventFilter(obj, ev);
     }
@@ -198,6 +198,7 @@ public:
     KAction *showFileVersions;
     KAction *importFile;
     KAction *exportFile;
+    KToggleAction *toggleDockers;
     KRecentFilesAction *recent;
 
     bool isImporting;
@@ -212,6 +213,7 @@ public:
     QMap<QDockWidget*, bool> dockWidgetVisibilityMap;
     KoDockerManager *dockerManager;
     QList<QDockWidget*> dockWidgets;
+    QList<QDockWidget*> hiddenDockwidgets; // List of dockers hiddent by the call to hideDocker
 };
 
 KoMainWindow::KoMainWindow(const KComponentData &componentData)
@@ -338,6 +340,14 @@ KoMainWindow::KoMainWindow(const KComponentData &componentData)
     sep->setSeparator(true);
     d->splitViewActionList.append(sep);
 
+    d->toggleDockers = new KToggleAction(i18n("Show Dockers"), this);
+    d->toggleDockers->setCheckedState(KGuiItem(i18n("Hide Dockers")));
+    d->toggleDockers->setChecked(true);
+    actionCollection()->addAction("view_toggledockers", d->toggleDockers);
+
+    d->toggleDockers->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_H));
+    connect(d->toggleDockers, SIGNAL(toggled(bool)), SLOT(toggleDockersVisibility(bool)));
+
     d->dockWidgetMenu  = new KActionMenu(i18n("Dockers"), this);
     actionCollection()->addAction("settings_dockers_menu", d->dockWidgetMenu);
     d->dockWidgetMenu->setVisible(false);
@@ -370,12 +380,18 @@ KoMainWindow::KoMainWindow(const KComponentData &componentData)
         }
     }
 
-   KConfigGroup config(KGlobal::config(), "MainWindow");
-   restoreWindowSize( config );
+    KConfigGroup config(KGlobal::config(), "MainWindow");
+    restoreWindowSize( config );
+
+    d->dockerManager = new KoDockerManager(this);
+    connect(this, SIGNAL(restoringDone()), d->dockerManager, SLOT(removeUnusedOptionWidgets()));
 }
 
 KoMainWindow::~KoMainWindow()
 {
+    // Explicitly delete the docker manager to ensure that it is deleted before the dockers
+    delete d->dockerManager;
+    d->dockerManager = 0;
     // The doc and view might still exist (this is the case when closing the window)
     if (d->rootDoc)
         d->rootDoc->removeShell(this);
@@ -417,15 +433,8 @@ void KoMainWindow::setRootDocument(KoDocument *doc)
     if (d->docToOpen && d->docToOpen != doc) {
         d->docToOpen->removeShell(this);
         delete d->docToOpen;
-        d->docToOpen = 0;
-    } else {
-        d->docToOpen = 0;
-    }
-
-    if (d->dockerManager) { // All the views will be deleted, so lets remove this one too
-        delete d->dockerManager;
-        d->dockerManager = 0;
     }
+    d->docToOpen = 0;
 
     //kDebug(30003) <<"KoMainWindow::setRootDocument this =" << this <<" doc =" << doc;
     QList<KoView*> oldRootViews = d->rootViews;
@@ -477,7 +486,7 @@ void KoMainWindow::setRootDocument(KoDocument *doc)
     d->closeFile->setEnabled(enable);
     updateCaption();
 
-    d->manager->setActivePart(d->rootDoc, currentView());
+    d->manager->setActivePart(d->rootDoc, doc ? d->rootViews.first() : 0);
     emit restoringDone();
 
     while(!oldRootViews.isEmpty()) {
@@ -574,7 +583,7 @@ void KoMainWindow::updateCaption()
     else if (rootDocument()->isCurrent()) {
         QString caption( rootDocument()->caption() );
         if (d->readOnly)
-            caption += " " + i18n("(write protected)");
+            caption += ' ' + i18n("(write protected)");
 
         updateCaption(caption, rootDocument()->isModified());
         if (!rootDocument()->url().fileName(KUrl::ObeyTrailingSlash).isEmpty())
@@ -962,6 +971,7 @@ bool KoMainWindow::saveDocument(bool saveas, bool silent)
                     if (ret) {
                         kDebug(30003) << "Successful Save As!";
                         addRecentURL(newURL);
+                        setReadWrite(true);
                     } else {
                         kDebug(30003) << "Failed Save As!";
                         pDoc->setUrl(oldURL);
@@ -1036,6 +1046,15 @@ bool KoMainWindow::saveDocument(bool saveas, bool silent)
 void KoMainWindow::closeEvent(QCloseEvent *e)
 {
     if (queryClose()) {
+        if (d->docToOpen) {
+            // The open pane is visible
+            d->docToOpen->deleteOpenPane(true);
+        }
+        // Reshow the docker that were temporarely hidden before saving settings
+        foreach(QDockWidget* dw, d->hiddenDockwidgets) {
+            dw->show();
+        }
+        d->hiddenDockwidgets.clear();
         saveWindowSettings();
         setRootDocument(0);
         if (!d->dockWidgetVisibilityMap.isEmpty()) { // re-enable dockers for persistency
@@ -1234,7 +1253,7 @@ void KoMainWindow::slotDocumentInfo()
     if (!docInfo)
         return;
 
-    KoDocumentInfoDlg *dlg = new KoDocumentInfoDlg(this, docInfo);
+    KoDocumentInfoDlg *dlg = new KoDocumentInfoDlg(this, docInfo, rootDocument()->documentRdf());
     if (dlg->exec()) {
         if (dlg->isDocumentSaved()) {
             rootDocument()->setModified(false);
@@ -1317,11 +1336,18 @@ KoPrintJob* KoMainWindow::exportToPdf(QString pdfFileName)
         if (dialog.exec() != QDialog::Accepted)
             return 0;
         KUrl url(dialog.selectedUrl());
-        // TODO warn when overwriting
+        if (KIO::NetAccess::exists(url,  KIO::NetAccess::DestinationSide, this)) {
+            bool overwrite = KMessageBox::questionYesNo(this,
+                                            i18n("A document with this name already exists.\n"\
+                                                "Do you want to overwrite it?"),
+                                            i18n("Warning")) == KMessageBox::Yes;
+            if (!overwrite) {
+                return 0;
+            }
+        }
         pdfFileName = url.toLocalFile();
     }
-
-    KoPrintJob *printJob = rootView()->createPrintJob();
+    KoPrintJob *printJob = rootView()->createPdfPrintJob();
     if (printJob == 0)
         return 0;
     d->applyDefaultSettings(printJob->printer());
@@ -1343,7 +1369,7 @@ void KoMainWindow::slotConfigureToolbars()
     if (rootDocument())
         saveMainWindowSettings(KGlobal::config()->group(rootDocument()->componentData().componentName()));
     KEditToolBar edit(factory(), this);
-    connect(&edit, SIGNAL(newToolbarConfig()), this, SLOT(slotNewToolbarConfig()));
+    connect(&edit, SIGNAL(newToolBarConfig()), this, SLOT(slotNewToolbarConfig()));
     (void) edit.exec();
 }
 
@@ -1460,9 +1486,6 @@ void KoMainWindow::slotRemoveView()
         kWarning() << "view not found in d->rootViews!";
     }
 
-    // Prevent the view's destroyed() signal from triggering GUI rebuilding (too early)
-    d->manager->setActivePart(0, 0);
-
     d->rootViews.removeAll(view);
     delete view;
     view = 0;
@@ -1508,7 +1531,7 @@ void KoMainWindow::slotProgress(int value)
     if (d->firstTime) {
         // The statusbar might not even be created yet.
         // So check for that first, and create it if necessary
-        QStatusBar* bar = qFindChild<QStatusBar *>(this);
+        QStatusBar *bar = findChild<QStatusBar *>();
         if (!bar) {
             statusBar()->show();
             QApplication::sendPostedEvents(this, QEvent::ChildAdded);
@@ -1522,6 +1545,7 @@ void KoMainWindow::slotProgress(int value)
 
         d->progress = new QProgressBar(statusBar());
         d->progress->setMaximumHeight(statusBar()->fontMetrics().height());
+        d->progress->setRange(0, 100);
         statusBar()->addPermanentWidget(d->progress);
         d->progress->show();
         d->firstTime = false;
@@ -1754,7 +1778,7 @@ void KoMainWindow::setDocToOpen(KoDocument *doc)
     d->docToOpen = doc;
 }
 
-QDockWidget* KoMainWindow::createDockWidget(KoDockFactory* factory)
+QDockWidget* KoMainWindow::createDockWidget(KoDockFactoryBase* factory)
 {
     QDockWidget* dockWidget = 0;
 
@@ -1768,11 +1792,10 @@ QDockWidget* KoMainWindow::createDockWidget(KoDockFactory* factory)
 
         KoDockWidgetTitleBar *titleBar = 0;
         // Check if the dock widget is supposed to be collapsable
-        if (factory->isCollapsable()) {
-            if (!dockWidget->titleBarWidget()) {
-                titleBar = new KoDockWidgetTitleBar(dockWidget);
-                dockWidget->setTitleBarWidget(titleBar);
-            }
+        if (!dockWidget->titleBarWidget()) {
+            titleBar = new KoDockWidgetTitleBar(dockWidget);
+            dockWidget->setTitleBarWidget(titleBar);
+            titleBar->setCollapsable(factory->isCollapsable());
         }
 
         dockWidget->setObjectName(factory->id());
@@ -1785,18 +1808,18 @@ QDockWidget* KoMainWindow::createDockWidget(KoDockFactory* factory)
         bool visible = true;
 
         switch (factory->defaultDockPosition()) {
-        case KoDockFactory::DockTornOff:
+        case KoDockFactoryBase::DockTornOff:
             dockWidget->setFloating(true); // position nicely?
             break;
-        case KoDockFactory::DockTop:
+        case KoDockFactoryBase::DockTop:
             side = Qt::TopDockWidgetArea; break;
-        case KoDockFactory::DockLeft:
+        case KoDockFactoryBase::DockLeft:
             side = Qt::LeftDockWidgetArea; break;
-        case KoDockFactory::DockBottom:
+        case KoDockFactoryBase::DockBottom:
             side = Qt::BottomDockWidgetArea; break;
-        case KoDockFactory::DockRight:
+        case KoDockFactoryBase::DockRight:
             side = Qt::RightDockWidgetArea; break;
-        case KoDockFactory::DockMinimized:
+        case KoDockFactoryBase::DockMinimized:
         default:
             side = Qt::RightDockWidgetArea;
             visible = false;
@@ -1859,13 +1882,13 @@ QList<QDockWidget*> KoMainWindow::dockWidgets()
     return d->dockWidgetsMap.values();
 }
 
-QList<KoCanvasObserver*> KoMainWindow::canvasObservers()
+QList<KoCanvasObserverBase*> KoMainWindow::canvasObservers()
 {
 
-    QList<KoCanvasObserver*> observers;
+    QList<KoCanvasObserverBase*> observers;
 
     foreach(QDockWidget *docker, dockWidgets()) {
-        KoCanvasObserver *observer = dynamic_cast<KoCanvasObserver*>(docker);
+        KoCanvasObserverBase *observer = dynamic_cast<KoCanvasObserverBase*>(docker);
         if (observer) {
             observers << observer;
         }
@@ -1879,13 +1902,25 @@ KoDockerManager * KoMainWindow::dockerManager() const
     return d->dockerManager;
 }
 
-void KoMainWindow::setDockerManager(KoDockerManager *dm)
+void KoMainWindow::toggleDockersVisibility(bool v) const
 {
-    d->dockerManager = dm;
-    if (dm) {
-        QObject *manager = static_cast<QObject*> (dm);
-        manager->setParent(this); // make sure that the dockerManager is deleted by us.
-        connect(this, SIGNAL(restoringDone()), d->dockerManager, SLOT(removeUnusedOptionWidgets()));
+    Q_UNUSED(v);
+    if (d->hiddenDockwidgets.isEmpty()){
+        foreach(QObject* widget, children()) {
+            if (widget->inherits("QDockWidget")) {
+                QDockWidget* dw = static_cast<QDockWidget*>(widget);
+                if (dw->isVisible()) {
+                    dw->hide();
+                    d->hiddenDockwidgets << dw;
+                }
+            }
+        }
+    }
+    else {
+        foreach(QDockWidget* dw, d->hiddenDockwidgets) {
+            dw->show();
+        }
+        d->hiddenDockwidgets.clear();
     }
 }
 
@@ -1906,4 +1941,4 @@ KoView* KoMainWindow::currentView() const
     return 0;
 }
 
-#include "KoMainWindow.moc"
+#include <KoMainWindow.moc>
