@@ -30,10 +30,20 @@
 #include <KoXmlReader.h>
 #include <KoXmlWriter.h>
 
+namespace KoOdfNumberStyles
+{
+
+    static bool saveOdfTimeFormat(KoXmlWriter &elementWriter, QString &format, QString &text, bool &antislash);
+    static void parseOdfDateKlocale(KoXmlWriter &elementWriter, QString &format, QString &text);
+    static bool saveOdfKlocaleTimeFormat(KoXmlWriter &elementWriter, QString &format, QString &text);
+    static void parseOdfTimeKlocale(KoXmlWriter &elementWriter, QString &format, QString &text);
+    static void addKofficeNumericStyleExtension(KoXmlWriter &elementWriter, const QString &_suffix, const QString &_prefix);
+
+
 // OO spec 2.5.4. p68. Conversion to Qt format: see qdate.html
 // OpenCalcImport::loadFormat has similar code, but slower, intermixed with other stuff,
 // lacking long-textual forms.
-QPair<QString, KoOdfNumberStyles::NumericStyleFormat> KoOdfNumberStyles::loadOdfNumberStyle(const KoXmlElement& parent)
+QPair<QString, NumericStyleFormat> loadOdfNumberStyle(const KoXmlElement &parent)
 {
     NumericStyleFormat dataStyle;
 
@@ -56,10 +66,9 @@ QPair<QString, KoOdfNumberStyles::NumericStyleFormat> KoOdfNumberStyles::loadOdf
     QString format;
     int precision = -1;
     int leadingZ  = 1;
-#ifdef __GNUC__
-#warning Nothing changes thousandsSep - dead constant
-#endif
-    bool thousandsSep = false;
+
+    // Long-standing todo: thousandsSep
+    //bool thousandsSep = false;
     //todo negred
     //bool negRed = false;
     bool ok = false;
@@ -83,7 +92,16 @@ QPair<QString, KoOdfNumberStyles::NumericStyleFormat> KoOdfNumberStyles::loadOdf
             }
             // TODO the spec has a strange mention of number:format-source
             else if (e.attributeNS(KoXmlNS::number, "textual", QString()) == "true") {
-                format += shortForm ? "MMM" : "MMMM";
+                bool isExtraShort = false;      // find out if we have to use the extra-short month name (just 1st letter)
+                if (e.attributeNS(KoXmlNS::koffice, "number-length", QString()) == "extra-short") {
+                    isExtraShort = true;
+                }
+
+                if (!isExtraShort) {            // for normal month format (first 3 letters or complete name)
+                    format += shortForm ? "MMM" : "MMMM";
+                } else {                        // for the extra-short month name use 'X' as a special mark
+                    format += "X";
+                }
             } else { // month number
                 format += shortForm ? "M" : "MM";
             }
@@ -128,15 +146,15 @@ QPair<QString, KoOdfNumberStyles::NumericStyleFormat> KoOdfNumberStyles::loadOdf
                 if (ok)
                     leadingZ = d;
             }
-            if (thousandsSep && leadingZ <= 3) {
-                format += "#,";
-                for (i = leadingZ; i <= 3; ++i)
-                    format += '#';
-            }
+//            if (thousandsSep && leadingZ <= 3) {
+//                format += "#,";
+//                for (i = leadingZ; i <= 3; ++i)
+//                    format += '#';
+//            }
             for (i = 1; i <= leadingZ; ++i) {
                 format +=  '0';
-                if ((i % 3 == 0) && thousandsSep)
-                    format = + ',' ;
+//                if ((i % 3 == 0) && thousandsSep)
+//                    format = + ',' ;
             }
             if (precision > -1) {
                 format += '.';
@@ -168,16 +186,16 @@ QPair<QString, KoOdfNumberStyles::NumericStyleFormat> KoOdfNumberStyles::loadOdf
                     exp = 1;
             }
 
-            if (thousandsSep && leadingZ <= 3) {
-                format += "#,";
-                for (i = leadingZ; i <= 3; ++i)
-                    format += '#';
-            }
+//            if (thousandsSep && leadingZ <= 3) {
+//                format += "#,";
+//                for (i = leadingZ; i <= 3; ++i)
+//                    format += '#';
+//            }
 
             for (i = 1; i <= leadingZ; ++i) {
                 format += '0';
-                if ((i % 3 == 0) && thousandsSep)
-                    format += ',';
+//                if ((i % 3 == 0) && thousandsSep)
+//                    format += ',';
             }
 
             if (precision > -1) {
@@ -234,13 +252,34 @@ QPair<QString, KoOdfNumberStyles::NumericStyleFormat> KoOdfNumberStyles::loadOdf
                     format += '?';
             }
         }
-        // Not needed:
-        //  <style:map style:condition="value()&gt;=0" style:apply-style-name="N106P0"/>
-        // we handle painting negative numbers in red differently
 
+        // stylesmap's are embedded into a style and are pointing to another style that
+        // should be used insteat ot this style if the defined condition is true. E.g.;
+        // <number:number-style style:name="N139P0" style:volatile="true"/>
+        // <number:number-style style:name="N139P1" style:volatile="true"/>
+        // <number:number-style style:name="N139P2" style:volatile="true"/>
+        // <number:text-style style:name="N139">
+        //   <style:map style:condition="value()&gt;0" style:apply-style-name="N139P0"/>
+        //   <style:map style:condition="value()&lt;0" style:apply-style-name="N139P1"/>
+        //   <style:map style:condition="value()=0" style:apply-style-name="N139P2"/>
+        // </number:text-style>
+        for (KoXmlNode node(e); !node.isNull(); node = node.nextSibling()) {
+            KoXmlElement elem = node.toElement();
+            if (elem.namespaceURI() == KoXmlNS::style && elem.localName() == "map") {
+                QString condition, applyStyleName;
+                if (elem.hasAttributeNS(KoXmlNS::style, "condition"))
+                    condition = elem.attributeNS(KoXmlNS::style, "condition");
+                if (elem.hasAttributeNS(KoXmlNS::style, "apply-style-name"))
+                    applyStyleName = elem.attributeNS(KoXmlNS::style, "apply-style-name");
+                dataStyle.styleMaps.append( QPair<QString,QString>(condition,applyStyleName) );
+            }
+        }
     }
 
     const QString styleName = parent.attributeNS(KoXmlNS::style, "name", QString());
+
+kDebug()<<"99 *****************************************************************************";
+//Q_ASSERT(false);
     kDebug(30003) << "data style:" << styleName << " qt format=" << format;
     if (!prefix.isEmpty()) {
         kDebug(30003) << " format.left( prefix.length() ) :" << format.left(prefix.length()) << " prefix :" << prefix;
@@ -275,7 +314,7 @@ QPair<QString, KoOdfNumberStyles::NumericStyleFormat> KoOdfNumberStyles::loadOdf
         } \
     }
 
-void KoOdfNumberStyles::parseOdfTimeKlocale(KoXmlWriter &elementWriter, QString & format, QString & text)
+void parseOdfTimeKlocale(KoXmlWriter &elementWriter, QString &format, QString &text)
 {
     kDebug(30003) << "parseOdfTimeKlocale(KoXmlWriter &elementWriter, QString & format, QString & text ) :" << format;
     do {
@@ -287,7 +326,7 @@ void KoOdfNumberStyles::parseOdfTimeKlocale(KoXmlWriter &elementWriter, QString
     addTextNumber(text, elementWriter);
 }
 
-bool KoOdfNumberStyles::saveOdfKlocaleTimeFormat(KoXmlWriter &elementWriter, QString & format, QString & text)
+bool saveOdfKlocaleTimeFormat(KoXmlWriter &elementWriter, QString &format, QString &text)
 {
     bool changed = false;
     if (format.startsWith("%H")) {   //hh
@@ -343,7 +382,7 @@ bool KoOdfNumberStyles::saveOdfKlocaleTimeFormat(KoXmlWriter &elementWriter, QSt
 }
 
 
-bool KoOdfNumberStyles::saveOdfTimeFormat(KoXmlWriter &elementWriter, QString & format, QString & text, bool &antislash)
+bool saveOdfTimeFormat(KoXmlWriter &elementWriter, QString &format, QString &text, bool &antislash)
 {
     bool changed = false;
     //we can also add time to date.
@@ -411,14 +450,14 @@ bool KoOdfNumberStyles::saveOdfTimeFormat(KoXmlWriter &elementWriter, QString &
     return changed;
 }
 
-QString KoOdfNumberStyles::saveOdfTimeStyle(KoGenStyles &mainStyles, const QString & _format, bool klocaleFormat,
-        const QString & _prefix, const QString & _suffix)
+QString saveOdfTimeStyle(KoGenStyles &mainStyles, const QString &_format, bool klocaleFormat,
+        const QString &_prefix, const QString &_suffix)
 {
     Q_UNUSED(_prefix);
     Q_UNUSED(_suffix);
-    kDebug(30003) << "QString KoOdfNumberStyles::saveOdfTimeStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
+    //kDebug(30003) << "QString KoOdfNumberStyles::saveOdfTimeStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
     QString format(_format);
-    KoGenStyle currentStyle(KoGenStyle::StyleNumericTime);
+    KoGenStyle currentStyle(KoGenStyle::NumericTimeStyle);
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter elementWriter(&buffer);    // TODO pass indentation level
@@ -443,13 +482,13 @@ QString KoOdfNumberStyles::saveOdfTimeStyle(KoGenStyles &mainStyles, const QStri
     }
     QString elementContents = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
     currentStyle.addChildElement("number", elementContents);
-    return mainStyles.lookup(currentStyle, "N");
+    return mainStyles.insert(currentStyle, "N");
 }
 
 //convert klocale string to good format
-void KoOdfNumberStyles::parseOdfDateKlocale(KoXmlWriter &elementWriter, QString & format, QString & text)
+void parseOdfDateKlocale(KoXmlWriter &elementWriter, QString &format, QString &text)
 {
-    kDebug(30003) << "KoOdfNumberStyles::parseOdfDateKlocale(KoXmlWriter &elementWriter, QString & format, QString & text ) :" << format;
+    kDebug(30003) << format;
     do {
         if (format.startsWith("%Y")) {
             addTextNumber(text, elementWriter);
@@ -530,17 +569,17 @@ void KoOdfNumberStyles::parseOdfDateKlocale(KoXmlWriter &elementWriter, QString
     addTextNumber(text, elementWriter);
 }
 
-QString KoOdfNumberStyles::saveOdfDateStyle(KoGenStyles &mainStyles, const QString & _format, bool klocaleFormat,
-        const QString & _prefix, const QString & _suffix)
+QString saveOdfDateStyle(KoGenStyles &mainStyles, const QString &_format, bool klocaleFormat,
+        const QString &_prefix, const QString &_suffix)
 {
     Q_UNUSED(_prefix);
     Q_UNUSED(_suffix);
-    kDebug(30003) << "QString KoOdfNumberStyles::saveOdfDateStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
+    //kDebug(30003) << _format;
     QString format(_format);
 
     // Not supported into Qt: "era" "week-of-year" "quarter"
 
-    KoGenStyle currentStyle(KoGenStyle::StyleNumericDate);
+    KoGenStyle currentStyle(KoGenStyle::NumericDateStyle);
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter elementWriter(&buffer);    // TODO pass indentation level
@@ -555,10 +594,11 @@ QString KoOdfNumberStyles::saveOdfDateStyle(KoGenStyles &mainStyles, const QStri
                 format = format.remove(0, 1);
             }
             //TODO implement loading ! What is it ?
-            else if (format.startsWith("MMMMM")) {
+            else if (format.startsWith("MMMMM")) {        // MMMMM is extra-short month name (only 1st character)
                 addTextNumber(text, elementWriter);
                 elementWriter.startElement("number:month");
                 elementWriter.addAttribute("number:textual", "true");
+                elementWriter.addAttribute("koffice:number-length", "extra-short");
                 elementWriter.endElement();
                 format = format.remove(0, 5);
             } else if (format.startsWith("MMMM")) {
@@ -668,17 +708,17 @@ QString KoOdfNumberStyles::saveOdfDateStyle(KoGenStyles &mainStyles, const QStri
 
     QString elementContents = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
     currentStyle.addChildElement("number", elementContents);
-    return mainStyles.lookup(currentStyle, "N");
+    return mainStyles.insert(currentStyle, "N");
 }
 
 
-QString KoOdfNumberStyles::saveOdfFractionStyle(KoGenStyles &mainStyles, const QString & _format,
+QString saveOdfFractionStyle(KoGenStyles &mainStyles, const QString &_format,
         const QString &_prefix, const QString &_suffix)
 {
-    kDebug(30003) << "QString saveOdfFractionStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
+    //kDebug(30003) << "QString saveOdfFractionStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
     QString format(_format);
 
-    KoGenStyle currentStyle(KoGenStyle::StyleNumericFraction);
+    KoGenStyle currentStyle(KoGenStyle::NumericFractionStyle);
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter elementWriter(&buffer);    // TODO pass indentation level
@@ -727,17 +767,17 @@ QString KoOdfNumberStyles::saveOdfFractionStyle(KoGenStyles &mainStyles, const Q
 
     QString elementContents = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
     currentStyle.addChildElement("number", elementContents);
-    return mainStyles.lookup(currentStyle, "N");
+    return mainStyles.insert(currentStyle, "N");
 }
 
 
-QString KoOdfNumberStyles::saveOdfNumberStyle(KoGenStyles &mainStyles, const QString & _format,
+QString saveOdfNumberStyle(KoGenStyles &mainStyles, const QString &_format,
         const QString &_prefix, const QString &_suffix)
 {
-    kDebug(30003) << "QString saveOdfNumberStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
+    //kDebug(30003) << "QString saveOdfNumberStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
     QString format(_format);
 
-    KoGenStyle currentStyle(KoGenStyle::StyleNumericNumber);
+    KoGenStyle currentStyle(KoGenStyle::NumericNumberStyle);
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter elementWriter(&buffer);    // TODO pass indentation level
@@ -759,7 +799,7 @@ QString KoOdfNumberStyles::saveOdfNumberStyle(KoGenStyles &mainStyles, const QSt
     text = _prefix ;
     addTextNumber(text, elementWriter);
     elementWriter.startElement("number:number");
-    kDebug(30003) << " decimalplaces :" << decimalplaces << " integerdigits :" << integerdigits;
+    //kDebug(30003) << " decimalplaces :" << decimalplaces << " integerdigits :" << integerdigits;
     if (!beforeSeparator)
         elementWriter.addAttribute("number:decimal-places", decimalplaces);
     elementWriter.addAttribute("number:min-integer-digits", integerdigits);
@@ -771,10 +811,10 @@ QString KoOdfNumberStyles::saveOdfNumberStyle(KoGenStyles &mainStyles, const QSt
 
     QString elementContents = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
     currentStyle.addChildElement("number", elementContents);
-    return mainStyles.lookup(currentStyle, "N");
+    return mainStyles.insert(currentStyle, "N");
 }
 
-QString KoOdfNumberStyles::saveOdfPercentageStyle(KoGenStyles &mainStyles, const QString & _format,
+QString saveOdfPercentageStyle(KoGenStyles &mainStyles, const QString &_format,
         const QString &_prefix, const QString &_suffix)
 {
     //<number:percentage-style style:name="N11">
@@ -782,10 +822,10 @@ QString KoOdfNumberStyles::saveOdfPercentageStyle(KoGenStyles &mainStyles, const
     //<number:text>%</number:text>
     //</number:percentage-style>
 
-    kDebug(30003) << "QString saveOdfPercentageStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
+    //kDebug(30003) << "QString saveOdfPercentageStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
     QString format(_format);
 
-    KoGenStyle currentStyle(KoGenStyle::StyleNumericPercentage);
+    KoGenStyle currentStyle(KoGenStyle::NumericPercentageStyle);
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter elementWriter(&buffer);    // TODO pass indentation level
@@ -820,11 +860,11 @@ QString KoOdfNumberStyles::saveOdfPercentageStyle(KoGenStyles &mainStyles, const
 
     QString elementContents = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
     currentStyle.addChildElement("number", elementContents);
-    return mainStyles.lookup(currentStyle, "N");
+    return mainStyles.insert(currentStyle, "N");
 
 }
 
-QString KoOdfNumberStyles::saveOdfScientificStyle(KoGenStyles &mainStyles, const QString & _format,
+QString saveOdfScientificStyle(KoGenStyles &mainStyles, const QString &_format,
         const QString &_prefix, const QString &_suffix)
 {
     //<number:number-style style:name="N60">
@@ -832,10 +872,10 @@ QString KoOdfNumberStyles::saveOdfScientificStyle(KoGenStyles &mainStyles, const
     //</number:number-style>
 
     //example 000,000e+0000
-    kDebug(30003) << "QString saveOdfScientificStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
+    //kDebug(30003) << "QString saveOdfScientificStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
     QString format(_format);
 
-    KoGenStyle currentStyle(KoGenStyle::StyleNumericScientific);
+    KoGenStyle currentStyle(KoGenStyle::NumericScientificStyle);
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     int decimalplace = 0;
@@ -878,7 +918,7 @@ QString KoOdfNumberStyles::saveOdfScientificStyle(KoGenStyles &mainStyles, const
     addTextNumber(text, elementWriter);
 
     elementWriter.startElement("number:scientific-number");
-    kDebug(30003) << " decimalplace :" << decimalplace << " integerdigits :" << integerdigits << " exponentdigits :" << exponentdigits;
+    //kDebug(30003) << " decimalplace :" << decimalplace << " integerdigits :" << integerdigits << " exponentdigits :" << exponentdigits;
     if (!beforeSeparator)
         elementWriter.addAttribute("number:decimal-places", decimalplace);
     elementWriter.addAttribute("number:min-integer-digits", integerdigits);
@@ -891,11 +931,11 @@ QString KoOdfNumberStyles::saveOdfScientificStyle(KoGenStyles &mainStyles, const
 
     QString elementContents = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
     currentStyle.addChildElement("number", elementContents);
-    return mainStyles.lookup(currentStyle, "N");
+    return mainStyles.insert(currentStyle, "N");
 }
 
-QString KoOdfNumberStyles::saveOdfCurrencyStyle(KoGenStyles &mainStyles,
-        const QString & _format, const QString &symbol,
+QString saveOdfCurrencyStyle(KoGenStyles &mainStyles,
+        const QString &_format, const QString &symbol,
         const QString &_prefix, const QString &_suffix)
 {
 
@@ -905,10 +945,10 @@ QString KoOdfNumberStyles::saveOdfCurrencyStyle(KoGenStyles &mainStyles,
     //<number:currency-symbol>VEB</number:currency-symbol>
     //</number:currency-style>
 
-    kDebug(30003) << "QString saveOdfCurrencyStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
+    //kDebug(30003) << "QString saveOdfCurrencyStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
     QString format(_format);
 
-    KoGenStyle currentStyle(KoGenStyle::StyleNumericCurrency);
+    KoGenStyle currentStyle(KoGenStyle::NumericCurrencyStyle);
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter elementWriter(&buffer);    // TODO pass indentation level
@@ -932,7 +972,7 @@ QString KoOdfNumberStyles::saveOdfCurrencyStyle(KoGenStyles &mainStyles,
     addTextNumber(text, elementWriter);
 
     elementWriter.startElement("number:number");
-    kDebug(30003) << " decimalplaces :" << decimalplaces << " integerdigits :" << integerdigits;
+    //kDebug(30003) << " decimalplaces :" << decimalplaces << " integerdigits :" << integerdigits;
     if (!beforeSeparator)
         elementWriter.addAttribute("number:decimal-places", decimalplaces);
     elementWriter.addAttribute("number:min-integer-digits", integerdigits);
@@ -943,26 +983,26 @@ QString KoOdfNumberStyles::saveOdfCurrencyStyle(KoGenStyles &mainStyles,
     addKofficeNumericStyleExtension(elementWriter, _suffix, _prefix);
 
     elementWriter.startElement("number:currency-symbol");
-    kDebug(30003) << " currency-symbol:" << symbol;
+    //kDebug(30003) << " currency-symbol:" << symbol;
     elementWriter.addTextNode(symbol.toUtf8());
     elementWriter.endElement();
 
     QString elementContents = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
     currentStyle.addChildElement("number", elementContents);
-    return mainStyles.lookup(currentStyle, "N");
+    return mainStyles.insert(currentStyle, "N");
 }
 
-QString KoOdfNumberStyles::saveOdfTextStyle(KoGenStyles &mainStyles, const QString & _format, const QString &_prefix, const QString &_suffix)
+QString saveOdfTextStyle(KoGenStyles &mainStyles, const QString &_format, const QString &_prefix, const QString &_suffix)
 {
 
     //<number:text-style style:name="N100">
     //<number:text-content/>
     ///</number:text-style>
 
-    kDebug(30003) << "QString saveOdfTextStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
+    //kDebug(30003) << "QString saveOdfTextStyle( KoGenStyles &mainStyles, const QString & _format ) :" << _format;
     QString format(_format);
 
-    KoGenStyle currentStyle(KoGenStyle::StyleNumericText);
+    KoGenStyle currentStyle(KoGenStyle::NumericTextStyle);
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter elementWriter(&buffer);    // TODO pass indentation level
@@ -982,12 +1022,12 @@ QString KoOdfNumberStyles::saveOdfTextStyle(KoGenStyles &mainStyles, const QStri
 
     QString elementContents = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
     currentStyle.addChildElement("number", elementContents);
-    return mainStyles.lookup(currentStyle, "N");
+    return mainStyles.insert(currentStyle, "N");
 }
 
 //This is an extension of numeric style. For the moment we used namespace of
 //oasis format for specific koffice extension. Change it for the future.
-void KoOdfNumberStyles::addKofficeNumericStyleExtension(KoXmlWriter & elementWriter, const QString &_suffix, const QString &_prefix)
+void addKofficeNumericStyleExtension(KoXmlWriter &elementWriter, const QString &_suffix, const QString &_prefix)
 {
     if (!_suffix.isEmpty()) {
         elementWriter.startElement("number:suffix");
@@ -1000,3 +1040,4 @@ void KoOdfNumberStyles::addKofficeNumericStyleExtension(KoXmlWriter & elementWri
         elementWriter.endElement();
     }
 }
+}
