@@ -1,6 +1,6 @@
 /*
  *  Copyright (c) 2003 Patrick Julien  <freak@codepimps.org>
- *  Copyright (c) 2004 Cyrille Berger <cberger@cberger.net>
+ *  Copyright (c) 2004,2010 Cyrille Berger <cberger@cberger.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,15 +27,13 @@
 #include "pigment_export.h"
 
 #include <KoGenericRegistry.h>
+#include <KoColorSpace.h>
+#include <KoColorSpaceFactory.h>
 
-class KoColorSpace;
-class KoColorSpaceFactory;
 class KoColorProfile;
 class KoColorConversionSystem;
 class KoColorConversionCache;
 
-// XXX: USE STATIC DELETER! USE STATIC DELETER!
-
 /**
  * The registry for colorspaces and profiles.
  * This class contains:
@@ -43,12 +41,17 @@ class KoColorConversionCache;
  *      - a registry of singleton colorspace factories.
  *      - a registry of icc profiles
  */
-class PIGMENTCMS_EXPORT KoColorSpaceRegistry : public QObject,  public KoGenericRegistry<KoColorSpaceFactory *> {
+class PIGMENTCMS_EXPORT KoColorSpaceRegistry : public QObject
+{
 public:
     enum ColorSpaceListVisibility {
         OnlyUserVisible = 1, ///< Only user visible color space
         AllColorSpaces = 4 ///< All color space even those not visible to the user
     };
+    enum ColorSpaceListProfilesSelection {
+        OnlyDefaultProfile = 1, ///< Only add the default profile
+        AllProfiles = 4 ///< Add all profiles
+    };
 
     /**
      * Return an instance of the KoColorSpaceRegistry
@@ -59,18 +62,25 @@ public:
     virtual ~KoColorSpaceRegistry();
 
 public:
-   /**
-     * add a color space to the registry
-     * @param item the color space factory to add
-     */
+    /**
+      * add a color space to the registry
+      * @param item the color space factory to add
+      */
     void add(KoColorSpaceFactory* item);
 
     /**
-     * add a color space to the registry
-     * @param id the id of the color space
-     * @param item the color space factory to add
+     * Remove a color space factory from the registry. Note that it is the
+     * responsibility of the caller to ensure that the colorspaces are not
+     * used anymore.
      */
-    void add(const QString &id, KoColorSpaceFactory* item);
+    void remove(KoColorSpaceFactory* item);
+
+    /**
+     * Add a profile to the profile map but do not add it to the
+     * color conversion system yet.
+     * @param profile the new profile to be registered.
+     */
+    void addProfileToMap(KoColorProfile *p);
 
     /**
      * register the profile with the color space registry
@@ -78,7 +88,19 @@ public:
      *  colorspaces.
      */
     void addProfile(KoColorProfile* profile);
-    void addProfile(const KoColorProfile* profile);
+    void addProfile(const KoColorProfile* profile); // TODO why ?
+    void removeProfile(KoColorProfile* profile);
+    
+    /**
+     * Create an alias to a profile with a different name. Then @ref profileByName
+     * will return the profile @p to when passed @p name as a parameter.
+     */
+    void addProfileAlias(const QString& name, const QString& to);
+
+    /**
+     * create a profile of the specified type.
+     */
+    const KoColorProfile* createColorProfile(const QString & colorModelId, const QString & colorDepthId, const QByteArray& rawData);
 
     /**
      * Return a profile by its given name, or 0 if none registered.
@@ -97,7 +119,7 @@ public:
      * @param factory the factory with which all the returned profiles will work.
      * @return a list of profiles for the factory
      */
-    QList<const KoColorProfile *>  profilesFor(const KoColorSpaceFactory * factory);
+    QList<const KoColorProfile *>  profilesFor(const KoColorSpaceFactory * factory) const;
 
     /**
      * Return the list of profiles for a colorspace with the argument id.
@@ -107,12 +129,12 @@ public:
      * @param id the colorspace-id with which all the returned profiles will work.
      * @return a list of profiles for the factory
      */
-    QList<const KoColorProfile *>  profilesFor(const KoID& id);
-    
+    QList<const KoColorProfile *>  profilesFor(const KoID& id) const;
+
     /**
      * @return a list of color spaces compatible with this profile
      */
-    QList<const KoColorSpaceFactory*> colorSpacesFor( const KoColorProfile* _profile);
+    QList<const KoColorSpaceFactory*> colorSpacesFor(const KoColorProfile* _profile) const;
 
     /**
      * Return the list of profiles for a colorspace with the argument id.
@@ -122,8 +144,12 @@ public:
      * @param colorSpaceId the colorspace-id with which all the returned profiles will work.
      * @return a list of profiles for the factory
      */
-    QList<const KoColorProfile *>  profilesFor(const QString &colorSpaceId);
+    QList<const KoColorProfile *>  profilesFor(const QString& id) const;
+    const KoColorSpaceFactory* colorSpaceFactory(const QString &colorSpaceId) const;
 
+    KoColorSpace* grabColorSpace(const KoColorSpace*);
+    void releaseColorSpace(KoColorSpace*);
+private:
     /**
      * Return a colorspace that works with the parameter profile.
      * @param csID the ID of the colorspace that you want to have returned
@@ -146,6 +172,21 @@ public:
      * @return the wanted colorspace, or 0 when the cs and profile can not be combined.
      */
     const KoColorSpace * colorSpace(const QString &colorSpaceId, const QString &profileName);
+public:
+    /**
+     * Return a colorspace that works with the parameter profile.
+     * @param colorSpaceId the ID string of the colorspace that you want to have returned
+     * @param profile the profile be combined with the colorspace
+     * @return the wanted colorspace, or 0 when the cs and profile can not be combined.
+     */
+    const KoColorSpace * colorSpace(const QString & colorModelId, const QString & colorDepthId, const KoColorProfile *profile);
+
+    /**
+     * Return a colorspace that works with the parameter profile.
+     * @param profileName the name of the KoColorProfile to be combined with the colorspace
+     * @return the wanted colorspace, or 0 when the cs and profile can not be combined.
+     */
+    const KoColorSpace * colorSpace(const QString & colorModelId, const QString & colorDepthId, const QString &profileName);
 
     /**
      * Return the id of the colorspace that have the defined colorModelId with colorDepthId.
@@ -153,7 +194,7 @@ public:
      * @param colorDepthId id of the color depth
      * @return the id of the wanted colorspace, or "" if no colorspace correspond to those ids
      */
-    QString colorSpaceId(const QString & colorModelId, const QString & colorDepthId);
+    QString colorSpaceId(const QString & colorModelId, const QString & colorDepthId) const;
     /**
      * It's a convenient function that behave like the above.
      * Return the id of the colorspace that have the defined colorModelId with colorDepthId.
@@ -161,8 +202,24 @@ public:
      * @param colorDepthId id of the color depth
      * @return the id of the wanted colorspace, or "" if no colorspace correspond to those ids
      */
-    QString colorSpaceId(const KoID& colorModelId, const KoID& colorDepthId);
-    
+    QString colorSpaceId(const KoID& colorModelId, const KoID& colorDepthId) const;
+
+    /**
+     * @return a the identifiant of the color model for the given color space id.
+     *
+     * This function is a compatibility function used to get the color space from
+     * all kra files.
+     */
+    KoID colorSpaceColorModelId(const QString & _colorSpaceId) const;
+
+    /**
+     * @return a the identifiant of the color depth for the given color space id.
+     *
+     * This function is a compatibility function used to get the color space from
+     * all kra files.
+     */
+    KoID colorSpaceColorDepthId(const QString & _colorSpaceId) const;
+
     /**
      * Convenience method to get the often used alpha colorspace
      */
@@ -175,14 +232,14 @@ public:
      * @return the wanted colorspace, or 0 if the color space and profile can not be combined.
      */
     const KoColorSpace * rgb8(const QString &profileName = QString());
-    
+
     /**
      * Convenience method to get an RGBA 8bit colorspace with the given profile.
      * @param profile an RGB profile
      * @return the wanted colorspace, or 0 if the color space and profile can not be combined.
      */
     const KoColorSpace * rgb8(const KoColorProfile * profile);
-    
+
     /**
      * Convenience method to get an RGBA 16bit colorspace. If a profile is not specified,
      * an sRGB profile will be used.
@@ -216,40 +273,52 @@ public:
     /**
      * @return the list of available color models
      */
-    QList<KoID> colorModelsList(ColorSpaceListVisibility option ) const;
+    QList<KoID> colorModelsList(ColorSpaceListVisibility option) const;
 
     /**
      * @return the list of available color models for the given colorModelId
      */
-    QList<KoID> colorDepthList(const KoID& colorModelId, ColorSpaceListVisibility option ) const;
+    QList<KoID> colorDepthList(const KoID& colorModelId, ColorSpaceListVisibility option) const;
 
     /**
      * @return the list of available color models for the given colorModelId
      */
-    QList<KoID> colorDepthList(const QString & colorModelId, ColorSpaceListVisibility option ) const;
-    
+    QList<KoID> colorDepthList(const QString & colorModelId, ColorSpaceListVisibility option) const;
+
     /**
      * @return the color conversion system use by the registry and the color
      * spaces to create color conversion transformation
      */
     const KoColorConversionSystem* colorConversionSystem() const;
-    
+
     /**
      * @return the cache of color conversion transformation to be use by KoColorSpace
      */
     KoColorConversionCache* colorConversionCache() const;
-    
+
     /**
      * @return a permanent colorspace owned by the registry, of the same type and profile
      *         as the one given in argument
      */
-    const KoColorSpace* permanentColorspace( const KoColorSpace* _colorSpace );
+    const KoColorSpace* permanentColorspace(const KoColorSpace* _colorSpace);
+
+    /**
+     * This function return a list of all the keys in KoID format by using the name() method
+     * on the objects stored in the registry.
+     */
+    QList<KoID> listKeys() const;
+
+    /**
+     * @return a list with an instance of all color space with their default profile
+     */
+    QList<const KoColorSpace*> allColorSpaces(ColorSpaceListVisibility visibility, ColorSpaceListProfilesSelection pSelection);
+
 private:
 
     bool isCached(const QString & csId, const QString & profileName) const;
-    
+
     QString idsToCacheName(const QString & csId, const QString & profileName) const;
-    
+
 private:
     KoColorSpaceRegistry();
     KoColorSpaceRegistry(const KoColorSpaceRegistry&);
