@@ -20,6 +20,7 @@
 #define KOCOMPOSITEOP_H
 
 #include <QtCore>
+#include <klocale.h>
 
 #include "pigment_export.h"
 
@@ -37,6 +38,7 @@ const QString COMPOSITE_PLUS = "plus";
 const QString COMPOSITE_MINUS = "minus";
 const QString COMPOSITE_ADD = "add";
 const QString COMPOSITE_SUBTRACT = "subtract";
+const QString COMPOSITE_INVERSED_SUBTRACT = "inversed_subtract";
 const QString COMPOSITE_DIFF = "diff";
 const QString COMPOSITE_MULT = "multiply";
 const QString COMPOSITE_DIVIDE = "divide";
@@ -57,16 +59,27 @@ const QString COMPOSITE_COLORIZE = "colorize";
 const QString COMPOSITE_LUMINIZE = "luminize";
 const QString COMPOSITE_SCREEN = "screen";
 const QString COMPOSITE_OVERLAY = "overlay";
-const QString COMPOSITE_UNDEF = "underfined";
 const QString COMPOSITE_COPY_RED = "copy_red";
 const QString COMPOSITE_COPY_GREEN = "copy_green";
 const QString COMPOSITE_COPY_BLUE = "copy_blue";
 const QString COMPOSITE_COPY_OPACITY = "copy_opacity";
+const QString COMPOSITE_HARD_LIGHT = "hard_light";
+const QString COMPOSITE_SOFT_LIGHT = "soft_light";
+const QString COMPOSITE_EXCLUSION = "exclusion"; // XXX: not implemented anywhere yet
+const QString COMPOSITE_INVERTED_DIVIDE = "inverted_divide"; // XXX: not implemented anywhere yet
+const QString COMPOSITE_VIVID_LIGHT = "vivid light"; // XXX: not implemented anywhere yet
+const QString COMPOSITE_LINEAR_LIGHT = "linear light"; // XXX: not implemented anywhere yet
+const QString COMPOSITE_PIN_LIGHT = "pin light"; // XXX: not implemented anywhere yet
+const QString COMPOSITE_HARD_MIX = "hard mix"; // XXX: not implemented anywhere yet
+const QString COMPOSITE_PASS_THROUGH = "pass through"; // XXX: not implemented anywhere yet
+
+const QString COMPOSITE_UNDEF = "underfined";
 
 /**
  * Base for colorspace-specific blending modes.
  */
-class PIGMENTCMS_EXPORT KoCompositeOp {
+class PIGMENTCMS_EXPORT KoCompositeOp
+{
 public:
     static QString categoryMix();
     static QString categoryLight();
@@ -106,7 +119,7 @@ public:
      * @return the category associated with the composite op
      */
     QString category() const;
-    
+
     /**
       @param dstRowStart pointer to the start of the byte array we will composite the source on
       @param dstRowStride length of the rows of the block of destination pixels in bytes
@@ -122,10 +135,10 @@ public:
       @param opacity transparency with which to blend
     */
     virtual void composite(quint8 *dstRowStart, qint32 dstRowStride,
-               const quint8 *srcRowStart, qint32 srcRowStride,
-               const quint8 *maskRowStart, qint32 maskRowStride,
-               qint32 rows, qint32 numColumns,
-               quint8 opacity) const;
+                           const quint8 *srcRowStart, qint32 srcRowStride,
+                           const quint8 *maskRowStart, qint32 maskRowStride,
+                           qint32 rows, qint32 numColumns,
+                           quint8 opacity) const;
 
     /**
      * Same as previous, but with a bit array that determines which channels
@@ -138,15 +151,15 @@ public:
      * only the gray channel is copied.
      */
     virtual void composite(quint8 *dstRowStart, qint32 dstRowStride,
-               const quint8 *srcRowStart, qint32 srcRowStride,
-               const quint8 *maskRowStart, qint32 maskRowStride,
-               qint32 rows, qint32 numColumns,
-               quint8 opacity,
-               const QBitArray & channelFlags) const = 0;
-    private:
-        KoCompositeOp();
-        struct Private;
-        Private* const d;
+                           const quint8 *srcRowStart, qint32 srcRowStride,
+                           const quint8 *maskRowStart, qint32 maskRowStride,
+                           qint32 rows, qint32 numColumns,
+                           quint8 opacity,
+                           const QBitArray & channelFlags) const = 0;
+private:
+    KoCompositeOp();
+    struct Private;
+    Private* const d;
 };
 
 #endif // KOCOMPOSITEOP_H
