@@ -89,8 +89,6 @@ KoCsvImportDialog::KoCsvImportDialog(QWidget* parent)
     d->codec = QTextCodec::codecForName("UTF-8");
 
     setButtons( KDialog::Ok|KDialog::Cancel );
-    setDefaultButton(KDialog::No);
-
     setCaption( i18n( "Import Data" ) );
 
     QStringList encodings;
@@ -541,9 +539,10 @@ void KoCsvImportDialog::Private::fillTable()
 
     for (column = 0; column < dialog->m_sheet->columnCount(); ++column)
     {
-        const QString header = dialog->m_sheet->model()->headerData(column, Qt::Horizontal).toString();
-        if ( formatList.contains( header ) )
-            dialog->m_sheet->model()->setHeaderData(column, Qt::Horizontal, i18n("Generic"));
+        const QTableWidgetItem* headerItem = dialog->m_sheet->horizontalHeaderItem(column);
+        if (!headerItem || !formatList.contains(headerItem->text())) {
+            dialog->m_sheet->setHorizontalHeaderItem(column, new QTableWidgetItem(i18n("Generic")));
+        }
     }
 
     dialog->m_rowStart->setMinimum(1);
@@ -664,15 +663,15 @@ void KoCsvImportDialog::delimiterClicked(int id)
 {
     const QButtonGroup* group = d->dialog->m_radioComma->group();
     if (id == group->id(d->dialog->m_radioComma) )
-        d->delimiter = ",";
+        d->delimiter = ',';
     else if (id == group->id(d->dialog->m_radioOther))
         d->delimiter = d->dialog->m_delimiterEdit->text();
     else if (id == group->id(d->dialog->m_radioTab))
-        d->delimiter = "\t";
+        d->delimiter = '\t';
     else if (id == group->id(d->dialog->m_radioSpace))
-        d->delimiter = " ";
+        d->delimiter = ' ';
     else if (id == group->id(d->dialog->m_radioSemicolon))
-        d->delimiter = ";";
+        d->delimiter = ';';
 
     kDebug(30501) << "Delimiter" << d->delimiter << "selected.";
     d->fillTable();
