@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2007 Boudewijn Rempt <boud@valdyas.org>
+ *  Copyright (c) 2007-2010 Boudewijn Rempt <boud@valdyas.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include <kactioncollection.h>
 #include <kdebug.h>
 
-#include "KoCanvasController.h"
+#include "KoCanvasControllerWidget.h"
 #include "KoZoomHandler.h"
 #include "KoZoomController.h"
 #include "KoDpi.h"
@@ -35,11 +35,12 @@
 void zoomcontroller_test::testApi()
 {
     KoZoomHandler zoomHandler;
-    KoZoomController zoomController(new KoCanvasController(), &zoomHandler, new KActionCollection(this), KoZoomAction::AspectMode);
+    KoZoomController zoomController(new KoCanvasControllerWidget(), &zoomHandler, new KActionCollection(this), KoZoomAction::AspectMode);
+    Q_UNUSED(zoomController);
 
 }
 
 QTEST_KDEMAIN(zoomcontroller_test, GUI)
 
-#include "zoomcontroller_test.moc"
+#include <zoomcontroller_test.moc>
 
