@@ -20,27 +20,21 @@
 #ifndef ARTISTICTEXTSHAPEFACTORY_H
 #define ARTISTICTEXTSHAPEFACTORY_H
 
-#include <KoShapeFactory.h>
+#include <KoShapeFactoryBase.h>
 
 class KoShape;
 
-class ArtisticTextShapeFactory : public KoShapeFactory
+class ArtisticTextShapeFactory : public KoShapeFactoryBase
 {
-    using KoShapeFactory::createDefaultShape;
-    using KoShapeFactory::createShape;
-
     Q_OBJECT
 
 public:
     explicit ArtisticTextShapeFactory(QObject *parent);
     ~ArtisticTextShapeFactory() {}
 
-    KoShape *createDefaultShape() const;
-    KoShape *createShape(const KoProperties * params) const;
-    // reimplemented from KoShapeFactory
-    virtual bool supports(const KoXmlElement & e) const;
-
-    QList<KoShapeConfigWidgetBase*> createShapeOptionPanels();
+    virtual KoShape *createDefaultShape(KoResourceManager *documentResources = 0) const;
+    // reimplemented from KoShapeFactoryBase
+    virtual bool supports(const KoXmlElement & e, KoShapeLoadingContext &context) const;
 };
 
 #endif // ARTISTICTEXTSHAPEFACTORY_H
