@@ -2,6 +2,7 @@
  * Copyright (C) 2007 Martin Pfeiffer <hubipete@gmx.net>
  * Copyright (C) 2007 Jan Hambrecht <jaham@gmx.net>
    Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,7 +25,7 @@
 
 #include <KoInteractionTool.h>
 #include <KoCanvasBase.h>
-#include <KoCanvasResourceProvider.h>
+#include <KoResourceManager.h>
 #include <KoShapeManager.h>
 #include <KoSelection.h>
 #include <commands/KoShapeMoveCommand.h>
@@ -41,18 +42,22 @@
 #include <QCheckBox>
 #include <QDoubleSpinBox>
 #include <QList>
-#include <QMatrix>
+#include <QTransform>
 
 DefaultToolWidget::DefaultToolWidget( KoInteractionTool* tool,
                                     QWidget* parent )
-    : QWidget( parent )
+    : QWidget( parent ),
+    m_tool(tool),
+    m_blockSignals(false)
 {
-    m_tool = tool;
-
     setupUi( this );
 
     setUnit( m_tool->canvas()->unit() );
 
+    aspectButton->setKeepAspectRatio( false );
+    updatePosition();
+    updateSize();
+
     connect( positionSelector, SIGNAL( positionSelected(KoFlake::Position) ), 
         this, SLOT( positionSelected(KoFlake::Position) ) );
 
@@ -69,18 +74,16 @@ DefaultToolWidget::DefaultToolWidget( KoInteractionTool* tool,
     connect( manager, SIGNAL( selectionContentChanged() ), this, SLOT( updatePosition() ) );
     connect( manager, SIGNAL( selectionContentChanged() ), this, SLOT( updateSize() ) );
 
-    connect( m_tool->canvas()->resourceProvider(), SIGNAL( resourceChanged( int, const QVariant& ) ),
+    connect( m_tool->canvas()->resourceManager(), SIGNAL( resourceChanged( int, const QVariant& ) ),
         this, SLOT( resourceChanged( int, const QVariant& ) ) );
 
-    aspectButton->setKeepAspectRatio( false );
-
-    updatePosition();
-    updateSize();
+    connect (aspectButton, SIGNAL(keepAspectRatioChanged(bool)),
+        this, SLOT(aspectButtonToggled(bool)));
 }
 
 void DefaultToolWidget::positionSelected( KoFlake::Position position )
 {
-    m_tool->canvas()->resourceProvider()->setResource( DefaultTool::HotPosition, QVariant(position) );
+    m_tool->canvas()->resourceManager()->setResource( DefaultTool::HotPosition, QVariant(position) );
     updatePosition();
 }
 
@@ -96,18 +99,18 @@ void DefaultToolWidget::updatePosition()
     positionXSpinBox->setEnabled( selection->count() );
     positionYSpinBox->setEnabled( selection->count() );
 
-    positionXSpinBox->blockSignals(true);
-    positionYSpinBox->blockSignals(true);
+    if (m_blockSignals)
+        return;
+    m_blockSignals = true;
     positionXSpinBox->changeValue( selPosition.x() );
     positionYSpinBox->changeValue( selPosition.y() );
-    positionXSpinBox->blockSignals(false);
-    positionYSpinBox->blockSignals(false);
 
     QList<KoShape*> selectedShapes = selection->selectedShapes( KoFlake::TopLevelSelection );
     bool aspectLocked = false;
     foreach (KoShape* shape, selectedShapes)
         aspectLocked = aspectLocked | shape->keepAspectRatio();
     aspectButton->setKeepAspectRatio(aspectLocked);
+    m_blockSignals = false;
 }
 
 void DefaultToolWidget::positionHasChanged()
@@ -147,18 +150,20 @@ void DefaultToolWidget::updateSize()
     widthSpinBox->setEnabled( selectionCount );
     heightSpinBox->setEnabled( selectionCount );
 
-    widthSpinBox->blockSignals(true);
-    heightSpinBox->blockSignals(true);
+    if (m_blockSignals)
+        return;
+    m_blockSignals = true;
     widthSpinBox->changeValue( selSize.width() );
     heightSpinBox->changeValue( selSize.height() );
-    widthSpinBox->blockSignals(false);
-    heightSpinBox->blockSignals(false);
+    m_blockSignals = false;
 }
 
 void DefaultToolWidget::sizeHasChanged()
 {
     if (aspectButton->hasFocus())
         return;
+    if (m_blockSignals)
+        return;
 
     QSizeF newSize( widthSpinBox->value(), heightSpinBox->value() );
 
@@ -179,31 +184,32 @@ void DefaultToolWidget::sizeHasChanged()
         // get the scale/resize center from the position selector
         QPointF scaleCenter = selection->absolutePosition( positionSelector->position() );
 
-        QMatrix resizeMatrix;
+        QTransform resizeMatrix;
         resizeMatrix.translate( scaleCenter.x(), scaleCenter.y() );
         resizeMatrix.scale( newSize.width() / rect.width(), newSize.height() / rect.height() );
         resizeMatrix.translate( -scaleCenter.x(), -scaleCenter.y() );
 
         QList<KoShape*> selectedShapes = selection->selectedShapes( KoFlake::StrippedSelection );
         QList<QSizeF> oldSizes, newSizes;
-        QList<QMatrix> oldState;
-        QList<QMatrix> newState;
+        QList<QTransform> oldState;
+        QList<QTransform> newState;
 
         foreach( KoShape* shape, selectedShapes )
         {
+            shape->update();
             QSizeF oldSize = shape->size();
             oldState << shape->transformation();
-            QMatrix shapeMatrix = shape->absoluteTransformation(0);
+            QTransform shapeMatrix = shape->absoluteTransformation(0);
 
             // calculate the matrix we would apply to the local shape matrix
             // that tells us the effective scale values we have to use for the resizing
-            QMatrix localMatrix = shapeMatrix * resizeMatrix * shapeMatrix.inverted();
+            QTransform localMatrix = shapeMatrix * resizeMatrix * shapeMatrix.inverted();
             // save the effective scale values
             qreal scaleX = localMatrix.m11();
             qreal scaleY = localMatrix.m22();
 
             // calculate the scale matrix which is equivalent to our resizing above
-            QMatrix scaleMatrix = (QMatrix().scale( scaleX, scaleY ));
+            QTransform scaleMatrix = (QTransform().scale( scaleX, scaleY ));
             scaleMatrix =  shapeMatrix.inverted() * scaleMatrix * shapeMatrix;
 
             // calculate the new size of the shape, using the effective scale values
@@ -229,16 +235,18 @@ void DefaultToolWidget::sizeHasChanged()
 
 void DefaultToolWidget::setUnit( const KoUnit &unit )
 {
+    m_blockSignals = true;
     positionXSpinBox->setUnit( unit );
     positionYSpinBox->setUnit( unit );
     widthSpinBox->setUnit( unit );
     heightSpinBox->setUnit( unit );
+    m_blockSignals = false;
 }
 
 void DefaultToolWidget::resourceChanged( int key, const QVariant & res )
 {
     if( key == KoCanvasResource::Unit )
-        setUnit( m_tool->canvas()->unit() );
+        setUnit(res.value<KoUnit>());
     else if( key == DefaultTool::HotPosition )
     {
         if( res.toInt() != positionSelector->position() )
@@ -249,4 +257,14 @@ void DefaultToolWidget::resourceChanged( int key, const QVariant & res )
     }
 }
 
+void DefaultToolWidget::aspectButtonToggled(bool keepAspect)
+{
+    if (m_blockSignals)
+        return;
+    KoSelection * selection = m_tool->canvas()->shapeManager()->selection();
+    foreach (KoShape *shape, selection->selectedShapes(KoFlake::TopLevelSelection)) {
+        shape->setKeepAspectRatio(keepAspect);
+    }
+}
+
 #include <DefaultToolWidget.moc>
