@@ -22,18 +22,20 @@
 #include <KLocale>
 
 GuidesToolFactory::GuidesToolFactory( QObject *parent )
-    : KoToolFactory(parent, KoGuidesTool_ID, i18n("Edit Guidelines"))
+    : KoToolFactoryBase(parent, GuidesToolId)
 {
     setToolTip(i18n("Edit guidelines"));
-    setToolType(mainToolType());
+    setToolType("never");
     setPriority(1);
+    setInputDeviceAgnostic(true);
+    setActivationShapeId("itShouldNeverBeActivated");
 }
 
 GuidesToolFactory::~GuidesToolFactory()
 {
 }
 
-KoTool* GuidesToolFactory::createTool(KoCanvasBase *canvas)
+KoToolBase* GuidesToolFactory::createTool(KoCanvasBase *canvas)
 {
-    return new GuidesTool( canvas );
+    return new GuidesTool(canvas);
 }
