@@ -20,11 +20,11 @@
 #ifndef SHAPEFACTORY_H
 #define SHAPEFACTORY_H
 
-#include <KoShapeFactory.h>
+#include <KoShapeFactoryBase.h>
 
 class KoShape;
 
-class DivineProportionShapeFactory : public KoShapeFactory {
+class DivineProportionShapeFactory : public KoShapeFactoryBase {
     Q_OBJECT
 
 public:
@@ -32,8 +32,8 @@ public:
     explicit DivineProportionShapeFactory(QObject *parent);
     ~DivineProportionShapeFactory() {}
 
-    KoShape *createDefaultShape() const;
-    KoShape *createShape(const KoProperties * params) const;
+    virtual KoShape *createDefaultShape(KoResourceManager *documentResources = 0) const;
+    virtual bool supports(const KoXmlElement & e, KoShapeLoadingContext &context) const;
 
     QList<KoShapeConfigWidgetBase*> createShapeOptionPanels();
 };
