@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007-2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,17 +25,32 @@
 
 #define FOLDERSHAPE_MIMETYPE "application/x-flake-shapeSelector-folder"
 
-class FolderShape : public KoShapeContainer {
+/**
+ * The shapeselector allows multiple 'folders' to show a selection of shape templates.
+ * In the startup phase there is only one folder, which has no border. If the user adds
+ * more folders multiple FolderShape items get created.
+ * A FolderShape represents one 'book', which is essentially a collection of items the
+ * user can drag to his koffice document to insert.  This class inherits KoShapeContainer
+ * and the items to insert are the IconShape or GroupShape or ClipboardProxyShape.
+ */
+class FolderShape : public KoShapeContainer
+{
 public:
     FolderShape();
 
+    /// reimplemented from KoShapeContainer
     virtual void paintComponent(QPainter &painter, const KoViewConverter &converter);
 
+    /// reimplemented from KoShapeContainer
     virtual bool loadOdf(const KoXmlElement&, KoShapeLoadingContext&) { return true; }
+    /// reimplemented from KoShapeContainer
     virtual void saveOdf(KoShapeSavingContext&) const {}
-    virtual void setSize( const QSizeF &size );
+    /// reimplemented from KoShapeContainer
+    virtual void setSize(const QSizeF &size);
 
-    QDomDocument save();
+    /// save the contents of the folder to an XML based QDomDocument
+    QDomDocument save() const;
+     /// load the QDomDocument based XML as saved in save()
     void load(const QDomDocument &document);
 };
 
