@@ -20,15 +20,15 @@
 #define MUSIC_TOOL_FACTORY
 
 
-#include <KoToolFactory.h>
+#include <KoToolFactoryBase.h>
 
-class MusicToolFactory : public KoToolFactory
+class MusicToolFactory : public KoToolFactoryBase
 {
 public:
     MusicToolFactory( QObject* parent );
     ~MusicToolFactory();
 
-    KoTool* createTool( KoCanvasBase* canvas );
+    KoToolBase* createTool( KoCanvasBase* canvas );
 };
 
 
