@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2008 Florian Merz <florianmerz@gmx.de>
+ * Copyright (C) 2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,14 +25,9 @@
 #include <KoShapeManager.h>
 #include <KoTextDocumentLayout.h>
 #include <KoTextShapeData.h>
+#include <KoParagraphStyle.h>
 
-#include <KDebug>
-#include <KLocalizedString>
-
-#include <QAbstractTextDocumentLayout>
-#include <QTextDocument>
-
-#include <assert.h>
+#include <KLocale>
 
 ParagraphBase::ParagraphBase(QObject *parent, KoCanvasBase *canvas)
         : QObject(parent),
@@ -54,23 +50,25 @@ bool ParagraphBase::hasActiveTextBlock() const {
 
 void ParagraphBase::activateTextBlockAt(const QPointF &point)
 {
-    KoShape *shape = dynamic_cast<KoShape*>(m_canvas->shapeManager()->shapeAt(point));
-    if (shape == NULL) {
-        // there is no shape below the mouse position
-        deactivateTextBlock();
-        return;
-    }
 
-    KoTextShapeData *textShapeData = qobject_cast<KoTextShapeData*>(shape->userData());
-    if (textShapeData == NULL) {
-        // the shape below the mouse position is not a text shape
+    KoShape *shape = 0;
+    KoTextShapeData *textShapeData = 0;
+    foreach (KoShape *s, m_canvas->shapeManager()->shapesAt(QRectF(point, QSizeF(4, 4)))) {
+        textShapeData = qobject_cast<KoTextShapeData*>(s->userData());
+        if (textShapeData) { // found it!
+            shape = s;
+            break;
+        }
+    }
+    if (shape == 0 || textShapeData == 0) {
+        // there is no shape below the mouse position
         deactivateTextBlock();
         return;
     }
 
     QTextDocument *document = textShapeData->document();
 
-    QPointF p = shape->transformation().inverted().map(point);
+    QPointF p = shape->absoluteTransformation(0).inverted().map(point);
     p += QPointF(0.0, textShapeData->documentOffset());
 
     int position = document->documentLayout()->hitTest(p, Qt::ExactHit);
@@ -81,7 +79,7 @@ void ParagraphBase::activateTextBlockAt(const QPointF &point)
     }
 
     QTextBlock newBlock(document->findBlock(position));
-    assert(newBlock.isValid());
+    Q_ASSERT(newBlock.isValid());
 
     activateTextBlock(newBlock, document);
 }
@@ -122,7 +120,7 @@ void ParagraphBase::activateTextBlock(QTextBlock newBlock, QTextDocument *docume
         // easy way to get from the cursor to a non-const document pointer
         // but it doesn't make any sense to pass in a document pointer which is
         // different to the document the text block belongs to
-        assert(newBlock.document() == document);
+        Q_ASSERT(newBlock.document() == document);
         m_document = document;
     }
 
@@ -153,7 +151,7 @@ void ParagraphBase::addFragments()
     m_fragments.clear();
 
     KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(textBlock().document()->documentLayout());
-    assert(layout != NULL);
+    Q_ASSERT(layout);
 
     QList<KoShape*> shapes = layout->shapes();
     foreach(KoShape *shape, shapes) {
@@ -177,7 +175,7 @@ bool ParagraphBase::shapeContainsBlock(const KoShape *shape)
 qreal ParagraphBase::shapeTop(const KoShape *shape) const
 {
     KoTextShapeData *textShapeData = qobject_cast<KoTextShapeData*>(shape->userData());
-    assert(textShapeData != NULL);
+    Q_ASSERT(textShapeData);
 
     return textShapeData->documentOffset();
 }
