@@ -22,14 +22,8 @@
 #define PARAGRAPHEDITOR_H
 
 #include "ParagraphBase.h"
-#include "ParagraphFragment.h"
 #include "Ruler.h"
 
-#include <QRectF>
-#include <QTextCursor>
-#include <QTextBlock>
-#include <QVector>
-
 class KoParagraphStyle;
 class KoCanvasBase;
 class KoViewConverter;
