@@ -31,11 +31,17 @@
 #include <KoXmlNS.h>
 #include <KoStoreDevice.h>
 #include <KoUnit.h>
+#include <KoGenStyle.h>
+#include <KoTextOnShapeContainer.h>
 
 #include <QPainter>
 #include <QTimer>
 #include <QPixmapCache>
 #include <kdebug.h>
+#include <KoFilterEffectStack.h>
+#include "GreyscaleFilterEffect.h"
+#include "MonoFilterEffect.h"
+#include "WatermarkFilterEffect.h"
 
 QString generate_key(qint64 key, const QSize & size)
 {
@@ -45,8 +51,9 @@ QString generate_key(qint64 key, const QSize & size)
 void RenderQueue::renderImage()
 {
     KoImageData *imageData = qobject_cast<KoImageData*>(m_pictureShape->userData());
-    if (m_wantedImageSize.isEmpty() || imageData == 0)
+    if (m_wantedImageSize.isEmpty() || imageData == 0) {
         return;
+    }
     QSize size = m_wantedImageSize.takeFirst();
     QString key(generate_key(imageData->key(), size));
     if (QPixmapCache::find(key) == 0) {
@@ -54,16 +61,25 @@ void RenderQueue::renderImage()
         QPixmapCache::insert(key, pixmap);
         m_pictureShape->update();
     }
-    if (! m_wantedImageSize.isEmpty())
+    if (! m_wantedImageSize.isEmpty()) {
         QTimer::singleShot(0, this, SLOT(renderImage()));
+    }
+}
+
+void RenderQueue::updateShape()
+{
+    m_pictureShape->update();
 }
 
 //////////////
 PictureShape::PictureShape()
     : KoFrameShape(KoXmlNS::draw, "image"),
-    m_renderQueue(new RenderQueue(this))
+    m_imageCollection(0),
+    m_renderQueue(new RenderQueue(this)),
+    m_mode(Standard)
 {
     setKeepAspectRatio(true);
+    setFilterEffectStack(new KoFilterEffectStack());
 }
 
 PictureShape::~PictureShape()
@@ -84,7 +100,11 @@ void PictureShape::paint(QPainter &painter, const KoViewConverter &converter)
 
     QString key(generate_key(imageData->key(), pixmapSize));
     QPixmap pixmap;
+#if QT_VERSION  >= 0x040600
+    if (!QPixmapCache::find(key, &pixmap)) { // first check cache.
+#else
     if (!QPixmapCache::find(key, pixmap)) { // first check cache.
+#endif
         // no? Does the imageData have it then?
         if (!(imageData->hasCachedPixmap() && imageData->pixmap().size() == pixmapSize)) {
             // ok, not what we want.
@@ -119,12 +139,18 @@ void PictureShape::paint(QPainter &painter, const KoViewConverter &converter)
         return;
     }
 
+#if QT_VERSION  >= 0x040600
+    if (!QPixmapCache::find(key, &pixmap)) {
+#else
     if (!QPixmapCache::find(key, pixmap)) {
+#endif
         m_renderQueue->addSize(pixmapSize);
         QTimer::singleShot(0, m_renderQueue, SLOT(renderImage()));
         if (!imageData->hasCachedPixmap()
-                || imageData->pixmap().size().width() > pixmapSize.width()) // don't scale down
+            || imageData->pixmap().size().width() > pixmapSize.width()) { // don't scale down
+            QTimer::singleShot(0, m_renderQueue, SLOT(updateShape()));
             return;
+        }
         pixmap = imageData->pixmap();
     }
     painter.drawPixmap(pixels, pixmap, QRect(0, 0, pixmap.width(), pixmap.height()));
@@ -133,17 +159,20 @@ void PictureShape::paint(QPainter &painter, const KoViewConverter &converter)
 void PictureShape::waitUntilReady(const KoViewConverter &converter, bool asynchronous) const
 {
     KoImageData *imageData = qobject_cast<KoImageData*>(userData());
-    if (imageData == 0)
+    if (imageData == 0) {
         return;
+    }
 
     if (asynchronous) {
         // get pixmap and schedule it if not
         QSize pixels = converter.documentToView(QRectF(QPointF(0,0), size())).size().toSize();
         QImage image = imageData->image();
-        if (image.isNull())
+        if (image.isNull()) {
             return;
-        if (image.size().width() < pixels.width()) // don't scale up.
+        }
+        if (image.size().width() < pixels.width()) { // don't scale up.
             pixels = image.size();
+        }
         m_printQualityImage = image.scaled(pixels, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
     }
     else {
@@ -160,8 +189,9 @@ void PictureShape::saveOdf(KoShapeSavingContext &context) const
 {
     // make sure we have a valid image data pointer before saving
     KoImageData *imageData = qobject_cast<KoImageData*>(userData());
-    if (imageData == 0)
+    if (imageData == 0) {
         return;
+    }
 
     KoXmlWriter &writer = context.xmlWriter();
 
@@ -174,6 +204,9 @@ void PictureShape::saveOdf(KoShapeSavingContext &context) const
     writer.addAttribute("xlink:show", "embed");
     writer.addAttribute("xlink:actuate", "onLoad");
     writer.addAttribute("xlink:href", name);
+    if (parent()) {
+        parent()->saveOdfChildElements(context);
+    }
     writer.endElement(); // draw:image
     saveOdfCommonChildElements(context);
     writer.endElement(); // draw:frame
@@ -209,6 +242,8 @@ bool PictureShape::loadOdfFrameElement(const KoXmlElement &element, KoShapeLoadi
         }
     }
 
+    KoTextOnShapeContainer::tryWrapShape(this, element, context);
+
     return true;
 }
 
@@ -217,7 +252,63 @@ KoImageCollection *PictureShape::imageCollection() const
     return m_imageCollection;
 }
 
-void PictureShape::init(const QMap<QString, KoDataCenter*> &dataCenterMap)
+QString PictureShape::saveStyle(KoGenStyle& style, KoShapeSavingContext& context) const
 {
-    m_imageCollection = dynamic_cast<KoImageCollection *>(dataCenterMap.value("ImageCollection"));
+    if(transparency() > 0.0) {
+        style.addProperty("draw:image-opacity", QString("%1%").arg((1.0 - transparency()) * 100.0));
+    }
+    return KoShape::saveStyle(style, context);
+}
+
+void PictureShape::loadStyle(const KoXmlElement& element, KoShapeLoadingContext& context)
+{
+    KoShape::loadStyle(element, context);
+    KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
+    styleStack.setTypeProperties("graphic");
+
+    //FIXME: are there other applicable properties?
+    if (styleStack.hasProperty(KoXmlNS::draw, "color-mode")) {
+        QString colorMode = styleStack.property(KoXmlNS::draw, "color-mode");
+        if (colorMode == "greyscale") {
+            setMode(Greyscale);
+        }
+        else if (colorMode == "mono") {
+            setMode(Mono);
+        }
+        else if (colorMode == "watermark") {
+            setMode(Watermark);
+        }
+    }
+    const QString opacity(styleStack.property(KoXmlNS::draw, "image-opacity"));
+    if (! opacity.isEmpty() && opacity.right(1) == "%") {
+        setTransparency(1.0 - (opacity.left(opacity.length() - 1).toFloat() / 100.0));
+    }
+}
+
+PictureShape::PictureMode PictureShape::mode() const
+{
+    return m_mode;
+}
+
+void PictureShape::setMode(PictureShape::PictureMode mode)
+{
+    if( mode != m_mode ) {
+        m_mode = mode;
+        KoFilterEffect* filterMode = filterEffectStack()->takeFilterEffect(0);
+        delete filterMode;
+        switch( mode ) {
+            case Greyscale:
+                filterMode = new GreyscaleFilterEffect();
+                break;
+            case Mono:
+                filterMode = new MonoFilterEffect();
+                break;
+            default:
+                filterMode = new WatermarkFilterEffect();
+                break;
+        }
+        if( filterMode )
+            filterEffectStack()->appendFilterEffect(filterMode);
+        update();
+    }
 }
