@@ -32,6 +32,13 @@ class RenderQueue;
 class PictureShape : public KoShape, public KoFrameShape
 {
 public:
+    enum PictureMode {
+        Standard,
+        Greyscale,
+        Mono,
+        Watermark
+    };
+
     PictureShape();
     virtual ~PictureShape();
 
@@ -44,21 +51,28 @@ public:
     // reimplemented
     virtual void waitUntilReady(const KoViewConverter &converter, bool asynchronous) const;
 
-    /// reimplemented
-    void init(const QMap<QString, KoDataCenter*> &dataCenterMap);
-
     /**
      * Get the collection used in the shape.
      */
     KoImageCollection *imageCollection() const;
 
+    void setImageCollection(KoImageCollection *collection) { m_imageCollection = collection; }
+
+    void setMode( PictureMode mode );
+    PictureMode mode() const;
+
 protected:
     virtual bool loadOdfFrameElement(const KoXmlElement &element, KoShapeLoadingContext &context);
 
+    virtual QString saveStyle(KoGenStyle &style, KoShapeSavingContext &context) const;
+    
+    virtual void loadStyle(const KoXmlElement& element, KoShapeLoadingContext& context);
+
 private:
     KoImageCollection *m_imageCollection;
     RenderQueue *m_renderQueue;
     mutable QImage m_printQualityImage;
+    PictureMode m_mode;
 };
 
 class RenderQueue : public QObject
@@ -71,6 +85,7 @@ public:
 
 public slots:
     void renderImage();
+    void updateShape();
 
 private:
     KoShape *m_pictureShape;
