@@ -20,17 +20,16 @@
 #ifndef PICTURE_TOOL_FACTORY
 #define PICTURE_TOOL_FACTORY
 
+#include <KoToolFactoryBase.h>
 
-#include <KoToolFactory.h>
-
-class PictureToolFactory : public KoToolFactory
+class PictureToolFactory : public KoToolFactoryBase
 {
     Q_OBJECT
 public:
     PictureToolFactory( QObject* parent);
     ~PictureToolFactory();
 
-    KoTool* createTool( KoCanvasBase* canvas );
+    KoToolBase *createTool(KoCanvasBase *canvas);
 };
 
 
