@@ -295,12 +295,20 @@ bool Autocorrect::autoBoldUnderline()
         m_cursor.setPosition(startPos + replacement.length(), QTextCursor::KeepAnchor);
 
         QTextCharFormat format;
-        format.setFontUnderline(underline);
-        format.setFontWeight(bold ? QFont::Bold : QFont::Normal);
+        format.setFontUnderline(underline ? true : m_cursor.charFormat().fontUnderline());
+        format.setFontWeight(bold ? QFont::Bold : m_cursor.charFormat().fontWeight());
         m_cursor.mergeCharFormat(format);
 
         // to avoid the selection being replaced by m_word
         m_word = m_cursor.selectedText();
+
+        // don't do this again if the text is already underlined and bold
+        if(m_cursor.charFormat().fontUnderline()
+            && m_cursor.charFormat().fontWeight() == QFont::Bold) {
+            return true;
+        } else {
+            return autoBoldUnderline();
+        }
     }
     else
         return false;
@@ -313,13 +321,21 @@ bool Autocorrect::autoFractions()
     if (! m_autoFractions) return false;
 
     QString trimmed = m_word.trimmed();
+    if (trimmed.length() > 3) {
+        QChar x = trimmed.at(3);
+        if (!(x.unicode() == '.' || x.unicode() == ',' || x.unicode() == '?' || x.unicode() == '!'
+                || x.unicode() == ':' || x.unicode() == ';'))
+            return false;
+    } else if (trimmed.length() < 3) {
+        return false;
+    }
 
-    if (trimmed == QString("1/2"))
-        m_word.replace(0, 3, QString("½"));
-    else if (trimmed == QString("1/4"))
-        m_word.replace(0, 3, QString("¼"));
-    else if (trimmed == QString("3/4"))
-        m_word.replace(0, 3, QString("¾"));
+    if (trimmed.startsWith("1/2"))
+        m_word.replace(0, 3, QString::fromUtf8("½"));
+    else if (trimmed.startsWith("1/4"))
+        m_word.replace(0, 3, QString::fromUtf8("¼"));
+    else if (trimmed.startsWith("3/4"))
+        m_word.replace(0, 3, QString::fromUtf8("¾"));
     else
         return false;
 
@@ -494,11 +510,37 @@ void Autocorrect::advancedAutocorrect()
     int startPos = m_cursor.selectionStart();
     int length = m_word.length();
 
-    QString word = m_word.toLower().trimmed();
-    if (m_autocorrectEntries.contains(word)) {
-        int pos = m_word.toLower().indexOf(word);
-        QString replacement = m_autocorrectEntries.value(word);
-        m_word.replace(pos, pos + word.length(), replacement);
+    QString trimmedWord = m_word.toLower().trimmed();
+    QString actualWord = trimmedWord;
+
+    if (actualWord.isEmpty()) return;
+
+    // If the last char is punctuation, drop it for now
+    bool hasPunctuation = false;
+    QChar lastChar = actualWord.at(actualWord.length() - 1);
+    if (lastChar.unicode() == '.' || lastChar.unicode() == ',' || lastChar.unicode() == '?' ||
+          lastChar.unicode() == '!' || lastChar.unicode() == ':' || lastChar.unicode() == ';') {
+        hasPunctuation = true;
+        actualWord.chop(1);
+    }
+
+    if (m_autocorrectEntries.contains(actualWord)) {
+        int pos = m_word.toLower().indexOf(trimmedWord);
+        QString replacement = m_autocorrectEntries.value(actualWord);
+        // Keep capitalized words capitalized.
+        // (Necessary to make sure the first letters match???)
+        if (actualWord.at(0) == replacement.at(0).toLower()) {
+            if (m_word.at(0).isUpper()) {
+                replacement[0] = replacement[0].toUpper();
+            }
+        }
+
+        // If a punctuation mark was on the end originally, add it back on
+        if (hasPunctuation) {
+            replacement.append(lastChar);
+        }
+
+        m_word.replace(pos, pos + trimmedWord.length(), replacement);
 
         // We do replacement here, since the length of new word might be different from length of
         // the old world. Length difference might affect other type of autocorrection
