@@ -25,14 +25,14 @@
 
 #include <KoTextEditingRegistry.h>
 
-K_PLUGIN_FACTORY( SpellCheckPluginFactory, registerPlugin<SpellCheckPlugin>(); )
-K_EXPORT_PLUGIN( SpellCheckPluginFactory("SpellCheckPlugin") )
+K_PLUGIN_FACTORY(SpellCheckPluginFactory, registerPlugin<SpellCheckPlugin>();)
+K_EXPORT_PLUGIN(SpellCheckPluginFactory("SpellCheckPlugin"))
 
-SpellCheckPlugin::SpellCheckPlugin( QObject *parent, const QVariantList& )
+SpellCheckPlugin::SpellCheckPlugin(QObject *parent, const QVariantList&)
     : QObject(parent)
 {
-    KoTextEditingRegistry::instance()->add( new SpellCheckFactory( parent));
+    KoTextEditingRegistry::instance()->add(new SpellCheckFactory(parent));
 }
 
-#include "SpellCheckPlugin.moc"
+#include <SpellCheckPlugin.moc>
 
