@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (C) 2009 Elvis Stansvik <elvstone@gmail.org>
  * Copyright (C) 2009 KO GmbH <cbo@kogmbh.com>
+ * Copyright (C) 2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -33,7 +34,6 @@
 #include <QTextTable>
 #include <QTextLine>
 #include <QPainter>
-#include <QDebug>
 #include <QRectF>
 #include <QPointF>
 
@@ -54,11 +54,11 @@ TableLayout::TableLayout() :
 void TableLayout::setTable(QTextTable *table)
 {
     Q_ASSERT(table);
-    TableLayoutData *tableLayoutData;
 
-    if(table == m_table)
+    if (table == m_table)
         return; // we are already set
 
+    TableLayoutData *tableLayoutData;
     if (!m_tableLayoutDataMap.contains(table)) {
         // Set up new table layout data.
         tableLayoutData = new TableLayoutData();
@@ -141,47 +141,47 @@ void TableLayout::startNewTableRect(QPointF position, qreal parentWidth, int fro
     KoTableColumnAndRowStyleManager *carsManager =
     reinterpret_cast<KoTableColumnAndRowStyleManager *>(
             tableFormat.property(KoTableStyle::ColumnAndRowStyleManager).value<void *>());
+    if (!carsManager)
+        carsManager = new KoTableColumnAndRowStyleManager();
+
     // Column widths.
     qreal availableWidth = tableWidth; // Width available for columns.
     QList<int> relativeWidthColumns; // List of relative width columns.
     qreal relativeWidthSum = 0; // Sum of relative column width values.
     int numNonStyleColumns = 0;
     for (int col = 0; col < tableRect.columnPositions.size(); ++col) {
-        KoTableColumnStyle *columnStyle = carsManager ? carsManager->columnStyle(col) : 0;
-        if (!columnStyle) {
-            // No style so neither width nor relative width specified. Can this happen?
-           tableRect.columnWidths[col] = 0.0;
-            relativeWidthColumns.append(col); // handle it as a relative width column without asking for anything
-            ++numNonStyleColumns;
-        } else if (columnStyle->hasProperty(KoTableColumnStyle::RelativeColumnWidth)) {
+        KoTableColumnStyle columnStyle = carsManager->columnStyle(col);
+        if (columnStyle.hasProperty(KoTableColumnStyle::RelativeColumnWidth)) {
             // Relative width specified. Will be handled in the next loop.
             relativeWidthColumns.append(col);
-            relativeWidthSum += columnStyle->relativeColumnWidth();
-        } else if (columnStyle->hasProperty(KoTableColumnStyle::ColumnWidth)) {
+            relativeWidthSum += columnStyle.relativeColumnWidth();
+        } else if (columnStyle.hasProperty(KoTableColumnStyle::ColumnWidth)) {
             // Only width specified, so use it.
-            tableRect.columnWidths[col] = columnStyle->columnWidth();
-            availableWidth -= columnStyle->columnWidth();
+            tableRect.columnWidths[col] = columnStyle.columnWidth();
+            availableWidth -= columnStyle.columnWidth();
         } else {
-            // Neither width nor relative width specified. Can this happen?
+            // Neither width nor relative width specified.
             kWarning(32600) << "Neither column-width nor rel-column-width specified";
             tableRect.columnWidths[col] = 0.0;
+            relativeWidthColumns.append(col); // handle it as a relative width column without asking for anything
+            ++numNonStyleColumns;
         }
     }
 
     // Calculate width to those columns that don't actually request it
-    qreal widthForNonStyleColumn = ((1.0 - qMin<qreal>(relativeWidthSum, 1.0)) * availableWidth);
-    availableWidth -= widthForNonStyleColumn; // might as well do this calc before dividing by numNonStyleColumns
-    if(numNonStyleColumns)
-        widthForNonStyleColumn /= numNonStyleColumns;
+    qreal widthForNonWidthColumn = ((1.0 - qMin<qreal>(relativeWidthSum, 1.0)) * availableWidth);
+    availableWidth -= widthForNonWidthColumn; // might as well do this calc before dividing by numNonStyleColumns
+    if (numNonStyleColumns)
+        widthForNonWidthColumn /= numNonStyleColumns;
 
     // Relative column widths have now been summed up and can be distributed.
     foreach (int col, relativeWidthColumns) {
-        KoTableColumnStyle *columnStyle = carsManager ? carsManager->columnStyle(col) : 0;
-        if (columnStyle) {
+        KoTableColumnStyle columnStyle = carsManager->columnStyle(col);
+        if (columnStyle.hasProperty(KoTableColumnStyle::RelativeColumnWidth) || columnStyle.hasProperty(KoTableColumnStyle::ColumnWidth)) {
             tableRect.columnWidths[col] =
-                qMax<qreal>(columnStyle->relativeColumnWidth() * availableWidth / relativeWidthSum, 0.0);
+                qMax<qreal>(columnStyle.relativeColumnWidth() * availableWidth / relativeWidthSum, 0.0);
         } else {
-            tableRect.columnWidths[col] = widthForNonStyleColumn;
+            tableRect.columnWidths[col] = widthForNonWidthColumn;
         }
     }
 
@@ -223,7 +223,7 @@ void TableLayout::layoutRow(int row)
     }
 
     // First remove the following (by now invalid) table rects
-    while(m_tableLayoutData->m_tableRects.back().fromRow > row) {
+    while (m_tableLayoutData->m_tableRects.back().fromRow > row) {
         m_tableLayoutData->m_tableRects.removeLast();
     }
     // Now the following calls to m_tableRects.back() are hitting the right table rect
@@ -235,6 +235,9 @@ void TableLayout::layoutRow(int row)
     reinterpret_cast<KoTableColumnAndRowStyleManager *>(
             tableFormat.property(KoTableStyle::ColumnAndRowStyleManager).value<void *>());
 
+    if (!carsManager)
+        carsManager = new KoTableColumnAndRowStyleManager();
+
     /*
      * Implementation Note:
      *
@@ -257,10 +260,13 @@ void TableLayout::layoutRow(int row)
      * cells that should contribute to the row height.
      */
 
-    KoTableRowStyle *rowStyle = carsManager ? carsManager->rowStyle(row) : 0;
+    KoTableRowStyle rowStyle = carsManager->rowStyle(row);
 
     // Adjust row height.
-    qreal rowHeight = rowStyle ? rowStyle->minimumRowHeight() : 0.0;
+    qreal minimumRowHeight = rowStyle.minimumRowHeight();
+    qreal rowHeight = rowStyle.rowHeight();
+    bool rowHasExactHeight = rowStyle.hasProperty(KoTableRowStyle::RowHeight);
+
     int col = 0;
     while (col < m_table->columns()) {
         // Get the cell format.
@@ -284,18 +290,21 @@ void TableLayout::layoutRow(int row)
                 cellHeight -= m_tableLayoutData->m_rowHeights[rowAbove];
             }
 
-            /*
-             * Now we know how much height this cell contributes to the row,
-             * and can determine wheather the row height will grow.
-             */
-            rowHeight = qMax(rowHeight, cellHeight);
+            // if the row does not have exact height defined
+            if (!rowHasExactHeight) {
+                /*
+                 * Now we know how much height this cell contributes to the row,
+                 * and can determine wheather the row height will grow.
+                 */
+                rowHeight = qMax(qMax(minimumRowHeight, cellHeight), rowHeight);
+            }
         }
         col += cell.columnSpan(); // Skip across column spans.
     }
     m_tableLayoutData->m_rowHeights[row] = rowHeight;
 
     // Adjust Y position of NEXT row.
-    // This is nice since the outside layout routine relies on the next row having a correct y posiition
+    // This is nice since the outside layout routine relies on the next row having a correct y position
     // the first row y position is set in createFirstLayoutRect()
     if (row+1 <  m_table->rows()) {
         m_tableLayoutData->m_rowPositions[row+1] =
@@ -305,7 +314,8 @@ void TableLayout::layoutRow(int row)
 
     // Adjust table rect height for new height.
     m_tableLayoutData->m_tableRects.last().rect.setHeight(m_tableLayoutData->m_rowPositions[row]
- + m_tableLayoutData->m_rowHeights[row] - m_tableLayoutData->m_rowPositions[m_tableLayoutData->m_tableRects.last().fromRow]);//FIXME review when breaking inside a row
+        + m_tableLayoutData->m_rowHeights[row]
+        - m_tableLayoutData->m_rowPositions[m_tableLayoutData->m_tableRects.last().fromRow]);//FIXME review when breaking inside a row
 }
 
 void TableLayout::drawBackground(QPainter *painter) const
@@ -313,7 +323,7 @@ void TableLayout::drawBackground(QPainter *painter) const
     if (m_tableLayoutData->m_tableRects.isEmpty()) {
         return;
     }
-    
+
     painter->save();
 
     // Draw table background.
@@ -342,17 +352,17 @@ void TableLayout::drawBackground(QPainter *painter) const
     painter->restore();
 }
 
-void TableLayout::drawBorders(QPainter *painter) const
+void TableLayout::drawBorders(QPainter *painter, QPainterPath *accuBlankBorders) const
 {
     if (m_tableLayoutData->m_tableRects.isEmpty()) {
         return;
     }
 
     painter->save();
-    
+
     KoTableStyle tableStyle(m_table->format());
     bool collapsing = tableStyle.collapsingBorderModel();
-    
+
     // Draw cell borders using their styles.
     for (int row = 0; row < m_table->rows(); ++row) {
         for (int column = 0; column < m_table->columns(); ++column) {
@@ -365,42 +375,38 @@ void TableLayout::drawBorders(QPainter *painter) const
             if (row == tableCell.row() && column == tableCell.column()) {
                 // This is an actual cell we want to draw, and not a covered one.
                 KoTableCellStyle cellStyle(tableCell.format().toTableCellFormat());
-                
+
                 if (collapsing) {
                     QRectF bRect = cellBoundingRect(tableCell);
-                    
+
                     // First the horizontal borders
                     if (row == 0) {
-                        cellStyle.drawTopHorizontalBorder(*painter, bRect.x(), bRect.y(), bRect.width());
+                        cellStyle.drawTopHorizontalBorder(*painter, bRect.x(), bRect.y(), bRect.width(), accuBlankBorders);
                     }
-                    
                     if (row + tableCell.rowSpan() == m_table->rows()) {
                         // we hit the bottom of the table so just draw the bottom border
-                        cellStyle.drawBottomHorizontalBorder(*painter, bRect.x(), bRect.bottom(), bRect.width());
+                        cellStyle.drawBottomHorizontalBorder(*painter, bRect.x(), bRect.bottom(), bRect.width(), accuBlankBorders);
                     } else {
                         // we have cells below so draw sharedborders
                         QTextTableCell tableCellBelow;
-                        
                         for (int c = column; c < column + tableCell.columnSpan();
                                                     c = tableCellBelow.column()+tableCellBelow.columnSpan()) {
                             tableCellBelow = m_table->cellAt(row + tableCell.rowSpan()-1+1, c);
                             QRectF belowBRect = cellBoundingRect(tableCellBelow);
-                            qreal x = qMax(bRect.x(), belowBRect.x()); 
-                            qreal x2 = qMin(bRect.right(), belowBRect.right()); 
-
+                            qreal x = qMax(bRect.x(), belowBRect.x());
+                            qreal x2 = qMin(bRect.right(), belowBRect.right());
                             KoTableCellStyle cellBelowStyle(tableCellBelow.format().toTableCellFormat());
-                            cellStyle.drawSharedHorizontalBorder(*painter, cellBelowStyle, x, bRect.bottom(), x2 - x);
+                            cellStyle.drawSharedHorizontalBorder(*painter, cellBelowStyle, x, bRect.bottom(), x2 - x, accuBlankBorders);
                         }
                     }
-                    
+
                     // And then the same treatment for vertical borders
                     if (column == 0) {
-                        cellStyle.drawLeftmostVerticalBorder(*painter, bRect.x(), bRect.y(), bRect.height());
+                        cellStyle.drawLeftmostVerticalBorder(*painter, bRect.x(), bRect.y(), bRect.height(), accuBlankBorders);
                     }
-                    
                     if (column + tableCell.columnSpan() == m_table->columns()) {
                         // we hit the rightmost edge of the table so draw the rightmost border
-                        cellStyle.drawRightmostVerticalBorder(*painter, bRect.right(), bRect.y(), bRect.height());
+                        cellStyle.drawRightmostVerticalBorder(*painter, bRect.right(), bRect.y(), bRect.height(), accuBlankBorders);
                     } else {
                         // we have cells to the right so draw sharedborders
                         QTextTableCell tableCellRight;
@@ -408,13 +414,15 @@ void TableLayout::drawBorders(QPainter *painter) const
                                                 r = tableCellRight.row() + tableCellRight.rowSpan()) {
                             tableCellRight = m_table->cellAt(r, column + tableCell.columnSpan()-1+1);
                             QRectF rightBRect = cellBoundingRect(tableCellRight);
-                            qreal y = qMax(bRect.y(), rightBRect.y()); 
-                            qreal y2 = qMin(bRect.bottom(), rightBRect.bottom()); 
+                            qreal y = qMax(bRect.y(), rightBRect.y());
+                            qreal y2 = qMin(bRect.bottom(), rightBRect.bottom());
 
                             KoTableCellStyle cellBelowRight(tableCellRight.format().toTableCellFormat());
-                            cellStyle.drawSharedVerticalBorder(*painter, cellBelowRight, bRect.right(), y, y2-y);
+                            cellStyle.drawSharedVerticalBorder(*painter, cellBelowRight, bRect.right(), y, y2-y, accuBlankBorders);
                         }
                     }
+                    // Paint diagonal borders for current cell
+                    cellStyle.paintDiagonalBorders(*painter, bRect);
                 } else { // separating border model
                     cellStyle.paintBorders(*painter, cellBoundingRect(tableCell));
                 }
@@ -427,18 +435,17 @@ void TableLayout::drawBorders(QPainter *painter) const
 
 QTextTableCell TableLayout::hitTestTable(const QPointF &point) const
 {
-    
     foreach (TableRect tableRect, m_tableLayoutData->m_tableRects) {
         if (tableRect.rect.contains(point)) {
             int col;
             for (col = 1; col < m_table->columns(); ++col) {
-                if(point.x() < tableRect.columnPositions[col])
+                if (point.x() < tableRect.columnPositions[col])
                     break;
             }
             --col;
             int row;
             for (row = tableRect.fromRow; row < m_table->rows(); ++row) {
-                if(point.y() < m_tableLayoutData->m_rowPositions[row])
+                if (point.y() < m_tableLayoutData->m_rowPositions[row])
                     break;
             }
             --row;
@@ -449,6 +456,19 @@ QTextTableCell TableLayout::hitTestTable(const QPointF &point) const
     return QTextTableCell();
 }
 
+qreal TableLayout::cellContentY(const QTextTableCell &cell) const
+{
+    Q_ASSERT(isValid());
+    Q_ASSERT(cell.isValid());
+
+    /*
+     * Get the cell style and return the bounding rect adjusted for
+     * borders and paddings by calling KoTableCellStyle::contentRect().
+     */
+    KoTableCellStyle cellStyle(cell.format().toTableCellFormat());
+    return m_tableLayoutData->m_rowPositions[cell.row()] + cellStyle.topPadding() + cellStyle.topBorderWidth();
+}
+
 QRectF TableLayout::cellContentRect(const QTextTableCell &cell) const
 {
     Q_ASSERT(isValid());
@@ -466,9 +486,15 @@ QRectF TableLayout::cellBoundingRect(const QTextTableCell &cell) const
 {
     Q_ASSERT(isValid());
     Q_ASSERT(cell.row() < m_tableLayoutData->m_rowPositions.size());
-    TableRect tableRect = m_tableLayoutData->m_tableRects.last(); 
-    while(tableRect.fromRow > cell.row()) {
-        tableRect =  m_tableLayoutData->m_tableRects.first();
+    
+    if (m_tableLayoutData->m_tableRects.isEmpty())
+        return QRectF(QPointF(0.0,0.0), QSizeF(0.0,0.0));
+
+    TableRect tableRect = m_tableLayoutData->m_tableRects.last();
+    int i = m_tableLayoutData->m_tableRects.size()-1;
+    while (tableRect.fromRow > cell.row()) {
+        --i;
+        tableRect =  m_tableLayoutData->m_tableRects[i];
     }
     Q_ASSERT(cell.column() + cell.columnSpan() <=  tableRect.columnPositions.size());
 
@@ -485,52 +511,39 @@ QRectF TableLayout::cellBoundingRect(const QTextTableCell &cell) const
         // TODO  when breaking within a row the tableRect needs to be switched at some point
     }
 
-    return QRectF(
-            tableRect.columnPositions[cell.column()], m_tableLayoutData->m_rowPositions[cell.row()],
+    return QRectF(tableRect.columnPositions[cell.column()], m_tableLayoutData->m_rowPositions[cell.row()],
             width, height);
 }
 
 QRectF TableLayout::rowBoundingRect(int row) const
 {
     Q_ASSERT(row < m_tableLayoutData->m_rowPositions.size());
-    TableRect tableRect = m_tableLayoutData->m_tableRects.last(); 
-    while(tableRect.fromRow > row) {
-        tableRect =  m_tableLayoutData->m_tableRects.first();
+    TableRect tableRect = m_tableLayoutData->m_tableRects.last();
+    int i = m_tableLayoutData->m_tableRects.size()-1;
+    while (tableRect.fromRow > row) {
+        --i;
+        tableRect =  m_tableLayoutData->m_tableRects[i];
     }
     return QRectF(tableRect.rect.left(), m_tableLayoutData->m_rowPositions[row],
                 tableRect.rect.width(),  m_tableLayoutData->m_rowHeights[row]);
 }
 
-void TableLayout::calculateCellContentHeight(const QTextTableCell &cell)
+void TableLayout::setCellContentHeight(const QTextTableCell &cell, qreal bottom)
 {
     Q_ASSERT(isValid());
     Q_ASSERT(cell.isValid());
 
-    if (!isValid() || !cell.isValid()) {
+    if (!isValid() || !cell.isValid())
         return;
-    }
 
-    // Get the first line in the first block in the cell.
-    QTextFrame::iterator cellIterator = cell.begin();
-    Q_ASSERT(cellIterator.currentFrame() == 0); // TODO: Nested tables?
-    QTextBlock firstBlock = cellIterator.currentBlock();
-    Q_ASSERT(firstBlock.isValid());
-    QTextLine topLine = firstBlock.layout()->lineAt(0);
-    Q_ASSERT(topLine.isValid());
+    KoTableCellStyle cellStyle(cell.format().toTableCellFormat());
+    qreal top = m_tableLayoutData->m_rowPositions[cell.row()]
+    + cellStyle.topPadding() + cellStyle.topBorderWidth();
+    qreal contentHeight = bottom - top;
+    if (contentHeight < (qreal)0.126) // rounding problems due to Qt-scribe internally using ints.
+        contentHeight = (qreal)0.0;
 
-    // Get the last line in the last block in the cell.
-    cellIterator = cell.end();
-    cellIterator--;
-    Q_ASSERT(cellIterator.currentFrame() == 0); // TODO: Nested tables?
-    QTextBlock lastBlock = cellIterator.currentBlock();
-    Q_ASSERT(lastBlock.isValid());
-    QTextLine bottomLine = lastBlock.layout()->lineAt(lastBlock.layout()->lineCount() - 1);
-    Q_ASSERT(bottomLine.isValid());
-
-    // Content height is the difference between bottomLine and topLine.
-    qreal contentHeight = (bottomLine.y() + bottomLine.height()) - topLine.y();
-    Q_ASSERT(contentHeight >= 0); // Sanity check.
-    contentHeight = qMax(contentHeight, (qreal)0.);
+    Q_ASSERT(contentHeight >= 0);
 
     // Update content height value of the cell.
     m_tableLayoutData->m_contentHeights[cell.row()][cell.column()] = contentHeight;
@@ -540,9 +553,8 @@ QTextTableCell TableLayout::cellAt(int position) const
 {
     Q_ASSERT(isValid());
 
-    if (!isValid()) {
+    if (!isValid())
         return QTextTableCell(); // Return invalid cell.
-    }
 
     return m_table->cellAt(position);
 }
@@ -584,4 +596,4 @@ void TableLayout::tableDestroyed(QObject *object)
     }
 }
 
-#include "TableLayout.moc"
+#include <TableLayout.moc>
