@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Pierre Stirnweiss \pierre.stirnweiss_koffice@gadz.org>
  *
@@ -35,6 +35,7 @@
 
 class KoInlineTextObjectManager;
 class KoPageProvider;
+class KoImageCollection;
 
 /**
  * A text shape.
@@ -57,6 +58,9 @@ public:
     /// helper method.
     QPointF convertScreenPos(const QPointF &point);
 
+    /// set the image collection which is needed to draw bullet from images
+    void setImageCollection(KoImageCollection *collection) { m_imageCollection = collection; }
+
     /**
      * Set the shape's text to be demo text or not.
      * If true, replace the content with an lorem ipsum demo text and don't complain
@@ -80,24 +84,21 @@ public:
      * @param element element which represents the shape in odf.
      * @return false if loading failed.
      */
-    virtual bool loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context);
+    virtual bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context);
 
     /**
      * From KoShape reimplemented method to store the TextShape data as ODF.
      *
      * @param context the KoShapeSavingContext used for saving.
      */
-    virtual void saveOdf(KoShapeSavingContext & context) const;
-
-    /// reimplemented
-    virtual void init(const QMap<QString, KoDataCenter *> & dataCenterMap);
+    virtual void saveOdf(KoShapeSavingContext &context) const;
 
     KoTextShapeData *textShapeData() {
         return m_textShapeData;
     }
 
     bool hasFootnoteDocument() {
-        return m_footnotes != 0;
+        return m_footnotes != 0 && !m_footnotes->isEmpty();
     }
     QTextDocument *footnoteDocument();
 
@@ -107,8 +108,20 @@ public:
 
     virtual void update(const QRectF &shape) const;
 
+    // required for kpresenter hack
+    void setPageProvider(KoPageProvider *provider) { m_pageProvider = provider; }
+
+    /// reimplemented
+    virtual bool loadOdfFrame(const KoXmlElement &element, KoShapeLoadingContext &context);
+
 protected:
-    virtual bool loadOdfFrameElement(const KoXmlElement & element, KoShapeLoadingContext & context);
+    virtual bool loadOdfFrameElement(const KoXmlElement &element, KoShapeLoadingContext &context);
+
+    /// reimplemented
+    virtual void loadStyle(const KoXmlElement &element, KoShapeLoadingContext &context);
+
+    /// reimplemented
+    virtual QString saveStyle(KoGenStyle &style, KoShapeSavingContext &context) const;
 
 private:
     void shapeChanged(ChangeType type, KoShape *shape = 0);
@@ -119,7 +132,8 @@ private:
     bool m_demoText;
     mutable QMutex m_mutex;
     mutable QWaitCondition m_waiter;
-    KoPageProvider * m_pageProvider;
+    KoPageProvider *m_pageProvider;
+    KoImageCollection *m_imageCollection;
 
     QRegion m_paintRegion;
 };
