@@ -24,22 +24,21 @@
 
 #include "TextShape.h"
 
-#include <KoTool.h>
+#include <KoToolBase.h>
 
 #include <QClipboard>
 #include <QHash>
 #include <QTextBlock>
 #include <QTextCursor>
 #include <QTimer>
-#include <QPointer>
+#include <QWeakPointer>
 
+class TextEditingPluginContainer;
 class InsertCharacter;
 class KoChangeTracker;
 class KoCharacterStyle;
 class KoColor;
-#ifndef NO_PIGMENT
 class KoColorPopupAction;
-#endif
 class KoParagraphStyle;
 class KoStyleManager;
 class KoTextEditingPlugin;
@@ -57,11 +56,14 @@ class MockCanvas;
 /**
  * This is the tool for the text-shape (which is a flake-based plugin).
  */
-class TextTool : public KoTool
+class TextTool : public KoToolBase
 {
     Q_OBJECT
 public:
     explicit TextTool(KoCanvasBase *canvas);
+#ifndef NDEBUG
+    explicit TextTool(MockCanvas *canvas);
+#endif
     ~TextTool();
 
     /// reimplemented from superclass
@@ -81,7 +83,7 @@ public:
     virtual void keyReleaseEvent(QKeyEvent *event);
 
     /// reimplemented from superclass
-    virtual void activate(bool temporary = false);
+    virtual void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes);
     /// reimplemented from superclass
     virtual void deactivate();
     /// reimplemented from superclass
@@ -117,13 +119,18 @@ public:
 
     const QTextCursor cursor();
 
+    void setShapeData(KoTextShapeData *data);
+
+    KoTextEditor *textEditor() { return m_textEditor.data(); }
+
+    QRectF textRect(int startPosition, int endPosition) const;
 
 public slots:
     /// start the textedit-plugin.
     void startTextEditingPlugin(const QString &pluginId);
     /// add a command to the undo stack, executing it as well.
     void addCommand(QUndoCommand *command);
-    /// reimplemented from KoTool
+    /// reimplemented from KoToolBase
     virtual void resourceChanged(int key, const QVariant &res);
 
     /// call this when the 'is-bidi' boolean has been changed.
@@ -188,8 +195,12 @@ private slots:
     void insertTable();
     /// shows a dialog to alter the paragraph properties
     void formatParagraph();
+    //When enabled, display changes
+    void toggleShowChanges(bool);
     /// When enabled, make the change tracker record changes made while typing
-    void toggleTrackChanges(bool);
+    void toggleRecordChanges(bool);
+    /// Configure Change Tracking
+    void configureChangeTracking();
     /// select all text in the current document.
     void selectAll();
     /// show the style manager
@@ -234,27 +245,26 @@ private slots:
     void debugTextDocument();
     /// print debug about the details of the styles on the current text document
     void debugTextStyles();
+    /// the document we are editing has received an extra shape
+    void shapeAddedToDoc(KoShape *shape);
+    void ensureCursorVisible();
 
-#ifndef NDEBUG
-protected:
-    explicit TextTool(MockCanvas *canvas);
-#endif
+    void testSlot(bool);
 
 private:
     void repaintCaret();
     void repaintSelection();
     void repaintSelection(int from, int to);
-    void ensureCursorVisible();
-    QRectF textRect(int startPosition, int endPosition) const;
     int pointToPosition(const QPointF & point) const;
     void updateActions();
     void updateStyleManager();
-    void setShapeData(KoTextShapeData *data);
     void updateSelectedShape(const QPointF &point);
     void updateSelectionHandler();
     void editingPluginEvents();
     void finishedWord();
     void finishedParagraph();
+    void readConfig();
+    void writeConfig();
 
 private:
     friend class UndoTextCommand;
@@ -262,14 +272,17 @@ private:
     friend class ChangeTracker;
     friend class TextPasteCommand;
     friend class TextCutCommand;
+    friend class ShowChangesCommand;
+    friend class ChangeTrackedDeleteCommand;
+    friend class DeleteCommand;
     TextShape *m_textShape;
     KoTextShapeData *m_textShapeData;
-    QPointer<KoTextEditor> m_textEditor;
+    QWeakPointer<KoTextEditor> m_textEditor;
     KoChangeTracker *m_changeTracker;
     bool m_allowActions;
     bool m_allowAddUndoCommand;
     bool m_trackChanges;
-    bool m_allowResourceProviderUpdates;
+    bool m_allowResourceManagerUpdates;
     int m_prevCursorPosition; /// used by editingPluginEvents
 
     QTimer m_caretTimer;
@@ -286,15 +299,13 @@ private:
     KAction *m_actionFormatSub;
     KAction *m_actionFormatIncreaseIndent;
     KAction *m_actionFormatDecreaseIndent;
+    KAction *m_actionShowChanges;
+    KAction *m_actionRecordChanges;
+    KAction *m_configureChangeTracking;
     KFontSizeAction *m_actionFormatFontSize;
     KFontAction *m_actionFormatFontFamily;
-#ifndef NO_PIGMENT
     KoColorPopupAction *m_actionFormatTextColor;
     KoColorPopupAction *m_actionFormatBackgroundColor;
-#endif
-
-    QHash<QString, KoTextEditingPlugin*> m_textEditingPlugins;
-    KoTextEditingPlugin *m_spellcheckPlugin;
 
     QUndoCommand *m_currentCommand; //this command will be the direct parent of undoCommands generated as the result of QTextDocument changes
 
@@ -310,6 +321,8 @@ private:
 
     InsertCharacter *m_specialCharacterDocker;
 
+    TextEditingPluginContainer *m_textEditingPlugins;
+
     bool m_textTyping;
     bool m_textDeleting;
 
