@@ -21,7 +21,9 @@
 
 #include <KoTextEditor.h>
 #include <TextTool.h>
-
+#include "ChangeTrackedDeleteCommand.h"
+#include "DeleteCommand.h"
+#include <KAction>
 #include <klocale.h>
 
 TextCutCommand::TextCutCommand(TextTool *tool, QUndoCommand *parent) :
@@ -44,6 +46,12 @@ void TextCutCommand::redo()
     } else {
         m_first = false;
         m_tool->copy();
-        m_tool->m_textEditor->deleteChar();
+        KoTextEditor *te = m_tool->m_textEditor.data();
+        if (te == 0)
+            return;
+        if (m_tool->m_actionShowChanges->isChecked())
+            te->addCommand(new ChangeTrackedDeleteCommand(ChangeTrackedDeleteCommand::NextChar, m_tool));
+        else
+            te->addCommand(new DeleteCommand(DeleteCommand::NextChar, m_tool));
     }
 }
